@@ -38,7 +38,7 @@
 
 #include "jsapi.h"
 #include "nsCOMPtr.h"
-#include "nsIServiceManager.h"
+#include "nsIServiceManagerUtils.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptContext.h"
 #include "nsIDOMScriptObjectFactory.h"
@@ -53,16 +53,17 @@
 #include "nsIURI.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsDOMError.h"
-
+#include "nsIDOMWindowInternal.h"
 #include "nsIJSContextStack.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellTreeItem.h"
 
-static const char *sJSStackContractID = "@mozilla.org/js/xpc/ContextStack;1";
+static const char *kJSStackContractID = "@mozilla.org/js/xpc/ContextStack;1";
 
 nsIDOMScriptObjectFactory *nsContentUtils::sDOMScriptObjectFactory = nsnull;
 nsIXPConnect *nsContentUtils::sXPConnect = nsnull;
 nsIScriptSecurityManager *nsContentUtils::sSecurityManager = nsnull;
+nsIThreadJSContextStack *nsContentUtils::sThreadJSContextStack = nsnull;
 
 // static
 nsresult
@@ -70,25 +71,27 @@ nsContentUtils::Init()
 {
   NS_ENSURE_TRUE(!sXPConnect, NS_ERROR_ALREADY_INITIALIZED);
 
-  nsresult rv = nsServiceManager::GetService(nsIXPConnect::GetCID(),
-                                             nsIXPConnect::GetIID(),
-                                             (nsISupports **)&sXPConnect);
+  nsresult rv = CallGetService(nsIXPConnect::GetCID(), &sXPConnect);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = nsServiceManager::GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID,
-                                    nsIScriptSecurityManager::GetIID(),
-                                    (nsISupports **)&sSecurityManager);
+  rv = CallGetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID,
+                      &sSecurityManager);
   if (NS_FAILED(rv)) {
     sSecurityManager = nsnull;
   }
 
+  rv = CallGetService(kJSStackContractID, &sThreadJSContextStack);
+  if (NS_FAILED(rv)) {
+    sThreadJSContextStack = nsnull;
+  }
+
   return rv;
 }
 
 // static
 nsresult
 nsContentUtils::GetStaticScriptGlobal(JSContext* aContext, JSObject* aObj,
-                                     nsIScriptGlobalObject** aNativeGlobal)
+                                      nsIScriptGlobalObject** aNativeGlobal)
 {
   if (!sXPConnect) {
     *aNativeGlobal = nsnull;
@@ -123,8 +126,8 @@ nsContentUtils::GetStaticScriptGlobal(JSContext* aContext, JSObject* aObj,
 //static
 nsresult
 nsContentUtils::GetStaticScriptContext(JSContext* aContext,
-                                      JSObject* aObj,
-                                      nsIScriptContext** aScriptContext)
+                                       JSObject* aObj,
+                                       nsIScriptContext** aScriptContext)
 {
   nsCOMPtr<nsIScriptGlobalObject> nativeGlobal;
   GetStaticScriptGlobal(aContext, aObj, getter_AddRefs(nativeGlobal));
@@ -139,7 +142,7 @@ nsContentUtils::GetStaticScriptContext(JSContext* aContext,
 //static
 nsresult
 nsContentUtils::GetDynamicScriptGlobal(JSContext* aContext,
-                                      nsIScriptGlobalObject** aNativeGlobal)
+                                       nsIScriptGlobalObject** aNativeGlobal)
 {
   nsCOMPtr<nsIScriptContext> scriptCX;
   GetDynamicScriptContext(aContext, getter_AddRefs(scriptCX));
@@ -153,7 +156,7 @@ nsContentUtils::GetDynamicScriptGlobal(JSContext* aContext,
 //static
 nsresult
 nsContentUtils::GetDynamicScriptContext(JSContext *aContext,
-                                       nsIScriptContext** aScriptContext)
+                                        nsIScriptContext** aScriptContext)
 {
   *aScriptContext = nsnull;
 
@@ -342,6 +345,7 @@ nsContentUtils::Shutdown()
   NS_IF_RELEASE(sDOMScriptObjectFactory);
   NS_IF_RELEASE(sXPConnect);
   NS_IF_RELEASE(sSecurityManager);
+  NS_IF_RELEASE(sThreadJSContextStack);
 }
 
 // static
@@ -587,6 +591,46 @@ nsContentUtils::CanCallerAccess(nsIDOMNode *aNode)
   return NS_SUCCEEDED(rv);
 }
 
+//static
+PRBool
+nsContentUtils::InProlog(nsIDOMNode *aNode)
+{
+  NS_PRECONDITION(aNode, "missing node to nsContentUtils::InProlog");
+
+  // Check that there is an ancestor and that it is a document
+  nsCOMPtr<nsIDOMNode> parent;
+  aNode->GetParentNode(getter_AddRefs(parent));
+  if (!parent) {
+    return PR_FALSE;
+  }
+
+  PRUint16 type;
+  parent->GetNodeType(&type);
+  if (type != nsIDOMNode::DOCUMENT_NODE) {
+    return PR_FALSE;
+  }
+
+  nsCOMPtr<nsIDocument> doc = do_QueryInterface(parent);
+  NS_ASSERTION(doc, "document doesn't implement nsIDocument");
+  nsCOMPtr<nsIContent> cont = do_QueryInterface(aNode);
+  NS_ASSERTION(cont, "node doesn't implement nsIContent");
+
+  // Check that there are no elements before aNode to make sure we are not
+  // in the epilog
+  PRInt32 pos;
+  doc->IndexOf(cont, pos);
+  while (pos > 0) {
+    --pos;
+    nsCOMPtr<nsIContent> sibl;
+    doc->ChildAt(pos, *getter_AddRefs(sibl));
+    if (sibl->IsContentOfType(nsIContent::eELEMENT)) {
+      return PR_FALSE;
+    }
+  }
+
+  return PR_TRUE;
+}
+
 // static
 nsresult
 nsContentUtils::doReparentContentWrapper(nsIContent *aChild,
@@ -761,11 +805,56 @@ nsContentUtils::ReparentContentWrapper(nsIContent *aContent,
                                   obj);
 }
 
+void
+nsContentUtils::GetDocShellFromCaller(nsIDocShell** aDocShell)
+{
+  *aDocShell = nsnull;
+  if (!sThreadJSContextStack) {
+    return;
+  }
+
+  JSContext *cx = nsnull;
+  sThreadJSContextStack->Peek(&cx);
+
+  if (cx) {
+    nsCOMPtr<nsIScriptGlobalObject> sgo;
+    GetDynamicScriptGlobal(cx, getter_AddRefs(sgo));
+
+    if (sgo) {
+      sgo->GetDocShell(aDocShell);
+    }
+  }
+}
+
+void
+nsContentUtils::GetDocumentFromCaller(nsIDOMDocument** aDocument)
+{
+  *aDocument = nsnull;
+  if (!sThreadJSContextStack) {
+    return;
+  }
+
+  JSContext *cx = nsnull;
+  sThreadJSContextStack->Peek(&cx);
+
+  if (cx) {
+    nsCOMPtr<nsIScriptGlobalObject> sgo;
+    GetDynamicScriptGlobal(cx, getter_AddRefs(sgo));
+
+    nsCOMPtr<nsIDOMWindowInternal> win(do_QueryInterface(sgo));
+    if (!win) {
+      return;
+    }
+
+    win->GetDocument(aDocument);
+  }
+}
+
 PRBool
 nsContentUtils::IsCallerChrome()
 {
   if (!sSecurityManager) {
-    return NS_OK;
+    return PR_FALSE;
   }
 
   PRBool is_caller_chrome = PR_FALSE;
@@ -1031,3 +1120,96 @@ nsContentUtils::TrimCharsInSet(const char* aSet,
   // valueEnd should point to the char after the last to copy
   return Substring(valueCurrent, valueEnd);
 }
+
+void
+nsCxPusher::Push(nsISupports *aCurrentTarget)
+{
+  if (mScx) {
+    NS_ERROR("Whaaa! No double pushing with nsCxPusher::Push()!");
+
+    return;
+  }
+
+  nsCOMPtr<nsIScriptGlobalObject> sgo;
+  nsCOMPtr<nsIContent> content(do_QueryInterface(aCurrentTarget));
+  nsCOMPtr<nsIDocument> document;
+
+  if (content) {
+    content->GetDocument(*getter_AddRefs(document));
+  }
+
+  if (!document) {
+    document = do_QueryInterface(aCurrentTarget);
+  }
+
+  if (document) {
+    document->GetScriptGlobalObject(getter_AddRefs(sgo));
+  }
+
+  if (!document && !sgo) {
+    sgo = do_QueryInterface(aCurrentTarget);
+  }
+
+  JSContext *cx = nsnull;
+
+  if (sgo) {
+    sgo->GetContext(getter_AddRefs(mScx));
+
+    if (mScx) {
+      cx = (JSContext *)mScx->GetNativeContext();
+    }
+  }
+
+  if (cx) {
+    if (!mStack) {
+      mStack = do_GetService(kJSStackContractID);
+    }
+
+    if (mStack) {
+      JSContext *current = nsnull;
+      mStack->Peek(&current);
+
+      if (current) {
+        // If there's a context on the stack, that means that a script
+        // is running at the moment.
+
+        mScriptIsRunning = PR_TRUE;
+      }
+
+      mStack->Push(cx);
+    }
+  } else {
+    // If there's no native context in the script context it must be
+    // in the process or being torn down. We don't want to notify the
+    // script context about scripts having been evaluated in such a
+    // case, so null out mScx.
+
+    mScx = nsnull;
+  }
+}
+
+void
+nsCxPusher::Pop()
+{
+  if (!mScx || !mStack) {
+    mScx = nsnull;
+
+    NS_ASSERTION(!mScriptIsRunning, "Huh, this can't be happening, "
+                 "mScriptIsRunning can't be set here!");
+
+    return;
+  }
+
+  JSContext *unused;
+  mStack->Pop(&unused);
+
+  if (!mScriptIsRunning) {
+    // No JS is running, but executing the event handler might have
+    // caused some JS to run. Tell the script context that it's done.
+
+    mScx->ScriptEvaluated(PR_TRUE);
+  }
+
+  mScx = nsnull;
+  mScriptIsRunning = PR_FALSE;
+}
