@@ -173,8 +173,8 @@ public:
                          PRBool aCompileEventHandlers);
   NS_IMETHOD GetParent(nsIContent*& aResult) const;
   NS_IMETHOD SetParent(nsIContent* aParent);
-  NS_IMETHOD_(PRBool) IsAnonymous() const;
-  NS_IMETHOD_(void) SetAnonymous(PRBool aAnonymous);
+  NS_IMETHOD_(PRBool) IsNativeAnonymous() const;
+  NS_IMETHOD_(void) SetNativeAnonymous(PRBool aAnonymous);
   NS_IMETHOD GetNameSpaceID(PRInt32& aID) const;
   NS_IMETHOD NormalizeAttrString(const nsAString& aStr,
                                  nsINodeInfo*& aNodeInfo);
@@ -294,12 +294,14 @@ private:
 
   PRBool HasRangeList() const
   {
-    return mParentPtrBits & PARENT_BIT_RANGELISTS;
+    return (mParentPtrBits & PARENT_BIT_RANGELISTS &&
+            nsGenericElement::sRangeListsHash.ops);
   }
 
   PRBool HasEventListenerManager() const
   {
-    return mParentPtrBits & PARENT_BIT_LISTENERMANAGER;
+    return (mParentPtrBits & PARENT_BIT_LISTENERMANAGER &&
+            nsGenericElement::sEventListenerManagersHash.ops);
   }
 
   // Weak parent pointer (nsIContent *) and bits for knowing if
