@@ -93,7 +93,7 @@ static PRInt32 numAllocFromPool=0;
 // allocate the memory for the object from the recycler, if possible
 // otherwise, just grab it from the heap.
 void* 
-nsDOMEvent::operator new(size_t aSize)
+nsDOMEvent::operator new(size_t aSize) CPP_THROW_NEW
 {
 
 #ifdef NS_DEBUG
@@ -102,7 +102,7 @@ nsDOMEvent::operator new(size_t aSize)
 
   void *result = nsnull;
 
-  if (!gEventPoolInUse) {
+  if (!gEventPoolInUse && aSize <= sizeof(gEventPool)) {
 #ifdef NS_DEBUG
     numAllocFromPool++;
 #endif
@@ -117,7 +117,7 @@ nsDOMEvent::operator new(size_t aSize)
   }
   
   if (result) {
-    nsCRT::zero(result, aSize);
+    memset(result, 0, aSize);
   }
 
   return result;
@@ -146,9 +146,9 @@ nsDOMEvent::operator delete(void* aPtr)
 
 
 nsDOMEvent::nsDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
-                       const nsAReadableString& aEventType) 
+                       const nsAString& aEventType) 
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mPresContext = aPresContext;
   mEventIsTrusted = PR_FALSE;
@@ -184,6 +184,10 @@ nsDOMEvent::nsDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
       mEvent = PR_NEWZAP(nsMutationEvent);
       mEvent->eventStructType = NS_MUTATION_EVENT;
     }
+    else if (eventType.EqualsIgnoreCase("PopupEvents")) {
+      mEvent = PR_NEWZAP(nsGUIEvent);
+      mEvent->eventStructType = NS_POPUP_EVENT;
+    }
     else {
       mEvent = PR_NEWZAP(nsEvent);
       mEvent->eventStructType = NS_EVENT;
@@ -294,7 +298,7 @@ NS_INTERFACE_MAP_BEGIN(nsDOMEvent)
 NS_INTERFACE_MAP_END
 
 // nsIDOMEventInterface
-NS_METHOD nsDOMEvent::GetType(nsAWritableString& aType)
+NS_METHOD nsDOMEvent::GetType(nsAString& aType)
 {
   const char* mName = GetEventName(mEvent->message);
 
@@ -332,7 +336,7 @@ NS_METHOD nsDOMEvent::GetTarget(nsIDOMEventTarget** aTarget)
   if (targetContent) {    
     if (NS_OK == targetContent->QueryInterface(NS_GET_IID(nsIDOMEventTarget), (void**)&mTarget)) {
       *aTarget = mTarget;
-      NS_ADDREF(mTarget);
+      NS_ADDREF(*aTarget);
     }
   }
   else {
@@ -603,7 +607,9 @@ nsDOMEvent::GetReconversionReply(nsReconversionEventReply** aReply)
 NS_METHOD nsDOMEvent::GetScreenX(PRInt32* aScreenX)
 {
   if (!mEvent || 
-       (mEvent->eventStructType != NS_MOUSE_EVENT && mEvent->eventStructType != NS_DRAGDROP_EVENT) ) {
+       (mEvent->eventStructType != NS_MOUSE_EVENT &&
+        mEvent->eventStructType != NS_POPUP_EVENT &&
+        mEvent->eventStructType != NS_DRAGDROP_EVENT)) {
     *aScreenX = 0;
     return NS_OK;
   }
@@ -625,7 +631,9 @@ NS_METHOD nsDOMEvent::GetScreenX(PRInt32* aScreenX)
 NS_METHOD nsDOMEvent::GetScreenY(PRInt32* aScreenY)
 {
   if (!mEvent || 
-        (mEvent->eventStructType != NS_MOUSE_EVENT && mEvent->eventStructType != NS_DRAGDROP_EVENT) ) {
+       (mEvent->eventStructType != NS_MOUSE_EVENT &&
+        mEvent->eventStructType != NS_POPUP_EVENT &&
+        mEvent->eventStructType != NS_DRAGDROP_EVENT)) {
     *aScreenY = 0;
     return NS_OK;
   }
@@ -647,9 +655,9 @@ NS_METHOD nsDOMEvent::GetScreenY(PRInt32* aScreenY)
 NS_METHOD nsDOMEvent::GetClientX(PRInt32* aClientX)
 {
   if (!mEvent || 
-       (mEvent->eventStructType != NS_MOUSE_EVENT 
-        && mEvent->eventStructType != NS_EVENT 
-        && mEvent->eventStructType != NS_DRAGDROP_EVENT) ||
+       (mEvent->eventStructType != NS_MOUSE_EVENT &&
+        mEvent->eventStructType != NS_POPUP_EVENT &&
+        mEvent->eventStructType != NS_DRAGDROP_EVENT) ||
       !mPresContext) {
     *aClientX = 0;
     return NS_OK;
@@ -700,10 +708,10 @@ NS_METHOD nsDOMEvent::GetClientX(PRInt32* aClientX)
 NS_METHOD nsDOMEvent::GetClientY(PRInt32* aClientY)
 {
   if (!mEvent || 
-       (mEvent->eventStructType != NS_MOUSE_EVENT 
-        && mEvent->eventStructType != NS_EVENT 
-        && mEvent->eventStructType != NS_DRAGDROP_EVENT) ||
-      !mPresContext) {
+       (mEvent->eventStructType != NS_MOUSE_EVENT &&
+        mEvent->eventStructType != NS_POPUP_EVENT &&
+        mEvent->eventStructType != NS_DRAGDROP_EVENT) ||
+       !mPresContext) {
     *aClientY = 0;
     return NS_OK;
   }
@@ -934,10 +942,10 @@ NS_METHOD nsDOMEvent::GetPageX(PRInt32* aPageX)
 {
   nsresult ret = NS_OK;
   PRInt32 scrollX = 0;
-  nsCOMPtr<nsIScrollableView> view;
+  nsIScrollableView* view = nsnull;
   float p2t, t2p;
 
-  GetScrollInfo(getter_AddRefs(view), &p2t, &t2p);
+  GetScrollInfo(&view, &p2t, &t2p);
   if(view) {
     nscoord xPos, yPos;
     ret = view->GetScrollPosition(xPos, yPos);
@@ -959,10 +967,10 @@ NS_METHOD nsDOMEvent::GetPageY(PRInt32* aPageY)
 {
   nsresult ret = NS_OK;
   PRInt32 scrollY = 0;
-  nsCOMPtr<nsIScrollableView> view;
+  nsIScrollableView* view = nsnull;
   float p2t, t2p;
 
-  GetScrollInfo(getter_AddRefs(view), &p2t, &t2p);
+  GetScrollInfo(&view, &p2t, &t2p);
   if(view) {
     nscoord xPos, yPos;
     ret = view->GetScrollPosition(xPos, yPos);
@@ -1133,7 +1141,7 @@ NS_METHOD nsDOMEvent::GetPreventDefault(PRBool* aReturn)
 }
 
 nsresult
-nsDOMEvent::SetEventType(const nsAReadableString& aEventTypeArg)
+nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
 {
   nsCOMPtr<nsIAtom> atom(dont_AddRef(NS_NewAtom(NS_LITERAL_STRING("on") + aEventTypeArg)));
 
@@ -1226,7 +1234,7 @@ nsDOMEvent::SetEventType(const nsAReadableString& aEventTypeArg)
 }
 
 NS_IMETHODIMP
-nsDOMEvent::InitEvent(const nsAReadableString& aEventTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg)
+nsDOMEvent::InitEvent(const nsAString& aEventTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg)
 {
   NS_ENSURE_SUCCESS(SetEventType(aEventTypeArg), NS_ERROR_FAILURE);
   mEvent->flags |= aCanBubbleArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_BUBBLE;
@@ -1236,14 +1244,14 @@ nsDOMEvent::InitEvent(const nsAReadableString& aEventTypeArg, PRBool aCanBubbleA
 }
 
 NS_IMETHODIMP
-nsDOMEvent::InitUIEvent(const nsAReadableString& aTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg, 
+nsDOMEvent::InitUIEvent(const nsAString& aTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg, 
                         nsIDOMAbstractView* aViewArg, PRInt32 aDetailArg)
 {
   return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
-nsDOMEvent::InitMouseEvent(const nsAReadableString & aTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg, 
+nsDOMEvent::InitMouseEvent(const nsAString & aTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg, 
                            nsIDOMAbstractView *aViewArg, PRInt32 aDetailArg, PRInt32 aScreenXArg, 
                            PRInt32 aScreenYArg, PRInt32 aClientXArg, PRInt32 aClientYArg, 
                            PRBool aCtrlKeyArg, PRBool aAltKeyArg, PRBool aShiftKeyArg, 
@@ -1282,7 +1290,7 @@ nsDOMEvent::InitMouseEvent(const nsAReadableString & aTypeArg, PRBool aCanBubble
 }
 
 NS_IMETHODIMP
-nsDOMEvent::InitKeyEvent(const nsAReadableString& aTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg, 
+nsDOMEvent::InitKeyEvent(const nsAString& aTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg, 
                          nsIDOMAbstractView* aViewArg, PRBool aCtrlKeyArg, PRBool aAltKeyArg, 
                          PRBool aShiftKeyArg, PRBool aMetaKeyArg, 
                          PRUint32 aKeyCodeArg, PRUint32 aCharCodeArg)
@@ -1505,7 +1513,7 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
 
 nsresult NS_NewDOMUIEvent(nsIDOMEvent** aInstancePtrResult,
                           nsIPresContext* aPresContext,
-                          const nsAReadableString& aEventType,
+                          const nsAString& aEventType,
                           nsEvent *aEvent) 
 {
   nsDOMEvent* it = new nsDOMEvent(aPresContext, aEvent, aEventType);
