@@ -66,6 +66,10 @@
 #include "nsLayoutAtoms.h"
 #include "nsINameSpaceManager.h"
 #include "nsIContent.h"
+#include "nsIFrame.h"
+#include "nsIView.h"
+#include "nsIViewManager.h"
+#include "nsIScrollableView.h"
 #include "nsCOMPtr.h"
 #include "nsIServiceManager.h"
 #include "nsIScriptSecurityManager.h"
@@ -84,6 +88,7 @@
 #include "nsIBoxObject.h"
 #include "nsIDOMNSDocument.h"
 #include "nsIWidget.h"
+#include "nsContentUtils.h"
 
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
@@ -100,7 +105,7 @@ nsEventListenerManager::nsEventListenerManager()
   mListenersRemoved = PR_FALSE;
 
   mTarget = nsnull;
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 static PRBool PR_CALLBACK
@@ -1171,49 +1176,9 @@ nsEventListenerManager::HandleEventSubType(nsListenerStruct* aListenerStruct,
     }
   }
 
-  nsCOMPtr<nsIScriptGlobalObject> sgo;
-  nsCOMPtr<nsIContent> content(do_QueryInterface(aCurrentTarget));
-  nsCOMPtr<nsIDocument> document;
-
-  if (content) {
-    content->GetDocument(*getter_AddRefs(document));
-  }
-
-  if (!document) {
-    document = do_QueryInterface(aCurrentTarget);
-  }
-
-  if (document) {
-    document->GetScriptGlobalObject(getter_AddRefs(sgo));
-  }
-
-  if (!sgo) {
-    sgo = do_QueryInterface(aCurrentTarget);
-  }
-
-  nsCOMPtr<nsIScriptContext> scx;
-  JSContext *cx = nsnull;
-
-  if (sgo) {
-    sgo->GetContext(getter_AddRefs(scx));
-
-    if (scx) {
-      cx = (JSContext *)scx->GetNativeContext();
-    }
-  }
-
-  nsCOMPtr<nsIJSContextStack> stack;
-  JSContext *current_cx = nsnull;
-
-  if (cx) {
-    stack = do_GetService("@mozilla.org/js/xpc/ContextStack;1");
-
-    if (stack) {
-      stack->Peek(&current_cx);
-
-      stack->Push(cx);
-    }
-  }
+  // nsCxPusher will automatically push and pop the current cx onto the
+  // context stack
+  nsCxPusher pusher(aCurrentTarget);
 
   if (NS_SUCCEEDED(result)) {
     nsCOMPtr<nsIPrivateDOMEvent> aPrivDOMEvent(do_QueryInterface(aDOMEvent));
@@ -1222,17 +1187,6 @@ nsEventListenerManager::HandleEventSubType(nsListenerStruct* aListenerStruct,
     aPrivDOMEvent->SetCurrentTarget(nsnull);
   }
 
-  if (cx && stack) {
-    stack->Pop(&cx);
-
-    if (!current_cx) {
-      // No JS is running, but executing the event handler might have
-      // caused some JS to run. Tell the script context that it's done.
-
-      scx->ScriptEvaluated(PR_TRUE);
-    }
-  }
-
   return result;
 }
 
@@ -1457,8 +1411,8 @@ nsresult nsEventListenerManager::HandleEvent(nsIPresContext* aPresContext,
         nsCOMPtr<nsIDOMEventTarget> currentTarget ( aCurrentTarget );
         nsCOMPtr<nsIDOMElement> currentFocus;
         nsCOMPtr<nsIDocument> doc;
+        nsCOMPtr<nsIPresShell> shell;
         if ( aEvent->message == NS_CONTEXTMENU_KEY ) {
-          nsCOMPtr<nsIPresShell> shell;
           aPresContext->GetShell(getter_AddRefs(shell));
           shell->GetDocument(getter_AddRefs(doc));
           if ( doc ) {
@@ -1491,6 +1445,14 @@ nsresult nsEventListenerManager::HandleEvent(nsIPresContext* aPresContext,
         if (NS_OK == ret) {
           // update the target
           if ( currentFocus ) {
+            // Reset event coordinates relative to focused frame in view
+            nsPoint targetPt;
+            GetCoordinatesFor(currentFocus, aPresContext, shell, targetPt);
+            aEvent->point.x += targetPt.x - aEvent->refPoint.x;
+            aEvent->point.y += targetPt.y - aEvent->refPoint.y;
+            aEvent->refPoint.x = targetPt.x;
+            aEvent->refPoint.y = targetPt.y;
+
             currentTarget = do_QueryInterface(currentFocus);
             nsCOMPtr<nsIPrivateDOMEvent> pEvent ( do_QueryInterface(*aDOMEvent) );
             pEvent->SetTarget ( currentTarget );
@@ -2705,6 +2667,49 @@ nsEventListenerManager::HandleEvent(nsIDOMEvent *aEvent)
   return DispatchEvent(aEvent, &noDefault);
 }
 
+void nsEventListenerManager::GetCoordinatesFor(nsIDOMElement *aCurrentEl, 
+                                               nsIPresContext *aPresContext,
+                                               nsIPresShell *aPresShell, 
+                                               nsPoint& aTargetPt)
+{
+  nsCOMPtr<nsIContent> focusedContent(do_QueryInterface(aCurrentEl));
+  nsIFrame *frame = nsnull;
+  aPresShell->GetPrimaryFrameFor(focusedContent, &frame);
+  if (frame) {
+    nsIView *view;
+    frame->GetOffsetFromView(aPresContext, aTargetPt, &view);
+    float t2p;
+    aPresContext->GetTwipsToPixels(&t2p);
+
+    // Start context menu down and to the right from top left of frame
+    // use the lineheight. This is a good distance to move the context
+    // menu away from the top left corner of the frame. If we always 
+    // used the frame height, the context menu could end up far away,
+    // for example when we're focused on linked images.
+    nsCOMPtr<nsIViewManager> vm;
+    aPresShell->GetViewManager(getter_AddRefs(vm));
+    if (vm) {
+      nsIScrollableView* scrollableView = nsnull;
+      vm->GetRootScrollableView(&scrollableView);
+      nscoord extraDistance;
+      if (scrollableView) {
+        scrollableView->GetLineHeight(&extraDistance);
+      }
+      else {
+        // No scrollable view, use height of frame as fallback
+        nsRect frameRect;
+        frame->GetRect(frameRect);
+        extraDistance = frameRect.height;
+      }
+      aTargetPt.x += extraDistance;
+      aTargetPt.y += extraDistance;
+    }
+     // Convert to pixels using that scale
+    aTargetPt.x = NSTwipsToIntPixels(aTargetPt.x, t2p);
+    aTargetPt.y = NSTwipsToIntPixels(aTargetPt.y, t2p);
+  }
+}
+
 nsresult
 NS_NewEventListenerManager(nsIEventListenerManager** aInstancePtrResult) 
 {
