@@ -21,6 +21,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Daniel Glazman <glazman@netscape.com>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -48,7 +49,6 @@
 #include "nsGenericHTMLElement.h"
 #include "nsILink.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIEventStateManager.h"
@@ -56,6 +56,7 @@
 #include "nsIEventStateManager.h"
 #include "nsIDOMEvent.h"
 #include "nsNetUtil.h"
+#include "nsCRT.h"
 
 // For GetText().
 #include "nsIContentIterator.h"
@@ -66,7 +67,7 @@
 #include "nsIFrameManager.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
-#include "nsIHTMLAttributes.h"
+#include "nsHTMLAttributes.h"
 
 nsresult NS_NewContentIterator(nsIContentIterator** aInstancePtrResult);
 
@@ -112,18 +113,26 @@ public:
   NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
   NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
+
+  NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                     const nsAString& aValue,
+                     PRBool aNotify);
+  NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
+                     const nsAString& aValue,
+                     PRBool aNotify);
+  NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                       PRBool aNotify);
+
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
 #endif
 
 protected:
-  nsresult RegUnRegAccessKey(PRBool aDoReg);
-
   // The cached visited state
   nsLinkState mLinkState;
 
@@ -150,7 +159,7 @@ NS_NewHTMLAnchorElement(nsIHTMLContent** aInstancePtrResult,
     return rv;
   }
 
-  *aInstancePtrResult = NS_STATIC_CAST(nsIHTMLContent *, it);
+  *aInstancePtrResult = it;
   NS_ADDREF(*aInstancePtrResult);
 
   return NS_OK;
@@ -203,7 +212,7 @@ nsHTMLAnchorElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 
   CopyInnerTo(this, it, aDeep);
 
-  *aReturn = NS_STATIC_CAST(nsIDOMNode *, it);
+  *aReturn = it;
 
   NS_ADDREF(*aReturn);
 
@@ -220,93 +229,23 @@ NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Rev, rev)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Shape, shape)
 NS_IMPL_INT_ATTR(nsHTMLAnchorElement, TabIndex, tabindex)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Type, type)
+NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, AccessKey, accesskey)
 
 
-NS_IMETHODIMP
-nsHTMLAnchorElement::GetAccessKey(nsAWritableString& aValue)
-{
-  NS_STATIC_CAST(nsIHTMLContent *, this)->GetAttr(kNameSpaceID_None,
-                                                  nsHTMLAtoms::accesskey,
-                                                  aValue);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLAnchorElement::SetAccessKey(const nsAReadableString& aValue)
-{
-  RegUnRegAccessKey(PR_FALSE);
-
-  nsresult rv = NS_STATIC_CAST(nsIHTMLContent *,
-                               this)->SetAttr(kNameSpaceID_None,
-                                              nsHTMLAtoms::accesskey,
-                                              aValue,
-                                              PR_TRUE);
-
-  if (!aValue.IsEmpty()) {
-    RegUnRegAccessKey(PR_TRUE);
-  }
-
-  return rv;
-}
-
-
-// This goes and gets the proper PresContext in order
-// for it to get the EventStateManager so it can register 
-// the access key
-nsresult nsHTMLAnchorElement::RegUnRegAccessKey(PRBool aDoReg)
-{
-  // first check to see if it even has an acess key
-  nsAutoString accessKey;
-  nsresult rv;
-
-  rv = NS_STATIC_CAST(nsIContent *, this)->GetAttr(kNameSpaceID_None,
-                                                   nsHTMLAtoms::accesskey,
-                                                   accessKey);
-
-  if (NS_CONTENT_ATTR_NOT_THERE != rv) {
-    nsCOMPtr<nsIPresContext> presContext;
-    GetPresContext(this, getter_AddRefs(presContext));
-
-    // With a valid PresContext we can get the EVM 
-    // and register the access key
-    if (presContext) {
-      nsCOMPtr<nsIEventStateManager> stateManager;
-      presContext->GetEventStateManager(getter_AddRefs(stateManager));
-
-      if (stateManager) {
-        if (aDoReg) {
-          return stateManager->RegisterAccessKey(nsnull, this,
-                                                 (PRUint32)accessKey.First());
-        } else {
-          return stateManager->UnregisterAccessKey(nsnull, this,
-                                                   (PRUint32)accessKey.First());
-        }
-      }
-    }
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
 NS_IMETHODIMP
 nsHTMLAnchorElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
                                  PRBool aCompileEventHandlers)
 {
-  // The document gets set to null before it is destroyed,
-  // so we unregister the the access key here (if it has one)
-  // before setting it to null
-  if (aDocument == nsnull) {
+  // Unregister the access key for the old document.
+  if (mDocument) {
     RegUnRegAccessKey(PR_FALSE);
   }
 
-  nsresult rv;
-  rv = nsGenericHTMLContainerElement::SetDocument(aDocument,
-                                                  aDeep,
-                                                  aCompileEventHandlers);
+  nsresult rv = nsGenericHTMLContainerElement::SetDocument(aDocument, aDeep,
+                                                        aCompileEventHandlers);
 
-  // Register the access key here (if it has one) 
-  // if the document isn't null
-  if (aDocument != nsnull) {
+  // Register the access key for the new document.
+  if (mDocument) {
     RegUnRegAccessKey(PR_TRUE);
   }
 
@@ -389,7 +328,7 @@ nsHTMLAnchorElement::RemoveFocus(nsIPresContext* aPresContext)
 
 NS_IMETHODIMP
 nsHTMLAnchorElement::StringToAttribute(nsIAtom* aAttribute,
-                                       const nsAReadableString& aValue,
+                                       const nsAString& aValue,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::tabindex) {
@@ -398,8 +337,8 @@ nsHTMLAnchorElement::StringToAttribute(nsIAtom* aAttribute,
     }
   }
   else if (aAttribute == nsHTMLAtoms::suppress) {
-    if (Compare(aValue,NS_LITERAL_STRING("true"),
-                nsCaseInsensitiveStringComparator())) {
+    if (!aValue.Equals(NS_LITERAL_STRING("true"),
+                       nsCaseInsensitiveStringComparator())) {
       aResult.SetEmptyValue();  // XXX? shouldn't just leave "true"
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
@@ -420,7 +359,7 @@ nsHTMLAnchorElement::HandleDOMEvent(nsIPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::GetHref(nsAWritableString& aValue)
+nsHTMLAnchorElement::GetHref(nsAString& aValue)
 {
   char *buf;
   nsresult rv = GetHrefCString(buf);
@@ -434,26 +373,18 @@ nsHTMLAnchorElement::GetHref(nsAWritableString& aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetHref(const nsAReadableString& aValue)
+nsHTMLAnchorElement::SetHref(const nsAString& aValue)
 {
-  // Clobber our "cache", so we'll recompute it the next time
-  // somebody asks for it.
-  mLinkState = eLinkState_Unknown;
-
-  return NS_STATIC_CAST(nsIContent *, this)->SetAttr(kNameSpaceID_HTML,
-                                                     nsHTMLAtoms::href,
-                                                     aValue, PR_TRUE);
+  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::href, aValue, PR_TRUE);
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::GetTarget(nsAWritableString& aValue)
+nsHTMLAnchorElement::GetTarget(nsAString& aValue)
 {
   aValue.Truncate();
 
   nsresult rv;
-  rv = NS_STATIC_CAST(nsIContent *, this)->GetAttr(kNameSpaceID_HTML,
-                                                   nsHTMLAtoms::target,
-                                                   aValue);
+  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::target, aValue);
   if (rv == NS_CONTENT_ATTR_NOT_THERE && mDocument) {
     rv = mDocument->GetBaseTarget(aValue);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -462,11 +393,9 @@ nsHTMLAnchorElement::GetTarget(nsAWritableString& aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetTarget(const nsAReadableString& aValue)
+nsHTMLAnchorElement::SetTarget(const nsAString& aValue)
 {
-  return NS_STATIC_CAST(nsIContent *, this)->SetAttr(kNameSpaceID_HTML,
-                                                     nsHTMLAtoms::target,
-                                                     aValue, PR_TRUE);
+  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::target, aValue, PR_TRUE);
 }
 
 #ifdef DEBUG
@@ -481,7 +410,7 @@ nsHTMLAnchorElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
 
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetProtocol(nsAWritableString& aProtocol)
+nsHTMLAnchorElement::GetProtocol(nsAString& aProtocol)
 {
   nsAutoString href;
 
@@ -496,7 +425,7 @@ nsHTMLAnchorElement::GetProtocol(nsAWritableString& aProtocol)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetProtocol(const nsAReadableString& aProtocol)
+nsHTMLAnchorElement::SetProtocol(const nsAString& aProtocol)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -512,7 +441,7 @@ nsHTMLAnchorElement::SetProtocol(const nsAReadableString& aProtocol)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetHost(nsAWritableString& aHost)
+nsHTMLAnchorElement::GetHost(nsAString& aHost)
 {
   nsAutoString href;
   
@@ -524,7 +453,7 @@ nsHTMLAnchorElement::GetHost(nsAWritableString& aHost)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetHost(const nsAReadableString& aHost)
+nsHTMLAnchorElement::SetHost(const nsAString& aHost)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -540,7 +469,7 @@ nsHTMLAnchorElement::SetHost(const nsAReadableString& aHost)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetHostname(nsAWritableString& aHostname)
+nsHTMLAnchorElement::GetHostname(nsAString& aHostname)
 {
   nsAutoString href;
   nsresult rv = GetHref(href);
@@ -551,7 +480,7 @@ nsHTMLAnchorElement::GetHostname(nsAWritableString& aHostname)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetHostname(const nsAReadableString& aHostname)
+nsHTMLAnchorElement::SetHostname(const nsAString& aHostname)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -567,7 +496,7 @@ nsHTMLAnchorElement::SetHostname(const nsAReadableString& aHostname)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetPathname(nsAWritableString& aPathname)
+nsHTMLAnchorElement::GetPathname(nsAString& aPathname)
 {
   nsAutoString href;
  
@@ -579,7 +508,7 @@ nsHTMLAnchorElement::GetPathname(nsAWritableString& aPathname)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetPathname(const nsAReadableString& aPathname)
+nsHTMLAnchorElement::SetPathname(const nsAString& aPathname)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -595,7 +524,7 @@ nsHTMLAnchorElement::SetPathname(const nsAReadableString& aPathname)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetSearch(nsAWritableString& aSearch)
+nsHTMLAnchorElement::GetSearch(nsAString& aSearch)
 {
   nsAutoString href;
 
@@ -607,7 +536,7 @@ nsHTMLAnchorElement::GetSearch(nsAWritableString& aSearch)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetSearch(const nsAReadableString& aSearch)
+nsHTMLAnchorElement::SetSearch(const nsAString& aSearch)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -624,7 +553,7 @@ nsHTMLAnchorElement::SetSearch(const nsAReadableString& aSearch)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetPort(nsAWritableString& aPort)
+nsHTMLAnchorElement::GetPort(nsAString& aPort)
 {
   nsAutoString href;
   
@@ -636,7 +565,7 @@ nsHTMLAnchorElement::GetPort(nsAWritableString& aPort)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetPort(const nsAReadableString& aPort)
+nsHTMLAnchorElement::SetPort(const nsAString& aPort)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -653,7 +582,7 @@ nsHTMLAnchorElement::SetPort(const nsAReadableString& aPort)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetHash(nsAWritableString& aHash)
+nsHTMLAnchorElement::GetHash(nsAString& aHash)
 {
   nsAutoString href;
 
@@ -665,7 +594,7 @@ nsHTMLAnchorElement::GetHash(nsAWritableString& aHash)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetHash(const nsAReadableString& aHash)
+nsHTMLAnchorElement::SetHash(const nsAString& aHash)
 {
   nsAutoString href, new_href;
   nsresult rv = GetHref(href);
@@ -682,7 +611,7 @@ nsHTMLAnchorElement::SetHash(const nsAReadableString& aHash)
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetText(nsAWritableString& aText)
+nsHTMLAnchorElement::GetText(nsAString& aText)
 {
   aText.Truncate();
 
@@ -727,7 +656,7 @@ nsHTMLAnchorElement::GetText(nsAWritableString& aText)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::ToString(nsAWritableString& aSource)
+nsHTMLAnchorElement::ToString(nsAString& aSource)
 {
   return GetHref(aSource);
 }
@@ -753,9 +682,7 @@ nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
   nsAutoString relURLSpec;
 
   if (NS_CONTENT_ATTR_HAS_VALUE ==
-      NS_STATIC_CAST(nsIContent *, this)->GetAttr(kNameSpaceID_HTML,
-                                                  nsHTMLAtoms::href,
-                                                  relURLSpec)) {
+      GetAttr(kNameSpaceID_None, nsHTMLAtoms::href, relURLSpec)) {
     // Clean up any leading or trailing whitespace
     relURLSpec.Trim(" \t\n\r");
 
@@ -765,9 +692,11 @@ nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
 
     if (baseURL) {
       // Get absolute URL.
-      NS_MakeAbsoluteURIWithCharset(&aBuf, relURLSpec, mDocument, baseURL,
+      nsCAutoString buf;
+      NS_MakeAbsoluteURIWithCharset(buf, relURLSpec, mDocument, baseURL,
                                     nsHTMLUtils::IOService,
                                     nsHTMLUtils::CharsetMgr);
+      aBuf = ToNewCString(buf);
     }
     else {
       // Absolute URL is same as relative URL.
@@ -781,3 +710,54 @@ nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
 
   return NS_OK;
 }
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                             const nsAString& aValue,
+                             PRBool aNotify)
+{
+  if (aName == nsHTMLAtoms::href && kNameSpaceID_None == aNameSpaceID) {
+    nsAutoString val;
+    GetHref(val);
+    if (!val.Equals(aValue)) {
+      SetLinkState(eLinkState_Unknown);
+    }
+  }
+
+  if (aName == nsHTMLAtoms::accesskey && kNameSpaceID_None == aNameSpaceID) {
+    RegUnRegAccessKey(PR_FALSE);
+  }
+
+  nsresult rv =
+      nsGenericHTMLElement::SetAttr(aNameSpaceID, aName, aValue, aNotify);
+
+  if (aName == nsHTMLAtoms::accesskey && kNameSpaceID_None == aNameSpaceID &&
+      !aValue.IsEmpty()) {
+    RegUnRegAccessKey(PR_TRUE);
+  }
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetAttr(nsINodeInfo* aNodeInfo,
+                             const nsAString& aValue,
+                             PRBool aNotify)
+{
+  return nsGenericHTMLElement::SetAttr(aNodeInfo, aValue, aNotify);
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute, PRBool aNotify)
+{
+  if (aAttribute == nsHTMLAtoms::href && kNameSpaceID_None == aNameSpaceID) {
+    SetLinkState(eLinkState_Unknown);
+  }
+
+  if (aAttribute == nsHTMLAtoms::accesskey &&
+      kNameSpaceID_None == aNameSpaceID) {
+    RegUnRegAccessKey(PR_FALSE);
+  }
+
+  return nsGenericHTMLElement::UnsetAttr(aNameSpaceID, aAttribute, aNotify);
+}
