@@ -61,7 +61,8 @@
 #include "nsIFormControl.h"
 #include "nsIForm.h"
 #include "nsIFormSubmission.h"
-#include "nsIGfxTextControlFrame.h"
+#include "nsITextControlFrame.h"
+#include "nsIRadioControlFrame.h"
 #include "nsIDocument.h"
 #include "nsIPresShell.h"
 #include "nsIFormControlFrame.h"
@@ -84,6 +85,7 @@
 #include "nsIImageControlFrame.h"
 #include "nsLinebreakConverter.h" //to strip out carriage returns
 #include "nsReadableUtils.h"
+#include "nsUnicharUtils.h"
 
 #include "nsIDOMMutationEvent.h"
 #include "nsIDOMEventReceiver.h"
@@ -102,13 +104,13 @@
 #include "nsIFile.h"
 #include "nsILocalFile.h"
 #include "nsIFileStreams.h"
-
+#include "nsNetUtil.h"
 
 // XXX align=left, hspace, vspace, border? other nav4 attrs
 
 static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
 
-typedef nsIGfxTextControlFrame2 textControlPlace;
+typedef nsITextControlFrame textControlPlace;
 
 //
 // Accessors for mBitField
@@ -157,8 +159,6 @@ public:
   NS_DECL_NSIDOMNSHTMLINPUTELEMENT
 
   // Overriden nsIFormControl methods
-  NS_IMETHOD SetForm(nsIDOMHTMLFormElement* aForm,
-                     PRBool aRemoveFromForm = PR_TRUE);
   NS_IMETHOD GetType(PRInt32* aType);
   NS_IMETHOD Reset();
   NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
@@ -177,7 +177,7 @@ public:
                                const nsHTMLValue& aValue,
                                nsAString& aResult) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
@@ -221,7 +221,7 @@ public:
   NS_IMETHOD DoneCreatingElement();
 
   // nsITextControlElement
-  NS_IMETHOD SetValueGuaranteed(const nsAString& aValue, nsIGfxTextControlFrame2* aFrame);
+  NS_IMETHOD SetValueGuaranteed(const nsAString& aValue, nsITextControlFrame* aFrame);
   NS_IMETHOD SetValueChanged(PRBool aValueChanged);
 
   // nsIRadioControlElement
@@ -235,17 +235,26 @@ public:
 protected:
   // Helper method
   NS_IMETHOD SetValueSecure(const nsAString& aValue,
-                            nsIGfxTextControlFrame2* aFrame,
+                            nsITextControlFrame* aFrame,
                             PRBool aCheckSecurity);
 
   nsresult GetSelectionRange(PRInt32* aSelectionStart, PRInt32* aSelectionEnd);
-  nsresult MouseClickForAltText(nsIPresContext* aPresContext);
   //Helper method
 #ifdef ACCESSIBILITY
   nsresult FireEventForAccessibility(nsIPresContext* aPresContext,
 			    		                       const nsAString& aEventType);
 #endif
 
+  /**
+   * Get the name if it exists and return whether it did exist
+   * @param aName the name returned [OUT]
+   * @param true if the name existed, false if not
+   */
+  PRBool GetNameIfExists(nsAString& aName) {
+    return GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, aName) !=
+           NS_CONTENT_ATTR_NOT_THERE;
+  }
+
   /**
    * Called when an attribute is about to be changed
    */
@@ -378,9 +387,6 @@ nsHTMLInputElement::nsHTMLInputElement(PRBool aFromParser)
 
 nsHTMLInputElement::~nsHTMLInputElement()
 {
-  // Null out form's pointer to us - no ref counting here!
-  SetForm(nsnull);
-
   if (mValue) {
     nsMemory::Free(mValue);
   }
@@ -503,6 +509,8 @@ nsHTMLInputElement::AfterSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
   }
 }
 
+// nsIDOMHTMLInputElement
+
 NS_IMETHODIMP
 nsHTMLInputElement::GetForm(nsIDOMHTMLFormElement** aForm)
 {
@@ -600,7 +608,7 @@ nsHTMLInputElement::GetValue(nsAString& aValue)
 
     PRBool frameOwnsValue = PR_FALSE;
     if (formControlFrame) {
-      nsIGfxTextControlFrame2* textControlFrame = nsnull;
+      nsITextControlFrame* textControlFrame = nsnull;
       CallQueryInterface(formControlFrame, &textControlFrame);
 
       if (textControlFrame) {
@@ -646,14 +654,14 @@ nsHTMLInputElement::SetValue(const nsAString& aValue)
 
 NS_IMETHODIMP
 nsHTMLInputElement::SetValueGuaranteed(const nsAString& aValue,
-                                       nsIGfxTextControlFrame2* aFrame)
+                                       nsITextControlFrame* aFrame)
 {
   return SetValueSecure(aValue, aFrame, PR_FALSE);
 }
 
 NS_IMETHODIMP
 nsHTMLInputElement::SetValueSecure(const nsAString& aValue,
-                                   nsIGfxTextControlFrame2* aFrame,
+                                   nsITextControlFrame* aFrame,
                                    PRBool aCheckSecurity)
 {
   PRInt32 type;
@@ -682,7 +690,7 @@ nsHTMLInputElement::SetValueSecure(const nsAString& aValue,
       }
     }
 
-    nsIGfxTextControlFrame2* textControlFrame = aFrame;
+    nsITextControlFrame* textControlFrame = aFrame;
     nsIFormControlFrame* formControlFrame = textControlFrame;
     if (!textControlFrame) {
       // No need to flush here, if there's no frame at this point we
@@ -828,8 +836,9 @@ nsHTMLInputElement::SetChecked(PRBool aChecked)
       nsCOMPtr<nsIRadioGroupContainer> container = GetRadioGroupContainer();
       if (container) {
         nsAutoString name;
-        GetName(name);
-        container->SetCurrentRadioButton(name, nsnull);
+        if (GetNameIfExists(name)) {
+          container->SetCurrentRadioButton(name, nsnull);
+        }
       }
     }
   } else {
@@ -851,9 +860,12 @@ nsHTMLInputElement::RadioSetChecked()
   nsCOMPtr<nsIRadioGroupContainer> container = GetRadioGroupContainer();
   // This is ONLY INITIALIZED IF container EXISTS
   nsAutoString name;
+  PRBool nameExists = PR_FALSE;
   if (container) {
-    GetName(name);
-    container->GetCurrentRadioButton(name, getter_AddRefs(currentlySelected));
+    nameExists = GetNameIfExists(name);
+    if (nameExists) {
+      container->GetCurrentRadioButton(name, getter_AddRefs(currentlySelected));
+    }
   }
 
   //
@@ -876,7 +888,7 @@ nsHTMLInputElement::RadioSetChecked()
   // Let the group know that we are now the One True Radio Button
   //
   NS_ENSURE_SUCCESS(rv, rv);
-  if (container) {
+  if (container && nameExists) {
     rv = container->SetCurrentRadioButton(name, this);
   }
 
@@ -895,14 +907,6 @@ nsHTMLInputElement::GetRadioGroupContainer()
   return retval;
 }
 
-NS_IMETHODIMP
-nsHTMLInputElement::SetForm(nsIDOMHTMLFormElement* aForm,
-                            PRBool aRemoveFromForm)
-{
-  return nsGenericHTMLLeafFormElement::SetForm(aForm, aRemoveFromForm);
-}
-
-
 nsresult
 nsHTMLInputElement::SetCheckedInternal(PRBool aChecked)
 {
@@ -1228,38 +1232,6 @@ nsHTMLInputElement::Click()
   return NS_OK;
 }
 
-nsresult
-nsHTMLInputElement::MouseClickForAltText(nsIPresContext* aPresContext)
-{
-  NS_ENSURE_ARG_POINTER(aPresContext);
-  PRBool disabled;
-
-  nsresult rv = GetDisabled(&disabled);
-  if (NS_FAILED(rv) || disabled) {
-    return rv;
-  }
-
-  // Generate a submit event targeted at the form content
-  nsCOMPtr<nsIContent> form(do_QueryInterface(mForm));
-
-  if (form) {
-    nsCOMPtr<nsIPresShell> shell;
-    aPresContext->GetShell(getter_AddRefs(shell));
-    if (shell) {
-      nsCOMPtr<nsIContent> formControl = this; // kungFuDeathGrip
-
-      nsFormEvent event;
-      event.eventStructType = NS_FORM_EVENT;
-      event.message         = NS_FORM_SUBMIT;
-      event.originator      = formControl;
-      nsEventStatus status  = nsEventStatus_eIgnore;
-      shell->HandleDOMEventWithTarget(form, &event, &status);
-    }
-  }
-
-  return rv;
-}
-
 NS_IMETHODIMP
 nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
                                    nsEvent* aEvent,
@@ -1408,9 +1380,10 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
           nsCOMPtr<nsIRadioGroupContainer> container = GetRadioGroupContainer();
           if (container) {
             nsAutoString name;
-            GetName(name);
-            container->GetCurrentRadioButton(name,
-                                             getter_AddRefs(selectedRadioButton));
+            if (GetNameIfExists(name)) {
+              container->GetCurrentRadioButton(name,
+                                               getter_AddRefs(selectedRadioButton));
+            }
           }
 
           PRBool checked;
@@ -1493,14 +1466,6 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
   // Bugscape 2369: type might have changed during event handler
   GetType(&type);
 
-  if (type == NS_FORM_INPUT_IMAGE && 
-      aEvent->message == NS_MOUSE_LEFT_BUTTON_UP && 
-      nsEventStatus_eIgnore == *aEventStatus &&
-      aFlags & NS_EVENT_FLAG_BUBBLE) {
-    // Tell the frame about the click
-    return MouseClickForAltText(aPresContext);
-  }
-
   if ((NS_OK == rv) && (nsEventStatus_eIgnore == *aEventStatus) &&
       !(aFlags & NS_EVENT_FLAG_CAPTURE)) {
     switch (aEvent->message) {
@@ -1508,9 +1473,14 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
       case NS_FOCUS_CONTENT:
       {
         if (formControlFrame) {
-          SET_BOOLBIT(mBitField, BF_SKIP_FOCUS_EVENT, PR_TRUE);
-          formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
-          SET_BOOLBIT(mBitField, BF_SKIP_FOCUS_EVENT, PR_FALSE);
+          // If this is a file control, check to see if focus has bubbled
+          // up from it's child textfield or child button.  If that's the case, 
+          // don't focus this parent file control -- leave focus on the child.
+          if (type != NS_FORM_INPUT_FILE || !(aFlags & NS_EVENT_FLAG_BUBBLE)) {
+            SET_BOOLBIT(mBitField, BF_SKIP_FOCUS_EVENT, PR_TRUE);
+            formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
+            SET_BOOLBIT(mBitField, BF_SKIP_FOCUS_EVENT, PR_FALSE);
+          }
           return NS_OK;
         }
       }                                                                         
@@ -1596,7 +1566,7 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
           if (mForm) {
             nsIFrame* primaryFrame = GetPrimaryFrame(PR_FALSE);
             if (primaryFrame) {
-              nsIGfxTextControlFrame2* textFrame = nsnull;
+              nsITextControlFrame* textFrame = nsnull;
               CallQueryInterface(primaryFrame, &textFrame);
 
               // Fire onChange (if necessary)
@@ -1610,11 +1580,16 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
             nsCOMPtr<nsIContent> submitControl;
             PRInt32 numTextControlsFound = 0;
 
+            nsCOMPtr<nsISimpleEnumerator> formControls;
+            mForm->GetControlEnumerator(getter_AddRefs(formControls));
+
+            nsCOMPtr<nsISupports> currentControlSupports;
             nsCOMPtr<nsIFormControl> currentControl;
-            PRUint32 count = 0;
-            mForm->GetElementCount(&count);
-            for (PRUint32 i=0; i < count; i++) {
-              mForm->GetElementAt(i, getter_AddRefs(currentControl));
+            PRBool hasMoreElements;
+            while (NS_SUCCEEDED(formControls->HasMoreElements(&hasMoreElements)) &&
+                   hasMoreElements) {
+              formControls->GetNext(getter_AddRefs(currentControlSupports));
+              currentControl = do_QueryInterface(currentControlSupports);
               if (currentControl) {
                 PRInt32 type;
                 currentControl->GetType(&type);
@@ -1695,22 +1670,23 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
           case NS_FORM_INPUT_SUBMIT:
           case NS_FORM_INPUT_IMAGE:
             {
-              nsFormEvent event;
-              event.eventStructType = NS_FORM_EVENT;
-              event.message         = (type == NS_FORM_INPUT_RESET) ? NS_FORM_RESET : NS_FORM_SUBMIT;
-              event.originator      = this;
-              nsEventStatus status  = nsEventStatus_eIgnore;
+              if (mForm) {
+                nsFormEvent event;
+                event.eventStructType = NS_FORM_EVENT;
+                event.message         = (type == NS_FORM_INPUT_RESET) ? NS_FORM_RESET : NS_FORM_SUBMIT;
+                event.originator      = this;
+                nsEventStatus status  = nsEventStatus_eIgnore;
 
-              nsCOMPtr<nsIPresShell> presShell;
-              aPresContext->GetShell(getter_AddRefs(presShell));
-              // If |nsIPresShell::Destroy| has been called due to
-              // handling the event (base class HandleDOMEvent, above),
-              // the pres context will return a null pres shell.  See
-              // bug 125624.
-              if (presShell) {
-                nsCOMPtr<nsIContent> form(do_QueryInterface(mForm));
-                presShell->HandleEventWithTarget(&event, nsnull, form, 
-                                                 NS_EVENT_FLAG_INIT, &status);
+                nsCOMPtr<nsIPresShell> presShell;
+                aPresContext->GetShell(getter_AddRefs(presShell));
+                // If |nsIPresShell::Destroy| has been called due to
+                // handling the event (base class HandleDOMEvent, above),
+                // the pres context will return a null pres shell.  See
+                // bug 125624.
+                if (presShell) {
+                  nsCOMPtr<nsIContent> form(do_QueryInterface(mForm));
+                  presShell->HandleDOMEventWithTarget(form, &event, &status);
+                }
               }
             }
             break;
@@ -1863,9 +1839,7 @@ nsHTMLInputElement::AttributeToString(nsIAtom* aAttribute,
       // The DOM spec says that input types should be capitalized but
       // AFAIK all existing browsers return the type in lower case,
       // http://bugzilla.mozilla.org/show_bug.cgi?id=32369 is the bug
-      // about this problem, we pass PR_FALSE as the last argument
-      // here to avoid capitalizing the input type (this is required for
-      // backwards compatibility). -- jst@netscape.com
+      // about this problem.  -- jst@netscape.com
       // Update.  The DOM spec will be changed to have input types be
       // all-lowercase.  See
       // http://bugzilla.mozilla.org/show_bug.cgi?id=113174#c12
@@ -1878,7 +1852,7 @@ nsHTMLInputElement::AttributeToString(nsIAtom* aAttribute,
   }
   else if (aAttribute == nsHTMLAtoms::align) {
     if (eHTMLUnit_Enumerated == aValue.GetUnit()) {
-      VAlignValueToString(aValue, aResult);
+      AlignValueToString(aValue, aResult);
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -1906,22 +1880,31 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
   aAttributes->GetAttribute(nsHTMLAtoms::type, value);
   if (eHTMLUnit_Enumerated == value.GetUnit()) {  
     switch (value.GetIntValue()) {
-      case NS_FORM_INPUT_IMAGE: {
+      case NS_FORM_INPUT_TEXT:
+      case NS_FORM_INPUT_PASSWORD:
+      case NS_FORM_INPUT_FILE:
+        nsGenericHTMLElement::MapDivAlignAttributeInto(aAttributes, aData);
+        break;
+      case NS_FORM_INPUT_IMAGE:
         nsGenericHTMLElement::MapImageBorderAttributeInto(aAttributes, aData);
         nsGenericHTMLElement::MapImageMarginAttributeInto(aAttributes, aData);
         nsGenericHTMLElement::MapImagePositionAttributeInto(aAttributes, aData);
+        // fall through
+      default:
+        nsGenericHTMLElement::MapAlignAttributeInto(aAttributes, aData);
         break;
-      }
     }
-  }
+  } else {
+    // No type attr means this is a text input
+    nsGenericHTMLElement::MapDivAlignAttributeInto(aAttributes, aData);
+  }    
 
-  nsGenericHTMLElement::MapAlignAttributeInto(aAttributes, aData);
   nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
 }
 
 NS_IMETHODIMP
 nsHTMLInputElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                             PRInt32& aHint) const
+                                             nsChangeHint& aHint) const
 {
   if (aAttribute == nsHTMLAtoms::value) {
     aHint = NS_STYLE_HINT_REFLOW;
@@ -2290,29 +2273,31 @@ nsHTMLInputElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
     PRInt32 clickedY;
     nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
 
-    nsCOMPtr<nsIImageControlFrame> imageControlFrame(
-        do_QueryInterface(formControlFrame));
-    if (imageControlFrame) {
-      imageControlFrame->GetClickedX(&clickedX);
-      imageControlFrame->GetClickedY(&clickedY);
+    nsIImageControlFrame* imageControlFrame = nsnull;
+    if (formControlFrame) {
+      CallQueryInterface(formControlFrame, &imageControlFrame);
+      if (imageControlFrame) {
+        imageControlFrame->GetClickedX(&clickedX);
+        imageControlFrame->GetClickedY(&clickedY);
 
-      // Convert the values to strings for submission
-      char buf[20];
-      sprintf(&buf[0], "%d", clickedX);
-      nsAutoString xVal = NS_ConvertASCIItoUCS2(buf);
-      sprintf(&buf[0], "%d", clickedY);
-      nsAutoString yVal = NS_ConvertASCIItoUCS2(buf);
+        // Convert the values to strings for submission
+        char buf[20];
+        sprintf(&buf[0], "%d", clickedX);
+        nsAutoString xVal = NS_ConvertASCIItoUCS2(buf);
+        sprintf(&buf[0], "%d", clickedY);
+        nsAutoString yVal = NS_ConvertASCIItoUCS2(buf);
 
-      if (!name.IsEmpty()) {
-        aFormSubmission->AddNameValuePair(this,
-                                          name + NS_LITERAL_STRING(".x"), xVal);
-        aFormSubmission->AddNameValuePair(this,
-                                          name + NS_LITERAL_STRING(".y"), yVal);
-      } else {
-        // If the Image Element has no name, simply return x and y
-        // to Nav and IE compatability.
-        aFormSubmission->AddNameValuePair(this, NS_LITERAL_STRING("x"), xVal);
-        aFormSubmission->AddNameValuePair(this, NS_LITERAL_STRING("y"), yVal);
+        if (!name.IsEmpty()) {
+          aFormSubmission->AddNameValuePair(this,
+                                            name + NS_LITERAL_STRING(".x"), xVal);
+          aFormSubmission->AddNameValuePair(this,
+                                            name + NS_LITERAL_STRING(".y"), yVal);
+        } else {
+          // If the Image Element has no name, simply return x and y
+          // to Nav and IE compatability.
+          aFormSubmission->AddNameValuePair(this, NS_LITERAL_STRING("x"), xVal);
+          aFormSubmission->AddNameValuePair(this, NS_LITERAL_STRING("y"), yVal);
+        }
       }
     }
   }
@@ -2337,15 +2322,26 @@ nsHTMLInputElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
   // Submit file if it's input type=file and this encoding method accepts files
   //
   if (type == NS_FORM_INPUT_FILE) {
-
     //
     // Open the file
     //
-    nsCOMPtr<nsILocalFile> file(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID,
-                                                  &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIFile> file;
+ 
+    if (Substring(value, 0, 5).Equals(NS_LITERAL_STRING("file:"),
+                                      nsCaseInsensitiveStringComparator())) {
+      // Converts the URL string into the corresponding nsIFile if possible.
+      // A local file will be created if the URL string begins with file://.
+      rv = NS_GetFileFromURLSpec(NS_ConvertUCS2toUTF8(value),
+                                 getter_AddRefs(file));
+    }
+
+    if (!file) {
+      // this is no "file://", try as local file
+      nsCOMPtr<nsILocalFile> localFile;
+      rv = NS_NewLocalFile(value, PR_FALSE, getter_AddRefs(localFile));
+      file = localFile;
+    }
 
-    rv = file->InitWithPath(value);
     if (NS_SUCCEEDED(rv)) {
 
       //
@@ -2629,8 +2625,9 @@ nsHTMLInputElement::AddedToRadioGroup()
   nsCOMPtr<nsIRadioGroupContainer> container = GetRadioGroupContainer();
   if (container) {
     nsAutoString name;
-    GetName(name);
-    container->AddToRadioGroup(name, NS_STATIC_CAST(nsIFormControl*, this));
+    if (GetNameIfExists(name)) {
+      container->AddToRadioGroup(name, NS_STATIC_CAST(nsIFormControl*, this));
+    }
   }
 
   return NS_OK;
@@ -2658,7 +2655,10 @@ nsHTMLInputElement::WillRemoveFromRadioGroup()
   PRBool gotName = PR_FALSE;
   if (checked) {
     if (!gotName) {
-      GetName(name);
+      if (!GetNameIfExists(name)) {
+        // If the name doesn't exist, nothing is going to happen anyway
+        return NS_OK;
+      }
       gotName = PR_TRUE;
     }
 
@@ -2674,7 +2674,10 @@ nsHTMLInputElement::WillRemoveFromRadioGroup()
   nsCOMPtr<nsIRadioGroupContainer> container = GetRadioGroupContainer();
   if (container) {
     if (!gotName) {
-      GetName(name);
+      if (!GetNameIfExists(name)) {
+        // If the name doesn't exist, nothing is going to happen anyway
+        return NS_OK;
+      }
       gotName = PR_TRUE;
     }
     container->RemoveFromRadioGroup(name,
@@ -2687,12 +2690,16 @@ nsHTMLInputElement::WillRemoveFromRadioGroup()
 nsresult
 nsHTMLInputElement::VisitGroup(nsIRadioVisitor* aVisitor)
 {
-  nsresult rv;
+  nsresult rv = NS_OK;
   nsCOMPtr<nsIRadioGroupContainer> container = GetRadioGroupContainer();
   if (container) {
     nsAutoString name;
-    GetName(name);
-    rv = container->WalkRadioGroup(name, aVisitor);
+    if (GetNameIfExists(name)) {
+      rv = container->WalkRadioGroup(name, aVisitor);
+    } else {
+      PRBool stop;
+      aVisitor->Visit(this, &stop);
+    }
   } else {
     PRBool stop;
     aVisitor->Visit(this, &stop);
