@@ -75,7 +75,7 @@ public:
                                const nsHTMLValue& aValue,
                                nsAString& aResult) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
@@ -242,7 +242,7 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
 
 NS_IMETHODIMP
 nsHTMLLIElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                          PRInt32& aHint) const
+                                          nsChangeHint& aHint) const
 {
   if (aAttribute == nsHTMLAtoms::type) {
     aHint = NS_STYLE_HINT_REFLOW;
