@@ -69,7 +69,11 @@
 #include "nsLayoutAtoms.h"
 #include "nsIEventStateManager.h"
 #include "nsXULAtoms.h"
+#include "nsIDOMDocument.h"
 
+/**
+ * Implementation of &lt;option&gt;
+ */
 class nsHTMLOptionElement : public nsGenericHTMLContainerElement,
                             public nsIDOMHTMLOptionElement,
                             public nsIJSNativeInitializer,
@@ -102,14 +106,13 @@ public:
                                const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
 #endif
 
   // nsIOptionElement
   NS_IMETHOD SetSelectedInternal(PRBool aValue, PRBool aNotify);
-  NS_IMETHOD GetValueOrText(nsAString& aValue);
 
   // nsIContent
   NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify, 
@@ -152,15 +155,21 @@ NS_NewHTMLOptionElement(nsIHTMLContent** aInstancePtrResult,
 
   /*
    * nsHTMLOptionElement's will be created without a nsINodeInfo passed in
-   * if someone creates new option elements in JavaScript, in a case like
-   * that we request the nsINodeInfo from the anonymous nodeinfo list.
+   * if someone says "var opt = new Option();" in JavaScript, in a case like
+   * that we request the nsINodeInfo from the document's nodeinfo list.
    */
+  nsresult rv;
   nsCOMPtr<nsINodeInfo> nodeInfo(aNodeInfo);
   if (!nodeInfo) {
+    nsCOMPtr<nsIDOMDocument> dom_doc;
+    nsContentUtils::GetDocumentFromCaller(getter_AddRefs(dom_doc));
+
+    nsCOMPtr<nsIDocument> doc(do_QueryInterface(dom_doc));
+    NS_ENSURE_TRUE(doc, NS_ERROR_UNEXPECTED);
+
     nsCOMPtr<nsINodeInfoManager> nodeInfoManager;
-    nsresult rv;
-    rv = nsNodeInfoManager::GetAnonymousManager(*getter_AddRefs(nodeInfoManager));
-    NS_ENSURE_SUCCESS(rv, rv);
+    doc->GetNodeInfoManager(*getter_AddRefs(nodeInfoManager));
+    NS_ENSURE_TRUE(nodeInfoManager, NS_ERROR_UNEXPECTED);
 
     rv = nodeInfoManager->GetNodeInfo(nsHTMLAtoms::option, nsnull,
                                       kNameSpaceID_None,
@@ -174,7 +183,7 @@ NS_NewHTMLOptionElement(nsIHTMLContent** aInstancePtrResult,
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  nsresult rv = NS_STATIC_CAST(nsGenericElement *, it)->Init(nodeInfo);
+  rv = NS_STATIC_CAST(nsGenericElement *, it)->Init(nodeInfo);
 
   if (NS_FAILED(rv)) {
     delete it;
@@ -275,23 +284,20 @@ nsHTMLOptionElement::SetSelectedInternal(PRBool aValue, PRBool aNotify)
 }
 
 NS_IMETHODIMP
-nsHTMLOptionElement::GetValueOrText(nsAString& aValue)
+nsHTMLOptionElement::SetValue(const nsAString& aValue)
 {
-  nsHTMLValue value;
-  nsresult rv = GetHTMLAttribute(nsHTMLAtoms::value, value);
+  SetAttr(kNameSpaceID_None, nsHTMLAtoms::value, aValue, PR_TRUE);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLOptionElement::GetValue(nsAString& aValue)
+{
+  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, aValue);
+  // If the value attr is there, that is *exactly* what we use.  If it is
+  // not, we compress whitespace .text.
   if (NS_CONTENT_ATTR_NOT_THERE == rv) {
-    // XXX When an equivalent of CompressWhitespace exists for nsAString,
-    // somebody please clean this up.  The only reason I can think that we're
-    // we're doing it anyway is because GetText() leaves whitespace around if
-    // the text is all whitespace we apparently want to compress even that.
-    nsAutoString getVal;
-    rv = GetText(getVal);
-    getVal.CompressWhitespace();
-    aValue = getVal;
-  } else {
-    // Need to use GetValue to get the real value because it does extra
-    // processing on the return value. (i.e it trims it.)
-    rv = GetValue(aValue);
+    GetText(aValue);
   }
 
   return NS_OK;
@@ -311,7 +317,11 @@ nsHTMLOptionElement::GetSelected(PRBool* aValue)
     // This does not need to be SetSelected (which sets selected in the select)
     // because we *will* be initialized when we are placed into a select.  Plus
     // it seems like that's just inviting an infinite loop.
-    SetSelectedInternal(selected, PR_TRUE);
+    // We can pass |aNotify == PR_FALSE| since |GetSelected| is called
+    // from |nsHTMLSelectElement::InsertOptionsIntoList|, which is
+    // guaranteed to be called before frames are created for the
+    // content.
+    SetSelectedInternal(selected, PR_FALSE);
   }
 
   *aValue = mIsSelected;
@@ -327,8 +337,12 @@ nsHTMLOptionElement::SetSelected(PRBool aValue)
   GetSelect(getter_AddRefs(selectElement));
   nsCOMPtr<nsISelectElement> selectInt(do_QueryInterface(selectElement));
   if (selectInt) {
+    PRInt32 index;
+    GetIndex(&index);
     // This should end up calling SetSelectedInternal
-    return selectInt->SetOptionSelected(this, aValue);
+    return selectInt->SetOptionsSelectedByIndex(index, index, aValue,
+                                                PR_FALSE, PR_TRUE, PR_TRUE,
+                                                nsnull);
   } else {
     return SetSelectedInternal(aValue, PR_TRUE);
   }
@@ -339,7 +353,7 @@ nsHTMLOptionElement::SetSelected(PRBool aValue)
 //NS_IMPL_BOOL_ATTR(nsHTMLOptionElement, DefaultSelected, defaultselected)
 //NS_IMPL_INT_ATTR(nsHTMLOptionElement, Index, index)
 //NS_IMPL_STRING_ATTR(nsHTMLOptionElement, Label, label)
-NS_IMPL_STRING_ATTR(nsHTMLOptionElement, Value, value)
+//NS_IMPL_STRING_ATTR(nsHTMLOptionElement, Value, value)
 
 NS_IMETHODIMP
 nsHTMLOptionElement::GetDisabled(PRBool* aDisabled)
@@ -477,7 +491,7 @@ nsHTMLOptionElement::StringToAttribute(nsIAtom* aAttribute,
 
 NS_IMETHODIMP
 nsHTMLOptionElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                              PRInt32& aHint) const
+                                              nsChangeHint& aHint) const
 {
   nsIFormControlFrame* fcFrame = GetSelectFrame();
   
@@ -505,13 +519,14 @@ nsHTMLOptionElement::GetText(nsAString& aText)
 {
   PRInt32 numNodes, i;
 
-  aText.SetLength(0);
+  aText.Truncate();
 
   nsresult rv = ChildCount(numNodes);
   if (NS_FAILED(rv)) {
     return rv;
   }
 
+  nsAutoString text;
   for (i = 0; i < numNodes; i++) {
     nsCOMPtr<nsIContent> node;
 
@@ -521,25 +536,22 @@ nsHTMLOptionElement::GetText(nsAString& aText)
       nsCOMPtr<nsIDOMText> domText(do_QueryInterface(node));
 
       if (domText) {
-        rv = domText->GetData(aText);
-
-        nsAutoString text(aText);
-
-        // the option could be all spaces, so compress the white space
-        // then make sure it's not empty, if the option is all
-        // whitespace then we return the whitespace
-
-        text.CompressWhitespace(PR_TRUE, PR_TRUE);
-
-        if (!text.IsEmpty()) {
-          aText.Assign(text);
-          break;
+        rv = domText->GetData(text);
+        if (NS_FAILED(rv)) {
+          aText.Truncate();
+          return rv;
         }
-      }
 
+        aText.Append(text);
+      }
     }
   }
 
+  // XXX No CompressWhitespace for nsAString.  Sad.
+  text = aText;
+  text.CompressWhitespace(PR_TRUE, PR_TRUE);
+  aText = text;
+
   return NS_OK;
 }
 
