@@ -81,7 +81,7 @@ public:
                                nsAString& aResult) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
 #endif
@@ -324,7 +324,7 @@ void ColMapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
 NS_IMETHODIMP
 nsHTMLTableColElement::GetMappedAttributeImpact(const nsIAtom* aAttribute,
                                                 PRInt32 aModType,
-                                                PRInt32& aHint) const
+                                                nsChangeHint& aHint) const
 {
   if ((aAttribute == nsHTMLAtoms::width) ||
       (aAttribute == nsHTMLAtoms::align) ||
