@@ -58,7 +58,6 @@
 #include "nsStyleConsts.h"
 #include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
-#include "nsStyleUtil.h"
 #include "nsIFontMetrics.h"
 #include "nsIDOMCSSStyleSheet.h"
 #include "nsIDOMCSSRule.h"
@@ -68,8 +67,8 @@
 #include "nsINameSpaceManager.h"
 #include "nsINameSpace.h"
 #include "nsILookAndFeel.h"
-#include "xp_core.h"
 #include "nsRuleNode.h"
+#include "nsUnicharUtils.h"
 
 #include "nsIStyleSet.h"
 #include "nsISizeOfHandler.h"
@@ -118,7 +117,6 @@ nsAtomList::nsAtomList(nsIAtom* aAtom)
     mNext(nsnull)
 {
   MOZ_COUNT_CTOR(nsAtomList);
-  NS_IF_ADDREF(mAtom);
 }
 
 nsAtomList::nsAtomList(const nsString& aAtomValue)
@@ -126,7 +124,7 @@ nsAtomList::nsAtomList(const nsString& aAtomValue)
     mNext(nsnull)
 {
   MOZ_COUNT_CTOR(nsAtomList);
-  mAtom = NS_NewAtom(aAtomValue);
+  mAtom = do_GetAtom(aAtomValue);
 }
 
 nsAtomList::nsAtomList(const nsAtomList& aCopy)
@@ -134,14 +132,12 @@ nsAtomList::nsAtomList(const nsAtomList& aCopy)
     mNext(nsnull)
 {
   MOZ_COUNT_CTOR(nsAtomList);
-  NS_IF_ADDREF(mAtom);
   NS_IF_COPY(mNext, aCopy.mNext, nsAtomList);
 }
 
 nsAtomList::~nsAtomList(void)
 {
   MOZ_COUNT_DTOR(nsAtomList);
-  NS_IF_RELEASE(mAtom);
   NS_IF_DELETE(mNext);
 }
 
@@ -161,6 +157,63 @@ PRBool nsAtomList::Equals(const nsAtomList* aOther) const
   return PR_FALSE;
 }
 
+MOZ_DECL_CTOR_COUNTER(nsAtomStringList)
+
+nsAtomStringList::nsAtomStringList(nsIAtom* aAtom, const PRUnichar* aString)
+  : mAtom(aAtom),
+    mString(nsnull),
+    mNext(nsnull)
+{
+  MOZ_COUNT_CTOR(nsAtomStringList);
+  if (aString)
+    mString = nsCRT::strdup(aString);
+}
+
+nsAtomStringList::nsAtomStringList(const nsString& aAtomValue,
+                                   const PRUnichar* aString)
+  : mAtom(nsnull),
+    mString(nsnull),
+    mNext(nsnull)
+{
+  MOZ_COUNT_CTOR(nsAtomStringList);
+  mAtom = do_GetAtom(aAtomValue);
+  if (aString)
+    mString = nsCRT::strdup(aString);
+}
+
+nsAtomStringList::nsAtomStringList(const nsAtomStringList& aCopy)
+  : mAtom(aCopy.mAtom),
+    mString(nsnull),
+    mNext(nsnull)
+{
+  MOZ_COUNT_CTOR(nsAtomStringList);
+  if (aCopy.mString)
+    mString = nsCRT::strdup(aCopy.mString);
+  NS_IF_COPY(mNext, aCopy.mNext, nsAtomStringList);
+}
+
+nsAtomStringList::~nsAtomStringList(void)
+{
+  MOZ_COUNT_DTOR(nsAtomStringList);
+  if (mString)
+    nsCRT::free(mString);
+  NS_IF_DELETE(mNext);
+}
+
+PRBool nsAtomStringList::Equals(const nsAtomStringList* aOther) const
+{
+  return (this == aOther) ||
+         (aOther &&
+          mAtom == aOther->mAtom &&
+          !mString == !aOther->mString &&
+          !mNext == !aOther->mNext &&
+          (!mNext || mNext->Equals(aOther->mNext)) &&
+          // Check strings last, since it's the slowest check.
+          (!mString || nsDependentString(mString).Equals(
+                                        nsDependentString(aOther->mString),
+                                        nsCaseInsensitiveStringComparator())));
+}
+
 MOZ_DECL_CTOR_COUNTER(nsAttrSelector)
 
 #ifdef DEBUG_REFS
@@ -337,10 +390,9 @@ nsCSSSelector::nsCSSSelector(const nsCSSSelector& aCopy)
     mNext(nsnull)
 {
   MOZ_COUNT_CTOR(nsCSSSelector);
-  NS_IF_ADDREF(mTag);
   NS_IF_COPY(mIDList, aCopy.mIDList, nsAtomList);
   NS_IF_COPY(mClassList, aCopy.mClassList, nsAtomList);
-  NS_IF_COPY(mPseudoClassList, aCopy.mPseudoClassList, nsAtomList);
+  NS_IF_COPY(mPseudoClassList, aCopy.mPseudoClassList, nsAtomStringList);
   NS_IF_COPY(mAttrList, aCopy.mAttrList, nsAttrSelector);
   NS_IF_COPY(mNegations, aCopy.mNegations, nsCSSSelector);
   
@@ -363,7 +415,6 @@ nsCSSSelector::~nsCSSSelector(void)
 
 nsCSSSelector& nsCSSSelector::operator=(const nsCSSSelector& aCopy)
 {
-  NS_IF_RELEASE(mTag);
   NS_IF_DELETE(mIDList);
   NS_IF_DELETE(mClassList);
   NS_IF_DELETE(mPseudoClassList);
@@ -374,12 +425,11 @@ nsCSSSelector& nsCSSSelector::operator=(const nsCSSSelector& aCopy)
   mTag          = aCopy.mTag;
   NS_IF_COPY(mIDList, aCopy.mIDList, nsAtomList);
   NS_IF_COPY(mClassList, aCopy.mClassList, nsAtomList);
-  NS_IF_COPY(mPseudoClassList, aCopy.mPseudoClassList, nsAtomList);
+  NS_IF_COPY(mPseudoClassList, aCopy.mPseudoClassList, nsAtomStringList);
   NS_IF_COPY(mAttrList, aCopy.mAttrList, nsAttrSelector);
   mOperator     = aCopy.mOperator;
   NS_IF_COPY(mNegations, aCopy.mNegations, nsCSSSelector);
 
-  NS_IF_ADDREF(mTag);
   return *this;
 }
 
@@ -447,7 +497,7 @@ PRBool nsCSSSelector::Equals(const nsCSSSelector* aOther) const
 void nsCSSSelector::Reset(void)
 {
   mNameSpace = kNameSpaceID_Unknown;
-  NS_IF_RELEASE(mTag);
+  mTag = nsnull;
   NS_IF_DELETE(mIDList);
   NS_IF_DELETE(mClassList);
   NS_IF_DELETE(mPseudoClassList);
@@ -463,15 +513,15 @@ void nsCSSSelector::SetNameSpace(PRInt32 aNameSpace)
 
 void nsCSSSelector::SetTag(const nsString& aTag)
 {
-  NS_IF_RELEASE(mTag);
-  if (0 < aTag.Length()) {
-    mTag = NS_NewAtom(aTag);
-  }
+  if (aTag.IsEmpty())
+    mTag = nsnull;
+  else
+    mTag = do_GetAtom(aTag);
 }
 
 void nsCSSSelector::AddID(const nsString& aID)
 {
-  if (0 < aID.Length()) {
+  if (!aID.IsEmpty()) {
     nsAtomList** list = &mIDList;
     while (nsnull != *list) {
       list = &((*list)->mNext);
@@ -482,7 +532,7 @@ void nsCSSSelector::AddID(const nsString& aID)
 
 void nsCSSSelector::AddClass(const nsString& aClass)
 {
-  if (0 < aClass.Length()) {
+  if (!aClass.IsEmpty()) {
     nsAtomList** list = &mClassList;
     while (nsnull != *list) {
       list = &((*list)->mNext);
@@ -491,31 +541,33 @@ void nsCSSSelector::AddClass(const nsString& aClass)
   }
 }
 
-void nsCSSSelector::AddPseudoClass(const nsString& aPseudoClass)
+void nsCSSSelector::AddPseudoClass(const nsString& aPseudoClass,
+                                   const PRUnichar* aString)
 {
-  if (0 < aPseudoClass.Length()) {
-    nsAtomList** list = &mPseudoClassList;
+  if (!aPseudoClass.IsEmpty()) {
+    nsAtomStringList** list = &mPseudoClassList;
     while (nsnull != *list) {
       list = &((*list)->mNext);
     }
-    *list = new nsAtomList(aPseudoClass);
+    *list = new nsAtomStringList(aPseudoClass, aString);
   }
 }
 
-void nsCSSSelector::AddPseudoClass(nsIAtom* aPseudoClass)
+void nsCSSSelector::AddPseudoClass(nsIAtom* aPseudoClass,
+                                   const PRUnichar* aString)
 {
   if (nsnull != aPseudoClass) {
-    nsAtomList** list = &mPseudoClassList;
+    nsAtomStringList** list = &mPseudoClassList;
     while (nsnull != *list) {
       list = &((*list)->mNext);
     }
-    *list = new nsAtomList(aPseudoClass);
+    *list = new nsAtomStringList(aPseudoClass, aString);
   }
 }
 
 void nsCSSSelector::AddAttribute(PRInt32 aNameSpace, const nsString& aAttr)
 {
-  if (0 < aAttr.Length()) {
+  if (!aAttr.IsEmpty()) {
     nsAttrSelector** list = &mAttrList;
     while (nsnull != *list) {
       list = &((*list)->mNext);
@@ -527,7 +579,7 @@ void nsCSSSelector::AddAttribute(PRInt32 aNameSpace, const nsString& aAttr)
 void nsCSSSelector::AddAttribute(PRInt32 aNameSpace, const nsString& aAttr, PRUint8 aFunc, 
                                  const nsString& aValue, PRBool aCaseSensitive)
 {
-  if (0 < aAttr.Length()) {
+  if (!aAttr.IsEmpty()) {
     nsAttrSelector** list = &mAttrList;
     while (nsnull != *list) {
       list = &((*list)->mNext);
@@ -558,10 +610,10 @@ PRInt32 nsCSSSelector::CalcWeight(void) const
     weight += 0x000100;
     list = list->mNext;
   }
-  list = mPseudoClassList;
-  while (nsnull != list) {
+  nsAtomStringList *plist = mPseudoClassList;
+  while (nsnull != plist) {
     weight += 0x000100;
-    list = list->mNext;
+    plist = plist->mNext;
   }
   nsAttrSelector* attr = mAttrList;
   while (nsnull != attr) {
@@ -648,7 +700,7 @@ void nsCSSSelector::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
     }
   }
   if(mPseudoClassList && uniqueItems->AddItem(mPseudoClassList)){
-    nsAtomList *pNext = nsnull;
+    nsAtomStringList *pNext = nsnull;
     pNext = mPseudoClassList;    
     while(pNext){
       if(pNext->mAtom && uniqueItems->AddItem(pNext->mAtom)){
@@ -695,7 +747,7 @@ static PRBool IsPseudoElement(nsIAtom* aAtom)
   return PR_FALSE;
 }
 
-void nsCSSSelector::AppendNegationToString(nsAWritableString& aString)
+void nsCSSSelector::AppendNegationToString(nsAString& aString)
 {
   aString.Append(NS_LITERAL_STRING(":not("));
 }
@@ -704,7 +756,7 @@ void nsCSSSelector::AppendNegationToString(nsAWritableString& aString)
 // Builds the textual representation of a selector. Called by DOM 2 CSS 
 // StyleRule:selectorText
 //
-nsresult nsCSSSelector::ToString( nsAWritableString& aString, nsICSSStyleSheet* aSheet, PRBool aIsPseudoElem,
+nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet, PRBool aIsPseudoElem,
                                   PRInt8 aNegatedIndex) const
 {
   const PRUnichar* temp;
@@ -833,11 +885,16 @@ nsresult nsCSSSelector::ToString( nsAWritableString& aString, nsICSSStyleSheet*
 
   // Append each pseudo-class in the linked list
   if (mPseudoClassList) {
-    nsAtomList* list = mPseudoClassList;
+    nsAtomStringList* list = mPseudoClassList;
     while (list != nsnull) {
       list->mAtom->GetUnicode(&temp);
       NS_IF_NEGATED_START(aIsNegated, aString)
       aString.Append(temp);
+      if (nsnull != list->mString) {
+        aString.Append(PRUnichar('('));
+        aString.Append(list->mString);
+        aString.Append(PRUnichar(')'));
+      }
       NS_IF_NEGATED_END(aIsNegated, aString)
       list = list->mNext;
     }
@@ -888,14 +945,8 @@ public:
 
   NS_DECL_ISUPPORTS
 
-//  NS_IMETHOD Equals(const nsIStyleRule* aRule, PRBool& aResult) const;
-//  NS_IMETHOD HashValue(PRUint32& aValue) const;
-
   NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const;
 
-  // Strength is an out-of-band weighting, useful for mapping CSS ! important
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
-
   // The new mapping function.
   NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
 
@@ -918,7 +969,7 @@ CSSImportantRule::CSSImportantRule(nsICSSStyleSheet* aSheet, nsCSSDeclaration* a
   : mDeclaration(aDeclaration),
     mSheet(aSheet)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 CSSImportantRule::~CSSImportantRule(void)
@@ -928,22 +979,6 @@ CSSImportantRule::~CSSImportantRule(void)
 
 NS_IMPL_ISUPPORTS1(CSSImportantRule, nsIStyleRule)
 
-#if 0
-NS_IMETHODIMP
-CSSImportantRule::Equals(const nsIStyleRule* aRule, PRBool& aResult) const
-{
-  aResult = PRBool(aRule == this);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-CSSImportantRule::HashValue(PRUint32& aValue) const
-{
-  aValue = PRUint32(mDeclaration);
-  return NS_OK;
-}
-#endif
-
 NS_IMETHODIMP
 CSSImportantRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
 {
@@ -952,14 +987,6 @@ CSSImportantRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
   return NS_OK;
 }
 
-// Strength is an out-of-band weighting, useful for mapping CSS ! important
-NS_IMETHODIMP
-CSSImportantRule::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 1;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 CSSImportantRule::MapRuleInfoInto(nsRuleData* aRuleData)
 {
@@ -1068,8 +1095,8 @@ public:
   DOMCSSDeclarationImpl(nsICSSStyleRule *aRule);
   ~DOMCSSDeclarationImpl(void);
 
-  NS_IMETHOD RemoveProperty(const nsAReadableString& aPropertyName, 
-                            nsAWritableString& aReturn);
+  NS_IMETHOD RemoveProperty(const nsAString& aPropertyName, 
+                            nsAString& aReturn);
 
   virtual void DropReference(void);
   virtual nsresult GetCSSDeclaration(nsCSSDeclaration **aDecl,
@@ -1081,9 +1108,9 @@ public:
                                             nsIURI** aURI,
                                             nsICSSLoader** aCSSLoader,
                                             nsICSSParser** aCSSParser);
-  virtual nsresult ParsePropertyValue(const nsAReadableString& aPropName,
-                                      const nsAReadableString& aPropValue);
-  virtual nsresult ParseDeclaration(const nsAReadableString& aDecl,
+  virtual nsresult ParsePropertyValue(const nsAString& aPropName,
+                                      const nsAString& aPropValue);
+  virtual nsresult ParseDeclaration(const nsAString& aDecl,
                                     PRBool aParseOnlyOneDecl,
                                     PRBool aClearOldDecl);
   virtual nsresult GetParent(nsISupports **aParent);
@@ -1109,8 +1136,8 @@ DOMCSSDeclarationImpl::~DOMCSSDeclarationImpl(void)
 }
 
 NS_IMETHODIMP
-DOMCSSDeclarationImpl::RemoveProperty(const nsAReadableString& aPropertyName, 
-                                      nsAWritableString& aReturn)
+DOMCSSDeclarationImpl::RemoveProperty(const nsAString& aPropertyName, 
+                                      nsAString& aReturn)
 {
   aReturn.Truncate();
 
@@ -1118,6 +1145,19 @@ DOMCSSDeclarationImpl::RemoveProperty(const nsAReadableString& aPropertyName,
   nsresult rv = GetCSSDeclaration(&decl, PR_TRUE);
 
   if (NS_SUCCEEDED(rv) && decl) {
+    nsCOMPtr<nsICSSStyleSheet> cssSheet;
+    nsCOMPtr<nsIDocument> owningDoc;
+    if (mRule) {
+      nsCOMPtr<nsIStyleSheet> sheet;
+      mRule->GetStyleSheet(*getter_AddRefs(sheet));
+      cssSheet = do_QueryInterface(sheet);
+      if (sheet) {
+        sheet->GetOwningDocument(*getter_AddRefs(owningDoc));
+      }
+    }
+    if (owningDoc) {
+      owningDoc->BeginUpdate();
+    }
     nsCSSProperty prop = nsCSSProps::LookupProperty(aPropertyName);
     nsCSSValue val;
 
@@ -1127,11 +1167,20 @@ DOMCSSDeclarationImpl::RemoveProperty(const nsAReadableString& aPropertyName,
       // We pass in eCSSProperty_UNKNOWN here so that we don't get the
       // property name in the return string.
       val.ToString(aReturn, eCSSProperty_UNKNOWN);
+      if (cssSheet) {
+        cssSheet->SetModified(PR_TRUE);
+      }
+      if (owningDoc) {
+        owningDoc->StyleRuleChanged(cssSheet, mRule, nsCSSProps::kHintTable[prop]);
+      }
     } else {
       // If we tried to remove an invalid property or a property that wasn't 
       //  set we simply return success and an empty string
       rv = NS_OK;
     }
+    if (owningDoc) {
+      owningDoc->EndUpdate();
+    }
   }
 
   return rv;
@@ -1145,9 +1194,9 @@ DOMCSSDeclarationImpl::DropReference(void)
 
 nsresult
 DOMCSSDeclarationImpl::GetCSSDeclaration(nsCSSDeclaration **aDecl,
-                                             PRBool aAllocate)
+                                         PRBool aAllocate)
 {
-  if (nsnull != mRule) {
+  if (mRule) {
     *aDecl = mRule->GetDeclaration();
   }
   else {
@@ -1160,7 +1209,7 @@ DOMCSSDeclarationImpl::GetCSSDeclaration(nsCSSDeclaration **aDecl,
 nsresult
 DOMCSSDeclarationImpl::SetCSSDeclaration(nsCSSDeclaration *aDecl)
 {
-  if (nsnull != mRule) {
+  if (mRule) {
     mRule->SetDeclaration(aDecl);
   }
 
@@ -1211,8 +1260,8 @@ DOMCSSDeclarationImpl::GetCSSParsingEnvironment(nsICSSStyleRule* aRule,
 }
 
 nsresult
-DOMCSSDeclarationImpl::ParsePropertyValue(const nsAReadableString& aPropName,
-                                          const nsAReadableString& aPropValue)
+DOMCSSDeclarationImpl::ParsePropertyValue(const nsAString& aPropName,
+                                          const nsAString& aPropValue)
 {
   nsCSSDeclaration* decl;
   nsresult result = GetCSSDeclaration(&decl, PR_TRUE);
@@ -1235,7 +1284,7 @@ DOMCSSDeclarationImpl::ParsePropertyValue(const nsAReadableString& aPropName,
     return result;
   }
   
-  PRInt32 hint;
+  nsChangeHint hint;
   if (owningDoc) {
     owningDoc->BeginUpdate();
   }
@@ -1257,7 +1306,7 @@ DOMCSSDeclarationImpl::ParsePropertyValue(const nsAReadableString& aPropName,
 }
 
 nsresult 
-DOMCSSDeclarationImpl::ParseDeclaration(const nsAReadableString& aDecl,
+DOMCSSDeclarationImpl::ParseDeclaration(const nsAString& aDecl,
                                         PRBool aParseOnlyOneDecl,
                                         PRBool aClearOldDecl)
 {
@@ -1299,7 +1348,7 @@ DOMCSSDeclarationImpl::ParseDeclaration(const nsAReadableString& aDecl,
         }
       }
 
-      PRInt32 hint;
+      nsChangeHint hint;
       result = cssParser->ParseAndAppendDeclaration(aDecl, baseURI, decl,
                                                     aParseOnlyOneDecl, &hint);
 
@@ -1350,11 +1399,6 @@ public:
 
   NS_DECL_ISUPPORTS_INHERITED
 
-//  NS_IMETHOD Equals(const nsIStyleRule* aRule, PRBool& aResult) const;
-//  NS_IMETHOD HashValue(PRUint32& aValue) const;
-  // Strength is an out-of-band weighting, useful for mapping CSS ! important
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
-
   virtual nsCSSSelector* FirstSelector(void);
   virtual void AddSelector(const nsCSSSelector& aSelector);
   virtual void DeleteSelector(nsCSSSelector* aSelector);
@@ -1370,7 +1414,7 @@ public:
   virtual PRInt32 GetWeight(void) const;
   virtual void SetWeight(PRInt32 aWeight);
 
-  virtual nsIStyleRule* GetImportantRule(void);
+  virtual already_AddRefed<nsIStyleRule> GetImportantRule(void);
 
   // hook for inspector
   virtual nsresult GetValue(nsCSSProperty aProperty, nsCSSValue& aValue);
@@ -1506,60 +1550,6 @@ NS_IMPL_ADDREF_INHERITED(CSSStyleRuleImpl, nsCSSRule);
 NS_IMPL_RELEASE_INHERITED(CSSStyleRuleImpl, nsCSSRule);
 
 
-#if 0
-NS_IMETHODIMP CSSStyleRuleImpl::Equals(const nsIStyleRule* aRule, PRBool& aResult) const
-{
-  nsICSSStyleRule* iCSSRule;
-
-  if (this == aRule) {
-    aResult = PR_TRUE;
-  }
-  else {
-    aResult = PR_FALSE;
-    if ((nsnull != aRule) && 
-        (NS_OK == ((nsIStyleRule*)aRule)->QueryInterface(NS_GET_IID(nsICSSStyleRule), (void**) &iCSSRule))) {
-
-      CSSStyleRuleImpl* rule = (CSSStyleRuleImpl*)iCSSRule;
-      const nsCSSSelector* local = &mSelector;
-      const nsCSSSelector* other = &(rule->mSelector);
-      aResult = PR_TRUE;
-
-      if ((rule->mDeclaration != mDeclaration) || 
-          (rule->mWeight != mWeight)) {
-        aResult = PR_FALSE;
-      }
-      while ((PR_TRUE == aResult) && (nsnull != local) && (nsnull != other)) {
-        if (! local->Equals(other)) {
-          aResult = PR_FALSE;
-        }
-        local = local->mNext;
-        other = other->mNext;
-      }
-      if ((nsnull != local) || (nsnull != other)) { // more were left
-        aResult = PR_FALSE;
-      }
-      NS_RELEASE(iCSSRule);
-    }
-  }
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-CSSStyleRuleImpl::HashValue(PRUint32& aValue) const
-{
-  aValue = (PRUint32)this;
-  return NS_OK;
-}
-#endif
-
-// Strength is an out-of-band weighting, useful for mapping CSS ! important
-NS_IMETHODIMP
-CSSStyleRuleImpl::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 0;
-  return NS_OK;
-}
-
 nsCSSSelector* CSSStyleRuleImpl::FirstSelector(void)
 {
   return &mSelector;
@@ -1655,11 +1645,11 @@ void CSSStyleRuleImpl::SetWeight(PRInt32 aWeight)
   mWeight = aWeight;
 }
 
-nsIStyleRule* CSSStyleRuleImpl::GetImportantRule(void)
+already_AddRefed<nsIStyleRule> CSSStyleRuleImpl::GetImportantRule(void)
 {
-  if ((nsnull == mImportantRule) && (nsnull != mDeclaration)) {
+  if (!mImportantRule && mDeclaration) {
     nsCSSDeclaration*  important = mDeclaration->GetImportantValues();
-    if (nsnull != important) {
+    if (important) {
       mImportantRule = new CSSImportantRule(mSheet, important);
       NS_ADDREF(mImportantRule);
     }
@@ -2149,6 +2139,14 @@ MapColorForDeclaration(nsCSSDeclaration* aDecl, const nsStyleStructID& aID, nsCS
       aColor.mBackPositionX = ourColor->mBackPositionX;
     if (aColor.mBackPositionY.GetUnit() == eCSSUnit_Null && ourColor->mBackPositionY.GetUnit() != eCSSUnit_Null)
       aColor.mBackPositionY = ourColor->mBackPositionY;
+
+    // background-clip: enum, inherit
+    if (aColor.mBackClip.GetUnit() == eCSSUnit_Null && ourColor->mBackClip.GetUnit() != eCSSUnit_Null)
+      aColor.mBackClip = ourColor->mBackClip;
+
+    // background-origin: enum, inherit
+    if (aColor.mBackOrigin.GetUnit() == eCSSUnit_Null && ourColor->mBackOrigin.GetUnit() != eCSSUnit_Null)
+      aColor.mBackOrigin = ourColor->mBackOrigin;
   }
 
   return NS_OK;
@@ -2306,6 +2304,13 @@ MapDisplayForDeclaration(nsCSSDeclaration* aDecl, const nsStyleStructID& aID, ns
     if (aDisplay.mClear.GetUnit() == eCSSUnit_Null && ourDisplay->mClear.GetUnit() != eCSSUnit_Null)
       aDisplay.mClear = ourDisplay->mClear;
 
+    // temp fix for bug 24000
+    if (aDisplay.mBreakBefore.GetUnit() == eCSSUnit_Null && ourDisplay->mBreakBefore.GetUnit() != eCSSUnit_Null)
+      aDisplay.mBreakBefore = ourDisplay->mBreakBefore;
+    if (aDisplay.mBreakAfter.GetUnit() == eCSSUnit_Null && ourDisplay->mBreakAfter.GetUnit() != eCSSUnit_Null)
+      aDisplay.mBreakAfter = ourDisplay->mBreakAfter;
+    // end temp fix
+
     // float: enum, none, inherit
     if (aDisplay.mFloat.GetUnit() == eCSSUnit_Null && ourDisplay->mFloat.GetUnit() != eCSSUnit_Null)
       aDisplay.mFloat = ourDisplay->mFloat;
@@ -2377,6 +2382,9 @@ MapUIForDeclaration(nsCSSDeclaration* aDecl, const nsStyleStructID& aID, nsCSSUs
 
     if (aUI.mResizer.GetUnit() == eCSSUnit_Null && ourUI->mResizer.GetUnit() != eCSSUnit_Null)
       aUI.mResizer = ourUI->mResizer;
+
+    if (aUI.mForceBrokenImageIcon.GetUnit() == eCSSUnit_Null && ourUI->mForceBrokenImageIcon.GetUnit() == eCSSUnit_Integer)
+      aUI.mForceBrokenImageIcon = ourUI->mForceBrokenImageIcon;
   }
 
   return NS_OK;
@@ -2471,7 +2479,7 @@ CSSStyleRuleImpl::GetType(PRUint16* aType)
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::GetCssText(nsAWritableString& aCssText)
+CSSStyleRuleImpl::GetCssText(nsAString& aCssText)
 {
   mSelector.ToString( aCssText, mSheet, IsPseudoElement(mSelector.mTag),
                       0 );
@@ -2490,7 +2498,7 @@ CSSStyleRuleImpl::GetCssText(nsAWritableString& aCssText)
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::SetCssText(const nsAReadableString& aCssText)
+CSSStyleRuleImpl::SetCssText(const nsAString& aCssText)
 {
   // XXX TBI - need to re-parse rule & declaration
   return NS_OK;
@@ -2517,14 +2525,14 @@ CSSStyleRuleImpl::GetParentRule(nsIDOMCSSRule** aParentRule)
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::GetSelectorText(nsAWritableString& aSelectorText)
+CSSStyleRuleImpl::GetSelectorText(nsAString& aSelectorText)
 {
   mSelector.ToString( aSelectorText, mSheet, IsPseudoElement(mSelector.mTag), 0 );
   return NS_OK;
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::SetSelectorText(const nsAReadableString& aSelectorText)
+CSSStyleRuleImpl::SetSelectorText(const nsAString& aSelectorText)
 {
   // XXX TBI - get a parser and re-parse the selectors, 
   // XXX then need to re-compute the cascade
