@@ -35,11 +35,10 @@
 #include "nsIURL.h"
 #include "nsIServiceManager.h"
 #include "nsISupportsArray.h"
-#include "nsHashtable.h"
+#include "pldhash.h"
 #include "nsIHTMLContent.h"
 #include "nsHTMLAttributes.h"
 #include "nsILink.h"
-#include "nsStyleUtil.h"
 #include "nsIStyleRuleProcessor.h"
 #include "nsIStyleRule.h"
 #include "nsIFrame.h"
@@ -72,11 +71,7 @@ public:
 
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD Equals(const nsIStyleRule* aRule, PRBool& aValue) const;
-  NS_IMETHOD HashValue(PRUint32& aValue) const;
   NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const;
-  // Strength is an out-of-band weighting, always 0 here
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
 
   // The new mapping function.
   NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
@@ -115,7 +110,7 @@ protected:
 HTMLColorRule::HTMLColorRule(nsIHTMLStyleSheet* aSheet)
   : mSheet(aSheet)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 HTMLColorRule::~HTMLColorRule()
@@ -124,20 +119,6 @@ HTMLColorRule::~HTMLColorRule()
 
 NS_IMPL_ISUPPORTS1(HTMLColorRule, nsIStyleRule)
 
-NS_IMETHODIMP
-HTMLColorRule::Equals(const nsIStyleRule* aRule, PRBool& aResult) const
-{
-  aResult = PRBool(this == aRule);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-HTMLColorRule::HashValue(PRUint32& aValue) const
-{
-  aValue = (PRUint32)(mColor);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 HTMLColorRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
 {
@@ -146,14 +127,6 @@ HTMLColorRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
   return NS_OK;
 }
 
-// Strength is an out-of-band weighting, always 0 here
-NS_IMETHODIMP
-HTMLColorRule::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 HTMLColorRule::MapRuleInfoInto(nsRuleData* aRuleData)
 {
@@ -197,8 +170,7 @@ void HTMLColorRule::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
   }
 
   // get or create a tag for this instance
-  nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("HTMLColorRule"));
+  nsCOMPtr<nsIAtom> tag = do_GetAtom("HTMLColorRule");
   // get the size of an empty instance and add to the sizeof handler
   aSize = sizeof(*this);
   aSizeOfHandler->AddSize(tag,aSize);
@@ -285,8 +257,7 @@ void HTMLDocumentColorRule::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &a
   }
 
   // get or create a tag for this instance
-  nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("HTMLDocumentColorRule"));
+  nsCOMPtr<nsIAtom> tag = do_GetAtom("HTMLDocumentColorRule");
   // get the size of an empty instance and add to the sizeof handler
   aSize = sizeof(*this);
   aSizeOfHandler->AddSize(tag,aSize);
@@ -305,13 +276,8 @@ public:
 
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD Equals(const nsIStyleRule* aRule, PRBool& aResult) const;
-  NS_IMETHOD HashValue(PRUint32& aValue) const;
   NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const;
 
-  // Strength is an out-of-band weighting, always 0 here
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
-
   // The new mapping function.
   NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
 
@@ -330,7 +296,7 @@ public:
 
 GenericTableRule::GenericTableRule(nsIHTMLStyleSheet* aSheet)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mSheet = aSheet;
 }
 
@@ -340,20 +306,6 @@ GenericTableRule::~GenericTableRule()
 
 NS_IMPL_ISUPPORTS1(GenericTableRule, nsIStyleRule)
 
-NS_IMETHODIMP
-GenericTableRule::Equals(const nsIStyleRule* aRule, PRBool& aResult) const
-{
-  aResult = PRBool(this == aRule);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-GenericTableRule::HashValue(PRUint32& aValue) const
-{
-  aValue = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 GenericTableRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
 {
@@ -361,15 +313,6 @@ GenericTableRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
   return NS_OK;
 }
 
-// Strength is an out-of-band weighting, useful for mapping CSS ! important
-// always 0 here
-NS_IMETHODIMP
-GenericTableRule::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 GenericTableRule::MapRuleInfoInto(nsRuleData* aRuleData)
 {
@@ -410,8 +353,7 @@ void GenericTableRule::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
   }
 
   // get or create a tag for this instance
-  nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("GenericTableRule"));
+  nsCOMPtr<nsIAtom> tag = do_GetAtom("GenericTableRule");
   // get the size of an empty instance and add to the sizeof handler
   aSize = sizeof(*this);
   aSizeOfHandler->AddSize(tag,aSize);
@@ -452,7 +394,7 @@ static void PostResolveCallback(nsStyleStruct* aStyleStruct, nsRuleData* aRuleDa
 {
   nsStyleText* text = (nsStyleText*)aStyleStruct;
   if (text->mTextAlign == NS_STYLE_TEXT_ALIGN_DEFAULT) {
-    nsCOMPtr<nsIStyleContext> parentContext = getter_AddRefs(aRuleData->mStyleContext->GetParent());
+    nsCOMPtr<nsIStyleContext> parentContext = aRuleData->mStyleContext->GetParent();
 
     if (parentContext) {
       const nsStyleText* parentStyleText = 
@@ -485,9 +427,13 @@ ProcessTableRulesAttribute(nsStyleStruct* aStyleStruct,
 {
   if (!aStyleStruct || !aRuleData || !aRuleData->mPresContext) return;
 
-  nsCOMPtr<nsIStyleContext> tableContext = getter_AddRefs(aRuleData->mStyleContext->GetParent()); if (!tableContext) return;
+  nsCOMPtr<nsIStyleContext> tableContext = aRuleData->mStyleContext->GetParent();
+  if (!tableContext)
+    return;
   if (!aGroup) {
-    tableContext = getter_AddRefs(tableContext->GetParent()); if (!tableContext) return;
+    tableContext = tableContext->GetParent();
+    if (!tableContext)
+      return;
   } 
   
   const nsStyleTable* tableData = 
@@ -496,10 +442,14 @@ ProcessTableRulesAttribute(nsStyleStruct* aStyleStruct,
                     (aRulesArg2 == tableData->mRules) ||
                     (aRulesArg3 == tableData->mRules))) {
     const nsStyleBorder* tableBorderData = 
-      (const nsStyleBorder*)tableContext->GetStyleData(eStyleStruct_Border); if (!tableBorderData) return;
+      (const nsStyleBorder*)tableContext->GetStyleData(eStyleStruct_Border);
+    if (!tableBorderData)
+      return;
     PRUint8 tableBorderStyle = tableBorderData->GetBorderStyle(aSide);
 
-    nsStyleBorder* borderData = (nsStyleBorder*)aStyleStruct; if (!borderData) return;
+    nsStyleBorder* borderData = (nsStyleBorder*)aStyleStruct;
+    if (!borderData)
+      return;
     PRUint8 borderStyle = borderData->GetBorderStyle(aSide);
     // XXX It appears that the style system erronously applies the custom style rule after css style, 
     // consequently it does not properly fit into the casade. For now, assume that a border style of none
@@ -724,92 +674,65 @@ TableColRule::MapRuleInfoInto(nsRuleData* aRuleData)
 }
 // -----------------------------------------------------------
 
-class AttributeKey: public nsHashKey
-{
-public:
-  AttributeKey(nsIHTMLMappedAttributes* aAttributes);
-  virtual ~AttributeKey(void);
-
-  PRBool      Equals(const nsHashKey* aOther) const;
-  PRUint32    HashCode(void) const;
-  nsHashKey*  Clone(void) const;
-
-private:
-  AttributeKey(void);
-  AttributeKey(const AttributeKey& aCopy);
-  AttributeKey& operator=(const AttributeKey& aCopy);
-
-public:
-  nsIHTMLMappedAttributes*  mAttributes;
-  union {
-    struct {
-      PRUint32  mHashSet: 1;
-      PRUint32  mHashCode: 31;
-    } mBits;
-    PRUint32    mInitializer;
-  } mHash;
+struct MappedAttrTableEntry : public PLDHashEntryHdr {
+  nsIHTMLMappedAttributes *mAttributes;
 };
 
-MOZ_DECL_CTOR_COUNTER(AttributeKey)
-
-AttributeKey::AttributeKey(nsIHTMLMappedAttributes* aAttributes)
-  : mAttributes(aAttributes)
+PR_STATIC_CALLBACK(PLDHashNumber)
+MappedAttrTable_HashKey(PLDHashTable *table, const void *key)
 {
-  MOZ_COUNT_CTOR(AttributeKey);
-  NS_ADDREF(mAttributes);
-  mHash.mInitializer = 0;
+  nsIHTMLMappedAttributes *attributes =
+    NS_STATIC_CAST(nsIHTMLMappedAttributes*, NS_CONST_CAST(void*, key));
+
+  PRUint32 hash;
+  attributes->HashValue(hash);
+  return hash;
 }
 
-AttributeKey::~AttributeKey(void)
+PR_STATIC_CALLBACK(void)
+MappedAttrTable_ClearEntry(PLDHashTable *table, PLDHashEntryHdr *hdr)
 {
-  MOZ_COUNT_DTOR(AttributeKey);
-  NS_RELEASE(mAttributes);
+  MappedAttrTableEntry *entry = NS_STATIC_CAST(MappedAttrTableEntry*, hdr);
+
+  entry->mAttributes->DropStyleSheetReference();
+  memset(entry, 0, sizeof(MappedAttrTableEntry));
 }
 
-PRBool AttributeKey::Equals(const nsHashKey* aOther) const
+PR_STATIC_CALLBACK(PRBool)
+MappedAttrTable_MatchEntry(PLDHashTable *table, const PLDHashEntryHdr *hdr,
+                           const void *key)
 {
-  const AttributeKey* other = (const AttributeKey*)aOther;
-  PRBool  equals;
-  mAttributes->Equals(other->mAttributes, equals);
-  return equals;
+  nsIHTMLMappedAttributes *attributes =
+    NS_STATIC_CAST(nsIHTMLMappedAttributes*, NS_CONST_CAST(void*, key));
+  const MappedAttrTableEntry *entry =
+    NS_STATIC_CAST(const MappedAttrTableEntry*, hdr);
+
+  PRBool equal;
+  attributes->Equals(entry->mAttributes, equal);
+  return equal;
 }
 
-PRUint32 AttributeKey::HashCode(void) const
-{
-  if (0 == mHash.mBits.mHashSet) {
-    AttributeKey* self = (AttributeKey*)this; // break const
-    PRUint32  hash;
-    mAttributes->HashValue(hash);
-    self->mHash.mBits.mHashCode = (0x7FFFFFFF & hash);
-    self->mHash.mBits.mHashSet = 1;
-  }
-  return mHash.mBits.mHashCode;
-}
-
-nsHashKey* AttributeKey::Clone(void) const
-{
-  AttributeKey* clown = new AttributeKey(mAttributes);
-  if (nsnull != clown) {
-    clown->mHash.mInitializer = mHash.mInitializer;
-  }
-  return clown;
-}
+static PLDHashTableOps MappedAttrTable_Ops = {
+  PL_DHashAllocTable,
+  PL_DHashFreeTable,
+  PL_DHashGetKeyStub,
+  MappedAttrTable_HashKey,
+  MappedAttrTable_MatchEntry,
+  PL_DHashMoveEntryStub,
+  MappedAttrTable_ClearEntry,
+  PL_DHashFinalizeStub,
+  NULL
+};
 
 // -----------------------------------------------------------
 
 class HTMLStyleSheetImpl : public nsIHTMLStyleSheet,
                            public nsIStyleRuleProcessor {
 public:
-  void* operator new(size_t size);
-  void* operator new(size_t size, nsIArena* aArena);
-  void operator delete(void* ptr);
-
   HTMLStyleSheetImpl(void);
   nsresult Init();
 
-  NS_IMETHOD QueryInterface(const nsIID& aIID, void** aInstancePtr);
-  NS_IMETHOD_(nsrefcnt) AddRef();
-  NS_IMETHOD_(nsrefcnt) Release();
+  NS_DECL_ISUPPORTS
 
   // nsIStyleSheet api
   NS_IMETHOD GetURL(nsIURI*& aURL) const;
@@ -876,10 +799,6 @@ protected:
   virtual ~HTMLStyleSheetImpl();
 
 protected:
-  PRUint32 mInHeap : 1;
-  PRUint32 mRefCnt : 31;
-  NS_DECL_OWNINGTHREAD // for thread-safety checking
-
   nsIURI*              mURL;
   nsIDocument*         mDocument;
   HTMLColorRule*       mLinkRule;
@@ -894,46 +813,13 @@ protected:
     // NOTE: if adding more rules, be sure to update 
     // the SizeOf method to include them
 
-  nsHashtable          mMappedAttrTable;
+  PLDHashTable         mMappedAttrTable;
 };
 
 
-void* HTMLStyleSheetImpl::operator new(size_t size)
-{
-  HTMLStyleSheetImpl* rv = (HTMLStyleSheetImpl*) ::operator new(size);
-#ifdef NS_DEBUG
-  if (nsnull != rv) {
-    memset(rv, 0xEE, size);
-  }
-#endif
-  rv->mInHeap = 1;
-  return (void*) rv;
-}
-
-void* HTMLStyleSheetImpl::operator new(size_t size, nsIArena* aArena)
-{
-  HTMLStyleSheetImpl* rv = (HTMLStyleSheetImpl*) aArena->Alloc(PRInt32(size));
-#ifdef NS_DEBUG
-  if (nsnull != rv) {
-    memset(rv, 0xEE, size);
-  }
-#endif
-  rv->mInHeap = 0;
-  return (void*) rv;
-}
-
-void HTMLStyleSheetImpl::operator delete(void* ptr)
-{
-  HTMLStyleSheetImpl* sheet = (HTMLStyleSheetImpl*) ptr;
-  if (nsnull != sheet) {
-    if (sheet->mInHeap) {
-      ::operator delete(ptr);
-    }
-  }
-}
-
 HTMLStyleSheetImpl::HTMLStyleSheetImpl(void)
   : nsIHTMLStyleSheet(),
+    mRefCnt(0),
     mURL(nsnull),
     mDocument(nsnull),
     mLinkRule(nsnull),
@@ -941,7 +827,8 @@ HTMLStyleSheetImpl::HTMLStyleSheetImpl(void)
     mActiveRule(nsnull),
     mDocumentColorRule(nsnull)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
+  mMappedAttrTable.ops = nsnull;
 }
 
 nsresult
@@ -980,14 +867,6 @@ HTMLStyleSheetImpl::Init()
   return NS_OK;
 }
 
-PR_STATIC_CALLBACK(PRBool)
-MappedDropSheet(nsHashKey *aKey, void *aData, void* closure)
-{
-  nsIHTMLMappedAttributes* mapped = (nsIHTMLMappedAttributes*)aData;
-  mapped->DropStyleSheetReference();
-  return PR_TRUE;
-}
-
 HTMLStyleSheetImpl::~HTMLStyleSheetImpl()
 {
   NS_IF_RELEASE(mURL);
@@ -1027,7 +906,8 @@ HTMLStyleSheetImpl::~HTMLStyleSheetImpl()
     mTableTHRule->mSheet = nsnull;
     NS_RELEASE(mTableTHRule);
   }
-  mMappedAttrTable.Enumerate(MappedDropSheet);
+  if (mMappedAttrTable.ops)
+    PL_DHashTableFinish(&mMappedAttrTable);
 }
 
 NS_IMPL_ADDREF(HTMLStyleSheetImpl)
@@ -1134,7 +1014,7 @@ HTMLStyleSheetImpl::RulesMatching(ElementRuleProcessorData* aData,
         ruleWalker->Forward(mTableColgroupRule);
       }
       else if (tag == nsHTMLAtoms::table) {
-        if (aData->mIsQuirkMode)
+        if (aData->mCompatMode == eCompatibility_NavQuirks)
           ruleWalker->Forward(mDocumentColorRule);
       }
     } // end html element
@@ -1301,8 +1181,10 @@ HTMLStyleSheetImpl::Reset(nsIURI* aURL)
   mTableColRule->Reset();
   mTableTHRule->Reset();
 
-  mMappedAttrTable.Enumerate(MappedDropSheet);
-  mMappedAttrTable.Reset();
+  if (mMappedAttrTable.ops) {
+    PL_DHashTableFinish(&mMappedAttrTable);
+    mMappedAttrTable.ops = nsnull;
+  }
 
   return NS_OK;
 }
@@ -1390,40 +1272,49 @@ NS_IMETHODIMP
 HTMLStyleSheetImpl::UniqueMappedAttributes(nsIHTMLMappedAttributes* aMapped,
                                            nsIHTMLMappedAttributes*& aUniqueMapped)
 {
-  nsresult result = NS_OK;
-
-  AttributeKey key(aMapped);
-  nsIHTMLMappedAttributes* sharedAttrs =
-    (nsIHTMLMappedAttributes*)mMappedAttrTable.Get(&key);
-
-  if (!sharedAttrs) {  // we have a new unique set
-    mMappedAttrTable.Put(&key, aMapped);
+  aUniqueMapped = nsnull;
+  if (!mMappedAttrTable.ops) {
+    PRBool res = PL_DHashTableInit(&mMappedAttrTable, &MappedAttrTable_Ops,
+                                   nsnull, sizeof(MappedAttrTableEntry), 16);
+    if (!res) {
+      mMappedAttrTable.ops = nsnull;
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+  }
+  MappedAttrTableEntry *entry = NS_STATIC_CAST(MappedAttrTableEntry*,
+    PL_DHashTableOperate(&mMappedAttrTable, aMapped, PL_DHASH_ADD));
+  if (!entry)
+    return NS_ERROR_OUT_OF_MEMORY;
+  if (!entry->mAttributes) {
+    // We added a new entry to the hashtable, so we have a new unique set.
+    entry->mAttributes = aMapped;
     aMapped->SetUniqued(PR_TRUE);
-    NS_ADDREF(aMapped);
-    aUniqueMapped = aMapped;
   }
-  else {  // found existing set
-    aUniqueMapped = sharedAttrs;
-    NS_ADDREF(aUniqueMapped);
-  }
-  return result;
+  aUniqueMapped = entry->mAttributes;
+  NS_ADDREF(aUniqueMapped);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 HTMLStyleSheetImpl::DropMappedAttributes(nsIHTMLMappedAttributes* aMapped)
 {
-  if (aMapped) {
-    PRBool inTable = PR_FALSE;
-    aMapped->GetUniqued(inTable);
-    if (inTable) {
-      AttributeKey key(aMapped);
-      nsIHTMLMappedAttributes* old =
-        (nsIHTMLMappedAttributes*)mMappedAttrTable.Remove(&key);
-      NS_ASSERTION(old == aMapped, "not in table");
+  NS_ENSURE_TRUE(aMapped, NS_OK);
+
+  PRBool inTable = PR_FALSE;
+  aMapped->GetUniqued(inTable);
+  if (!inTable)
+    return NS_OK;
+
+  NS_ASSERTION(mMappedAttrTable.ops, "table uninitialized");
+#ifdef DEBUG
+  PRUint32 entryCount = mMappedAttrTable.entryCount - 1;
+#endif
+
+  PL_DHashTableOperate(&mMappedAttrTable, aMapped, PL_DHASH_REMOVE);
+
+  NS_ASSERTION(entryCount == mMappedAttrTable.entryCount, "not removed");
 
-      aMapped->SetUniqued(PR_FALSE);
-    }
-  }
   return NS_OK;
 }
 
@@ -1457,19 +1348,21 @@ struct MappedAttributeSizeEnumData
   nsUniqueStyleItems *uniqueItems;
 };
 
-PR_STATIC_CALLBACK(PRBool)
-MappedSizeAttributes(nsHashKey *aKey, void *aData, void* closure)
+PR_STATIC_CALLBACK(PLDHashOperator)
+MappedSizeAttributes(PLDHashTable *table, PLDHashEntryHdr *hdr,
+                     PRUint32 number, void *arg)
 {
-  MappedAttributeSizeEnumData *pData = (MappedAttributeSizeEnumData *)closure;
+  MappedAttributeSizeEnumData *pData = (MappedAttributeSizeEnumData *)arg;
   NS_ASSERTION(pData,"null closure is not supported");
-  nsIHTMLMappedAttributes* mapped = (nsIHTMLMappedAttributes*)aData;
+  MappedAttrTableEntry *entry = NS_STATIC_CAST(MappedAttrTableEntry*, hdr);
+  nsIHTMLMappedAttributes* mapped = entry->mAttributes;
   NS_ASSERTION(mapped, "null item in enumeration fcn is not supported");
   // if there is an attribute and it has not been counted, the get its size
   if(mapped){
     PRUint32 size=0;
     mapped->SizeOf(pData->aHandler, size);
   }  
-  return PR_TRUE;
+  return PL_DHASH_NEXT;
 }
 
 
@@ -1512,8 +1405,7 @@ HTMLStyleSheetImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
   PRUint32 localSize=0;
 
   // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("HTMLStyleSheet"));
+  nsCOMPtr<nsIAtom> tag = do_GetAtom("HTMLStyleSheet");
   // get the size of an empty instance and add to the sizeof handler
   aSize = sizeof(HTMLStyleSheetImpl);
   aSizeOfHandler->AddSize(tag,aSize);
@@ -1532,47 +1424,47 @@ HTMLStyleSheetImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
 
   if(mLinkRule && uniqueItems->AddItem((void*)mLinkRule)){
     localSize = sizeof(*mLinkRule);
-    tag = getter_AddRefs(NS_NewAtom("LinkRule"));
+    tag = do_GetAtom("LinkRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(mVisitedRule && uniqueItems->AddItem((void*)mVisitedRule)){
     localSize = sizeof(*mVisitedRule);
-    tag = getter_AddRefs(NS_NewAtom("VisitedRule"));
+    tag = do_GetAtom("VisitedRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(mActiveRule && uniqueItems->AddItem((void*)mActiveRule)){
     localSize = sizeof(*mActiveRule);
-    tag = getter_AddRefs(NS_NewAtom("ActiveRule"));
+    tag = do_GetAtom("ActiveRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(uniqueItems->AddItem((void*)mDocumentColorRule)){
     localSize = sizeof(*mDocumentColorRule);
-    tag = getter_AddRefs(NS_NewAtom("DocumentColorRule"));
+    tag = do_GetAtom("DocumentColorRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(uniqueItems->AddItem((void*)mTableTbodyRule)){
     localSize = sizeof(*mTableTbodyRule);
-    tag = getter_AddRefs(NS_NewAtom("TableTbodyRule"));
+    tag = do_GetAtom("TableTbodyRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(uniqueItems->AddItem((void*)mTableRowRule)){
     localSize = sizeof(*mTableRowRule);
-    tag = getter_AddRefs(NS_NewAtom("TableRowRule"));
+    tag = do_GetAtom("TableRowRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(uniqueItems->AddItem((void*)mTableColgroupRule)){
     localSize = sizeof(*mTableColgroupRule);
-    tag = getter_AddRefs(NS_NewAtom("TableColgroupRule"));
+    tag = do_GetAtom("TableColgroupRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(uniqueItems->AddItem((void*)mTableColRule)){
     localSize = sizeof(*mTableColRule);
-    tag = getter_AddRefs(NS_NewAtom("TableColRule"));
+    tag = do_GetAtom("TableColRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   if(uniqueItems->AddItem((void*)mTableTHRule)){
     localSize = sizeof(*mTableTHRule);
-    tag = getter_AddRefs(NS_NewAtom("TableTHRule"));
+    tag = do_GetAtom("TableTHRule");
     aSizeOfHandler->AddSize(tag,localSize);
   }
   
@@ -1584,17 +1476,17 @@ HTMLStyleSheetImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
   //   number of entries X sizeof each hash-entry 
   //   + the size of a hash table (see plhash.h and nsHashTable.h)
   //  then we add up the size of each unique attribute
-  localSize = sizeof(mMappedAttrTable);
-  if(mMappedAttrTable.Count() >0){
-    localSize += sizeof(PLHashTable);
-    localSize += mMappedAttrTable.Count() * sizeof(PLHashEntry);
-  }
-  tag = getter_AddRefs(NS_NewAtom("MappedAttrTable"));
-  aSizeOfHandler->AddSize(tag,localSize);
+  if (mMappedAttrTable.ops) {
+    localSize =
+      PL_DHASH_TABLE_SIZE(&mMappedAttrTable) * mMappedAttrTable.entrySize;
+    tag = do_GetAtom("MappedAttrTable");
+    aSizeOfHandler->AddSize(tag,localSize);
 
-  // now get each unique attribute
-  MappedAttributeSizeEnumData sizeEnumData(aSizeOfHandler, uniqueItems);
-  mMappedAttrTable.Enumerate(MappedSizeAttributes, &sizeEnumData);
+    // now get each unique attribute
+    MappedAttributeSizeEnumData sizeEnumData(aSizeOfHandler, uniqueItems);
+    PL_DHashTableEnumerate(&mMappedAttrTable, &MappedSizeAttributes,
+                           &sizeEnumData);
+  }
 
   // that's it
 }
