@@ -63,6 +63,7 @@ HTML_ATOM(mozSingleLineTextControlFrame, ":-moz-singleline-textcontrol-frame")
 HTML_ATOM(mozFocusInnerPseudo, ":-moz-focus-inner")
 HTML_ATOM(mozFocusOuterPseudo, ":-moz-focus-outer")
 HTML_ATOM(mozDisplayComboboxControlFrame, ":-moz-display-comboboxcontrol-frame")
+HTML_ATOM(mozSelectionPseudo, ":-moz-selection")
 
 HTML_ATOM(_baseHref, NS_HTML_BASE_HREF)
 HTML_ATOM(_baseTarget, NS_HTML_BASE_TARGET)
@@ -204,12 +205,14 @@ HTML_ATOM(longdesc, "longdesc")
 HTML_ATOM(map, "map")
 HTML_ATOM(marginheight, "marginheight")
 HTML_ATOM(marginwidth, "marginwidth")
+HTML_ATOM(marquee, "marquee")
 HTML_ATOM(maxlength, "maxlength")
 HTML_ATOM(mayscript, "mayscript")
 HTML_ATOM(media, "media")
 HTML_ATOM(menu, "menu")
 HTML_ATOM(meta, "meta")
 HTML_ATOM(method, "method")
+HTML_ATOM(msthemecompatible, "msthemecompatible")
 HTML_ATOM(multicol, "multicol")
 HTML_ATOM(multiple, "multiple")
 HTML_ATOM(name, "name")
