@@ -41,7 +41,6 @@
 #define nsRuleNode_h___
 
 #include "nsCOMPtr.h"
-#include "nsHashtable.h"
 #include "nsIStyleRule.h"
 #include "nsIFrame.h"
 #include "nsFixedSizeAllocator.h"
@@ -51,176 +50,122 @@
 
 class nsIStyleContext;
 struct nsRuleList;
+struct PLDHashTable;
 
 typedef void (*nsPostResolveFunc)(nsStyleStruct* aStyleStruct, nsRuleData* aData);
 
 struct nsInheritedStyleData
 {
-  nsStyleVisibility* mVisibilityData;
-  nsStyleFont* mFontData;
-  nsStyleList* mListData;
-  nsStyleTableBorder* mTableData;
-  nsStyleColor* mColorData;
-  nsStyleQuotes* mQuotesData;
-  nsStyleText* mTextData;
-  nsStyleUserInterface* mUIData;
-#ifdef MOZ_SVG
-  nsStyleSVG* mSVGData;
-#endif
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb) \
+  nsStyle##name * m##name##Data;
+#define STYLE_STRUCT_RESET(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_INHERITED
+#undef STYLE_STRUCT_RESET
+
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
   };
 
   void ClearInheritedData(PRUint32 aBits) {
-    if (mVisibilityData && (aBits & NS_STYLE_INHERIT_VISIBILITY))
-      mVisibilityData = nsnull;
-    if (mFontData && (aBits & NS_STYLE_INHERIT_FONT))
-      mFontData = nsnull;
-    if (mListData && (aBits & NS_STYLE_INHERIT_LIST))
-      mListData = nsnull;
-    if (mTableData && (aBits & NS_STYLE_INHERIT_TABLE_BORDER))
-      mTableData = nsnull;
-    if (mColorData && (aBits & NS_STYLE_INHERIT_COLOR))
-      mColorData = nsnull;
-    if (mQuotesData && (aBits & NS_STYLE_INHERIT_QUOTES))
-      mQuotesData = nsnull;
-    if (mTextData && (aBits & NS_STYLE_INHERIT_TEXT))
-      mTextData = nsnull;
-    if (mUIData && (aBits & NS_STYLE_INHERIT_UI))
-      mUIData = nsnull;
-#ifdef MOZ_SVG
-    if (mSVGData && (aBits & NS_STYLE_INHERIT_SVG))
-      mSVGData = nsnull;
-#endif 
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb) \
+    if (m##name##Data && (aBits & NS_STYLE_INHERIT_BIT(name))) \
+      m##name##Data = nsnull;
+#define STYLE_STRUCT_RESET(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_INHERITED
+#undef STYLE_STRUCT_RESET
   };
 
   void Destroy(PRUint32 aBits, nsIPresContext* aContext) {
-    if (mVisibilityData && !(aBits & NS_STYLE_INHERIT_VISIBILITY))
-      mVisibilityData->Destroy(aContext);
-    if (mFontData && !(aBits & NS_STYLE_INHERIT_FONT))
-      mFontData->Destroy(aContext);
-    if (mListData && !(aBits & NS_STYLE_INHERIT_LIST))
-      mListData->Destroy(aContext);
-    if (mTableData && !(aBits & NS_STYLE_INHERIT_TABLE_BORDER))
-      mTableData->Destroy(aContext);
-    if (mColorData && !(aBits & NS_STYLE_INHERIT_COLOR))
-      mColorData->Destroy(aContext);
-    if (mQuotesData && !(aBits & NS_STYLE_INHERIT_QUOTES))
-      mQuotesData->Destroy(aContext);
-    if (mTextData && !(aBits & NS_STYLE_INHERIT_TEXT))
-      mTextData->Destroy(aContext);
-    if (mUIData && !(aBits & NS_STYLE_INHERIT_UI))
-      mUIData->Destroy(aContext);
-#ifdef MOZ_SVG
-    if (mSVGData && !(aBits & NS_STYLE_INHERIT_SVG))
-      mSVGData->Destroy(aContext);
-#endif
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb) \
+    if (m##name##Data && !(aBits & NS_STYLE_INHERIT_BIT(name))) \
+      m##name##Data->Destroy(aContext);
+#define STYLE_STRUCT_RESET(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_INHERITED
+#undef STYLE_STRUCT_RESET
+
     aContext->FreeToShell(sizeof(nsInheritedStyleData), this);
   };
 
-  nsInheritedStyleData() 
-    :mVisibilityData(nsnull), mFontData(nsnull), mListData(nsnull), 
-     mTableData(nsnull), mColorData(nsnull), mQuotesData(nsnull), mTextData(nsnull), mUIData(nsnull)
-#ifdef MOZ_SVG
-    , mSVGData(nsnull)
-#endif
-  {};
+  nsInheritedStyleData() {
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb) \
+    m##name##Data = nsnull;
+#define STYLE_STRUCT_RESET(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_INHERITED
+#undef STYLE_STRUCT_RESET
+
+  };
 };
 
 struct nsResetStyleData
 {
   nsResetStyleData()
-    :mDisplayData(nsnull), mMarginData(nsnull), mBorderData(nsnull), mPaddingData(nsnull), 
-     mOutlineData(nsnull), mPositionData(nsnull), mTableData(nsnull), mBackgroundData(nsnull),
-     mContentData(nsnull), mTextData(nsnull), mUIData(nsnull)
   {
-#ifdef INCLUDE_XUL
-    mXULData = nsnull;
-#endif
+#define STYLE_STRUCT_RESET(name, checkdata_cb) \
+    m##name##Data = nsnull;
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_RESET
+#undef STYLE_STRUCT_INHERITED
   };
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
   }
 
   void ClearInheritedData(PRUint32 aBits) {
-    if (mDisplayData && (aBits & NS_STYLE_INHERIT_DISPLAY))
-      mDisplayData = nsnull;
-    if (mMarginData && (aBits & NS_STYLE_INHERIT_MARGIN))
-      mMarginData = nsnull;
-    if (mBorderData && (aBits & NS_STYLE_INHERIT_BORDER))
-      mBorderData = nsnull;
-    if (mPaddingData && (aBits & NS_STYLE_INHERIT_PADDING))
-      mPaddingData = nsnull;
-    if (mOutlineData && (aBits & NS_STYLE_INHERIT_OUTLINE))
-      mOutlineData = nsnull;
-    if (mPositionData && (aBits & NS_STYLE_INHERIT_POSITION))
-      mPositionData = nsnull;
-    if (mTableData && (aBits & NS_STYLE_INHERIT_TABLE))
-      mTableData = nsnull;
-    if (mBackgroundData && (aBits & NS_STYLE_INHERIT_BACKGROUND))
-      mBackgroundData = nsnull;
-    if (mContentData && (aBits & NS_STYLE_INHERIT_CONTENT))
-      mContentData = nsnull;
-    if (mTextData && (aBits & NS_STYLE_INHERIT_TEXT_RESET))
-      mTextData = nsnull;
-    if (mUIData && (aBits & NS_STYLE_INHERIT_UI_RESET))
-      mUIData = nsnull;
-#ifdef INCLUDE_XUL
-    if (mXULData && (aBits & NS_STYLE_INHERIT_XUL))
-      mXULData = nsnull;
-#endif
+#define STYLE_STRUCT_RESET(name, checkdata_cb) \
+    if (m##name##Data && (aBits & NS_STYLE_INHERIT_BIT(name))) \
+      m##name##Data = nsnull;
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_RESET
+#undef STYLE_STRUCT_INHERITED
   };
 
   void Destroy(PRUint32 aBits, nsIPresContext* aContext) {
-    if (mDisplayData && !(aBits & NS_STYLE_INHERIT_DISPLAY))
-      mDisplayData->Destroy(aContext);
-    if (mMarginData && !(aBits & NS_STYLE_INHERIT_MARGIN))
-      mMarginData->Destroy(aContext);
-    if (mBorderData && !(aBits & NS_STYLE_INHERIT_BORDER))
-      mBorderData->Destroy(aContext);
-    if (mPaddingData && !(aBits & NS_STYLE_INHERIT_PADDING))
-      mPaddingData->Destroy(aContext);
-    if (mOutlineData && !(aBits & NS_STYLE_INHERIT_OUTLINE))
-      mOutlineData->Destroy(aContext);
-    if (mPositionData && !(aBits & NS_STYLE_INHERIT_POSITION))
-      mPositionData->Destroy(aContext);
-    if (mTableData && !(aBits & NS_STYLE_INHERIT_TABLE))
-      mTableData->Destroy(aContext);
-    if (mBackgroundData && !(aBits & NS_STYLE_INHERIT_BACKGROUND))
-      mBackgroundData->Destroy(aContext);
-    if (mContentData && !(aBits & NS_STYLE_INHERIT_CONTENT))
-      mContentData->Destroy(aContext);
-    if (mTextData && !(aBits & NS_STYLE_INHERIT_TEXT_RESET))
-      mTextData->Destroy(aContext);
-    if (mUIData && !(aBits & NS_STYLE_INHERIT_UI_RESET))
-      mUIData->Destroy(aContext);
-#ifdef INCLUDE_XUL
-    if (mXULData && !(aBits & NS_STYLE_INHERIT_XUL))
-      mXULData->Destroy(aContext);
-#endif
+#define STYLE_STRUCT_RESET(name, checkdata_cb) \
+    if (m##name##Data && !(aBits & NS_STYLE_INHERIT_BIT(name))) \
+      m##name##Data->Destroy(aContext);
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_RESET
+#undef STYLE_STRUCT_INHERITED
+
     aContext->FreeToShell(sizeof(nsResetStyleData), this);
   };
 
-  nsStyleDisplay* mDisplayData;
-  nsStyleMargin* mMarginData;
-  nsStyleBorder* mBorderData;
-  nsStylePadding* mPaddingData;
-  nsStyleOutline* mOutlineData;
-  nsStylePosition* mPositionData;
-  nsStyleTable* mTableData;
-  nsStyleBackground* mBackgroundData;
-  nsStyleContent* mContentData;
-  nsStyleTextReset* mTextData;
-  nsStyleUIReset* mUIData;
-#ifdef INCLUDE_XUL
-  nsStyleXUL* mXULData;
-#endif
+#define STYLE_STRUCT_RESET(name, checkdata_cb) \
+  nsStyle##name * m##name##Data;
+#define STYLE_STRUCT_INHERITED(name, checkdata_cb)
+
+#include "nsStyleStructList.h"
+
+#undef STYLE_STRUCT_RESET
+#undef STYLE_STRUCT_INHERITED
+
 };
 
 struct nsCachedStyleData
@@ -241,15 +186,34 @@ struct nsCachedStyleData
   };
 
   static PRUint32 GetBitForSID(const nsStyleStructID& aSID) {
-    return 1 << (aSID - 1);
+    return 1 << aSID;
   };
 
   nsStyleStruct* GetStyleData(const nsStyleStructID& aSID) {
+    // Each struct is stored at this.m##type##Data->m##name##Data where
+    // |type| is either Inherit or Reset, and |name| is the name of the
+    // style struct.  The |gInfo| stores the offset of the appropriate
+    // m##type##Data for the struct within nsCachedStyleData (|this|)
+    // and the offset of the appropriate m##name##Data within the
+    // m##type##Data.  Note that if we don't have any reset structs,
+    // then mResetData is null, and likewise for mInheritedData.  This
+    // saves us from having to go through the long if-else cascade into
+    // which most compilers will turn a case statement.
+
+    // NOTE:  nsStyleContext::SetStyle works roughly the same way.
+
     const StyleStructInfo& info = gInfo[aSID];
+
+    // Get either &mInheritedData or &mResetData.
     char* resetOrInheritSlot = NS_REINTERPRET_CAST(char*, this) + info.mCachedStyleDataOffset;
+
+    // Get either mInheritedData or mResetData.
     char* resetOrInherit = NS_REINTERPRET_CAST(char*, *NS_REINTERPRET_CAST(void**, resetOrInheritSlot));
+
     nsStyleStruct* data = nsnull;
     if (resetOrInherit) {
+      // If we have the mInheritedData or mResetData, then we might have
+      // the struct, so get it.
       char* dataSlot = resetOrInherit + info.mInheritResetOffset;
       data = *NS_REINTERPRET_CAST(nsStyleStruct**, dataSlot);
     }
@@ -331,6 +295,10 @@ public:
     //    where the inheritance is computed by the style system
   enum RuleDetail {
     eRuleNone, // No props have been specified at all.
+    eRulePartialReset, // At least one prop with a non-inherited value
+                       // has been specified.  No props have been
+                       // specified with an inherited value.  At least
+                       // one prop remains unspecified.
     eRulePartialMixed, // At least one prop with a non-inherited value
                        // has been specified.  Some props may also have
                        // been specified with an inherited value.  At
@@ -338,6 +306,8 @@ public:
     eRulePartialInherited, // Only props with inherited values have
                            // have been specified.  At least one prop
                            // remains unspecified.
+    eRuleFullReset, // All props have been specified.  None has an
+                    // inherited value.
     eRuleFullMixed, // All props have been specified.  At least one has
                     // a non-inherited value.
     eRuleFullInherited, // All props have been specified with inherited
@@ -348,55 +318,96 @@ public:
 
   enum { // Types of RuleBits
     eNoneBits,
-    eInheritBits
+    eDependentBits
   };
 
 private:
   nsIPresContext* mPresContext; // Our pres context.
 
-  nsRuleNode* mParent; // A pointer to the parent node in the tree.  This enables us to
-                       // walk backwards from the most specific rule matched to the least
-                       // specific rule (which is the optimal order to use for lookups
-                       // of style properties.
+  nsRuleNode* mParent; // A pointer to the parent node in the tree.
+                       // This enables us to walk backwards from the
+                       // most specific rule matched to the least
+                       // specific rule (which is the optimal order to
+                       // use for lookups of style properties.
   nsCOMPtr<nsIStyleRule> mRule; // A pointer to our specific rule.
 
-  union {
-    nsHashtable* mRootChildren; // A hashtable that maps from rules to our RuleNode children.
-                                // When matching rules, we use this table to transition from
-                                // node to node (constructing new nodes as needed to flesh out
-                                // the tree).
-    nsRuleList*  mChildren;
+  // The children of this node are stored in either a hashtable or list
+  // that maps from rules to our nsRuleNode children.  When matching
+  // rules, we use this mapping to transition from node to node
+  // (constructing new nodes as needed to flesh out the tree).
+
+  void *mChildrenTaggedPtr; // Accessed only through the methods below.
+
+  enum {
+    kTypeMask = 0x1,
+    kListType = 0x0,
+    kHashType = 0x1
   };
+  enum {
+    // Maximum to have in a list before converting to a hashtable.
+    // XXX Need to optimize this.
+    kMaxChildrenInList = 32
+  };
+
+  PRBool HaveChildren() {
+    return mChildrenTaggedPtr != nsnull;
+  }
+  PRBool ChildrenAreHashed() {
+    return (PRWord(mChildrenTaggedPtr) & kTypeMask) == kHashType;
+  }
+  nsRuleList* ChildrenList() {
+    return NS_REINTERPRET_CAST(nsRuleList*, mChildrenTaggedPtr);
+  }
+  PLDHashTable* ChildrenHash() {
+    return (PLDHashTable*) (PRWord(mChildrenTaggedPtr) & ~PRWord(kTypeMask));
+  }
+  void SetChildrenList(nsRuleList *aList) {
+    NS_ASSERTION(!(PRWord(aList) & kTypeMask),
+                 "pointer not 2-byte aligned");
+    mChildrenTaggedPtr = aList;
+  }
+  void SetChildrenHash(PLDHashTable *aHashtable) {
+    NS_ASSERTION(!(PRWord(aHashtable) & kTypeMask),
+                 "pointer not 2-byte aligned");
+    mChildrenTaggedPtr = (void*)(PRWord(aHashtable) | kHashType);
+  }
+  void ConvertChildrenToHash();
 
   nsCachedStyleData mStyleData;   // Any data we cached on the rule node.
 
-  PRUint32 mInheritBits;          // Used to cache the fact that we can look up cached data under a parent
-                                  // rule.  This is not the same thing as CSS inheritance.  
+  PRUint32 mDependentBits; // Used to cache the fact that we can look up
+                           // cached data under a parent rule.
 
-  PRUint32 mNoneBits;             // Used to cache the fact that this entire branch specifies no data
-                                  // for a given struct type.  For example, if an entire rule branch
-                                  // specifies no color information, then a bit will be set along every
-                                  // rule node on that branch, so that you can break out of the rule tree
-                                  // early.
+  PRUint32 mNoneBits; // Used to cache the fact that the branch to this
+                      // node specifies no non-inherited data for a
+                      // given struct type.  (This usually implies that
+                      // the entire branch specifies no non-inherited
+                      // data, although not necessarily, if a
+                      // non-inherited value is overridden by an
+                      // explicit 'inherit' value.)  For example, if an
+                      // entire rule branch specifies no color
+                      // information, then a bit will be set along every
+                      // rule node on that branch, so that you can break
+                      // out of the rule tree early and just inherit
+                      // from the parent style context.  The presence of
+                      // this bit means we should just get inherited
+                      // data from the parent style context, and it is
+                      // never used for reset structs since their
+                      // Compute*Data functions don't initialize from
+                      // inherited data.
 
 friend struct nsRuleList;
 
-protected:
-  // The callback function for deleting rule nodes from our rule tree.
-  static PRBool PR_CALLBACK DeleteChildren(nsHashKey *aKey, void *aData, void *closure);
-
 public:
   // Overloaded new operator. Initializes the memory to 0 and relies on an arena
   // (which comes from the presShell) to perform the allocation.
-  void* operator new(size_t sz, nsIPresContext* aContext);
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW;
   void Destroy();
   static nsILanguageAtomService *gLangService;
 
 protected:
-  void PropagateInheritBit(PRUint32 aBit, nsRuleNode* aHighestNode);
+  void PropagateDependentBit(PRUint32 aBit, nsRuleNode* aHighestNode);
   void PropagateNoneBit(PRUint32 aBit, nsRuleNode* aHighestNode);
- 
-  PRBool InheritsFromParentRule(const nsStyleStructID aSID);
   
   const nsStyleStruct* SetDefaultOnRoot(const nsStyleStructID aSID, nsIStyleContext* aContext);
 
@@ -461,10 +472,12 @@ protected:
                                             nsIStyleContext* aContext,  
                                             nsRuleNode* aHighestNode,
                                             const RuleDetail& aRuleDetail, PRBool aInherited);
-  const nsStyleStruct* ComputeUIData(nsStyleStruct* aStartData, const nsCSSStruct& aData, 
-                                     nsIStyleContext* aContext,  
-                                     nsRuleNode* aHighestNode,
-                                     const RuleDetail& aRuleDetail, PRBool aInherited);
+  const nsStyleStruct* ComputeUserInterfaceData(nsStyleStruct* aStartData,
+                                                const nsCSSStruct& aData, 
+                                                nsIStyleContext* aContext,  
+                                                nsRuleNode* aHighestNode,
+                                                const RuleDetail& aRuleDetail,
+                                                PRBool aInherited);
   const nsStyleStruct* ComputeUIResetData(nsStyleStruct* aStartData, const nsCSSStruct& aData, 
                                           nsIStyleContext* aContext,  
                                           nsRuleNode* aHighestNode,
@@ -513,7 +526,7 @@ protected:
   const nsStyleStruct* GetQuotesData(nsIStyleContext* aContext, PRBool aComputeData);
   const nsStyleStruct* GetTextData(nsIStyleContext* aContext, PRBool aComputeData);
   const nsStyleStruct* GetTextResetData(nsIStyleContext* aContext, PRBool aComputeData);
-  const nsStyleStruct* GetUIData(nsIStyleContext* aContext, PRBool aComputeData);
+  const nsStyleStruct* GetUserInterfaceData(nsIStyleContext* aContext, PRBool aComputeData);
   const nsStyleStruct* GetUIResetData(nsIStyleContext* aContext, PRBool aComputeData);
 #ifdef INCLUDE_XUL
   const nsStyleStruct* GetXULData(nsIStyleContext* aContext, PRBool aComputeData);
@@ -532,7 +545,7 @@ public:
   static void CreateRootNode(nsIPresContext* aPresContext, nsRuleNode** aResult);
 
   nsresult GetBits(PRInt32 aType, PRUint32* aResult);
-  nsresult Transition(nsIStyleRule* aRule, nsRuleNode** aResult);
+  nsresult Transition(nsIStyleRule* aRule, PRBool aIsInlineStyle, nsRuleNode** aResult);
   nsRuleNode* GetParent() { return mParent; }
   PRBool IsRoot() { return mParent == nsnull; }
   nsresult GetRule(nsIStyleRule** aResult)
@@ -541,6 +554,14 @@ public:
     NS_IF_ADDREF(*aResult);
     return NS_OK;
   }
+  nsIStyleRule* Rule() {
+    // NOTE:  Does not |AddRef|.
+    return mRule;
+  }
+  nsIPresContext* PresContext() {
+    // NOTE: Does not |AddRef|.
+    return mPresContext;
+  }
 
   nsresult ClearCachedData(nsIStyleRule* aRule);
   nsresult ClearCachedDataInSubtree(nsIStyleRule* aRule);
