@@ -48,7 +48,7 @@
 #include "nsStaticNameTable.h"
 
 // define an array of all CSS properties
-#define CSS_PROP(_name, _id, _hint) #_name,
+#define CSS_PROP(_name, _id, _method, _hint) #_name,
 const char* kCSSRawProperties[] = {
 #include "nsCSSPropList.h"
 };
@@ -222,6 +222,19 @@ const PRInt32 nsCSSProps::kBackgroundColorKTable[] = {
   -1,-1
 };
 
+const PRInt32 nsCSSProps::kBackgroundClipKTable[] = {
+  eCSSKeyword_border,     NS_STYLE_BG_CLIP_BORDER,
+  eCSSKeyword_padding,    NS_STYLE_BG_CLIP_PADDING,
+  -1,-1
+};
+
+const PRInt32 nsCSSProps::kBackgroundOriginKTable[] = {
+  eCSSKeyword_border,     NS_STYLE_BG_ORIGIN_BORDER,
+  eCSSKeyword_padding,    NS_STYLE_BG_ORIGIN_PADDING,
+  eCSSKeyword_content,    NS_STYLE_BG_ORIGIN_CONTENT,
+  -1,-1
+};
+
 const PRInt32 nsCSSProps::kBackgroundRepeatKTable[] = {
   eCSSKeyword_no_repeat,  NS_STYLE_BG_REPEAT_OFF,
   eCSSKeyword_repeat,     NS_STYLE_BG_REPEAT_XY,
@@ -320,6 +333,7 @@ const PRInt32 nsCSSProps::kColorKTable[] = {
   eCSSKeyword__moz_dialog, nsILookAndFeel::eColor__moz_dialog,
   eCSSKeyword__moz_dialogtext, nsILookAndFeel::eColor__moz_dialogtext,
   eCSSKeyword__moz_dragtargetzone, nsILookAndFeel::eColor__moz_dragtargetzone,
+  eCSSKeyword__moz_hyperlinktext, NS_COLOR_MOZ_HYPERLINKTEXT,
   eCSSKeyword__moz_mac_focusring, nsILookAndFeel::eColor__moz_mac_focusring,
   eCSSKeyword__moz_mac_menuselect, nsILookAndFeel::eColor__moz_mac_menuselect,
   eCSSKeyword__moz_mac_menushadow, nsILookAndFeel::eColor__moz_mac_menushadow,
@@ -331,6 +345,7 @@ const PRInt32 nsCSSProps::kColorKTable[] = {
   eCSSKeyword__moz_mac_accentregularshadow, nsILookAndFeel::eColor__moz_mac_accentregularshadow,
   eCSSKeyword__moz_mac_accentdarkshadow, nsILookAndFeel::eColor__moz_mac_accentdarkshadow,
   eCSSKeyword__moz_mac_accentdarkestshadow, nsILookAndFeel::eColor__moz_mac_accentdarkestshadow,
+  eCSSKeyword__moz_visitedhyperlinktext, NS_COLOR_MOZ_VISITEDHYPERLINKTEXT,
   -1,-1
 };
 
@@ -632,7 +647,7 @@ const PRInt32 nsCSSProps::kPlayDuringKTable[] = {
 };
 
 const PRInt32 nsCSSProps::kPositionKTable[] = {
-  eCSSKeyword_static, NS_STYLE_POSITION_NORMAL,
+  eCSSKeyword_static, NS_STYLE_POSITION_STATIC,
   eCSSKeyword_relative, NS_STYLE_POSITION_RELATIVE,
   eCSSKeyword_absolute, NS_STYLE_POSITION_ABSOLUTE,
   eCSSKeyword_fixed, NS_STYLE_POSITION_FIXED,
@@ -700,6 +715,7 @@ const PRInt32 nsCSSProps::kTextDecorationKTable[] = {
   eCSSKeyword_overline, NS_STYLE_TEXT_DECORATION_OVERLINE,
   eCSSKeyword_line_through, NS_STYLE_TEXT_DECORATION_LINE_THROUGH,
   eCSSKeyword_blink, NS_STYLE_TEXT_DECORATION_BLINK,
+  eCSSKeyword__moz_anchor_decoration, NS_STYLE_TEXT_DECORATION_PREF_ANCHORS,
   -1,-1
 };
 
@@ -892,7 +908,10 @@ static const PRInt32 kBackgroundYPositionKTable[] = {
 
   case eCSSProperty__moz_border_radius:
     break;
-  
+
+  case eCSSProperty_appearance:
+    return SearchKeywordTable(aValue, kAppearanceKTable);
+
   case eCSSProperty_azimuth:
     return SearchKeywordTable(aValue, kAzimuthKTable);
 
@@ -914,6 +933,12 @@ static const PRInt32 kBackgroundYPositionKTable[] = {
   case eCSSProperty_background_repeat:
     return SearchKeywordTable(aValue, kBackgroundRepeatKTable);
 
+  case eCSSProperty__moz_background_clip:
+    return SearchKeywordTable(aValue, kBackgroundClipKTable);
+
+  case eCSSProperty__moz_background_origin:
+    return SearchKeywordTable(aValue, kBackgroundOriginKTable);
+
   case eCSSProperty_background_x_position:
     return SearchKeywordTable(aValue, kBackgroundXPositionKTable);
 
@@ -1012,8 +1037,8 @@ static const PRInt32 kBackgroundYPositionKTable[] = {
   case eCSSProperty_content:
     return SearchKeywordTable(aValue, kContentKTable);  
 
-  case eCSSProperty_counter_increment:
-  case eCSSProperty_counter_reset:
+  case eCSSProperty__moz_counter_increment:
+  case eCSSProperty__moz_counter_reset:
   case eCSSProperty_cue:
   case eCSSProperty_cue_after:
   case eCSSProperty_cue_before:
@@ -1107,16 +1132,16 @@ static const PRInt32 kBackgroundYPositionKTable[] = {
 
   case eCSSProperty_opacity:
   case eCSSProperty_orphans:
-  case eCSSProperty_outline:
+  case eCSSProperty__moz_outline:
     break;
 
-  case eCSSProperty_outline_color:
+  case eCSSProperty__moz_outline_color:
     return SearchKeywordTable(aValue, kOutlineColorKTable);
 
-  case eCSSProperty_outline_style:
+  case eCSSProperty__moz_outline_style:
     return SearchKeywordTable(aValue, kBorderStyleKTable);
 
-  case eCSSProperty_outline_width:
+  case eCSSProperty__moz_outline_width:
     return SearchKeywordTable(aValue, kBorderWidthKTable);
 
   case eCSSProperty_overflow:
@@ -1280,8 +1305,8 @@ PRBool nsCSSProps::GetColorName(PRInt32 aPropValue, nsCString &aStr)
 }
 
 // define array of all CSS property hints
-#define CSS_PROP(_name, _id, _hint) NS_STYLE_HINT_##_hint,
-const PRInt32 nsCSSProps::kHintTable[eCSSProperty_COUNT] = {
+#define CSS_PROP(_name, _id, _method, _hint) _hint,
+const nsChangeHint nsCSSProps::kHintTable[eCSSProperty_COUNT] = {
 #include "nsCSSPropList.h"
 };
 #undef CSS_PROP
