@@ -137,7 +137,7 @@ public:
 
 protected:
   nsXBLAttributeEntry(nsIAtom* aSrcAtom, nsIAtom* aDstAtom, nsIContent* aContent) {
-    NS_INIT_REFCNT(); mSrcAttribute = aSrcAtom; mDstAttribute = aDstAtom; mElement = aContent;
+    NS_INIT_ISUPPORTS(); mSrcAttribute = aSrcAtom; mDstAttribute = aDstAtom; mElement = aContent;
   }
 
   virtual ~nsXBLAttributeEntry() {}
@@ -208,7 +208,7 @@ public:
 
 protected:
   nsXBLInsertionPointEntry(nsIContent* aParent) {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
     mInsertionIndex = 0;
     mInsertionParent = aParent;
   };
@@ -266,7 +266,7 @@ nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsACString& aID, nsIXBLDocume
   mInsertionPointTable(nsnull),
   mInterfaceTable(nsnull)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   
   mID = ToNewCString(aID);
 
@@ -550,8 +550,12 @@ nsXBLPrototypeBinding::InstallImplementation(nsIContent* aBoundElement)
 }
 
 NS_IMETHODIMP
-nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceID, PRBool aRemoveFlag, 
-                                        nsIContent* aChangedElement, nsIContent* aAnonymousContent)
+nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute,
+                                        PRInt32 aNameSpaceID,
+                                        PRBool aRemoveFlag, 
+                                        nsIContent* aChangedElement,
+                                        nsIContent* aAnonymousContent,
+                                        PRBool aNotify)
 {
   if (!mAttributeTable)
     return NS_OK;
@@ -579,7 +583,7 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceI
       xblAttr->GetDstAttribute(getter_AddRefs(dstAttr));
 
       if (aRemoveFlag)
-        realElement->UnsetAttr(aNameSpaceID, dstAttr, PR_TRUE);
+        realElement->UnsetAttr(aNameSpaceID, dstAttr, aNotify);
       else {
         PRBool attrPresent = PR_TRUE;
         nsAutoString value;
@@ -601,7 +605,7 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceI
         }
 
         if (attrPresent)
-          realElement->SetAttr(aNameSpaceID, dstAttr, value, PR_TRUE);
+          realElement->SetAttr(aNameSpaceID, dstAttr, value, aNotify);
       }
 
       // See if we're the <html> tag in XUL, and see if value is being
@@ -614,7 +618,7 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceI
         PRInt32 childCount;
         realElement->ChildCount(childCount);
         for (PRInt32 i = 0; i < childCount; i++)
-          realElement->RemoveChildAt(0, PR_TRUE);
+          realElement->RemoveChildAt(0, aNotify);
       
         if (!aRemoveFlag) {
           // Construct a new text node and insert it.
