@@ -79,7 +79,6 @@
 #include "nsINameSpaceManager.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFContainerUtils.h" 
-#include "nsIRDFContentModelBuilder.h"
 #include "nsIXULDocument.h"
 #include "nsIXULTemplateBuilder.h"
 #include "nsIRDFNode.h"
@@ -162,7 +161,7 @@ nsXULTemplateBuilder::nsXULTemplateBuilder(void)
       mFlags(0),
       mTop(nsnull)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 nsXULTemplateBuilder::~nsXULTemplateBuilder(void)
@@ -272,6 +271,20 @@ nsXULTemplateBuilder::GetDatabase(nsIRDFCompositeDataSource** aResult)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXULTemplateBuilder::Init(nsIContent* aElement)
+{
+    NS_PRECONDITION(aElement, "null ptr");
+    mRoot = aElement;
+    return LoadDataSources();
+}
+
+NS_IMETHODIMP
+nsXULTemplateBuilder::CreateContents(nsIContent* aElement)
+{
+    return NS_OK;
+}
+
 //----------------------------------------------------------------------
 //
 // nsIDocumentOberver interface
@@ -324,7 +337,8 @@ nsXULTemplateBuilder::ContentChanged(nsIDocument *aDocument,
 NS_IMETHODIMP
 nsXULTemplateBuilder::ContentStatesChanged(nsIDocument* aDocument,
                                            nsIContent* aContent1,
-                                           nsIContent* aContent2)
+                                           nsIContent* aContent2,
+                                           PRInt32 aStateMask)
 {
     return NS_OK;
 }
@@ -335,7 +349,7 @@ nsXULTemplateBuilder::AttributeChanged(nsIDocument *aDocument,
                                        PRInt32      aNameSpaceID,
                                        nsIAtom*     aAttribute,
                                        PRInt32      aModType, 
-                                       PRInt32      aHint)
+                                       nsChangeHint aHint)
 {
     // Check for a change to the 'ref' attribute on an atom, in which
     // case we may need to nuke and rebuild the entire content model
@@ -408,7 +422,7 @@ NS_IMETHODIMP
 nsXULTemplateBuilder::StyleRuleChanged(nsIDocument *aDocument,
                                        nsIStyleSheet* aStyleSheet,
                                        nsIStyleRule* aStyleRule,
-                                       PRInt32 aHint)
+                                       nsChangeHint aHint)
 {
     return NS_OK;
 }
@@ -991,9 +1005,9 @@ nsXULTemplateBuilder::InitHTMLTemplateRoot()
 }
 
 void
-nsXULTemplateBuilder::ParseAttribute(const nsAReadableString& aAttributeValue,
-                                     void (*aVariableCallback)(nsXULTemplateBuilder*, const nsAReadableString&, void*),
-                                     void (*aTextCallback)(nsXULTemplateBuilder*, const nsAReadableString&, void*),
+nsXULTemplateBuilder::ParseAttribute(const nsAString& aAttributeValue,
+                                     void (*aVariableCallback)(nsXULTemplateBuilder*, const nsAString&, void*),
+                                     void (*aTextCallback)(nsXULTemplateBuilder*, const nsAString&, void*),
                                      void* aClosure)
 {
     nsAString::const_iterator done_parsing;
@@ -1080,7 +1094,7 @@ struct SubstituteTextClosure {
 
 nsresult
 nsXULTemplateBuilder::SubstituteText(nsTemplateMatch& aMatch,
-                                     const nsAReadableString& aAttributeValue,
+                                     const nsAString& aAttributeValue,
                                      nsAString& aResult)
 {
     // See if it's the special value "..."
@@ -1114,7 +1128,7 @@ nsXULTemplateBuilder::SubstituteText(nsTemplateMatch& aMatch,
 
 void
 nsXULTemplateBuilder::SubstituteTextAppendText(nsXULTemplateBuilder* aThis,
-                                               const nsAReadableString& aText,
+                                               const nsAString& aText,
                                                void* aClosure)
 {
     // Append aString to the closure's result
@@ -1124,7 +1138,7 @@ nsXULTemplateBuilder::SubstituteTextAppendText(nsXULTemplateBuilder* aThis,
 
 void
 nsXULTemplateBuilder::SubstituteTextReplaceVariable(nsXULTemplateBuilder* aThis,
-                                                    const nsAReadableString& aVariable,
+                                                    const nsAString& aVariable,
                                                     void* aClosure)
 {
     // Substitute the value for the variable and append to the
@@ -1194,7 +1208,7 @@ struct IsVarInSetClosure {
 
 void
 nsXULTemplateBuilder::IsVarInSet(nsXULTemplateBuilder* aThis,
-                                 const nsAReadableString& aVariable,
+                                 const nsAString& aVariable,
                                  void* aClosure)
 {
     IsVarInSetClosure* c = NS_STATIC_CAST(IsVarInSetClosure*, aClosure);
@@ -1215,7 +1229,7 @@ nsXULTemplateBuilder::IsVarInSet(nsXULTemplateBuilder* aThis,
 
 PRBool
 nsXULTemplateBuilder::IsAttrImpactedByVars(nsTemplateMatch& aMatch,
-                                           const nsAReadableString& aAttributeValue,
+                                           const nsAString& aAttributeValue,
                                            const VariableSet& aModifiedVars)
 {
     // XXX at some point, it might be good to remember what attributes
@@ -1468,7 +1482,7 @@ nsXULTemplateBuilder::InitializeRuleNetwork()
     nsAutoString flags;
     mRoot->GetAttr(kNameSpaceID_None, nsXULAtoms::flags, flags);
 
-    if (flags.Find(NS_LITERAL_STRING("dont-test-empty").get()) >= 0)
+    if (flags.Find(NS_LITERAL_STRING("dont-test-empty")) >= 0)
         mFlags |= eDontTestEmpty;
 
     // Initialize the rule network
@@ -1501,7 +1515,7 @@ nsXULTemplateBuilder::GetTemplateRoot(nsIContent** aResult)
     nsAutoString templateID;
     mRoot->GetAttr(kNameSpaceID_None, nsXULAtoms::templateAtom, templateID);
 
-    if (templateID.Length()) {
+    if (!templateID.IsEmpty()) {
         nsCOMPtr<nsIDocument> doc;
         mRoot->GetDocument(*getter_AddRefs(doc));
         NS_ASSERTION(doc != nsnull, "root element has no document");
@@ -1602,12 +1616,12 @@ nsXULTemplateBuilder::CompileRules()
     // specified them.
     mContainerSymbol.Truncate();
     tmpl->GetAttr(kNameSpaceID_None, nsXULAtoms::container, mContainerSymbol);
-    if (mContainerSymbol.Length())
+    if (!mContainerSymbol.IsEmpty())
         mRules.PutSymbol(mContainerSymbol.get(), mContainerVar);
 
     mMemberSymbol.Truncate();
     tmpl->GetAttr(kNameSpaceID_None, nsXULAtoms::member, mMemberSymbol);
-    if (mMemberSymbol.Length())
+    if (!mMemberSymbol.IsEmpty())
         mRules.PutSymbol(mMemberSymbol.get(), mMemberVar);
 
     // Compile the rules beneath the <template>
@@ -1710,7 +1724,7 @@ nsXULTemplateBuilder::CompileExtendedRule(nsIContent* aRuleElement,
 
     rule->SetContainerVariable(mContainerVar);
 
-    if (! mMemberSymbol.Length()) {
+    if (mMemberSymbol.IsEmpty()) {
         // If the member variable hasn't already been specified, then
         // grovel over <action> to find it. We'll use the first one
         // that we find in a breadth-first search.
@@ -1749,7 +1763,7 @@ nsXULTemplateBuilder::CompileExtendedRule(nsIContent* aRuleElement,
     }
 
     // If we can't find a member symbol, then we're out of luck. Bail.
-    if (! mMemberSymbol.Length()) {
+    if (mMemberSymbol.IsEmpty()) {
         PR_LOG(gXULTemplateLog, PR_LOG_ALWAYS,
                ("xultemplate[%p] could not deduce member variable", this));
 
@@ -1769,7 +1783,7 @@ nsXULTemplateBuilder::CompileExtendedRule(nsIContent* aRuleElement,
         return rv;
     }
 
-    if (!mContainerSymbol.Length()) {
+    if (mContainerSymbol.IsEmpty()) {
         PR_LOG(gXULTemplateLog, PR_LOG_ALWAYS,
                ("xultemplate[%p] could not deduce container variable", this));
 
@@ -2074,7 +2088,7 @@ nsXULTemplateBuilder::CompileBinding(nsTemplateRule* aRule,
     nsAutoString subject;
     aBinding->GetAttr(kNameSpaceID_None, nsXULAtoms::subject, subject);
 
-    if (! subject.Length()) {
+    if (subject.IsEmpty()) {
         PR_LOG(gXULTemplateLog, PR_LOG_ALWAYS,
                ("xultemplate[%p] <binding> requires `subject'", this));
 
@@ -2095,7 +2109,7 @@ nsXULTemplateBuilder::CompileBinding(nsTemplateRule* aRule,
     // predicate
     nsAutoString predicate;
     aBinding->GetAttr(kNameSpaceID_None, nsXULAtoms::predicate, predicate);
-    if (! predicate.Length()) {
+    if (predicate.IsEmpty()) {
         PR_LOG(gXULTemplateLog, PR_LOG_ALWAYS,
                ("xultemplate[%p] <binding> requires `predicate'", this));
 
@@ -2117,7 +2131,7 @@ nsXULTemplateBuilder::CompileBinding(nsTemplateRule* aRule,
     nsAutoString object;
     aBinding->GetAttr(kNameSpaceID_None, nsXULAtoms::object, object);
 
-    if (! object.Length()) {
+    if (object.IsEmpty()) {
         PR_LOG(gXULTemplateLog, PR_LOG_ALWAYS,
                ("xultemplate[%p] <binding> requires `object'", this));
 
@@ -2171,6 +2185,8 @@ nsXULTemplateBuilder::CompileSimpleRule(nsIContent* aRuleElement,
         // never compare against {}:id attribute
         else if ((attr.get() == nsXULAtoms::id) && (attrNameSpaceID == kNameSpaceID_None))
             continue;
+        else if ((attr.get() == nsXULAtoms::parsetype) && (attrNameSpaceID == kNameSpaceID_None))
+            continue;
 
         nsAutoString value;
         rv = aRuleElement->GetAttr(attrNameSpaceID, attr, value);
@@ -2247,12 +2263,26 @@ nsXULTemplateBuilder::CompileSimpleRule(nsIContent* aRuleElement,
 
                 target = do_QueryInterface(resource);
             }
-            else {
-                nsCOMPtr<nsIRDFLiteral> literal;
-                rv = gRDFService->GetLiteral(value.get(), getter_AddRefs(literal));
-                if (NS_FAILED(rv)) return rv;
+            else {                
+                if (aRuleElement->HasAttr(kNameSpaceID_None, nsXULAtoms::parsetype)) {
+                   nsAutoString parseType;
+                   aRuleElement->GetAttr(kNameSpaceID_None, nsXULAtoms::parsetype, parseType);
+                   if (parseType.Equals(NS_LITERAL_STRING("Integer"))) {                     
+                     nsCOMPtr<nsIRDFInt> intLiteral;
+                     PRInt32 errorCode = nsnull;                     
+                     rv = gRDFService->GetIntLiteral(value.ToInteger(&errorCode), getter_AddRefs(intLiteral));
+                     if (NS_FAILED(rv)) return rv;
+                     
+                     target = do_QueryInterface(intLiteral);
+                   }
+                }
+                else {
+                   nsCOMPtr<nsIRDFLiteral> literal;
+                   rv = gRDFService->GetLiteral(value.get(), getter_AddRefs(literal));
+                   if (NS_FAILED(rv)) return rv;
 
-                target = do_QueryInterface(literal);
+                   target = do_QueryInterface(literal);
+                }
             }
 
             testnode = new nsRDFPropertyTestNode(aParentNode, mConflictSet, mDB, mMemberVar, property, target);
@@ -2293,7 +2323,7 @@ nsXULTemplateBuilder::CompileSimpleRule(nsIContent* aRuleElement,
 PRBool
 nsXULTemplateBuilder::CompileSimpleAttributeCondition(PRInt32 aNameSpaceID,
                                                       nsIAtom* aAttribute,
-                                                      const nsAReadableString& aValue,
+                                                      const nsAString& aValue,
                                                       InnerNode* aParentNode,
                                                       TestNode** aResult)
 {
@@ -2351,7 +2381,7 @@ nsXULTemplateBuilder::AddSimpleRuleBindings(nsTemplateRule* aRule, nsIContent* a
 
 void
 nsXULTemplateBuilder::AddBindingsFor(nsXULTemplateBuilder* aThis,
-                                     const nsAReadableString& aVariable,
+                                     const nsAString& aVariable,
                                      void* aClosure)
 {
     // We should *only* be recieving "rdf:"-style variables. Make
@@ -2366,7 +2396,7 @@ nsXULTemplateBuilder::AddBindingsFor(nsXULTemplateBuilder* aThis,
 
     // Strip it down to the raw RDF property by clobbering the "rdf:"
     // prefix
-    const nsAReadableString& propertyStr = Substring(aVariable, PRUint32(4), aVariable.Length() - 4);
+    const nsAString& propertyStr = Substring(aVariable, PRUint32(4), aVariable.Length() - 4);
 
     nsCOMPtr<nsIRDFResource> property;
     gRDFService->GetUnicodeResource(nsAutoString(propertyStr).get(), getter_AddRefs(property));
