@@ -75,6 +75,7 @@
 #include "nsIThread.h"
 #include "nsITimer.h"
 #include "nsDOMClassInfo.h"
+#include "nsIAtom.h"
 
 #ifdef MOZ_LOGGING
 // Force PR_LOGGING so we can get JS strict warnings even in release builds
@@ -88,8 +89,6 @@
 
 const size_t gStackSize = 8192;
 
-static NS_DEFINE_IID(kPrefServiceCID, NS_PREF_CID);
-
 #ifdef PR_LOGGING
 static PRLogModuleInfo* gJSDiagnostics = nsnull;
 #endif
@@ -350,7 +349,7 @@ int PR_CALLBACK
 nsJSContext::JSOptionChangedCallback(const char *pref, void *data)
 {
   nsresult rv;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
+  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
   if (NS_SUCCEEDED(rv)) {
     nsJSContext *context = NS_REINTERPRET_CAST(nsJSContext *, data);
     PRUint32 oldDefaultJSOptions = context->mDefaultJSOptions;
@@ -387,7 +386,7 @@ nsJSContext::JSOptionChangedCallback(const char *pref, void *data)
 
 nsJSContext::nsJSContext(JSRuntime *aRuntime) : mGCOnDestruction(PR_TRUE)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   ++sContextCount;
 
@@ -413,7 +412,7 @@ nsJSContext::nsJSContext(JSRuntime *aRuntime) : mGCOnDestruction(PR_TRUE)
     ::JS_SetOptions(mContext, mDefaultJSOptions);
 
     // Check for the JS strict option, which enables extra error checks
-    nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
+    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv)) {
       (void) prefs->RegisterCallback(js_options_dot_str,
                                      JSOptionChangedCallback,
@@ -446,7 +445,7 @@ nsJSContext::~nsJSContext()
   ::JS_SetContextPrivate(mContext, nsnull);
 
   // Unregister our "javascript.options.*" pref-changed callback.
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID));
+  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID));
   if (prefs) {
     prefs->UnregisterCallback(js_options_dot_str, JSOptionChangedCallback,
                               this);
@@ -1587,7 +1586,7 @@ nsJSContext::ScriptExecuted()
   return ScriptEvaluated(PR_FALSE);
 }
 
-NS_IMETHODIMP_(void)
+NS_IMETHODIMP
 nsJSContext::Notify(nsITimer *timer)
 {
   NS_ASSERTION(mContext, "No context in nsJSContext::Notify()!");
@@ -1597,6 +1596,7 @@ nsJSContext::Notify(nsITimer *timer)
   sReadyForGC = PR_TRUE;
 
   NS_RELEASE(sGCTimer);
+  return NS_OK;
 }
 
 void
@@ -1626,8 +1626,9 @@ nsJSContext::FireGCTimer()
 
   static PRBool first = PR_TRUE;
 
-  sGCTimer->Init(this, first ? NS_FIRST_GC_DELAY : NS_GC_DELAY,
-                 NS_PRIORITY_LOWEST);
+  sGCTimer->InitWithCallback(this, 
+                             first ? NS_FIRST_GC_DELAY : NS_GC_DELAY,
+                             nsITimer::TYPE_ONE_SHOT);
 
   first = PR_FALSE;
 }
