@@ -38,9 +38,11 @@
 
 #include "nsIGenericFactory.h"
 
-#include "nsEditorShell.h"		      // for the CID
+#include "nsEditorShell.h"          // for the CID
 #include "nsEditingSession.h"       // for the CID
-#include "nsComposerController.h"		// for the CID
+#include "nsComposerController.h"   // for the CID
+#include "nsEditorSpellCheck.h"     // for the CID
+#include "nsEditorService.h"
 
 ////////////////////////////////////////////////////////////////////////
 // Define the contructor function for the objects
@@ -51,6 +53,8 @@
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsEditorShell)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsEditingSession)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsComposerController)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsEditorService)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsEditorSpellCheck)
 
 ////////////////////////////////////////////////////////////////////////
 // Define a table of CIDs implemented by this module along with other
@@ -67,6 +71,19 @@ static const nsModuleComponentInfo components[] = {
       "@mozilla.org/editor/editorspellcheck;1", nsEditorShellConstructor, },
     { "Editing Session", NS_EDITINGSESSION_CID,
       "@mozilla.org/editor/editingsession;1", nsEditingSessionConstructor, },
+    { "Editor Service", NS_EDITORSERVICE_CID,
+      "@mozilla.org/editor/editorservice;1", nsEditorServiceConstructor,},
+    { "Editor Spell Checker", NS_EDITORSPELLCHECK_CID,
+      "@mozilla.org/editor/editorspellchecker;1",
+      nsEditorSpellCheckConstructor,},
+    { "Editor Startup Handler", NS_EDITORSERVICE_CID,
+      "@mozilla.org/commandlinehandler/general-startup;1?type=editor",
+      nsEditorServiceConstructor,
+      nsEditorService::RegisterProc,
+      nsEditorService::UnregisterProc, },
+    { "Edit Startup Handler", NS_EDITORSERVICE_CID,
+      "@mozilla.org/commandlinehandler/general-startup;1?type=edit",
+      nsEditorServiceConstructor, },
 };
 
 ////////////////////////////////////////////////////////////////////////
