@@ -193,6 +193,8 @@ protected:
   NS_IMETHOD GetProperties(nsVoidArray *aPropList);
   */
   
+  nsCString mContentMIMEType;       // MIME type of the doc we are editing.
+
   /** create a transaction for setting aAttribute to aValue on aElement
     */
   NS_IMETHOD CreateTxnForSetAttribute(nsIDOMElement *aElement, 
@@ -541,7 +543,6 @@ protected:
   nsIViewManager *mViewManager;
   PRInt32         mUpdateCount;
   nsCOMPtr<nsITransactionManager> mTxnMgr;
-  nsCOMPtr<nsICSSStyleSheet> mLastStyleSheet;			// is owning this dangerous?
   nsWeakPtr         mPlaceHolderTxn;     // weak reference to placeholder for begin/end batch purposes
   nsIAtom          *mPlaceHolderName;    // name of placeholder transaction
   PRInt32           mPlaceHolderBatch;   // nesting count for batching
