@@ -63,6 +63,8 @@
 #include "nsHTMLCSSUtils.h"
 #include "nsIParserService.h"
 
+#include "nsVoidArray.h"
+
 class nsIDOMKeyEvent;
 class nsITransferable;
 class nsIDOMEventReceiver;
@@ -117,9 +119,10 @@ public:
   /* ------------ nsPlaintextEditor overrides -------------- */
   NS_IMETHODIMP HandleKeyPress(nsIDOMKeyEvent* aKeyEvent);
   NS_IMETHODIMP CollapseSelectionToStart();
+  NS_IMETHOD GetIsDocumentEditable(PRBool *aIsDocumentEditable);
+  NS_IMETHODIMP BeginningOfDocument();
 
   /* ------------ nsIHTMLEditor methods -------------- */
-
   NS_IMETHOD CopyLastEditableChildStyles(nsIDOMNode *aPreviousBlock, nsIDOMNode *aNewBlock,
                                          nsIDOMNode **aOutBrNode);
 
@@ -152,9 +155,11 @@ public:
   NS_IMETHOD IncreaseFontSize();
   NS_IMETHOD DecreaseFontSize();
 
+  NS_IMETHOD PasteNoFormatting(PRInt32 aSelectionType);
   NS_IMETHOD InsertHTML(const nsAString &aInputString);
   NS_IMETHOD InsertHTMLWithCharset(const nsAString& aInputString,
                                    const nsAString& aCharset);
+
   NS_IMETHOD LoadHTML(const nsAString &aInputString);
   NS_IMETHOD LoadHTMLWithCharset(const nsAString& aInputString,
                                    const nsAString& aCharset);
@@ -206,20 +211,22 @@ public:
 
   /* ------------ nsIEditorStyleSheets methods -------------- */
 
-  NS_IMETHOD ApplyStyleSheet(const nsAString & aURL, nsICSSStyleSheet **aStyleSheet);
-  NS_IMETHOD ApplyOverrideStyleSheet(const nsAString & aURL, nsICSSStyleSheet **aStyleSheet);
-  /* Above 2 methods call this with appropriate aOverride value 
-   * Not exposed to IDL interface 
-  */
-  nsresult   ApplyDocumentOrOverrideStyleSheet(const nsAString & aURL, PRBool aOverride, nsICSSStyleSheet **aStyleSheet);
-  NS_IMETHOD AddStyleSheet(nsICSSStyleSheet* aSheet);
-  NS_IMETHOD RemoveStyleSheet(nsICSSStyleSheet* aSheet);
-  NS_IMETHOD RemoveOverrideStyleSheet(nsICSSStyleSheet* aSheet);
+  NS_IMETHOD AddStyleSheet(const nsAString & aURL);
+  NS_IMETHOD ReplaceStyleSheet(const nsAString& aURL);
+  NS_IMETHOD RemoveStyleSheet(const nsAString &aURL);
+
+  NS_IMETHOD AddOverrideStyleSheet(const nsAString & aURL);
+  NS_IMETHOD ReplaceOverrideStyleSheet(const nsAString& aURL);
+  NS_IMETHOD RemoveOverrideStyleSheet(const nsAString &aURL);
+
+  NS_IMETHOD EnableStyleSheet(const nsAString& aURL, PRBool aEnable);
 
   /* ------------ nsIEditorMailSupport methods -------------- */
 
   NS_IMETHOD PasteAsQuotation(PRInt32 aSelectionType);
-  NS_IMETHOD InsertAsQuotation(const nsAString & aQuotedText, nsIDOMNode **aNodeInserted);
+  NS_IMETHOD InsertTextWithQuotations(const nsAString & aQuotedText);
+  NS_IMETHOD InsertAsQuotation(const nsAString & aQuotedText,
+                               nsIDOMNode **aNodeInserted);
   NS_IMETHOD PasteAsCitedQuotation(const nsAString & aCitation,
                                    PRInt32 aSelectionType);
   NS_IMETHOD InsertAsCitedQuotation(const nsAString & aQuotedText,
@@ -269,9 +276,9 @@ public:
 
   NS_IMETHOD SetSelectionAfterTableEdit(nsIDOMElement* aTable, PRInt32 aRow, PRInt32 aCol, 
                                         PRInt32 aDirection, PRBool aSelected);
-  NS_IMETHOD GetSelectedOrParentTableElement(nsIDOMElement** aTableElement,
-                                             nsAString& aTagName,
-                                             PRInt32 *aSelectedCount);
+  NS_IMETHOD GetSelectedOrParentTableElement(nsAString& aTagName,
+                                             PRInt32 *aSelectedCount,
+                                             nsIDOMElement** aTableElement);
   NS_IMETHOD GetSelectedCellsType(nsIDOMElement *aElement, PRUint32 *aSelectionType);
 
   nsresult GetCellFromRange(nsIDOMRange *aRange, nsIDOMElement **aCell);
@@ -281,13 +288,13 @@ public:
   // (i.e., each cell added to selection is added in another range 
   //  in the selection's rangelist, independent of location in table)
   // aRange is optional: returns the range around the cell
-  NS_IMETHOD GetFirstSelectedCell(nsIDOMElement **aCell, nsIDOMRange **aRange);
+  NS_IMETHOD GetFirstSelectedCell(nsIDOMRange **aRange, nsIDOMElement **aCell);
   // Get next cell until no more are found. Always use GetFirstSelected cell first
   // aRange is optional: returns the range around the cell
-  NS_IMETHOD GetNextSelectedCell(nsIDOMElement **aCell, nsIDOMRange **aRange);
+  NS_IMETHOD GetNextSelectedCell(nsIDOMRange **aRange, nsIDOMElement **aCell);
 
   // Upper-left-most selected cell in table
-  NS_IMETHOD GetFirstSelectedCellInTable(nsIDOMElement **aCell, PRInt32 *aRowIndex, PRInt32 *aColIndex);
+  NS_IMETHOD GetFirstSelectedCellInTable(PRInt32 *aRowIndex, PRInt32 *aColIndex, nsIDOMElement **aCell);
     
   /* miscellaneous */
   // This sets background on the appropriate container element (table, cell,)
@@ -395,9 +402,11 @@ public:
 
   NS_IMETHOD SetAttributeOrEquivalent(nsIDOMElement * aElement,
                                       const nsAString & aAttribute,
-                                      const nsAString & aValue);
+                                      const nsAString & aValue,
+                                      PRBool aSuppressTransaction);
   NS_IMETHOD RemoveAttributeOrEquivalent(nsIDOMElement * aElement,
-                                         const nsAString & aAttribute);
+                                         const nsAString & aAttribute,
+                                         PRBool aSuppressTransaction);
 
   /** join together any afjacent editable text nodes in the range */
   NS_IMETHOD CollapseAdjacentTextNodes(nsIDOMRange *aInRange);
@@ -451,6 +460,25 @@ public:
                            PRBool aListOrCellNotEmpty,
                            PRBool aSafeToAskFrames,
                            PRBool *aSeenBR);
+
+  // Stylesheet-related methods that aren't part of nsIEditorStyleSheets.
+  nsresult AddCSSStyleSheet(nsICSSStyleSheet* aSheet);
+  nsresult GetCSSLoader(const nsAString& aURL, nsICSSLoader** aCSSLoader);
+
+  // Returns TRUE if sheet was loaded, false if it wasn't
+  PRBool   EnableExistingStyleSheet(const nsAString& aURL);
+
+  // Dealing with the internal style sheet lists:
+  nsresult EnsureStyleSheetArrays();
+  NS_IMETHOD GetStyleSheetForURL(const nsAString &aURL,
+                               nsICSSStyleSheet **_retval);
+  NS_IMETHOD GetURLForStyleSheet(nsICSSStyleSheet *aStyleSheet, nsAString &aURL);
+
+  // Add a url + known style sheet to the internal lists:
+  nsresult AddNewStyleSheetToList(const nsAString &aURL,
+                                  nsICSSStyleSheet *aStyleSheet);
+
+  nsresult RemoveStyleSheetFromList(const nsAString &aURL);
                        
 protected:
 
@@ -471,13 +499,6 @@ protected:
     */
   NS_IMETHOD GetLayoutObject(nsIDOMNode *aInNode, nsISupports **aOutLayoutObject);
 
-  /* StyleSheet load callback */
-  static void ApplyStyleSheetToPresShellDocument(nsICSSStyleSheet* aSheet, void *aData);
-
-  /* remove the old style sheet, and apply the supplied one */
-  NS_IMETHOD ReplaceStyleSheet(nsICSSStyleSheet *aNewSheet);
-
-
   // Return TRUE if aElement is a table-related elemet and caret was set
   PRBool SetCaretInTableCell(nsIDOMElement* aElement);
   PRBool IsElementInBody(nsIDOMElement* aElement);
@@ -607,7 +628,18 @@ protected:
 
   // Methods for handling plaintext quotations
   NS_IMETHOD PasteAsPlaintextQuotation(PRInt32 aSelectionType);
+
+  /** Insert a string as quoted text,
+    * replacing the selected text (if any).
+    * @param aQuotedText     The string to insert.
+    * @param aAddCites       Whether to prepend extra ">" to each line
+    *                        (usually true, unless those characters
+    *                        have already been added.)
+    * @return aNodeInserted  The node spanning the insertion, if applicable.
+    *                        If aAddCites is false, this will be null.
+    */
   NS_IMETHOD InsertAsPlaintextQuotation(const nsAString & aQuotedText,
+                                        PRBool aAddCites,
                                         nsIDOMNode **aNodeInserted);
 
   // factored methods for handling insertion of data from transferables (drag&drop or clipboard)
@@ -728,10 +760,7 @@ protected:
   nsresult GetFirstEditableChild( nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *aOutFirstChild);
   nsresult GetLastEditableChild( nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *aOutLastChild);
 
-#ifdef XXX_DEAD_CODE
-  // these should be removed some day by jfrancis:  GetFirstEditableLeaf & GetLastEditableLeaf
   nsresult GetFirstEditableLeaf( nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *aOutFirstLeaf);
-#endif
   nsresult GetLastEditableLeaf( nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *aOutLastLeaf);
 
   nsresult GetDOMEventReceiver(nsIDOMEventReceiver **aEventReceiver);
@@ -770,14 +799,25 @@ protected:
   PRBool   mCachedUnderlineStyle;
   nsString mCachedFontName;
 
+  // Used to disable HTML formatting commands during HTML source editing
+  PRBool   mCanEditHTML;
+
   // Used by GetFirstSelectedCell and GetNextSelectedCell
   PRInt32  mSelectedCellIndex;
 
   nsCOMPtr<nsIRangeUtils> mRangeHelper;
 
+  nsString mLastStyleSheetURL;
+  nsString mLastOverrideStyleSheetURL;
+
   PRBool mCSSAware;
   nsHTMLCSSUtils *mHTMLCSSUtils;
 
+  // Maintain a list of associated style sheets and their urls.
+  nsStringArray mStyleSheetURLs;
+  nsCOMPtr<nsISupportsArray> mStyleSheets;
+  PRInt32 mNumStyleSheets;
+
   // Maintain a static parser service ...
   static nsCOMPtr<nsIParserService> sParserService;
   // ... which means that we need an instance count to know when to delete it
