@@ -80,7 +80,6 @@
 #include "nsISupportsArray.h"
 #include "nsVoidArray.h"
 #include "nsFileSpec.h"
-#include "nsIFile.h"
 #include "nsIURL.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -94,6 +93,7 @@
 #include "nsIImage.h"
 #include "nsAOLCiter.h"
 #include "nsInternetCiter.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 
 // netwerk
@@ -112,20 +112,9 @@
 #include "nsIPref.h"
 const PRUnichar nbsp = 160;
 
-static NS_DEFINE_CID(kHTMLEditorCID,  NS_HTMLEDITOR_CID);
-static NS_DEFINE_CID(kCContentIteratorCID, NS_CONTENTITERATOR_CID);
-static NS_DEFINE_IID(kSubtreeIteratorCID, NS_SUBTREEITERATOR_CID);
-static NS_DEFINE_CID(kCRangeCID,      NS_RANGE_CID);
-static NS_DEFINE_CID(kCDOMSelectionCID,      NS_DOMSELECTION_CID);
-static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-static NS_DEFINE_IID(kCParserIID, NS_IPARSER_IID); 
-static NS_DEFINE_CID(kCParserCID, NS_PARSER_CID); 
-static NS_DEFINE_CID(kCTransitionalDTDCID,  NS_CTRANSITIONAL_DTD_CID);
-
 // Drag & Drop, Clipboard Support
 static NS_DEFINE_CID(kCClipboardCID,    NS_CLIPBOARD_CID);
 static NS_DEFINE_CID(kCTransferableCID, NS_TRANSFERABLE_CID);
-static NS_DEFINE_CID(kCDragServiceCID, NS_DRAGSERVICE_CID);
 static NS_DEFINE_CID(kCHTMLFormatConverterCID, NS_HTMLFORMATCONVERTER_CID);
 // private clipboard data flavors for html copy/paste
 #define kHTMLContext   "text/_moz_htmlcontext"
@@ -165,16 +154,15 @@ NS_IMETHODIMP nsPlaintextEditor::InsertTextFromTransferable(nsITransferable *tra
     flavor.AssignWithConversion( bestFlavor );   // just so we can use flavor.Equals()
     if (flavor.Equals(NS_LITERAL_STRING(kUnicodeMime)))
     {
-      nsCOMPtr<nsISupportsWString> textDataObj ( do_QueryInterface(genericDataObj) );
+      nsCOMPtr<nsISupportsString> textDataObj ( do_QueryInterface(genericDataObj) );
       if (textDataObj && len > 0)
       {
-        PRUnichar* text = nsnull;
-        textDataObj->ToString ( &text );
-        stuffToPaste.Assign ( text, len / 2 );
+        nsAutoString text;
+        textDataObj->GetData ( text );
+        NS_ASSERTION(text.Length() <= (len/2), "Invalid length!");
+        stuffToPaste.Assign ( text.get(), len / 2 );
         nsAutoEditBatch beginBatching(this);
         rv = InsertText(stuffToPaste);
-        if (text)
-          nsMemory::Free(text);
       }
     }
   }
@@ -493,10 +481,10 @@ NS_IMETHODIMP nsPlaintextEditor::DoDrag(nsIDOMEvent *aDragEvent)
     // if we have an empty string, we're done; otherwise continue
     if ( !buffer.IsEmpty() )
     {
-      nsCOMPtr<nsISupportsWString> dataWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+      nsCOMPtr<nsISupportsString> dataWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
       NS_ENSURE_TRUE(dataWrapper, NS_ERROR_FAILURE);
 
-      rv = dataWrapper->SetData( NS_CONST_CAST(PRUnichar*, buffer.get()) );
+      rv = dataWrapper->SetData( buffer );
       if (NS_FAILED(rv)) return rv;
 
       if (bIsPlainTextControl)
@@ -601,12 +589,12 @@ NS_IMETHODIMP nsPlaintextEditor::CanPaste(PRInt32 aSelectionType, PRBool *aCanPa
   // add the flavors for text editors
   for (const char* const* flavor = textEditorFlavors; *flavor; flavor++)
   {
-    nsCOMPtr<nsISupportsString> flavorString;            
-    nsComponentManager::CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, nsnull, 
-         NS_GET_IID(nsISupportsString), getter_AddRefs(flavorString));
+    nsCOMPtr<nsISupportsCString> flavorString;            
+    nsComponentManager::CreateInstance(NS_SUPPORTS_CSTRING_CONTRACTID, nsnull, 
+         NS_GET_IID(nsISupportsCString), getter_AddRefs(flavorString));
     if (flavorString)
     {
-      flavorString->SetData(*flavor);
+      flavorString->SetData(nsDependentCString(*flavor));
       flavorsList->AppendElement(flavorString);
     }
   }
