@@ -21,6 +21,8 @@
  *
  * Contributor(s):
  *
+ *   Adam Lock <adamlock@netscape.com>
+ *   Paul Oswald <paul.oswald@isinet.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -40,33 +42,18 @@
 #include "jni.h"
 #include "npapi.h"
 
-#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
-#define IMPLEMENT_MozAxPlugin
-#include "MozAxPlugin.h"
-#include "netscape_plugin_Plugin.h"
-#endif
-
 #include "nsISupports.h"
+#include "nsString.h"
 
-// Plugin types supported
-enum PluginInstanceType
-{
-    itScript,
-    itControl
-};
-
-// Data associated with a plugin instance
-struct PluginInstanceData {
-    PluginInstanceType nType;
-    union
-    {
-        CActiveScriptSiteInstance *pScriptSite;
-        CControlSiteInstance *pControlSite;
-    };
 #ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
-    nsISupports *pScriptingPeer;
+#include "XPConnect.h"
 #endif
-};
+
+#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
+#include "LiveConnect.h"
+#endif
+
+#include "LegacyPlugin.h"
 
 
 // NPP_Initialize
@@ -89,11 +76,14 @@ NPError NPP_Initialize(void)
 void NPP_Shutdown(void)
 {
     NG_TRACE_METHOD(NPP_Shutdown);
+#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
+    liveconnect_Shutdown();
+#endif
     _Module.Unlock();
 }
 
 
-// npp_GetJavaClass
+// NPP_GetJavaClass
 //
 //    Return the Java class representing this plugin
 //
@@ -101,25 +91,15 @@ jref NPP_GetJavaClass(void)
 {
     NG_TRACE_METHOD(NPP_GetJavaClass);
 #ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
-    struct java_lang_Class* myClass;
-    JRIEnv* env = NPN_GetJavaEnv();
-    if (env == NULL)
-        return NULL;        /* Java disabled */
-
-    myClass = use_MozAxPlugin(env);
-    use_netscape_plugin_Plugin( env );
-    return (jref) myClass;
-#else
-    return NULL;
+    return liveconnect_GetJavaClass();
 #endif
+    return NULL;
 }
 
-
 #define MIME_OLEOBJECT1   "application/x-oleobject"
 #define MIME_OLEOBJECT2   "application/oleobject"
 #define MIME_ACTIVESCRIPT "text/x-activescript"
 
-
 NPError NewScript(const char *pluginType,
                     PluginInstanceData *pData,
                     uint16 mode,
@@ -135,6 +115,48 @@ NPError NewScript(const char *pluginType,
     return NPERR_GENERIC_ERROR;
 }
 
+static BOOL WillHandleCLSID(const CLSID &clsid)
+{
+    if (::IsEqualCLSID(clsid, CLSID_NULL))
+    {
+        return FALSE;
+    }
+
+    const BOOL kRestrictControls = FALSE;
+    if (!kRestrictControls)
+    {
+        return TRUE;
+    }
+
+    // Check if the CLSID belongs to a limited number the plugin is prepared to support
+    CRegKey key;
+    if (key.Open(HKEY_LOCAL_MACHINE, _T("Software\\Mozilla\\ActiveX\\CLSID"), KEY_READ) != ERROR_SUCCESS)
+    {
+        return FALSE;
+    }
+
+    // Enumerate CLSIDs looking for this one
+    int i = 0;
+    do {
+        USES_CONVERSION;
+        TCHAR szCLSID[64];
+        const DWORD nLength = sizeof(szCLSID) / sizeof(szCLSID[0]);
+        if (::RegEnumKey(key, i++, szCLSID, nLength) != ERROR_SUCCESS)
+        {
+            break;
+        }
+        szCLSID[nLength - 1] = TCHAR('\0');
+        CLSID clsidToCompare = GUID_NULL;
+        if (SUCCEEDED(::CLSIDFromString(T2OLE(szCLSID), &clsidToCompare)) &&
+            ::IsEqualCLSID(clsid, clsidToCompare))
+        {
+            return TRUE;
+        }
+    } while (1);
+
+    return FALSE;
+}
+
 NPError NewControl(const char *pluginType,
                     PluginInstanceData *pData,
                     uint16 mode,
@@ -144,10 +166,15 @@ NPError NewControl(const char *pluginType,
 {
     // Read the parameters
     CLSID clsid = CLSID_NULL;
-    tstring szName;
-    tstring szCodebase;
+    CComBSTR codebase;
     PropertyList pl;
 
+    if (strcmp(pluginType, MIME_OLEOBJECT1) != 0 &&
+        strcmp(pluginType, MIME_OLEOBJECT2) != 0)
+    {
+        clsid = xpc_GetCLSIDForType(pluginType);
+    }
+
     for (int16 i = 0; i < argc; i++)
     {
         if (stricmp(argn[i], "CLSID") == 0 ||
@@ -161,61 +188,62 @@ NPError NewControl(const char *pluginType,
             //
             // The first example is the proper way
 
-            char szCLSID[256];
+            const kCLSIDLen = 256;
+            char szCLSID[kCLSIDLen];
             if (strlen(argv[i]) < sizeof(szCLSID))
             {
                 if (strnicmp(argv[i], "CLSID:", 6) == 0)
                 {
-                    sprintf(szCLSID, "{%s}", argv[i]+6);
+                    _snprintf(szCLSID, kCLSIDLen - 1, "{%s}", argv[i]+6);
                 }
                 else if(argv[i][0] != '{')
                 {
-                    sprintf(szCLSID, "{%s}", argv[i]);
+                    _snprintf(szCLSID, kCLSIDLen - 1, "{%s}", argv[i]);
                 }
                 else
                 {
-                    strncpy(szCLSID, argv[i], sizeof(szCLSID));
+                    strncpy(szCLSID, argv[i], kCLSIDLen - 1);
                 }
+                szCLSID[kCLSIDLen - 1] = '\0';
                 USES_CONVERSION;
                 CLSIDFromString(A2OLE(szCLSID), &clsid);
             }
         }
-        else if (stricmp(argn[i], "NAME") == 0)
-        {
-            USES_CONVERSION;
-            szName = tstring(A2T(argv[i]));
-        }
         else if (stricmp(argn[i], "CODEBASE") == 0)
         {
-            szCodebase = tstring(A2T(argv[i]));
+            codebase = argv[i];
         }
         else 
         {
-            USES_CONVERSION;
-
-            std::wstring szName;
-            std::wstring szParam(A2W(argv[i]));
-
+            CComBSTR paramName;
             if (strnicmp(argn[i], "PARAM_", 6) == 0)
             {
-                szName = A2W(argn[i]+6);
+                paramName = argn[i] + 6;
+            }
+            else if (stricmp(argn[i], "PARAM") == 0)
+            {
+                // The next argn and argv values after this symbol
+                // will be <param> tag names and values
+                continue;
             }
             else
             {
-                szName = A2W(argn[i]);
+                paramName = argn[i];
             }
 
             // Empty parameters are ignored
-            if (szName.empty())
+            if (!paramName.m_str || paramName.Length() == 0)
             {
                 continue;
             }
 
+            nsAutoString paramValue; paramValue.AssignWithConversion(argv[i]);
+
             // Check for existing params with the same name
             BOOL bFound = FALSE;
-            for (PropertyList::const_iterator i = pl.begin(); i != pl.end(); i++)
+            for (PropertyList::const_iterator it = pl.begin(); it != pl.end(); it++)
             {
-                if (wcscmp((BSTR) (*i).szName, szName.c_str()) == 0)
+                if (wcscmp((BSTR) (*it).szName, (BSTR) paramName) == 0)
                 {
                     bFound = TRUE;
                     break;
@@ -228,12 +256,13 @@ NPError NewControl(const char *pluginType,
                 continue;
             }
 
-            CComVariant vsValue(szParam.c_str());
+            CComVariant vsValue(paramValue.get());
             CComVariant vIValue; // Value converted to int
             CComVariant vRValue; // Value converted to real
+            CComVariant vBValue; // Value converted to bool
             CComVariant &vValue = vsValue;
 
-            // See if the variant can be converted to an integer
+            // See if the variant can be converted to a number or boolean
             if (VariantChangeType(&vIValue, &vsValue, 0, VT_I4) == S_OK)
             {
                 vValue = vIValue;
@@ -242,21 +271,28 @@ NPError NewControl(const char *pluginType,
             {
                 vValue = vRValue;
             }
+            else if (VariantChangeType(&vBValue, &vsValue, 0, VT_BOOL) == S_OK)
+            {
+                vValue = vBValue;
+            }
+
 
             // Add named parameter to list
             Property p;
-            p.szName = szName.c_str();
+            p.szName = paramName;
             p.vValue = vValue;
             pl.push_back(p);
         }
     }
 
-    // Make sure we got a CLSID
-    if (memcmp(&clsid, &CLSID_NULL, sizeof(CLSID)) == 0)
+    // Make sure we got a CLSID we can handle
+    if (!WillHandleCLSID(clsid))
     {
         return NPERR_GENERIC_ERROR;
     }
 
+    pData->clsid = clsid;
+
     // Create the control site
     CControlSiteInstance *pSite = NULL;
     CControlSiteInstance::CreateInstance(&pSite);
@@ -264,13 +300,21 @@ NPError NewControl(const char *pluginType,
     {
         return NPERR_GENERIC_ERROR;
     }
+    pSite->m_bSupportWindowlessActivation = FALSE;
     pSite->AddRef();
 
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
+    CComPtr<IServiceProvider> sp;
+    xpc_GetServiceProvider(pData, &sp);
+    if (sp)
+        pSite->SetServiceProvider(sp);
+#endif
+
     // TODO check the object is installed and at least as recent as
     //      that specified in szCodebase
 
     // Create the object
-    if (FAILED(pSite->Create(clsid, pl, szName)))
+    if (FAILED(pSite->Create(clsid, pl)))
     {
         USES_CONVERSION;
         LPOLESTR szClsid;
@@ -284,8 +328,20 @@ NPError NewControl(const char *pluginType,
         return NPERR_GENERIC_ERROR;
     }
 
+    nsEventSinkInstance *pSink = NULL;
+    nsEventSinkInstance::CreateInstance(&pSink);
+    if (pSink)
+    {
+        pSink->AddRef();
+        pSink->mPlugin = pData;
+        CComPtr<IUnknown> control;
+        pSite->GetControlUnknown(&control);
+        pSink->SubscribeToEvents(control);
+    }
+
     pData->nType = itControl;
     pData->pControlSite = pSite;
+    pData->pControlEventSink = pSink;
 
     return NPERR_NO_ERROR;
 }
@@ -317,28 +373,40 @@ NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
     {
         return NPERR_GENERIC_ERROR;
     }
+    pData->pPluginInstance = instance;
+    pData->szUrl = NULL;
+    pData->szContentType = (pluginType) ? strdup(pluginType) : NULL;
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
+    pData->pScriptingPeer = NULL;
+#endif
 
     // Create a plugin according to the mime type
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
+    xpc_AddRef();
+#endif
 
     NPError rv = NPERR_GENERIC_ERROR;
     if (strcmp(pluginType, MIME_ACTIVESCRIPT) == 0)
     {
         rv = NewScript(pluginType, pData, mode, argc, argn, argv);
     }
-    else if (strcmp(pluginType, MIME_OLEOBJECT1) == 0 ||
-             strcmp(pluginType, MIME_OLEOBJECT2) == 0)
+    else /* if (strcmp(pluginType, MIME_OLEOBJECT1) == 0 ||
+             strcmp(pluginType, MIME_OLEOBJECT2) == 0) */
     {
         rv = NewControl(pluginType, pData, mode, argc, argn, argv);
     }
-    else
-    {
-        // Unknown MIME type
-    }
 
     // Test if plugin creation has succeeded and cleanup if it hasn't
     if (rv != NPERR_NO_ERROR)
     {
+        if (pData->szContentType)
+            free(pData->szContentType);
+        if (pData->szUrl)
+            free(pData->szUrl);
         delete pData;
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
+        xpc_Release();
+#endif
         return rv;
     }
 
@@ -372,17 +440,31 @@ NPP_Destroy(NPP instance, NPSavedData** save)
             pSite->Detach();
             pSite->Release();
         }
+        if (pData->pControlEventSink)
+        {
+            pData->pControlEventSink->UnsubscribeFromEvents();
+            pData->pControlEventSink->Release();
+        }
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
         if (pData->pScriptingPeer)
         {
             pData->pScriptingPeer->Release();
         }
+#endif
     }
     else if (pData->nType == itScript)
     {
         // TODO
     }
 
+    if (pData->szUrl)
+        free(pData->szUrl);
+    if (pData->szContentType)
+        free(pData->szContentType);
     delete pData;
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
+    xpc_Release();
+#endif
 
     instance->pdata = 0;
 
@@ -447,7 +529,7 @@ NPP_SetWindow(NPP instance, NPWindow* window)
                 pSite->SetPosition(rcPos);
             }
         }
-}
+    }
 
     return NPERR_NO_ERROR;
 }
@@ -604,161 +686,23 @@ NPP_Print(NPP instance, NPPrint* printInfo)
 void
 NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
 {
-}
-
-#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
-
-#include "nsIMozAxPlugin.h"
-#include "nsIClassInfo.h"
-
-// We must implement nsIClassInfo because it signals the
-// Mozilla Security Manager to allow calls from JavaScript.
-
-// helper class to implement all necessary nsIClassInfo method stubs
-// and to set flags used by the security system
-class nsClassInfoMozAxPlugin : public nsIClassInfo
-{
-  // These flags are used by the DOM and security systems to signal that 
-  // JavaScript callers are allowed to call this object's scritable methods.
-  NS_IMETHOD GetFlags(PRUint32 *aFlags)
-    {*aFlags = nsIClassInfo::PLUGIN_OBJECT | nsIClassInfo::DOM_OBJECT;
-     return NS_OK;}
-  NS_IMETHOD GetImplementationLanguage(PRUint32 *aImplementationLanguage)
-    {*aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
-     return NS_OK;}
-
-  // The rest of the methods can safely return error codes...
-  NS_IMETHOD GetInterfaces(PRUint32 *count, nsIID * **array)
-    {return NS_ERROR_NOT_IMPLEMENTED;}
-  NS_IMETHOD GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
-    {return NS_ERROR_NOT_IMPLEMENTED;}
-  NS_IMETHOD GetContractID(char * *aContractID)
-    {return NS_ERROR_NOT_IMPLEMENTED;}
-  NS_IMETHOD GetClassDescription(char * *aClassDescription)
-    {return NS_ERROR_NOT_IMPLEMENTED;}
-  NS_IMETHOD GetClassID(nsCID * *aClassID)
-    {return NS_ERROR_NOT_IMPLEMENTED;}
-  NS_IMETHOD GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
-    {return NS_ERROR_NOT_IMPLEMENTED;}
-};
-
-class nsScriptablePeer : public nsIMozAxPlugin,
-                         public nsClassInfoMozAxPlugin
-{
-    long mRef;
-
-protected:
-    virtual ~nsScriptablePeer();
-
-public:
-    nsScriptablePeer();
-
-    NS_DECL_ISUPPORTS
-    NS_DECL_NSIMOZAXPLUGIN
-};
-
-nsScriptablePeer::nsScriptablePeer()
-{
-    mRef = 0;
-}
-
-nsScriptablePeer::~nsScriptablePeer()
-{
-}
-
-///////////////////////////////////////////////////////////////////////////////
-// nsISupports
-
-// NOTE: We're not using the helpful NS_IMPL_ISUPPORTS macro because they drag
-//       in dependencies to xpcom that a plugin like ourselves is better off
-//       without.
-
-/* void QueryInterface (in nsIIDRef uuid, [iid_is (uuid), retval] out nsQIResult result); */
-NS_IMETHODIMP nsScriptablePeer::QueryInterface(const nsIID & uuid, void * *result)
-{
-    if (uuid.Equals(NS_GET_IID(nsISupports)))
+    PluginInstanceData *pData = (PluginInstanceData *) instance->pdata;
+    if (pData)
     {
-        *result = NS_STATIC_CAST(void *, this);
-        AddRef();
-        return NS_OK;
+        if (pData->szUrl)
+            free(pData->szUrl);
+        pData->szUrl = strdup(url);
     }
-    else if (uuid.Equals(NS_GET_IID(nsIMozAxPlugin)))
-    {
-        *result = NS_STATIC_CAST(void *, this);
-        AddRef();
-        return NS_OK;
-    }
-    return NS_NOINTERFACE;
 }
 
-/* [noscript, notxpcom] nsrefcnt AddRef (); */
-NS_IMETHODIMP_(nsrefcnt) nsScriptablePeer::AddRef()
-{
-    mRef++;
-    return NS_OK;
-}
-
-/* [noscript, notxpcom] nsrefcnt Release (); */
-NS_IMETHODIMP_(nsrefcnt) nsScriptablePeer::Release()
-{
-    mRef--;
-    if (mRef <= 0)
-    {
-        delete this;
-    }
-    return NS_OK;
-}
-
-///////////////////////////////////////////////////////////////////////////////
-// nsIMozAxPlugin
-
-// the following method will be callable from JavaScript
-NS_IMETHODIMP 
-nsScriptablePeer::X()
-{
-    return NS_OK;
-}
-
-
-// Happy happy fun fun - redefine some NPPVariable values that we might
-// be asked for but not defined by the PluginSDK 
-const int kVarScriptableInstance = 10; // NPPVpluginScriptableInstance
-const int kVarScriptableIID = 11; // NPPVpluginScriptableIID
-
 NPError	NP_LOADDS
 NPP_GetValue(NPP instance, NPPVariable variable, void *value)
 {
-    if (instance == NULL)
-    {
-        return NPERR_INVALID_INSTANCE_ERROR;
-    }
-
-    if (variable == kVarScriptableInstance)
-    {
-        PluginInstanceData *pData = (PluginInstanceData *) instance->pdata;
-        if (!pData->pScriptingPeer)
-        {
-            nsScriptablePeer *peer  = new nsScriptablePeer();
-            peer->AddRef();
-            pData->pScriptingPeer = (nsIMozAxPlugin *) peer;
-        }
-        if (pData->pScriptingPeer)
-        {
-            pData->pScriptingPeer->AddRef();
-            *((nsISupports **) value)= pData->pScriptingPeer;
-            return NPERR_NO_ERROR;
-        }
-    }
-    else if (variable == kVarScriptableIID)
-    {
-	static nsIID kIMozAxPluginIID = NS_IMOZAXPLUGIN_IID;
-        nsIID *piid = (nsIID *) NPN_MemAlloc(sizeof(nsIID));
-        *piid = kIMozAxPluginIID;
-        *((nsIID **) value) = piid;
-        return NPERR_NO_ERROR;
-    }
-
-    return NPERR_GENERIC_ERROR;
+    NPError rv = NPERR_GENERIC_ERROR;
+#ifdef MOZ_ACTIVEX_PLUGIN_XPCONNECT
+    rv = xpc_GetValue(instance, variable, value);
+#endif
+    return rv;
 }
 
 NPError	NP_LOADDS
@@ -766,8 +710,3 @@ NPP_SetValue(NPP instance, NPNVariable variable, void *value)
 {
     return NPERR_GENERIC_ERROR;
 }
-#endif
-
-#ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
-#include "_stubs/MozAxPlugin.c"
-#endif
