@@ -42,6 +42,7 @@
 
 // Interfaces needed to be included
 #include "nsIContextMenuListener.h"
+#include "nsIContextMenuListener2.h"
 #include "nsITooltipListener.h"
 #include "nsIPrivateDOMEvent.h"
 #include "nsIDOMNode.h"
@@ -53,6 +54,7 @@
 #include "nsIDOMMouseEvent.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOMNamedNodeMap.h"
+#include "nsIDOMHTMLInputElement.h"
 #include "nsIWebNavigation.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIPresShell.h"
@@ -68,9 +70,8 @@
 #include "nsIWebBrowserChromeFocus.h"
 #include "nsIDragDropOverride.h"
 #include "nsIContent.h"
-
-static const char sWindowWatcherContractID[] = "@mozilla.org/embedcomp/window-watcher;1";
-
+#include "imgIContainer.h"
+#include "nsContextMenuInfo.h"
 
 //
 // GetEventReceiver
@@ -118,7 +119,7 @@ nsDocShellTreeOwner::nsDocShellTreeOwner() :
    mChromeTooltipListener(nsnull),
    mChromeContextMenuListener(nsnull)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsDocShellTreeOwner::~nsDocShellTreeOwner()
@@ -300,7 +301,7 @@ nsresult nsDocShellTreeOwner::FindItemWithNameAcrossWindows(
                 nsIDocShellTreeItem** aFoundItem)
 {
   // search for the item across the list of top-level windows
-  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(sWindowWatcherContractID));
+  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   if (!wwatch)
     return NS_OK;
 
@@ -343,7 +344,7 @@ void nsDocShellTreeOwner::EnsurePrompter()
   if (mPrompter)
     return;
 
-  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   if (wwatch && mWebBrowser) {
     nsCOMPtr<nsIDOMWindow> domWindow;
     mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
@@ -357,7 +358,7 @@ void nsDocShellTreeOwner::EnsureAuthPrompter()
   if (mAuthPrompter)
     return;
 
-  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   if (wwatch && mWebBrowser) {
     nsCOMPtr<nsIDOMWindow> domWindow;
     mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
@@ -372,7 +373,7 @@ void nsDocShellTreeOwner::AddToWatcher() {
     nsCOMPtr<nsIDOMWindow> domWindow;
     mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
     if (domWindow) {
-      nsCOMPtr<nsPIWindowWatcher> wwatch(do_GetService(sWindowWatcherContractID));
+      nsCOMPtr<nsPIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
       if (wwatch)
         wwatch->AddWindow(domWindow, mWebBrowserChrome);
     }
@@ -385,7 +386,7 @@ void nsDocShellTreeOwner::RemoveFromWatcher() {
     nsCOMPtr<nsIDOMWindow> domWindow;
     mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
     if (domWindow) {
-      nsCOMPtr<nsPIWindowWatcher> wwatch(do_GetService(sWindowWatcherContractID));
+      nsCOMPtr<nsPIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
       if (wwatch)
         wwatch->RemoveWindow(domWindow);
     }
@@ -809,8 +810,9 @@ nsDocShellTreeOwner :: AddChromeListeners ( )
   
   // install context menus
   if ( !mChromeContextMenuListener ) {
+    nsCOMPtr<nsIContextMenuListener2> contextListener2 ( do_QueryInterface(mWebBrowserChrome) );
     nsCOMPtr<nsIContextMenuListener> contextListener ( do_QueryInterface(mWebBrowserChrome) );
-    if ( contextListener ) {
+    if ( contextListener2 || contextListener ) {
       mChromeContextMenuListener = new ChromeContextMenuListener ( mWebBrowser, mWebBrowserChrome );
       if ( mChromeContextMenuListener ) {
         NS_ADDREF(mChromeContextMenuListener);
@@ -820,7 +822,7 @@ nsDocShellTreeOwner :: AddChromeListeners ( )
         rv = NS_ERROR_OUT_OF_MEMORY;
     }
   }
-  
+   
   // install the external dragDrop handler
   if ( !mChromeDragHandler ) {
     mChromeDragHandler = do_CreateInstance("@mozilla.org:/content/content-area-dragdrop;1", &rv);
@@ -884,7 +886,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(DefaultTooltipTextProvider, nsITooltipTextProvider
 
 DefaultTooltipTextProvider::DefaultTooltipTextProvider()
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
     
     // There are certain element types which we don't want to use
     // as tool tip text. 
@@ -964,7 +966,7 @@ ChromeTooltipListener :: ChromeTooltipListener ( nsWebBrowser* inBrowser, nsIWeb
      mMouseClientX(0), mMouseClientY(0),
      mShowingTooltip(PR_FALSE)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mTooltipTextProvider = do_GetService(NS_TOOLTIPTEXTPROVIDER_CONTRACTID);
   if (!mTooltipTextProvider) {
@@ -1203,7 +1205,8 @@ ChromeTooltipListener::MouseMove(nsIDOMEvent* aMouseEvent)
     if ( eventTarget )
       mPossibleTooltipNode = do_QueryInterface(eventTarget);
     if ( mPossibleTooltipNode ) {
-      nsresult rv = mTooltipTimer->Init(sTooltipCallback, this, kTooltipShowTime, PR_TRUE);
+      nsresult rv = mTooltipTimer->InitWithFuncCallback(sTooltipCallback, this, kTooltipShowTime, 
+                                                        nsITimer::TYPE_ONE_SHOT);
       if (NS_FAILED(rv))
         mPossibleTooltipNode = nsnull;
     }
@@ -1335,7 +1338,8 @@ ChromeTooltipListener :: CreateAutoHideTimer ( )
   
   mAutoHideTimer = do_CreateInstance("@mozilla.org/timer;1");
   if ( mAutoHideTimer )
-    mAutoHideTimer->Init(sAutoHideCallback, this, kTooltipAutoHideTime, PR_TRUE);
+    mAutoHideTimer->InitWithFuncCallback(sAutoHideCallback, this, kTooltipAutoHideTime, 
+                                         nsITimer::TYPE_ONE_SHOT);
 
 } // CreateAutoHideTimer
 
@@ -1383,7 +1387,7 @@ ChromeContextMenuListener :: ChromeContextMenuListener ( nsWebBrowser* inBrowser
     mWebBrowser(inBrowser),
     mWebBrowserChrome(inChrome)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 } // ctor
 
 
@@ -1450,13 +1454,12 @@ ChromeContextMenuListener :: AddChromeListeners ( )
   // Register the appropriate events for context menus, but only if
   // the embedding chrome cares.
   nsresult rv = NS_OK;
+
+  nsCOMPtr<nsIContextMenuListener2> contextListener2 ( do_QueryInterface(mWebBrowserChrome) );
   nsCOMPtr<nsIContextMenuListener> contextListener ( do_QueryInterface(mWebBrowserChrome) );
-  if ( contextListener && !mContextMenuListenerInstalled ) {
+  if ( (contextListener || contextListener2) && !mContextMenuListenerInstalled )
     rv = AddContextMenuListener();
-    if ( NS_FAILED(rv) )
-      return rv;
-  }
-    
+
   return rv;
   
 } // AddChromeListeners
@@ -1506,11 +1509,23 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
 
   // Stop the context menu event going to other windows (bug 78396)
   aMouseEvent->PreventDefault();
+  
+  // If the listener is a nsIContextMenuListener2, create the info object
+  nsCOMPtr<nsIContextMenuListener2> menuListener2(do_QueryInterface(mWebBrowserChrome));
+  nsContextMenuInfo *menuInfoImpl = nsnull;
+  nsCOMPtr<nsIContextMenuInfo> menuInfo;
+  if (menuListener2) {
+    menuInfoImpl = new nsContextMenuInfo;
+    if (!menuInfoImpl)
+      return NS_ERROR_OUT_OF_MEMORY;
+    menuInfo = menuInfoImpl; 
+  }
 
   // Find the first node to be an element starting with this node and
   // working up through its parents.
 
   PRUint32 flags = nsIContextMenuListener::CONTEXT_NONE;
+  PRUint32 flags2 = nsIContextMenuListener2::CONTEXT_NONE;
   nsCOMPtr<nsIDOMHTMLElement> element;
   do {
     // XXX test for selected text
@@ -1524,6 +1539,7 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
       if (tag.Equals(NS_LITERAL_STRING("img"), nsCaseInsensitiveStringComparator()))
       {
         flags |= nsIContextMenuListener::CONTEXT_IMAGE;
+        flags2 |= nsIContextMenuListener2::CONTEXT_IMAGE;
         targetDOMnode = node;
         // if we see an image, keep searching for a possible anchor
       }
@@ -1531,21 +1547,42 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
       {
         // INPUT element - button, combo, checkbox, text etc.
         flags |= nsIContextMenuListener::CONTEXT_INPUT;
+        flags2 |= nsIContextMenuListener2::CONTEXT_INPUT;
         targetDOMnode = node;
+        
+        // See if the input type is an image
+        if (menuListener2) {
+          nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(node));
+          if (inputElement) {
+            nsAutoString inputElemType;
+            inputElement->GetType(inputElemType);
+            if (inputElemType.Equals(NS_LITERAL_STRING("image"), nsCaseInsensitiveStringComparator()))
+              flags2 |= nsIContextMenuListener2::CONTEXT_IMAGE;
+          }
+        }
         break; // exit do-while
       }
       else if (tag.Equals(NS_LITERAL_STRING("textarea"), nsCaseInsensitiveStringComparator()))
       {
         // text area
         flags |= nsIContextMenuListener::CONTEXT_TEXT;
+        flags2 |= nsIContextMenuListener2::CONTEXT_TEXT;
         targetDOMnode = node;
         break; // exit do-while
       }
       else if (tag.Equals(NS_LITERAL_STRING("html"), nsCaseInsensitiveStringComparator()))
       {
+        // first check if this is a background image that the user was trying to click on
+        // and if the listener is ready for that (only nsIContextMenuListener2 and up)
+        if (menuInfoImpl && menuInfoImpl->HasBackgroundImage(node)) {
+          flags2 |= nsIContextMenuListener2::CONTEXT_BACKGROUND_IMAGE;
+          targetDOMnode = node;
+        }
+
+        if (!flags && !flags2) { 
         // only care about this if no other context was found.
-        if (!flags) {
             flags |= nsIContextMenuListener::CONTEXT_DOCUMENT;
+            flags2 |= nsIContextMenuListener2::CONTEXT_DOCUMENT;
             targetDOMnode = node;
         }
         break; // exit do-while
@@ -1567,8 +1604,11 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
         if (hrefNode)
         {
           flags |= nsIContextMenuListener::CONTEXT_LINK;
+          flags2 |= nsIContextMenuListener2::CONTEXT_LINK;
           if (!targetDOMnode)
             targetDOMnode = node;
+          if (menuInfoImpl)
+            menuInfoImpl->SetAssociatedLink(node);
           break; // exit do-while
         }
       }
@@ -1602,10 +1642,19 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
   NS_ENSURE_SUCCESS(res, res);
 
   // Tell the listener all about the event
-  nsCOMPtr<nsIContextMenuListener> menuListener(do_QueryInterface(mWebBrowserChrome));
-  if ( menuListener )
-    menuListener->OnShowContextMenu(flags, aMouseEvent, targetDOMnode);
+  if ( menuListener2 ) {
+    menuInfoImpl->SetMouseEvent(aMouseEvent);
+    menuInfoImpl->SetDOMNode(targetDOMnode);
+    menuListener2->OnShowContextMenu(flags2, menuInfo);
+  }
+  else {
+    nsCOMPtr<nsIContextMenuListener> menuListener(do_QueryInterface(mWebBrowserChrome));
+    if ( menuListener )
+      menuListener->OnShowContextMenu(flags, aMouseEvent, targetDOMnode);
+  }
 
   return NS_OK;
 
 } // MouseDown
+
+
