@@ -66,6 +66,10 @@
 // PSM2 includes
 #include "nsISecureBrowserUI.h"
 
+#ifdef MOZ_PHOENIX
+#include "nsToolkitCompsCID.h"
+#endif
+
 static NS_DEFINE_CID(kWebShellCID, NS_WEB_SHELL_CID);
 static NS_DEFINE_CID(kChildCID, NS_CHILD_CID);
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
@@ -89,9 +93,9 @@ nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nsnull),
    mParent(nsnull),
    mListenerArray(nsnull)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
     mInitInfo = new nsWebBrowserInitInfo();
-    mWWatch = do_GetService("@mozilla.org/embedcomp/window-watcher;1");
+    mWWatch = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
     NS_ASSERTION(mWWatch, "failed to get WindowWatcher");
 }
 
@@ -1035,6 +1039,7 @@ NS_IMETHODIMP nsWebBrowser::Create()
       nsWidgetInitData  widgetInit;
 
       widgetInit.clipChildren = PR_TRUE;
+      widgetInit.mContentType = mContentType;
       widgetInit.mWindowType = eWindowType_child;
       nsRect bounds(mInitInfo->x, mInitInfo->y, mInitInfo->cx, mInitInfo->cy);
       
