@@ -67,14 +67,19 @@
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsISupportsArray.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIURI.h"
 #include "nsIWebBrowser.h"
 #include "nsIWebBrowserChrome.h"
 #include "nsIWebNavigation.h"
 #include "nsIWindowCreator.h"
+#include "nsIWindowCreator2.h"
 #include "nsIXPConnect.h"
 #include "nsPIDOMWindow.h"
+#include "nsIMarkupDocumentViewer.h"
+#include "nsIContentViewer.h"
+#include "nsIDocumentViewer.h"
 
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
@@ -193,7 +198,7 @@ nsWatcherWindowEnumerator::nsWatcherWindowEnumerator(nsWindowWatcher *inWatcher)
   : mWindowWatcher(inWatcher),
     mCurrentPosition(inWatcher->mOldestWindow)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mWindowWatcher->AddEnumerator(this);
   mWindowWatcher->AddRef();
 }
@@ -413,7 +418,7 @@ nsWindowWatcher::nsWindowWatcher() :
         mActiveWindow(0),
         mListLock(0)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsWindowWatcher::~nsWindowWatcher()
@@ -504,6 +509,7 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
   if (aFeatures) {
     features.Assign(aFeatures);
     featuresSpecified = PR_TRUE;
+    features.StripWhitespace();
   }
 
   chromeFlags = CalculateChromeFlags(features.get(), featuresSpecified, aDialog);
@@ -560,10 +566,46 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
     }
 
     NS_ASSERTION(mWindowCreator, "attempted to open a new window with no WindowCreator");
+    rv = NS_ERROR_FAILURE;
     if (mWindowCreator) {
       nsCOMPtr<nsIWebBrowserChrome> newChrome;
-      mWindowCreator->CreateChromeWindow(parentChrome, chromeFlags,
-                        getter_AddRefs(newChrome));
+
+      /* If the window creator is an nsIWindowCreator2, we can give it
+         some hints. The only hint at this time is whether the opening window
+         is in a situation that's likely to mean this is an unrequested
+         popup window we're creating. However we're not completely honest:
+         we clear that indicator if the opener is chrome, so that the
+         downstream consumer can treat the indicator to mean simply
+         that the new window is subject to popup control. */
+      nsCOMPtr<nsIWindowCreator2> windowCreator2(do_QueryInterface(mWindowCreator));
+      if (windowCreator2) {
+        PRUint32 contextFlags = 0;
+        PRBool popupConditions = PR_FALSE;
+
+        // is the parent under popup conditions?
+        nsCOMPtr<nsPIDOMWindow> piWindow(do_QueryInterface(aParent));
+        if (piWindow)
+          piWindow->IsLoadingOrRunningTimeout(&popupConditions);
+
+        // chrome is always allowed, so clear the flag if the opener is chrome
+        if (popupConditions) {
+          PRBool isChrome = PR_FALSE;
+          nsCOMPtr<nsIScriptSecurityManager>
+            sm(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
+          if (sm)
+            sm->SubjectPrincipalIsSystem(&isChrome);
+          popupConditions = !isChrome;
+        }
+
+        if (popupConditions)
+          contextFlags |= nsIWindowCreator2::PARENT_IS_LOADING_OR_RUNNING_TIMEOUT;
+
+        rv = windowCreator2->CreateChromeWindow2(parentChrome, chromeFlags,
+                               contextFlags, getter_AddRefs(newChrome));
+      }
+      else
+        rv = mWindowCreator->CreateChromeWindow(parentChrome, chromeFlags,
+                               getter_AddRefs(newChrome));
       if (newChrome) {
         /* It might be a chrome nsXULWindow, in which case it won't have
             an nsIDOMWindow (primary content shell). But in that case, it'll
@@ -573,13 +615,15 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
           GetWindowTreeItem(newWindow, getter_AddRefs(newDocShellItem));
         if (!newDocShellItem)
           newDocShellItem = do_GetInterface(newChrome);
+        if (!newDocShellItem)
+          rv = NS_ERROR_FAILURE;
       }
     }
   }
 
   // better have a window to use by this point
   if (!newDocShellItem)
-    return NS_ERROR_FAILURE;
+    return rv;
 
   rv = ReadyOpenedDocShellItem(newDocShellItem, aParent, _retval);
   if (NS_FAILED(rv))
@@ -616,6 +660,36 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
 
   nsCOMPtr<nsIDocShell> newDocShell(do_QueryInterface(newDocShellItem));
 
+  // When a new window is opened through JavaScript, we want it to use the
+  // charset of its opener as a fallback in the event the document being loaded 
+  // does not specify a charset. Failing to set this charset is not fatal, so we 
+  // want to continue in the face of errors.
+  nsCOMPtr<nsIScriptGlobalObject> parentSGO(do_QueryInterface(aParent));
+  if (parentSGO) {
+    nsCOMPtr<nsIDocShell> parentDocshell;
+    parentSGO->GetDocShell(getter_AddRefs(parentDocshell));
+    // parentDocshell may be null if the parent got closed in the meantime
+    if (parentDocshell) {
+      nsCOMPtr<nsIContentViewer> parentContentViewer;
+      parentDocshell->GetContentViewer(getter_AddRefs(parentContentViewer));
+      nsCOMPtr<nsIDocumentViewer> parentDocViewer(do_QueryInterface(parentContentViewer));
+      if (parentDocViewer) {
+        nsCOMPtr<nsIDocument> doc;
+        parentDocViewer->GetDocument(*getter_AddRefs(doc));
+        
+        nsCOMPtr<nsIContentViewer> newContentViewer;
+        newDocShell->GetContentViewer(getter_AddRefs(newContentViewer));
+        nsCOMPtr<nsIMarkupDocumentViewer> newMarkupDocViewer(do_QueryInterface(newContentViewer));
+        if (doc && newMarkupDocViewer) {
+          nsXPIDLString charset;
+          rv = doc->GetDocumentCharacterSet(charset);
+          if (NS_SUCCEEDED(rv))
+            newMarkupDocViewer->SetDefaultCharacterSet(charset);
+        }
+      }
+    }
+  }
+
   if (uriToLoad) { // get the script principal and pass it to docshell
 
     // get the security manager
@@ -1258,10 +1332,13 @@ nsWindowWatcher::WinHasOption(const char *aOptions, const char *aName,
   char *comma, *equal;
   PRInt32 found = 0;
 
-  while (PR_TRUE) {
-    while (nsCRT::IsAsciiSpace(*aOptions))
-      ++aOptions;
+#ifdef DEBUG
+    nsCAutoString options(aOptions);
+    NS_ASSERTION(options.FindCharInSet(" \n\r\t") == kNotFound, 
+                  "There should be no whitespace in this string!");
+#endif
 
+  while (PR_TRUE) {
     comma = PL_strchr(aOptions, ',');
     if (comma)
       *comma = '\0';
@@ -1682,33 +1759,36 @@ nsWindowWatcher::AddSupportsTojsvals(nsISupports *aArg,
   argPrimitive->GetType(&type);
 
   switch(type) {
-    case nsISupportsPrimitive::TYPE_STRING : {
-      nsCOMPtr<nsISupportsString> p(do_QueryInterface(argPrimitive));
+    case nsISupportsPrimitive::TYPE_CSTRING : {
+      nsCOMPtr<nsISupportsCString> p(do_QueryInterface(argPrimitive));
       NS_ENSURE_TRUE(p, NS_ERROR_UNEXPECTED);
 
-      char *data;
+      nsCAutoString data;
 
-      p->GetData(&data);
+      p->GetData(data);
 
-      JSString *str = ::JS_NewString(cx, data, strlen(data));
+      
+      JSString *str = ::JS_NewStringCopyN(cx, data.get(), data.Length());
       NS_ENSURE_TRUE(str, NS_ERROR_OUT_OF_MEMORY);
 
       *aArgv = STRING_TO_JSVAL(str);
 
       break;
     }
-    case nsISupportsPrimitive::TYPE_WSTRING : {
-      nsCOMPtr<nsISupportsWString> p(do_QueryInterface(argPrimitive));
+    case nsISupportsPrimitive::TYPE_STRING : {
+      nsCOMPtr<nsISupportsString> p(do_QueryInterface(argPrimitive));
       NS_ENSURE_TRUE(p, NS_ERROR_UNEXPECTED);
 
-      PRUnichar *data;
+      nsAutoString data;
 
-      p->GetData(&data);
+      p->GetData(data);
 
       // cast is probably safe since wchar_t and jschar are expected
       // to be equivalent; both unsigned 16-bit entities
-      JSString *str = ::JS_NewUCString(cx, NS_REINTERPRET_CAST(jschar *, data),
-                          nsCRT::strlen(data));
+      JSString *str =
+        ::JS_NewUCStringCopyN(cx,
+                              NS_REINTERPRET_CAST(const jschar *,data.get()),
+                              data.Length());
       NS_ENSURE_TRUE(str, NS_ERROR_OUT_OF_MEMORY);
 
       *aArgv = STRING_TO_JSVAL(str);
