@@ -47,12 +47,19 @@
 #include "nsIFileSpec.h"
 #include "nsIPrompt.h"
 #include "nsIWindowWatcher.h"
+#include "nsIDOMWindow.h"
+#include "nsIDOMWindowInternal.h"
+#include "nsIDialogParamBlock.h"
 #include "nsVoidArray.h"
 #include "prmem.h"
 #include "nsAppDirectoryServiceDefs.h"
-#include "nsIIOService.h"
+#include "nsIURI.h"
 #include "nsNetCID.h"
 #include "nsTextFormatter.h"
+#include "nsIObserverService.h"
+#include "nsComObsolete.h"
+
+#include "nsICookieAcceptDialog.h"
 
 static const char *kCookiesPermFileName = "cookperm.txt";
 
@@ -70,43 +77,97 @@ PRIVATE PRBool permission_changed = PR_FALSE;
 
 PRIVATE PRBool cookie_rememberChecked;
 PRIVATE PRBool image_rememberChecked;
+PRIVATE PRBool window_rememberChecked;
 
 PRIVATE nsVoidArray * permission_list=0;
 
 PRBool
-permission_CheckConfirmYN(nsIPrompt *aPrompter, PRUnichar * szMessage, PRUnichar * szCheckMessage, PRBool* checkValue) {
+permission_CheckConfirmYN(nsIPrompt *aPrompter, PRUnichar * szMessage, PRUnichar * szCheckMessage, cookie_CookieStruct *cookie_s, PRBool* checkValue) {
 
   nsresult res;
-  nsCOMPtr<nsIPrompt> dialog;
-
-  if (aPrompter)
-    dialog = aPrompter;
-  else {
-    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
-    if (wwatch)
-      wwatch->GetNewPrompter(0, getter_AddRefs(dialog));
-  }
-  if (!dialog) {
-    *checkValue = 0;
-    return PR_FALSE;
-  }
-
   PRInt32 buttonPressed = 1; /* in case user exits dialog by clickin X */
   PRUnichar * confirm_string = CKutil_Localize(NS_LITERAL_STRING("Confirm").get());
 
-  res = dialog->ConfirmEx(confirm_string, szMessage,
-                          (nsIPrompt::BUTTON_TITLE_YES * nsIPrompt::BUTTON_POS_0) +
-                          (nsIPrompt::BUTTON_TITLE_NO * nsIPrompt::BUTTON_POS_1),
-                          nsnull, nsnull, nsnull, szCheckMessage, checkValue, &buttonPressed);
+  if (cookie_s) {
+    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+    if (!wwatch) {
+      *checkValue = 0;
+      return PR_FALSE;
+    }
 
-  if (NS_FAILED(res)) {
-    *checkValue = 0;
+    nsCOMPtr<nsIDOMWindowInternal> activeParent;
+    nsCOMPtr<nsIDOMWindow> active;
+    wwatch->GetActiveWindow(getter_AddRefs(active));
+
+    nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+    if (!block) {
+      *checkValue = 0;
+      buttonPressed = 1;
+      return PR_TRUE;
+    }
+    block->SetString(nsICookieAcceptDialog::MESSAGETEXT, szMessage);
+    block->SetInt(nsICookieAcceptDialog::REMEMBER_DECISION, *checkValue);
+
+    NS_ConvertASCIItoUCS2 cookieName(cookie_s->name);
+    NS_ConvertASCIItoUCS2 cookieValue(cookie_s->cookie);
+    NS_ConvertASCIItoUCS2 cookieHost(cookie_s->host);
+    NS_ConvertASCIItoUCS2 cookiePath(cookie_s->path);
+    block->SetString(nsICookieAcceptDialog::COOKIE_NAME, cookieName.get());
+    block->SetString(nsICookieAcceptDialog::COOKIE_VALUE, cookieValue.get());
+    block->SetString(nsICookieAcceptDialog::COOKIE_HOST, cookieHost.get());
+    block->SetString(nsICookieAcceptDialog::COOKIE_PATH, cookiePath.get());
+    block->SetInt(nsICookieAcceptDialog::COOKIE_IS_SECURE, cookie_s->isSecure);
+    block->SetInt(nsICookieAcceptDialog::COOKIE_EXPIRES, cookie_s->expires);
+    block->SetInt(nsICookieAcceptDialog::COOKIE_IS_DOMAIN, cookie_s->isDomain);
+
+    nsCOMPtr<nsIDOMWindow> dialogwin; 
+    res = wwatch->OpenWindow(active, "chrome://cookie/content/cookieAcceptDialog.xul", "_blank",
+                             "centerscreen,chrome,modal,titlebar", block,
+                             getter_AddRefs(dialogwin));
+
+    if (NS_FAILED(res)) {
+      *checkValue = 0;
+      buttonPressed = 1;
+    }
+    else {
+      /* get back output parameters */
+      PRInt32 acceptCookie;
+      block->GetInt(nsICookieAcceptDialog::ACCEPT_COOKIE, &acceptCookie);
+      buttonPressed = acceptCookie ? 0 : 1;
+      block->GetInt(nsICookieAcceptDialog::REMEMBER_DECISION, checkValue);
+    }
   }
+  else {
+    nsCOMPtr<nsIPrompt> dialog;
+
+    if (aPrompter)
+      dialog = aPrompter;
+    else {
+      nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+      if (wwatch)
+        wwatch->GetNewPrompter(0, getter_AddRefs(dialog));
+    }
+    if (!dialog) {
+      *checkValue = 0;
+      return PR_FALSE;
+    }
+
+    res = dialog->ConfirmEx(confirm_string, szMessage,
+                            (nsIPrompt::BUTTON_TITLE_YES * nsIPrompt::BUTTON_POS_0) +
+                            (nsIPrompt::BUTTON_TITLE_NO * nsIPrompt::BUTTON_POS_1),
+                            nsnull, nsnull, nsnull, szCheckMessage, checkValue, &buttonPressed);
+
+    if (NS_FAILED(res)) {
+      *checkValue = 0;
+    }
+  }
+
   if (*checkValue!=0 && *checkValue!=1) {
     NS_ASSERTION(PR_FALSE, "Bad result from checkbox");
     *checkValue = 0; /* this should never happen but it is happening!!! */
   }
   Recycle(confirm_string);
+
   return (buttonPressed == 0);
 }
 
@@ -160,13 +221,13 @@ permission_CheckFromList(const char * hostname, PRBool &permission, PRInt32 type
 
 PRBool
 permission_GetRememberChecked(PRInt32 type) {
-  if (type == COOKIEPERMISSION) {
+  if (type == COOKIEPERMISSION)
     return cookie_rememberChecked;
-  } else if (type == IMAGEPERMISSION) {
+  if (type == IMAGEPERMISSION)
     return image_rememberChecked;
-  } else {
-    return PR_FALSE;
-  }
+  if (type == WINDOWPERMISSION)
+    return window_rememberChecked;
+  return PR_FALSE;
 }
 
 void
@@ -175,6 +236,8 @@ permission_SetRememberChecked(PRInt32 type, PRBool value) {
     cookie_rememberChecked = value;
   } else if (type == IMAGEPERMISSION) {
     image_rememberChecked = value;
+  } else if (type == WINDOWPERMISSION) {
+    window_rememberChecked = value;
   }
 }
 
@@ -184,6 +247,7 @@ Permission_Check(
      const char * hostname,
      PRInt32 type,
      PRBool warningPref,
+     cookie_CookieStruct *cookie_s,
      const char * message_string,
      int count_for_message)
 {
@@ -206,39 +270,35 @@ Permission_Check(
                             hostname ? hostname : "", count_for_message);
   PRBool rememberChecked = permission_GetRememberChecked(type);
   PRUnichar * remember_string = CKutil_Localize(NS_LITERAL_STRING("RememberThisDecision").get());
-  permission = permission_CheckConfirmYN(aPrompter, message, remember_string, &rememberChecked);
+  permission = permission_CheckConfirmYN(aPrompter, message, remember_string, cookie_s, &rememberChecked);
   nsTextFormatter::smprintf_free(message);
   nsMemory::Free(message_fmt);
 
   /* see if we need to remember this decision */
   if (rememberChecked) {
-    char * hostname2 = NULL;
     /* ignore leading periods in host name */
     const char * hostnameAfterDot = hostname;
     while (hostnameAfterDot && (*hostnameAfterDot == '.')) {
       hostnameAfterDot++;
     }
-    CKutil_StrAllocCopy(hostname2, hostnameAfterDot);
-    Permission_AddHost(hostname2, permission, type, PR_TRUE);
+    Permission_AddHost(nsDependentCString(hostnameAfterDot), permission, type, PR_TRUE);
   }
   if (rememberChecked != permission_GetRememberChecked(type)) {
     permission_SetRememberChecked(type, rememberChecked);
     permission_changed = PR_TRUE;
-    Permission_Save();
+    Permission_Save(PR_TRUE);
   }
   Recycle(remember_string);
   return permission;
 }
 
 PUBLIC nsresult
-Permission_AddHost(char * host, PRBool permission, PRInt32 type, PRBool save) {
+Permission_AddHost(const nsAFlatCString & host, PRBool permission, PRInt32 type, PRBool save) {
   /* create permission list if it does not yet exist */
   if(!permission_list) {
     permission_list = new nsVoidArray();
-    if(!permission_list) {
-      Recycle(host);
+    if(!permission_list)
       return NS_ERROR_OUT_OF_MEMORY;
-    }
   }
 
   /* find existing entry for host */
@@ -249,14 +309,13 @@ Permission_AddHost(char * host, PRBool permission, PRInt32 type, PRBool save) {
   for (i = 0; i < hostCount; ++i) {
     hostStruct = NS_STATIC_CAST(permission_HostStruct*, permission_list->ElementAt(i));
     if (hostStruct) {
-      if (PL_strcasecmp(host,hostStruct->host)==0) {
+      if (PL_strcasecmp(host.get(),hostStruct->host)==0) {
 
         /* host found in list */
-        Recycle(host);
         HostFound = PR_TRUE;
         break;
 #ifdef alphabetize
-      } else if (PL_strcasecmp(host, hostStruct->host) < 0) {
+      } else if (PL_strcasecmp(host.get(), hostStruct->host) < 0) {
 
         /* need to insert new entry here */
         break;
@@ -269,15 +328,12 @@ Permission_AddHost(char * host, PRBool permission, PRInt32 type, PRBool save) {
 
     /* create a host structure for the host */
     hostStruct = PR_NEW(permission_HostStruct);
-    if (!hostStruct) {
-      Recycle(host);
+    if (!hostStruct)
       return NS_ERROR_FAILURE;
-    }
-    hostStruct->host = host;
+    hostStruct->host = ToNewCString(host);
     hostStruct->permissionList = new nsVoidArray();
     if(!hostStruct->permissionList) {
       PR_Free(hostStruct);
-      Recycle(host);
       return NS_ERROR_FAILURE;
     }
 
@@ -316,13 +372,13 @@ Permission_AddHost(char * host, PRBool permission, PRInt32 type, PRBool save) {
   /* write the changes out to a file */
   if (save) {
     permission_changed = PR_TRUE;
-    Permission_Save();
+    Permission_Save(PR_TRUE);
   }
   return NS_OK;
 }
 
 PRIVATE void
-permission_Unblock(char * host, PRInt32 type) {
+permission_Unblock(const char * host, PRInt32 type) {
 
   /* nothing to do if permission list does not exist */
   if(!permission_list) {
@@ -354,7 +410,7 @@ permission_Unblock(char * host, PRInt32 type) {
               PR_Free(hostStruct);
             }
             permission_changed = PR_TRUE;
-            Permission_Save();
+            Permission_Save(PR_TRUE);
             return;
           }
           break;
@@ -368,7 +424,7 @@ permission_Unblock(char * host, PRInt32 type) {
 
 /* saves the permissions to disk */
 PUBLIC void
-Permission_Save() {
+Permission_Save(PRBool notify) {
   permission_HostStruct * hostStruct;
   permission_TypeStruct * typeStruct;
 
@@ -442,6 +498,14 @@ Permission_Save() {
   permission_changed = PR_FALSE;
   strm.flush();
   strm.close();
+
+  /* Notify cookie manager dialog to update its display */
+  if (notify) {
+    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+    if (os) {
+      os->NotifyObservers(nsnull, "cookieChanged", NS_LITERAL_STRING("permissions").get());
+    }
+  }
 }
 
 /* reads the permissions from disk */
@@ -456,7 +520,7 @@ PERMISSION_Read() {
   if(!permission_list) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
-  nsAutoString buffer;
+  nsCAutoString buffer;
   nsFileSpec dirSpec;
   nsresult rv = CKutil_ProfileDirectory(dirSpec);
   if (NS_FAILED(rv)) {
@@ -475,9 +539,13 @@ PERMISSION_Read() {
    * host \t number permission \t number permission ... \n
    * if this format isn't respected we move onto the next line in the file.
    */
-  while(CKutil_GetLine(strm,buffer) != -1) {
+
+#define BUFSIZE 4096
+  char readbuffer[BUFSIZE];
+  PRInt32 next = BUFSIZE, count = BUFSIZE;
+  while(CKutil_GetLine(strm, readbuffer, BUFSIZE, next, count, buffer) != -1) {
     if ( !buffer.IsEmpty() ) {
-      PRUnichar firstChar = buffer.CharAt(0);
+      char firstChar = buffer.CharAt(0);
       if (firstChar == '#' || firstChar == nsCRT::CR ||
           firstChar == nsCRT::LF || firstChar == 0) {
         continue;
@@ -497,10 +565,9 @@ PERMISSION_Read() {
       hostIndex++;
     }
 
-    nsAutoString host;
-    buffer.Mid(host, hostIndex, permissionIndex-hostIndex-1);
+    nsDependentCSubstring host(buffer, hostIndex, permissionIndex-hostIndex-1);
 
-    nsAutoString permissionString;
+    nsCAutoString permissionString;
     for (;;) {
       if (nextPermissionIndex == buffer.Length()+1) {
         break;
@@ -508,6 +575,8 @@ PERMISSION_Read() {
       if ((nextPermissionIndex=buffer.FindChar('\t', permissionIndex)+1) == 0) {
         nextPermissionIndex = buffer.Length()+1;
       }
+      // XXX would like to use nsDependentCSubstring to avoid this allocation,
+      // but it unfortunately doesn't provide the CharAt method.
       buffer.Mid(permissionString, permissionIndex, nextPermissionIndex-permissionIndex-1);
       permissionIndex = nextPermissionIndex;
 
@@ -531,13 +600,13 @@ PERMISSION_Read() {
        * a host value of "@@@@" is a special code designating the
        * state of the nag-box's checkmark
        */
-      if (host.Equals(NS_LITERAL_STRING("@@@@"))) {
+      if (host.Equals(NS_LITERAL_CSTRING("@@@@"))) {
         if (!permissionString.IsEmpty()) {
           permission_SetRememberChecked(type, permission);
         }
       } else {
         if (!permissionString.IsEmpty()) {
-          rv = Permission_AddHost(ToNewCString(host), permission, type, PR_FALSE);
+          rv = Permission_AddHost(PromiseFlatCString(host), permission, type, PR_FALSE);
           if (NS_FAILED(rv)) {
             strm.close();
             return rv;
@@ -631,7 +700,7 @@ permission_remove (PRInt32 hostNumber, PRInt32 type) {
 }
 
 PUBLIC void
-PERMISSION_Remove(const char* host, PRInt32 type) {
+PERMISSION_Remove(const nsACString & host, PRInt32 type) {
 
   /* get to the indicated host in the list */
   if (permission_list) {
@@ -642,7 +711,7 @@ PERMISSION_Remove(const char* host, PRInt32 type) {
       hostStruct =
         NS_STATIC_CAST(permission_HostStruct*, permission_list->ElementAt(hostCount));
       NS_ASSERTION(hostStruct, "corrupt permission list");
-      if ((PL_strcmp(hostStruct->host, host) == 0)) {
+      if (host.Equals(hostStruct->host)) {
 
         /* get to the indicated permission in the list */
         PRInt32 typeCount = hostStruct->permissionList->Count();
@@ -656,7 +725,7 @@ PERMISSION_Remove(const char* host, PRInt32 type) {
           if (typeStruct->type == type) {
             permission_remove(hostCount, typeCount);
             permission_changed = PR_TRUE;
-            Permission_Save();
+            Permission_Save(PR_FALSE);
             break;
           }
         }
@@ -700,17 +769,14 @@ PERMISSION_DeletePersistentUserData(void)
 }
 
 PUBLIC void
-PERMISSION_Add(const char * objectURL, PRBool permission, PRInt32 type,
-               nsIIOService* ioService) {
-  if (!objectURL) {
+PERMISSION_Add(nsIURI * objectURI, PRBool permission, PRInt32 type) {
+  if (!objectURI) {
     return;
   }
-  nsresult rv = NS_OK;
-  nsCAutoString host;
-  NS_ASSERTION(ioService, "IOService not available");
-  rv = ioService->ExtractUrlPart(nsDependentCString(objectURL),
-                                 nsIIOService::url_Host |
-                                 nsIIOService::url_Port, host);
+  nsCAutoString hostPort;
+  objectURI->GetHostPort(hostPort);
+  if (hostPort.IsEmpty())
+    return;
 
   /*
    * if permission is false, it will be added to the permission list
@@ -718,34 +784,32 @@ PERMISSION_Add(const char * objectURL, PRBool permission, PRInt32 type,
    *    true permission being added
    */
   if (permission) {
-    char * hostPtr = (char *)host.get();
+    const char * hostPtr = hostPort.get();
     while (PR_TRUE) {
       permission_Unblock(hostPtr, type);
       hostPtr = PL_strchr(hostPtr, '.');
       if (!hostPtr) {
         break;
       }
-      hostPtr++; /* get passed the period */
+      hostPtr++; /* get past the period */
     }
     return;
   }
-  Permission_AddHost(ToNewCString(host), permission, type, PR_TRUE);
+  Permission_AddHost(hostPort, permission, type, PR_TRUE);
 }
 
 PUBLIC void
-PERMISSION_TestForBlocking(const char * objectURL, PRBool* blocked, PRInt32 type,
-               nsIIOService* ioService) {
-  if (!objectURL) {
+PERMISSION_TestForBlocking(nsIURI * objectURI, PRBool* blocked, PRInt32 type) {
+  if (!objectURI) {
     return;
   }
   nsresult rv = NS_OK;
-  nsCAutoString host;
-  NS_ASSERTION(ioService, "IOService not available");
-  rv = ioService->ExtractUrlPart(nsDependentCString(objectURL),
-                                 nsIIOService::url_Host |
-                                 nsIIOService::url_Port, host);
+  nsCAutoString hostPort;
+  objectURI->GetHostPort(hostPort);
+  if (hostPort.IsEmpty())
+    return;
 
-  const char * hostPtr = host.get();
+  const char * hostPtr = hostPort.get();
   while (PR_TRUE) {
     PRBool permission;
     rv = permission_CheckFromList(hostPtr, permission, type);
