@@ -17,9 +17,6 @@
  * Copyright (C) 1999 New Dimenstions Consulting, Inc. All
  * Rights Reserved.
  *
- * Contributor(s): 
- *
- *
  * Contributor(s):
  *  Robert Ginda, rginda@ndcico.com, original author
  *
@@ -111,6 +108,14 @@ CIRCNetwork.prototype.TYPE = "IRCNetwork";
 CIRCNetwork.prototype.getURL =
 function net_geturl ()
 {
+    if (this.serverList.length == 1 &&
+        this.serverList[0].name == this.name &&
+        this.serverList[0].port != 6667)
+    {
+        return "irc://" + this.serverList[0].name + ":" +
+            this.serverList[0].port + "/";
+    }
+    
     return "irc://" + escape(this.name) + "/";
 }
 
@@ -442,7 +447,7 @@ function serv_messto (code, target, msg, ctcpCode)
 
     if (ctcpCode)
     {
-        pfx = "\01" + ctcpCode + " ";
+        pfx = "\01" + ctcpCode;
         sfx = "\01";
     }
 
@@ -464,8 +469,16 @@ function serv_messto (code, target, msg, ctcpCode)
             
         if ((lines[i] != "") || ctcpCode)
         {
+            var line = code + " " + target + " :" + pfx;
             this.sendsThisRound++;
-            var line = code + " " + target + " :" + pfx + lines[i] + sfx;
+            if (lines[i] != "")
+            {
+                if (ctcpCode)
+                    line += " ";
+                line += lines[i] + sfx;
+            }
+            else
+                line += sfx;
             //dd ("-*- irc sending '" +  line + "'");
             this.sendData (line + "\n");
         }
@@ -997,8 +1010,9 @@ function serv_mode (e)
 {
 
     e.destObject = this;
-    
-    if ((e.params[1][0] == "#") || (e.params[1][0] == "&"))
+    /* modes are not allowed in +channels -> no need to test that here.. */
+    if ((e.params[1][0] == "#") || (e.params[1][0] == "&") ||
+        (e.params[1][0] == "!"))
     {
         e.channel = new CIRCChannel (this, e.params[1]);
         if ("user" in e && e.user)
@@ -1316,10 +1330,10 @@ CIRCServer.prototype.onJoin =
 function serv_join (e)
 {
 
-    e.channel = new CIRCChannel (this, e.meat);
+    e.channel = new CIRCChannel (this, (e.meat ? e.meat : e.params[1]));
     if (e.user == this.me)
-        e.server.sendData ("MODE " + e.channel.name + "\n" /* +
-                           "BANS " + e.channel.name + "\n" */);
+        e.server.sendData ("MODE " + e.channel.encodedName + "\n" /* +
+                           "BANS " + e.channel.encodedName + "\n" */);
     e.user = new CIRCChanUser (e.channel, e.user.nick);
     if (userIsMe(e.user))
         e.channel.active = true;
@@ -1354,7 +1368,7 @@ function serv_pong (e)
         return true;
     
     if (this.lastPingSent)
-        this.lag = roundTo ((new Date() - this.lastPingSent) / 1000, 1);
+        this.lag = roundTo ((new Date() - this.lastPingSent) / 1000, 2);
 
     delete this.lastPingSent;
     
@@ -1376,7 +1390,8 @@ function serv_notice (e)
         return true;
     }
         
-    if ((e.params[1][0] == "#") || (e.params[1][0] == "&"))
+    if ((e.params[1][0] == "#") || (e.params[1][0] == "&") ||
+        (e.params[1][0] == "+") || (e.params[1][0] == "!"))
     {
         e.channel = new CIRCChannel(this, e.params[1]);
         e.user = new CIRCChanUser (e.channel, e.user.nick);
@@ -1409,7 +1424,8 @@ function serv_privmsg (e)
     
     /* setting replyTo provides a standard place to find the target for     */
     /* replys associated with this event.                                   */
-    if ((e.params[1][0] == "#") || (e.params[1][0] == "&"))
+    if ((e.params[1][0] == "#") || (e.params[1][0] == "&") ||
+        (e.params[1][0] == "+") || (e.params[1][0] == "!"))
     {
         e.channel = new CIRCChannel(this, e.params[1]);
         e.user = new CIRCChanUser (e.channel, e.user.nick);
@@ -1490,6 +1506,12 @@ function serv_ctcp (e)
     e.CTCPData = ary[2] ? ary[2] : "";
 
     e.CTCPCode = ary[1].toLowerCase();
+    if (e.CTCPCode.search (/^reply/i) == 0)
+    {
+        dd ("dropping spoofed reply.");
+        return false;
+    }
+    
     e.type = "ctcp-" + e.CTCPCode;
     e.destMethod = "onCTCP" + ary[1][0].toUpperCase() +
         ary[1].substr (1, ary[1].length).toLowerCase();
@@ -1512,6 +1534,24 @@ function serv_ctcp (e)
     
 }
 
+CIRCServer.prototype.onCTCPClientinfo =
+function serv_ccinfo (e)
+{
+    var clientinfo = new Array();
+
+    for (var fname in this)
+    {
+        var ary = fname.match(/^onCTCP(.*)/);
+        if (ary && ary[1].search(/^Reply/) == -1)
+            clientinfo.push (ary[1].toUpperCase());
+    }
+
+    e.server.ctcpTo (e.user.nick, "CLIENTINFO", clientinfo.join(" "),
+                     "NOTICE");
+
+    return true;
+}
+
 CIRCServer.prototype.onCTCPAction =
 function serv_cact (e)
 {
@@ -1520,6 +1560,15 @@ function serv_cact (e)
 
 }
 
+CIRCServer.prototype.onCTCPTime = 
+function serv_cping (e)
+{
+
+    e.server.ctcpTo (e.user.nick, "TIME", new Date(), "NOTICE");
+
+    return true;
+    
+}
 
 CIRCServer.prototype.onCTCPVersion = 
 function serv_cver (e)
@@ -1617,13 +1666,20 @@ function serv_dccsend (e)
 function CIRCChannel (parent, name)
 {
 
-    name = name.toLowerCase();
+    var encodedName = fromUnicode(name + " ");
+    /* bug 114923 */
+    encodedName = encodedName.substr(0,encodedName.length -1);
+    var unicodeName = toUnicode(encodedName);
+    name = encodedName.toLowerCase();
     
     if (name in parent.channels)
         return parent.channels[name];
     
     this.parent = parent;
-    this.name = name;
+    this.name = name;               // used internally, lowercased
+    this.unicodeName = unicodeName; // converted to unicode for display
+    this.encodedName = encodedName; // encoded for communication with server
+
     this.users = new Object();
     this.bans = new Object();
     this.mode = new CIRCChanMode (this);
@@ -1643,9 +1699,9 @@ function chan_geturl ()
 {
     var target;
     if (this.name[0] == "#")
-        target = this.name.substr(1);
+        target = escape(this.encodedName.substr(1));
     else
-        target = escape(this.name);
+        target = escape(this.encodedName);
     return this.parent.parent.getURL() + target;
 }
 
@@ -1702,7 +1758,8 @@ function chan_topic (str)
     
     str = String(str).split("\n");
     for (var i in str)
-        this.parent.sendData ("TOPIC " + this.name + " :" + str[i] + "\n");
+        this.parent.sendData ("TOPIC " + this.encodedName + " :" + str[i] +
+                              "\n");
     
     return true;
 
@@ -1712,7 +1769,7 @@ CIRCChannel.prototype.say =
 function chan_say (msg)
 {
 
-    this.parent.sayTo (this.name, msg);
+    this.parent.sayTo (this.encodedName, msg);
     
 }
 
@@ -1720,7 +1777,7 @@ CIRCChannel.prototype.act =
 function chan_say (msg)
 {
 
-    this.parent.actTo (this.name, msg);
+    this.parent.actTo (this.encodedName, msg);
     
 }
 
@@ -1728,7 +1785,7 @@ CIRCChannel.prototype.notice =
 function chan_notice (msg)
 {
 
-    this.parent.noticeTo (this.name, msg);
+    this.parent.noticeTo (this.encodedName, msg);
     
 }
 
@@ -1752,7 +1809,7 @@ function chan_join (key)
     if (!key)
         key = "";
     
-    this.parent.sendData ("JOIN " + this.name + " " + key + "\n");
+    this.parent.sendData ("JOIN " + this.encodedName + " " + key + "\n");
     return true;
     
 }
@@ -1761,7 +1818,7 @@ CIRCChannel.prototype.part =
 function chan_part ()
 {
     
-    this.parent.sendData ("PART " + this.name + "\n");
+    this.parent.sendData ("PART " + this.encodedName + "\n");
     this.users = new Object();
     return true;
     
@@ -1776,7 +1833,7 @@ CIRCChannel.prototype.invite =
 function chan_inviteuser (nick)
 {
 
-    this.parent.sendData("INVITE " + nick + " " + this.name + "\n");
+    this.parent.sendData("INVITE " + nick + " " + this.encodedName + "\n");
     return true;
 
 }
@@ -1836,7 +1893,7 @@ function chanm_mode (modestr)
     if (!this.parent.users[this.parent.parent.me.nick].isOp)
         return false;
 
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modestr + "\n");
 
     return true;
@@ -1850,10 +1907,15 @@ function chanm_limit (n)
         return false;
 
     if ((typeof n == "undefined") || (n <= 0))
-        this.parent.parent.sendData ("MODE " + this.parent.name + " -l\n");
+    {
+        this.parent.parent.sendData ("MODE " + this.parent.encodedName +
+                                     " -l\n");
+    }
     else
-        this.parent.parent.sendData ("MODE " + this.parent.name + " +l " +
+    {
+        this.parent.parent.sendData ("MODE " + this.parent.encodedName + " +l " +
                                      Number(n) + "\n");
+    }
 
     return true;
     
@@ -1865,7 +1927,7 @@ function chanm_lock (k)
     if (!this.parent.users[this.parent.parent.me.nick].isOp)
         return false;
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " +k " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " +k " +
                                  k + "\n");
     return true;
     
@@ -1877,7 +1939,7 @@ function chan_unlock (k)
     if (!this.parent.users[this.parent.parent.me.nick].isOp)
         return false;
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " -k " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " -k " +
                                  k + "\n");
     return true;
     
@@ -1891,7 +1953,7 @@ function chan_moderate (f)
 
     var modifier = (f) ? "+" : "-";
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modifier + "m\n");
     return true;
     
@@ -1905,7 +1967,7 @@ function chan_pmessages (f)
 
     var modifier = (f) ? "-" : "+";
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modifier + "n\n");
     return true;
     
@@ -1919,7 +1981,7 @@ function chan_ptopic (f)
 
     var modifier = (f) ? "-" : "+";
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modifier + "t\n");
     return true;
     
@@ -1933,7 +1995,7 @@ function chan_invite (f)
 
     var modifier = (f) ? "+" : "-";
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modifier + "i\n");
     return true;
     
@@ -1947,7 +2009,7 @@ function chan_pvt (f)
 
     var modifier = (f) ? "+" : "-";
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modifier + "p\n");
     return true;
     
@@ -1961,7 +2023,7 @@ function chan_secret (f)
 
     var modifier = (f) ? "+" : "-";
     
-    this.parent.parent.sendData ("MODE " + this.parent.name + " " +
+    this.parent.parent.sendData ("MODE " + this.parent.encodedName + " " +
                                  modifier + "p\n");
     return true;
     
@@ -2154,7 +2216,7 @@ function cusr_kick (reason)
     if (!this.parent.users[me.nick].isOp)
         return false;
     
-    server.sendData("KICK " + this.parent.name + " " + this.nick + " :" +
+    server.sendData("KICK " + this.parent.encodedName + " " + this.nick + " :" +
                     reason + "\n");
 
     return true;
@@ -2195,8 +2257,8 @@ function cusr_kban (reason)
     reason = (typeof reason != "undefined") ? reason : this.nick;
     var modifier = " -o+b " + this.nick + " " + this.getHostMask() + " ";
     
-    server.sendData("MODE " + this.parent.name + modifier + "\n" +
-                    "KICK " + this.parent.name + " " + this.nick + " :" +
+    server.sendData("MODE " + this.parent.encodedName + modifier + "\n" +
+                    "KICK " + this.parent.encodedName + " " + this.nick + " :" +
                     reason + "\n");
 
     return true;
