@@ -39,35 +39,34 @@
 #include "XMLUtils.h"
 #include "XMLDOMUtils.h"
 #include "Tokenizer.h"
-#include "VariableBinding.h"
 #include "ExprResult.h"
 #include "Names.h"
 #include "XMLParser.h"
 #include "TxLog.h"
 #include "txAtoms.h"
+#include "txSingleNodeContext.h"
+#include "txVariableMap.h"
+#include "XSLTProcessor.h"
 
 /**
  * Creates a new ProcessorState for the given XSL document
- * and resultDocument
 **/
 ProcessorState::ProcessorState(Document* aSourceDocument,
-                               Document* aXslDocument,
-                               Document* aResultDocument)
+                               Document* aXslDocument)
     : mXslKeys(MB_TRUE),
       mDecimalFormats(MB_TRUE),
       mEvalContext(0),
+      mLocalVariables(0),
+      mGlobalVariableValues(MB_TRUE),
       mSourceDocument(aSourceDocument),
       xslDocument(aXslDocument),
-      resultDocument(aResultDocument)
+      mRTFDocument(0),
+      mOutputHandler(0),
+      mResultHandler(0),
+      mOutputHandlerFactory(0)
 {
     NS_ASSERTION(aSourceDocument, "missing source document");
     NS_ASSERTION(aXslDocument, "missing xslt document");
-    NS_ASSERTION(aResultDocument, "missing result document");
-
-    // add global variable set
-    NamedMap* globalVars = new NamedMap();
-    globalVars->setObjectDeletion(MB_TRUE);
-    variableSets.push(globalVars);
 
     /* turn object deletion on for some of the Maps (NamedMap) */
     mExprHashes[SelectAttr].setOwnership(Map::eOwnsItems);
@@ -93,10 +92,6 @@ ProcessorState::ProcessorState(Document* aSourceDocument,
 **/
 ProcessorState::~ProcessorState()
 {
-  while (! variableSets.empty()) {
-      delete (NamedMap*) variableSets.pop();
-  }
-
   // Delete all ImportFrames
   txListIterator iter(&mImportFrames);
   while (iter.hasNext())
@@ -109,6 +104,10 @@ ProcessorState::~ProcessorState()
   if (mSourceDocument)
       loadedDocuments.remove(mSourceDocument->getBaseURI());
 
+  // in module the outputhandler is refcounted
+#ifdef TX_EXE
+  delete mOutputHandler;
+#endif
 } //-- ~ProcessorState
 
 
@@ -428,7 +427,6 @@ Node* ProcessorState::findTemplate(Node* aNode,
         return 0;
 
     Node* matchTemplate = 0;
-    double currentPriority = Double::NEGATIVE_INFINITY;
     ImportFrame* endFrame = 0;
     txListIterator frameIter(&mImportFrames);
 
@@ -469,41 +467,33 @@ Node* ProcessorState::findTemplate(Node* aNode,
     }
 
 #ifdef PR_LOGGING
-    char *nodeBuf = 0, *modeBuf = 0;
     String mode;
     if (aMode.mLocalName) {
         TX_GET_ATOM_STRING(aMode.mLocalName, mode);
     }
     if (matchTemplate) {
-        char *matchBuf = 0, *uriBuf = 0;
+        String matchAttr;
+        // matchTemplate can be a document (see addLREStylesheet)
+        unsigned short nodeType = matchTemplate->getNodeType();
+        if (nodeType == Node::ELEMENT_NODE) {
+            ((Element*)matchTemplate)->getAttr(txXSLTAtoms::match,
+                                               kNameSpaceID_None,
+                                               matchAttr);
+        }
         PR_LOG(txLog::xslt, PR_LOG_DEBUG,
                ("MatchTemplate, Pattern %s, Mode %s, Stylesheet %s, " \
                 "Node %s\n",
-                (matchBuf = ((Element*)matchTemplate)->getAttribute(String("match")).toCharArray()),
-                (modeBuf = mode.toCharArray()),
-                (uriBuf = matchTemplate->getBaseURI().toCharArray()),
-                (nodeBuf = aNode->getNodeName().toCharArray())));
-#ifdef TX_EXE
-        delete [] matchBuf;
-        delete [] uriBuf;
-#else
-        nsMemory::Free(matchBuf);
-        nsMemory::Free(uriBuf);
-#endif
+                NS_LossyConvertUCS2toASCII(matchAttr).get(),
+                NS_LossyConvertUCS2toASCII(mode).get(),
+                NS_LossyConvertUCS2toASCII(matchTemplate->getBaseURI()).get(),
+                NS_LossyConvertUCS2toASCII(aNode->getNodeName()).get()));
     }
     else {
         PR_LOG(txLog::xslt, PR_LOG_DEBUG,
                ("No match, Node %s, Mode %s\n", 
-                (nodeBuf  = aNode->getNodeName().toCharArray()),
-                (modeBuf = mode.toCharArray())));
+                NS_LossyConvertUCS2toASCII(aNode->getNodeName()).get(),
+                NS_LossyConvertUCS2toASCII(mode).get()));
     }
-#ifdef TX_EXE
-        delete [] nodeBuf;
-        delete [] modeBuf;
-#else
-        nsMemory::Free(nodeBuf);
-        nsMemory::Free(modeBuf);
-#endif
 #endif
     return matchTemplate;
 }
@@ -555,7 +545,7 @@ Expr* ProcessorState::getExpr(Element* aElem, ExprAttr aAttr)
         return expr;
     }
     String attr;
-    MBool hasAttr;
+    MBool hasAttr = MB_FALSE;
     switch (aAttr) {
         case SelectAttr:
             hasAttr = aElem->getAttr(txXSLTAtoms::select, kNameSpaceID_None,
@@ -649,25 +639,64 @@ txOutputFormat* ProcessorState::getOutputFormat()
     return &mOutputFormat;
 }
 
-Document* ProcessorState::getResultDocument()
+Document* ProcessorState::getRTFDocument()
 {
-    return resultDocument;
+    return mRTFDocument;
 }
 
-Stack* ProcessorState::getVariableSetStack()
+void ProcessorState::setRTFDocument(Document* aDoc)
 {
-    return &variableSets;
+    mRTFDocument = aDoc;
+}
+
+Document* ProcessorState::getStylesheetDocument()
+{
+    NS_ASSERTION(xslDocument, "missing stylesheet document");
+    return xslDocument;
 }
 
 /*
- * Determines if the given XSL node allows Whitespace stripping
+ * Add a global variable
  */
-MBool ProcessorState::isXSLStripSpaceAllowed(Node* node) {
+nsresult ProcessorState::addGlobalVariable(const txExpandedName& aVarName,
+                                           Element* aVarElem,
+                                           ImportFrame* aImportFrame,
+                                           ExprResult* aDefaultValue)
+{
+    // If we don't know the value, it's a plain global var, add it
+    // to the import frame for late evaluation.
+    if (!aDefaultValue) {
+        return aImportFrame->mVariables.add(aVarName, aVarElem);
+    }
+    // Otherwise, add a GlobalVariableValue not owning the value.
+    GlobalVariableValue* var =
+        (GlobalVariableValue*)mGlobalVariableValues.get(aVarName);
+    if (var) {
+        // we set this parameter twice, we should set it to the same
+        // value;
+        NS_ENSURE_TRUE(var->mValue == aDefaultValue, NS_ERROR_UNEXPECTED);
+        return NS_OK;
+    }
+    var = new GlobalVariableValue(aDefaultValue);
+    NS_ENSURE_TRUE(var, NS_ERROR_OUT_OF_MEMORY);
 
-    if (!node)
-        return MB_FALSE;
-    return (MBool)(getXMLSpaceMode(node) != PRESERVE);
+    return mGlobalVariableValues.add(aVarName, var);
+}
 
+/*
+ * Returns map on top of the stack of local variable-bindings
+ */
+txVariableMap* ProcessorState::getLocalVariables()
+{
+    return mLocalVariables;
+}
+
+/*
+ * Sets top map of the local variable-bindings stack
+ */
+void ProcessorState::setLocalVariables(txVariableMap* aMap)
+{
+    mLocalVariables = aMap;
 }
 
 void ProcessorState::processAttrValueTemplate(const String& aAttValue,
@@ -929,31 +958,81 @@ txDecimalFormat* ProcessorState::getDecimalFormat(const txExpandedName& aName)
 nsresult ProcessorState::getVariable(PRInt32 aNamespace, txAtom* aLName,
                                      ExprResult*& aResult)
 {
-    String name;
-    // XXX TODO, bug 117658
-    TX_GET_ATOM_STRING(aLName, name);
-    txStackIterator iter(&variableSets);
-    ExprResult* exprResult = 0;
-    while (iter.hasNext()) {
-        NamedMap* map = (NamedMap*)iter.next();
-        if (map->get(name)) {
-            exprResult = ((VariableBinding*)map->get(name))->getValue();
-            break;
+    nsresult rv;
+    aResult = 0;
+    ExprResult* exprResult;
+    txExpandedName varName(aNamespace, aLName);
+    
+    // Check local variables
+    if (mLocalVariables) {
+        exprResult = mLocalVariables->getVariable(varName);
+        if (exprResult) {
+            aResult = exprResult;
+            return NS_OK;
         }
     }
-    aResult = exprResult;
-    return aResult ? NS_OK : NS_ERROR_INVALID_ARG;
-} //-- getVariable
+
+    // Check if global variable is already evaluated
+    GlobalVariableValue* globVar;
+    globVar = (GlobalVariableValue*)mGlobalVariableValues.get(varName);
+    if (globVar) {
+        if (globVar->mFlags == GlobalVariableValue::evaluating) {
+            String err("Cyclic variable-value detected");
+            receiveError(err, NS_ERROR_FAILURE);
+            return NS_ERROR_FAILURE;
+        }
+        aResult = globVar->mValue;
+        return NS_OK;
+    }
+
+    // We need to evaluate the variable
+
+    // Search ImportFrames for the variable
+    ImportFrame* frame;
+    txListIterator frameIter(&mImportFrames);
+    Element* varElem = 0;
+    while (!varElem && (frame = (ImportFrame*)frameIter.next()))
+        varElem = (Element*)frame->mVariables.get(varName);
+
+    if (!varElem)
+        return NS_ERROR_FAILURE;
+
+    // Evaluate the variable
+    globVar = new GlobalVariableValue();
+    if (!globVar)
+        return NS_ERROR_OUT_OF_MEMORY;
+    rv = mGlobalVariableValues.add(varName, globVar);
+    if (NS_FAILED(rv)) {
+        delete globVar;
+        return rv;
+    }
+
+    // Set up the state we have at the beginning of the transformation
+    txVariableMap *oldVars = mLocalVariables;
+    mLocalVariables = 0;
+    txSingleNodeContext evalContext(mSourceDocument, this);
+    txIEvalContext* priorEC = setEvalContext(&evalContext);
+    // Compute the variable value
+    globVar->mFlags = GlobalVariableValue::evaluating;
+    globVar->mValue = txXSLTProcessor::processVariable(varElem, this);
+    setEvalContext(priorEC);
+    mLocalVariables = oldVars;
+
+    // evaluation is over, the gvv now owns the ExprResult
+    globVar->mFlags = GlobalVariableValue::owned;
+    aResult = globVar->mValue;
+    return NS_OK;
+}
 
 /**
  * Determines if the given XML node allows Whitespace stripping
 **/
-MBool ProcessorState::isStripSpaceAllowed(Node* node)
+MBool ProcessorState::isStripSpaceAllowed(Node* aNode)
 {
-    if (!node)
+    if (!aNode)
         return MB_FALSE;
 
-    switch (node->getNodeType()) {
+    switch (aNode->getNodeType()) {
         case Node::ELEMENT_NODE:
         {
             // check Whitespace stipping handling list against given Node
@@ -964,34 +1043,30 @@ MBool ProcessorState::isStripSpaceAllowed(Node* node)
                 txListIterator iter(&frame->mWhiteNameTests);
                 while (iter.hasNext()) {
                     txNameTestItem* iNameTest = (txNameTestItem*)iter.next();
-                    if (iNameTest->matches(node, this))
-                        return iNameTest->stripsSpace();
+                    if (iNameTest->matches(aNode, this)) {
+                        if (iNameTest->stripsSpace() && 
+                            !XMLUtils::getXMLSpacePreserve(aNode)) {
+                            return MB_TRUE;
+                        }
+                        return MB_FALSE;
+                    }
                 }
             }
-            if (mOutputFormat.mMethod == eHTMLOutput) {
-                String ucName = node->getNodeName();
-                ucName.toUpperCase();
-                if (ucName.isEqual("SCRIPT"))
-                    return MB_FALSE;
-            }
             break;
         }
         case Node::TEXT_NODE:
         case Node::CDATA_SECTION_NODE:
         {
-            if (!XMLUtils::isWhitespace(node->getNodeValue()))
+            if (!XMLUtils::isWhitespace(aNode->getNodeValue()))
                 return MB_FALSE;
-            return isStripSpaceAllowed(node->getParentNode());
+            return isStripSpaceAllowed(aNode->getParentNode());
         }
         case Node::DOCUMENT_NODE:
         {
             return MB_TRUE;
         }
     }
-    XMLSpaceMode mode = getXMLSpaceMode(node);
-    if (mode == DEFAULT)
-        return MB_FALSE;
-    return (MBool)(STRIP == mode);
+    return MB_FALSE;
 }
 
 /**
@@ -1011,7 +1086,7 @@ void ProcessorState::receiveError(const String& errorMessage, nsresult aRes)
  * This method is used for XPath Extension Functions.
  * @return the FunctionCall for the function with the given name.
 **/
-#define CHECK_FN(_name) aName == txXSLTAtoms::##_name
+#define CHECK_FN(_name) aName == txXSLTAtoms::_name
 
 nsresult ProcessorState::resolveFunctionCall(txAtom* aName, PRInt32 aID,
                                              Element* aElem,
@@ -1054,7 +1129,7 @@ nsresult ProcessorState::resolveFunctionCall(txAtom* aName, PRInt32 aID,
        return NS_OK;
    }
    if (CHECK_FN(functionAvailable)) {
-       aFunction = new FunctionAvailableFunctionCall();
+       aFunction = new FunctionAvailableFunctionCall(aElem);
        return NS_OK;
    }
 
@@ -1065,47 +1140,12 @@ nsresult ProcessorState::resolveFunctionCall(txAtom* aName, PRInt32 aID,
  //- Private Methods -/
 //-------------------/
 
-/*
- * Returns the closest xml:space value for the given Text node
- */
-ProcessorState::XMLSpaceMode ProcessorState::getXMLSpaceMode(Node* aNode)
-{
-    NS_ASSERTION(aNode, "Calling getXMLSpaceMode with NULL node!");
-
-    Node* parent = aNode;
-    while (parent) {
-        switch (parent->getNodeType()) {
-            case Node::ELEMENT_NODE:
-            {
-                String value;
-                ((Element*)parent)->getAttr(txXMLAtoms::space,
-                                            kNameSpaceID_XML, value);
-                if (value.isEqual(PRESERVE_VALUE))
-                    return PRESERVE;
-                break;
-            }
-            case Node::TEXT_NODE:
-            case Node::CDATA_SECTION_NODE:
-            {
-                // We will only see this the first time through the loop
-                // if the argument node is a text node.
-                break;
-            }
-            default:
-            {
-                return DEFAULT;
-            }
-        }
-        parent = parent->getParentNode();
-    }
-    return DEFAULT;
-}
-
 ProcessorState::ImportFrame::ImportFrame(ImportFrame* aFirstNotImported)
     : mNamedTemplates(MB_FALSE),
       mMatchableTemplates(MB_TRUE),
       mNamedAttributeSets(MB_TRUE),
-      mFirstNotImported(aFirstNotImported)
+      mFirstNotImported(aFirstNotImported),
+      mVariables(MB_FALSE)
 {
 }
 
@@ -1158,3 +1198,15 @@ void txPSParseContext::receiveError(const String& aMsg, nsresult aRes)
 {
     mPS->receiveError(aMsg, aRes);
 }
+
+/*
+ * GlobalVariableValue, Used avoid circular dependencies of variables
+ */
+
+ProcessorState::GlobalVariableValue::~GlobalVariableValue()
+{
+    NS_ASSERTION(mFlags != evaluating, "deleted while evaluating");
+    if (mFlags == owned) {
+        delete mValue;
+    }
+}
