@@ -55,6 +55,7 @@
 #include "nsNetUtil.h"
 #include "nsIUploadChannel.h"
 #include "nsIDOMSerializer.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIEventListenerManager.h"
@@ -79,9 +80,12 @@
 #endif
 #include "nsIDOMClassInfo.h"
 #include "nsIDOMElement.h"
+#include "nsIDOMWindow.h"
 #include "nsIVariant.h"
 #include "nsIParser.h"
 #include "nsLoadListenerProxy.h"
+#include "nsIWindowWatcher.h"
+#include "nsIAuthPrompt.h"
 
 static const char* kLoadAsData = "loadAsData";
 #define LOADSTR NS_LITERAL_STRING("load")
@@ -139,6 +143,7 @@ nsXMLHttpRequest::nsXMLHttpRequest()
   NS_INIT_ISUPPORTS();
   ChangeState(XML_HTTP_REQUEST_UNINITIALIZED,PR_FALSE);
   mAsync = PR_TRUE;
+  mCrossSiteAccessEnabled = PR_FALSE;
 }
 
 nsXMLHttpRequest::~nsXMLHttpRequest()
@@ -163,6 +168,8 @@ NS_INTERFACE_MAP_BEGIN(nsXMLHttpRequest)
   NS_INTERFACE_MAP_ENTRY(nsIDOMEventTarget)
   NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
   NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
+  NS_INTERFACE_MAP_ENTRY(nsIHttpEventSink)
+  NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
   NS_INTERFACE_MAP_ENTRY_EXTERNAL_DOM_CLASSINFO(XMLHttpRequest)
 NS_INTERFACE_MAP_END
@@ -374,7 +381,8 @@ nsXMLHttpRequest::ConvertBodyToText(PRUnichar **aOutBuffer)
       return rv;
   } else {
     if (NS_FAILED(DetectCharset(dataCharset)) || dataCharset.IsEmpty()) {
-      dataCharset.Assign(NS_LITERAL_STRING("ISO-8859-1")); // Parser assumes ISO-8859-1
+      // MS documentation states UTF-8 is default for responseText
+      dataCharset.Assign(NS_LITERAL_STRING("UTF-8"));
     }
   }
 
@@ -558,6 +566,33 @@ nsXMLHttpRequest::GetResponseHeader(const char *header, char **_retval)
   return NS_OK;
 }
 
+nsresult 
+nsXMLHttpRequest::GetLoadGroup(nsILoadGroup **aLoadGroup)
+{
+  NS_ENSURE_ARG_POINTER(aLoadGroup);
+  *aLoadGroup = nsnull;
+
+  if (!mScriptContext) {
+    GetCurrentContext(getter_AddRefs(mScriptContext));
+  }
+
+  if (mScriptContext) {
+    nsCOMPtr<nsIScriptGlobalObject> global;
+    mScriptContext->GetGlobalObject(getter_AddRefs(global));
+    nsCOMPtr<nsIDOMWindow> window = do_QueryInterface(global);
+    if (window) {
+      nsCOMPtr<nsIDOMDocument> domdoc;
+      window->GetDocument(getter_AddRefs(domdoc));
+      nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
+      if (doc) {
+        doc->GetDocumentLoadGroup(aLoadGroup);
+      }
+    }
+  }
+
+  return NS_OK;
+}
+
 /* noscript void openRequest (in string method, in string url, in boolean async, in string user, in string password); */
 NS_IMETHODIMP 
 nsXMLHttpRequest::OpenRequest(const char *method, 
@@ -594,7 +629,16 @@ nsXMLHttpRequest::OpenRequest(const char *method,
     authp = PR_TRUE;
   }
 
-  rv = NS_NewChannel(getter_AddRefs(mChannel), uri, nsnull, nsnull);
+  // When we are called from JS we can find the load group for the page,
+  // and add ourselves to it. This way any pending requests
+  // will be automatically aborted if the user leaves the page.
+  nsCOMPtr<nsILoadGroup> loadGroup;
+  GetLoadGroup(getter_AddRefs(loadGroup));
+
+  // nsIRequest::LOAD_BACKGROUND prevents throbber from becoming active,
+  // which in turn keeps STOP button from becoming active
+  rv = NS_NewChannel(getter_AddRefs(mChannel), uri, nsnull, loadGroup, 
+                     nsnull, nsIRequest::LOAD_BACKGROUND);
   if (NS_FAILED(rv)) return rv;
   
   //mChannel->SetAuthTriedWithPrehost(authp);
@@ -661,6 +705,14 @@ nsXMLHttpRequest::Open(const char *method, const char *url)
       return NS_OK;
     }
 
+    // Find out if UniversalBrowserRead privileges are enabled
+    // we will need this in case of a redirect
+    PRBool crossSiteAccessEnabled;
+    rv = secMan->IsCapabilityEnabled("UniversalBrowserRead",
+                                     &crossSiteAccessEnabled);
+    if (NS_FAILED(rv)) return rv;
+    mCrossSiteAccessEnabled = crossSiteAccessEnabled;
+
     if (argc > 2) {
       JSBool asyncBool;
       JS_ValueToBoolean(cx, argv[2], &asyncBool);
@@ -873,14 +925,21 @@ nsXMLHttpRequest::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult
   mReadRequest = nsnull;
   mContext = nsnull;
 
-  // The parser needs to be enabled for us to safely call RequestCompleted().
-  // If the parser is not enabled, it means it was blocked, by xml-stylesheet PI
-  // for example, and is still building the document. RequestCompleted() must be
-  // called later, when we get the load event from the document.
-  if (parser && parser->IsParserEnabled()) {
+  if (NS_FAILED(status)) {
+    // This can happen if the user leaves the page while the request was still
+    // active. This might also happen if request was active during the regular
+    // page load and the user was able to hit STOP button. If XMLHttpRequest is
+    // the only active network connection on the page the throbber nor the STOP
+    // button are active.
+    Abort();
+  } else if (parser && parser->IsParserEnabled()) {
+    // The parser needs to be enabled for us to safely call RequestCompleted().
+    // If the parser is not enabled, it means it was blocked, by xml-stylesheet PI
+    // for example, and is still building the document. RequestCompleted() must be
+    // called later, when we get the load event from the document.
     RequestCompleted();
   } else {
-    ChangeState(XML_HTTP_REQUEST_STOPPED,PR_FALSE);
+    ChangeState(XML_HTTP_REQUEST_STOPPED, PR_FALSE);
   }
 
   return rv;
@@ -1046,10 +1105,10 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
           if (NS_FAILED(rv)) 
             return NS_ERROR_FAILURE;
         } else {
-          // nsISupportsWString?
-          nsCOMPtr<nsISupportsWString> wstr(do_QueryInterface(supports));
+          // nsISupportsString?
+          nsCOMPtr<nsISupportsString> wstr(do_QueryInterface(supports));
           if (wstr) {
-            wstr->GetData(getter_Copies(serial));
+            wstr->GetData(serial);
           } else {
             // stream?
             nsCOMPtr<nsIInputStream> stream(do_QueryInterface(supports));
@@ -1194,8 +1253,16 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
   }
 #endif
 
+  if (!mScriptContext) {
+    // We need a context to check if redirect (if any) is allowed
+    GetCurrentContext(getter_AddRefs(mScriptContext));
+  }
+
+  nsCOMPtr<nsILoadGroup> loadGroup;
+  mChannel->GetLoadGroup(getter_AddRefs(loadGroup));
+
   rv = document->StartDocumentLoad(kLoadAsData, mChannel, 
-                                   nsnull, nsnull, 
+                                   loadGroup, nsnull, 
                                    getter_AddRefs(listener),
                                    PR_FALSE);
 
@@ -1210,6 +1277,9 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 #endif
 
+  // Hook us up to listen to redirects and the like
+  mChannel->SetNotificationCallbacks(this);
+
   // Start reading from the channel
   ChangeState(XML_HTTP_REQUEST_SENT);
   mXMLParserStreamListener = listener;
@@ -1249,9 +1319,17 @@ nsXMLHttpRequest::SetRequestHeader(const char *header, const char *value)
 
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
 
-  if (httpChannel)
+  if (httpChannel) {
+    // We need to set, not add to, the header. Using empty value will
+    // clear existing header.
+    nsresult rv = httpChannel->SetRequestHeader(nsDependentCString(header),
+                                                nsCString());
+    if (NS_FAILED(rv))
+      return rv;
+    // Now set it for real
     return httpChannel->SetRequestHeader(nsDependentCString(header),
                                          nsDependentCString(value));
+  }
   
   return NS_OK;
 }
@@ -1429,6 +1507,80 @@ nsXMLHttpRequest::ChangeState(nsXMLHttpRequestState aState, PRBool aBroadcast)
   return rv;
 }
 
+/////////////////////////////////////////////////////
+// nsIHttpEventSink methods:
+//
+NS_IMETHODIMP
+nsXMLHttpRequest::OnRedirect(nsIHttpChannel *aHttpChannel, nsIChannel *aNewChannel)
+{
+  NS_ENSURE_ARG_POINTER(aNewChannel);
+
+  if (mScriptContext && !mCrossSiteAccessEnabled) {
+    nsresult rv = NS_ERROR_FAILURE;
+
+    nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1", & rv));
+    if (NS_FAILED(rv))
+      return rv;
+
+    JSContext *cx = (JSContext *)mScriptContext->GetNativeContext();
+    if (!cx)
+      return NS_ERROR_UNEXPECTED;
+
+    nsCOMPtr<nsIScriptSecurityManager> secMan = 
+             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+    if (NS_FAILED(rv))
+      return rv;
+
+    nsCOMPtr<nsIURI> newURI;
+    rv = aNewChannel->GetURI(getter_AddRefs(newURI)); // The redirected URI
+    if (NS_FAILED(rv)) 
+      return rv;
+
+    stack->Push(cx);
+
+    rv = secMan->CheckSameOrigin(cx, newURI);
+
+    stack->Pop(&cx);
+    
+    if (NS_FAILED(rv))
+      return rv;
+  }
+
+  mChannel = aNewChannel;
+
+  return NS_OK;
+}
+
+/////////////////////////////////////////////////////
+// nsIInterfaceRequestor methods:
+//
+NS_IMETHODIMP
+nsXMLHttpRequest::GetInterface(const nsIID & aIID, void **aResult)
+{
+  if (aIID.Equals(NS_GET_IID(nsIAuthPrompt))) {
+    NS_ENSURE_ARG_POINTER(aResult);
+    *aResult = nsnull;
+
+    nsresult rv;
+    nsCOMPtr<nsIWindowWatcher> ww(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
+    if (NS_FAILED(rv))
+      return rv;
+
+    nsCOMPtr<nsIAuthPrompt> prompt;
+    rv = ww->GetNewAuthPrompter(nsnull, getter_AddRefs(prompt));
+    if (NS_FAILED(rv))
+      return rv;
+
+    nsIAuthPrompt *p = prompt.get();
+    NS_ADDREF(p);
+    *aResult = p;
+    return NS_OK;
+  }
+
+  return QueryInterface(aIID, aResult);
+}
+
+
 NS_IMPL_ISUPPORTS1(nsXMLHttpRequest::nsHeaderVisitor, nsIHttpHeaderVisitor)
 
 NS_IMETHODIMP nsXMLHttpRequest::
