@@ -38,6 +38,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "nsCOMPtr.h"
 #include "wspproxytest.h"
 #include "nsAString.h"
 #include "nsIInterfaceInfoManager.h"
@@ -123,7 +124,7 @@ WSPProxyTest::CreateComplexTypeWrapper(nsIWebServiceComplexTypeWrapper** aWrappe
 }
 
 NS_IMETHODIMP
-WSPProxyTest::TestComplexTypeWrapper(nsAWritableString& aResult)
+WSPProxyTest::TestComplexTypeWrapper(nsAString& aResult)
 {
   nsCOMPtr<nsIInterfaceInfo> info;
   nsCOMPtr<nsIWebServiceComplexTypeWrapper> wrapper;
@@ -148,7 +149,7 @@ WSPProxyTest::TestComplexTypeWrapper(nsAWritableString& aResult)
 
 nsresult
 WSPProxyTest::TestComplexTypeWrapperInstance(nsIPropertyBag* propBag,
-                                             nsAWritableString& aResult)
+                                             nsAString& aResult)
 {
   nsCOMPtr<nsISimpleEnumerator> enumerator;
   nsresult rv = propBag->GetEnumerator(getter_AddRefs(enumerator));
@@ -306,7 +307,7 @@ WSPProxyTest::TestComplexTypeWrapperInstance(nsIPropertyBag* propBag,
 
 
 NS_IMETHODIMP
-WSPProxyTest::TestPropertyBagWrapper(nsAWritableString& aResult)
+WSPProxyTest::TestPropertyBagWrapper(nsAString& aResult)
 {
   nsCOMPtr<nsIInterfaceInfo> info;
   nsCOMPtr<nsIWebServiceComplexTypeWrapper> ctwrapper;
@@ -453,7 +454,7 @@ WSPProxyTest::TestIsPrimeProxy(nsIWSPProxyTestListener* aListener)
     return NS_OK;
   }
   
-#define ISPRIMEURL "http://green.nscp.aoltw.net/vidur/wsdl/statistics.wsdl"
+#define ISPRIMEURL "http://ray.dsl.xmission.com:8080/wsdl/statistics.wsdl"
 #define ISPRIMEPORT "SpheonJSAOPStatisticsPort"
   nsresult rv = loader->LoadAsync(NS_LITERAL_STRING(ISPRIMEURL),
                                   NS_LITERAL_STRING(ISPRIMEPORT),
@@ -489,7 +490,7 @@ WSPProxyTest::OnLoad(nsIWSDLPort *port)
     return NS_OK;    
   }
 
-  proxy->Init(port, iinfo, NS_LITERAL_STRING("foo"), PR_TRUE);
+  proxy->Init(port, iinfo, manager, NS_LITERAL_STRING("foo"), PR_TRUE);
 
   mProxy = do_QueryInterface(proxy);
   if (!mProxy) {
@@ -516,17 +517,19 @@ WSPProxyTest::OnError(nsresult status, const nsAString & statusMessage)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-
-/* void getStatisticsCallback (in nsIException error, in statisticStruct retval); */
-NS_IMETHODIMP 
-WSPProxyTest::GetStatisticsCallback(nsIException *error, statisticStruct *retval)
+/* void onError (in nsIException error, in nsIWebServiceCallContext cx); */
+NS_IMETHODIMP WSPProxyTest::OnError(nsIException *error, nsIWebServiceCallContext *cx)
 {
-  if (error) {
-    nsXPIDLCString str;
-    error->ToString(getter_Copies(str));
-    mListener->OnIsPrimeProxyTestComplete(NS_ConvertASCIItoUCS2(str.get()));
-  }
-  else if (!retval) {
+  nsXPIDLCString str;
+  error->ToString(getter_Copies(str));
+  mListener->OnIsPrimeProxyTestComplete(NS_ConvertASCIItoUCS2(str.get()));
+  return NS_OK;
+}
+
+/* void getStatisticsCallback (in statisticStruct retval, in nsIWebServiceCallContext cx); */
+NS_IMETHODIMP WSPProxyTest::GetStatisticsCallback(statisticStruct *retval, nsIWebServiceCallContext *cx)
+{
+  if (!retval) {
     mListener->OnIsPrimeProxyTestComplete(NS_LITERAL_STRING("Incorrect value returned from IsPrimeNumber"));
   }
   else {
@@ -543,16 +546,10 @@ WSPProxyTest::GetStatisticsCallback(nsIException *error, statisticStruct *retval
   return NS_OK;
 }
 
-/* void isPrimeNumberCallback (in nsIException error, in boolean retval); */
-NS_IMETHODIMP 
-WSPProxyTest::IsPrimeNumberCallback(nsIException *error, PRBool retval)
+/* void isPrimeNumberCallback (in boolean retval, in nsIWebServiceCallContext cx); */
+NS_IMETHODIMP WSPProxyTest::IsPrimeNumberCallback(PRBool retval, nsIWebServiceCallContext *cx)
 {
-  if (error) {
-    nsXPIDLCString str;
-    error->ToString(getter_Copies(str));
-    mListener->OnIsPrimeProxyTestComplete(NS_ConvertASCIItoUCS2(str.get()));
-  }
-  else if (!retval) {
+  if (!retval) {
     mListener->OnIsPrimeProxyTestComplete(NS_LITERAL_STRING("Incorrect value returned from IsPrimeNumber"));
   }
   else {
@@ -572,14 +569,12 @@ WSPProxyTest::IsPrimeNumberCallback(nsIException *error, PRBool retval)
   return NS_OK;
 }
 
-/* void crossSumCallback (in nsIException error, in PRInt32 retval); */
-NS_IMETHODIMP 
-WSPProxyTest::CrossSumCallback(nsIException *error, PRInt32 retval)
+/* void crossSumCallback (in PRInt32 retval, in nsIWebServiceCallContext cx); */
+NS_IMETHODIMP WSPProxyTest::CrossSumCallback(PRInt32 retval, nsIWebServiceCallContext *cx)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-
 ///////////////////////////////////////////////////
 //
 // Implementation of WSPTestComplexType
