@@ -311,7 +311,7 @@ public:
   LoadListener(nsSchemaLoader* aLoader,
                nsISchemaLoadListener* aListener,
                nsIXMLHttpRequest* aRequest);
-  ~LoadListener();
+  virtual ~LoadListener();
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSIDOMEVENTLISTENER
@@ -358,7 +358,9 @@ LoadListener::HandleEvent(nsIDOMEvent *event)
     if (NS_SUCCEEDED(rv)) {
       nsCOMPtr<nsIDOMElement> element;
       
-      document->GetDocumentElement(getter_AddRefs(element));
+      if (document)
+        document->GetDocumentElement(getter_AddRefs(element));
+
       if (element) {
         rv = mLoader->ProcessSchemaElement(element, getter_AddRefs(schema));
       }
@@ -386,14 +388,12 @@ LoadListener::HandleEvent(nsIDOMEvent *event)
   return NS_OK;
 }
 
-
 ////////////////////////////////////////////////////////////
 //
-// nsSchemaLoader implementation
+// nsBuiltinSchemaCollection implementation
 //
 ////////////////////////////////////////////////////////////
-
-nsSchemaLoader::nsSchemaLoader()
+nsBuiltinSchemaCollection::nsBuiltinSchemaCollection()
 {
   NS_INIT_ISUPPORTS();
   if (!nsSchemaAtoms::sAnyType_atom) {
@@ -401,69 +401,49 @@ nsSchemaLoader::nsSchemaLoader()
   }
 }
 
-nsSchemaLoader::~nsSchemaLoader()
+nsBuiltinSchemaCollection::~nsBuiltinSchemaCollection()
 {
   mBuiltinTypesHash.Reset();
+  mSOAPTypeHash.Reset();
 }
 
-NS_IMPL_ISUPPORTS2_CI(nsSchemaLoader, 
-                      nsISchemaLoader, 
-                      nsISchemaCollection)
-
+NS_IMPL_ISUPPORTS1(nsBuiltinSchemaCollection,
+                   nsISchemaCollection)
 
 /* nsISchema getSchema (in AString targetNamespace); */
 NS_IMETHODIMP 
-nsSchemaLoader::GetSchema(const nsAReadableString & targetNamespace, 
-                          nsISchema **_retval)
+nsBuiltinSchemaCollection::GetSchema(const nsAString & targetNamespace,
+                                     nsISchema **_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
-
-  nsStringKey key(targetNamespace);
-  nsCOMPtr<nsISupports> sup = dont_AddRef(mSchemas.Get(&key));
-  nsCOMPtr<nsISchema> schema(do_QueryInterface(sup));
-
-  if (!schema) {
-    return NS_ERROR_SCHEMA_UNKNOWN_TARGET_NAMESPACE;  // no schema for specified targetname
-  }
-
-  *_retval = schema;
-  NS_ADDREF(*_retval);
-
-  return NS_OK;
+  *_retval = nsnull;
+  return NS_ERROR_SCHEMA_UNKNOWN_TARGET_NAMESPACE;
 }
 
 /* nsISchemaElement getElement (in AString name, in AString namespace); */
 NS_IMETHODIMP 
-nsSchemaLoader::GetElement(const nsAReadableString & aName, 
-                           const nsAReadableString & aNamespace, 
-                           nsISchemaElement **_retval)
+nsBuiltinSchemaCollection::GetElement(const nsAString & aName, 
+                                      const nsAString & aNamespace, 
+                                      nsISchemaElement **_retval)
 {
-  nsCOMPtr<nsISchema> schema;
-  nsresult rv = GetSchema(aNamespace, getter_AddRefs(schema));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  return schema->GetElementByName(aName, _retval);
+  NS_ENSURE_ARG_POINTER(_retval);
+  *_retval = nsnull;
+  return NS_ERROR_FAILURE;
 }
 
 /* nsISchemaAttribute getAttribute (in AString name, in AString namespace); */
 NS_IMETHODIMP 
-nsSchemaLoader::GetAttribute(const nsAReadableString & aName, 
-                             const nsAReadableString & aNamespace, 
-                             nsISchemaAttribute **_retval)
+nsBuiltinSchemaCollection::GetAttribute(const nsAString & aName, 
+                                        const nsAString & aNamespace, 
+                                        nsISchemaAttribute **_retval)
 {
-  nsCOMPtr<nsISchema> schema;
-  nsresult rv = GetSchema(aNamespace, getter_AddRefs(schema));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  return schema->GetAttributeByName(aName, _retval);
+  NS_ENSURE_ARG_POINTER(_retval);
+  *_retval = nsnull;
+  return NS_ERROR_FAILURE;
 }
 
-PRBool
-nsSchemaLoader::IsSchemaNamespace(const nsAReadableString& aNamespace)
+static PRBool
+IsSchemaNamespace(const nsAString& aNamespace)
 {
   if (aNamespace.Equals(NS_LITERAL_STRING(NS_SCHEMA_2001_NAMESPACE)) ||
       aNamespace.Equals(NS_LITERAL_STRING(NS_SCHEMA_1999_NAMESPACE))) {
@@ -474,8 +454,8 @@ nsSchemaLoader::IsSchemaNamespace(const nsAReadableString& aNamespace)
   }
 }
 
-PRBool
-nsSchemaLoader::IsSOAPNamespace(const nsAReadableString& aNamespace)
+static PRBool
+IsSOAPNamespace(const nsAString& aNamespace)
 {
   if (aNamespace.Equals(NS_LITERAL_STRING(NS_SOAP_1_1_ENCODING_NAMESPACE)) ||
       aNamespace.Equals(NS_LITERAL_STRING(NS_SOAP_1_2_ENCODING_NAMESPACE))) {
@@ -488,9 +468,9 @@ nsSchemaLoader::IsSOAPNamespace(const nsAReadableString& aNamespace)
 
 /* nsISchemaType getType (in AString name, in AString namespace); */
 NS_IMETHODIMP 
-nsSchemaLoader::GetType(const nsAReadableString & aName, 
-                        const nsAReadableString & aNamespace, 
-                        nsISchemaType **_retval)
+nsBuiltinSchemaCollection::GetType(const nsAString & aName, 
+                                   const nsAString & aNamespace, 
+                                   nsISchemaType **_retval)
 {
   if (IsSchemaNamespace(aNamespace)) {
     return GetBuiltinType(aName, aNamespace, _retval);
@@ -499,360 +479,17 @@ nsSchemaLoader::GetType(const nsAReadableString & aName,
   if (IsSOAPNamespace(aNamespace)) {
     return GetSOAPType(aName, aNamespace, _retval);
   }
-
-  nsCOMPtr<nsISchema> schema;
-  nsresult rv = GetSchema(aNamespace, getter_AddRefs(schema));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  return schema->GetTypeByName(aName, _retval);
+  
+  return NS_ERROR_SCHEMA_UNKNOWN_TYPE;
 }
 
 nsresult
-nsSchemaLoader::GetResolvedURI(const nsAReadableString& aSchemaURI,
-                               const char* aMethod,
-                               nsIURI** aURI)
-{
-  nsresult rv;
-  nsCOMPtr<nsIXPCNativeCallContext> cc;
-  nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
-  if(NS_SUCCEEDED(rv)) {
-    rv = xpc->GetCurrentNativeCallContext(getter_AddRefs(cc));
-  }
-
-  if (NS_SUCCEEDED(rv) && cc) {
-    JSContext* cx;
-    rv = cc->GetJSContext(&cx);
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsIScriptSecurityManager> secMan(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsIURI> baseURI;
-    nsCOMPtr<nsIPrincipal> principal;
-    rv = secMan->GetSubjectPrincipal(getter_AddRefs(principal));
-    if (NS_SUCCEEDED(rv)) {
-      nsCOMPtr<nsICodebasePrincipal> codebase = do_QueryInterface(principal);
-      if (codebase) {
-        codebase->GetURI(getter_AddRefs(baseURI));
-      }
-    }
-    
-    rv = NS_NewURI(aURI, aSchemaURI, baseURI);
-    if (NS_FAILED(rv)) return rv;
-    
-    rv = secMan->CheckLoadURIFromScript(cx, *aURI);
-    if (NS_FAILED(rv))
-    {
-      // Security check failed. The above call set a JS exception. The
-      // following lines ensure that the exception is propagated.
-      cc->SetExceptionWasThrown(PR_TRUE);
-      return rv;
-    }
-  }
-  else {
-    rv = NS_NewURI(aURI, aSchemaURI, nsnull);
-    if (NS_FAILED(rv)) return rv;
-  }
-
-  return NS_OK;
-}
-
-/* nsISchema load (in AString schemaURI); */
-NS_IMETHODIMP 
-nsSchemaLoader::Load(const nsAReadableString& schemaURI, 
-                     nsISchema **_retval)
-{
-  NS_ENSURE_ARG_POINTER(_retval);
-
-  nsCOMPtr<nsIURI> resolvedURI;
-  nsresult rv = GetResolvedURI(schemaURI, "load", getter_AddRefs(resolvedURI));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  nsXPIDLCString spec;
-  resolvedURI->GetSpec(getter_Copies(spec));
-  
-  nsCOMPtr<nsIXMLHttpRequest> request(do_CreateInstance(NS_XMLHTTPREQUEST_CONTRACTID, &rv));
-  if (!request) {
-    return rv;
-  }
-
-  rv = request->OpenRequest("GET",
-                            spec.get(),
-                            PR_FALSE, nsnull, nsnull);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  rv = request->Send(nsnull);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIDOMDocument> document;
-  rv = request->GetResponseXML(getter_AddRefs(document));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIDOMElement> element;
-  document->GetDocumentElement(getter_AddRefs(element));
-  if (element) {
-    rv = ProcessSchemaElement(element, _retval);
-  }
-  else {
-    rv = NS_ERROR_SCHEMA_NOT_SCHEMA_ELEMENT;
-  }
-  
-  return rv;
-}
-
-/* void loadAsync (in AString schemaURI, in nsISchemaLoadListener listener); */
-NS_IMETHODIMP 
-nsSchemaLoader::LoadAsync(const nsAReadableString& schemaURI, 
-                          nsISchemaLoadListener *aListener)
-{
-  NS_ENSURE_ARG(aListener);
-
-  nsCOMPtr<nsIURI> resolvedURI;
-  nsresult rv = GetResolvedURI(schemaURI, "loadAsync", getter_AddRefs(resolvedURI));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  nsXPIDLCString spec;
-  resolvedURI->GetSpec(getter_Copies(spec));
-
-  nsCOMPtr<nsIXMLHttpRequest> request(do_CreateInstance(NS_XMLHTTPREQUEST_CONTRACTID, &rv));
-  if (!request) {
-    return rv;
-  }
-
-  rv = request->OpenRequest("GET",
-                            spec.get(),
-                            PR_TRUE, nsnull, nsnull);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIDOMEventListener> listener;
-  LoadListener* listenerInst = new LoadListener(this, aListener, request);
-  if (!listenerInst) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  listener = listenerInst;
-
-  nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(request));
-  if (!target) {
-    return NS_ERROR_UNEXPECTED;
-  }
-  
-  rv = target->AddEventListener(NS_LITERAL_STRING("load"),
-                                listener, PR_FALSE);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  rv = target->AddEventListener(NS_LITERAL_STRING("error"),
-                                listener, PR_FALSE);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  // The listener keeps the request alive until its complete
-  rv = request->Send(nsnull);
-  
-  return rv;
-}
-
-static const char* kSchemaNamespaces[] = {NS_SCHEMA_1999_NAMESPACE, 
-                                          NS_SCHEMA_2001_NAMESPACE};
-static PRUint32 kSchemaNamespacesLength = sizeof(kSchemaNamespaces) / sizeof(const char*);
-
-/* nsISchema processSchemaElement (in nsIDOMElement element); */
-NS_IMETHODIMP 
-nsSchemaLoader::ProcessSchemaElement(nsIDOMElement *element, 
-                                     nsISchema **_retval)
-{
-  NS_ENSURE_ARG(element);
-  NS_ENSURE_ARG_POINTER(_retval);
-
-  nsresult rv = NS_OK;
-
-  // Get target namespace and create the schema instance
-  nsAutoString targetNamespace, schemaNamespace;
-  element->GetAttribute(NS_LITERAL_STRING("targetNamespace"), 
-                        targetNamespace);
-  element->GetNamespaceURI(schemaNamespace);
-  nsSchema* schemaInst = new nsSchema(this, targetNamespace, schemaNamespace);
-  nsCOMPtr<nsISchema> schema = schemaInst;
-  if (!schema) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  nsChildElementIterator iterator(element, 
-                                  kSchemaNamespaces, kSchemaNamespacesLength);
-  nsCOMPtr<nsIDOMElement> childElement;
-  nsCOMPtr<nsIAtom> tagName;
-
-  while (NS_SUCCEEDED(iterator.GetNextChild(getter_AddRefs(childElement),
-                                            getter_AddRefs(tagName))) &&
-         childElement) {
-    if (tagName == nsSchemaAtoms::sElement_atom) {
-      nsCOMPtr<nsISchemaElement> schemaElement;
-      rv = ProcessElement(schemaInst, childElement,  
-                          getter_AddRefs(schemaElement));
-      if (NS_SUCCEEDED(rv)) {
-        rv = schemaInst->AddElement(schemaElement);
-      }
-    }
-    else if (tagName == nsSchemaAtoms::sComplexType_atom) {
-      nsCOMPtr<nsISchemaComplexType> complexType;
-      rv = ProcessComplexType(schemaInst, childElement,
-                              getter_AddRefs(complexType));
-      if (NS_SUCCEEDED(rv)) {
-        rv = schemaInst->AddType(complexType);
-      }
-    }
-    else if (tagName == nsSchemaAtoms::sSimpleType_atom) {
-      nsCOMPtr<nsISchemaSimpleType> simpleType;
-      rv = ProcessSimpleType(schemaInst, childElement,
-                             getter_AddRefs(simpleType));
-      if (NS_SUCCEEDED(rv)) {
-        rv = schemaInst->AddType(simpleType);
-      }
-    }
-    else if (tagName == nsSchemaAtoms::sAttribute_atom) {
-      nsCOMPtr<nsISchemaAttribute> attribute;
-      rv = ProcessAttribute(schemaInst, childElement,
-                            getter_AddRefs(attribute));
-      if (NS_SUCCEEDED(rv)) {
-        rv = schemaInst->AddAttribute(attribute);
-      }
-    }
-    else if (tagName == nsSchemaAtoms::sAttributeGroup_atom) {
-      nsCOMPtr<nsISchemaAttributeGroup> attributeGroup;
-      rv = ProcessAttributeGroup(schemaInst, childElement,
-                                 getter_AddRefs(attributeGroup));
-      if (NS_SUCCEEDED(rv)) {
-        rv = schemaInst->AddAttributeGroup(attributeGroup);
-      }
-    }
-    else if (tagName == nsSchemaAtoms::sModelGroup_atom) {
-      nsCOMPtr<nsISchemaModelGroup> modelGroup;
-      rv = ProcessModelGroup(schemaInst, childElement,
-                             tagName, nsnull, getter_AddRefs(modelGroup));
-      if (NS_SUCCEEDED(rv)) {
-        rv = schemaInst->AddModelGroup(modelGroup);
-      }
-    }
-    // For now, ignore the following
-    // annotations
-    // include
-    // import
-    // redefine
-    // notation
-    // identity-constraint elements
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-  }
-
-  // Resolve all forward references 
-  rv = schemaInst->Resolve();
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsStringKey key(targetNamespace);
-  mSchemas.Put(&key, schema);
-
-  *_retval = schema;
-  NS_ADDREF(*_retval);
-
-  return NS_OK;
-}
-
-PRBool 
-ParseQualifiedName(nsIDOMElement* aContext,
-                   const nsAReadableString& aQualifiedName,
-                   nsAWritableString& aPrefix,
-                   nsAWritableString& aLocalName,
-                   nsAWritableString& aNamespaceURI) 
-{
-  nsReadingIterator<PRUnichar> pos, begin, end;
-  
-  aQualifiedName.BeginReading(begin);
-  aQualifiedName.EndReading(end); 
-  pos = begin;
-  
-  if (FindCharInReadable(PRUnichar(':'), pos, end)) {
-    CopyUnicodeTo(begin, pos, aPrefix);
-    CopyUnicodeTo(++pos, end, aLocalName);
-  }
-  else {
-    CopyUnicodeTo(begin, end, aLocalName);
-  }
-  
-  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(aContext));
-  
-  return node->LookupNamespaceURI(aPrefix, aNamespaceURI);
-}
-
-nsresult
-nsSchemaLoader::GetSOAPType(const nsAReadableString& aName,
-                            const nsAReadableString& aNamespace,
-                            nsISchemaType** aType)
+nsBuiltinSchemaCollection::GetBuiltinType(const nsAString& aName,
+                                          const nsAString& aNamespace,
+                                          nsISchemaType** aType)
 {
   nsresult rv = NS_OK;
-  nsAutoString concat(aNamespace);
-  concat.Append(aName);
-  nsStringKey key(concat);
-  nsCOMPtr<nsISupports> sup = dont_AddRef(mSOAPTypeHash.Get(&key));
-  if (sup) {
-    rv = CallQueryInterface(sup, aType);
-  }
-  else {
-    if (aName.Equals(NS_LITERAL_STRING("Array"))) {
-      nsSOAPArray* array = new nsSOAPArray(aNamespace);
-      if (!array) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
-      sup = array;
-      mSOAPTypeHash.Put(&key, sup);
-    
-      *aType = array;
-      NS_ADDREF(*aType);
-    }
-    else if (aName.Equals(NS_LITERAL_STRING("arrayType"))) {
-      nsSOAPArrayType* arrayType = new nsSOAPArrayType(aNamespace);
-      if (!arrayType) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
-      sup = arrayType;
-      mSOAPTypeHash.Put(&key, sup);
-    
-      *aType = arrayType;
-      NS_ADDREF(*aType);
-    }
-    else {
-      rv = NS_ERROR_SCHEMA_UNKNOWN_TYPE;
-    }
-  }
-
-  return rv;
-}
-
-nsresult
-nsSchemaLoader::GetBuiltinType(const nsAReadableString& aName,
-                               const nsAReadableString& aNamespace,
-                               nsISchemaType** aType)
-{
-  nsresult rv = NS_OK;
-  nsAutoString concat(aName);
-  concat.Append(aNamespace);
-  nsStringKey key(concat);
+  nsStringKey key(aName);
   nsCOMPtr<nsISupports> sup = dont_AddRef(mBuiltinTypesHash.Get(&key));
   if (sup) {
     rv = CallQueryInterface(sup, aType);
@@ -1000,8 +637,7 @@ nsSchemaLoader::GetBuiltinType(const nsAReadableString& aName,
       return NS_ERROR_SCHEMA_UNKNOWN_TYPE;
     }
 
-    nsSchemaBuiltinType* builtin = new nsSchemaBuiltinType(typeVal,
-                                                           aNamespace);
+    nsSchemaBuiltinType* builtin = new nsSchemaBuiltinType(typeVal);
     if (!builtin) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -1015,10 +651,454 @@ nsSchemaLoader::GetBuiltinType(const nsAReadableString& aName,
   return NS_OK;
 }
 
+nsresult
+nsBuiltinSchemaCollection::GetSOAPType(const nsAString& aName,
+                                       const nsAString& aNamespace,
+                                       nsISchemaType** aType)
+{
+  nsresult rv = NS_OK;
+  nsStringKey key(aName);
+  nsCOMPtr<nsISupports> sup = dont_AddRef(mSOAPTypeHash.Get(&key));
+  if (sup) {
+    rv = CallQueryInterface(sup, aType);
+  }
+  else {
+    if (aName.Equals(NS_LITERAL_STRING("Array"))) {
+      nsCOMPtr<nsISchemaType> anyType;
+      rv = GetBuiltinType(NS_LITERAL_STRING("anyType"),
+                          NS_LITERAL_STRING(NS_SCHEMA_2001_NAMESPACE),
+                          getter_AddRefs(anyType));
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
+
+      nsSOAPArray* array = new nsSOAPArray(anyType);
+      if (!array) {
+        return NS_ERROR_OUT_OF_MEMORY;
+      }
+      sup = array;
+      mSOAPTypeHash.Put(&key, sup);
+    
+      *aType = array;
+      NS_ADDREF(*aType);
+    }
+    else if (aName.Equals(NS_LITERAL_STRING("arrayType"))) {
+      nsSOAPArrayType* arrayType = new nsSOAPArrayType();
+      if (!arrayType) {
+        return NS_ERROR_OUT_OF_MEMORY;
+      }
+      sup = arrayType;
+      mSOAPTypeHash.Put(&key, sup);
+    
+      *aType = arrayType;
+      NS_ADDREF(*aType);
+    }
+    else {
+      rv = NS_ERROR_SCHEMA_UNKNOWN_TYPE;
+    }
+  }
+
+  return rv;
+}
+
+////////////////////////////////////////////////////////////
+//
+// nsSchemaLoader implementation
+//
+////////////////////////////////////////////////////////////
+
+nsSchemaLoader::nsSchemaLoader()
+{
+  NS_INIT_ISUPPORTS();
+  mBuiltinCollection = do_GetService(NS_BUILTINSCHEMACOLLECTION_CONTRACTID);
+}
+
+nsSchemaLoader::~nsSchemaLoader()
+{
+}
+
+NS_IMPL_ISUPPORTS2_CI(nsSchemaLoader, 
+                      nsISchemaLoader, 
+                      nsISchemaCollection)
+
+
+/* nsISchema getSchema (in AString targetNamespace); */
+NS_IMETHODIMP 
+nsSchemaLoader::GetSchema(const nsAString & targetNamespace, 
+                          nsISchema **_retval)
+{
+  NS_ENSURE_ARG_POINTER(_retval);
+
+  nsStringKey key(targetNamespace);
+  nsCOMPtr<nsISupports> sup = dont_AddRef(mSchemas.Get(&key));
+  nsCOMPtr<nsISchema> schema(do_QueryInterface(sup));
+
+  if (!schema) {
+    return NS_ERROR_SCHEMA_UNKNOWN_TARGET_NAMESPACE;  // no schema for specified targetname
+  }
+
+  *_retval = schema;
+  NS_ADDREF(*_retval);
+
+  return NS_OK;
+}
+
+/* nsISchemaElement getElement (in AString name, in AString namespace); */
+NS_IMETHODIMP 
+nsSchemaLoader::GetElement(const nsAString & aName, 
+                           const nsAString & aNamespace, 
+                           nsISchemaElement **_retval)
+{
+  nsCOMPtr<nsISchema> schema;
+  nsresult rv = GetSchema(aNamespace, getter_AddRefs(schema));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  return schema->GetElementByName(aName, _retval);
+}
+
+/* nsISchemaAttribute getAttribute (in AString name, in AString namespace); */
+NS_IMETHODIMP 
+nsSchemaLoader::GetAttribute(const nsAString & aName, 
+                             const nsAString & aNamespace, 
+                             nsISchemaAttribute **_retval)
+{
+  nsCOMPtr<nsISchema> schema;
+  nsresult rv = GetSchema(aNamespace, getter_AddRefs(schema));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  return schema->GetAttributeByName(aName, _retval);
+}
+
+/* nsISchemaType getType (in AString name, in AString namespace); */
+NS_IMETHODIMP 
+nsSchemaLoader::GetType(const nsAString & aName, 
+                        const nsAString & aNamespace, 
+                        nsISchemaType **_retval)
+{
+  if (IsSchemaNamespace(aNamespace) || IsSOAPNamespace(aNamespace)) {
+    return mBuiltinCollection->GetType(aName, aNamespace, _retval);
+  }
+
+  nsCOMPtr<nsISchema> schema;
+  nsresult rv = GetSchema(aNamespace, getter_AddRefs(schema));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  return schema->GetTypeByName(aName, _retval);
+}
+
+nsresult
+nsSchemaLoader::GetResolvedURI(const nsAString& aSchemaURI,
+                               const char* aMethod,
+                               nsIURI** aURI)
+{
+  nsresult rv;
+  nsCOMPtr<nsIXPCNativeCallContext> cc;
+  nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
+  if(NS_SUCCEEDED(rv)) {
+    rv = xpc->GetCurrentNativeCallContext(getter_AddRefs(cc));
+  }
+
+  if (NS_SUCCEEDED(rv) && cc) {
+    JSContext* cx;
+    rv = cc->GetJSContext(&cx);
+    if (NS_FAILED(rv)) return rv;
+
+    nsCOMPtr<nsIScriptSecurityManager> secMan(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv));
+    if (NS_FAILED(rv)) return rv;
+
+    nsCOMPtr<nsIURI> baseURI;
+    nsCOMPtr<nsIPrincipal> principal;
+    rv = secMan->GetSubjectPrincipal(getter_AddRefs(principal));
+    if (NS_SUCCEEDED(rv)) {
+      nsCOMPtr<nsICodebasePrincipal> codebase = do_QueryInterface(principal);
+      if (codebase) {
+        codebase->GetURI(getter_AddRefs(baseURI));
+      }
+    }
+    
+    rv = NS_NewURI(aURI, aSchemaURI, nsnull, baseURI);
+    if (NS_FAILED(rv)) return rv;
+    
+    rv = secMan->CheckLoadURIFromScript(cx, *aURI);
+    if (NS_FAILED(rv))
+    {
+      // Security check failed. The above call set a JS exception. The
+      // following lines ensure that the exception is propagated.
+      cc->SetExceptionWasThrown(PR_TRUE);
+      return rv;
+    }
+  }
+  else {
+    rv = NS_NewURI(aURI, aSchemaURI, nsnull);
+    if (NS_FAILED(rv)) return rv;
+  }
+
+  return NS_OK;
+}
+
+/* nsISchema load (in AString schemaURI); */
+NS_IMETHODIMP 
+nsSchemaLoader::Load(const nsAString& schemaURI, 
+                     nsISchema **_retval)
+{
+  NS_ENSURE_ARG_POINTER(_retval);
+
+  nsCOMPtr<nsIURI> resolvedURI;
+  nsresult rv = GetResolvedURI(schemaURI, "load", getter_AddRefs(resolvedURI));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  nsCAutoString spec;
+  resolvedURI->GetSpec(spec);
+  
+  nsCOMPtr<nsIXMLHttpRequest> request(do_CreateInstance(NS_XMLHTTPREQUEST_CONTRACTID, &rv));
+  if (!request) {
+    return rv;
+  }
+
+  rv = request->OpenRequest("GET",
+                            spec.get(),
+                            PR_FALSE, nsnull, nsnull);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  // Force the mimetype of the returned stream to be xml.
+  rv = request->OverrideMimeType("text/xml");
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  rv = request->Send(nsnull);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsCOMPtr<nsIDOMDocument> document;
+  rv = request->GetResponseXML(getter_AddRefs(document));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsCOMPtr<nsIDOMElement> element;
+  document->GetDocumentElement(getter_AddRefs(element));
+  if (element) {
+    rv = ProcessSchemaElement(element, _retval);
+  }
+  else {
+    rv = NS_ERROR_SCHEMA_NOT_SCHEMA_ELEMENT;
+  }
+  
+  return rv;
+}
+
+/* void loadAsync (in AString schemaURI, in nsISchemaLoadListener listener); */
+NS_IMETHODIMP 
+nsSchemaLoader::LoadAsync(const nsAString& schemaURI, 
+                          nsISchemaLoadListener *aListener)
+{
+  NS_ENSURE_ARG(aListener);
+
+  nsCOMPtr<nsIURI> resolvedURI;
+  nsresult rv = GetResolvedURI(schemaURI, "loadAsync", getter_AddRefs(resolvedURI));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  nsCAutoString spec;
+  resolvedURI->GetSpec(spec);
+
+  nsCOMPtr<nsIXMLHttpRequest> request(do_CreateInstance(NS_XMLHTTPREQUEST_CONTRACTID, &rv));
+  if (!request) {
+    return rv;
+  }
+
+  rv = request->OpenRequest("GET",
+                            spec.get(),
+                            PR_TRUE, nsnull, nsnull);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  // Force the mimetype of the returned stream to be xml.
+  rv = request->OverrideMimeType("text/xml");
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsCOMPtr<nsIDOMEventListener> listener;
+  LoadListener* listenerInst = new LoadListener(this, aListener, request);
+  if (!listenerInst) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  listener = listenerInst;
+
+  nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(request));
+  if (!target) {
+    return NS_ERROR_UNEXPECTED;
+  }
+  
+  rv = target->AddEventListener(NS_LITERAL_STRING("load"),
+                                listener, PR_FALSE);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  rv = target->AddEventListener(NS_LITERAL_STRING("error"),
+                                listener, PR_FALSE);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  // The listener keeps the request alive until its complete
+  rv = request->Send(nsnull);
+  
+  return rv;
+}
+
+static const char* kSchemaNamespaces[] = {NS_SCHEMA_1999_NAMESPACE, 
+                                          NS_SCHEMA_2001_NAMESPACE};
+static PRUint32 kSchemaNamespacesLength = sizeof(kSchemaNamespaces) / sizeof(const char*);
+
+/* nsISchema processSchemaElement (in nsIDOMElement element); */
+NS_IMETHODIMP 
+nsSchemaLoader::ProcessSchemaElement(nsIDOMElement *element, 
+                                     nsISchema **_retval)
+{
+  NS_ENSURE_ARG(element);
+  NS_ENSURE_ARG_POINTER(_retval);
+
+  nsresult rv = NS_OK;
+
+  // Get target namespace and create the schema instance
+  nsAutoString targetNamespace, schemaNamespace;
+  element->GetAttribute(NS_LITERAL_STRING("targetNamespace"), 
+                        targetNamespace);
+  element->GetNamespaceURI(schemaNamespace);
+  nsSchema* schemaInst = new nsSchema(this, targetNamespace, schemaNamespace);
+  nsCOMPtr<nsISchema> schema = schemaInst;
+  if (!schema) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  nsChildElementIterator iterator(element, 
+                                  kSchemaNamespaces, kSchemaNamespacesLength);
+  nsCOMPtr<nsIDOMElement> childElement;
+  nsCOMPtr<nsIAtom> tagName;
+
+  while (NS_SUCCEEDED(iterator.GetNextChild(getter_AddRefs(childElement),
+                                            getter_AddRefs(tagName))) &&
+         childElement) {
+    if (tagName == nsSchemaAtoms::sElement_atom) {
+      nsCOMPtr<nsISchemaElement> schemaElement;
+      rv = ProcessElement(schemaInst, childElement,  
+                          getter_AddRefs(schemaElement));
+      if (NS_SUCCEEDED(rv)) {
+        rv = schemaInst->AddElement(schemaElement);
+      }
+    }
+    else if (tagName == nsSchemaAtoms::sComplexType_atom) {
+      nsCOMPtr<nsISchemaComplexType> complexType;
+      rv = ProcessComplexType(schemaInst, childElement,
+                              getter_AddRefs(complexType));
+      if (NS_SUCCEEDED(rv)) {
+        rv = schemaInst->AddType(complexType);
+      }
+    }
+    else if (tagName == nsSchemaAtoms::sSimpleType_atom) {
+      nsCOMPtr<nsISchemaSimpleType> simpleType;
+      rv = ProcessSimpleType(schemaInst, childElement,
+                             getter_AddRefs(simpleType));
+      if (NS_SUCCEEDED(rv)) {
+        rv = schemaInst->AddType(simpleType);
+      }
+    }
+    else if (tagName == nsSchemaAtoms::sAttribute_atom) {
+      nsCOMPtr<nsISchemaAttribute> attribute;
+      rv = ProcessAttribute(schemaInst, childElement,
+                            getter_AddRefs(attribute));
+      if (NS_SUCCEEDED(rv)) {
+        rv = schemaInst->AddAttribute(attribute);
+      }
+    }
+    else if (tagName == nsSchemaAtoms::sAttributeGroup_atom) {
+      nsCOMPtr<nsISchemaAttributeGroup> attributeGroup;
+      rv = ProcessAttributeGroup(schemaInst, childElement,
+                                 getter_AddRefs(attributeGroup));
+      if (NS_SUCCEEDED(rv)) {
+        rv = schemaInst->AddAttributeGroup(attributeGroup);
+      }
+    }
+    else if (tagName == nsSchemaAtoms::sModelGroup_atom) {
+      nsCOMPtr<nsISchemaModelGroup> modelGroup;
+      rv = ProcessModelGroup(schemaInst, childElement,
+                             tagName, nsnull, getter_AddRefs(modelGroup));
+      if (NS_SUCCEEDED(rv)) {
+        rv = schemaInst->AddModelGroup(modelGroup);
+      }
+    }
+    // For now, ignore the following
+    // annotations
+    // include
+    // import
+    // redefine
+    // notation
+    // identity-constraint elements
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  }
+
+  // Resolve all forward references 
+  rv = schemaInst->Resolve();
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsStringKey key(targetNamespace);
+  mSchemas.Put(&key, schema);
+
+  *_retval = schema;
+  NS_ADDREF(*_retval);
+
+  return NS_OK;
+}
+
+PRBool 
+ParseQualifiedName(nsIDOMElement* aContext,
+                   const nsAString& aQualifiedName,
+                   nsAString& aPrefix,
+                   nsAString& aLocalName,
+                   nsAString& aNamespaceURI) 
+{
+  nsReadingIterator<PRUnichar> pos, begin, end;
+  
+  aQualifiedName.BeginReading(begin);
+  aQualifiedName.EndReading(end); 
+  pos = begin;
+  
+  if (FindCharInReadable(PRUnichar(':'), pos, end)) {
+    CopyUnicodeTo(begin, pos, aPrefix);
+    CopyUnicodeTo(++pos, end, aLocalName);
+  }
+  else {
+    CopyUnicodeTo(begin, end, aLocalName);
+  }
+  
+  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(aContext));
+  
+  return node->LookupNamespaceURI(aPrefix, aNamespaceURI);
+}
+
 nsresult
 nsSchemaLoader::GetNewOrUsedType(nsSchema* aSchema,
                                  nsIDOMElement* aContext,
-                                 const nsAReadableString& aTypeName,
+                                 const nsAString& aTypeName,
                                  nsISchemaType** aType)
 {
   nsresult rv = NS_OK;
@@ -1161,9 +1241,9 @@ nsSchemaLoader::ProcessElement(nsSchema* aSchema,
     if (!schemaType) {
       nsAutoString ns;
       aElement->GetNamespaceURI(ns);
-      rv = GetBuiltinType(NS_LITERAL_STRING("anyType"),
-                          ns,
-                          getter_AddRefs(schemaType));
+      rv = GetType(NS_LITERAL_STRING("anyType"),
+                   ns,
+                   getter_AddRefs(schemaType));
       if (NS_FAILED(rv)) {
         return rv;
       }
@@ -1257,6 +1337,125 @@ nsSchemaLoader::ProcessComplexType(nsSchema* aSchema,
   return NS_OK;
 }
 
+void
+nsSchemaLoader::ConstructArrayName(nsISchemaType* aType,
+                                   nsAString& aName)
+{
+  nsAutoString typeName;
+  
+  aType->GetName(typeName);
+  aName.Assign(NS_LITERAL_STRING("ArrayOf") + typeName);
+}
+
+nsresult
+nsSchemaLoader::ParseDimensions(nsSchema* aSchema,
+                                nsIDOMElement* aAttrElement,
+                                const nsAString& aStr,
+                                nsISchemaType* aBaseType,
+                                nsISchemaType** aArrayType,
+                                PRUint32* aDimension)
+{
+  nsReadingIterator<PRUnichar> iter, done_reading;
+  aStr.BeginReading(iter);
+  aStr.EndReading(done_reading);
+
+  PRUint32 dimension = 1;
+  PRUnichar uc = *iter++;
+  if (uc != PRUnichar('[')) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  while (iter != done_reading) {
+    uc = *iter++;
+    if (uc == PRUnichar(',')) {
+      dimension++;
+    }
+    else if (uc == PRUnichar(']')) {
+      break;
+    }
+  }
+  *aDimension = dimension;
+
+  while ((iter != done_reading) && (*iter == PRUnichar(' '))) {
+    ++iter;
+  }
+
+  // If there's still more to go, then create an array type
+  // based on the base and continue to parse
+  if ((iter != done_reading) && (*iter == PRUnichar('['))) {
+    nsAutoString name;
+    nsCOMPtr<nsISchemaType> myArrayType;
+    PRUint32 myDimension;
+    
+    nsresult rv = ParseDimensions(aSchema, aAttrElement,
+                                  nsDependentSubstring(iter, done_reading),
+                                  aBaseType, getter_AddRefs(myArrayType), 
+                                  &myDimension);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+
+    ConstructArrayName(myArrayType, name);
+    nsSchemaComplexType* typeInst = new nsSchemaComplexType(aSchema,
+                                                            name, 
+                                                            PR_FALSE);
+    if (!typeInst) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+    nsCOMPtr<nsISchemaComplexType> complexType = typeInst;
+
+    nsCOMPtr<nsISchemaType> soapArray;
+    rv = GetType(NS_LITERAL_STRING("Array"),
+                 NS_LITERAL_STRING(NS_SOAP_1_2_ENCODING_NAMESPACE),
+                 getter_AddRefs(soapArray));
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+
+    typeInst->SetContentModel(nsISchemaComplexType::CONTENT_MODEL_ELEMENT_ONLY);
+    typeInst->SetDerivation(nsISchemaComplexType::DERIVATION_RESTRICTION_COMPLEX,
+                            soapArray);
+    typeInst->SetArrayInfo(myArrayType, myDimension);
+
+    *aArrayType = typeInst;
+  }
+  else {
+    *aArrayType = aBaseType;
+  }
+  NS_ADDREF(*aArrayType);
+
+  return NS_OK;
+}
+
+nsresult
+nsSchemaLoader::ParseArrayType(nsSchema* aSchema,
+                               nsIDOMElement* aAttrElement,
+                               const nsAString& aStr,
+                               nsISchemaType** aType,
+                               PRUint32* aDimension)
+{
+  PRInt32 offset;
+  
+  offset = aStr.FindChar(PRUnichar('['));
+  if (offset == -1) {
+    return NS_ERROR_SCHEMA_UNKNOWN_TYPE;
+  }
+  nsDependentSubstring typeStr(aStr, 0, offset);
+          
+  nsCOMPtr<nsISchemaType> type;
+  nsresult rv = GetNewOrUsedType(aSchema, aAttrElement, typeStr, 
+                                 getter_AddRefs(type));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  
+  nsDependentSubstring dimensionStr(aStr, offset, 
+                                    aStr.Length() - offset);
+  return ParseDimensions(aSchema, aAttrElement, dimensionStr, type,
+                         aType, aDimension);
+}
+
+
 nsresult
 nsSchemaLoader::ProcessComplexTypeBody(nsSchema* aSchema, 
                                        nsIDOMElement* aElement,
@@ -1273,7 +1472,6 @@ nsSchemaLoader::ProcessComplexTypeBody(nsSchema* aSchema,
 
   *aContentModel = nsISchemaComplexType::CONTENT_MODEL_EMPTY;
 
-  nsCOMPtr<nsISchemaType> baseType;
   nsCOMPtr<nsISchemaModelGroup> modelGroup;
   
   while (NS_SUCCEEDED(iterator.GetNextChild(getter_AddRefs(childElement),
@@ -1343,6 +1541,35 @@ nsSchemaLoader::ProcessComplexTypeBody(nsSchema* aSchema,
       if (NS_FAILED(rv)) {
         return rv;
       }
+
+      // XXX WSDL ugliness making itself into schemas. Hopefully this
+      // mechanism for specifying an array type in schemas will die
+      // when the Schema WG address qualified names in attribute
+      // default values.
+      if (tagName == nsSchemaAtoms::sAttribute_atom) {
+#define NS_WSDL_NAMESPACE "http://schemas.xmlsoap.org/wsdl/"
+        nsAutoString arrayType;
+        childElement->GetAttributeNS(NS_LITERAL_STRING(NS_WSDL_NAMESPACE),
+                                     NS_LITERAL_STRING("arrayType"), 
+                                     arrayType);
+        if (!arrayType.IsEmpty()) {
+          nsCOMPtr<nsISchemaType> arraySchemaType;
+          PRUint32 arrayDimension;
+          rv = ParseArrayType(aSchema, 
+                              childElement,
+                              arrayType, 
+                              getter_AddRefs(arraySchemaType),
+                              &arrayDimension);
+          if (NS_FAILED(rv)) {
+            return rv;
+          }
+
+          rv = aComplexType->SetArrayInfo(arraySchemaType, arrayDimension);
+          if (NS_FAILED(rv)) {
+            return rv;
+          }
+        }
+      }
     }
   }
 
@@ -1807,7 +2034,7 @@ nsSchemaLoader::ProcessSimpleType(nsSchema* aSchema,
 nsresult 
 nsSchemaLoader::ProcessSimpleTypeRestriction(nsSchema* aSchema, 
                                              nsIDOMElement* aElement,
-                                             const nsAReadableString& aName,
+                                             const nsAString& aName,
                                              nsISchemaSimpleType** aSimpleType)
 {
   nsresult rv = NS_OK;
@@ -1899,7 +2126,7 @@ nsSchemaLoader::ProcessSimpleTypeRestriction(nsSchema* aSchema,
 nsresult 
 nsSchemaLoader::ProcessSimpleTypeList(nsSchema* aSchema, 
                                       nsIDOMElement* aElement,
-                                      const nsAReadableString& aName,
+                                      const nsAString& aName,
                                       nsISchemaSimpleType** aSimpleType)
 {
   nsresult rv = NS_OK;
@@ -1962,7 +2189,7 @@ nsSchemaLoader::ProcessSimpleTypeList(nsSchema* aSchema,
 nsresult 
 nsSchemaLoader::ProcessSimpleTypeUnion(nsSchema* aSchema, 
                                        nsIDOMElement* aElement,
-                                       const nsAReadableString& aName,
+                                       const nsAString& aName,
                                        nsISchemaSimpleType** aSimpleType)
 {
   nsresult rv = NS_OK;
@@ -2283,7 +2510,7 @@ nsSchemaLoader::ProcessAttribute(nsSchema* aSchema,
   nsAutoString defaultValue, fixedValue;
   aElement->GetAttribute(NS_LITERAL_STRING("default"), defaultValue);
   aElement->GetAttribute(NS_LITERAL_STRING("fixed"), fixedValue);
-  
+
   PRUint16 use;
   GetUse(aElement, &use);
 
@@ -2481,6 +2708,9 @@ nsSchemaLoader::ProcessFacet(nsSchema* aSchema,
   else if (aTagName == nsSchemaAtoms::sFractionDigits_atom) {
     facetType = nsISchemaFacet::FACET_TYPE_FRACTIONDIGITS;
   }
+  else {
+    return NS_ERROR_UNEXPECTED;
+  }
   facetInst->SetFacetType(facetType);
 
   nsAutoString valueStr;
