@@ -60,7 +60,7 @@ NS_IMPL_ISUPPORTS1(nsImageGTK, nsIImage)
 
 nsImageGTK::nsImageGTK()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mImageBits = nsnull;
   mWidth = 0;
   mHeight = 0;
@@ -434,7 +434,7 @@ void nsImageGTK::UpdateCachedImage()
       if (mIsSpacer && (leftindex <= rightindex)) {
         for (unsigned y=rect->y; (y<bottom) && mIsSpacer; y++) {
           unsigned char *alpha = mAlphaBits + mAlphaRowBytes*y + leftindex;
-          for (unsigned x=left; x<right; x++) {
+          for (unsigned x=leftindex; x<=rightindex; x++) {
             if (*(alpha++)!=0) {
               mIsSpacer = PR_FALSE;
               break;
@@ -842,15 +842,40 @@ nsImageGTK::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
     copyGC = ((nsRenderingContextGTK&)aContext).GetGC();
   }
 
-  gdk_window_copy_area(drawing->GetDrawable(),      // dest window
-                       copyGC,                      // gc
-                       aDX,                         // xdest
-                       aDY,                         // ydest
-                       mImagePixmap,                // source window
-                       aSX,                         // xsrc
-                       aSY,                         // ysrc
-                       aSWidth,                     // width
-                       aSHeight);                   // height
+  nsRegionGTK clipRgn;
+  if (mAlphaPixmap &&
+      NS_SUCCEEDED(((nsRenderingContextGTK&)aContext).CopyClipRegion(clipRgn))) {
+    // we have both a set of rectangles and a bitmap defining the clip
+    // let X11 clip to the bitmap, do the rectangles by hand
+    nsRegionRectSet *rectSet = nsnull;
+    clipRgn.Intersect(aDX, aDY, aSWidth, aSHeight);
+    if (NS_SUCCEEDED(clipRgn.GetRects(&rectSet))) {
+      for (PRUint32 i=0; i<rectSet->mRectsLen; i++) {
+        nsRegionRect *rect = &(rectSet->mRects[i]);
+        gdk_window_copy_area(drawing->GetDrawable(),      // dest window
+                             copyGC,                      // gc
+                             rect->x,                     // xdest
+                             rect->y,                     // ydest
+                             mImagePixmap,                // source window
+                             aSX+(rect->x-aDX),           // xsrc
+                             aSY+(rect->y-aDY),           // ysrc
+                             rect->width,                 // width
+                             rect->height);               // height
+      }
+      clipRgn.FreeRects(rectSet);
+    }
+  } else {
+    // normal case - let X11 take care of all the clipping
+    gdk_window_copy_area(drawing->GetDrawable(),      // dest window
+                         copyGC,                      // gc
+                         aDX,                         // xdest
+                         aDY,                         // ydest
+                         mImagePixmap,                // source window
+                         aSX,                         // xsrc
+                         aSY,                         // ysrc
+                         aSWidth,                     // width
+                         aSHeight);                   // height
+  }
  
   gdk_gc_unref(copyGC);
   mFlags = 0;
@@ -1049,8 +1074,6 @@ nsImageGTK::DrawCompositedGeneral(PRBool isLSB, PRBool flipBytes,
   GdkVisual *visual     = gdk_rgb_get_visual();
   GdkColormap *colormap = gdk_rgb_get_cmap();
 
-  unsigned char *target = readData;
-
   // flip bytes
   if (flipBytes && (ximage->bits_per_pixel>=16)) {
     for (int row=0; row<ximage->height; row++) {
@@ -1101,9 +1124,10 @@ nsImageGTK::DrawCompositedGeneral(PRBool isLSB, PRBool flipBytes,
   greenFill = 0xff>>visual->green_prec;
   blueFill =  0xff>>visual->blue_prec;
 
-  for (int row=0; row<ximage->height; row++) {
+  for (int row=0; row<height; row++) {
     unsigned char *ptr = srcData + row*ximage->bytes_per_line;
-    for (int col=0; col<ximage->width; col++) {
+    unsigned char *target = readData+3*row*ximage->width;
+    for (int col=0; col<width; col++) {
       unsigned pix;
       switch (ximage->bits_per_pixel) {
       case 1:
@@ -1168,7 +1192,7 @@ nsImageGTK::DrawCompositedGeneral(PRBool isLSB, PRBool flipBytes,
 
   // now composite
   for (unsigned y=0; y<height; y++) {
-    unsigned char *targetRow = readData+3*y*width;
+    unsigned char *targetRow = readData+3*y*ximage->width;
     unsigned char *imageRow  = imageOrigin + y*imageStride;
     unsigned char *alphaRow  = alphaOrigin + y*alphaStride;
     
