@@ -46,6 +46,7 @@
 #include "nsIDocument.h"
 #include "nsIContent.h"
 #include "nsIEventStateManager.h"
+#include "nsIViewManager.h"
 #include "nsINameSpaceManager.h"
 #include "nsILookAndFeel.h"
 #include "nsIDeviceContext.h"
@@ -70,6 +71,18 @@ GtkWidget* gProgressWidget;
 GtkWidget* gTabWidget;
 GtkTooltips* gTooltipWidget;
 
+static int gLastXError;
+const char* nsNativeThemeGTK::sDisabledEngines[] = {
+  "xeno",   // xenophilia
+  nsnull
+};
+
+typedef struct GtkThemeEnginePrivate {
+  GtkThemeEngine engine;
+  void* library;
+  char* name;
+} GtkThemeEnginePrivate;
+
 nsNativeThemeGTK::nsNativeThemeGTK()
   : mProtoLayout(nsnull)
 {
@@ -81,6 +94,29 @@ nsNativeThemeGTK::nsNativeThemeGTK()
   mInputAtom = do_GetAtom("input");
   mFocusedAtom = do_GetAtom("focused");
   mFirstTabAtom = do_GetAtom("first-tab");
+
+  memset(mDisabledWidgetTypes, 0, sizeof(mDisabledWidgetTypes));
+
+  // Check for a blacklisted theme engine.  These are known to crash.
+  // If we're using a blacklisted engine, we need to disable all widget types.
+
+  EnsureButtonWidget();  // arbitrary, but we need a widget's style
+  GtkThemeEngine* gtkThemeEng = gButtonWidget->style->engine;
+  if (!gtkThemeEng)  // gtk's built-in drawing routines are safe
+    return;
+
+  const char* curEngine = ((GtkThemeEnginePrivate*) gtkThemeEng)->name;
+  const char* eng;
+  int i = 0;
+  while ((eng = sDisabledEngines[i++])) {
+    if (!strcmp(eng, curEngine)) {
+#ifdef DEBUG
+      printf("Disabling GTK themed widgets due to blacklisted theme engine: %s\n", eng);
+#endif
+      memset(mDisabledWidgetTypes, 0xff, sizeof(mDisabledWidgetTypes));
+      break;
+    }
+  }
 }
 
 nsNativeThemeGTK::~nsNativeThemeGTK() {
@@ -89,6 +125,7 @@ nsNativeThemeGTK::~nsNativeThemeGTK() {
     gtk_widget_destroy(gProtoWindow);
   if (gTooltipWidget)
     gtk_object_unref(GTK_OBJECT(gTooltipWidget));
+  moz_gtk_shutdown();
 }
 
 static void GetPrimaryPresShell(nsIFrame* aFrame, nsIPresShell** aResult)
@@ -106,6 +143,22 @@ static void GetPrimaryPresShell(nsIFrame* aFrame, nsIPresShell** aResult)
     doc->GetShellAt(0, aResult); // Addref happens here.
 }
 
+static void RefreshWidgetWindow(nsIFrame* aFrame)
+{
+  nsCOMPtr<nsIPresShell> shell;
+
+  GetPrimaryPresShell(aFrame, getter_AddRefs(shell));
+  if (!shell)
+    return;
+
+  nsCOMPtr<nsIViewManager> vm;
+  shell->GetViewManager(getter_AddRefs(vm));
+  if (!vm)
+    return;
+ 
+  vm->UpdateAllViews(NS_VMREFRESH_NO_SYNC);
+}
+
 static PRInt32 GetContentState(nsIFrame* aFrame)
 {
   if (!aFrame)
@@ -172,6 +225,14 @@ GetSystemFont(PRUint8 aWidgetType, nsSystemFontID& aFont)
   return NS_ERROR_FAILURE;
 }
 
+static PRBool IsWidgetTypeDisabled(PRUint8* aDisabledVector, PRUint8 aWidgetType) {
+  return aDisabledVector[aWidgetType >> 3] & (1 << (aWidgetType & 7));
+}
+
+static void SetWidgetTypeDisabled(PRUint8* aDisabledVector, PRUint8 aWidgetType) {
+  aDisabledVector[aWidgetType >> 3] |= (1 << (aWidgetType & 7));
+}
+
 void
 nsNativeThemeGTK::GetGtkWidgetState(PRUint8 aWidgetType,
                                     nsIFrame* aFrame, GtkWidgetState* aState)
@@ -198,6 +259,12 @@ nsNativeThemeGTK::GetGtkWidgetState(PRUint8 aWidgetType,
   }
 }
 
+static int
+NativeThemeErrorHandler(Display* dpy, XErrorEvent* error) {
+  gLastXError = error->error_code;
+  return 0;
+}
+
 NS_IMETHODIMP
 nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
                                        nsIFrame* aFrame,
@@ -221,12 +288,22 @@ nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
   GtkWidgetState state;
   GetGtkWidgetState(aWidgetType, aFrame, &state);
 
+  NS_ASSERTION(!IsWidgetTypeDisabled(mDisabledWidgetTypes, aWidgetType), "Trying to render an unsafe widget!");
+
+  // The widget code on Solaris replaces gdk's X error handler with its own,
+  // which means that gdk_error_trap_push/pop won't have any effect.
+  // So, instead of using those, we just use our own error handler which
+  // records the last X error to occur, then restore the old error handler
+  // when we're done drawing.
+
+  gLastXError = 0;
+  XErrorHandler oldHandler = XSetErrorHandler(NativeThemeErrorHandler);
+
   switch (aWidgetType) {
     
   case NS_THEME_BUTTON:
   case NS_THEME_TOOLBAR_BUTTON:
     {
-      EnsureButtonWidget();
       GtkReliefStyle relief = (aWidgetType == NS_THEME_BUTTON) ? GTK_RELIEF_NORMAL : GTK_RELIEF_NONE;
       moz_gtk_button_paint(window, gButtonWidget->style, &gdk_rect, &gdk_clip,
                            &state, relief );
@@ -383,6 +460,20 @@ nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
     }
   }
 
+  gdk_flush();
+  XSetErrorHandler(oldHandler);
+  if (gLastXError) {
+#ifdef DEBUG
+    printf("GTK theme failed for widget type %d, error was %d, state was [active=%d,focused=%d,inHover=%d,disabled=%d]\n",
+           aWidgetType, gLastXError, state.active, state.focused, state.inHover, state.disabled);
+#endif
+    NS_WARNING("GTK theme failed; disabling unsafe widget");
+    SetWidgetTypeDisabled(mDisabledWidgetTypes, aWidgetType);
+    // force refresh of the window, because the widget was not successfully drawn
+    // it must be redrawn using the default look
+    RefreshWidgetWindow(aFrame);
+  }
+
   return NS_OK;
 }
 
@@ -416,7 +507,6 @@ nsNativeThemeGTK::GetWidgetBorder(nsIDeviceContext* aContext, nsIFrame* aFrame,
     break;
   case NS_THEME_BUTTON:
   case NS_THEME_TOOLBAR_BUTTON:
-    EnsureButtonWidget();
     WidgetBorderToMargin(gButtonWidget, aResult);
     break;
   case NS_THEME_TOOLBAR_GRIPPER:
@@ -518,10 +608,13 @@ nsNativeThemeGTK::GetMinimumWidgetSize(nsIRenderingContext* aContext,
       EnsureCheckBoxWidget();
 
       gint indicator_size, indicator_spacing;
-      _gtk_check_button_get_props (GTK_CHECK_BUTTON(gCheckboxWidget), &indicator_size,
-                                   &indicator_spacing);
+      moz_gtk_checkbox_get_metrics(&indicator_size, &indicator_spacing);
 
-      aResult->width = aResult->height = indicator_size;
+      // Hack alert: several themes have indicators larger than the default
+      // 10px size, but don't set the indicator size property.  So, leave
+      // a little slop room by making the minimum size 14px.
+
+      aResult->width = aResult->height = MAX(indicator_size, 14);
       *aIsOverridable = PR_FALSE;
     }
     break;
@@ -569,13 +662,27 @@ nsNativeThemeGTK::WidgetStateChanged(nsIFrame* aFrame, PRUint8 aWidgetType,
 NS_IMETHODIMP
 nsNativeThemeGTK::ThemeChanged()
 {
+  memset(mDisabledWidgetTypes, 0, sizeof(mDisabledWidgetTypes));
   return NS_OK;
 }
 
 NS_IMETHODIMP_(PRBool)
 nsNativeThemeGTK::ThemeSupportsWidget(nsIPresContext* aPresContext,
+                                      nsIFrame* aFrame,
                                       PRUint8 aWidgetType)
 {
+  // Check for specific widgets to see if HTML has overridden the style.
+  if (aFrame) {
+    // For now don't support HTML.
+    nsCOMPtr<nsIContent> content;
+    aFrame->GetContent(getter_AddRefs(content));
+    if (content->IsContentOfType(nsIContent::eHTML))
+      return PR_FALSE;
+  }
+
+  if (IsWidgetTypeDisabled(mDisabledWidgetTypes, aWidgetType))
+    return PR_FALSE;
+
   switch (aWidgetType) {
   case NS_THEME_BUTTON:
   case NS_THEME_RADIO:
