@@ -64,7 +64,7 @@ static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
 nsFontMetricsPh::nsFontMetricsPh()
 {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 	mDeviceContext = nsnull;
 	mFont = nsnull;
 
@@ -262,7 +262,9 @@ NS_IMETHODIMP nsFontMetricsPh::Init ( const nsFont& aFont, nsIAtom* aLangGroup,
 	mAscent = mMaxAscent = NSToCoordRound(node->ascender * dev2app * -1.0);
 	mDescent = mMaxDescent = NSToCoordRound(node->descender * dev2app);
 	mMaxAdvance = NSToCoordRound(node->width * dev2app);
-	mAveCharWidth = PR_MAX(1, NSToCoordRound(mSpaceWidth * dev2app));
+
+	PfExtentText(&extent, NULL, NSFullFontName, "x", 1);
+	mAveCharWidth = (int) ((extent.lr.x - extent.ul.x + 1) * dev2app);
 
 	mXHeight = NSToCoordRound((float)node->ascender * dev2app * 0.56f * -1.0); // 56% of ascent, best guess for non-true type
 	mSuperscriptOffset = mXHeight;     // XXX temporary code!
@@ -407,6 +409,12 @@ NS_IMETHODIMP nsFontMetricsPh :: GetMaxAdvance( nscoord &aAdvance )
 	return NS_OK;
 }
 
+NS_IMETHODIMP nsFontMetricsPh :: GetAveCharWidth( nscoord &aAveCharWidth)
+{
+	aAveCharWidth = mAveCharWidth;
+	return NS_OK;
+}
+
 NS_IMETHODIMP nsFontMetricsPh :: GetFont(const nsFont *&aFont)
 {
 	aFont = mFont;
@@ -444,7 +452,7 @@ struct nsFontFamily
 // The Font Enumerator
 nsFontEnumeratorPh::nsFontEnumeratorPh()
 {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 }
 
 NS_IMPL_ISUPPORTS1(nsFontEnumeratorPh, nsIFontEnumerator)
