@@ -38,7 +38,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "xp_core.h"
 #include "nsQuickSort.h"
 #include "nsFontMetricsQT.h"
 #include "nsFont.h"
@@ -591,7 +590,7 @@ static void ParseFontPref(nsCAutoString &aPrefValue,QString &aFontName,
  
     /* Try both foundry-name and name */
     start = aPrefValue.FindChar('-');
-    end = aPrefValue.FindChar('-',PR_FALSE,start + 1);
+    end = aPrefValue.FindChar('-',start + 1);
     aPrefValue.Left(fontFoundryName,end);
     aPrefValue.Right(fontCharSetName,len - end - 1);
     aPrefValue.Mid(fontName,start + 1,end - start - 1);
@@ -635,7 +634,7 @@ nsFontMetricsQT::nsFontMetricsQT()
   mID = gFontMetricsID++;
   printf("JCG: nsFontMetricsQT CTOR (%p) ID: %d, Count: %d\n",this,mID,gFontMetricsCount);
 #endif
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   gFontMetricsQTCount++;
   mDeviceContext = nsnull;
   mFont = nsnull;
@@ -645,6 +644,7 @@ nsFontMetricsQT::nsFontMetricsQT()
   mMaxAscent = 0;
   mMaxDescent = 0;
   mMaxAdvance = 0;
+  mAveCharWidth = 0;
   mXHeight = 0;
   mSuperscriptOffset = 0;
   mSubscriptOffset = 0;
@@ -780,11 +780,9 @@ NS_IMETHODIMP nsFontMetricsQT::Init(const nsFont &aFont,nsIAtom *aLangGroup,
   mDeviceContext = aContext;
 
   float a2d;
-  float textZoom = 1.0;
  
   aContext->GetAppUnitsToDevUnits(a2d);
-  aContext->GetTextZoom(textZoom);
-  mPixelSize = NSToIntRound(a2d * textZoom * mFont->size); 
+  mPixelSize = NSToIntRound(a2d * mFont->size); 
   if (mFont->style == NS_FONT_STYLE_NORMAL) {
 #if (QT_VERSION >= 230)
     if (use_qt_xft())
@@ -916,6 +914,8 @@ void nsFontMetricsQT::RealizeFont()
   mEmHeight = nscoord(fm.height() * f);
   mMaxAdvance = nscoord(fm.maxWidth() * f);
 
+  mAveCharWidth = nscoord(fm.width(QChar('x')) * f);
+
   mEmAscent = mMaxAscent;  
   mEmDescent = mMaxDescent;
  
@@ -1028,6 +1028,12 @@ NS_IMETHODIMP nsFontMetricsQT::GetMaxAdvance(nscoord &aAdvance)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsFontMetricsQT::GetAveCharWidth(nscoord &aAveCharWidth)
+{
+  aAveCharWidth = mAveCharWidth;
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsFontMetricsQT::GetSpaceWidth(nscoord &aSpaceWidth)
 {
   aSpaceWidth = mSpaceWidth;
@@ -2017,7 +2023,7 @@ nsFontQTUserDefined::GetBoundingMetrics(const PRUnichar *aString,
 // The Font Enumerator
 nsFontEnumeratorQT::nsFontEnumeratorQT()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
  
 NS_IMPL_ISUPPORTS1(nsFontEnumeratorQT, nsIFontEnumerator)
