@@ -49,6 +49,10 @@
 
 #include "nsString.h"
 
+#if defined(DEBUG_rods) && defined(MOZ_LAYOUTDEBUG)
+#include "nsIDebugObject.h"
+#endif
+
 #define DOC_TITLE_LENGTH      64
 
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
@@ -58,6 +62,14 @@ PRUint32 nsDeviceContextWin::sNumberOfScreens = 0;
 
 static char* nav4rounding = "font.size.nav4rounding";
 
+#include "prlog.h"
+#ifdef PR_LOGGING 
+PRLogModuleInfo * kGfxPrintingLogMod = PR_NewLogModule("printing-gfx");
+#define PR_PL(_p1)  PR_LOG(kGfxPrintingLogMod, PR_LOG_DEBUG, _p1)
+#else
+#define PR_PL(_p1)
+#endif
+
 nsDeviceContextWin :: nsDeviceContextWin()
   : DeviceContextImpl()
 {
@@ -407,6 +419,13 @@ nsresult nsDeviceContextWin::CopyLogFontToNSFont(HDC* aHDC, const LOGFONT* ptrLo
   LONG logHeight = LONG((float(ptrLogFont->lfHeight) * mPixelScale) + (ptrLogFont->lfHeight < 0 ? -0.5 : 0.5)); // round up
   int pointSize = -MulDiv(logHeight, 72, ::GetDeviceCaps(*aHDC, LOGPIXELSY));
 
+  // we have problem on Simplified Chinese system because the system report
+  // the default font size is 8. but if we use 8, the text display very
+  // Ugly. force it to be at 9 on that system (cp936), but leave other sizes alone.
+  if ((pointSize == 8) && 
+      (936 == ::GetACP())) 
+    pointSize = 9;
+
   aFont->size = NSIntPointsToTwips(pointSize);
   return NS_OK;
 }
@@ -498,15 +517,6 @@ nsresult nsDeviceContextWin :: GetSysFontInfo(HDC aHDC, nsSystemFontID anID, nsF
     return NS_ERROR_FAILURE;
   }
 
-  // we have problem on Simplified Chinese system because the system report
-  // the default font size is 8. but if we use 8, the text display very
-  // Ugly. force it to be at least 9 on that system (cp936)
-
-  if ((-9 < ptrLogFont->lfHeight) && 
-      (ptrLogFont->lfHeight < 9) && 
-      (936 == ::GetACP())) 
-    ptrLogFont->lfHeight = (ptrLogFont->lfHeight > 0) ? 9 : -9;
-
   return CopyLogFontToNSFont(&aHDC, ptrLogFont, aFont);
 }
 
@@ -731,7 +741,6 @@ NS_IMETHODIMP nsDeviceContextWin :: GetDeviceContextFor(nsIDeviceContextSpec *aD
   if (devmode) {
     dc = ::CreateDC(drivername, devicename, NULL, devmode);
   }
-  devSpecWin->UnlockDevMode();
 
   return devConWin->Init(dc, this); // take ownership of the DC
 }
@@ -785,7 +794,30 @@ NS_IMETHODIMP nsDeviceContextWin :: BeginDocument(PRUnichar * aTitle, PRUnichar*
     docinfo.lpszDocName = title != nsnull?title:"Mozilla Document";
 
 #ifdef DEBUG_rods
-    docinfo.lpszOutput = "p.ps";
+    docinfo.lpszOutput = "\\p.ps";
+
+#ifdef MOZ_LAYOUTDEBUG
+    // This is for overriding the above when doing the runtime checking
+    char * tempFileName = nsnull;
+    nsCOMPtr<nsIDebugObject>debugObj = do_GetService("@mozilla.org/debug/debugobject;1");
+    if (debugObj) {
+      PRBool isDoingTests;
+      if (NS_SUCCEEDED(debugObj->GetDoRuntimeTests(&isDoingTests)) && isDoingTests) {
+        PRUnichar * name;
+        debugObj->GetPrintFileName(&name);
+        if (name) {
+          if (*name) {
+            nsCString cStrName;
+            cStrName.AssignWithConversion(name);
+            tempFileName = ToNewCString(cStrName);
+          }
+          nsMemory::Free(name);
+        }
+        docinfo.lpszOutput = tempFileName;
+      }
+    }
+#endif
+
 #else
     docinfo.lpszOutput = docName;
 #endif
@@ -798,10 +830,17 @@ NS_IMETHODIMP nsDeviceContextWin :: BeginDocument(PRUnichar * aTitle, PRUnichar*
     } else {
       DISPLAY_LAST_ERROR
       rv = NS_ERROR_GFX_PRINTER_STARTDOC;
+      PR_PL(("nsDeviceContextWin::BeginDocument - StartDoc Error!\n"));
     }
 
     if (title != nsnull) delete [] title;
     if (docName != nsnull) nsMemory::Free(docName);
+
+#if defined(DEBUG_rods) && defined(MOZ_LAYOUTDEBUG)
+    if (tempFileName) {
+      nsMemory::Free(tempFileName);
+    }
+#endif
   }
 
   return rv;
@@ -815,6 +854,7 @@ NS_IMETHODIMP nsDeviceContextWin :: EndDocument(void)
       return NS_OK;
     } else {
       DISPLAY_LAST_ERROR
+      PR_PL(("nsDeviceContextWin::EndDocument - EndDoc Error!\n"));
       return NS_ERROR_GFX_PRINTER_ENDDOC;
     }
   }
@@ -830,6 +870,7 @@ NS_IMETHODIMP nsDeviceContextWin :: AbortDocument(void)
       return NS_OK;
     } else {
       DISPLAY_LAST_ERROR
+      PR_PL(("nsDeviceContextWin::AbortDocument - AbortDoc Error!\n"));
       return NS_ERROR_ABORT;
     }
   }
@@ -845,6 +886,7 @@ NS_IMETHODIMP nsDeviceContextWin :: BeginPage(void)
       return NS_OK;
     else {
       DISPLAY_LAST_ERROR
+      PR_PL(("nsDeviceContextWin::BeginPage - StartPage Error!\n"));
       return NS_ERROR_GFX_PRINTER_STARTPAGE;
     }
   }
@@ -860,6 +902,7 @@ NS_IMETHODIMP nsDeviceContextWin :: EndPage(void)
       return NS_OK;
     } else {
       DISPLAY_LAST_ERROR
+      PR_PL(("nsDeviceContextWin::EndPage - EndPage Error!\n"));
       return NS_ERROR_GFX_PRINTER_ENDPAGE;
     }
   }
