@@ -216,7 +216,7 @@ int            GlobalPrinters::mGlobalNumPrinters = 0;
 nsDeviceContextSpecXlib::nsDeviceContextSpecXlib()
 {
   DO_PR_DEBUG_LOG(("nsDeviceContextSpecXlib::nsDeviceContextSpecXlib()\n"));
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsDeviceContextSpecXlib::~nsDeviceContextSpecXlib()
@@ -546,7 +546,7 @@ nsresult CopyPrinterCharPref(nsIPref *pref, const char *modulename, const char *
 //  Printer Enumerator
 nsPrinterEnumeratorXlib::nsPrinterEnumeratorXlib()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 NS_IMPL_ISUPPORTS1(nsPrinterEnumeratorXlib, nsIPrinterEnumerator)
@@ -673,6 +673,7 @@ NS_IMETHODIMP nsPrinterEnumeratorXlib::InitPrintSettingsFromPrinter(const PRUnic
   DO_PR_DEBUG_LOG(("Setting default filename to '%s'\n", filename.get()));
   aPrintSettings->SetToFileName(NS_ConvertUTF8toUCS2(filename).get());
 
+  aPrintSettings->SetIsInitializedFromPrinter(PR_TRUE);
 #ifdef USE_XPRINT
   if (type == pmXprint) {
     DO_PR_DEBUG_LOG(("InitPrintSettingsFromPrinter() for Xprint printer\n"));
