@@ -36,14 +36,21 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "nsStringBundle.h"
 #include "nsID.h"
 #include "nsFileSpec.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsIPersistentProperties2.h"
 #include "nsIStringBundle.h"
+#include "nsStringBundleService.h"
+#include "nsStringBundle.h"
+#include "nsStringBundleTextOverride.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
+#include "nsArray.h"
+#include "nsArrayEnumerator.h"
 #include "nscore.h"
+#include "nsHashtable.h"
 #include "nsMemory.h"
 #include "plstr.h"
 #include "nsNetUtil.h"
@@ -51,18 +58,11 @@
 #include "nsIComponentManager.h"
 #include "nsIGenericFactory.h"
 #include "nsIMemory.h"
-#include "nsIObserver.h"
 #include "nsIObserverService.h"
-#include "nsWeakReference.h"
-#include "nsCOMPtr.h"
 #include "pratom.h"
-#include "prclist.h"
-#include "plarena.h"
-#include "prlog.h"              // for PR_ASSERT
 #include "prmem.h"
 #include "nsIModule.h"
-#include "nsISupportsArray.h"
-#include "nsHashtable.h"
+#include "nsCOMArray.h"
 #include "nsAutoLock.h"
 #include "nsTextFormatter.h"
 #include "nsIErrorService.h"
@@ -82,59 +82,20 @@
 #include "prenv.h"
 
 static NS_DEFINE_CID(kErrorServiceCID, NS_ERRORSERVICE_CID);
-
-// XXX investigate need for proper locking in this module
-//static PRInt32 gLockCount = 0;
-
-NS_DEFINE_IID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
-
-class nsStringBundle : public nsIStringBundle
-#ifdef ASYNC_LOADING
-                     , public nsIStreamLoaderObserver
-#endif
-{
-public:
-  // init version
-  nsStringBundle(const char* aURLSpec);
-  nsresult LoadProperties();
-  virtual ~nsStringBundle();
-
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSISTRINGBUNDLE
-#ifdef ASYNC_LOADING
-  NS_DECL_NSISTREAMLOADEROBSERVER
-#endif
-
-  nsCOMPtr<nsIPersistentProperties> mProps;
-
-protected:
-  //
-  // functional decomposition of the funitions repeatively called 
-  //
-  nsresult GetStringFromID(PRInt32 aID, nsAString& aResult);
-  nsresult GetStringFromName(const nsAString& aName, nsAString& aResult);
-
-private:
-  nsCString              mPropertiesURL;
-  PRPackedBool                 mAttemptedLoad;
-  PRPackedBool                 mLoaded;
-
-public:
-  static nsresult FormatString(const PRUnichar *formatStr,
-                               const PRUnichar **aParams, PRUint32 aLength,
-                               PRUnichar **aResult);
- };
+static NS_DEFINE_CID(kPersistentPropertiesCID, NS_IPERSISTENTPROPERTIES_CID);
 
 nsStringBundle::~nsStringBundle()
 {
 }
 
-nsStringBundle::nsStringBundle(const char* aURLSpec) :
+nsStringBundle::nsStringBundle(const char* aURLSpec,
+                               nsIStringBundleOverride* aOverrideStrings) :
   mPropertiesURL(aURLSpec),
+  mOverrideStrings(aOverrideStrings),
   mAttemptedLoad(PR_FALSE),
   mLoaded(PR_FALSE)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsresult
@@ -155,150 +116,50 @@ nsStringBundle::LoadProperties()
 
   nsresult rv;
 
-  // to be turned on once we figure out how to ensure we're loading on
-  // the main thread
-#ifdef ASYNC_LOADING
-
-  // load the string bundle asynchronously on another thread,
-  // but make sure we block this thread so that LoadProperties() is
-  // synchronous
-  
-  // create an event queue for this thread.
-  nsCOMPtr<nsIEventQueueService> service = do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
-  
-  nsCOMPtr<nsIEventQueue> currentThreadQ;
-  rv = service->PushThreadEventQueue(getter_AddRefs(currentThreadQ));
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
-
+  // do it synchronously
   nsCOMPtr<nsIURI> uri;
   rv = NS_NewURI(getter_AddRefs(uri), mPropertiesURL);
   if (NS_FAILED(rv)) return rv;
 
-  //   create and loader, then wait
-  nsCOMPtr<nsIStreamLoader> loader;
-  rv = NS_NewStreamLoader(getter_AddRefs(loader), 
-                          uri, 
-                          this /*the load observer*/);
- NS_ASSERTION(NS_SUCCEEDED(rv), "\n-->nsStringBundle::Init: NS_NewStreamLoader failed...\n");
-
- // process events until we're finished.
- PLEvent *event;
- while (!mLoaded) {
-   rv = currentThreadQ->WaitForEvent(&event);
-   NS_ASSERTION(NS_SUCCEEDED(rv), "\n-->nsStringBundle::Init: currentThreadQ->WaitForEvent failed...\n");
-   if (NS_FAILED(rv)) break;
-
-   rv = currentThreadQ->HandleEvent(event);
-   NS_ASSERTION(NS_SUCCEEDED(rv), "\n-->nsStringBundle::Init: currentThreadQ->HandleEvent failed...\n");
-   if (NS_FAILED(rv)) break;
- }
-  
- rv = service->PopThreadEventQueue(currentThreadQ);
-
-#else
-
- // do it synchronously
- nsCOMPtr<nsIURI> uri;
- rv = NS_NewURI(getter_AddRefs(uri), mPropertiesURL);
- if (NS_FAILED(rv)) return rv;
-
- nsCOMPtr<nsIInputStream> in;
- rv = NS_OpenURI(getter_AddRefs(in), uri);
- if (NS_FAILED(rv)) return rv;
-    
- NS_TIMELINE_MARK_FUNCTION("loading properties");
-
- NS_ASSERTION(NS_SUCCEEDED(rv) && in, "Error in OpenBlockingStream");
- NS_ENSURE_TRUE(NS_SUCCEEDED(rv) && in, NS_ERROR_FAILURE);
-    
- mProps = do_CreateInstance(kPersistentPropertiesCID, &rv);
- NS_ENSURE_SUCCESS(rv, rv);
-    
- mAttemptedLoad = mLoaded = PR_TRUE;
- rv = mProps->Load(in);
-
- mLoaded = NS_SUCCEEDED(rv);
-#endif
-  
-  return rv;
-}
-
-#ifdef ASYNC_LOADING
-NS_IMETHODIMP
-nsStringBundle::OnStreamComplete(nsIStreamLoader* aLoader,
-                                 nsISupports* context,
-                                 nsresult aStatus,
-                                 PRUint32 stringLen,
-                                 const char* string)
-{
-
-  nsXPIDLCString uriSpec;
-  if (NS_FAILED(aStatus)) {
-  // print a load error on bad status
-#if defined(DEBUG)
-    if (aLoader) {
-      nsCOMPtr<nsIRequest> request;
-      aLoader->GetRequest(getter_AddRefs(request));
-      nsCOMPtr<nsIChannel> channel(do_QueryInterface(request));
-
-      if (channel) {
-        nsCOMPtr<nsIURI> uri;
-        channel->GetURI(getter_AddRefs(uri));
-        if (uri) {
-            uri->GetSpec(getter_Copies(uriSpec));
-            printf("\n -->nsStringBundle::OnStreamComplete: Failed to load %s\n", 
-                   uriSpec ? (const char *) uriSpec : "");
-        }
-      }
-    }
-#endif
-    return aStatus;
-  }
-
-  nsresult rv;
-  nsCOMPtr<nsISupports> stringStreamSupports;
-  rv = NS_NewByteInputStream(getter_AddRefs(stringStreamSupports), 
-                             string, stringLen);
-  if (NS_FAILED(rv)) return rv;
-      
-  nsCOMPtr<nsIInputStream> in = do_QueryInterface(stringStreamSupports, &rv);
+  nsCOMPtr<nsIInputStream> in;
+  rv = NS_OpenURI(getter_AddRefs(in), uri);
   if (NS_FAILED(rv)) return rv;
+    
+  NS_TIMELINE_MARK_FUNCTION("loading properties");
 
+  NS_ASSERTION(NS_SUCCEEDED(rv) && in, "Error in OpenBlockingStream");
+  NS_ENSURE_TRUE(NS_SUCCEEDED(rv) && in, NS_ERROR_FAILURE);
+    
   mProps = do_CreateInstance(kPersistentPropertiesCID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   
-  // load the stream
+  mAttemptedLoad = mLoaded = PR_TRUE;
   rv = mProps->Load(in);
+
+  mLoaded = NS_SUCCEEDED(rv);
   
-  // 
-  // notify
-  if (NS_SUCCEEDED(rv)) {
-    mLoaded = PR_TRUE;
-
-    // observer notification
-    nsCOMPtr<nsIObserverService> os = do_GetService("@mozilla.org/observer-service;1");
-    if (os)
-      (void) os->NotifyObservers((nsIStringBundle *) this, 
-                        NS_STRBUNDLE_LOADED_TOPIC, 
-                        nsnull);
-
-#if defined(DEBUG_tao)
-  printf("\n --> nsStringBundle::OnStreamComplete: end sending NOTIFICATIONS!!\n");
-#endif
-  }
   return rv;
 }
 
-#endif
 
 nsresult
 nsStringBundle::GetStringFromID(PRInt32 aID, nsAString& aResult)
 {  
   nsAutoCMonitor(this);
-  nsAutoString name;
+  nsCAutoString name;
   name.AppendInt(aID, 10);
-  nsresult rv = mProps->GetStringProperty(name, aResult);
+
+  nsresult rv;
+  
+  // try override first
+  if (mOverrideStrings) {
+    rv = mOverrideStrings->GetStringFromName(mPropertiesURL,
+                                             name,
+                                             aResult);
+    if (NS_SUCCEEDED(rv)) return rv;
+  }
+  
+  rv = mProps->GetStringProperty(name, aResult);
 
 #ifdef DEBUG_tao_
   char *s = ToNewCString(aResult);
@@ -316,7 +177,15 @@ nsStringBundle::GetStringFromName(const nsAString& aName,
 {
   nsresult rv;
 
-  rv = mProps->GetStringProperty(aName, aResult);
+  // try override first
+  if (mOverrideStrings) {
+    rv = mOverrideStrings->GetStringFromName(mPropertiesURL,
+                                             NS_ConvertUCS2toUTF8(aName),
+                                             aResult);
+    if (NS_SUCCEEDED(rv)) return rv;
+  }
+  
+  rv = mProps->GetStringProperty(NS_ConvertUCS2toUTF8(aName), aResult);
 #ifdef DEBUG_tao_
   char *s = ToNewCString(aResult),
        *ss = ToNewCString(aName);
@@ -358,13 +227,8 @@ nsStringBundle::FormatStringFromName(const PRUnichar *aName,
 }
                                      
 
-#ifdef ASYNC_LOADING
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsStringBundle, 
-                              nsIStringBundle, nsIStreamLoaderObserver)
-#else
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsStringBundle,
                               nsIStringBundle)
-#endif
 
 /* void GetStringFromID (in long aID, out wstring aResult); */
 NS_IMETHODIMP
@@ -401,6 +265,67 @@ nsStringBundle::GetStringFromName(const PRUnichar *aName, PRUnichar **aResult)
   return rv;
 }
 
+nsresult
+nsStringBundle::GetCombinedEnumeration(nsIStringBundleOverride* aOverrideStrings,
+                                       nsISimpleEnumerator** aResult)
+{
+  nsCOMPtr<nsISupports> supports;
+  nsCOMPtr<nsIPropertyElement> propElement;
+  
+  nsresult rv;
+
+  nsCOMPtr<nsIMutableArray> resultArray;
+  NS_NewArray(getter_AddRefs(resultArray));
+
+  // first, append the override elements
+  nsCOMPtr<nsISimpleEnumerator> overrideEnumerator;
+  rv = aOverrideStrings->EnumerateKeysInBundle(mPropertiesURL,
+                                               getter_AddRefs(overrideEnumerator));
+  
+  PRBool hasMore;
+  overrideEnumerator->HasMoreElements(&hasMore);
+  while (hasMore) {
+
+    rv = overrideEnumerator->GetNext(getter_AddRefs(supports));
+    if (NS_SUCCEEDED(rv))
+      resultArray->AppendElement(supports, PR_FALSE);
+
+    overrideEnumerator->HasMoreElements(&hasMore);
+  }
+
+  // ok, now we have the override elements in resultArray
+  nsCOMPtr<nsISimpleEnumerator> propEnumerator;
+  rv = mProps->Enumerate(getter_AddRefs(propEnumerator));
+  if (NS_FAILED(rv)) {
+    // no elements in mProps anyway, just return what we have
+    return NS_NewArrayEnumerator(aResult, resultArray);
+  }
+
+  // second, append all the elements that are in mProps
+  do {
+    rv = propEnumerator->GetNext(getter_AddRefs(supports));
+    if (NS_SUCCEEDED(rv) &&
+        (propElement = do_QueryInterface(supports, &rv))) {
+
+      // now check if its in the override bundle
+      nsCAutoString key;
+      propElement->GetKey(key);
+
+      nsAutoString value;
+      rv = aOverrideStrings->GetStringFromName(mPropertiesURL, key, value);
+
+      // if it isn't there, then it is safe to append
+      if (NS_FAILED(rv))
+        resultArray->AppendElement(propElement, PR_FALSE);
+    }
+
+    propEnumerator->HasMoreElements(&hasMore);
+  } while (hasMore);
+
+  return resultArray->Enumerate(aResult);
+}
+                                
+
 NS_IMETHODIMP
 nsStringBundle::GetSimpleEnumeration(nsISimpleEnumerator** elements)
 {
@@ -411,7 +336,10 @@ nsStringBundle::GetSimpleEnumeration(nsISimpleEnumerator** elements)
   rv = LoadProperties();
   if (NS_FAILED(rv)) return rv;
   
-  return mProps->SimpleEnumerateProperties(elements);
+  if (mOverrideStrings)
+      return GetCombinedEnumeration(mOverrideStrings, elements);
+  
+  return mProps->Enumerate(elements);
 }
 
 nsresult
@@ -443,39 +371,13 @@ nsStringBundle::FormatString(const PRUnichar *aFormatStr,
   return NS_OK;
 }
 
-/**
- * An extesible implementation of the StringBudle interface.
- *
- * @created         28/Dec/1999
- * @author  Catalin Rotaru [CATA]
- */
-class nsExtensibleStringBundle : public nsIStringBundle
-{
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSISTRINGBUNDLE
-
-  nsresult Init(const char * aCategory, nsIStringBundleService *);
-private:
-  
-  nsISupportsArray * mBundle;
-  PRBool             mLoaded;
-
-public:
-
-  nsExtensibleStringBundle();
-  virtual ~nsExtensibleStringBundle();
-};
-
 NS_IMPL_ISUPPORTS1(nsExtensibleStringBundle, nsIStringBundle)
 
 nsExtensibleStringBundle::nsExtensibleStringBundle()
-                                                  :mBundle(NULL)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mLoaded = PR_FALSE;
-
-
 }
 
 nsresult
@@ -492,10 +394,6 @@ nsExtensibleStringBundle::Init(const char * aCategory,
   rv = catman->EnumerateCategory(aCategory, getter_AddRefs(enumerator));
   if (NS_FAILED(rv)) return rv;
 
-  // create the bundles array
-  rv = NS_NewISupportsArray(&mBundle);
-  if (NS_FAILED(rv)) return rv;
-
   PRBool hasMore;
   while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore) {
     nsCOMPtr<nsISupports> supports;
@@ -503,21 +401,21 @@ nsExtensibleStringBundle::Init(const char * aCategory,
     if (NS_FAILED(rv))
       continue;
 
-    nsCOMPtr<nsISupportsString> supStr = do_QueryInterface(supports, &rv);
+    nsCOMPtr<nsISupportsCString> supStr = do_QueryInterface(supports, &rv);
     if (NS_FAILED(rv))
       continue;
 
-    nsXPIDLCString name;
-    rv = supStr->GetData(getter_Copies(name));
+    nsCAutoString name;
+    rv = supStr->GetData(name);
     if (NS_FAILED(rv))
       continue;
 
     nsCOMPtr<nsIStringBundle> bundle;
-    rv = aBundleService->CreateBundle(name, getter_AddRefs(bundle));
+    rv = aBundleService->CreateBundle(name.get(), getter_AddRefs(bundle));
     if (NS_FAILED(rv))
       continue;
 
-    mBundle->AppendElement(bundle);
+    mBundles.AppendObject(bundle);
   }
 
   return rv;
@@ -525,7 +423,6 @@ nsExtensibleStringBundle::Init(const char * aCategory,
 
 nsExtensibleStringBundle::~nsExtensibleStringBundle() 
 {
-  NS_IF_RELEASE(mBundle);
 }
 
 nsresult nsExtensibleStringBundle::GetStringFromID(PRInt32 aID, PRUnichar ** aResult)
@@ -534,13 +431,11 @@ nsresult nsExtensibleStringBundle::GetStringFromID(PRInt32 aID, PRUnichar ** aRe
   
   PRUint32 size, i;
 
-  rv = mBundle->Count(&size);
-  if (NS_FAILED(rv)) return rv;
+  size = mBundles.Count();
 
   for (i = 0; i < size; i++) {
-    nsCOMPtr<nsIStringBundle> bundle;
-    rv = mBundle->QueryElementAt(i, NS_GET_IID(nsIStringBundle), getter_AddRefs(bundle));
-    if (NS_SUCCEEDED(rv)) {
+    nsIStringBundle *bundle = mBundles[i];
+    if (bundle) {
         rv = bundle->GetStringFromID(aID, aResult);
         if (NS_SUCCEEDED(rv))
             return NS_OK;
@@ -556,13 +451,11 @@ nsresult nsExtensibleStringBundle::GetStringFromName(const PRUnichar *aName,
   nsresult res = NS_OK;
   PRUint32 size, i;
 
-  res = mBundle->Count(&size);
-  if (NS_FAILED(res)) return res;
+  size = mBundles.Count();
 
   for (i = 0; i < size; i++) {
-    nsCOMPtr<nsIStringBundle> bundle;
-    res = mBundle->QueryElementAt(i, NS_GET_IID(nsIStringBundle), getter_AddRefs(bundle));
-    if (NS_SUCCEEDED(res)) {
+    nsIStringBundle* bundle = mBundles[i];
+    if (bundle) {
         res = bundle->GetStringFromName(aName, aResult);
         if (NS_SUCCEEDED(res))
             return NS_OK;
@@ -613,40 +506,6 @@ struct bundleCacheEntry_t {
   nsIStringBundle* mBundle;
 };
 
-class nsStringBundleService : public nsIStringBundleService,
-                              public nsIObserver,
-                              public nsSupportsWeakReference
-{
-public:
-  nsStringBundleService();
-  virtual ~nsStringBundleService();
-
-  nsresult Init();
-
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSISTRINGBUNDLESERVICE
-  NS_DECL_NSIOBSERVER
-    
-private:
-  nsresult getStringBundle(const char *aUrl, nsIStringBundle** aResult);
-  nsresult FormatWithBundle(nsIStringBundle* bundle, nsresult aStatus, 
-                            PRUint32 argCount, PRUnichar** argArray,
-                            PRUnichar* *result);
-
-  void flushBundleCache();
-  
-  bundleCacheEntry_t *insertIntoCache(nsIStringBundle *aBundle,
-                                      nsCStringKey *aHashKey);
-
-  static void recycleEntry(bundleCacheEntry_t*);
-  
-  nsHashtable mBundleMap;
-  PRCList mBundleCache;
-  PLArenaPool mCacheEntryPool;
-
-  nsCOMPtr<nsIErrorService>     mErrorService;
-
-};
 
 nsStringBundleService::nsStringBundleService() :
   mBundleMap(MAX_CACHED_BUNDLES, PR_TRUE)
@@ -654,7 +513,7 @@ nsStringBundleService::nsStringBundleService() :
 #ifdef DEBUG_tao_
   printf("\n++ nsStringBundleService::nsStringBundleService ++\n");
 #endif
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   PR_INIT_CLIST(&mBundleCache);
   PL_InitArenaPool(&mCacheEntryPool, "srEntries",
@@ -686,6 +545,11 @@ nsStringBundleService::Init()
     os->AddObserver(this, "profile-do-change", PR_TRUE);
   }
 
+  // instantiate the override service, if there is any.
+  // at some point we probably want to make this a category, and
+  // support multiple overrides
+  mOverrideStrings = do_GetService(NS_STRINGBUNDLETEXTOVERRIDE_CONTRACTID);
+  
   return NS_OK;
 }
 
@@ -745,7 +609,7 @@ nsStringBundleService::getStringBundle(const char *aURLSpec,
   } else {
 
     // hasn't been cached, so insert it into the hash table
-    nsStringBundle* bundle = new nsStringBundle(aURLSpec);
+    nsStringBundle* bundle = new nsStringBundle(aURLSpec, mOverrideStrings);
     if (!bundle) return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(bundle);
     
@@ -837,12 +701,6 @@ nsStringBundleService::CreateBundle(const char* aURLSpec,
   return getStringBundle(aURLSpec,aResult);
 }
   
-NS_IMETHODIMP
-nsStringBundleService::CreateAsyncBundle(const char* aURLSpec, nsIStringBundle** aResult)
-{
-  return getStringBundle(aURLSpec, aResult);
-}
-
 NS_IMETHODIMP
 nsStringBundleService::CreateExtensibleBundle(const char* aCategory, 
                                               nsIStringBundle** aResult)
@@ -979,11 +837,3 @@ done:
 }
 
 
-NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsStringBundleService, Init)
-static const nsModuleComponentInfo components[] =
-{
-  { "String Bundle", NS_STRINGBUNDLESERVICE_CID, NS_STRINGBUNDLE_CONTRACTID, nsStringBundleServiceConstructor},
-};
-
-NS_IMPL_NSGETMODULE(nsStringBundleModule, components)
-
