@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -290,7 +290,7 @@ js_Disassemble1(JSContext *cx, JSScript *script, jsbytecode *pc, uintN loc,
 
 /*
  * Sprintf, but with unlimited and automatically allocated buffering.
-*/
+ */
 typedef struct Sprinter {
     JSContext       *context;       /* context executing the decompiler */
     JSArenaPool     *pool;          /* string allocation pool */
@@ -305,6 +305,7 @@ typedef struct Sprinter {
 
 #define OFF2STR(sp,off) ((sp)->base + (off))
 #define STR2OFF(sp,str) ((str) - (sp)->base)
+#define RETRACT(sp,str) ((sp)->offset = STR2OFF(sp, str))
 
 static JSBool
 SprintAlloc(Sprinter *sp, size_t nb)
@@ -385,7 +386,7 @@ QuoteString(Sprinter *sp, JSString *str, jschar quote)
 
     /* Sample off first for later return value pointer computation. */
     off = sp->offset;
-    if (Sprint(sp, "%c", (char)quote) < 0)
+    if (quote && Sprint(sp, "%c", (char)quote) < 0)
 	return NULL;
 
     /* Loop control variables: z points at end of string sentinel. */
@@ -426,7 +427,7 @@ QuoteString(Sprinter *sp, JSString *str, jschar quote)
     }
 
     /* Sprint the closing quote and return the quoted string. */
-    if (Sprint(sp, "%c", (char)quote) < 0)
+    if (quote && Sprint(sp, "%c", (char)quote) < 0)
 	return NULL;
     return OFF2STR(sp, off);
 }
@@ -728,6 +729,7 @@ DecompileSwitch(SprintStack *ss, TableEntry *table, uintN tableLength,
 		    rval = QuoteString(&ss->sprinter, str, (jschar)'"');
 		    if (!rval)
 			return JS_FALSE;
+                    RETRACT(&ss->sprinter, rval);
 		} else {
 		    rval = JS_GetStringBytes(str);
 		}
@@ -771,11 +773,13 @@ GetSlotAtom(JSPrinter *jp, JSPropertyOp getter, uintN slot)
 
     scope = jp->scope;
     while (scope) {
-        for (sprop = scope->props; sprop; sprop = sprop->next) {
-            if (SPROP_GETTER_SCOPE(sprop, scope) != getter)
+        for (sprop = SCOPE_LAST_PROP(scope); sprop; sprop = sprop->parent) {
+            if (sprop->getter != getter)
                 continue;
-            if ((uintN)JSVAL_TO_INT(sprop->id) == slot)
-                return sym_atom(sprop->symbols);
+            JS_ASSERT(sprop->flags & SPROP_HAS_SHORTID);
+            JS_ASSERT(!JSVAL_IS_INT(sprop->id));
+            if ((uintN) sprop->shortid == slot)
+                return (JSAtom *) sprop->id;
         }
         obj = scope->object;
         if (!obj)
@@ -848,7 +852,7 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
 /*
  * Local macros
-*/
+ */
 #define DECOMPILE_CODE(pc,nb)	if (!Decompile(ss, pc, nb)) return JS_FALSE
 #define POP_STR()		OFF2STR(&ss->sprinter, PopOff(ss, op))
 #define LOCAL_ASSERT(expr)	JS_ASSERT(expr); if (!(expr)) return JS_FALSE
@@ -979,14 +983,22 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 		    atom = js_GetAtom(cx, &jp->script->atomMap,
 				      (jsatomid) js_GetSrcNoteOffset(sn, 0));
 		    jp->indent -= 4;
-		    js_printf(jp, "\t%s:\n", ATOM_BYTES(atom));
+                    rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                    if (!rval)
+                        return JS_FALSE;
+                    RETRACT(&ss->sprinter, rval);
+                    js_printf(jp, "\t%s:\n", rval);
 		    jp->indent += 4;
 		    break;
 
 		  case SRC_LABELBRACE:
 		    atom = js_GetAtom(cx, &jp->script->atomMap,
 				      (jsatomid) js_GetSrcNoteOffset(sn, 0));
-		    js_printf(jp, "\t%s: {\n", ATOM_BYTES(atom));
+                    rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                    if (!rval)
+                        return JS_FALSE;
+                    RETRACT(&ss->sprinter, rval);
+		    js_printf(jp, "\t%s: {\n", rval);
 		    jp->indent += 4;
 		    break;
 
@@ -1010,8 +1022,12 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
                     LOCAL_ASSERT(*pc == JSOP_EXCEPTION);
                     pc += js_CodeSpec[JSOP_EXCEPTION].length;
                     LOCAL_ASSERT(*pc == JSOP_INITCATCHVAR);
-		    js_printf(jp, "%s",
-			      ATOM_BYTES(GET_ATOM(cx, jp->script, pc)));
+                    atom = GET_ATOM(cx, jp->script, pc);
+                    rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                    if (!rval)
+                        return JS_FALSE;
+                    RETRACT(&ss->sprinter, rval);
+                    js_printf(jp, "%s", rval);
                     pc += js_CodeSpec[JSOP_INITCATCHVAR].length;
                     LOCAL_ASSERT(*pc == JSOP_ENTERWITH);
                     pc += js_CodeSpec[JSOP_ENTERWITH].length;
@@ -1253,7 +1269,11 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 		  case SRC_CONT2LABEL:
 		    atom = js_GetAtom(cx, &jp->script->atomMap,
 				      (jsatomid) js_GetSrcNoteOffset(sn, 0));
-		    js_printf(jp, "\tcontinue %s;\n", ATOM_BYTES(atom));
+                    rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                    if (!rval)
+                        return JS_FALSE;
+                    RETRACT(&ss->sprinter, rval);
+		    js_printf(jp, "\tcontinue %s;\n", rval);
 		    break;
 		  case SRC_CONTINUE:
 		    js_printf(jp, "\tcontinue;\n");
@@ -1261,7 +1281,11 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 		  case SRC_BREAK2LABEL:
 		    atom = js_GetAtom(cx, &jp->script->atomMap,
 				      (jsatomid) js_GetSrcNoteOffset(sn, 0));
-		    js_printf(jp, "\tbreak %s;\n", ATOM_BYTES(atom));
+                    rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                    if (!rval)
+                        return JS_FALSE;
+                    RETRACT(&ss->sprinter, rval);
+		    js_printf(jp, "\tbreak %s;\n", rval);
 		    break;
 		  case SRC_HIDDEN:
 		    break;
@@ -1394,18 +1418,27 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 	      case JSOP_FORARG:
 		atom = GetSlotAtom(jp, js_GetArgument, GET_ARGNO(pc));
 		LOCAL_ASSERT(atom);
-		lval = ATOM_BYTES(atom);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, lval);
 		goto do_fornameinloop;
 
 	      case JSOP_FORVAR:
 		atom = GetSlotAtom(jp, js_GetLocalVariable, GET_VARNO(pc));
 		LOCAL_ASSERT(atom);
-		lval = ATOM_BYTES(atom);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, lval);
 		goto do_fornameinloop;
 
 	      case JSOP_FORNAME:
                 atom = GET_ATOM(cx, jp->script, pc);
-		lval = ATOM_BYTES(atom);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, lval);
 
               do_fornameinloop:
 		sn = js_GetSrcNote(jp->script, pc);
@@ -1429,10 +1462,15 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
               do_forinbody:
 		js_printf(jp, "\tfor (%s%s", VarPrefix(sn), lval);
-		if (atom)
-		    js_printf(jp, ".%s", ATOM_BYTES(atom));
-		else if (xval)
-		    js_printf(jp, "[%s]", xval);
+                if (atom) {
+                    lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                    if (!lval)
+                        return JS_FALSE;
+                    RETRACT(&ss->sprinter, lval);
+                    js_printf(jp, ".%s", lval);
+                } else if (xval) {
+                    js_printf(jp, "[%s]", xval);
+                }
                 rval = OFF2STR(&ss->sprinter, ss->offsets[ss->top-1]);
 		js_printf(jp, " in %s) {\n", rval);
 		jp->indent += 4;
@@ -1509,7 +1547,9 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 	      case JSOP_SETNAME:
 		atom = GET_ATOM(cx, jp->script, pc);
 	      do_setname:
-		lval = ATOM_BYTES(atom);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
 		rval = POP_STR();
 		if (op == JSOP_SETNAME)
 		    (void) PopOff(ss, op);
@@ -1598,15 +1638,21 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
 	      case JSOP_DELNAME:
 		atom = GET_ATOM(cx, jp->script, pc);
-		lval = ATOM_BYTES(atom);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, lval);
 		todo = Sprint(&ss->sprinter, "%s %s", js_delete_str, lval);
 		break;
 
 	      case JSOP_DELPROP:
 		atom = GET_ATOM(cx, jp->script, pc);
+                rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!rval)
+                    return JS_FALSE;
 		lval = POP_STR();
 		todo = Sprint(&ss->sprinter, "%s %s.%s",
-			      js_delete_str, lval, ATOM_BYTES(atom));
+			      js_delete_str, lval, rval);
 		break;
 
 	      case JSOP_DELELEM:
@@ -1638,7 +1684,10 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 	      case JSOP_DECNAME:
 		atom = GET_ATOM(cx, jp->script, pc);
 	      do_incatom:
-		lval = ATOM_BYTES(atom);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, lval);
 		todo = Sprint(&ss->sprinter, "%s%s",
 			      js_incop_str[!(cs->format & JOF_INC)], lval);
 		break;
@@ -1646,10 +1695,13 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 	      case JSOP_INCPROP:
 	      case JSOP_DECPROP:
 		atom = GET_ATOM(cx, jp->script, pc);
+                rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!rval)
+                    return JS_FALSE;
 		lval = POP_STR();
 		todo = Sprint(&ss->sprinter, "%s%s.%s",
 			      js_incop_str[!(cs->format & JOF_INC)],
-			      lval, ATOM_BYTES(atom));
+			      lval, rval);
 		break;
 
 	      case JSOP_INCELEM:
@@ -1677,17 +1729,24 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 	      case JSOP_NAMEDEC:
 		atom = GET_ATOM(cx, jp->script, pc);
 	      do_atominc:
-		lval = ATOM_BYTES(atom);
-		todo = Sprint(&ss->sprinter, "%s%s",
-			      lval, js_incop_str[!(cs->format & JOF_INC)]);
+                lval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!lval)
+                    return JS_FALSE;
+		todo = STR2OFF(&ss->sprinter, lval);
+                SprintPut(&ss->sprinter,
+                          js_incop_str[!(cs->format & JOF_INC)],
+                          2);
 		break;
 
 	      case JSOP_PROPINC:
 	      case JSOP_PROPDEC:
 		atom = GET_ATOM(cx, jp->script, pc);
+                rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!rval)
+                    return JS_FALSE;
 		lval = POP_STR();
 		todo = Sprint(&ss->sprinter, "%s.%s%s",
-			      lval, ATOM_BYTES(atom),
+			      lval, rval,
 			      js_incop_str[!(cs->format & JOF_INC)]);
 		break;
 
@@ -1707,28 +1766,25 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
 	      case JSOP_GETPROP:
 		atom = GET_ATOM(cx, jp->script, pc);
-		lval = POP_STR();
                 str = ATOM_TO_STRING(atom);
                 if (IsASCIIIdentifier(str)) {
+                    lval = POP_STR();
                     todo = Sprint(&ss->sprinter, "%s.%s",
                                   lval, JS_GetStringBytes(str));
                 } else {
-                    lval = JS_strdup(cx, lval);
-                    if (!lval)
-                        return JS_FALSE;
                     xval = QuoteString(&ss->sprinter, str, (jschar)'\'');
+                    lval = POP_STR();
                     todo = Sprint(&ss->sprinter, "%s[%s]", lval, xval);
-                    JS_free(cx, (char *)lval);
                 }
 		break;
 
 	      case JSOP_SETPROP:
-		rval = POP_STR();
-		atom = GET_ATOM(cx, jp->script, pc);
-		lval = POP_STR();
                 sn = js_GetSrcNote(jp->script, pc - 1);
+                atom = GET_ATOM(cx, jp->script, pc);
                 str = ATOM_TO_STRING(atom);
                 if (IsASCIIIdentifier(str)) {
+                    rval = POP_STR();
+                    lval = POP_STR();
                     xval = JS_GetStringBytes(str);
                     if (sn && SN_TYPE(sn) == SRC_ASSIGNOP) {
                         todo = Sprint(&ss->sprinter, "%s.%s %s= %s",
@@ -1739,13 +1795,9 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
                                       lval, xval, rval);
                     }
                 } else {
-                    rval = JS_strdup(cx, rval);
-                    lval = JS_strdup(cx, lval);
-                    if (!rval || !lval) {
-                        JS_free(cx, (char *)rval);
-                        return JS_FALSE;
-                    }
                     xval = QuoteString(&ss->sprinter, str, (jschar)'\'');
+                    rval = POP_STR();
+                    lval = POP_STR();
                     if (sn && SN_TYPE(sn) == SRC_ASSIGNOP) {
                         todo = Sprint(&ss->sprinter, "%s[%s] %s= %s",
                                       lval, xval,
@@ -1754,8 +1806,6 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
                         todo = Sprint(&ss->sprinter, "%s[%s] = %s",
                                       lval, xval, rval);
                     }
-                    JS_free(cx, (char *)rval);
-                    JS_free(cx, (char *)lval);
                 }
 		break;
 
@@ -1819,8 +1869,11 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 		atom = GET_ATOM(cx, jp->script, pc);
 	      do_name:
 		sn = js_GetSrcNote(jp->script, pc);
-		todo = Sprint(&ss->sprinter, "%s%s",
-			      VarPrefix(sn), ATOM_BYTES(atom));
+                rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!rval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, rval);
+		todo = Sprint(&ss->sprinter, "%s%s", VarPrefix(sn), rval);
 		break;
 
 	      case JSOP_UINT16:
@@ -1852,7 +1905,7 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
                                    (jschar)'"');
                 if (!rval)
                     return JS_FALSE;
-                todo = Sprint(&ss->sprinter, "%s", rval);
+                todo = STR2OFF(&ss->sprinter, rval);
                 break;
 
               case JSOP_OBJECT:
@@ -1902,8 +1955,8 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 			    table[j++].offset = off2;
 			}
 		    }
-                    js_qsort(table, (size_t)j, sizeof *table, CompareOffsets,
-                             NULL);
+                    js_HeapSort(table, (size_t) j, sizeof *table,
+                                CompareOffsets, NULL);
 		}
 
 		ok = DecompileSwitch(ss, table, (uintN)j, pc, len, off,
@@ -2073,7 +2126,11 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
 	      case JSOP_EXPORTNAME:
 		atom = GET_ATOM(cx, jp->script, pc);
-		js_printf(jp, "\texport %s\n", ATOM_BYTES(atom));
+                rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!rval)
+                    return JS_FALSE;
+                RETRACT(&ss->sprinter, rval);
+                js_printf(jp, "\texport %s\n", rval);
 		todo = -2;
 		break;
 
@@ -2085,8 +2142,11 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
 	      case JSOP_IMPORTPROP:
 		atom = GET_ATOM(cx, jp->script, pc);
+                rval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!rval)
+                    return JS_FALSE;
 		lval = POP_STR();
-		js_printf(jp, "\timport %s.%s\n", lval, ATOM_BYTES(atom));
+                js_printf(jp, "\timport %s.%s\n", lval, rval);
 		todo = -2;
 		break;
 
@@ -2144,9 +2204,11 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 
 	      case JSOP_INITPROP:
               case JSOP_INITCATCHVAR:
-		rval = POP_STR();
 		atom = GET_ATOM(cx, jp->script, pc);
-		xval = ATOM_BYTES(atom);
+                xval = QuoteString(&ss->sprinter, ATOM_TO_STRING(atom), 0);
+                if (!xval)
+                    return JS_FALSE;
+		rval = POP_STR();
 		lval = POP_STR();
 	      do_initprop:
 #ifdef OLD_GETTER_SETTER
@@ -2163,18 +2225,18 @@ Decompile(SprintStack *ss, jsbytecode *pc, intN nb)
 #else
                 if (lastop == JSOP_GETTER || lastop == JSOP_SETTER) {
                     todo = Sprint(&ss->sprinter, "%s%s%s %s%s",
-                              lval,
-                              (lval[1] != '\0') ? ", " : "",
-                              (lastop == JSOP_GETTER)
-                              ? js_get_str : js_set_str,
-                              xval,
-                              rval + strlen(js_function_str) + 1);
+                                  lval,
+                                  (lval[1] != '\0') ? ", " : "",
+                                  (lastop == JSOP_GETTER)
+                                  ? js_get_str : js_set_str,
+                                  xval,
+                                  rval + strlen(js_function_str) + 1);
                 } else {
                     todo = Sprint(&ss->sprinter, "%s%s%s:%s",
-                              lval,
-                              (lval[1] != '\0') ? ", " : "",
-                              xval,
-                              rval);
+                                  lval,
+                                  (lval[1] != '\0') ? ", " : "",
+                                  xval,
+                                  rval);
                 }
 #endif
 		break;
@@ -2321,12 +2383,13 @@ js_DecompileFunctionBody(JSPrinter *jp, JSFunction *fun)
 JSBool
 js_DecompileFunction(JSPrinter *jp, JSFunction *fun)
 {
+    JSContext *cx;
+    uintN i, nargs, indent;
+    void *mark;
+    JSAtom **params;
     JSScope *scope, *oldscope;
-    JSScopeProperty *sprop, *snext;
+    JSScopeProperty *sprop;
     JSBool ok;
-    JSAtom *atom;
-    uintN indent;
-    intN i;
 
     /*
      * If pretty, conform to ECMA-262 Edition 3, 15.3.4.2, by decompiling a
@@ -2344,43 +2407,54 @@ js_DecompileFunction(JSPrinter *jp, JSFunction *fun)
         js_printf(jp, "%s ", js_getter_str);
     else if (fun->flags & JSFUN_SETTER)
         js_printf(jp, "%s ", js_setter_str);
-    js_printf(jp, "%s %s(",
-              js_function_str, fun->atom ? ATOM_BYTES(fun->atom) : "");
 
-    scope = NULL;
+    js_printf(jp, "%s ", js_function_str);
+    if (fun->atom && !QuoteString(&jp->sprinter, ATOM_TO_STRING(fun->atom), 0))
+        return JS_FALSE;
+    js_puts(jp, "(");
+
     if (fun->script && fun->object) {
-	/*
-	 * Print the parameters.
-	 *
-	 * This code is complicated by the need to handle duplicate parameter
-	 * names.  A duplicate parameter is stored as a property with id equal
-	 * to the parameter number, but will not be in order in the linked list
-	 * of symbols. So for each parameter we search the list of symbols for
-	 * the appropriately numbered parameter, which we can then print.
-	 */
-	for (i = 0; ; i++) {
-	    jsid id;
-	    atom = NULL;
-	    scope = OBJ_SCOPE(fun->object);
-	    for (sprop = scope->props; sprop; sprop = snext) {
-		snext = sprop->next;
-		if (SPROP_GETTER_SCOPE(sprop, scope) != js_GetArgument)
-		    continue;
-		if (JSVAL_IS_INT(sprop->id) && JSVAL_TO_INT(sprop->id) == i) {
-		    atom = sym_atom(sprop->symbols);
-		    break;
-		}
-		id = (jsid) sym_atom(sprop->symbols);
-		if (JSVAL_IS_INT(id) && JSVAL_TO_INT(id) == i) {
-		    atom = (JSAtom *) sprop->id;
-		    break;
-		}
-	    }
-	    if (atom == NULL)
-		break;
-	    js_printf(jp, (i > 0 ? ", %s" : "%s"), ATOM_BYTES(atom));
-	}
+        /*
+         * Print the parameters.
+         *
+         * This code is complicated by the need to handle duplicate parameter
+         * names, as required by ECMA (bah!).  A duplicate parameter is stored
+         * as another node with the same id (the parameter name) but different
+         * shortid (the argument index) along the property tree ancestor line
+         * starting at SCOPE_LAST_PROP(scope).  Only the last duplicate param
+         * is mapped by the scope's hash table.
+         */
+        cx = jp->sprinter.context;
+        nargs = fun->nargs;
+        mark = JS_ARENA_MARK(&cx->tempPool);
+        JS_ARENA_ALLOCATE_CAST(params, JSAtom **, &cx->tempPool,
+                               nargs * sizeof(JSAtom *));
+        if (!params) {
+            JS_ReportOutOfMemory(cx);
+            return JS_FALSE;
+        }
+        scope = OBJ_SCOPE(fun->object);
+        for (sprop = SCOPE_LAST_PROP(scope); sprop; sprop = sprop->parent) {
+            if (sprop->getter != js_GetArgument)
+                continue;
+            JS_ASSERT(sprop->flags & SPROP_HAS_SHORTID);
+            JS_ASSERT((uintN) sprop->shortid < nargs);
+            JS_ASSERT(!JSVAL_IS_INT(sprop->id));
+            params[(uintN) sprop->shortid] = (JSAtom *) sprop->id;
+        }
+        for (i = 0; i < nargs; i++) {
+            if (i > 0)
+                js_puts(jp, ", ");
+            if (!QuoteString(&jp->sprinter, ATOM_TO_STRING(params[i]), 0))
+                return JS_FALSE;
+        }
+        JS_ARENA_RELEASE(&cx->tempPool, mark);
+#ifdef __GNUC__
+    } else {
+        scope = NULL;
+#endif
     }
+
     js_printf(jp, ") {\n");
     indent = jp->indent;
     jp->indent += 4;
@@ -2398,6 +2472,7 @@ js_DecompileFunction(JSPrinter *jp, JSFunction *fun)
     }
     jp->indent -= 4;
     js_printf(jp, "\t}");
+
     if (jp->pretty) {
         js_puts(jp, "\n");
     } else {
