@@ -21,10 +21,6 @@
  *   Robert Ginda <rginda@netscape.com>
  */
 
-#ifdef XPCONNECT_STANDALONE
-#define NO_SUBSCRIPT_LOADER
-#endif
-
 #include "prlog.h"
 
 #include "nsCOMPtr.h"
@@ -42,7 +38,6 @@
 #include "nsIXPConnect.h"
 #include "nsCRT.h"
 #include "nsMemory.h"
-#include "nsIRegistry.h"
 #include "nsXPIDLString.h"
 #include "nsIObserverService.h"
 #include "nsIXPCScriptable.h"
@@ -53,9 +48,6 @@
 #include "nsIStandardURL.h"
 #include "nsNetUtil.h"
 #endif
-#ifndef NO_SUBSCRIPT_LOADER
-#include "mozJSSubScriptLoader.h"
-#endif
 #include "nsIComponentLoaderManager.h"
 // For reporting errors with the console service
 #include "nsIScriptError.h"
@@ -64,10 +56,6 @@
 const char mozJSComponentLoaderContractID[] = "@mozilla.org/moz/jsloader;1";
 const char jsComponentTypeName[] = "text/javascript";
 
-#ifndef NO_SUBSCRIPT_LOADER
-const char mozJSSubScriptLoadContractID[] = "@mozilla.org/moz/jssubscript-loader;1";
-#endif
-
 // same as in nsComponentManager.cpp (but without the JS)
 const char JSfileSizeValueName[] = "FileSize";
 const char JSlastModValueName[] = "LastModTimeStamp";
@@ -394,10 +382,9 @@ mozJSComponentLoader::mozJSComponentLoader()
     : mRuntime(nsnull),
       mModules(nsnull),
       mGlobals(nsnull),
-      mXPCOMKey(0),
       mInitialized(PR_FALSE)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 static PRIntn PR_CALLBACK
@@ -486,15 +473,6 @@ mozJSComponentLoader::Init(nsIComponentManager *aCompMgr, nsISupports *aReg)
     if (NS_FAILED(rv))
         return rv;
 
-    /* initialize registry handles */
-    mRegistry = do_QueryInterface(aReg, &rv);
-    if (NS_SUCCEEDED(rv)) {
-        rv = mRegistry->GetSubtree(nsIRegistry::Common, JSxpcomKeyName,
-                                   &mXPCOMKey);
-        if (NS_FAILED(rv))
-            /* if we can't get the XPCOM key, just skip all registry ops */
-            mRegistry = nsnull;
-    }
     return NS_OK;
 }
 
@@ -627,7 +605,6 @@ mozJSComponentLoader::SetRegistryInfo(const char *registryLocation,
 nsresult
 mozJSComponentLoader::RemoveRegistryInfo(nsIFile *component, const char *registryLocation)
 {
-    nsresult rv;
     if (!mLoaderManager)
         return NS_ERROR_FAILURE;
     
@@ -639,7 +616,6 @@ PRBool
 mozJSComponentLoader::HasChanged(const char *registryLocation,
                                  nsIFile *component)
 {
-    nsresult rv;
     if (!mLoaderManager)
         return NS_ERROR_FAILURE;
     
@@ -1243,57 +1219,3 @@ JSCLAutoContext::~JSCLAutoContext()
 //----------------------------------------------------------------------
 
 /* XXX this should all be data-driven, via NS_IMPL_GETMODULE_WITH_CATEGORIES */
-static NS_METHOD
-RegisterJSLoader(nsIComponentManager *aCompMgr, nsIFile *aPath,
-                 const char *registryLocation, const char *componentType,
-                 const nsModuleComponentInfo *info)
-{
-    nsresult rv;
-    nsCOMPtr<nsICategoryManager> catman =
-        do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    nsXPIDLCString previous;
-    return catman->AddCategoryEntry("component-loader", jsComponentTypeName,
-                                    mozJSComponentLoaderContractID,
-                                    PR_TRUE, PR_TRUE, getter_Copies(previous));
-}
-
-static NS_METHOD
-UnregisterJSLoader(nsIComponentManager *aCompMgr, nsIFile *aPath,
-                   const char *registryLocation,
-                   const nsModuleComponentInfo *info)
-{
-    nsresult rv;
-    nsCOMPtr<nsICategoryManager> catman =
-        do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    nsXPIDLCString jsLoader;
-    rv = catman->GetCategoryEntry("component-loader", jsComponentTypeName,
-                                  getter_Copies(jsLoader));
-    if (NS_FAILED(rv)) return rv;
-
-    // only unregister if we're the current JS component loader
-    if (!strcmp(jsLoader, mozJSComponentLoaderContractID)) {
-        return catman->DeleteCategoryEntry("component-loader",
-                                           jsComponentTypeName, PR_TRUE);
-    }
-    return NS_OK;
-}
-
-NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSComponentLoader);
-
-#ifndef NO_SUBSCRIPT_LOADER
-NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSSubScriptLoader);
-#endif
-
-static const nsModuleComponentInfo components[] = {
-    { "JS component loader", MOZJSCOMPONENTLOADER_CID,
-      mozJSComponentLoaderContractID, mozJSComponentLoaderConstructor,
-      RegisterJSLoader, UnregisterJSLoader },
-#ifndef NO_SUBSCRIPT_LOADER
-   { "JS subscript loader", MOZ_JSSUBSCRIPTLOADER_CID,
-      mozJSSubScriptLoadContractID, mozJSSubScriptLoaderConstructor }
-#endif
-};
-
-NS_IMPL_NSGETMODULE(JS_component_loader, components);
