@@ -180,7 +180,7 @@ NS_IMPL_ISUPPORTS2(nsTestXPCFoo, nsITestXPCFoo, nsITestXPCFoo2)
 nsTestXPCFoo::nsTestXPCFoo()
     : mFoo(NULL)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
     NS_ADDREF_THIS();
 }
 
@@ -216,7 +216,7 @@ NS_IMPL_ISUPPORTS1(MySecMan, nsIXPCSecurityManager)
 MySecMan::MySecMan()
     : mMode(OK_ALL)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
     NS_ADDREF_THIS();
 }
 
@@ -472,8 +472,8 @@ TestArgFormatter(JSContext* jscontext, JSObject* glob, nsIXPConnect* xpc)
 
     if(!JS_ConvertArguments(jscontext, 5, argv, "s %ip %iv %is s",
                             &a_out, 
-                            getter_AddRefs(b_out), 
-                            getter_AddRefs(c_out),
+                            NS_STATIC_CAST(nsISupports**, getter_AddRefs(b_out)), 
+                            NS_STATIC_CAST(nsIVariant**, getter_AddRefs(c_out)),
                             NS_STATIC_CAST(nsAString*, &d_out), 
                             &e_out))
     {
