@@ -42,7 +42,7 @@
 
 class nsIFrame;
 class nsIPresContext;
-class nsHTMLReflowCommand;
+class nsReflowPath;
 class nsIRenderingContext;
 class nsSpaceManager;
 class nsLineLayout;
@@ -139,8 +139,13 @@ struct nsHTMLReflowState {
   // the reason for the reflow
   nsReflowReason      reason;
 
-  // the reflow command. only set for a reflow reason of eReflowReason_Incremental
-  nsHTMLReflowCommand* reflowCommand;
+  // the incremental reflow path, when the reflow reason is
+  // eReflowReason_Incremental. Specifically, this corresponds to the
+  // portion of the incremental reflow path from `frame' down. Note
+  // that it is safe to assume that this is non-null: we maintain the
+  // invariant that it contains a valid nsReflowPath pointer when
+  // reason == eReflowReason_Incremental.
+  nsReflowPath        *path;
 
   // the available space in which to reflow the frame. The space represents the
   // amount of room for the frame's border, padding, and content area (not the
@@ -164,6 +169,12 @@ struct nsHTMLReflowState {
   // LineLayout object (only for inline reflow; set to NULL otherwise)
   nsLineLayout*    mLineLayout;
 
+  // The appropriate reflow state for the containing block (for
+  // percentage widths, etc.) of any *children* of this reflow state's
+  // frame.  In other words, this is the nearest containing block reflow
+  // state that is *either* this frame or an ancestor thereof.
+  const nsHTMLReflowState *mCBReflowState;
+
   // The computed width specifies the frame's content area width, and it does
   // not apply to inline non-replaced elements
   //
@@ -225,6 +236,9 @@ struct nsHTMLReflowState {
   // reflow for percent height calculations 
   nsIPercentHeightObserver* mPercentHeightObserver;
 
+  // a frame (e.g. nsTableFrame) which initiates a special reflow for percent height calculations 
+  nsIFrame* mPercentHeightReflowInitiator;
+
   // This value keeps track of how deeply nested a given reflow state
   // is from the top of the frame tree.
   PRInt16 mReflowDepth;
@@ -234,7 +248,8 @@ struct nsHTMLReflowState {
                                      // percent height frames inside cells which may not have computed heights
     PRUint16 mIsTopOfPage:1;         // is the current context at the top of a page?
     PRUint16 mBlinks:1;              // Keep track of text-decoration: blink
-    PRUint16 mUnused:13;             // for future use             
+    PRUint16 mVisualBidiFormControl:1; // Keep track of descendants of form controls on Visual Bidi pages
+    PRUint16 mUnused:12;             // for future use             
   } mFlags;
 
 #ifdef IBMBIDI
@@ -248,9 +263,13 @@ struct nsHTMLReflowState {
   static const char* ReasonToString(nsReflowReason aReason);
 #endif
 
-  // Note: The copy constructor is written by the compiler
-  // automatically. You can use that and then override specific values
-  // if you want, or you can call Init as desired...
+  // A simple copy constructor.  (It fixes up |mCBReflowState|, which
+  // can point to |this|, to point to the copy's |this|.)
+  nsHTMLReflowState(const nsHTMLReflowState& aOther);
+
+  // A simple assignment operator.  It does the same fixups as the
+  // copy-consturctor.
+  nsHTMLReflowState& operator=(const nsHTMLReflowState& aOther);
 
   // Initialize a <b>root</b> reflow state with a rendering context to
   // use for measuring things.
@@ -264,18 +283,19 @@ struct nsHTMLReflowState {
   // reflow.
   nsHTMLReflowState(nsIPresContext*          aPresContext,
                     nsIFrame*                aFrame,
-                    nsHTMLReflowCommand&     aReflowCommand,
+                    nsReflowPath*            aReflowPath,
                     nsIRenderingContext*     aRenderingContext,
                     const nsSize&            aAvailableSpace);
 
   // Initialize a reflow state for a child frames reflow. Some state
   // is copied from the parent reflow state; the remaining state is
-  // computed.
+  // computed. 
   nsHTMLReflowState(nsIPresContext*          aPresContext,
                     const nsHTMLReflowState& aParentReflowState,
                     nsIFrame*                aFrame,
                     const nsSize&            aAvailableSpace,
-                    nsReflowReason           aReason);
+                    nsReflowReason           aReason, 
+                    PRBool                   aInit = PR_TRUE);
 
   // Same as the previous except that the reason is taken from the
   // parent's reflow state.
@@ -291,16 +311,16 @@ struct nsHTMLReflowState {
                     nsIFrame*                aFrame,
                     const nsSize&            aAvailableSpace,
                     nscoord                  aContainingBlockWidth,
-                    nscoord                  aContainingBlockHeight);
-
-  /**
-   * Get the containing block reflow state, starting from a frames
-   * <B>parent</B> reflow state (the parent reflow state may or may not end
-   * up being the containing block reflow state)
-   */
-  static const nsHTMLReflowState*
-    GetContainingBlockReflowState(const nsHTMLReflowState* aParentRS);
+                    nscoord                  aContainingBlockHeight,
+                    nsReflowReason           aReason);
 
+  // This method initializes various data members. It is automatically
+  // called by the various constructors
+  void Init(nsIPresContext* aPresContext,
+            nscoord         aContainingBlockWidth = -1,
+            nscoord         aContainingBlockHeight = -1,
+            nsMargin*       aBorder = nsnull,
+            nsMargin*       aPadding = nsnull);
   /**
    * First find the containing block's reflow state using
    * GetContainingBlockReflowState, then ask the containing block for
@@ -352,15 +372,14 @@ struct nsHTMLReflowState {
 
 
 protected:
-  // This method initializes various data members. It is automatically
-  // called by the various constructors
-  void Init(nsIPresContext* aPresContext,
-            nscoord         aContainingBlockWidth = -1,
-            nscoord         aContainingBlockHeight = -1);
+
+  void InitCBReflowState();
 
   void InitConstraints(nsIPresContext* aPresContext,
                        nscoord         aContainingBlockWidth,
-                       nscoord         aContainingBlockHeight);
+                       nscoord         aContainingBlockHeight,
+                       nsMargin*       aBorder,
+                       nsMargin*       aPadding);
 
   void CalculateHypotheticalBox(nsIPresContext*    aPresContext,
                                 nsIFrame*          aPlaceholderFrame,
@@ -423,6 +442,18 @@ protected:
   //   the width/height and it is removed due to box sizing) then it is driven to 0
   void AdjustComputedHeight(void);
   void AdjustComputedWidth(void);
+
+#ifdef IBMBIDI
+  /**
+   * Test whether the frame is a form control in a visual Bidi page.
+   * This is necessary for backwards-compatibility, because most visual
+   * pages use logical order for form controls so that they will
+   * display correctly on native widgets in OSs with Bidi support
+   * @param aPresContext the pres context
+   * @return whether the frame is a BIDI form control
+   */
+  PRBool IsBidiFormControl(nsIPresContext* aPresContext);
+#endif
 };
 
 #endif /* nsHTMLReflowState_h___ */
