@@ -111,6 +111,8 @@
 #define NS_FRAME_TRACE_REFLOW_OUT(_method, _status)
 #endif
 
+// handy utilities
+extern void SetFontFromStyle(nsIRenderingContext* aRC, nsIStyleContext* aSC);
 
 //----------------------------------------------------------------------
 
@@ -135,7 +137,7 @@ public:
 
   // Overloaded new operator. Initializes the memory to 0 and relies on an arena
   // (which comes from the presShell) to perform the allocation.
-  void* operator new(size_t sz, nsIPresShell* aPresShell);
+  void* operator new(size_t sz, nsIPresShell* aPresShell) CPP_THROW_NEW;
 
   // Overridden to prevent the global delete from being called, since the memory
   // came out of an nsIArena instead of the global delete operator's heap.  
@@ -145,11 +147,7 @@ public:
 
 private:
   // The normal operator new is disallowed on nsFrames.
-#if (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95))
-  void* operator new(size_t sz) throw () { return nsnull; };
-#else
-  void* operator new(size_t sz) { return nsnull; };
-#endif
+  void* operator new(size_t sz) CPP_THROW_NEW { return nsnull; };
 
 public:
 
@@ -249,6 +247,7 @@ public:
   NS_IMETHOD  SetView(nsIPresContext* aPresContext, nsIView* aView);
   NS_IMETHOD  GetParentWithView(nsIPresContext* aPresContext, nsIFrame** aParent) const;
   NS_IMETHOD  GetOffsetFromView(nsIPresContext* aPresContext, nsPoint& aOffset, nsIView** aView) const;
+  NS_IMETHOD  GetOriginToViewOffset(nsIPresContext *aPresContext, nsPoint& aOffset, nsIView **aView) const;
   NS_IMETHOD  GetWindow(nsIPresContext* aPresContext, nsIWidget**) const;
   NS_IMETHOD  GetFrameType(nsIAtom** aType) const;
   NS_IMETHOD  IsPercentageBase(PRBool& aBase) const;
@@ -273,17 +272,14 @@ public:
                                   nsPeekOffsetStruct *aPos);
   NS_IMETHOD  GetOffsets(PRInt32 &aStart, PRInt32 &aEnd) const;
   NS_IMETHOD  ReflowDirtyChild(nsIPresShell* aPresShell, nsIFrame* aChild);
-  NS_IMETHOD  ReflowCommandNotify(nsIPresShell*        aPresShell,
-                                  nsHTMLReflowCommand* aRC,
-                                  PRBool               aCommandAdded);
 
 #ifdef ACCESSIBILITY
   NS_IMETHOD  GetAccessible(nsIAccessible** aAccessible);
 #endif
 
-  NS_IMETHOD GetParentStyleContextProvider(nsIPresContext* aPresContext,
-                                           nsIFrame** aProviderFrame, 
-                                           nsContextProviderRelationship& aRelationship);
+  NS_IMETHOD GetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                        nsIFrame**      aProviderFrame,
+                                        PRBool*         aIsChild);
 
   // Check Style Visibility and mState for Selection (when printing)
   NS_IMETHOD IsVisibleForPainting(nsIPresContext *     aPresContext, 
@@ -291,7 +287,7 @@ public:
                                   PRBool               aCheckVis,
                                   PRBool*              aIsVisible);
 
-  NS_IMETHOD IsEmpty(PRBool aIsQuirkMode,
+  NS_IMETHOD IsEmpty(nsCompatibility aCompatMode,
                      PRBool aIsPre,
                      PRBool* aResult);
 
@@ -375,6 +371,18 @@ public:
                                              nsIAtom*                        aAttribute,
                                              nsIAtom*                        aListName);
 
+  // Do the work for getting the parent style context frame so that
+  // other frame's |GetParentStyleContextFrame| methods can call this
+  // method on *another* frame.  (This function handles out-of-flow
+  // frames by using the frame manager's placeholder map and it handles
+  // block-within-inline by calling |GetIBSpecialParent|.)
+  nsresult DoGetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                        nsIFrame**      aProviderFrame,
+                                        PRBool*         aIsChild);
+
+  nsresult GetIBSpecialParent(nsIPresContext* aPresContext,
+                              nsIFrame** aSpecialParent);
+
   //Mouse Capturing code used by the frames to tell the view to capture all the following events
   NS_IMETHOD CaptureMouse(nsIPresContext* aPresContext, PRBool aGrabMouseEvents);
   PRBool   IsMouseCaptured(nsIPresContext* aPresContext);
@@ -476,8 +484,10 @@ protected:
   //   of the enclosing cell or table (if not inside a cell)
   //  aTarget tells us what table element to select (currently only cell and table supported)
   //  (enums for this are defined in nsIFrame.h)
-  NS_IMETHOD GetDataForTableSelection(nsIFrameSelection *aFrameSelection, nsMouseEvent *aMouseEvent, 
-                                      nsIContent **aParentContent, PRInt32 *aContentOffset, PRInt32 *aTarget);
+  NS_IMETHOD GetDataForTableSelection(nsIFrameSelection *aFrameSelection, 
+                                      nsIPresShell *aPresShell, nsMouseEvent *aMouseEvent, 
+                                      nsIContent **aParentContent, PRInt32 *aContentOffset, 
+                                      PRInt32 *aTarget);
 
   static void XMLQuote(nsString& aString);
 
@@ -527,4 +537,12 @@ protected:
 #endif
 // End Display Reflow Debugging
 
+// similar to NS_ENSURE_TRUE but with no return value
+#define ENSURE_TRUE(x)                                        \
+  PR_BEGIN_MACRO                                              \
+    if (!(x)) {                                               \
+       NS_WARNING("ENSURE_TRUE(" #x ") failed");              \
+       return;                                                \
+    }                                                         \
+  PR_END_MACRO
 #endif /* nsFrame_h___ */
