@@ -49,7 +49,6 @@ public:
                nsSpaceManager* aSpaceManager,
                const nsHTMLReflowState* aOuterReflowState,
                PRBool aComputeMaxElementSize);
-  nsLineLayout(nsIPresContext* aPresContext);
   ~nsLineLayout();
 
   void Init(nsBlockReflowState* aState, nscoord aMinLineHeight,
@@ -90,8 +89,6 @@ public:
   void EndSpan(nsIFrame* aFrame, nsSize& aSizeResult,
                nsSize* aMaxElementSize);
 
-  PRBool InStrictMode();
-
   PRInt32 GetCurrentSpanCount() const;
 
   void SplitLineTo(PRInt32 aNewCount);
@@ -137,9 +134,7 @@ protected:
 #define LL_IMPACTEDBYFLOATERS          0x00000040
 #define LL_LASTFLOATERWASLETTERFRAME   0x00000080
 #define LL_CANPLACEFLOATER             0x00000100
-#define LL_KNOWSTRICTMODE              0x00000200
-#define LL_INSTRICTMODE                0x00000400
-#define LL_LINEENDSINBR                0x00000800
+#define LL_LINEENDSINBR                0x00000200
 #define LL_LASTFLAG                    LL_LINEENDSINBR
 
   PRUint16 mFlags;
@@ -223,6 +218,16 @@ public:
     SetFlag(LL_LINEENDSINBR, aOn); 
   }
 
+  PRBool InStrictMode() const
+  {
+    return mCompatMode != eCompatibility_NavQuirks;
+  }
+
+  nsCompatibility GetCompatMode() const
+  {
+    return mCompatMode;
+  }
+
   //----------------------------------------
   // Inform the line-layout about the presence of a floating frame
   // XXX get rid of this: use get-frame-type?
@@ -267,7 +272,12 @@ protected:
   nsSpaceManager* mSpaceManager;
   const nsStyleText* mStyleText; // for the block
   const nsHTMLReflowState* mBlockReflowState;
+
+  // XXX remove this when landing bug 154892 (splitting absolute positioned frames)
+  friend class nsInlineFrame;
+
   nsBlockReflowState* mBlockRS;/* XXX hack! */
+  nsCompatibility mCompatMode;
   nscoord mMinLineHeight;
   PRPackedBool mComputeMaxElementSize;
   PRUint8 mTextAlign;
