@@ -41,7 +41,6 @@
 #include "nsIRenderingContext.h"
 #include "nsIStyleContext.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsIDeviceContext.h"
 #include "nsIViewManager.h"
 #include "nsIPresShell.h"
@@ -52,7 +51,8 @@
 #include "nsIPrintPreviewContext.h"
 #include "nsIPrintContext.h"
 #include "nsStyleConsts.h"
-#include "nsIRegion.h"
+#include "nsRegion.h"
+#include "nsLayoutAtoms.h"
 
 #include "nsIPref.h" // for header/footer gap & ExtraMargin for Print Preview
 
@@ -72,35 +72,25 @@ static NS_DEFINE_CID(kLocaleServiceCID, NS_LOCALESERVICE_CID);
 // this class contains a helper method we need to get 
 // a string from a string bundle
 #include "nsFormControlHelper.h"
-#define PRINTING_PROPERTIES "chrome://communicator/locale/printing.properties"
+#define PRINTING_PROPERTIES "chrome://global/locale/printing.properties"
 
 // Print Options
 #include "nsIPrintSettings.h"
 #include "nsIPrintOptions.h"
 #include "nsGfxCIID.h"
 #include "nsIServiceManager.h"
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
+
+static const char sPrintOptionsContractID[] = "@mozilla.org/gfx/printsettings-service;1";
+
 //
 static NS_DEFINE_CID(kRegionCID, NS_REGION_CID);
 
-
-
-#if defined(DEBUG_rods) || defined(DEBUG_dcone)
-#define DEBUG_PRINTING
-#endif
-
-#ifdef DEBUG_PRINTING
-#define PRINT_DEBUG_MSG1(_msg1) fprintf(mDebugFD, (_msg1))
-#define PRINT_DEBUG_MSG2(_msg1, _msg2) fprintf(mDebugFD, (_msg1), (_msg2))
-#define PRINT_DEBUG_MSG3(_msg1, _msg2, _msg3) fprintf(mDebugFD, (_msg1), (_msg2), (_msg3))
-#define PRINT_DEBUG_MSG4(_msg1, _msg2, _msg3, _msg4) fprintf(mDebugFD, (_msg1), (_msg2), (_msg3), (_msg4))
-#define PRINT_DEBUG_MSG5(_msg1, _msg2, _msg3, _msg4, _msg5) fprintf(mDebugFD, (_msg1), (_msg2), (_msg3), (_msg4), (_msg5))
-#else //--------------
-#define PRINT_DEBUG_MSG1(_msg) 
-#define PRINT_DEBUG_MSG2(_msg1, _msg2) 
-#define PRINT_DEBUG_MSG3(_msg1, _msg2, _msg3) 
-#define PRINT_DEBUG_MSG4(_msg1, _msg2, _msg3, _msg4) 
-#define PRINT_DEBUG_MSG5(_msg1, _msg2, _msg3, _msg4, _msg5) 
+#include "prlog.h"
+#ifdef PR_LOGGING 
+PRLogModuleInfo * kLayoutPrintingLogMod = PR_NewLogModule("printing-layout");
+#define PR_PL(_p1)  PR_LOG(kLayoutPrintingLogMod, PR_LOG_DEBUG, _p1)
+#else
+#define PR_PL(_p1)
 #endif
 
 // This object a shared by all the nsPageFrames 
@@ -110,12 +100,16 @@ nsSharedPageData::nsSharedPageData() :
   mHeadFootFont(nsnull),
   mPageNumFormat(nsnull),
   mPageNumAndTotalsFormat(nsnull),
+  mDocTitle(nsnull),
+  mDocURL(nsnull),
   mReflowRect(0,0,0,0),
   mReflowMargin(0,0,0,0),
   mShadowSize(0,0),
   mDeadSpaceMargin(0,0,0,0),
   mExtraMargin(0,0,0,0),
-  mHeadFooterGap(0)
+  mEdgePaperMargin(0,0,0,0),
+  mPageContentXMost(0),
+  mPageContentSize(0)
 {
 }
 
@@ -125,6 +119,8 @@ nsSharedPageData::~nsSharedPageData()
   if (mHeadFootFont) delete mHeadFootFont;
   nsMemory::Free(mPageNumFormat);
   nsMemory::Free(mPageNumAndTotalsFormat);
+  if (mDocTitle) nsMemory::Free(mDocTitle);
+  if (mDocURL) nsMemory::Free(mDocURL);
 }
 
 nsresult
@@ -144,6 +140,7 @@ NS_NewSimplePageSequenceFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
 
 nsSimplePageSequenceFrame::nsSimplePageSequenceFrame() :
   mIsPrintingSelection(PR_FALSE),
+  mTotalPages(-1),
   mSelectionHeight(-1),
   mYSelOffset(0)
 {
@@ -163,7 +160,7 @@ nsSimplePageSequenceFrame::nsSimplePageSequenceFrame() :
   // XXX this code and the object data member "mIsPrintingSelection" is only needed
   // for the hack for printing selection where we make the page the max size
   nsresult rv;
-  mPageData->mPrintOptions = do_GetService(kPrintOptionsCID, &rv);
+  mPageData->mPrintOptions = do_GetService(sPrintOptionsContractID, &rv);
   if (NS_SUCCEEDED(rv) && mPageData->mPrintOptions) {
     // now get the default font form the print options
     mPageData->mPrintOptions->GetDefaultFont(*mPageData->mHeadFootFont);
@@ -177,10 +174,6 @@ nsSimplePageSequenceFrame::nsSimplePageSequenceFrame() :
   // Doing this here so we only have to go get these formats once
   SetPageNumberFormat("pagenumber",  "%1$d", PR_TRUE);
   SetPageNumberFormat("pageofpages", "%1$d of %2$d", PR_FALSE);
-
-#ifdef NS_DEBUG
-  mDebugFD = stdout;
-#endif
 }
 
 nsSimplePageSequenceFrame::~nsSimplePageSequenceFrame()
@@ -195,7 +188,7 @@ nsSimplePageSequenceFrame::QueryInterface(const nsIID& aIID, void** aInstancePtr
   if (NULL == aInstancePtr) {
     return NS_ERROR_NULL_POINTER;
   }
-  if (aIID.Equals(kIPageSequenceFrameIID)) {
+  if (aIID.Equals(NS_GET_IID(nsIPageSequenceFrame))) {
     *aInstancePtr = (void*)(nsIPageSequenceFrame*)this;
     return NS_OK;
   }
@@ -223,6 +216,25 @@ nsSimplePageSequenceFrame::CreateContinuingPageFrame(nsIPresContext* aPresContex
   return rv;
 }
 
+void nsSimplePageSequenceFrame::GetEdgePaperMarginCoord(nsIPref* aPref, char* aPrefName, nscoord& aCoord)
+{
+  if (NS_SUCCEEDED(mPageData->mPrintOptions->GetPrinterPrefInt(mPageData->mPrintSettings, 
+                                                               NS_ConvertASCIItoUCS2(aPrefName).get(), &aCoord))) {
+    nscoord inchInTwips = NS_INCHES_TO_TWIPS(1.0);
+    aCoord = PR_MAX(NS_INCHES_TO_TWIPS(float(aCoord)/100.0f), 0);
+    aCoord = PR_MIN(aCoord, inchInTwips); // an inch is still probably excessive
+  }
+}
+
+void nsSimplePageSequenceFrame::GetEdgePaperMargin(nsIPref* aPref, nsMargin& aMargin)
+{
+  aMargin.SizeTo(0,0,0,0);
+  GetEdgePaperMarginCoord(aPref, "print_edge_top",    aMargin.top);
+  GetEdgePaperMarginCoord(aPref, "print_edge_left",   aMargin.left);
+  GetEdgePaperMarginCoord(aPref, "print_edge_bottom", aMargin.bottom);
+  GetEdgePaperMarginCoord(aPref, "print_edge_right",  aMargin.right);
+}
+
 NS_IMETHODIMP
 nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
                                   nsHTMLReflowMetrics&     aDesiredSize,
@@ -235,6 +247,27 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
 
   aStatus = NS_FRAME_COMPLETE;  // we're always complete
 
+  // absolutely ignore all other types of reflows
+  // we only want to have done the Initial Reflow
+  if (eReflowReason_Resize == aReflowState.reason ||
+      eReflowReason_Incremental == aReflowState.reason ||
+      eReflowReason_StyleChange == aReflowState.reason ||
+      eReflowReason_Dirty == aReflowState.reason) {
+    // Return our desired size
+    aDesiredSize.height  = mSize.height;
+    aDesiredSize.width   = mSize.width;
+    aDesiredSize.ascent  = aDesiredSize.height;
+    aDesiredSize.descent = 0;
+    return NS_OK;
+  }
+
+  // Turn on the scaling of twips so any of the scrollbars
+  // in the UI no longer get scaled
+  nsCOMPtr<nsIPrintPreviewContext> printPreviewContext(do_QueryInterface(aPresContext));
+  if (printPreviewContext) {
+    printPreviewContext->SetScalingOfTwips(PR_TRUE);
+  }
+
   nsCOMPtr<nsIPrintPreviewContext> ppContext = do_QueryInterface(aPresContext);
 
   // See if we can get a Print Settings from the Context
@@ -252,6 +285,10 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
   // now get out margins
   if (mPageData->mPrintSettings) {
     mPageData->mPrintSettings->GetMarginInTwips(mMargin);
+    PRInt16 printType;
+    mPageData->mPrintSettings->GetPrintRange(&printType);
+    mPrintRangeType = printType;
+    mIsPrintingSelection = nsIPrintSettings::kRangeSelection == printType;
   }
 
   // *** Special Override ***
@@ -274,27 +311,25 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
   nscoord extraGap = 0;
   nsCOMPtr<nsIPref> pref = do_GetService(NS_PREF_CONTRACTID);
   if (pref) {
-    nscoord inchInTwips = NS_INCHES_TO_TWIPS(1.0);
+    GetEdgePaperMargin(pref, mPageData->mEdgePaperMargin);
+    nscoord extraThreshold = PR_MAX(pageSize.width, pageSize.height)/10;
     PRInt32 gapInTwips;
-    if (NS_SUCCEEDED(pref->GetIntPref("print.print_headerfooter_gap", &gapInTwips))) {
-      gapInTwips = PR_MAX(gapInTwips, 0);
-      gapInTwips = PR_MIN(gapInTwips, inchInTwips); // an inch is still probably excessive
-      mPageData->mHeadFooterGap = nscoord(gapInTwips);
-    }
     if (NS_SUCCEEDED(pref->GetIntPref("print.print_extra_margin", &gapInTwips))) {
       gapInTwips = PR_MAX(gapInTwips, 0);
-      gapInTwips = PR_MIN(gapInTwips, inchInTwips); // an inch is still probably excessive
-      extraGap = nscoord(gapInTwips);
+      gapInTwips = PR_MIN(gapInTwips, extraThreshold); // clamp to 1/10 of the largest dim of the page
+      extraGap   = nscoord(gapInTwips);
     }
   }
 
-  nscoord  quarterInch = NS_INCHES_TO_TWIPS(0.25);
+  nscoord  deadSpaceGap;
+  GetDeadSpaceValue(&deadSpaceGap);
+
   nsMargin deadSpaceMargin(0,0,0,0);
   nsMargin extraMargin(0,0,0,0);
   nsSize   shadowSize(0,0);
   if (ppContext) {
     if (adjSize.width == width && adjSize.height == height) {
-      deadSpaceMargin.SizeTo(quarterInch, quarterInch, quarterInch, quarterInch);
+      deadSpaceMargin.SizeTo(deadSpaceGap, deadSpaceGap, deadSpaceGap, deadSpaceGap);
       extraMargin.SizeTo(extraGap, extraGap, extraGap, extraGap);
       float p2t;
       aPresContext->GetScaledPixelsToTwips(&p2t);
@@ -307,20 +342,6 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
   mPageData->mExtraMargin     = extraMargin;
   mPageData->mDeadSpaceMargin = deadSpaceMargin;
 
-  // absolutely ignore all other types of reflows
-  // we only want to have done the Initial Reflow
-  if (eReflowReason_Resize == aReflowState.reason ||
-      eReflowReason_Incremental == aReflowState.reason ||
-      eReflowReason_StyleChange == aReflowState.reason ||
-      eReflowReason_Dirty == aReflowState.reason) {
-    // Return our desired size
-    aDesiredSize.height  = mSize.height;
-    aDesiredSize.width   = mSize.width;
-    aDesiredSize.ascent  = aDesiredSize.height;
-    aDesiredSize.descent = 0;
-    return NS_OK;
-  }
-
   PRBool suppressLeftMargin   = PR_FALSE;
   PRBool suppressRightMargin  = PR_FALSE;
   PRBool suppressTopMargin    = PR_FALSE;
@@ -328,8 +349,8 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
 
   if (pageSize != adjSize &&
       (adjSize.x != 0 || adjSize.y != 0 || adjSize.width != 0 || adjSize.height != 0)) {
-    suppressLeftMargin   = pageSize.x != adjSize.x;
-    suppressTopMargin    = pageSize.y != adjSize.y;
+    suppressLeftMargin   = pageSize.x != adjSize.x || (pageSize.x == adjSize.x && !adjSize.x);
+    suppressTopMargin    = pageSize.y != adjSize.y || (pageSize.y == adjSize.y && !adjSize.y);
     if (pageSize.width  != adjSize.width) {
       suppressRightMargin = PR_TRUE;
       pageSize.width = adjSize.width;
@@ -373,6 +394,9 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
     // in fact, all we want is the initial reflow
     y = mRect.height;
   } else {
+    // XXX Part of Temporary fix for Bug 127263
+    nsPageFrame::SetCreateWidget(PR_TRUE);
+
     nsReflowReason  reflowReason = aReflowState.reason;
 
     SetPageSizes(pageSize, margin);
@@ -381,17 +405,28 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
     nsHTMLReflowMetrics kidSize(nsnull);
     for (nsIFrame* kidFrame = mFrames.FirstChild(); nsnull != kidFrame; ) {
       // Reflow the page
+      // The availableHeight always comes in NS_UNCONSTRAINEDSIZE, so we need to check
+      // the "adjusted rect" to see if that is being reflowed NS_UNCONSTRAINEDSIZE or not
+      // When it is NS_UNCONSTRAINEDSIZE it means we are reflowing a single page
+      // to print selection. So this means we want to use NS_UNCONSTRAINEDSIZE without altering it
+      nsRect actualRect;
+      nsRect adjRect;
+      aPresContext->GetPageDim(&actualRect, &adjRect);
+      nscoord avHeight;
+      if (adjRect.height == NS_UNCONSTRAINEDSIZE) {
+        avHeight = NS_UNCONSTRAINEDSIZE;
+      } else {
+        avHeight = pageSize.height+deadSpaceMargin.top+deadSpaceMargin.bottom+shadowSize.height+extraMargin.top+extraMargin.bottom;
+      }
       nsSize availSize(pageSize.width+deadSpaceMargin.right+deadSpaceMargin.left+shadowSize.width+extraMargin.right+extraMargin.left, 
-                       pageSize.height+deadSpaceMargin.top+deadSpaceMargin.bottom+shadowSize.height+extraMargin.top+extraMargin.bottom);
+                       avHeight);
       nsHTMLReflowState kidReflowState(aPresContext, aReflowState, kidFrame,
                                        availSize, reflowReason);
       nsReflowStatus  status;
-      kidReflowState.availableWidth  = availSize.width;
-      kidReflowState.availableHeight = availSize.height;
 
       kidReflowState.mComputedWidth  = kidReflowState.availableWidth;
       //kidReflowState.mComputedHeight = kidReflowState.availableHeight;
-      PRINT_DEBUG_MSG3("AV W: %d   H: %d\n", kidReflowState.availableWidth, kidReflowState.availableHeight);
+      PR_PL(("AV W: %d   H: %d\n", kidReflowState.availableWidth, kidReflowState.availableHeight));
 
       // Set the shared data into the page frame before reflow
       nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, kidFrame);
@@ -406,8 +441,19 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
       FinishReflowChild(kidFrame, aPresContext, nsnull, kidSize, x, y, 0);
       y += kidSize.height;
 
+      // XXX Temporary fix for Bug 127263
+      // This tells the nsPageFrame class to stop creating clipping widgets
+      // once we reach the 32k boundary for positioning
+      if (nsPageFrame::GetCreateWidget()) {
+        float t2p;
+        aPresContext->GetTwipsToPixels(&t2p);
+        nscoord xp = NSTwipsToIntPixels(x, t2p);
+        nscoord yp = NSTwipsToIntPixels(y, t2p);
+        nsPageFrame::SetCreateWidget(xp < 0x8000 && yp < 0x8000);
+      }
+
       // Leave a slight gap between the pages
-      y += quarterInch;
+      y += deadSpaceGap;
 
       // Is the page complete?
       nsIFrame* kidNextInFlow;
@@ -484,7 +530,14 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
   mSize.width  = aDesiredSize.width;
   mSize.height = aDesiredSize.height;
 
+  // Turn off the scaling of twips so any of the scrollbars
+  // in the document get scaled
+  if (printPreviewContext) {
+    printPreviewContext->SetScalingOfTwips(PR_FALSE);
+  }
+
   NS_FRAME_TRACE_REFLOW_OUT("nsSimplePageSequeceFrame::Reflow", aStatus);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -585,23 +638,11 @@ nsSimplePageSequenceFrame::SetPageNumberFormat(const char* aPropName, const char
 
 }
 
-static nsIRegion* CreateRegion()
-{
-  nsIRegion* region;
-  nsresult rv = nsComponentManager::CreateInstance(kRegionCID, nsnull, NS_GET_IID(nsIRegion), (void**)&region);
-  if (NS_SUCCEEDED(rv)) {
-    if (NS_SUCCEEDED(region->Init())) {
-      return region;
-    } else {
-      NS_RELEASE(region);
-    }
-  }
-  return nsnull;
-}
-
 NS_IMETHODIMP
-nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
-                                      nsIPrintSettings* aPrintSettings)
+nsSimplePageSequenceFrame::StartPrint(nsIPresContext*   aPresContext,
+                                      nsIPrintSettings* aPrintSettings,
+                                      PRUnichar*        aDocTitle,
+                                      PRUnichar*        aDocURL)
 {
   NS_ENSURE_ARG_POINTER(aPresContext);
   NS_ENSURE_ARG_POINTER(aPrintSettings);
@@ -610,10 +651,10 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
     mPageData->mPrintSettings = aPrintSettings;
   }
 
-  PRInt16 printType;
-  aPrintSettings->GetPrintRange(&printType);
-  mPrintRangeType = printType;
-  mIsPrintingSelection = nsIPrintSettings::kRangeSelection == printType;
+  // Only set them if they are not null
+  if (aDocTitle) mPageData->mDocTitle = aDocTitle;
+  if (aDocURL) mPageData->mDocURL   = aDocURL;
+
   aPrintSettings->GetMarginInTwips(mMargin);
 
   aPrintSettings->GetStartPageRange(&mFromPageNum);
@@ -638,10 +679,6 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
   aPresContext->GetDeviceContext(getter_AddRefs(dc));
   NS_ASSERTION(dc, "nsIDeviceContext can't be NULL!");
 
-  nsCOMPtr<nsIPresShell> presShell;
-  aPresContext->GetShell(getter_AddRefs(presShell));
-  NS_ASSERTION(presShell, "nsIPresShell can't be NULL!");
-
   nsresult rv = NS_OK;
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
@@ -650,14 +687,14 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
     GetView(aPresContext, &seqView);
     nsRect rect;
     GetRect(rect);
-    fprintf(mDebugFD, "Seq Frame: %p - [%5d,%5d,%5d,%5d] ", this, rect.x, rect.y, rect.width, rect.height);
-    fprintf(mDebugFD, "view: %p ", seqView);
+    PR_PL(("Seq Frame: %p - [%5d,%5d,%5d,%5d] ", this, rect.x, rect.y, rect.width, rect.height));
+    PR_PL(("view: %p ", seqView));
     nsRect viewRect;
     if (seqView) {
       seqView->GetBounds(viewRect);
-      fprintf(mDebugFD, " [%5d,%5d,%5d,%5d]", viewRect.x, viewRect.y, viewRect.width, viewRect.height);
+      PR_PL((" [%5d,%5d,%5d,%5d]", viewRect.x, viewRect.y, viewRect.width, viewRect.height));
     }
-    fprintf(mDebugFD, "\n");
+    PR_PL(("\n"));
   }
 
   {
@@ -670,8 +707,8 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
       page->GetRect(rect);
       nsRect viewRect;
       view->GetBounds(viewRect);
-      fprintf(mDebugFD, " Page: %p  No: %d - [%5d,%5d,%5d,%5d] ", page, pageNum, rect.x, rect.y, rect.width, rect.height);
-      fprintf(mDebugFD, " [%5d,%5d,%5d,%5d]\n", viewRect.x, viewRect.y, viewRect.width, viewRect.height);
+      PR_PL((" Page: %p  No: %d - [%5d,%5d,%5d,%5d] ", page, pageNum, rect.x, rect.y, rect.width, rect.height));
+      PR_PL((" [%5d,%5d,%5d,%5d]\n", viewRect.x, viewRect.y, viewRect.width, viewRect.height));
       pageNum++;
     }
   }
@@ -691,7 +728,6 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
     PRInt32 pageNum = 1;
     nscoord y = 0;//mMargin.top;
 
-    nsCOMPtr<nsIRegion> emptyRegion = getter_AddRefs(CreateRegion());
     for (nsIFrame* page = mFrames.FirstChild(); nsnull != page; page->GetNextSibling(&page)) {
       nsIView*  view = nsnull;
       page->GetView(aPresContext, &view);
@@ -707,7 +743,8 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
         // sure the child views don't get asked to print
         // but my guess is that there won't be any
         vm->SetViewVisibility(view, nsViewVisibility_kHide);
-        vm->SetViewChildClipRegion(view, emptyRegion);
+        nsRegion emptyRegion;
+        vm->SetViewChildClipRegion(view, &emptyRegion);
       } else {
         nsRect rect;
         page->GetRect(rect);
@@ -729,9 +766,7 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
     }
 
     // adjust total number of pages
-    if (nsIPrintSettings::kRangeSelection == mPrintRangeType) {
-      totalPages = mToPageNum - mFromPageNum + 1;
-    } else {
+    if (nsIPrintSettings::kRangeSelection != mPrintRangeType) {
       totalPages = pageNum - 1;
     }
   }
@@ -768,9 +803,12 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*  aPresContext,
 
   mPageNum          = 1;
   mPrintedPageNum   = 1;
-  mTotalPages       = totalPages;
   mCurrentPageFrame = mFrames.FirstChild();
 
+  if (mTotalPages == -1) {
+    mTotalPages = totalPages;
+  }
+
   return rv;
 }
 
@@ -840,14 +878,6 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
   }
 
   if (mPrintThisPage) {
-    if (!mSkipPageBegin) {
-      PRINT_DEBUG_MSG1("\n***************** BeginPage *****************\n");
-      rv = dc->BeginPage();
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-    }
-
     // XXX This is temporary fix for printing more than one page of a selection
     // This does a poor man's "dump" pagination (see Bug 89353)
     // It has laid out as one long page and now we are just moving or view up/down 
@@ -859,6 +889,7 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
     PRInt32 width, height;
     dc->GetDeviceSurfaceDimensions(width, height);
     nsRect clipRect(0, 0, width, height);
+    nsRect slidingRect(-1, -1, -1, -1);
     height -= mMargin.top + mMargin.bottom;
     width  -= mMargin.left + mMargin.right;
     nscoord selectionY = height;
@@ -872,25 +903,32 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
       NS_ASSERTION(containerView != nsnull, "Container view can't be null!");
       containerView->GetBounds(containerRect);
       containerRect.y -= mYSelOffset;
-      containerRect.height = height-mYSelOffset;
+      slidingRect.SetRect(0,mYSelOffset,width,height);
       
       vm->MoveViewTo(containerView, containerRect.x, containerRect.y);
       nsRect r(0, 0, containerRect.width, containerRect.height);
       vm->ResizeView(containerView, r, PR_FALSE);
       clipRect.SetRect(mMargin.left, mMargin.right, width, height);
-
-      nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, childFrame);
-      nsRect nullClipRect(-1,-1,-1,-1);
-      pf->SetClipRect(&nullClipRect);
     }
 
     while (continuePrinting) {
+      if (!mSkipPageBegin) {
+        PR_PL(("\n"));
+        PR_PL(("***************** BeginPage *****************\n"));
+        rv = dc->BeginPage();
+        if (NS_FAILED(rv)) {
+          return rv;
+        }
+      }
 
       // cast the frame to be a page frame
       nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, mCurrentPageFrame);
       if (pf != nsnull) {
         pf->SetPageNumInfo(mPrintedPageNum, mTotalPages);
         pf->SetSharedPageData(mPageData);
+        if (mSelectionHeight > -1) {
+          pf->SetClipRect(&slidingRect);
+        }
       }
 
       // Print the page
@@ -899,8 +937,8 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
 
       NS_ASSERTION(nsnull != view, "no page view");
 
-      PRINT_DEBUG_MSG4("SeqFr::Paint -> %p PageNo: %d  View: %p", pf, mPageNum, view);
-      PRINT_DEBUG_MSG3(" At: %d,%d\n", mMargin.left+mOffsetX, mMargin.top+mOffsetY);
+      PR_PL(("SeqFr::Paint -> %p PageNo: %d  View: %p", pf, mPageNum, view));
+      PR_PL((" At: %d,%d\n", mMargin.left+mOffsetX, mMargin.top+mOffsetY));
 
       vm->SetViewContentTransparency(view, PR_FALSE);
 
@@ -915,7 +953,7 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
       //view->SetVisibility(nsViewVisibility_kHide);
 
       if (!mSkipPageEnd) {
-        PRINT_DEBUG_MSG1("***************** End Page (PrintNextPage) *****************\n");
+        PR_PL(("***************** End Page (PrintNextPage) *****************\n"));
         rv = dc->EndPage();
         if (NS_FAILED(rv)) {
           return rv;
@@ -932,6 +970,10 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
         vm->MoveViewTo(containerView, containerRect.x, containerRect.y);
         nsRect r(0, 0, containerRect.width, containerRect.height);
         vm->ResizeView(containerView, r, PR_FALSE);
+        if (pf != nsnull) {
+          slidingRect.y += height;
+        }
+
       } else {
         continuePrinting = PR_FALSE;
       }
@@ -962,7 +1004,7 @@ nsSimplePageSequenceFrame::DoPageEnd(nsIPresContext*  aPresContext)
     NS_ASSERTION(dc, "nsIDeviceContext can't be NULL!");
 
     if(mSkipPageEnd){
-	    PRINT_DEBUG_MSG1("***************** End Page (DoPageEnd) *****************\n");
+	    PR_PL(("***************** End Page (DoPageEnd) *****************\n"));
       nsresult rv = dc->EndPage();
 	    if (NS_FAILED(rv)) {
 	      return rv;
@@ -1008,21 +1050,6 @@ nsSimplePageSequenceFrame::SetClipRect(nsIPresContext*  aPresContext, nsRect* aR
   return NS_OK;
 }
 
-#ifdef NS_DEBUG
-NS_IMETHODIMP 
-nsSimplePageSequenceFrame::SetDebugFD(FILE* aFD)
-{
-  mDebugFD = aFD;
-  for (nsIFrame* f = mFrames.FirstChild(); f != nsnull; f->GetNextSibling(&f)) {
-    nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, f);
-    if (pf != nsnull) {
-      pf->SetDebugFD(aFD);
-    }
-  }
-  return NS_OK;
-}
-#endif
-
 //------------------------------------------------------------------------------
 NS_IMETHODIMP
 nsSimplePageSequenceFrame::Paint(nsIPresContext*      aPresContext,
@@ -1054,6 +1081,14 @@ NS_IMETHODIMP nsSimplePageSequenceFrame::SizeTo(nsIPresContext* aPresContext, ns
   return nsFrame::SizeTo(aPresContext, aWidth, aHeight);
 }
 
+NS_IMETHODIMP
+nsSimplePageSequenceFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::sequenceFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
 
 //------------------------------------------------------------------------------
 void
@@ -1097,3 +1132,19 @@ nsSimplePageSequenceFrame::SetPageSizes(const nsRect& aRect, const nsMargin& aMa
   mPageData->mReflowRect   = aRect;
   mPageData->mReflowMargin = aMarginRect;
 }
+
+//------------------------------------------------------------------------------
+// For Shrink To Fit
+//
+// Return the percentage that the page needs to shrink to 
+//
+NS_IMETHODIMP
+nsSimplePageSequenceFrame::GetSTFPercent(float& aSTFPercent)
+{
+  NS_ENSURE_TRUE(mPageData, NS_ERROR_UNEXPECTED);
+  aSTFPercent = 1.0f;
+  if (mPageData && (mPageData->mPageContentXMost > mPageData->mPageContentSize)) {
+    aSTFPercent = float(mPageData->mPageContentSize) / float(mPageData->mPageContentXMost);
+  }
+  return NS_OK;
+}
