@@ -321,6 +321,10 @@ public:
 
   void PaintFocus(nsIRenderingContext& aRC, nsFramePaintLayer aWhichLayer);
 
+#ifdef ACCESSIBILITY
+  void FireMenuItemActiveEvent(); // Inform assistive tech what got focused
+#endif
+
 protected:
 
   nsresult   IsOptionDisabled(PRInt32 anIndex, PRBool &aIsDisabled);
@@ -430,6 +434,9 @@ protected:
 private:
   NS_IMETHOD_(nsrefcnt) AddRef() { return NS_OK; }
   NS_IMETHOD_(nsrefcnt) Release() { return NS_OK; }
+
+  // for incremental typing navigation
+  static nsAString& GetIncrementalString ();
 };
 
 #endif /* nsListControlFrame_h___ */
