@@ -67,7 +67,8 @@ struct nsTableCellReflowState : public nsHTMLReflowState
                          const nsSize&            aAvailableSpace,
                          nsReflowReason           aReason);
 
-  void FixUp(const nsSize& aAvailSpace);
+  void FixUp(const nsSize& aAvailSpace,
+             PRBool        aResetComputedWidth);
 };
 
 nsTableCellReflowState::nsTableCellReflowState(nsIPresContext*          aPresContext,
@@ -79,11 +80,15 @@ nsTableCellReflowState::nsTableCellReflowState(nsIPresContext*          aPresCon
 {
 }
 
-void nsTableCellReflowState::FixUp(const nsSize& aAvailSpace)
+void nsTableCellReflowState::FixUp(const nsSize& aAvailSpace,
+                                   PRBool        aResetComputedWidth)
 {
   // fix the mComputed values during a pass 2 reflow since the cell can be a percentage base
   if (NS_UNCONSTRAINEDSIZE != aAvailSpace.width) {
-    if (NS_UNCONSTRAINEDSIZE != mComputedWidth) {
+    if (aResetComputedWidth) {
+      mComputedWidth = NS_UNCONSTRAINEDSIZE;
+    }
+    else if (NS_UNCONSTRAINEDSIZE != mComputedWidth) {
       mComputedWidth = aAvailSpace.width - mComputedBorderPadding.left - mComputedBorderPadding.right;
       mComputedWidth = PR_MAX(0, mComputedWidth);
     }
@@ -101,7 +106,8 @@ nsTableRowFrame::InitChildReflowState(nsIPresContext&         aPresContext,
                                       const nsSize&           aAvailSize,
                                       PRBool                  aBorderCollapse,
                                       float                   aPixelsToTwips,
-                                      nsTableCellReflowState& aReflowState)                                    
+                                      nsTableCellReflowState& aReflowState,
+                                      PRBool                  aResetComputedWidth)
 {
   nsMargin collapseBorder;
   nsMargin* pCollapseBorder = nsnull;
@@ -113,7 +119,7 @@ nsTableRowFrame::InitChildReflowState(nsIPresContext&         aPresContext,
     }
   }
   aReflowState.Init(&aPresContext, -1, -1, pCollapseBorder);
-  aReflowState.FixUp(aAvailSize);
+  aReflowState.FixUp(aAvailSize, aResetComputedWidth);
 }
 
 void 
@@ -790,8 +796,11 @@ CalcAvailWidth(nsTableFrame&     aTableFrame,
   } 
   aCellAvailWidth = aColAvailWidth;
 
-  // for a cell with a colspan > 1, use its fix width (if set) as the  avail width 
-  if (aTableFrame.GetEffectiveColSpan(aCellFrame) > 1) {
+  nsFrameState  frameState;
+  aCellFrame.GetFrameState(&frameState);
+  // for a cell with a colspan > 1, use its fix width (if set) as the avail width 
+  // if this is its initial reflow
+  if ((frameState & NS_FRAME_FIRST_REFLOW) && (aTableFrame.GetEffectiveColSpan(aCellFrame) > 1)) {
     // see if the cell has a style width specified
     const nsStylePosition* cellPosition;
     aCellFrame.GetStyleData(eStyleStruct_Position, (const nsStyleStruct *&)cellPosition);
@@ -940,6 +949,13 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
     if (aDirtyOnly && ((frameState & NS_FRAME_IS_DIRTY) == 0)) {
       doReflowChild = PR_FALSE;
     }
+    else if ((NS_UNCONSTRAINEDSIZE != aReflowState.availableHeight) && IS_TABLE_CELL(frameType)) {
+      // We don't reflow a rowspan >1 cell here with a constrained height. 
+      // That happens in nsTableRowGroupFrame::SplitSpanningCells.
+      if (aTableFrame.GetEffectiveRowSpan((nsTableCellFrame&)*kidFrame) > 1) {
+        doReflowChild = PR_FALSE;
+      }
+    }
     if (aReflowState.mFlags.mSpecialHeightReflow) {
       if (!isPaginated && (IS_TABLE_CELL(frameType.get()) &&
                            !((nsTableCellFrame*)kidFrame)->NeedSpecialReflow())) {
@@ -1280,7 +1296,13 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     GET_PIXELS_TO_TWIPS(aPresContext, p2t);
     nsTableCellReflowState kidRS(aPresContext, aReflowState, aNextFrame, cellAvailSize, 
                                  aReflowState.reason);
-    InitChildReflowState(*aPresContext, cellAvailSize, aTableFrame.IsBorderCollapse(), p2t, kidRS); 
+    // If the table will intialize the strategy (and balance) or balance, make the computed 
+    // width unconstrained. This avoids having the cell block compute a bogus max width 
+    // which will bias the balancing. Leave the avail width alone, since it is a best guess.
+    // After the table balances, the cell will get reflowed with the correct computed width.
+    PRBool resetComputedWidth = aTableFrame.NeedStrategyInit() || aTableFrame.NeedStrategyBalance();
+    InitChildReflowState(*aPresContext, cellAvailSize, aTableFrame.IsBorderCollapse(), 
+                         p2t, kidRS, resetComputedWidth); 
 
     // Remember the current desired size, we'll need it later
     nscoord oldCellMinWidth     = cellFrame->GetPass1MaxElementWidth();
@@ -1327,7 +1349,7 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
         PRInt32 rowSpan = aTableFrame.GetEffectiveRowSpan(*cellFrame);
         if (rowSpan == 1) {
          tallestCellGotShorter = 
-           TallestCellGotShorter(oldCellDesAscent, cellMet.descent, mMaxCellDescent);
+           TallestCellGotShorter(oldCellDesDescent, cellMet.descent, mMaxCellDescent);
         }
       }
     }
@@ -1530,11 +1552,16 @@ nsTableRowFrame::ReflowCellFrame(nsIPresContext*          aPresContext,
 
   ReflowChild(aCellFrame, aPresContext, desiredSize, cellReflowState,
               0, 0, NS_FRAME_NO_MOVE_FRAME, aStatus);
-  aCellFrame->SizeTo(aPresContext, cellSize.width, aAvailableHeight);
+  PRBool fullyComplete = NS_FRAME_IS_COMPLETE(aStatus) && !NS_FRAME_IS_TRUNCATED(aStatus);
+
+  aCellFrame->SizeTo(aPresContext, cellSize.width, 
+                     (fullyComplete) ? aAvailableHeight : desiredSize.height);
 
   // XXX What happens if this cell has 'vertical-align: baseline' ?
   // XXX Why is it assumed that the cell's ascent hasn't changed ?
-  aCellFrame->VerticallyAlignChild(aPresContext, aReflowState, mMaxCellAscent);
+  if (fullyComplete) {
+    aCellFrame->VerticallyAlignChild(aPresContext, aReflowState, mMaxCellAscent);
+  }
   aCellFrame->DidReflow(aPresContext, nsnull, NS_FRAME_REFLOW_FINISHED);
 
   return desiredSize.height;
