@@ -20,19 +20,14 @@
  */
 
 #include "nsCOMPtr.h"
-#include "nsHTMLParts.h"
-#include "nsIHTMLContent.h"
 #include "nsFrame.h"
-#include "nsLineLayout.h"
 #include "nsIPresContext.h"
-#include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsINameSpaceManager.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
-#include "nsStyleUtil.h"
 
 #include "nsCSSRendering.h"
 #include "prprf.h"         // For PR_snprintf()
@@ -274,12 +269,14 @@ nsMathMLmactionFrame::Paint(nsIPresContext*      aPresContext,
       PRIntn skipSides = GetSkipSides();
       const nsStyleBorder* border = (const nsStyleBorder*)
         mStyleContext->GetStyleData(eStyleStruct_Border);
+      const nsStylePadding* padding = (const nsStylePadding*)
+        mStyleContext->GetStyleData(eStyleStruct_Padding);
       const nsStyleOutline* outline = (const nsStyleOutline*)
         mStyleContext->GetStyleData(eStyleStruct_Outline);
 
       nsRect  rect(0, 0, mRect.width, mRect.height);
       nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                      aDirtyRect, rect, *border, 0, 0);
+                                      aDirtyRect, rect, *border, *padding, 0, 0);
       nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
                                   aDirtyRect, rect, *border, mStyleContext, skipSides);
       nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, this,
