@@ -310,18 +310,19 @@ nsSliderFrame::Paint(nsIPresContext*      aPresContext,
     if (crect.width < thumbRect.width || crect.height < thumbRect.height)
     {
       if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
-      const nsStyleVisibility* vis =
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
+      const nsStyleVisibility* vis = (const nsStyleVisibility*)
+        mStyleContext->GetStyleData(eStyleStruct_Visibility);
       if (vis->IsVisibleOrCollapsed()) {
-        const nsStyleBackground* myColor = (const nsStyleBackground*)
-        mStyleContext->GetStyleData(eStyleStruct_Background);
         const nsStyleBorder* myBorder = (const nsStyleBorder*)
-        mStyleContext->GetStyleData(eStyleStruct_Border);
+          mStyleContext->GetStyleData(eStyleStruct_Border);
+        const nsStylePadding* myPadding = (const nsStylePadding*)
+          mStyleContext->GetStyleData(eStyleStruct_Padding);
         nsRect rect(0, 0, mRect.width, mRect.height);
         nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                    aDirtyRect, rect, *myBorder, 0, 0);
+                                        aDirtyRect, rect, *myBorder, *myPadding,
+                                        0, 0);
         nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
-                                aDirtyRect, rect, *myBorder, mStyleContext, 0);
+                                    aDirtyRect, rect, *myBorder, mStyleContext, 0);
         }
       }
       return NS_OK;
@@ -1246,10 +1247,11 @@ nsSliderFrame::SetScrollbarListener(nsIScrollbarListener* aListener)
   mScrollbarListener = aListener;
 }
 
-NS_IMETHODIMP_(void) nsSliderMediator::Notify(nsITimer *timer)
+NS_IMETHODIMP nsSliderMediator::Notify(nsITimer *timer)
 {
   if (mSlider)
     mSlider->Notify(timer);
+  return NS_OK;
 }
 
 NS_IMETHODIMP_(void) nsSliderFrame::Notify(nsITimer *timer)
