@@ -1709,7 +1709,7 @@ nsBoxSize::Clear()
 
 
 void* 
-nsBoxSize::operator new(size_t sz, nsBoxLayoutState& aState)
+nsBoxSize::operator new(size_t sz, nsBoxLayoutState& aState) CPP_THROW_NEW
 {
    void* mem = 0;
    aState.AllocateStackMemory(sz,&mem);
@@ -1724,7 +1724,7 @@ nsBoxSize::operator delete(void* aPtr, size_t sz)
 
 
 void* 
-nsComputedBoxSize::operator new(size_t sz, nsBoxLayoutState& aState)
+nsComputedBoxSize::operator new(size_t sz, nsBoxLayoutState& aState) CPP_THROW_NEW
 {
   
    void* mem = 0;
