@@ -49,6 +49,7 @@
 #include "nsIMsgMailSession.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgFolderFlags.h"
+#include "nsMsgLocalSearch.h"
 
 NS_IMPL_ISUPPORTS4(nsMsgSearchSession, nsIMsgSearchSession, nsIUrlListener, nsIFolderListener, nsISupportsWeakReference)
 
@@ -197,16 +198,23 @@ nsMsgSearchSession::AddScopeTerm(nsMsgSearchScopeValue scope,
 			return NS_ERROR_NULL_POINTER;
 	}
 
-	nsresult err = NS_OK;
-
 	nsMsgSearchScopeTerm *pScopeTerm = new nsMsgSearchScopeTerm(this, scope, folder);
-	if (pScopeTerm)
-		m_scopeList.AppendElement(pScopeTerm);
-		else
-			err = NS_ERROR_OUT_OF_MEMORY;
-	
+  if (!pScopeTerm)
+    return NS_ERROR_OUT_OF_MEMORY;
 
-	return err;
+  m_scopeList.AppendElement(pScopeTerm);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgSearchSession::AddDirectoryScopeTerm(nsMsgSearchScopeValue scope)
+{
+	nsMsgSearchScopeTerm *pScopeTerm = new nsMsgSearchScopeTerm(this, scope, nsnull);
+  if (!pScopeTerm)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  m_scopeList.AppendElement(pScopeTerm);
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgSearchSession::ClearScopes()
@@ -215,12 +223,6 @@ NS_IMETHODIMP nsMsgSearchSession::ClearScopes()
     return NS_OK;
 }
 
-/* [noscript] void AddLdapScope (in nsMsgDIRServer server); */
-NS_IMETHODIMP nsMsgSearchSession::AddLdapScope(nsMsgDIRServer * server)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 /* [noscript] boolean ScopeUsesCustomHeaders (in nsMsgSearchScope scope, in voidStar selection, in boolean forFilters); */
 NS_IMETHODIMP
 nsMsgSearchSession::ScopeUsesCustomHeaders(nsMsgSearchScopeValue scope,
@@ -275,8 +277,13 @@ NS_IMETHODIMP nsMsgSearchSession::InterruptSearch()
 {
   if (m_window)
   {
-    ReleaseFolderDBRef();
-    m_idxRunningScope = m_scopeList.Count(); // this'll make us not run another url
+    EnableFolderNotifications(PR_TRUE);
+    while (m_idxRunningScope < m_scopeList.Count())
+    {
+      ReleaseFolderDBRef();
+      m_idxRunningScope++;
+    }
+    //m_idxRunningScope = m_scopeList.Count() so it will make us not run another url
     m_window->StopUrls();
   }
   if (m_backgroundTimer)
@@ -366,6 +373,7 @@ NS_IMETHODIMP nsMsgSearchSession::OnStopRunningUrl(nsIURI *url, nsresult aExitCo
   if (NS_SUCCEEDED(rv) && runningAdapter)
   {
     runningAdapter->CurrentUrlDone(aExitCode);
+    EnableFolderNotifications(PR_TRUE);
     ReleaseFolderDBRef();
   }
   m_idxRunningScope++;
@@ -412,7 +420,6 @@ nsresult nsMsgSearchSession::Initialize()
 		// NS_ASSERTION(scopeTerm->IsValid());
 
 		err = scopeTerm->InitializeAdapter (m_termList);
-
 //		if (scopeTerm->m_folder->GetType() == FOLDER_MAIL)
 //			m_offlineProgressTotal += scopeTerm->m_folder->GetTotalMessages();
 	}
@@ -479,17 +486,18 @@ nsresult nsMsgSearchSession::GetNextUrl()
 
   m_urlQueue.CStringAt(m_idxRunningScope, nextUrl);
   nsMsgSearchScopeTerm *currentTerm = GetRunningScope();
+  EnableFolderNotifications(PR_FALSE);
   nsCOMPtr <nsIMsgFolder> folder = currentTerm->m_folder;
   if (folder)
   {
     nsXPIDLCString folderUri;
     folder->GetURI(getter_Copies(folderUri));
-  nsresult rv = GetMessageServiceFromURI(folderUri.get(), getter_AddRefs(msgService));
+    nsresult rv = GetMessageServiceFromURI(folderUri.get(), getter_AddRefs(msgService));
 
-  if (NS_SUCCEEDED(rv) && msgService && currentTerm)
-    msgService->Search(this, m_window, currentTerm->m_folder, nextUrl.get());
+    if (NS_SUCCEEDED(rv) && msgService && currentTerm)
+      msgService->Search(this, m_window, currentTerm->m_folder, nextUrl.get());
 
-	return rv;
+	  return rv;
   }
   return NS_OK;
 }
@@ -525,13 +533,15 @@ nsresult nsMsgSearchSession::StartTimer()
   PRBool done;
 
   m_backgroundTimer = do_CreateInstance("@mozilla.org/timer;1", &err);
-  m_backgroundTimer->Init(TimerCallback, (void *) this, 0, NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_SLACK);
+  m_backgroundTimer->InitWithFuncCallback(TimerCallback, (void *) this, 0, 
+                                          nsITimer::TYPE_REPEATING_SLACK);
   // ### start meteors?
   return TimeSlice(&done);
 }
 
 nsresult nsMsgSearchSession::SearchWOUrls ()
 {
+  EnableFolderNotifications(PR_FALSE);
   return StartTimer();
 }
 
@@ -671,28 +681,46 @@ nsresult nsMsgSearchSession::TimeSliceSerial (PRBool *aDone)
 	// disk, this is the fastest way to do it.
 
   NS_ENSURE_ARG(aDone);
-
-	nsMsgSearchScopeTerm *scope = GetRunningScope();
+  nsresult rv = NS_OK;
+  nsMsgSearchScopeTerm *scope = GetRunningScope();
   if (scope)
   {
-    scope->TimeSlice (aDone);
-    if (*aDone)
+    rv = scope->TimeSlice (aDone);
+    if (NS_FAILED(rv))
+      *aDone = PR_TRUE;
+    if (*aDone || NS_FAILED(rv))
     {
+      EnableFolderNotifications(PR_TRUE);
       ReleaseFolderDBRef();
       m_idxRunningScope++;
+      EnableFolderNotifications(PR_FALSE);
+
       //			if (m_idxRunningScope < m_scopeList.Count())
       //  			UpdateStatusBar (MK_MSG_SEARCH_STATUS);
-		}
+    }
     *aDone = PR_FALSE;
-    return NS_OK;
+    return rv;
   }
   else
-	{
+  {
     *aDone = PR_TRUE;
     return NS_OK;
   }
 }
 
+void 
+nsMsgSearchSession::EnableFolderNotifications(PRBool aEnable)
+{
+  nsMsgSearchScopeTerm *scope = GetRunningScope();
+  if (scope)
+  {
+    nsCOMPtr<nsIMsgFolder> folder;
+    scope->GetFolder(getter_AddRefs(folder));
+    if (folder)  //enable msg count notifications
+      folder->EnableNotifications(nsIMsgFolder::allMessageCountNotifications, aEnable, PR_FALSE);
+  }
+}
+
 NS_IMETHODIMP
 nsMsgSearchSession::AddFolderListener(nsIFolderListener *listener)
 {
@@ -803,3 +831,21 @@ nsMsgSearchSession::OnItemPropertyFlagChanged(nsISupports *item,
     return NS_OK;
 }
 
+//this method is used for adding new hdrs to quick search view
+NS_IMETHODIMP
+nsMsgSearchSession::MatchHdr(nsIMsgDBHdr *aMsgHdr, nsIMsgDatabase *aDatabase, PRBool *aResult)
+{
+  nsMsgSearchScopeTerm *scope = (nsMsgSearchScopeTerm *)m_scopeList.SafeElementAt(0);
+  if (scope)
+  {
+    if (scope->m_adapter)
+    {  
+      nsXPIDLString nullCharset, folderCharset;
+      scope->m_adapter->GetSearchCharsets(getter_Copies(nullCharset), getter_Copies(folderCharset));
+      NS_ConvertUCS2toUTF8 charset(folderCharset.get());
+      nsMsgSearchOfflineMail::MatchTermsForSearch(aMsgHdr, m_termList, charset.get(), scope, aDatabase, aResult);
+    }
+  }
+  return NS_OK;
+}
+
