@@ -67,6 +67,7 @@
 #include "nsNetCID.h"
 #include "nsISmtpService.h"
 #include "nsIMsgBiffManager.h"
+#include "nsIMsgPurgeService.h"
 #include "nsIObserverService.h"
 #include "nsIMsgMailSession.h"
 #include "nsIEventQueueService.h"
@@ -100,7 +101,6 @@ static NS_DEFINE_CID(kMsgAccountCID, NS_MSGACCOUNT_CID);
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 static NS_DEFINE_CID(kMsgBiffManagerCID, NS_MSGBIFFMANAGER_CID);
 static NS_DEFINE_CID(kMsgFolderCacheCID, NS_MSGFOLDERCACHE_CID);
-static NS_DEFINE_CID(kMsgAccountManagerCID, NS_MSGACCOUNTMANAGER_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
@@ -152,7 +152,7 @@ nsMsgAccountManager::nsMsgAccountManager() :
   m_shutdownInProgress(PR_FALSE),
   m_prefs(0)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsMsgAccountManager::~nsMsgAccountManager()
@@ -202,6 +202,11 @@ nsresult nsMsgAccountManager::Init()
 
 nsresult nsMsgAccountManager::Shutdown()
 {
+  if (m_haveShutdown) {
+    // do not shutdown twice
+    return NS_OK;
+  }
+
   nsresult rv;
 
   if(m_msgFolderCache)
@@ -209,13 +214,18 @@ nsresult nsMsgAccountManager::Shutdown()
     WriteToFolderCache(m_msgFolderCache);
   }
 
-  CloseCachedConnections();
-  UnloadAccounts();
+  (void)ShutdownServers();
+  (void)UnloadAccounts();
   
   //shutdown removes nsIIncomingServer listener from biff manager, so do it after accounts have been unloaded
   nsCOMPtr<nsIMsgBiffManager> biffService = do_GetService(NS_MSGBIFFMANAGER_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv) && biffService)
     biffService->Shutdown();
+
+  //shutdown removes nsIIncomingServer listener from purge service, so do it after accounts have been unloaded
+  nsCOMPtr<nsIMsgPurgeService> purgeService = do_GetService(NS_MSGPURGESERVICE_CONTRACTID, &rv);
+  if (NS_SUCCEEDED(rv) && purgeService)
+    purgeService->Shutdown();
   
   if (m_prefs) {
     nsServiceManager::ReleaseService(kPrefServiceCID, m_prefs);
@@ -819,16 +829,18 @@ nsMsgAccountManager::GetDefaultAccount(nsIMsgAccount * *aDefaultAccount)
 NS_IMETHODIMP
 nsMsgAccountManager::SetDefaultAccount(nsIMsgAccount * aDefaultAccount)
 {
-  nsCOMPtr<nsIMsgAccount> oldAccount = m_defaultAccount;
+  if (m_defaultAccount != aDefaultAccount)
+  {
+    nsCOMPtr<nsIMsgAccount> oldAccount = m_defaultAccount;
 
-  m_defaultAccount = aDefaultAccount;
+    m_defaultAccount = aDefaultAccount;
 
-  // it's ok if this fails
-  setDefaultAccountPref(aDefaultAccount);
+    // it's ok if this fails
+    setDefaultAccountPref(aDefaultAccount);
 
-  // ok if notifications fail
-  notifyDefaultServerChange(oldAccount, aDefaultAccount);
-  
+    // ok if notifications fail
+    notifyDefaultServerChange(oldAccount, aDefaultAccount);
+  }
   return NS_OK;
 }
 
@@ -863,6 +875,16 @@ nsMsgAccountManager::notifyDefaultServerChange(nsIMsgAccount *aOldAccount,
                                               PR_FALSE, PR_TRUE);
     }
   }
+
+  if (aOldAccount && aNewAccount)  //only notify if the user goes and changes default account
+  {
+    nsCOMPtr<nsIObserverService> observerService = 
+      do_GetService("@mozilla.org/observer-service;1", &rv);
+
+    if (NS_SUCCEEDED(rv))
+      observerService->NotifyObservers(nsnull,"mailDefaultAccountChanged",nsnull);
+  }
+
   return NS_OK;
 }
 
@@ -918,8 +940,8 @@ nsMsgAccountManager::hashUnloadServer(nsHashKey *aKey, void *aData,
 
 /* static */ void nsMsgAccountManager::LogoutOfServer(nsIMsgIncomingServer *aServer)
 {
-    nsresult rv = aServer->CloseCachedConnections();
-    NS_ASSERTION(NS_SUCCEEDED(rv), "CloseCachedConnections failed");
+    nsresult rv = aServer->Shutdown();
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Shutdown of server failed");
     rv = aServer->ForgetSessionPassword();
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to remove the password associated with server");
 }
@@ -1022,7 +1044,7 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
          {
            nsCOMPtr<nsIUrlListener> urlListener;
            nsCOMPtr<nsIMsgAccountManager> accountManager = 
-                    do_GetService(kMsgAccountManagerCID, &rv);
+                    do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
            if (NS_FAILED(rv)) return rv;
            nsCOMPtr<nsIEventQueueService> pEventQService = 
                     do_GetService(kEventQueueServiceCID, &rv);
@@ -1101,14 +1123,22 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
    return PR_TRUE;
 }
 
-
-// enumaration for closing cached connections.
 PRBool nsMsgAccountManager::closeCachedConnections(nsHashKey *aKey, void *aData, void *closure)
 {
-    nsIMsgIncomingServer *server = (nsIMsgIncomingServer*)aData;
+  nsIMsgIncomingServer *server = (nsIMsgIncomingServer*)aData;
 
-	server->CloseCachedConnections();
-	return PR_TRUE;
+  server->CloseCachedConnections();
+
+  return PR_TRUE;
+}
+
+PRBool nsMsgAccountManager::shutdown(nsHashKey *aKey, void *aData, void *closure)
+{
+  nsIMsgIncomingServer *server = (nsIMsgIncomingServer*)aData;
+
+  server->Shutdown();
+
+  return PR_TRUE;
 }
 
 
@@ -1295,6 +1325,13 @@ nsMsgAccountManager::LoadAccounts()
 
   if (NS_SUCCEEDED(rv))
     biffService->Init();
+
+  //Ensure purge service has started
+  nsCOMPtr<nsIMsgPurgeService> purgeService = 
+           do_GetService(NS_MSGPURGESERVICE_CONTRACTID, &rv);
+
+  if (NS_SUCCEEDED(rv))
+    purgeService->Init();
   
   // Ensure messenger OS integration service has started
   // note, you can't expect the integrationService to be there
@@ -1384,6 +1421,8 @@ nsMsgAccountManager::LoadAccounts()
       }
     }
   }
+
+  m_accountsLoaded = PR_TRUE;  //It is ok to return null accounts like when we create new profile
   
   if (!accountList || !accountList[0]) {
 #ifdef DEBUG_ACCOUNTMANAGER
@@ -1391,8 +1430,6 @@ nsMsgAccountManager::LoadAccounts()
 #endif
     return NS_OK;
   }
-
-  m_accountsLoaded = PR_TRUE;
   
     /* parse accountList and run loadAccount on each string, comma-separated */
 #ifdef DEBUG_ACCOUNTMANAGER
@@ -1528,7 +1565,7 @@ nsMsgAccountManager::SetSpecialFoldersForIdentities()
   return NS_OK;
 }
 
-nsresult
+NS_IMETHODIMP
 nsMsgAccountManager::UnloadAccounts()
 {
   // release the default account
@@ -1545,18 +1582,25 @@ nsMsgAccountManager::UnloadAccounts()
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgAccountManager::ShutdownServers()
+{
+  m_incomingServers.Enumerate(shutdown, nsnull);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMsgAccountManager::CloseCachedConnections()
 {
-	m_incomingServers.Enumerate(closeCachedConnections, nsnull);
-	return NS_OK;
+  m_incomingServers.Enumerate(closeCachedConnections, nsnull);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgAccountManager::CleanupOnExit()
 {
-	m_incomingServers.Enumerate(cleanupOnExit, nsnull);
-	return NS_OK;
+  m_incomingServers.Enumerate(cleanupOnExit, nsnull);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1974,11 +2018,21 @@ nsMsgAccountManager::GetFirstIdentityForServer(nsIMsgIncomingServer *aServer, ns
   nsresult rv = GetIdentitiesForServer(aServer, getter_AddRefs(identities));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIMsgIdentity> identity;
-  rv = identities->QueryElementAt(0, NS_GET_IID(nsIMsgIdentity),
-                                  (void **)getter_AddRefs(identity));
+  // not all servers have identities
+  // for example, Local Folders
+  PRUint32 numIdentities;
+  rv = identities->Count(&numIdentities);
   NS_ENSURE_SUCCESS(rv, rv);
-  NS_IF_ADDREF(*aIdentity = identity);
+
+  if (numIdentities > 0) {
+    nsCOMPtr<nsIMsgIdentity> identity;
+    rv = identities->QueryElementAt(0, NS_GET_IID(nsIMsgIdentity),
+                                  (void **)getter_AddRefs(identity));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_IF_ADDREF(*aIdentity = identity);
+  }
+  else
+    *aIdentity = nsnull;
   return rv;
 }
 
