@@ -59,7 +59,6 @@
 
 static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
 static NS_DEFINE_CID(kRDFServiceCID,							NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 //////////////////////////////////////////////////////////////////////////////
 // nsFolderCompactState
 //////////////////////////////////////////////////////////////////////////////
@@ -264,6 +263,7 @@ nsFolderCompactState::Init(nsIMsgFolder *folder, const char *baseMsgUri, nsIMsgD
 
   pathSpec->GetFileSpec(&m_fileSpec);
   m_fileSpec.SetLeafName("nstmp");
+  m_fileSpec.MakeUnique();   //make sure we are not crunching existing nstmp file
   m_window = aMsgWindow;
   m_keyArray.RemoveAll();
   InitDB(db);
@@ -328,13 +328,11 @@ nsresult nsFolderCompactState::StartCompacting()
   {
     m_folder->NotifyCompactCompleted();
     m_folder->ThrowAlertMsg("compactFolderDeniedLock", m_window);
+    CleanupTempFilesAfterError();
     if (m_compactAll)
-      CompactNextFolder();
+      return CompactNextFolder();
     else
-    {
-      CleanupTempFilesAfterError();
       return rv;
-    }
   }
   if (m_size > 0)
   {
