@@ -61,11 +61,12 @@
 #include "nsICharsetConverterManager2.h"
 #include "nsIChannel.h"
 #include "nsIRequestObserver.h"
+#include "netCore.h"
 
 #include "plbase64.h"
 #include "nsMsgI18N.h"
 #include "nsIWebNavigation.h"
-
+#include "nsISupportsObsolete.h"
 // XXX Remove
 #include "nsIWebShell.h"
 
@@ -208,7 +209,7 @@ NS_IMETHODIMP nsMsgWindow::SetStatusFeedback(nsIMsgStatusFeedback * aStatusFeedb
   if (webProgress && mStatusFeedback && messageWindowDocShell)
   {
     nsCOMPtr<nsIWebProgressListener> webProgressListener = do_QueryInterface(mStatusFeedback);
-    webProgress->AddProgressListener(webProgressListener);
+    webProgress->AddProgressListener(webProgressListener, nsIWebProgress::NOTIFY_ALL);
   }
 
 	return NS_OK;
@@ -476,7 +477,7 @@ NS_IMETHODIMP nsMsgWindow::CanHandleContent(const char * aContentType,
 
 {
   // the mail window knows nothing about the default content types
-  // it's docshell can handle...ask the content area if it can handle
+  // its docshell can handle...ask the content area if it can handle
   // the content type...
   
   nsCOMPtr<nsIDocShell> messageWindowDocShell; 
@@ -578,14 +579,14 @@ nsMsgWindow::DisplayHTMLInMessagePane(const PRUnichar *title, const PRUnichar *b
     nsCOMPtr <nsIURI> uri = do_CreateInstance("@mozilla.org/network/simple-uri;1");
     if (!uri) return NS_ERROR_UNEXPECTED;
 
-    rv = uri->SetSpec(dataSpec.get());
+    rv = uri->SetSpec(dataSpec);
     NS_ENSURE_SUCCESS(rv,rv);
 
     nsCOMPtr <nsIDocShell> docShell;
     GetMessageWindowDocShell(getter_AddRefs(docShell));
     if (!docShell) return NS_ERROR_UNEXPECTED;
 
-    rv = docShell->LoadURI(uri,nsnull,nsIWebNavigation::LOAD_FLAGS_NONE);
+    rv = docShell->LoadURI(uri,nsnull,nsIWebNavigation::LOAD_FLAGS_NONE, PR_FALSE);
     NS_ENSURE_SUCCESS(rv,rv);
 
     return NS_OK;
