@@ -45,7 +45,7 @@
 #include "nsILoadGroup.h"
 #include "nsIIOService.h"
 #include "nsNetUtil.h"
-#include "nsIFileChannel.h"
+#include "nsIFileURL.h"
 #include "nsFileStream.h"
 #include "nsIFileTransportService.h"
 #include "nsIDNSService.h"
@@ -56,11 +56,12 @@
 #include "nsIPrompt.h"
 #include "prprf.h"
 #include "nsIStringBundle.h"
+#include "nsIProtocolProxyService.h"
+#include "nsIProxyInfo.h"
 
 static NS_DEFINE_CID(kSocketTransportServiceCID, NS_SOCKETTRANSPORTSERVICE_CID);
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kFileTransportServiceCID, NS_FILETRANSPORTSERVICE_CID);
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
 NS_IMPL_THREADSAFE_ADDREF(nsMsgProtocol)
 NS_IMPL_THREADSAFE_RELEASE(nsMsgProtocol)
@@ -79,7 +80,7 @@ static PRUnichar *FormatStringWithHostNameByID(PRInt32 stringID, nsIMsgMailNewsU
 
 nsMsgProtocol::nsMsgProtocol(nsIURI * aURL)
 {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 	m_flags = 0;
 	m_startPosition = 0;
 	m_readCount = 0;
@@ -114,6 +115,7 @@ nsresult
 nsMsgProtocol::OpenNetworkSocketWithInfo(const char * aHostName,
                                          PRInt32 aGetPort,
                                          const char *connectionType,
+                                         nsIProxyInfo *aProxyInfo,
                                          nsIInterfaceRequestor* callbacks)
 {
   NS_ENSURE_ARG(aHostName);
@@ -127,7 +129,7 @@ nsMsgProtocol::OpenNetworkSocketWithInfo(const char * aHostName,
   m_startPosition = 0;
 
   rv = socketService->CreateTransportOfType(connectionType, aHostName,
-                                            aGetPort, nsnull, 0, 0,
+                                            aGetPort, aProxyInfo, 0, 0,
                                             getter_AddRefs(m_transport));
   if (NS_FAILED(rv)) return rv;
 
@@ -143,13 +145,52 @@ nsMsgProtocol::OpenNetworkSocket(nsIURI * aURL, const char *connectionType,
 {
   NS_ENSURE_ARG(aURL);
 
-  nsXPIDLCString hostName;
+  nsCAutoString hostName;
   PRInt32 port = 0;
 
   aURL->GetPort(&port);
-  aURL->GetHost(getter_Copies(hostName));
+  aURL->GetAsciiHost(hostName);
 
-  return OpenNetworkSocketWithInfo(hostName, port, connectionType, callbacks);
+  nsCOMPtr<nsIProxyInfo> proxyInfo;
+
+  nsCOMPtr<nsIProtocolProxyService> pps =
+      do_GetService("@mozilla.org/network/protocol-proxy-service;1");
+
+  NS_ASSERTION(pps, "Couldn't get the protocol proxy service!");
+
+  if (pps) {
+      nsresult rv = NS_OK;
+
+      // Yes, this is ugly. But necko needs to grap a protocol handler
+      // to ask for flags, and smtp isn't registered as a handler, only
+      // mailto.
+      // Note that I cannot just clone, and call SetSpec, since Clone on
+      // nsSmtpUrl calls nsStandardUrl's clone method, which fails
+      // because smtp isn't a registered protocol.
+      // So we cheat. Whilst creating a uri manually is valid here,
+      // do _NOT_ copy this to use in your own code - bbaetz
+      nsCOMPtr<nsIURI> proxyUri = aURL;
+      PRBool isSMTP = PR_FALSE;
+      if (NS_SUCCEEDED(aURL->SchemeIs("smtp", &isSMTP)) && isSMTP) {
+          nsCAutoString spec;
+          rv = aURL->GetSpec(spec);
+          if (NS_SUCCEEDED(rv)) {
+              static NS_DEFINE_CID(kSTDURLCID, NS_STANDARDURL_CID);    
+              proxyUri = do_CreateInstance(kSTDURLCID, &rv);
+          }
+          if (NS_SUCCEEDED(rv))
+              rv = proxyUri->SetSpec(spec);
+          if (NS_SUCCEEDED(rv))
+              rv = proxyUri->SetScheme(NS_LITERAL_CSTRING("mailto"));
+      }
+      if (NS_SUCCEEDED(rv))
+          rv = pps->ExamineForProxy(proxyUri, getter_AddRefs(proxyInfo));
+      NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't successfully call ExamineForProxy");
+      if (NS_FAILED(rv)) proxyInfo = nsnull;
+  }
+
+  return OpenNetworkSocketWithInfo(hostName.get(), port, connectionType,
+                                   proxyInfo, callbacks);
 }
 
 nsresult nsMsgProtocol::GetFileFromURL(nsIURI * aURL, nsIFile **aResult)
@@ -157,10 +198,9 @@ nsresult nsMsgProtocol::GetFileFromURL(nsIURI * aURL, nsIFile **aResult)
   NS_ENSURE_ARG_POINTER(aURL);
   NS_ENSURE_ARG_POINTER(aResult);
   // extract the file path from the uri...
-  nsXPIDLCString filePath;
-  aURL->GetPath(getter_Copies(filePath));
-  nsCAutoString urlSpec("file://");
-  urlSpec.Append(filePath.get());
+  nsCAutoString urlSpec;
+  aURL->GetPath(urlSpec);
+  urlSpec.Insert(NS_LITERAL_CSTRING("file://"), 0);
   nsresult rv;
 
 // dougt - there should be an easier way!
@@ -193,7 +233,7 @@ nsresult nsMsgProtocol::OpenFileSocket(nsIURI * aURL, PRUint32 aStartPosition, P
   if (NS_FAILED(rv)) return rv;
   //we are always using this file socket to read data from the mailbox.
   rv = fts->CreateTransport(file, PR_RDONLY,
-                            0664, getter_AddRefs(m_transport));
+                            0664, PR_TRUE, getter_AddRefs(m_transport));
   m_socketIsOpen = PR_FALSE;
 
 	return rv;
@@ -455,13 +495,13 @@ NS_IMETHODIMP nsMsgProtocol::AsyncOpen(nsIStreamListener *listener, nsISupports
     if (NS_FAILED(rv))
         return rv;
     
-    nsXPIDLCString scheme;
-    rv = m_url->GetScheme(getter_Copies(scheme));
+    nsCAutoString scheme;
+    rv = m_url->GetScheme(scheme);
     if (NS_FAILED(rv))
         return rv;
     
 
-    rv = NS_CheckPortSafety(port, scheme);
+    rv = NS_CheckPortSafety(port, scheme.get());
     if (NS_FAILED(rv))
         return rv;
 
@@ -483,7 +523,7 @@ NS_IMETHODIMP nsMsgProtocol::SetLoadFlags(nsLoadFlags aLoadFlags)
   return NS_OK;       // don't fail when trying to set this
 }
 
-NS_IMETHODIMP nsMsgProtocol::GetContentType(char * *aContentType)
+NS_IMETHODIMP nsMsgProtocol::GetContentType(nsACString &aContentType)
 {
   // as url dispatching matures, we'll be intelligent and actually start
   // opening the url before specifying the content type. This will allow
@@ -491,18 +531,30 @@ NS_IMETHODIMP nsMsgProtocol::GetContentType(char * *aContentType)
   // a part in the message that has a content type that is not message/rfc822
 
   if (m_ContentType.IsEmpty())
-	  *aContentType = nsCRT::strdup("message/rfc822");
+	  aContentType = NS_LITERAL_CSTRING("message/rfc822");
   else
-    *aContentType = ToNewCString(m_ContentType);
+    aContentType = m_ContentType;
 	return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgProtocol::SetContentType(const char *aContentType)
+NS_IMETHODIMP nsMsgProtocol::SetContentType(const nsACString &aContentType)
 {
     m_ContentType = aContentType;
     return NS_OK;
 }
 
+NS_IMETHODIMP nsMsgProtocol::GetContentCharset(nsACString &aContentCharset)
+{
+  aContentCharset.Truncate();
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgProtocol::SetContentCharset(const nsACString &aContentCharset)
+{
+  NS_NOTREACHED("nsMsgProtocol::SetContentCharset");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsMsgProtocol::GetContentLength(PRInt32 * aContentLength)
 {
   *aContentLength = -1;
@@ -515,7 +567,7 @@ NS_IMETHODIMP nsMsgProtocol::GetSecurityInfo(nsISupports * *aSecurityInfo)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsMsgProtocol::GetName(PRUnichar * *aName)
+NS_IMETHODIMP nsMsgProtocol::GetName(nsACString &aName)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -732,7 +784,7 @@ class nsMsgProtocolStreamProvider : public nsIStreamProvider
 public:
     NS_DECL_ISUPPORTS
 
-    nsMsgProtocolStreamProvider() { NS_INIT_REFCNT(); }
+    nsMsgProtocolStreamProvider() { NS_INIT_ISUPPORTS(); }
     virtual ~nsMsgProtocolStreamProvider() {}
 
     void Init(nsMsgAsyncWriteProtocol *aProtInstance, nsIInputStream *aInputStream) { mMsgProtocol = aProtInstance; mInStream = aInputStream;}
@@ -796,7 +848,7 @@ public:
   NS_DECL_NSIREQUESTOBSERVER
   NS_DECL_NSISTREAMLISTENER
 
-  nsMsgFilePostHelper() { NS_INIT_REFCNT(); mSuspendedPostFileRead = PR_FALSE;}
+  nsMsgFilePostHelper() { NS_INIT_ISUPPORTS(); mSuspendedPostFileRead = PR_FALSE;}
   nsresult Init(nsIOutputStream * aOutStream, nsMsgAsyncWriteProtocol * aProtInstance, nsIFile *aFileToPost);
   virtual ~nsMsgFilePostHelper() {}
   nsCOMPtr<nsIRequest> mPostFileRequest;
@@ -827,10 +879,10 @@ nsresult nsMsgFilePostHelper::Init(nsIOutputStream * aOutStream, nsMsgAsyncWrite
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsITransport> transport;
-  rv = fts->CreateTransport(aFileToPost, PR_RDONLY, 0664, getter_AddRefs(transport));
+  rv = fts->CreateTransport(aFileToPost, PR_RDONLY, 0664, PR_TRUE, getter_AddRefs(transport));
   if (transport)
   {
-    rv = transport->AsyncRead(this, nsnull, 0, -1, 0, getter_AddRefs(mPostFileRequest));
+    rv = transport->AsyncRead(this, nsnull, 0, PRUint32(-1), 0, getter_AddRefs(mPostFileRequest));
   }
   return rv;
 }
@@ -1194,7 +1246,7 @@ void nsMsgAsyncWriteProtocol::UpdateProgress(PRUint32 aNewBytes)
 
 PRInt32 nsMsgAsyncWriteProtocol::SendData(nsIURI * aURL, const char * dataBuffer, PRBool aSuppressLogging)
 {
-  PRUint32 len = nsCRT::strlen(dataBuffer);
+  PRUint32 len = strlen(dataBuffer);
   PRUint32 cnt;
   nsresult rv = m_outputStream->Write(dataBuffer, len, &cnt);
   if (NS_SUCCEEDED(rv) && len==cnt) 
