@@ -60,11 +60,13 @@
 #include "nsIMimeConverter.h"
 #include "nsMsgMimeCID.h"
 #include "nsICategoryManager.h"
+#include "nsCategoryManagerUtils.h"
+#include "nsISpamSettings.h"
+
 
 static NS_DEFINE_CID(kImapUrlCID, NS_IMAPURL_CID);
 static NS_DEFINE_CID(kCMailboxUrl, NS_MAILBOXURL_CID);
 static NS_DEFINE_CID(kCNntpUrlCID, NS_NNTPURL_CID);
-static NS_DEFINE_CID(kCMimeConverterCID, NS_MIME_CONVERTER_CID);
 
 #define NS_PASSWORDMANAGER_CATEGORY "passwordmanager"
 static PRBool gInitPasswordManager = PR_FALSE;
@@ -90,17 +92,17 @@ nsresult GetMessageServiceContractIDForURI(const char *uri, nsCString &contractI
 
 nsresult GetMessageServiceFromURI(const char *uri, nsIMsgMessageService **aMessageService)
 {
-	nsresult rv;
+  nsresult rv;
 
   nsCAutoString contractID;
-	rv = GetMessageServiceContractIDForURI(uri, contractID);
+  rv = GetMessageServiceContractIDForURI(uri, contractID);
   NS_ENSURE_SUCCESS(rv,rv);
 
   nsCOMPtr <nsIMsgMessageService> msgService = do_GetService(contractID.get(), &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
   NS_IF_ADDREF(*aMessageService = msgService);
-	return rv;
+  return rv;
 }
 
 nsresult GetMsgDBHdrFromURI(const char *uri, nsIMsgDBHdr **msgHdr)
@@ -365,7 +367,7 @@ PRBool NS_MsgStripRE(const char **stringP, PRUint32 *lengthP, char **modifiedSub
   // we cannot strip "Re:" for MIME encoded subject without modifying the original
   if (modifiedSubject && strstr(*stringP, "=?"))
   {
-    mimeConverter = do_GetService(kCMimeConverterCID, &rv);
+    mimeConverter = do_GetService(NS_MIME_CONVERTER_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv))
       rv = mimeConverter->DecodeMimeHeader(*stringP, getter_Copies(decodedString));
   }
@@ -533,27 +535,26 @@ PRBool WeAreOffline()
   return offline;
 }
 
-PRBool IsValidFolderURI(const char *aFolderURI)
+nsresult GetExistingFolder(const char *aFolderURI, nsIMsgFolder **aFolder)
 {
   nsresult rv;
   nsCOMPtr<nsIRDFService> rdf(do_GetService("@mozilla.org/rdf/rdf-service;1", &rv));
-  if (NS_FAILED(rv))
-    return PR_FALSE;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIRDFResource> resource;
   rv = rdf->GetResource(aFolderURI, getter_AddRefs(resource));
-  if (NS_FAILED(rv))
-    return PR_FALSE;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr <nsIMsgFolder> thisFolder;
   thisFolder = do_QueryInterface(resource, &rv);
-  if (NS_FAILED(rv) || !thisFolder)
-    return PR_FALSE;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Parent doesn't exist means that this folder doesn't exist.
   nsCOMPtr<nsIFolder> parentFolder;
   rv = thisFolder->GetParent(getter_AddRefs(parentFolder));
-  return (NS_SUCCEEDED(rv) && parentFolder);
+  if (NS_SUCCEEDED(rv) && parentFolder)
+    NS_ADDREF(*aFolder = thisFolder);
+  return rv;
 }
 
 PRBool IsAFromSpaceLine(char *start, const char *end)
@@ -620,3 +621,21 @@ nsresult CreateServicesForPasswordManager()
   }
   return NS_OK;
 }
+  
+nsresult IsRFC822HeaderFieldName(const char *aHdr, PRBool *aResult)
+{
+  NS_ENSURE_ARG_POINTER(aHdr);
+  NS_ENSURE_ARG_POINTER(aResult);
+  PRUint32 length = strlen(aHdr);
+  for(PRUint32 i=0; i<length; i++)
+  {
+    char c = aHdr[i];
+    if ( c < '!' || c == ':' || c > '~') 
+    {
+      *aResult = PR_FALSE;
+      return NS_OK;
+    }
+  }
+  *aResult = PR_TRUE;
+  return NS_OK;
+}
