@@ -82,7 +82,7 @@ CopyListener::CopyListener(void)
 { 
   mCopyObject = nsnull;
   mCopyInProgress = PR_FALSE;
-  NS_INIT_REFCNT(); 
+  NS_INIT_ISUPPORTS(); 
 }
 
 CopyListener::~CopyListener(void) 
@@ -183,7 +183,7 @@ nsMsgCopy::nsMsgCopy()
   mMode = nsIMsgSend::nsMsgDeliverNow;
   mSavePref = nsnull;
 
-  NS_INIT_REFCNT(); 
+  NS_INIT_ISUPPORTS(); 
 }
 
 nsMsgCopy::~nsMsgCopy()
@@ -316,9 +316,11 @@ nsMsgCopy::DoCopy(nsIFileSpec *aDiskFile, nsIMsgFolder *dstFolder,
     // ** make sure we have a valid copy listener while waiting for copy
     // server to finish
     nsCOMPtr<CopyListener> aCopyListener = do_QueryInterface(tPtr);
+    nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = dstFolder->CopyFileMessage(aDiskFile, aMsgToReplace,
-                                      aIsDraft, msgWindow, mCopyListener);
+    rv = copyService->CopyFileMessage(aDiskFile, dstFolder, aMsgToReplace, 
+                                      aIsDraft, mCopyListener, msgWindow);
     // aCopyListener->mCopyInProgress can only be set when we are in the
     // middle of the shutdown process
     while (aCopyListener->mCopyInProgress)
@@ -465,12 +467,16 @@ LocateMessageFolder(nsIMsgIdentity   *userIdentity,
           nsCOMPtr<nsIImapIncomingServer> imapServer = do_QueryInterface(server);
           // Make sure an specific IMAP folder has correct personal namespace
           // See bugzilla bug 90494 (http://bugzilla.mozilla.org/show_bug.cgi?id=90494)
+          PRBool namespacePrefixAdded = PR_FALSE;
+          nsXPIDLCString folderUriWithNamespace;
           if (imapServer)
           {
-            nsXPIDLCString folderUriWithNamespace;
             imapServer->GetUriWithNamespacePrefixIfNecessary(kPersonalNamespace, aFolderURI, getter_Copies(folderUriWithNamespace));
             if (!folderUriWithNamespace.IsEmpty())
+            {
               rv = rootMsgFolder->GetChildWithURI(folderUriWithNamespace, PR_TRUE, PR_FALSE, msgFolder);
+              namespacePrefixAdded = PR_TRUE;
+            }
             else
               rv = rootMsgFolder->GetChildWithURI(aFolderURI, PR_TRUE, PR_FALSE, msgFolder);
           }
@@ -480,8 +486,24 @@ LocateMessageFolder(nsIMsgIdentity   *userIdentity,
              CreateIfMissing does that provided we pass in a dummy folder */
           if (!*msgFolder)
           {
-            *msgFolder = folderResource;  
-            NS_ADDREF(*msgFolder);
+            if (namespacePrefixAdded)
+            {
+              nsCOMPtr<nsIRDFResource> resource;
+              rv = rdf->GetResource(folderUriWithNamespace, getter_AddRefs(resource));
+              if (NS_FAILED(rv)) return rv;
+
+              nsCOMPtr <nsIMsgFolder> folderResource;
+              folderResource = do_QueryInterface(resource, &rv);
+              if (NS_FAILED(rv)) return rv;
+
+              *msgFolder = folderResource;
+              NS_ADDREF(*msgFolder);
+            }
+            else
+            {
+              *msgFolder = folderResource;
+              NS_ADDREF(*msgFolder);
+            }
           }
           return rv;
         }
