@@ -158,7 +158,6 @@ protected:
 	nsIMsgThread *	GetThreadForSubject(nsCString &subject);
 	nsIMsgThread *	GetThreadForThreadId(nsMsgKey threadId);
 	nsMsgHdr	*	GetMsgHdrForReference(nsCString &reference);
-	nsIMsgDBHdr	*	GetMsgHdrForMessageID(nsCString &msgID);
 	nsIMsgDBHdr	*	GetMsgHdrForSubject(nsCString &msgID);
 	// threading interfaces
 	virtual nsresult CreateNewThread(nsMsgKey key, const char *subject, nsMsgThread **newThread);
@@ -221,7 +220,7 @@ virtual nsresult AdjustExpungedBytesOnDelete(nsIMsgDBHdr *msgHdr);
 	nsFileSpec		    m_dbName;
 	nsMsgKeySet		    *m_newSet;	// new messages since last open.
 	PRBool				m_mdbTokensInitialized;
-    nsVoidArray/*<nsIDBChangeListener>*/ *m_ChangeListeners;
+  nsCOMPtr <nsISupportsArray>  m_ChangeListeners;
 	mdb_token			m_hdrRowScopeToken;
 	mdb_token			m_threadRowScopeToken;
 	mdb_token			m_hdrTableKindToken;
