@@ -242,6 +242,13 @@ NS_IMETHODIMP nsMailDatabase::StartBatch()
 {
   if (!m_folderStream)  //only if we create a stream, set m_ownFolderStream to true.
   {
+    PRBool isLocked;
+    m_folder->GetLocked(&isLocked);
+    if (isLocked)
+    {
+      NS_ASSERTION(PR_FALSE, "Some other operation is in progress");
+      return NS_MSG_FOLDER_BUSY;
+    }
     m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
     m_ownFolderStream = PR_TRUE;
   }
@@ -270,6 +277,13 @@ NS_IMETHODIMP nsMailDatabase::DeleteMessages(nsMsgKeyArray* nsMsgKeys, nsIDBChan
 	nsresult ret = NS_OK;
   if (!m_folderStream)
   {
+    PRBool isLocked;
+    m_folder->GetLocked(&isLocked);
+    if (isLocked)
+    {
+      NS_ASSERTION(PR_FALSE, "Some other operation is in progress");
+      return NS_MSG_FOLDER_BUSY;
+    }
 	  m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
     m_ownFolderStream = PR_TRUE;
   }
@@ -288,13 +302,22 @@ NS_IMETHODIMP nsMailDatabase::DeleteMessages(nsMsgKeyArray* nsMsgKeys, nsIDBChan
 	return ret;
 }
 
-
 // Helper routine - lowest level of flag setting
 PRBool nsMailDatabase::SetHdrFlag(nsIMsgDBHdr *msgHdr, PRBool bSet, MsgFlags flag)
 {
   nsIOFileStream *fileStream = NULL;
   PRBool		ret = PR_FALSE;
 
+  if (!m_folderStream)  //we are going to create a stream, bail out if someone else has lock
+  {
+    PRBool isLocked;
+    m_folder->GetLocked(&isLocked);
+    if (isLocked)
+    {
+      NS_ASSERTION(PR_FALSE, "Some other operation is in progress");
+      return PR_FALSE;
+    }
+  }
   if (nsMsgDatabase::SetHdrFlag(msgHdr, bSet, flag))
   {
     UpdateFolderFlag(msgHdr, bSet, flag, &fileStream);
@@ -531,16 +554,16 @@ nsresult nsMailDatabase::GetFolderName(nsString &folderName)
 
 NS_IMETHODIMP  nsMailDatabase::RemoveOfflineOp(nsIMsgOfflineImapOperation *op)
 {
-
+  
   nsresult rv = GetAllOfflineOpsTable();
   NS_ENSURE_SUCCESS(rv, rv);
-
-	if (!op || !m_mdbAllOfflineOpsTable)
-		return NS_ERROR_NULL_POINTER;
+  
+  if (!op || !m_mdbAllOfflineOpsTable)
+    return NS_ERROR_NULL_POINTER;
   nsMsgOfflineImapOperation* offlineOp = NS_STATIC_CAST(nsMsgOfflineImapOperation*, op);  // closed system, so this is ok
-	nsIMdbRow* row = offlineOp->GetMDBRow();
-	nsresult ret = m_mdbAllOfflineOpsTable->CutRow(GetEnv(), row);
-	row->CutAllColumns(GetEnv());
+  nsIMdbRow* row = offlineOp->GetMDBRow();
+  rv = m_mdbAllOfflineOpsTable->CutRow(GetEnv(), row);
+  row->CutAllColumns(GetEnv());
   return rv;
 }
 
@@ -651,52 +674,52 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineOpIds(nsMsgKeyArray *offlineOpIds)
   NS_ENSURE_ARG(offlineOpIds);
   nsresult rv = GetAllOfflineOpsTable();
   NS_ENSURE_SUCCESS(rv, rv);
-	nsIMdbTableRowCursor *rowCursor;
-	if (m_mdbAllOfflineOpsTable)
-	{
-		nsresult err = m_mdbAllOfflineOpsTable->GetTableRowCursor(GetEnv(), -1, &rowCursor);
-		while (err == NS_OK && rowCursor)
-		{
-			mdbOid outOid;
-			mdb_pos	outPos;
-
-			err = rowCursor->NextRowOid(GetEnv(), &outOid, &outPos);
-			// is this right? Mork is returning a 0 id, but that should valid.
-			if (outPos < 0 || outOid.mOid_Id == (mdb_id) -1)	
-				break;
-			if (err == NS_OK)
-				offlineOpIds->Add(outOid.mOid_Id);
-		}
+  nsIMdbTableRowCursor *rowCursor;
+  if (m_mdbAllOfflineOpsTable)
+  {
+    nsresult err = m_mdbAllOfflineOpsTable->GetTableRowCursor(GetEnv(), -1, &rowCursor);
+    while (err == NS_OK && rowCursor)
+    {
+      mdbOid outOid;
+      mdb_pos	outPos;
+      
+      err = rowCursor->NextRowOid(GetEnv(), &outOid, &outPos);
+      // is this right? Mork is returning a 0 id, but that should valid.
+      if (outPos < 0 || outOid.mOid_Id == (mdb_id) -1)	
+        break;
+      if (err == NS_OK)
+        offlineOpIds->Add(outOid.mOid_Id);
+    }
     rv = (err == NS_OK) ? NS_OK : NS_ERROR_FAILURE;
-		rowCursor->Release();
-	}
-
-	offlineOpIds->QuickSort();
-	return rv;
+    rowCursor->Release();
+  }
+  
+  offlineOpIds->QuickSort();
+  return rv;
 }
 
 NS_IMETHODIMP nsMailDatabase::ListAllOfflineDeletes(nsMsgKeyArray *offlineDeletes)
 {
-	if (!offlineDeletes)
-		return NS_ERROR_NULL_POINTER;
-
+  if (!offlineDeletes)
+    return NS_ERROR_NULL_POINTER;
+  
   nsresult rv = GetAllOfflineOpsTable();
   NS_ENSURE_SUCCESS(rv, rv);
-	nsIMdbTableRowCursor *rowCursor;
-	if (m_mdbAllOfflineOpsTable)
-	{
-		nsresult err = m_mdbAllOfflineOpsTable->GetTableRowCursor(GetEnv(), -1, &rowCursor);
-		while (err == NS_OK && rowCursor)
-		{
-			mdbOid outOid;
-			mdb_pos	outPos;
+  nsIMdbTableRowCursor *rowCursor;
+  if (m_mdbAllOfflineOpsTable)
+  {
+    nsresult err = m_mdbAllOfflineOpsTable->GetTableRowCursor(GetEnv(), -1, &rowCursor);
+    while (err == NS_OK && rowCursor)
+    {
+      mdbOid outOid;
+      mdb_pos	outPos;
       nsIMdbRow* offlineOpRow;
-
-			err = rowCursor->NextRow(GetEnv(), &offlineOpRow, &outPos);
-			// is this right? Mork is returning a 0 id, but that should valid.
-			if (outPos < 0 || offlineOpRow == nsnull)	
-				break;
-			if (err == NS_OK)
+      
+      err = rowCursor->NextRow(GetEnv(), &offlineOpRow, &outPos);
+      // is this right? Mork is returning a 0 id, but that should valid.
+      if (outPos < 0 || offlineOpRow == nsnull)	
+        break;
+      if (err == NS_OK)
       {
         offlineOpRow->GetOid(GetEnv(), &outOid);
         nsIMsgOfflineImapOperation *offlineOp = new nsMsgOfflineImapOperation(this, offlineOpRow);
@@ -705,22 +728,22 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineDeletes(nsMsgKeyArray *offlineDelete
           NS_ADDREF(offlineOp);
           imapMessageFlagsType newFlags;
           nsOfflineImapOperationType opType;
-
+          
           offlineOp->GetOperation(&opType);
           offlineOp->GetNewFlags(&newFlags);
           if (opType & nsIMsgOfflineImapOperation::kMsgMoved || 
-              ((opType & nsIMsgOfflineImapOperation::kFlagsChanged) 
-              && (newFlags & nsIMsgOfflineImapOperation::kMsgMarkedDeleted)))
-    				offlineDeletes->Add(outOid.mOid_Id);
+            ((opType & nsIMsgOfflineImapOperation::kFlagsChanged) 
+            && (newFlags & nsIMsgOfflineImapOperation::kMsgMarkedDeleted)))
+            offlineDeletes->Add(outOid.mOid_Id);
           NS_RELEASE(offlineOp);
         }
         offlineOpRow->Release();
       }
-		}
+    }
     rv = (err == NS_OK) ? NS_OK : NS_ERROR_FAILURE;
-		rowCursor->Release();
-	}
-	return rv;
+    rowCursor->Release();
+  }
+  return rv;
 }
 
 /* static */
@@ -842,7 +865,7 @@ protected:
 nsMsgOfflineOpEnumerator::nsMsgOfflineOpEnumerator(nsMailDatabase* db)
     : mDB(db), mRowCursor(nsnull), mDone(PR_FALSE)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   NS_ADDREF(mDB);
   mNextPrefetched = PR_FALSE;
 }
