@@ -25,8 +25,8 @@
 #include "nsMsgComposeSecure.h"
 #include "nspr.h"
 #include "nsCOMPtr.h"
-#include "nsICMS.h"
 #include "nsIX509Cert.h"
+#include "nsISMimeCert.h"
 #include "nsIMimeConverter.h"
 #include "nsMimeStringResources.h"
 #include "nsMimeTypes.h"
@@ -40,6 +40,7 @@
 #include "nsIMsgCompFields.h"
 #include "nsCRT.h"
 #include "nsReadableUtils.h"
+#include "nsArray.h"
 
 // String bundle for smime. Class static.
 nsCOMPtr<nsIStringBundle> nsMsgComposeSecure::mSMIMEBundle = nsnull;
@@ -70,15 +71,12 @@ static char *mime_make_separator(const char *prefix);
 #include "nsMsgMimeCID.h"
 #include "nsIMimeConverter.h"
 
-/* This is the next generation string retrieval call */
-static NS_DEFINE_CID(kCMimeConverterCID, NS_MIME_CONVERTER_CID);
-
 MimeEncoderData *
 MIME_B64EncoderInit(nsresult (*PR_CALLBACK output_fn) (const char *buf, PRInt32 size, void *closure), void *closure) 
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -93,7 +91,7 @@ MIME_QPEncoderInit(nsresult (*PR_CALLBACK output_fn) (const char *buf, PRInt32 s
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -108,7 +106,7 @@ MIME_UUEncoderInit(char *filename, nsresult (*PR_CALLBACK output_fn) (const char
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -123,7 +121,7 @@ MIME_EncoderDestroy(MimeEncoderData *data, PRBool abort_p)
 {
   //MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -140,7 +138,7 @@ MIME_EncoderWrite(MimeEncoderData *data, const char *buffer, PRInt32 size)
   //  MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
   PRInt32 written = 0;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
     NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) {
     res = converter->EncoderWrite(data, buffer, size, &written);
@@ -615,7 +613,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign, nsIMsgSendReport *
 
   if (!mIsDraft) {
     PRUint32 numCerts;
-    mCerts->Count(&numCerts);
+    mCerts->GetLength(&numCerts);
     PR_ASSERT(numCerts > 0);
 	  if (numCerts == 0) return NS_ERROR_FAILURE;
   }
@@ -867,7 +865,7 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
   nsCOMPtr<nsIMsgHeaderParser> pHeader = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &res);
   NS_ENSURE_SUCCESS(res,res);
 
-  res = NS_NewISupportsArray(getter_AddRefs(mCerts));
+  res = NS_NewArray(getter_AddRefs(mCerts));
   if (NS_FAILED(res)) {
     return res;
   }
@@ -915,7 +913,10 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
     // Make sure self's configured cert is prepared for being used
     // as an email recipient cert.
     
-    mSelfEncryptionCert->SaveSMimeProfile();
+    nsCOMPtr<nsISMimeCert> sc = do_QueryInterface(mSelfEncryptionCert);
+    if (sc) {
+      sc->SaveSMimeProfile();
+    }
   }
 
   /* If the message is to be encrypted, then get the recipient certs */
@@ -964,14 +965,14 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
         already_added_self_cert = PR_TRUE;
       }
 
-      mCerts->AppendElement(cert);
+      mCerts->AppendElement(cert, PR_FALSE);
       // To understand this loop, especially the "+= strlen +1", look at the documentation
       // of ParseHeaderAddresses. Basically, it returns a list of zero terminated strings.
 		  mailbox += strlen(mailbox) + 1;
 	  }
     
     if (!already_added_self_cert) {
-      mCerts->AppendElement(mSelfEncryptionCert);
+      mCerts->AppendElement(mSelfEncryptionCert, PR_FALSE);
     }
 	}
 FAIL:
