@@ -44,6 +44,7 @@
 #include "nsIImapIncomingServer.h"
 #include "nsCOMPtr.h"
 #include "nsIMsgIncomingServer.h"
+#include "nsIObserverService.h"
 
 nsIMAPHostInfo::nsIMAPHostInfo(const char *serverKey, 
                                nsIImapIncomingServer *server)
@@ -56,7 +57,12 @@ nsIMAPHostInfo::nsIMAPHostInfo(const char *serverKey,
 	fCachedPassword = NULL;
 	fCapabilityFlags = kCapabilityUndefined;
 	fHierarchyDelimiters = NULL;
-	fHaveWeEverDiscoveredFolders = PR_FALSE;
+#ifdef DEBUG_bienvenu1
+	fHaveWeEverDiscoveredFolders = PR_TRUE; // try this, see what bad happens - we'll need to 
+                                                // figure out a way to make new accounts have it be false
+#else
+       	fHaveWeEverDiscoveredFolders = PR_FALSE; // try this, see what bad happens
+#endif
 	fCanonicalOnlineSubDir = NULL;
 	fNamespaceList = nsIMAPNamespaceList::CreatensIMAPNamespaceList();
 	fUsingSubscription = PR_TRUE;
@@ -85,44 +91,15 @@ nsIMAPHostInfo::~nsIMAPHostInfo()
 	delete fShellCache;
 }
 
-/* the following macros actually implement addref, release and query interface for our component. */
-NS_IMPL_THREADSAFE_ADDREF(nsIMAPHostSessionList)
-NS_IMPL_THREADSAFE_RELEASE(nsIMAPHostSessionList)
-
-NS_IMETHODIMP nsIMAPHostSessionList::QueryInterface(const nsIID &aIID, void** aInstancePtr)
-{                                                                        
-  if (NULL == aInstancePtr)
-    return NS_ERROR_NULL_POINTER;
-        
-  *aInstancePtr = NULL;
-                                                                         
-  static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID); 
-  static NS_DEFINE_IID(kIsThreadsafeIID, NS_ISTHREADSAFE_IID); 
-
-  if (aIID.Equals(NS_GET_IID(nsIImapHostSessionList)))
-  {
-	  *aInstancePtr = (nsIImapHostSessionList *) this;
-	  NS_ADDREF_THIS();
-	  return NS_OK;
-  }
-  if (aIID.Equals(kISupportsIID)) 
-  {
-	  *aInstancePtr = (void*) ((nsISupports*)this);
-	  NS_ADDREF_THIS();
-    return NS_OK;                                                        
-  }                                                                      
-  
-  if (aIID.Equals(kIsThreadsafeIID)) 
-  {
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
+NS_IMPL_THREADSAFE_ISUPPORTS3(nsIMAPHostSessionList,
+                              nsIImapHostSessionList,
+                              nsIObserver,
+                              nsISupportsWeakReference)
 
 
 nsIMAPHostSessionList::nsIMAPHostSessionList()
 {
-    NS_INIT_REFCNT ();
+    NS_INIT_ISUPPORTS ();
 	gCachedHostInfoMonitor = PR_NewMonitor(/* "accessing-hostlist-monitor"*/);
 	fHostInfoList = nsnull;
 }
@@ -133,6 +110,36 @@ nsIMAPHostSessionList::~nsIMAPHostSessionList()
 	PR_DestroyMonitor(gCachedHostInfoMonitor);
 }
 
+nsresult nsIMAPHostSessionList::Init()
+{
+  nsresult rv;
+  nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
+  if (NS_SUCCEEDED(rv))
+  {
+    observerService->AddObserver(this, "profile-before-change", PR_TRUE);
+    observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
+  }
+  return rv;
+}
+
+
+NS_IMETHODIMP nsIMAPHostSessionList::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
+{
+  nsresult rv;  
+  if (!strcmp(aTopic, "profile-before-change")) 
+    ResetAll();
+  else if (!strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID))
+  {
+    nsCOMPtr<nsIObserverService> observerService =  do_GetService("@mozilla.org/observer-service;1", &rv);
+    if (NS_SUCCEEDED(rv))
+    {    
+      observerService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
+      observerService->RemoveObserver(this, "profile-before-change");
+    }
+  }
+  return NS_OK;
+}
+
 nsIMAPHostInfo *nsIMAPHostSessionList::FindHost(const char *serverKey)
 {
 	nsIMAPHostInfo *host;
@@ -611,84 +618,84 @@ nsresult nsIMAPHostSessionList::SetNamespacesPrefForHost(nsIImapIncomingServer *
 // Make sure this is running in the Mozilla thread when called
 NS_IMETHODIMP nsIMAPHostSessionList::CommitNamespacesForHost(nsIImapIncomingServer *aHost)
 {
-	char * serverKey = nsnull;
-	
-	if (!aHost)
-		return NS_ERROR_NULL_POINTER;
-
-	nsCOMPtr <nsIMsgIncomingServer> incomingServer = do_QueryInterface(aHost);
-	if (!incomingServer)
-		return NS_ERROR_NULL_POINTER;
-
-	nsresult rv = incomingServer->GetKey(&serverKey);
-	if (NS_FAILED(rv)) return rv;
-
-	PR_EnterMonitor(gCachedHostInfoMonitor);
-	nsIMAPHostInfo *host = FindHost(serverKey);
-	if (host)
-	{
-		host->fGotNamespaces = PR_TRUE;	// so we only issue NAMESPACE once per host per session.
-		EIMAPNamespaceType type = kPersonalNamespace;
-		for (int i = 1; i <= 3; i++)
-		{
-			switch(i)
-			{
-			case 1:
-				type = kPersonalNamespace;
-				break;
-			case 2:
-				type = kPublicNamespace;
-				break;
-			case 3:
-				type = kOtherUsersNamespace;
-				break;
-			default:
-				type = kPersonalNamespace;
-				break;
-			}
-			
-			int32 numInNS = host->fNamespaceList->GetNumberOfNamespaces(type);
-			if (numInNS == 0)
-			{
-				SetNamespacesPrefForHost(aHost, type, NULL);
-			}
-			else if (numInNS >= 1)
-			{
-				char *pref = PR_smprintf("");
-				for (int count = 1; count <= numInNS; count++)
-				{
-					nsIMAPNamespace *ns = host->fNamespaceList->GetNamespaceNumber(count, type);
-					if (ns)
-					{
-						if (count > 1)
-						{
-							// append the comma
-							char *tempPref = PR_smprintf("%s,",pref);
-							PR_FREEIF(pref);
-							pref = tempPref;
-						}
-						char *tempPref = PR_smprintf("%s\"%s\"",pref,ns->GetPrefix());
-						PR_FREEIF(pref);
-						pref = tempPref;
-					}
-				}
-				if (pref)
-				{
-					SetNamespacesPrefForHost(aHost, type, pref);
-					PR_Free(pref);
-				}
-			}
-		}
-		// clear, but don't delete the entries in, the temp namespace list
-		host->fTempNamespaceList->ClearNamespaces(PR_TRUE, PR_TRUE, PR_FALSE);
-		
-		// Now reset all of libmsg's namespace references.
-		// Did I mention this needs to be running in the mozilla thread?
-		aHost->ResetNamespaceReferences();
-	}
-	PR_FREEIF(serverKey);
-	PR_ExitMonitor(gCachedHostInfoMonitor);
-	return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
+  char * serverKey = nsnull;
+  
+  if (!aHost)
+    return NS_ERROR_NULL_POINTER;
+  
+  nsCOMPtr <nsIMsgIncomingServer> incomingServer = do_QueryInterface(aHost);
+  if (!incomingServer)
+    return NS_ERROR_NULL_POINTER;
+  
+  nsresult rv = incomingServer->GetKey(&serverKey);
+  if (NS_FAILED(rv)) return rv;
+  
+  PR_EnterMonitor(gCachedHostInfoMonitor);
+  nsIMAPHostInfo *host = FindHost(serverKey);
+  if (host)
+  {
+    host->fGotNamespaces = PR_TRUE;	// so we only issue NAMESPACE once per host per session.
+    EIMAPNamespaceType type = kPersonalNamespace;
+    for (int i = 1; i <= 3; i++)
+    {
+      switch(i)
+      {
+      case 1:
+        type = kPersonalNamespace;
+        break;
+      case 2:
+        type = kPublicNamespace;
+        break;
+      case 3:
+        type = kOtherUsersNamespace;
+        break;
+      default:
+        type = kPersonalNamespace;
+        break;
+      }
+      
+      int32 numInNS = host->fNamespaceList->GetNumberOfNamespaces(type);
+      if (numInNS == 0)
+      {
+        SetNamespacesPrefForHost(aHost, type, NULL);
+      }
+      else if (numInNS >= 1)
+      {
+        char *pref = PR_smprintf("");
+        for (int count = 1; count <= numInNS; count++)
+        {
+          nsIMAPNamespace *ns = host->fNamespaceList->GetNamespaceNumber(count, type);
+          if (ns)
+          {
+            if (count > 1)
+            {
+              // append the comma
+              char *tempPref = PR_smprintf("%s,",pref);
+              PR_FREEIF(pref);
+              pref = tempPref;
+            }
+            char *tempPref = PR_smprintf("%s\"%s\"",pref,ns->GetPrefix());
+            PR_FREEIF(pref);
+            pref = tempPref;
+          }
+        }
+        if (pref)
+        {
+          SetNamespacesPrefForHost(aHost, type, pref);
+          PR_Free(pref);
+        }
+      }
+    }
+    // clear, but don't delete the entries in, the temp namespace list
+    host->fTempNamespaceList->ClearNamespaces(PR_TRUE, PR_TRUE, PR_FALSE);
+    
+    // Now reset all of libmsg's namespace references.
+    // Did I mention this needs to be running in the mozilla thread?
+    aHost->ResetNamespaceReferences();
+  }
+  PR_FREEIF(serverKey);
+  PR_ExitMonitor(gCachedHostInfoMonitor);
+  return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
 }
 
 NS_IMETHODIMP nsIMAPHostSessionList::FlushUncommittedNamespacesForHost(const char *serverKey, PRBool &result)
@@ -753,18 +760,18 @@ NS_IMETHODIMP nsIMAPHostSessionList::SetShouldAlwaysListInboxForHost(const char
 
 NS_IMETHODIMP nsIMAPHostSessionList::SetNamespaceHierarchyDelimiterFromMailboxForHost(const char *serverKey, const char *boxName, char delimiter)
 {
-	PR_EnterMonitor(gCachedHostInfoMonitor);
-	nsIMAPHostInfo *host = FindHost(serverKey);
-	if (host)
-	{
-		nsIMAPNamespace *ns = host->fNamespaceList->GetNamespaceForMailbox(boxName);
-		if (ns && !ns->GetIsDelimiterFilledIn())
-		{
-			ns->SetDelimiter(delimiter);
-		}
-	}
-	PR_ExitMonitor(gCachedHostInfoMonitor);
-	return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
+  PR_EnterMonitor(gCachedHostInfoMonitor);
+  nsIMAPHostInfo *host = FindHost(serverKey);
+  if (host)
+  {
+    nsIMAPNamespace *ns = host->fNamespaceList->GetNamespaceForMailbox(boxName);
+    if (ns && !ns->GetIsDelimiterFilledIn())
+    {
+      ns->SetDelimiter(delimiter, PR_TRUE);
+    }
+  }
+  PR_ExitMonitor(gCachedHostInfoMonitor);
+  return (host) ? NS_OK : NS_ERROR_ILLEGAL_VALUE ;
 }
 
 NS_IMETHODIMP nsIMAPHostSessionList::AddShellToCacheForHost(const char *serverKey, nsIMAPBodyShell *shell)
@@ -804,4 +811,3 @@ NS_IMETHODIMP nsIMAPHostSessionList::FindShellInCacheForHost(const char *serverK
 	PR_ExitMonitor(gCachedHostInfoMonitor);
 	return (host == NULL) ? NS_ERROR_ILLEGAL_VALUE : NS_OK;
 }
-
