@@ -65,8 +65,6 @@
 static NS_DEFINE_IID(kISupportsIID,        	NS_ISUPPORTS_IID);
 static NS_DEFINE_CID(kComponentManagerCID, 	NS_COMPONENTMANAGER_CID);
 static NS_DEFINE_CID(kSmtpServiceCID,		NS_SMTPSERVICE_CID); 
-static NS_DEFINE_CID(kMsgAccountMgrCID, NS_MSGACCOUNTMANAGER_CID);
-
 
 class OutlookSettings {
 public:
@@ -102,7 +100,7 @@ nsresult nsOutlookSettings::Create(nsIImportSettings** aImport)
 
 nsOutlookSettings::nsOutlookSettings()
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 nsOutlookSettings::~nsOutlookSettings()
@@ -180,7 +178,7 @@ PRBool OutlookSettings::DoImport( nsIMsgAccount **ppAccount)
 	nsresult	rv;
 	
 	nsCOMPtr<nsIMsgAccountManager> accMgr = 
-	         do_GetService(kMsgAccountMgrCID, &rv);
+	         do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) {
 		IMPORT_LOG0( "*** Failed to create a account manager!\n");
 		return( PR_FALSE);
