@@ -56,6 +56,7 @@
 #include "nsImportFieldMap.h"
 #include "nsICategoryManager.h"
 #include "nsXPIDLString.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsImportStringBundle.h"
 #include "plstr.h"
@@ -185,7 +186,7 @@ NS_IMETHODIMP NS_NewImportService( nsISupports* aOuter, REFNSIID aIID, void **aR
 
 nsImportService::nsImportService() : m_pModules( nsnull)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 
 	IMPORT_LOG0( "* nsImport Service Created\n");
 
@@ -615,7 +616,7 @@ nsresult nsImportService::DoDiscover( void)
 	nsCOMPtr<nsISimpleEnumerator> e;
 	rv = catMan->EnumerateCategory( "mailnewsimport", getter_AddRefs( e));
 	if (NS_FAILED( rv)) return( rv);
-	nsCOMPtr<nsISupportsString> contractid;
+	nsCOMPtr<nsISupportsCString> contractid;
 	rv = e->GetNext( getter_AddRefs( contractid));
 	while (NS_SUCCEEDED( rv) && contractid) {
 		nsXPIDLCString	contractIdStr;
