@@ -196,7 +196,7 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest *request, nsISupports
     }
 #endif
 
-
+    ReleaseFolderLock();
     // be sure to clear any status text and progress info..
     m_graph_progress_received = 0;
     UpdateProgressPercent();
@@ -248,12 +248,26 @@ NS_IMETHODIMP nsMsgMailboxParser::OnReadChanged(nsIDBChangeListener *instigator)
     return NS_OK;
 }
 
-
 nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nsnull, PR_FALSE)
 {
-  /* the following macro is used to initialize the ref counting data */
-	NS_INIT_REFCNT();
+  Init();
+}
 
+nsMsgMailboxParser::nsMsgMailboxParser(nsIMsgFolder *aFolder) : nsMsgLineBuffer(nsnull, PR_FALSE)
+{
+  Init();
+  m_folder = getter_AddRefs(NS_GetWeakReference(aFolder));
+}
+
+nsMsgMailboxParser::~nsMsgMailboxParser()
+{
+  ReleaseFolderLock();
+}
+
+void nsMsgMailboxParser::Init()
+{
+  /* the following macro is used to initialize the ref counting data */
+	NS_INIT_ISUPPORTS();
 	m_obuffer = nsnull;
 	m_obuffer_size = 0;
 	m_graph_progress_total = 0;
@@ -261,18 +275,6 @@ nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nsnull, PR_FALSE)
 	m_updateAsWeGo = PR_TRUE;
 	m_ignoreNonMailFolder = PR_FALSE;
 	m_isRealMailFolder = PR_TRUE;
-  m_folder=nsnull;
-
-}
-
-nsMsgMailboxParser::~nsMsgMailboxParser()
-{
-}
-
-void nsMsgMailboxParser::SetFolder(nsIMsgFolder *aFolder)
-{
-  if (aFolder)
-    m_folder = getter_AddRefs(NS_GetWeakReference(aFolder));
 }
 
 void nsMsgMailboxParser::UpdateStatusText (PRUint32 stringID)
@@ -486,11 +488,26 @@ PRInt32 nsMsgMailboxParser::HandleLine(char *line, PRUint32 lineLength)
 
 }
 
+void
+nsMsgMailboxParser::ReleaseFolderLock()
+{
+  nsresult result;
+  nsCOMPtr<nsIMsgFolder> folder = do_QueryReferent(m_folder);
+  if (!folder) 
+    return;
+  PRBool haveSemaphore;
+  nsCOMPtr <nsISupports> supports = do_QueryInterface(NS_STATIC_CAST(nsIMsgParseMailMsgState*, this));
+  result = folder->TestSemaphore(supports, &haveSemaphore);
+  if(NS_SUCCEEDED(result) && haveSemaphore)
+    result = folder->ReleaseSemaphore(supports);
+  return;
+}
+
 NS_IMPL_ISUPPORTS1(nsParseMailMessageState, nsIMsgParseMailMsgState)
 
 nsParseMailMessageState::nsParseMailMessageState()
 {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 	m_position = 0;
 	m_IgnoreXMozillaStatus = PR_FALSE;
 	m_state = nsIMsgParseMailMsgState::ParseBodyState;
@@ -762,6 +779,24 @@ NS_IMETHODIMP nsParseMailMessageState::GetAllHeaders(char ** pHeaders, PRInt32 *
 	return NS_OK;
 }
 
+// generate headers as a string, with CRLF between the headers
+NS_IMETHODIMP nsParseMailMessageState::GetHeaders(char ** pHeaders)
+{
+  NS_ENSURE_ARG_POINTER(pHeaders);
+  nsCString crlfHeaders;
+  char *curHeader = m_headers.GetBuffer();
+  for (PRInt32 headerPos = 0; headerPos < m_headers.GetBufferPos();)
+  {
+    crlfHeaders.Append(curHeader);
+    crlfHeaders.Append(CRLF);
+    PRInt32 headerLen = strlen(curHeader);
+    curHeader += headerLen + 1;
+    headerPos += headerLen + 1;
+  }
+  *pHeaders = nsCRT::strdup(crlfHeaders.get());
+  return NS_OK;
+}
+
 struct message_header *nsParseMailMessageState::GetNextHeaderInAggregate (nsVoidArray &list)
 {
 	// When parsing a message with multiple To or CC header lines, we're storing each line in a 
@@ -1266,8 +1301,8 @@ int nsParseMailMessageState::FinalizeHeaders()
         if (ret == NS_OK)
         {
           m_newMsgHdr->SetRecipientsArray(names, addresses, numAddresses);
-          PR_FREEIF(addresses);
-          PR_FREEIF(names);
+          PR_Free(addresses);
+          PR_Free(names);
         }
         else {	// hmm, should we just use the original string?
           m_newMsgHdr->SetRecipients(recipient->value);
@@ -1283,8 +1318,8 @@ int nsParseMailMessageState::FinalizeHeaders()
         if (ret == NS_OK)
         {
           m_newMsgHdr->SetCCListArray(names, addresses, numAddresses);
-          PR_FREEIF(addresses);
-          PR_FREEIF(names);
+          PR_Free(addresses);
+          PR_Free(names);
         }
         else	// hmm, should we just use the original string?
           m_newMsgHdr->SetCcList(ccList->value);
@@ -1411,7 +1446,7 @@ int nsParseMailMessageState::FinalizeHeaders()
 
   //### why is this stuff const?
   char *tmp = (char*) to.value;
-  PR_FREEIF(tmp);
+  PR_Free(tmp);
   tmp = (char*) cc.value;
   PR_Free(tmp);
 
@@ -1422,7 +1457,6 @@ nsParseNewMailState::nsParseNewMailState()
     : m_tmpdbName(nsnull), m_usingTempDB(PR_FALSE), m_disableFilters(PR_FALSE)
 {
 	m_inboxFileStream = nsnull;
-	m_logFile = nsnull;
 	m_ibuffer = nsnull;
 	m_ibuffer_size = 0;
 	m_ibuffer_fp = 0;
@@ -1459,7 +1493,6 @@ nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, n
     rv = rootMsgFolder->GetServer(getter_AddRefs(server));
     if (NS_SUCCEEDED(rv))
       rv = server->GetFilterList(aMsgWindow, getter_AddRefs(m_filterList));
-    m_logFile = nsnull;
    
 	if (m_filterList)
       rv = server->ConfigureTemporaryReturnReceiptsFilter(m_filterList);
@@ -1473,11 +1506,6 @@ nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, n
 
 nsParseNewMailState::~nsParseNewMailState()
 {
-	if (m_logFile != nsnull)
-	{
-		m_logFile->close();
-		delete m_logFile;
-	}
 	if (m_mailDB)
 		m_mailDB->Close(PR_TRUE);
 //	if (m_usingTempDB)
@@ -1560,29 +1588,6 @@ void	nsParseNewMailState::SetUsingTempDB(PRBool usingTempDB, char *tmpDBName)
 	m_tmpdbName = tmpDBName;
 }
 
-
-nsOutputFileStream * nsParseNewMailState::GetLogFile ()
-{
-	// This log file is used by regular filters and JS filters
-	if (m_logFile == nsnull)
-	{
-    nsCOMPtr<nsIFile> logDir;
-    NS_GetSpecialDirectory(NS_APP_MAIL_50_DIR, getter_AddRefs(logDir));
-
-    logDir->AppendNative(NS_LITERAL_CSTRING("filter.log"));
-
-    nsCAutoString pathBuf;
-    nsresult rv = logDir->GetNativePath(pathBuf);
-    if (NS_FAILED(rv)) return nsnull;
-
-		nsFileSpec logFile(pathBuf.get());
-
-		m_logFile = new nsOutputFileStream(logFile, PR_WRONLY | PR_CREATE_FILE, 00600);
-	}
-	return m_logFile;
-}
-
-
 nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder **pTrashFolder)
 {
 	nsresult rv=NS_ERROR_UNEXPECTED;
@@ -1663,7 +1668,6 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
 
 		msgHdr->GetFlags(&msgFlags);
 
-		PRBool isRead = (msgFlags & MSG_FLAG_READ);
 		switch (actionType)
 		{
 		case nsMsgFilterAction::Delete :
@@ -1728,7 +1732,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
         if (m_filterList)
 	  m_filterList->GetLoggingEnabled(&loggingEnabled);
 	if (loggingEnabled && !m_msgMovedByFilter && actionType != nsMsgFilterAction::MoveToFolder)
-		filter->LogRuleHit(GetLogFile(), msgHdr);
+		filter->LogRuleHit(msgHdr);
 	}
 	return rv;
 }
@@ -1768,7 +1772,8 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
   PRBool canFileMessages = PR_TRUE;
   nsCOMPtr<nsIFolder> parentFolder;
   destIFolder->GetParent(getter_AddRefs(parentFolder));
-  destIFolder->GetCanFileMessages(&canFileMessages);
+  if (parentFolder)
+    destIFolder->GetCanFileMessages(&canFileMessages);
   if (!parentFolder || !canFileMessages)
   {
     filter->SetEnabled(PR_FALSE);
@@ -1941,7 +1946,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
   if (m_filterList)
     m_filterList->GetLoggingEnabled(&loggingEnabled);
   if (loggingEnabled)
-    filter->LogRuleHit(GetLogFile(), mailHdr);
+    filter->LogRuleHit(mailHdr);
   
   // cleanup after mailHdr in source DB because we moved the message.
   m_mailDB->RemoveHeaderMdbRow(mailHdr);
@@ -2060,8 +2065,11 @@ nsresult ParseIMAPMailboxState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
 					idsToMoveFromInbox->Add(keyToFilter);
 
 					// this is our last best chance to log this
-					if (m_filterList->LoggingEnabled())
-						filter->LogRuleHit(GetLogFile(), mailHdr);
+                                        PRBool loggingEnabled = PR_FALSE;
+                                        if (m_filterList)
+                                          m_filterList->GetLoggingEnabled(&loggingEnabled);
+                                        if (loggingEnabled)
+                                          filter->LogRuleHit(mailHdr);
 
 					if (imapDeleteIsMoveToTrash)
 					{
