@@ -200,18 +200,19 @@ mime_dump_attachments ( nsMsgAttachmentData *attachData )
 #endif
 
 nsresult
-CreateTheComposeWindow(nsIMsgCompFields     *compFields,
-                       nsMsgAttachmentData  *attachmentList,
-                       MSG_ComposeType    composeType,
+CreateTheComposeWindow(nsIMsgCompFields *   compFields,
+                       nsMsgAttachmentData *attachmentList,
+                       MSG_ComposeType      composeType,
                        MSG_ComposeFormat    composeFormat,
-                       nsIMsgIdentity *       identity
+                       nsIMsgIdentity *     identity,
+                       const char *         originalMsgURI
                        )
 {
-nsresult            rv;
-MSG_ComposeFormat   format = nsIMsgCompFormat::Default;
+  nsresult            rv;
+  MSG_ComposeFormat   format = nsIMsgCompFormat::Default;
 
 #ifdef NS_DEBUG
-mime_dump_attachments ( attachmentList );
+  mime_dump_attachments ( attachmentList );
 #endif
 
   nsMsgAttachmentData *curAttachment = attachmentList;
@@ -272,7 +273,9 @@ mime_dump_attachments ( attachmentList );
     pMsgComposeParams->SetType(composeType);
     pMsgComposeParams->SetFormat(format);
     pMsgComposeParams->SetIdentity(identity);
-    pMsgComposeParams->SetComposeFields(compFields);    
+    pMsgComposeParams->SetComposeFields(compFields);
+    if (originalMsgURI)
+      pMsgComposeParams->SetOriginalMsgURI(originalMsgURI);
     
     rv = msgComposeService->OpenComposeWindowWithParams(nsnull /* default chrome */, pMsgComposeParams);
   }
@@ -1290,13 +1293,13 @@ mime_parse_stream_complete (nsMIMESession *stream)
     {
       // save the override flag before it's unavailable
       charsetOverride = mdd->options->override_charset;
-      if ((!mdd->mailcharset || charsetOverride) && mdd->options->default_charset)
+      if ((!mdd->mailcharset || charsetOverride) && mdd->options->default_charset) 
       {
         PR_Free(mdd->mailcharset);
         mdd->mailcharset = nsCRT::strdup(mdd->options->default_charset);
       }
 
-      // mscott: aren't we leaking a bunch of trings here like the charset strings and such?
+      // mscott: aren't we leaking a bunch of strings here like the charset strings and such?
       delete mdd->options;
       mdd->options = 0;
     }
@@ -1465,6 +1468,15 @@ mime_parse_stream_complete (nsMIMESession *stream)
           {
             if (body && composeFormat == nsIMsgCompFormat::PlainText)
             {
+              //We need to convert the plain/text to HTML in order to escape any HTML markup
+              char *escapedBody = nsEscapeHTML(body);
+              if (escapedBody)
+              {
+                PR_Free(body);
+                body = escapedBody;
+                bodyLen = strlen(body);
+              }
+
               char* newbody = (char *)PR_MALLOC (bodyLen + 12); //+11 chars for <pre> & </pre> tags
               if (newbody)
               {
@@ -1528,7 +1540,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
 #ifdef NS_DEBUG
         printf("RICHIE: Time to create the EDITOR with this template - HAS a body!!!!\n");
 #endif
-        CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Template, composeFormat, mdd->identity);
+        CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Template, composeFormat, mdd->identity, nsnull);
       }
       else
       {
@@ -1536,11 +1548,11 @@ mime_parse_stream_complete (nsMIMESession *stream)
         printf("Time to create the composition window WITH a body!!!!\n");
 #endif
         if (mdd->forwardInline)
-          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::ForwardInline, composeFormat, mdd->identity);
-        else
+          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::ForwardInline, composeFormat, mdd->identity, mdd->originalMsgURI);
+          else
         {
           fields->SetDraftId(mdd->url_name);
-          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Draft, composeFormat, mdd->identity);
+          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Draft, composeFormat, mdd->identity, nsnull);
         }
       }
     }
@@ -1555,7 +1567,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
 #ifdef NS_DEBUG
         printf("RICHIE: Time to create the EDITOR with this template - NO body!!!!\n");
 #endif
-        CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Template, nsIMsgCompFormat::Default, mdd->identity);
+        CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Template, nsIMsgCompFormat::Default, mdd->identity, nsnull);
       }
       else
       {
@@ -1563,11 +1575,11 @@ mime_parse_stream_complete (nsMIMESession *stream)
         printf("Time to create the composition window WITHOUT a body!!!!\n");
 #endif
         if (mdd->forwardInline)
-          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::ForwardInline, nsIMsgCompFormat::Default, mdd->identity);
+          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::ForwardInline, nsIMsgCompFormat::Default, mdd->identity, mdd->originalMsgURI);
         else
         {
           fields->SetDraftId(mdd->url_name);
-          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Draft, nsIMsgCompFormat::Default, mdd->identity);
+          CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Draft, nsIMsgCompFormat::Default, mdd->identity, nsnull);
         }
       }
     }    
@@ -1581,7 +1593,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
       mdd->mailcharset,
       getter_AddRefs(fields));
     if (fields)
-      CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::New, nsIMsgCompFormat::Default, mdd->identity);
+      CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::New, nsIMsgCompFormat::Default, mdd->identity, nsnull);
   }
   
   if ( mdd->headers )
@@ -1626,7 +1638,9 @@ mime_parse_stream_complete (nsMIMESession *stream)
     nsServiceManager::ReleaseService(kPrefCID, obj->options->prefs);
   
   mdd->identity = nsnull;
-  PR_Free (mdd);
+  PR_Free(mdd->url_name);
+  PR_Free(mdd->originalMsgURI);
+  PR_Free(mdd);
   
   PR_FREEIF(host);
   PR_FREEIF(to_and_cc);
@@ -1819,7 +1833,14 @@ mime_decompose_file_init_fn ( void *stream_closure, MimeHeaders *headers )
   newAttachment->type =  MimeHeaders_get ( headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE );
 
   if (PL_strstr(newAttachment->type, MESSAGE_RFC822))
-     PL_strcat(newAttachment->real_name, ".eml");      
+  {
+    char *newName = PR_smprintf("%s.eml", newAttachment->real_name);
+    if (newName)
+    {
+      PR_Free(newAttachment->real_name);
+      newAttachment->real_name = newName;
+    }
+  }      
   
   //
   // This is to handle the degenerated Apple Double attachment.
@@ -2015,7 +2036,7 @@ mime_bridge_create_draft_stream(
   int                     status = 0;
   nsMIMESession           *stream = nsnull;
   struct mime_draft_data  *mdd = nsnull;
-  MimeObject              *obj;
+  MimeObject              *obj = nsnull;
 
   if ( !uri ) 
     return nsnull;
@@ -2024,42 +2045,38 @@ mime_bridge_create_draft_stream(
   if (!mdd) 
     return nsnull;
 
-  // first, convert the rdf msg uri into a url that represents the message...
   nsCAutoString turl;
-  if (NS_FAILED(uri->GetSpec(turl)))
-    return nsnull;
-
   nsCOMPtr <nsIMsgMessageService> msgService;
-  nsresult rv = GetMessageServiceFromURI(turl.get(), getter_AddRefs(msgService));
-  if (NS_FAILED(rv)) 
-    return nsnull;
-
   nsCOMPtr<nsIURI> aURL;
+  nsCAutoString urlString;
+  nsresult rv;
+
+  // first, convert the rdf msg uri into a url that represents the message...
+  if (NS_FAILED(uri->GetSpec(turl)))
+    goto FAIL;
+
+  rv = GetMessageServiceFromURI(turl.get(), getter_AddRefs(msgService));
+  if (NS_FAILED(rv)) 
+    goto FAIL;
+
   rv = msgService->GetUrlForUri(turl.get(), getter_AddRefs(aURL), nsnull);
   if (NS_FAILED(rv)) 
-    return nsnull;
+    goto FAIL;
 
-  nsCAutoString urlString;
   if (NS_SUCCEEDED(aURL->GetSpec(urlString)))
   {
     mdd->url_name = ToNewCString(urlString);
     if (!(mdd->url_name))
-    {
-      PR_FREEIF(mdd);
-      return nsnull;
-    }
+      goto FAIL;
   }
 
   newPluginObj2->GetForwardInline(&mdd->forwardInline);
   newPluginObj2->GetIdentity(getter_AddRefs(mdd->identity));
+  newPluginObj2->GetOriginalMsgURI(&mdd->originalMsgURI);
   mdd->format_out = format_out;
   mdd->options = new  MimeDisplayOptions ;
-  if ( !mdd->options ) 
-  {
-    PR_FREEIF(mdd->url_name);
-    PR_FREEIF(mdd);
-    return nsnull;
-  }
+  if (!mdd->options)
+    goto FAIL;
 
   mdd->options->url = nsCRT::strdup(mdd->url_name);
   mdd->options->format_out = format_out;     // output format
@@ -2073,41 +2090,27 @@ mime_bridge_create_draft_stream(
 
   rv = nsServiceManager::GetService(kPrefCID, NS_GET_IID(nsIPref), (nsISupports**)&(mdd->options->prefs));
   if (! (mdd->options->prefs && NS_SUCCEEDED(rv)))
-  {
-    PR_FREEIF(mdd);
-    return nsnull;
-  }
+    goto FAIL;
 
-#ifdef FO_MAIL_MESSAGE_TO
+#ifdef ENABLE_SMIME
   /* If we're attaching a message (for forwarding) then we must eradicate all
    traces of xlateion from it, since forwarding someone else a message
    that wasn't xlated for them doesn't work.  We have to dexlate it
    before sending it.
    */
-// RICHIE  mdd->options->dexlate_p = PR_TRUE;
-#endif /* FO_MAIL_MESSAGE_TO */
+  mdd->options->decrypt_p = PR_TRUE;
+#endif /* ENABLE_SMIME */
 
   obj = mime_new ( (MimeObjectClass *) &mimeMessageClass, (MimeHeaders *) NULL, MESSAGE_RFC822 );
   if ( !obj ) 
-  {
-    PR_FREEIF(mdd->url_name);
-    delete mdd->options;
-    PR_FREEIF(mdd );
-    return nsnull;
-  }
+    goto FAIL;
   
   obj->options = mdd->options;
   mdd->obj = obj;
 
   stream = PR_NEWZAP ( nsMIMESession );
   if ( !stream ) 
-  {
-    PR_FREEIF(mdd->url_name);
-    delete mdd->options;
-    PR_FREEIF ( mdd );
-    PR_FREEIF ( obj );
-    return nsnull;
-  }
+    goto FAIL;
 
   stream->name = "MIME To Draft Converter Stream";
   stream->complete = mime_parse_stream_complete;
@@ -2118,15 +2121,22 @@ mime_bridge_create_draft_stream(
   status = obj->clazz->initialize ( obj );
   if ( status >= 0 )
     status = obj->clazz->parse_begin ( obj );
-  if ( status < 0 ) 
-  {
-    PR_FREEIF(mdd->url_name);
-    PR_FREEIF ( stream );
-    delete mdd->options;
-    PR_FREEIF ( mdd );
-    PR_FREEIF ( obj );
-    return nsnull;
-  }
+  if ( status < 0 )
+    goto FAIL;
 
   return stream;
+
+FAIL:
+  if (mdd)
+  {
+    PR_Free(mdd->url_name);
+    PR_Free(mdd->originalMsgURI);
+    if (mdd->options)
+      delete mdd->options;
+    PR_Free ( mdd );
+  }
+  PR_Free ( stream );
+  PR_Free ( obj );
+
+  return nsnull;
 }
