@@ -1541,6 +1541,7 @@ mime_bridge_create_display_stream(
     case nsMimeOutput::nsMimeMessageRaw:              // the raw RFC822 data (view source) and attachments
 		case nsMimeOutput::nsMimeMessageDraftOrTemplate:  // Loading drafts & templates
 		case nsMimeOutput::nsMimeMessageEditorTemplate:   // Loading templates into editor
+    case nsMimeOutput::nsMimeMessageFilterSniffer:    // generating an output that can be scan by a message filter
       break;
   }
 
@@ -1942,9 +1943,6 @@ mimeSetNewURL(nsMIMESession *stream, char *url)
   return NS_OK;
 }
 
-/* This is the next generation string retrieval call */
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
-
 #define     MIME_URL "chrome://messenger/locale/mime.properties"
 
 extern "C" 
@@ -1962,7 +1960,7 @@ MimeGetStringByID(PRInt32 stringID)
     propertyURL = MIME_URL;
 
     nsCOMPtr<nsIStringBundleService> sBundleService = 
-             do_GetService(kStringBundleServiceCID, &res); 
+             do_GetService(NS_STRINGBUNDLE_CONTRACTID, &res); 
     if (NS_SUCCEEDED(res) && (nsnull != sBundleService)) 
     {
       res = sBundleService->CreateBundle(propertyURL, getter_AddRefs(stringBundle));
