@@ -23,6 +23,7 @@
 
 #include "imgRequest.h"
 
+#include "imgLoader.h"
 #include "imgCache.h"
 #include "imgRequestProxy.h"
 
@@ -32,10 +33,14 @@
 
 #include "gfxIImageFrame.h"
 
+#include "netCore.h"
+
 #include "nsIChannel.h"
 #include "nsICachingChannel.h"
 #include "nsILoadGroup.h"
 #include "nsIInputStream.h"
+#include "nsIMultiPartChannel.h"
+#include "nsIHttpChannel.h"
 
 #include "nsIComponentManager.h"
 #include "nsIProxyObjectManager.h"
@@ -44,6 +49,7 @@
 #include "nsAutoLock.h"
 #include "nsString.h"
 #include "nsXPIDLString.h"
+#include "plstr.h" // PL_strcasestr(...)
 
 #if defined(PR_LOGGING)
 PRLogModuleInfo *gImgLog = PR_NewLogModule("imgRequest");
@@ -58,7 +64,7 @@ imgRequest::imgRequest() :
   mObservers(0),
   mLoading(PR_FALSE), mProcessing(PR_FALSE),
   mImageStatus(imgIRequest::STATUS_NONE), mState(0),
-  mContentType(nsnull), mCacheId(0), mValidator(nsnull)
+  mCacheId(0), mValidator(nsnull), mIsMultiPartChannel(PR_FALSE)
 {
   NS_INIT_ISUPPORTS();
   /* member initializers and constructor code */
@@ -67,8 +73,6 @@ imgRequest::imgRequest() :
 imgRequest::~imgRequest()
 {
   /* destructor code */
-  if (mContentType)
-    nsCRT::free(mContentType);
 }
 
 nsresult imgRequest::Init(nsIChannel *aChannel,
@@ -217,6 +221,7 @@ nsresult imgRequest::NotifyProxyListener(imgRequestProxy *proxy)
   if (mImage && (mObservers.Count() == 1)) {
     LOG_MSG(gImgLog, "imgRequest::AddProxy", "starting animation");
 
+    mImage->ResetAnimation();
     mImage->StartAnimation();
   }
 
@@ -307,6 +312,14 @@ NS_IMETHODIMP imgRequest::GetImage(imgIContainer **aImage)
   return NS_OK;
 }
 
+NS_IMETHODIMP imgRequest::GetIsMultiPartChannel(PRBool *aIsMultiPartChannel)
+{
+  LOG_FUNC(gImgLog, "imgRequest::GetIsMultiPartChannel");
+
+  *aIsMultiPartChannel = mIsMultiPartChannel;
+
+  return NS_OK;
+}
 
 /** imgIContainerObserver methods **/
 
@@ -437,6 +450,8 @@ NS_IMETHODIMP imgRequest::OnStopFrame(imgIRequest *request, nsISupports *cx, gfx
 
   LOG_SCOPE(gImgLog, "imgRequest::OnStopFrame");
 
+  mImageStatus |= imgIRequest::STATUS_FRAME_COMPLETE;
+
   if (mCacheEntry) {
     PRUint32 cacheSize = 0;
 
@@ -519,6 +534,8 @@ NS_IMETHODIMP imgRequest::OnStopDecode(imgIRequest *aRequest, nsISupports *aCX,
 /* void onStartRequest (in nsIRequest request, in nsISupports ctxt); */
 NS_IMETHODIMP imgRequest::OnStartRequest(nsIRequest *aRequest, nsISupports *ctxt)
 {
+  nsresult rv;
+
   LOG_SCOPE(gImgLog, "imgRequest::OnStartRequest");
 
   NS_ASSERTION(!mDecoder, "imgRequest::OnStartRequest -- we already have a decoder");
@@ -531,9 +548,19 @@ NS_IMETHODIMP imgRequest::OnStartRequest(nsIRequest *aRequest, nsISupports *ctxt
      open forever.
    */
   if (!mChannel) {
-    mChannel = do_QueryInterface(aRequest);
+    nsCOMPtr<nsIMultiPartChannel> mpchan(do_QueryInterface(aRequest));
+    if (mpchan)
+      mpchan->GetBaseChannel(getter_AddRefs(mChannel));
+    else
+      mChannel = do_QueryInterface(aRequest);
   }
 
+  nsCAutoString contentType;
+  mChannel->GetContentType(contentType);
+  if (contentType.Equals(NS_LITERAL_CSTRING("multipart/x-mixed-replace"),
+                         nsCaseInsensitiveCStringComparator()))
+      mIsMultiPartChannel = PR_TRUE;
+
   /* set our state variables to their initial values. */
   mImageStatus = imgIRequest::STATUS_NONE;
   mState = 0;
@@ -556,8 +583,8 @@ NS_IMETHODIMP imgRequest::OnStartRequest(nsIRequest *aRequest, nsISupports *ctxt
   nsCOMPtr<nsIChannel> chan(do_QueryInterface(aRequest));
 
   /* get the expires info */
-  if (mCacheEntry && chan) {
-    nsCOMPtr<nsICachingChannel> cacheChannel(do_QueryInterface(chan));
+  if (mCacheEntry) {
+    nsCOMPtr<nsICachingChannel> cacheChannel(do_QueryInterface(aRequest));
     if (cacheChannel) {
       nsCOMPtr<nsISupports> cacheToken;
       cacheChannel->GetCacheToken(getter_AddRefs(cacheToken));
@@ -573,6 +600,43 @@ NS_IMETHODIMP imgRequest::OnStartRequest(nsIRequest *aRequest, nsISupports *ctxt
         }
       }
     }
+    //
+    // Determine whether the cache entry must be revalidated when it expires.
+    // If so, then the cache entry must *not* be used during HISTORY loads if
+    // it has expired.
+    //
+    // Currently, only HTTP specifies this information...
+    //
+    nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(aRequest));
+    if (httpChannel) {
+      PRBool bMustRevalidate = PR_FALSE;
+
+      rv = httpChannel->IsNoStoreResponse(&bMustRevalidate);
+
+      if (!bMustRevalidate) {
+        rv = httpChannel->IsNoCacheResponse(&bMustRevalidate);
+      }
+
+      if (!bMustRevalidate) {
+        nsCAutoString cacheHeader;
+
+        rv = httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Cache-Control"),
+                                            cacheHeader);
+        if (PL_strcasestr(cacheHeader.get(), "must-revalidate")) {
+          bMustRevalidate = PR_TRUE;
+        }
+      }
+
+      if (bMustRevalidate) {
+        mCacheEntry->SetMetaDataElement("MustValidateIfExpired", "true");
+      }
+    }
+  }
+
+
+  // Shouldn't we be dead already if this gets hit?  Probably multipart/x-mixed-replace...
+  if (mObservers.Count() == 0) {
+    this->Cancel(NS_IMAGELIB_ERROR_FAILURE);
   }
 
   return NS_OK;
@@ -608,7 +672,7 @@ NS_IMETHODIMP imgRequest::OnStopRequest(nsIRequest *aRequest, nsISupports *ctxt,
     mDecoder->Close();
     mDecoder = nsnull; // release the decoder so that it can rest peacefully ;)
   }
- 
+
   // if there was an error loading the image, (mState & onStopDecode) won't be true.
   // Send an onStopDecode message
   if (!(mState & onStopDecode)) {
@@ -661,15 +725,14 @@ NS_IMETHODIMP imgRequest::OnDataAvailable(nsIRequest *aRequest, nsISupports *ctx
     /* NS_WARNING if the content type from the channel isn't the same if the sniffing */
 #endif
 
-    if (!mContentType) {
+    if (mContentType.IsEmpty()) {
       LOG_SCOPE(gImgLog, "imgRequest::OnDataAvailable |sniffing of mimetype failed|");
 
-      nsXPIDLCString contentType;
       nsCOMPtr<nsIChannel> chan(do_QueryInterface(aRequest));
 
       nsresult rv = NS_ERROR_FAILURE;
       if (chan) {
-        rv = chan->GetContentType(getter_Copies(contentType));
+        rv = chan->GetContentType(mContentType);
       }
 
       if (NS_FAILED(rv)) {
@@ -683,14 +746,11 @@ NS_IMETHODIMP imgRequest::OnDataAvailable(nsIRequest *aRequest, nsISupports *ctx
       }
 
       LOG_MSG(gImgLog, "imgRequest::OnDataAvailable", "Got content type from the channel");
-
-      mContentType = nsCRT::strdup(contentType.get());
     }
 
-    LOG_MSG_WITH_PARAM(gImgLog, "imgRequest::OnDataAvailable", "content type", mContentType);
+    LOG_MSG_WITH_PARAM(gImgLog, "imgRequest::OnDataAvailable", "content type", mContentType.get());
 
-    nsCAutoString conid("@mozilla.org/image/decoder;2?type=");
-    conid += mContentType;
+    nsCAutoString conid(NS_LITERAL_CSTRING("@mozilla.org/image/decoder;2?type=") + mContentType);
 
     mDecoder = do_CreateInstance(conid.get());
 
@@ -760,85 +820,5 @@ static NS_METHOD sniff_mimetype_callback(nsIInputStream* in,
 void
 imgRequest::SniffMimeType(const char *buf, PRUint32 len)
 {
-  /* Is it a GIF? */
-
-  if (mContentType) {
-    nsCRT::free(mContentType);
-    mContentType = nsnull;
-  }
-
-  if (len >= 4 && !nsCRT::strncmp(buf, "GIF8", 4))  {
-    mContentType = nsCRT::strndup("image/gif", 9);
-    return;
-  }
-
-  /* or a PNG? */
-  if (len >= 4 && ((unsigned char)buf[0]==0x89 &&
-                   (unsigned char)buf[1]==0x50 &&
-                   (unsigned char)buf[2]==0x4E &&
-                   (unsigned char)buf[3]==0x47))
-  { 
-    mContentType = nsCRT::strndup("image/png", 9);
-    return;
-  }
-
-  /* maybe a JPEG (JFIF)? */
-  /* JFIF files start with SOI APP0 but older files can start with SOI DQT
-   * so we test for SOI followed by any marker, i.e. FF D8 FF
-   * this will also work for SPIFF JPEG files if they appear in the future.
-   *
-   * (JFIF is 0XFF 0XD8 0XFF 0XE0 <skip 2> 0X4A 0X46 0X49 0X46 0X00)
-   */
-  if (len >= 3 &&
-     ((unsigned char)buf[0])==0xFF &&
-     ((unsigned char)buf[1])==0xD8 &&
-     ((unsigned char)buf[2])==0xFF)
-  {
-    mContentType = nsCRT::strndup("image/jpeg", 10);
-    return;
-  }
-
-  /* or how about ART? */
-  /* ART begins with JG (4A 47). Major version offset 2.
-   * Minor version offset 3. Offset 4 must be NULL.
-   */
-  if (len >= 5 &&
-   ((unsigned char) buf[0])==0x4a &&
-   ((unsigned char) buf[1])==0x47 &&
-   ((unsigned char) buf[4])==0x00 )
-  {
-    mContentType = nsCRT::strndup("image/x-jg", 10);
-    return;
-  }
-
-  if (len >= 2 && !nsCRT::strncmp(buf, "BM", 2)) {
-    mContentType = nsCRT::strndup("image/bmp", 9);
-    return;
-  }
-
-  // ICOs always begin with a 2-byte 0 followed by a 2-byte 1.
-  if (len >= 4 && !nsCRT::memcmp(buf, "\000\000\001\000", 4)) {
-    mContentType = nsCRT::strndup("image/x-icon", 12);
-    return;
-  }
-
-  if (len >= 4 && ((unsigned char)buf[0]==0x8A &&
-                   (unsigned char)buf[1]==0x4D &&
-                   (unsigned char)buf[2]==0x4E &&
-                   (unsigned char)buf[3]==0x47))
-  { 
-    mContentType = nsCRT::strndup("video/x-mng", 11);
-    return;
-  }
-
-  if (len >= 4 && ((unsigned char)buf[0]==0x8B &&
-                   (unsigned char)buf[1]==0x4A &&
-                   (unsigned char)buf[2]==0x4E &&
-                   (unsigned char)buf[3]==0x47))
-  { 
-    mContentType = nsCRT::strndup("image/x-jng", 11);
-    return;
-  }
-
-  /* none of the above?  I give up */
+  imgLoader::GetMimeTypeFromContent(buf, len, getter_Copies(mContentType));
 }
