@@ -42,6 +42,7 @@
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsICategoryManager.h"
+#include "nsCategoryManagerUtils.h"
 #include "nsIFile.h"
 #include "nsIFileStreams.h"
 #include "nsILocalFile.h"
@@ -84,7 +85,7 @@ nsPrefService::nsPrefService()
 {
   nsPrefBranch *rootBranch;
 
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   rootBranch = new nsPrefBranch("", PR_FALSE); 
   mRootBranch = (nsIPrefBranch *)rootBranch;
@@ -179,9 +180,8 @@ NS_IMETHODIMP nsPrefService::ReadUserPrefs(nsIFile *aFile)
   nsresult rv;
 
   if (nsnull == aFile) {
-    rv = UseDefaultPrefFile();  // really should return a value...
-    if (NS_SUCCEEDED(rv))
-      UseUserPrefFile(); 
+    rv = UseDefaultPrefFile();
+    UseUserPrefFile();
 
     NotifyServiceObservers(NS_PREFSERVICE_READ_TOPIC_ID);
     
@@ -447,6 +447,11 @@ static nsresult openPrefFile(nsIFile* aFile, PRBool aIsErrorFatal,
   PRUint32 amtRead = 0;
   rv = inStr->Read(readBuf, fileSize, &amtRead);
   NS_ASSERTION((amtRead == fileSize), "failed to read the entire prefs file!!");
+ #ifdef XP_OS2 /* OS/2 workaround - our system editor adds an EOF character */
+     if (readBuf[amtRead - 1] == 0x1A) {
+        amtRead--;
+     }
+ #endif
   if (NS_SUCCEEDED(rv)) {
     if (!PREF_EvaluateConfigScript(readBuf, amtRead, nsnull, aIsGlobalContext, PR_TRUE,
                                    aSkipFirstLine))
@@ -508,11 +513,8 @@ JSBool pref_InitInitialObjects()
   PRBool            hasMoreElements;
         
   static const char* specialFiles[] = {
-#ifdef XP_MAC
+#if defined(XP_MAC) || defined(XP_MACOSX)
       "macprefs.js"
-#if defined (TARGET_CARBON)
-      , "macxprefs.js"
-#endif
 #elif defined(XP_WIN)
       "winpref.js"
 #elif defined(XP_UNIX)
