@@ -38,7 +38,7 @@
 
 
 /*
- *  npapi.h $Revision: 3.23 $
+ *  npapi.h $Revision: 3.25 $
  *  Netscape client plug-in API spec
  */
 
@@ -353,7 +353,9 @@ typedef enum {
   NPNVisOfflineBool,
 
   /* 10 and over are available on Mozilla builds starting with 0.9.4 */
-  NPNVserviceManager = 10
+  NPNVserviceManager = 10,
+  NPNVDOMElement     = 11,   /* available in Mozilla 1.2 */
+  NPNVDOMWindow      = 12
 } NPNVariable;
 
 /*
