@@ -74,7 +74,7 @@ nsBufferedStream::nsBufferedStream()
       mBufferDisabled(PR_FALSE),
       mGetBufferCount(0)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 nsBufferedStream::~nsBufferedStream()
@@ -90,7 +90,7 @@ nsBufferedStream::Init(nsISupports* stream, PRUint32 bufferSize)
     NS_ASSERTION(stream, "need to supply a stream");
     NS_ASSERTION(mStream == nsnull, "already inited");
     mStream = stream;
-    NS_ADDREF(mStream);
+    NS_IF_ADDREF(mStream);
     mBufferSize = bufferSize;
     mBufferStartOffset = 0;
     mCursor = 0;
@@ -103,7 +103,7 @@ nsBufferedStream::Init(nsISupports* stream, PRUint32 bufferSize)
 nsresult
 nsBufferedStream::Close()
 {
-    nsresult rv = NS_OK;
+    NS_IF_RELEASE(mStream);
     if (mBuffer) {
         delete[] mBuffer;
         mBuffer = nsnull;
@@ -111,7 +111,7 @@ nsBufferedStream::Close()
         mBufferStartOffset = 0;
         mCursor = 0;
     }
-    return rv;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -251,8 +251,13 @@ nsBufferedInputStream::Close()
 NS_IMETHODIMP
 nsBufferedInputStream::Available(PRUint32 *result)
 {
-    *result = mFillPoint - mCursor;
-    return NS_OK;
+    nsresult rv = NS_OK;
+    *result = 0;
+    if (mStream) {
+        rv = Source()->Available(result);
+    }
+    *result += (mFillPoint - mCursor);
+    return rv;
 }
 
 NS_IMETHODIMP
@@ -260,6 +265,7 @@ nsBufferedInputStream::Read(char * buf, PRUint32 count, PRUint32 *result)
 {
     nsresult rv;
     if (mBufferDisabled) {
+        NS_ENSURE_TRUE(mStream, NS_BASE_STREAM_CLOSED);
         rv = Source()->Read(buf, count, result);
         if (NS_SUCCEEDED(rv))
             mBufferStartOffset += *result;  // so nsBufferedStream::Tell works
@@ -312,24 +318,11 @@ nsBufferedInputStream::ReadSegments(nsWriteSegmentFun writer, void * closure, PR
 }
 
 NS_IMETHODIMP
-nsBufferedInputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsBufferedInputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
-    NS_NOTREACHED("GetNonBlocking");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsBufferedInputStream::GetObserver(nsIInputStreamObserver * *aObserver)
-{
-    NS_NOTREACHED("GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsBufferedInputStream::SetObserver(nsIInputStreamObserver * aObserver)
-{
-    NS_NOTREACHED("SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    if (mStream)
+        return Source()->IsNonBlocking(aNonBlocking);
+    return NS_ERROR_NOT_INITIALIZED;
 }
 
 NS_IMETHODIMP
@@ -337,6 +330,7 @@ nsBufferedInputStream::Fill()
 {
     if (mBufferDisabled)
         return NS_OK;
+    NS_ENSURE_TRUE(mStream, NS_BASE_STREAM_CLOSED);
 
     nsresult rv;
     PRInt32 rem = PRInt32(mFillPoint - mCursor);
@@ -555,33 +549,11 @@ nsBufferedOutputStream::WriteSegments(nsReadSegmentFun reader, void * closure, P
 }
 
 NS_IMETHODIMP
-nsBufferedOutputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsBufferedOutputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
     if (mStream)
-        return Sink()->GetNonBlocking(aNonBlocking);
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsBufferedOutputStream::SetNonBlocking(PRBool aNonBlocking)
-{
-    if (mStream)
-        return Sink()->SetNonBlocking(aNonBlocking);
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsBufferedOutputStream::GetObserver(nsIOutputStreamObserver * *aObserver)
-{
-    NS_NOTREACHED("GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsBufferedOutputStream::SetObserver(nsIOutputStreamObserver * aObserver)
-{
-    NS_NOTREACHED("SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
+        return Sink()->IsNonBlocking(aNonBlocking);
+    return NS_ERROR_NOT_INITIALIZED;
 }
 
 NS_IMETHODIMP_(char*)
