@@ -58,7 +58,7 @@
 #include "nsIPrefBranchInternal.h"
 #include "nsIPrefLocalizedString.h"
 #include "nsICategoryManager.h"
-#include "nsIURLParser.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIProxiedProtocolHandler.h"
 #include "nsIProxyInfo.h"
@@ -154,7 +154,7 @@ nsIOService::nsIOService()
     : mOffline(PR_FALSE),
       mOfflineForProfileChange(PR_FALSE)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 
     // Get the allocator ready
     if (!gBufferCache)
@@ -266,13 +266,6 @@ nsIOService::Init()
 
 nsIOService::~nsIOService()
 {
-    // mURLParsers is a voidarray; we must release ourselves
-    for (PRInt32 i = 0; i < mURLParsers.Count(); i++)
-    {
-        nsISupports *temp;
-        temp = NS_STATIC_CAST(nsISupports*, mURLParsers[i]);
-        NS_IF_RELEASE(temp);
-    }
 }   
 
 NS_METHOD
@@ -323,7 +316,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS3(nsIOService,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-NS_IMETHODIMP
+nsresult
 nsIOService::CacheProtocolHandler(const char *scheme, nsIProtocolHandler *handler)
 {
     for (unsigned int i=0; i<NS_N(gScheme); i++)
@@ -350,7 +343,7 @@ nsIOService::CacheProtocolHandler(const char *scheme, nsIProtocolHandler *handle
     return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP
+nsresult
 nsIOService::GetCachedProtocolHandler(const char *scheme, nsIProtocolHandler **result, PRUint32 start, PRUint32 end)
 {
     PRUint32 len = end - start - 1;
@@ -377,59 +370,11 @@ nsIOService::GetCachedProtocolHandler(const char *scheme, nsIProtocolHandler **r
     return NS_ERROR_FAILURE;
 }
  
-NS_IMETHODIMP
-nsIOService::CacheURLParser(const char *scheme, nsIURLParser *parser)
-{
-    NS_ENSURE_ARG_POINTER(scheme);
-    NS_ENSURE_ARG_POINTER(parser);
-    for (PRInt32 i=0; i< (PRInt32) NS_N(gScheme); i++)
-    {
-        if (!nsCRT::strcasecmp(scheme, gScheme[i]))
-        {
-            // we're going to store this in an nsVoidArray, which extends,
-            // unlike nsSupportsArray, which doesn't.  We must release
-            // them on delete!
-            // grab this before overwriting it
-            nsIURLParser *old_parser;
-            old_parser = NS_STATIC_CAST(nsIURLParser*,mURLParsers.SafeElementAt(i));
-
-            NS_ADDREF(parser);
-            mURLParsers.ReplaceElementAt(parser, i);
-            // release any old entry, if any, AFTER adding new entry in
-            // case they were the same (paranoia)
-            NS_IF_RELEASE(old_parser);
-            return NS_OK;
-        }
-    }
-    return NS_ERROR_FAILURE;
-}
-
-
-NS_IMETHODIMP
-nsIOService::GetCachedURLParser(const char *scheme, nsIURLParser **result)
-{
-    PRInt32 count = mURLParsers.Count();
-    for (PRInt32 i=0; i < count; i++)
-    {
-        if (!nsCRT::strcasecmp(scheme, gScheme[i]))
-        {
-            *result = NS_STATIC_CAST(nsIURLParser*, mURLParsers[i]);
-            NS_IF_ADDREF(*result);
-            return *result ? NS_OK : NS_ERROR_FAILURE;
-        }
-    }
-    return NS_ERROR_FAILURE;
-}
-
 NS_IMETHODIMP
 nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
 {
     nsresult rv;
 
-    NS_ASSERTION(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX_LENGTH
-                 == strlen(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX),
-                 "need to fix NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX_LENGTH");
-
     NS_ENSURE_ARG_POINTER(scheme);
     // XXX we may want to speed this up by introducing our own protocol 
     // scheme -> protocol handler mapping, avoiding the string manipulation
@@ -439,14 +384,17 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
     if (NS_SUCCEEDED(rv)) return NS_OK;
 
     PRBool externalProtocol = PR_FALSE;
+    PRBool listedProtocol   = PR_TRUE;
     nsCOMPtr<nsIPrefBranch> prefBranch;
     GetPrefBranch(getter_AddRefs(prefBranch));
     if (prefBranch) {
         nsCAutoString externalProtocolPref("network.protocol-handler.external.");
         externalProtocolPref += scheme;
         rv = prefBranch->GetBoolPref(externalProtocolPref.get(), &externalProtocol);
-        if (NS_FAILED(rv))
+        if (NS_FAILED(rv)) {
             externalProtocol = PR_FALSE;
+            listedProtocol   = PR_FALSE;
+        }
     }
 
     if (!externalProtocol) {
@@ -455,10 +403,14 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
         ToLowerCase(contractID);
 
         rv = CallGetService(contractID.get(), result);
+
+        // If the pref for this protocol was explicitly set to false,
+        // stop here and do not invoke the default handler.
+        if (NS_FAILED(rv) && listedProtocol)
+            return NS_ERROR_UNKNOWN_PROTOCOL;
     }
     
-    if (externalProtocol || NS_FAILED(rv)) 
-    {
+    if (externalProtocol || NS_FAILED(rv)) {
       // okay we don't have a protocol handler to handle this url type, so use the default protocol handler.
       // this will cause urls to get dispatched out to the OS ('cause we can't do anything with them) when 
       // we try to read from a channel created by the default protocol handler.
@@ -476,235 +428,7 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
 NS_IMETHODIMP
 nsIOService::ExtractScheme(const nsACString &inURI, nsACString &scheme)
 {
-    return ExtractURLScheme(inURI, nsnull, nsnull, &scheme);
-}
-
-/* nsIURLParser getParserForScheme (in string scheme); */
-NS_IMETHODIMP 
-nsIOService::GetParserForScheme(const char *scheme, nsIURLParser **_retval)
-{
-    nsresult rv;
-    
-    rv = GetCachedURLParser(scheme, _retval);
-    if (NS_SUCCEEDED(rv) && *_retval) return NS_OK;
-
-
-    if (!scheme) {
-        if (!mDefaultURLParser) {
-            rv = nsServiceManager::GetService(kStdURLParserCID, 
-                                              NS_GET_IID(nsIURLParser), 
-                                              getter_AddRefs(mDefaultURLParser));
-            if (NS_FAILED(rv)) return rv;
-        }
-        
-        *_retval = mDefaultURLParser;
-        NS_ADDREF(*_retval);
-        return NS_OK;
-    }
-
-    nsCOMPtr<nsICategoryManager> catmgr(do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsISimpleEnumerator> parserList;
-    rv = catmgr->EnumerateCategory(NS_IURLPARSER_KEY, getter_AddRefs(parserList));
-    if (NS_FAILED(rv)) return rv;
-    
-    nsCOMPtr<nsISupportsString> entry;
-
-    // Walk the list of parsers...
-    while (1) {
-        rv = parserList->GetNext(getter_AddRefs(entry));
-        if (NS_FAILED(rv)) break;
-
-        // get the entry string
-        nsXPIDLCString entryString;
-        rv = entry->GetData(getter_Copies(entryString));
-        if (NS_FAILED(rv)) break;
-
-        if (strcmp(entryString, scheme) == 0) {
-            nsXPIDLCString contractID;
-            rv = catmgr->GetCategoryEntry(NS_IURLPARSER_KEY,(const char *)entryString, getter_Copies(contractID));
-            if (NS_FAILED(rv)) break;
-
-            rv = nsServiceManager::GetService(contractID, NS_GET_IID(nsIURLParser), (nsISupports **)_retval);
-            if (NS_FAILED(rv))
-                return rv;
-            if (*_retval)
-                CacheURLParser(scheme, *_retval);
-            return *_retval ? NS_OK : NS_ERROR_FAILURE;
-        }
-    }
-
-    // no registered url parser. Just use the default...
-    if (!mDefaultURLParser) {
-        rv = nsServiceManager::GetService(kStdURLParserCID, 
-                                          NS_GET_IID(nsIURLParser), 
-                                          getter_AddRefs(mDefaultURLParser));
-        if (NS_FAILED(rv)) return rv;
-    }
-
-    *_retval = mDefaultURLParser;
-    NS_ADDREF(*_retval);
-    CacheURLParser(scheme, *_retval);
-    return NS_OK;
-}
-
-static inline void
-ExtractUrlPart_Helper(const nsACString &src, PRUint32 pos, PRInt32 len, nsACString &result)
-{
-    if (len >= 0)
-        result = Substring(src, pos, len);
-}
-
-NS_IMETHODIMP 
-nsIOService::ExtractUrlPart(const nsACString &urlString, PRInt16 flag, nsACString &urlPart)
-{
-    nsresult rv;
-    nsCAutoString scheme;
-
-    rv = ExtractURLScheme(urlString, nsnull, nsnull, &scheme);
-    if (NS_FAILED(rv)) return rv;
-
-    if (flag == url_Scheme) {
-        urlPart = scheme;
-        return NS_OK;
-    }
-
-    urlPart.Truncate();
-
-    nsCOMPtr<nsIURLParser> parser;
-    rv = GetParserForScheme(scheme.get(), getter_AddRefs(parser));
-    if (NS_FAILED(rv)) return rv;
-    
-    // we work only with flat strings around these parts
-    const nsPromiseFlatCString &flat = PromiseFlatCString(urlString);
-    const char *url = flat.get();
-
-    PRUint32 authPos, pathPos;
-    PRInt32 authLen, pathLen;
-
-    rv = parser->ParseURL(url, -1,
-                          nsnull, nsnull,
-                          &authPos, &authLen,
-                          &pathPos, &pathLen);
-    if (NS_FAILED(rv)) return rv;
-
-    if (flag == url_Path)
-        ExtractUrlPart_Helper(urlString, pathPos, pathLen, urlPart);
-    else if (flag < url_Directory || flag & url_Port) {
-        PRUint32 usernamePos, passwordPos, hostnamePos;
-        PRInt32 usernameLen, passwordLen, hostnameLen;
-        PRInt32 port;
-
-        if (authLen < 0)
-            return NS_OK;
-
-        rv = parser->ParseAuthority(url + authPos, authLen,
-                                    &usernamePos, &usernameLen,
-                                    &passwordPos, &passwordLen,
-                                    &hostnamePos, &hostnameLen,
-                                    &port);
-        if (NS_FAILED(rv)) return rv;
-
-        usernamePos += authPos;
-        passwordPos += authPos;
-        hostnamePos += authPos;
-
-        switch (flag) {
-        case url_Username:
-            ExtractUrlPart_Helper(urlString, usernamePos, usernameLen, urlPart);
-            break;
-        case url_Password:
-            ExtractUrlPart_Helper(urlString, passwordPos, passwordLen, urlPart);
-            break;
-        case url_Host:
-            ExtractUrlPart_Helper(urlString, hostnamePos, hostnameLen, urlPart);
-            break;
-        case url_Port:
-            if (port != -1) {
-                PRInt32 pos = hostnamePos + hostnameLen + 1;
-                ExtractUrlPart_Helper(urlString, pos, pathPos - pos, urlPart);
-            }
-            break;
-        case url_Host | url_Port:
-            if (port != -1 && hostnameLen > 0) {
-                urlPart = Substring(urlString, hostnamePos, hostnameLen)
-                        + NS_LITERAL_CSTRING(":")
-                        + nsPrintfCString("%d", port);
-            }
-            else
-                ExtractUrlPart_Helper(urlString, hostnamePos, hostnameLen, urlPart);
-            break;
-        default:
-            NS_NOTREACHED("unexpected flag");
-            return NS_ERROR_UNEXPECTED;
-        }
-    }
-    else {
-        PRUint32 filepathPos, paramPos, queryPos, refPos;
-        PRInt32 filepathLen, paramLen, queryLen, refLen;
-
-        if (pathLen < 0)
-            return NS_OK;
-
-        rv = parser->ParsePath(url + pathPos, pathLen,
-                               &filepathPos, &filepathLen,
-                               &paramPos, &paramLen,
-                               &queryPos, &queryLen,
-                               &refPos, &refLen);
-        if (NS_FAILED(rv)) return rv;
-
-        filepathPos += pathPos;
-        paramPos += pathPos;
-        queryPos += pathPos;
-        refPos += pathPos;
-
-        if (flag < url_Param) {
-            PRUint32 directoryPos, basenamePos, extensionPos;
-            PRInt32 directoryLen, basenameLen, extensionLen;
-
-            rv = parser->ParseFilePath(url + filepathPos, filepathLen,
-                                       &directoryPos, &directoryLen,
-                                       &basenamePos, &basenameLen,
-                                       &extensionPos, &extensionLen);
-
-            directoryPos += filepathPos;
-            basenamePos += filepathPos;
-            extensionPos += filepathPos;
-
-            switch (flag) {
-            case url_Directory:
-                ExtractUrlPart_Helper(urlString, directoryPos, directoryLen, urlPart);
-                break;
-            case url_FileBaseName:
-                ExtractUrlPart_Helper(urlString, basenamePos, basenameLen, urlPart);
-                break;
-            case url_FileExtension:
-                ExtractUrlPart_Helper(urlString, extensionPos, extensionLen, urlPart);
-                break;
-            default:
-                NS_NOTREACHED("unexpected flag");
-                return NS_ERROR_UNEXPECTED;
-            }
-        }
-        else {
-            switch (flag) {
-            case url_Param:
-                ExtractUrlPart_Helper(urlString, paramPos, paramLen, urlPart);
-                break;
-            case url_Query:
-                ExtractUrlPart_Helper(urlString, queryPos, queryLen, urlPart);
-                break;
-            case url_Ref:
-                ExtractUrlPart_Helper(urlString, refPos, refLen, urlPart);
-                break;
-            default:
-                NS_NOTREACHED("unexpected flag");
-                return NS_ERROR_UNEXPECTED;
-            }
-        }
-    }
-    return NS_OK;
+    return net_ExtractURLScheme(inURI, nsnull, nsnull, &scheme);
 }
 
 NS_IMETHODIMP 
@@ -718,7 +442,6 @@ nsIOService::GetProtocolFlags(const char* scheme, PRUint32 *flags)
     return rv;
 }
 
-
 nsresult
 nsIOService::NewURI(const nsACString &aSpec, const char *aCharset, nsIURI *aBaseURI, nsIURI **result)
 {
@@ -726,11 +449,7 @@ nsIOService::NewURI(const nsACString &aSpec, const char *aCharset, nsIURI *aBase
     nsCAutoString scheme;
 
     rv = ExtractScheme(aSpec, scheme);
-    if (NS_SUCCEEDED(rv)) {
-        // then aSpec is absolute... ignore aBaseURI in this case
-        aBaseURI = nsnull;
-    }
-    else {
+    if (NS_FAILED(rv)) {
         // then aSpec is relative
         if (!aBaseURI)
             return NS_ERROR_MALFORMED_URI;
@@ -904,77 +623,6 @@ nsIOService::AllowPort(PRInt32 inPort, const char *scheme, PRBool *_retval)
     *_retval = PR_TRUE;
     return NS_OK;
 }
-////////////////////////////////////////////////////////////////////////////////
-// URL parsing utilities
-
-NS_IMETHODIMP
-nsIOService::ResolveRelativePath(const nsACString &relativePath, const nsACString &basePath,
-                                 nsACString &result)
-{
-    nsCAutoString name;
-    nsCAutoString path(basePath);
-	PRBool needsDelim = PR_FALSE;
-
-	if ( !path.IsEmpty() ) {
-		PRUnichar last = path.Last();
-		needsDelim = !(last == '/' || last == '\\' );
-	}
-
-    nsACString::const_iterator beg, end;
-    relativePath.BeginReading(beg);
-    relativePath.EndReading(end);
-
-    PRBool stop = PR_FALSE;
-    char c;
-    for (; !stop; ++beg) {
-        c = (beg == end) ? '\0' : *beg;
-        //printf("%c [name=%s] [path=%s]\n", c, name.get(), path.get());
-        switch (c) {
-          case '\0':
-          case '#':
-          case ';':
-          case '?':
-            stop = PR_TRUE;
-            // fall through...
-          case '/':
-          case '\\':
-            // delimiter found
-            if (name.Equals("..")) {
-                // pop path
-                // If we already have the delim at end, then
-                //  skip over that when searching for next one to the left
-                PRInt32 offset = path.Length() - (needsDelim ? 1 : 2);
-                PRInt32 pos = path.RFind("/", PR_FALSE, offset);
-                if (pos > 0)
-                    path.Truncate(pos + 1);
-                else
-                    return NS_ERROR_MALFORMED_URI;
-            }
-            else if (name.Equals(".") || name.Equals("")) {
-                // do nothing
-            }
-            else {
-                // append name to path
-                if (needsDelim)
-                    path += "/";
-                path += name;
-                needsDelim = PR_TRUE;
-            }
-            name = "";
-            break;
-
-          default:
-            // append char to name
-            name += c;
-        }
-    }
-    // append anything left on relativePath (e.g. #..., ;..., ?...)
-    if (c != '\0')
-        path += Substring(--beg, end);
-
-    result = path;
-    return NS_OK;
-}
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -1082,71 +730,3 @@ nsIOService::Observe(nsISupports *subject,
     }
     return NS_OK;
 }
-
-nsresult
-nsIOService::ParseFileURL(const nsACString &inURL,
-                          nsACString &outDirectory,
-                          nsACString &outFileBaseName,
-                          nsACString &outFileExtension)
-{
-    nsresult rv;
-
-    outDirectory.Truncate();
-    outFileBaseName.Truncate();
-    outFileExtension.Truncate();
-
-	nsCAutoString scheme;
-    rv = ExtractScheme(inURL, scheme);
-    if (NS_FAILED(rv)) return rv;
-
-    if (strcmp(scheme.get(), "file") != 0) {
-        NS_ERROR("must be a file:// url");
-        return NS_ERROR_UNEXPECTED;
-    }
-
-    const nsPromiseFlatCString &flatURL = PromiseFlatCString(inURL);
-    const char *url = flatURL.get();
-    
-    nsCOMPtr<nsIURLParser> parser;
-    rv = GetParserForScheme(scheme.get(), getter_AddRefs(parser));
-    if (NS_FAILED(rv)) return rv;
-
-    PRUint32 pathPos, filepathPos, directoryPos, basenamePos, extensionPos;
-    PRInt32 pathLen, filepathLen, directoryLen, basenameLen, extensionLen;
-
-    // invoke the parser to extract the URL path
-    rv = parser->ParseURL(url, -1,
-                          nsnull, nsnull, // dont care about scheme
-                          nsnull, nsnull, // dont care about authority
-                          &pathPos, &pathLen);
-    if (NS_FAILED(rv)) return rv;
-
-    // invoke the parser to extract filepath from the path
-    rv = parser->ParsePath(url + pathPos, pathLen,
-                           &filepathPos, &filepathLen,
-                           nsnull, nsnull,  // dont care about param
-                           nsnull, nsnull,  // dont care about query
-                           nsnull, nsnull); // dont care about ref
-    if (NS_FAILED(rv)) return rv;
-
-    filepathPos += pathPos;
-
-    // invoke the parser to extract the directory and filename from filepath
-    rv = parser->ParseFilePath(url + filepathPos, filepathLen,
-                               &directoryPos, &directoryLen,
-                               &basenamePos, &basenameLen,
-                               &extensionPos, &extensionLen);
-    if (NS_FAILED(rv)) return rv;
-
-    if (directoryLen > 0)
-        outDirectory = Substring(inURL, filepathPos + directoryPos, directoryLen);
-    if (basenameLen > 0)
-        outFileBaseName = Substring(inURL, filepathPos + basenamePos, basenameLen);
-    if (extensionLen > 0)
-        outFileExtension = Substring(inURL, filepathPos + extensionPos, extensionLen);
-    // since we are using a no-auth url parser, there will never be a host
-    // XXX not strictly true... file://localhost/foo/bar.html is a valid URL
-
-    return NS_OK;
-}
- 
