@@ -24,9 +24,32 @@
 
 #include <limits.h>
 
+// include files for ftruncate (or equivalent)
+#if defined(XP_UNIX)
+#include <unistd.h>
+#elif defined(XP_MAC)
+#include <Files.h>
+#elif defined(XP_WIN)
+#include <windows.h>
+#elif defined(XP_OS2)
+#define INCL_DOSERRORS
+#include <os2.h>
+#else
+// XXX add necessary include file for ftruncate (or equivalent)
+#endif
+
+#if defined(XP_MAC)
+#include "pprio.h"
+#else
+#include "private/pprio.h"
+#endif
+
+
 #include "nsDiskCacheDevice.h"
 #include "nsDiskCacheEntry.h"
 #include "nsDiskCacheMap.h"
+#include "nsDiskCacheStreams.h"
+
 #include "nsDiskCache.h"
 
 #include "nsCacheService.h"
@@ -39,6 +62,8 @@
 #include "nsReadableUtils.h"
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
+#include "nsAutoLock.h"
+#include "nsCRT.h"
 
 
 static const char DISK_CACHE_DEVICE_ID[] = { "disk" };
@@ -114,9 +139,9 @@ nsDiskCacheEvictor::VisitRecord(nsDiskCacheRecord *  mapRecord)
         // we are currently using this entry, so all we can do is doom it
         
         // since we're enumerating the records, we don't want to call DeleteRecord
-        // when nsCacheService::GlobalInstance()->DoomEntry_Locked() calls us back.
+        // when nsCacheService::DoomEntry() calls us back.
         binding->mDoomed = PR_TRUE;         // mark binding record as 'deleted'
-        nsCacheService::GlobalInstance()->DoomEntry_Locked(binding->mCacheEntry);
+        nsCacheService::DoomEntry(binding->mCacheEntry);
         result = kDeleteRecordAndContinue;  // this will REALLY delete the record
 
     } else {
@@ -178,11 +203,11 @@ NS_IMETHODIMP nsDiskCacheDeviceInfo::GetUsageReport(char ** usageReport)
 
     buffer.Append("<tr><td><b>Cache Directory:</b></td><td><tt> ");
     nsCOMPtr<nsILocalFile> cacheDir;
-    char *                 path;
+    nsAutoString           path;
     mDevice->getCacheDirectory(getter_AddRefs(cacheDir)); 
-    nsresult rv = cacheDir->GetPath(&path);
+    nsresult rv = cacheDir->GetPath(path);
     if (NS_SUCCEEDED(rv)) {
-        buffer.Append(path);
+        buffer.Append(NS_ConvertUCS2toUTF8(path));
     } else {
         buffer.Append("directory unavailable");
     }
@@ -246,6 +271,43 @@ nsDiskCache::Hash(const char * key)
 }
 
 
+nsresult
+nsDiskCache::Truncate(PRFileDesc *  fd, PRUint32  newEOF)
+{
+    // use modified SetEOF from nsFileStreams::SetEOF()
+
+#if defined(XP_UNIX)
+    if (ftruncate(PR_FileDesc2NativeHandle(fd), newEOF) != 0) {
+        NS_ERROR("ftruncate failed");
+        return NS_ERROR_FAILURE;
+    }
+
+#elif defined(XP_MAC)
+    if (::SetEOF(PR_FileDesc2NativeHandle(fd), newEOF) != 0) {
+        NS_ERROR("SetEOF failed");
+        return NS_ERROR_FAILURE;
+    }
+
+#elif defined(XP_WIN)
+    PRInt32 cnt = PR_Seek(fd, newEOF, PR_SEEK_SET);
+    if (cnt == -1)  return NS_ERROR_FAILURE;
+    if (!SetEndOfFile((HANDLE) PR_FileDesc2NativeHandle(fd))) {
+        NS_ERROR("SetEndOfFile failed");
+        return NS_ERROR_FAILURE;
+    }
+
+#elif defined(XP_OS2)
+    if (DosSetFileSize((HFILE) PR_FileDesc2NativeHandle(fd), newEOF) != NO_ERROR) {
+        NS_ERROR("DosSetFileSize failed");
+        return NS_ERROR_FAILURE;
+    }
+#else
+    // add implementations for other platforms here
+#endif
+    return NS_OK;
+}
+
+
 /******************************************************************************
  *  nsDiskCacheDevice
  *****************************************************************************/
@@ -257,7 +319,9 @@ static nsCOMPtr<nsIFileTransportService> gFileTransportService;
 #endif
 
 nsDiskCacheDevice::nsDiskCacheDevice()
-    :   mInitialized(PR_FALSE), mCacheCapacity(0), mCacheMap(nsnull)
+    : mCacheCapacity(0)
+    , mCacheMap(nsnull)
+    , mInitialized(PR_FALSE)
 {
 }
 
@@ -276,8 +340,9 @@ nsDiskCacheDevice::Init()
 {
     nsresult rv;
 
-    NS_ENSURE_TRUE(!mInitialized, NS_ERROR_FAILURE);
-    
+    NS_ENSURE_TRUE(!Initialized(), NS_ERROR_FAILURE);
+       
+    if (!mCacheDirectory) return NS_ERROR_FAILURE;
     rv = mBindery.Init();
     if (NS_FAILED(rv)) return rv;
     
@@ -285,7 +350,7 @@ nsDiskCacheDevice::Init()
     gFileTransportService = do_GetService("@mozilla.org/network/file-transport-service;1", &rv);
     if (NS_FAILED(rv)) return rv;
 
-    // XXX are we sure we want to do this on startup?
+    // XXX we should spawn another thread to do this after startup
     // delete "Cache.Trash" folder
     nsCOMPtr<nsIFile> cacheTrashDir;
     rv = GetCacheTrashDirectory(getter_AddRefs(cacheTrashDir));
@@ -305,7 +370,7 @@ nsDiskCacheDevice::Init()
         if (NS_FAILED(rv))  goto error_exit;
     }
     
-    mInitialized = PR_TRUE;                     // record that initialization succeeded.
+    mInitialized = PR_TRUE;
     return NS_OK;
 
 error_exit:
@@ -321,12 +386,15 @@ error_exit:
 }
 
 
+/**
+ *  NOTE: called while holding the cache service lock
+ */
 nsresult
 nsDiskCacheDevice::Shutdown()
 {
-    if (mInitialized) {
+    if (Initialized()) {
         // check cache limits in case we need to evict.
-        EvictDiskCacheEntries();
+        EvictDiskCacheEntries((PRInt32)mCacheCapacity);
 
         // write out persistent information about the cache.
         (void) mCacheMap->Close();
@@ -335,7 +403,6 @@ nsDiskCacheDevice::Shutdown()
 
         mBindery.Reset();
 
-        // no longer initialized.
         mInitialized = PR_FALSE;
     }
 
@@ -375,6 +442,8 @@ nsDiskCacheDevice::GetDeviceID()
  *      cases:  key not in disk cache, hash number free
  *              key not in disk cache, hash number used
  *              key in disk cache
+ *
+ *  NOTE: called while holding the cache service lock
  */
 nsCacheEntry *
 nsDiskCacheDevice::FindEntry(nsCString * key)
@@ -418,6 +487,9 @@ nsDiskCacheDevice::FindEntry(nsCString * key)
 }
 
 
+/**
+ *  NOTE: called while holding the cache service lock
+ */
 nsresult
 nsDiskCacheDevice::DeactivateEntry(nsCacheEntry * entry)
 {
@@ -458,6 +530,8 @@ nsDiskCacheDevice::DeactivateEntry(nsCacheEntry * entry)
  *              walk matching hashnumber list to find lowest generation number
  *              take generation number from other (data/meta) location,
  *                  or walk active list
+ *
+ *  NOTE: called while holding the cache service lock
  */
 nsresult
 nsDiskCacheDevice::BindEntry(nsCacheEntry * entry)
@@ -483,7 +557,7 @@ nsDiskCacheDevice::BindEntry(nsCacheEntry * entry)
 
                 if (!oldBinding->mCacheEntry->IsDoomed()) {
                 // we've got a live one!
-                    nsCacheService::GlobalInstance()->DoomEntry_Locked(oldBinding->mCacheEntry);
+                    nsCacheService::DoomEntry(oldBinding->mCacheEntry);
                     // storage will be delete when oldBinding->mCacheEntry is Deactivated
                 }
             } else {
@@ -505,6 +579,9 @@ nsDiskCacheDevice::BindEntry(nsCacheEntry * entry)
 }
 
 
+/**
+ *  NOTE: called while holding the cache service lock
+ */
 void
 nsDiskCacheDevice::DoomEntry(nsCacheEntry * entry)
 {
@@ -521,10 +598,13 @@ nsDiskCacheDevice::DoomEntry(nsCacheEntry * entry)
 }
 
 
+/**
+ *  NOTE: called while holding the cache service lock
+ */
 nsresult
-nsDiskCacheDevice::GetTransportForEntry(nsCacheEntry * entry,
-                                        nsCacheAccessMode mode, 
-                                        nsITransport ** result)
+nsDiskCacheDevice::GetTransportForEntry(nsCacheEntry *      entry,
+                                        nsCacheAccessMode   mode, 
+                                        nsITransport **     result)
 {
     NS_ENSURE_ARG_POINTER(entry);
     NS_ENSURE_ARG_POINTER(result);
@@ -536,66 +616,40 @@ nsDiskCacheDevice::GetTransportForEntry(nsCacheEntry * entry,
     
     NS_ASSERTION(binding->mCacheEntry == entry, "binding & entry don't point to each other");
 
-#ifdef MOZ_NEW_CACHE_REUSE_TRANSPORTS
-    nsCOMPtr<nsITransport>& transport = binding->getTransport(mode);
-    if (transport) {
-        NS_ADDREF(*result = transport);
-        return NS_OK;
+    if (!binding->mStreamIO) {
+        binding->mStreamIO = new nsDiskCacheStreamIO(binding);
+        if (!binding->mStreamIO)  return NS_ERROR_OUT_OF_MEMORY;
     }
-#endif
-
-    // check/set binding->mRecord for separate file, sync w/mCacheMap
-    if (binding->mRecord.DataLocationInitialized()) {
-        NS_ASSERTION(binding->mRecord.DataFile() == 0, "error: cache block file"); // make sure it's a separate file
-        NS_ASSERTION(binding->mRecord.DataFileGeneration() == binding->mGeneration, "error generations out of sync");
-    } else {
-        binding->mRecord.SetDataFileGeneration(binding->mGeneration);
-        binding->mRecord.SetDataFileSize(0);    // 1k minimum
-        if (!binding->mDoomed) {
-            // record stored in cache map, so update it
-            rv = mCacheMap->UpdateRecord(&binding->mRecord);
-            if (NS_FAILED(rv))  return rv;
-        }
-    }
-
-    // generate the name of the cache entry from the hash code of its key,
-    // modulo the number of files we're willing to keep cached.
-    nsCOMPtr<nsIFile> file;
-    rv = mCacheMap->GetFileForDiskCacheRecord(&binding->mRecord,
-                                              nsDiskCache::kData,
-                                              getter_AddRefs(file));
-    if (NS_FAILED(rv))  return rv;
-    
-    PRInt32 ioFlags = 0;
-    switch (mode) {
-    case nsICache::ACCESS_READ:
-        ioFlags = PR_RDONLY;
-        break;
-    case nsICache::ACCESS_WRITE:
-        ioFlags = PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE;
-        break;
-    case nsICache::ACCESS_READ_WRITE:
-        ioFlags = PR_RDWR | PR_CREATE_FILE;
-        break;
-    }
-
-    rv = gFileTransportService->CreateTransport(file, ioFlags, PR_IRUSR | PR_IWUSR, result);
-    return rv;
+    // XXX assumption: CreateTransportFromStreamIO() is light-weight
+    // PR_FALSE = keep streamIO open for lifetime of transport
+    rv = gFileTransportService->CreateTransportFromStreamIO(binding->mStreamIO, PR_FALSE, result);
+    return rv;    
 }
 
 
+/**
+ *  NOTE: called while holding the cache service lock
+ */
 nsresult
 nsDiskCacheDevice::GetFileForEntry(nsCacheEntry *    entry,
                                    nsIFile **        result)
 {
+    NS_ENSURE_ARG_POINTER(result);
+    *result = nsnull;
+
     nsresult             rv;
+        
     nsDiskCacheBinding * binding = GetCacheEntryBinding(entry);
-    NS_ASSERTION(binding, "GetFileForEntry: binding == nsnull");
-    if (!binding)  return NS_ERROR_UNEXPECTED;
+    if (!binding) {
+        NS_WARNING("GetFileForEntry: binding == nsnull");
+        return NS_ERROR_UNEXPECTED;
+    }
     
     // check/set binding->mRecord for separate file, sync w/mCacheMap
     if (binding->mRecord.DataLocationInitialized()) {
-        NS_ASSERTION(binding->mRecord.DataFile() == 0, "error: cache block file"); // make sure it's a separate file
+        if (binding->mRecord.DataFile() != 0)
+            return NS_ERROR_NOT_AVAILABLE;  // data not stored as separate file
+
         NS_ASSERTION(binding->mRecord.DataFileGeneration() == binding->mGeneration, "error generations out of sync");
     } else {
         binding->mRecord.SetDataFileGeneration(binding->mGeneration);
@@ -619,7 +673,9 @@ nsDiskCacheDevice::GetFileForEntry(nsCacheEntry *    entry,
 
 
 /**
- * This routine will get called every time an open descriptor is written to.
+ *  This routine will get called every time an open descriptor is written to.
+ *
+ *  NOTE: called while holding the cache service lock
  */
 nsresult
 nsDiskCacheDevice::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
@@ -630,17 +686,22 @@ nsDiskCacheDevice::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
 
     NS_ASSERTION(binding->mRecord.ValidRecord(), "bad record");
 
-    PRUint32  sizeK = ((entry->DataSize() + 0x0399) >> 10); // round up to next 1k
-    PRUint32  newSizeK =  ((entry->DataSize() + deltaSize + 0x399) >> 10);
+    PRUint32  newSize = entry->DataSize() + deltaSize;
+    if (newSize > mCacheCapacity) {
+        nsresult rv = nsCacheService::DoomEntry(entry);
+        NS_ASSERTION(NS_SUCCEEDED(rv),"DoomEntry() failed.");
+        return rv;
+    }
 
-    NS_ASSERTION((sizeK < USHRT_MAX) && (sizeK == binding->mRecord.DataFileSize()),
-                 "data size out of sync");
+    PRUint32  sizeK = ((entry->DataSize() + 0x03FF) >> 10); // round up to next 1k
+    PRUint32  newSizeK =  ((newSize + 0x3FF) >> 10);
 
-    mCacheMap->IncrementTotalSize((newSizeK - sizeK) * 1024);
-    newSizeK = (newSizeK < USHRT_MAX) ? newSizeK : USHRT_MAX;   // record file size ceiling
-    binding->mRecord.SetDataFileSize(newSizeK);     // update binding->mRecord
+    NS_ASSERTION(sizeK < USHRT_MAX, "data size out of range");
+    NS_ASSERTION(newSizeK < USHRT_MAX, "data size out of range");
 
-    EvictDiskCacheEntries();
+    // pre-evict entries to make space for new data
+    PRInt32  targetCapacity = (PRInt32)(mCacheCapacity - ((newSizeK - sizeK) * 1024));
+    EvictDiskCacheEntries(targetCapacity);
     
     return NS_OK;
 }
@@ -719,6 +780,9 @@ nsDiskCacheDevice::EvictEntries(const char * clientID)
 {
     nsDiskCacheEvictor  evictor(this, mCacheMap, &mBindery, 0, clientID);
     nsresult       rv = mCacheMap->VisitRecords(&evictor);
+    
+    if (clientID == nsnull)     // we tried to clear the entire cache
+        rv = mCacheMap->Trim(); // so trim cache block files (if possible)
     return rv;
 }
 
@@ -757,7 +821,7 @@ nsDiskCacheDevice::InitializeCacheDirectory()
         }
         
         // create a directory with unique name to contain existing cache directory
-        rv = cacheTrashDir->Append("Cache");
+        rv = cacheTrashDir->AppendNative(NS_LITERAL_CSTRING("Cache"));
         if (NS_FAILED(rv))  return rv;
         rv = cacheTrashDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0777); 
         if (NS_FAILED(rv))  return rv;
@@ -766,7 +830,7 @@ nsDiskCacheDevice::InitializeCacheDirectory()
         nsCOMPtr<nsIFile> existingCacheDir;
         rv = mCacheDirectory->Clone(getter_AddRefs(existingCacheDir));
         if (NS_FAILED(rv))  return rv;
-        rv = existingCacheDir->MoveTo(cacheTrashDir, nsnull);
+        rv = existingCacheDir->MoveToNative(cacheTrashDir, nsCString());
         if (NS_FAILED(rv))  return rv;
     }
     
@@ -785,7 +849,7 @@ nsDiskCacheDevice::GetCacheTrashDirectory(nsIFile ** result)
     nsCOMPtr<nsIFile> cacheTrashDir;
     nsresult rv = mCacheDirectory->Clone(getter_AddRefs(cacheTrashDir));
     if (NS_FAILED(rv))  return rv;
-    rv = cacheTrashDir->SetLeafName("Cache.Trash");
+    rv = cacheTrashDir->SetNativeLeafName(NS_LITERAL_CSTRING("Cache.Trash"));
     if (NS_FAILED(rv))  return rv;
     
     *result = cacheTrashDir.get();
@@ -795,13 +859,13 @@ nsDiskCacheDevice::GetCacheTrashDirectory(nsIFile ** result)
 
 
 nsresult
-nsDiskCacheDevice::EvictDiskCacheEntries()
+nsDiskCacheDevice::EvictDiskCacheEntries(PRInt32  targetCapacity)
 {
     nsresult rv;
     
-    if (mCacheMap->TotalSize() < (PRInt32) mCacheCapacity)  return NS_OK;
+    if (mCacheMap->TotalSize() < targetCapacity)  return NS_OK;
 
-    nsDiskCacheEvictor  evictor(this, mCacheMap, &mBindery, mCacheCapacity, nsnull);
+    nsDiskCacheEvictor  evictor(this, mCacheMap, &mBindery, targetCapacity, nsnull);
     rv = mCacheMap->EvictRecords(&evictor);
     
     return rv;
@@ -822,7 +886,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     nsresult rv;
     PRBool  exists;
 
-    if (mInitialized) {
+    if (Initialized()) {
         NS_ASSERTION(PR_FALSE, "Cannot switch cache directory when initialized");
         return;
     }
@@ -843,7 +907,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     
     rv = parentDir->Clone(getter_AddRefs(directory));
     if (NS_FAILED(rv))  return;
-    rv = directory->Append("Cache");
+    rv = directory->AppendNative(NS_LITERAL_CSTRING("Cache"));
     if (NS_FAILED(rv))  return;
     
     rv = directory->Exists(&exists);
@@ -856,7 +920,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     // clean up Cache.Trash directories
     rv = parentDir->Clone(getter_AddRefs(directory));
     if (NS_FAILED(rv))  return;    
-    rv = directory->Append("Cache.Trash");
+    rv = directory->AppendNative(NS_LITERAL_CSTRING("Cache.Trash"));
     if (NS_FAILED(rv))  return;
     
     rv = directory->Exists(&exists);
@@ -866,7 +930,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     // clean up obsolete NewCache directory
     rv = parentDir->Clone(getter_AddRefs(directory));
     if (NS_FAILED(rv))  return;    
-    rv = directory->Append("NewCache");
+    rv = directory->AppendNative(NS_LITERAL_CSTRING("NewCache"));
     if (NS_FAILED(rv))  return;
     
     rv = directory->Exists(&exists);
@@ -883,13 +947,16 @@ nsDiskCacheDevice::getCacheDirectory(nsILocalFile ** result)
 }
 
 
+/**
+ *  NOTE: called while holding the cache service lock
+ */
 void
 nsDiskCacheDevice::SetCapacity(PRUint32  capacity)
 {
     mCacheCapacity = capacity * 1024;
-    if (mInitialized) {
+    if (Initialized()) {
         // start evicting entries if the new size is smaller!
-        EvictDiskCacheEntries();
+        EvictDiskCacheEntries((PRInt32)mCacheCapacity);
     }
 }
 
