@@ -24,24 +24,26 @@
 
 #include <stdlib.h>
 #include "nsHttpResponseHead.h"
+#include "nsPrintfCString.h"
 #include "prprf.h"
 #include "prtime.h"
+#include "nsCRT.h"
 
 //-----------------------------------------------------------------------------
 // nsHttpResponseHead <public>
 //-----------------------------------------------------------------------------
 
 nsresult
-nsHttpResponseHead::SetHeader(nsHttpAtom hdr, const char *val)
+nsHttpResponseHead::SetHeader(nsHttpAtom hdr, const nsACString &val)
 {
     nsresult rv = mHeaders.SetHeader(hdr, val);
     if (NS_FAILED(rv)) return rv;
 
     // response to changes in these headers
     if (hdr == nsHttp::Cache_Control)
-        ParseCacheControl(val);
+        ParseCacheControl(PromiseFlatCString(val).get());
     else if (hdr == nsHttp::Pragma)
-        ParsePragma(val);
+        ParsePragma(PromiseFlatCString(val).get());
 
     return NS_OK;
 }
@@ -51,12 +53,9 @@ nsHttpResponseHead::SetContentLength(PRInt32 len)
 {
     mContentLength = len;
     if (len < 0)
-        SetHeader(nsHttp::Content_Length, nsnull);
-    else {
-        nsCAutoString buf;
-        buf.AppendInt(len);
-        SetHeader(nsHttp::Content_Length, buf.get());
-    }
+        SetHeader(nsHttp::Content_Length, NS_LITERAL_CSTRING(""));
+    else
+        SetHeader(nsHttp::Content_Length, nsPrintfCString("%d", len));
 }
 
 void
@@ -80,7 +79,7 @@ nsHttpResponseHead::Flatten(nsACString &buf, PRBool pruneTransients)
     buf.Append("\r\n");
 
     if (!pruneTransients) {
-        mHeaders.Flatten(buf);
+        mHeaders.Flatten(buf, PR_FALSE);
         return;
     }
 
@@ -158,7 +157,7 @@ nsHttpResponseHead::ParseStatusLine(char *line)
     
     if ((mVersion == NS_HTTP_VERSION_0_9) || !(line = PL_strchr(line, ' '))) {
         mStatus = 200;
-        mStatusText = nsCRT::strdup("OK");
+        mStatusText = NS_LITERAL_CSTRING("OK");
     }
     else {
         // Status-Code
@@ -171,14 +170,14 @@ nsHttpResponseHead::ParseStatusLine(char *line)
         // Reason-Phrase is whatever is remaining of the line
         if (!(line = PL_strchr(line, ' '))) {
             LOG(("mal-formed response status line; assuming statusText = 'OK'\n"));
-            mStatusText = nsCRT::strdup("OK");
+            mStatusText = NS_LITERAL_CSTRING("OK");
         }
         else
-            mStatusText = nsCRT::strdup(++line);
+            mStatusText = ++line;
     }
 
     LOG(("Have status line [version=%u status=%u statusText=%s]\n",
-        PRUintn(mVersion), PRUintn(mStatus), mStatusText));
+        PRUintn(mVersion), PRUintn(mStatus), mStatusText.get()));
 }
 
 void
@@ -257,8 +256,6 @@ nsHttpResponseHead::ComputeCurrentAge(PRUint32 now,
 nsresult
 nsHttpResponseHead::ComputeFreshnessLifetime(PRUint32 *result)
 {
-    PRUint32 date, date2;
-
     *result = 0;
 
     // Try HTTP/1.1 style max-age directive...
@@ -267,22 +264,24 @@ nsHttpResponseHead::ComputeFreshnessLifetime(PRUint32 *result)
 
     *result = 0;
 
-    if (NS_SUCCEEDED(GetDateValue(&date))) {
-        // Try HTTP/1.0 style expires header...
-        if (NS_SUCCEEDED(GetExpiresValue(&date2))) {
-            if (date2 > date)
-                *result = date2 - date;
-            // the Expires header can specify a date in the past.
-            return NS_OK;
-        }
+    PRUint32 date = 0, date2 = 0;
+    if (NS_FAILED(GetDateValue(&date)))
+        date = NowInSeconds(); // synthesize a date header if none exists
 
-        // Fallback on heuristic using last modified header...
-        if (NS_SUCCEEDED(GetLastModifiedValue(&date2))) {
-            LOG(("using last-modified to determine freshness-lifetime\n"));
-            LOG(("last-modified = %u, date = %u\n", date2, date));
-            *result = (date - date2) / 10;
-            return NS_OK;
-        }
+    // Try HTTP/1.0 style expires header...
+    if (NS_SUCCEEDED(GetExpiresValue(&date2))) {
+        if (date2 > date)
+            *result = date2 - date;
+        // the Expires header can specify a date in the past.
+        return NS_OK;
+    }
+    
+    // Fallback on heuristic using last modified header...
+    if (NS_SUCCEEDED(GetLastModifiedValue(&date2))) {
+        LOG(("using last-modified to determine freshness-lifetime\n"));
+        LOG(("last-modified = %u, date = %u\n", date2, date));
+        *result = (date - date2) / 10;
+        return NS_OK;
     }
 
     // These responses can be cached indefinitely.
@@ -365,6 +364,17 @@ nsHttpResponseHead::MustValidateIfExpired()
     return val && PL_strcasestr(val, "must-revalidate");
 }
 
+PRBool
+nsHttpResponseHead::IsResumable()
+{
+    // even though some HTTP/1.0 servers may support byte range requests, we're not
+    // going to bother with them, since those servers wouldn't understand If-Range.
+    return mVersion >= NS_HTTP_VERSION_1_1 &&
+           PeekHeader(nsHttp::Content_Length) && 
+          (PeekHeader(nsHttp::ETag) || PeekHeader(nsHttp::Last_Modified)) &&
+           PL_strcasestr(PeekHeader(nsHttp::Accept_Ranges), "bytes");
+}
+
 PRBool
 nsHttpResponseHead::ExpiresInPast()
 {
@@ -418,10 +428,10 @@ nsHttpResponseHead::UpdateHeaders(nsHttpHeaderArray &headers)
             LOG(("new response header [%s: %s]\n", header.get(), val));
 
             // delete the current header value (if any)
-            mHeaders.SetHeader(header, nsnull);
+            mHeaders.SetHeader(header, NS_LITERAL_CSTRING(""));
 
             // copy the new header value...
-            mHeaders.SetHeader(header, val);
+            mHeaders.SetHeader(header, nsDependentCString(val));
         }
     }
 
@@ -441,9 +451,9 @@ nsHttpResponseHead::Reset()
     mCacheControlNoStore = PR_FALSE;
     mCacheControlNoCache = PR_FALSE;
     mPragmaNoCache = PR_FALSE;
-    CRTFREEIF(mStatusText);
-    CRTFREEIF(mContentType);
-    CRTFREEIF(mContentCharset);
+    mStatusText.Truncate();
+    mContentType.Truncate();
+    mContentCharset.Truncate();
 }
 
 nsresult
@@ -490,6 +500,30 @@ nsHttpResponseHead::GetMaxAgeValue(PRUint32 *result)
     return NS_OK;
 }
 
+nsresult
+nsHttpResponseHead::GetExpiresValue(PRUint32 *result)
+{
+    const char *val = PeekHeader(nsHttp::Expires);
+    if (!val)
+        return NS_ERROR_NOT_AVAILABLE;
+
+    PRTime time;
+    PRStatus st = PR_ParseTimeString(val, PR_TRUE, &time);
+    if (st != PR_SUCCESS) {
+        // parsing failed... maybe this is an "Expires: 0"
+        nsCAutoString buf(val);
+        buf.StripWhitespace();
+        if (buf.Length() == 1 && buf[0] == '0') {
+            *result = 0;
+            return NS_OK;
+        }
+        return NS_ERROR_NOT_AVAILABLE;
+    }
+
+    *result = PRTimeToSeconds(time); 
+    return NS_OK;
+}
+
 //-----------------------------------------------------------------------------
 // nsHttpResponseHead <private>
 //-----------------------------------------------------------------------------
@@ -549,26 +583,26 @@ nsHttpResponseHead::ParseContentType(char *type)
     if (!*type)
         return;
 
-    // a response could have multiple content type headers... we'll honor
-    // the last one.
-    CRTFREEIF(mContentCharset);
-    CRTFREEIF(mContentType);
+    // a response could have multiple content type headers... 
+    // we'll honor the last one. But for charset, we will only 
+    // honor the last one that comes with charset. 
+    mContentType.Truncate();
 
     // we don't care about comments (although they are invalid here)
-    char *p = PL_strchr(type, '(');
+    char *p = (char *) strchr(type, '(');
     if (p)
         *p = 0;
 
     // check if the content-type has additional fields...
-    if ((p = PL_strchr(type, ';')) != nsnull) {
+    if ((p = (char *) strchr(type, ';')) != nsnull) {
         char *p2, *p3;
         // is there a charset field?
         if ((p2 = PL_strcasestr(p, "charset=")) != nsnull) {
             p2 += 8;
 
             // check end of charset parameter
-            if ((p3 = PL_strchr(p2, ';')) == nsnull)
-                p3 = p2 + PL_strlen(p2);
+            if ((p3 = (char *) strchr(p2, ';')) == nsnull)
+                p3 = p2 + strlen(p2);
 
             // trim any trailing whitespace
             do {
@@ -576,11 +610,11 @@ nsHttpResponseHead::ParseContentType(char *type)
             } while ((*p3 == ' ') || (*p3 == '\t'));
             *++p3 = 0; // overwrite first char after the charset field
 
-            mContentCharset = nsCRT::strdup(p2);
+            mContentCharset = p2;
         }
     }
     else
-        p = type + PL_strlen(type);
+        p = type + strlen(type);
 
     // trim any trailing whitespace
     while (--p >= type && ((*p == ' ') || (*p == '\t')))
@@ -591,7 +625,9 @@ nsHttpResponseHead::ParseContentType(char *type)
     while (--p >= type)
         *p = nsCRT::ToLower(*p);
 
-    mContentType = nsCRT::strdup(type);
+    // If the server sent "*/*", it is meaningless, so do not store it.
+    if (PL_strcmp(type, "*/*"))
+        mContentType = type;
 }
 
 void
@@ -623,6 +659,8 @@ nsHttpResponseHead::ParseCacheControl(const char *val)
 void
 nsHttpResponseHead::ParsePragma(const char *val)
 {
+    LOG(("nsHttpResponseHead::ParsePragma [val=%s]\n", val));
+
     if (!val) {
         // clear no-cache flag
         mPragmaNoCache = PR_FALSE;
@@ -632,6 +670,6 @@ nsHttpResponseHead::ParsePragma(const char *val)
     // Although 'Pragma:no-cache' is not a standard HTTP response header (it's
     // a request header), caching is inhibited when this header is present so
     // as to match existing Navigator behavior.
-    if (*val && !PL_strcasestr(val, "no-cache"))
+    if (*val && PL_strcasestr(val, "no-cache"))
         mPragmaNoCache = PR_TRUE;
 }
