@@ -34,6 +34,9 @@
 #include "pratom.h"
 #include "plevent.h"
 
+// mLineBuf is limited to this number of bytes.
+#define MAX_LINEBUF_LENGTH (1024 * 10)
+
 //-----------------------------------------------------------------------------
 // helpers
 //-----------------------------------------------------------------------------
@@ -85,6 +88,8 @@ nsHttpTransaction::nsHttpTransaction(nsIStreamListener *listener,
     : mListener(listener)
     , mCallbacks(callbacks)
     , mConnection(nsnull)
+    , mReqUploadStreamOffset(0)
+    , mReqUploadStreamLength(0)
     , mResponseHead(nsnull)
     , mContentLength(-1)
     , mContentRead(0)
@@ -95,6 +100,7 @@ nsHttpTransaction::nsHttpTransaction(nsIStreamListener *listener,
     , mCapabilities(caps)
     , mHaveStatusLine(PR_FALSE)
     , mHaveAllHeaders(PR_FALSE)
+    , mResponseIsComplete(PR_FALSE)
     , mFiredOnStart(PR_FALSE)
     , mNoContent(PR_FALSE)
     , mPrematureEOF(PR_FALSE)
@@ -119,7 +125,8 @@ nsHttpTransaction::~nsHttpTransaction()
 nsresult
 nsHttpTransaction::SetupRequest(nsHttpRequestHead *requestHead,
                                 nsIInputStream *requestBody,
-                                PRBool requestBodyHasHeaders)
+                                PRBool requestBodyHasHeaders,
+                                PRBool pruneProxyHeaders)
 {
     nsresult rv;
 
@@ -134,17 +141,19 @@ nsHttpTransaction::SetupRequest(nsHttpRequestHead *requestHead,
         eqs->ResolveEventQueue(NS_CURRENT_EVENTQ, getter_AddRefs(mConsumerEventQ));
 
     // build a proxy for the progress event sink
-    if (mCallbacks && mConsumerEventQ) {
-        nsCOMPtr<nsIProgressEventSink> temp = do_GetInterface(mCallbacks);
-        if (temp) {
-            nsCOMPtr<nsIProxyObjectManager> mgr;
-            nsHttpHandler::get()->GetProxyObjectManager(getter_AddRefs(mgr));
-            if (mgr)
-                mgr->GetProxyForObject(mConsumerEventQ,
-                                       NS_GET_IID(nsIProgressEventSink),
-                                       temp,
-                                       PROXY_ASYNC | PROXY_ALWAYS,
-                                       getter_AddRefs(mProgressSink));
+    if (!(mCapabilities & NS_HTTP_DONT_REPORT_PROGRESS)) {
+        if (mCallbacks && mConsumerEventQ) {
+            nsCOMPtr<nsIProgressEventSink> temp = do_GetInterface(mCallbacks);
+            if (temp) {
+                nsCOMPtr<nsIProxyObjectManager> mgr;
+                nsHttpHandler::get()->GetProxyObjectManager(getter_AddRefs(mgr));
+                if (mgr)
+                    mgr->GetProxyForObject(mConsumerEventQ,
+                                           NS_GET_IID(nsIProgressEventSink),
+                                           temp,
+                                           PROXY_ASYNC | PROXY_ALWAYS,
+                                           getter_AddRefs(mProgressSink));
+            }
         }
     }
 
@@ -155,7 +164,7 @@ nsHttpTransaction::SetupRequest(nsHttpRequestHead *requestHead,
     mRequestHead = requestHead;
 
     mReqHeaderBuf.SetLength(0);
-    requestHead->Flatten(mReqHeaderBuf);
+    requestHead->Flatten(mReqHeaderBuf, pruneProxyHeaders);
 
 #if defined(PR_LOGGING)
     if (LOG2_ENABLED()) {
@@ -166,6 +175,10 @@ nsHttpTransaction::SetupRequest(nsHttpRequestHead *requestHead,
 #endif
 
     mReqUploadStream = requestBody;
+    if (mReqUploadStream) {
+        mReqUploadStream->Available(&mReqUploadStreamLength);
+        mReqUploadStreamOffset = 0;
+    }
 
     // If the request body does not include headers or if there is no request
     // body, then we must add the header/body separator manually.
@@ -198,6 +211,15 @@ nsHttpTransaction::TakeResponseHead()
 // nsHttpTransaction::nsAHttpTransaction
 //----------------------------------------------------------------------------
 
+PRUint32
+nsHttpTransaction::GetRequestSize()
+{
+    PRUint32 n = 0;
+    if (mReqHeaderStream)
+        mReqHeaderStream->Available(&n);
+    return n;
+}
+
 // called on the socket transport thread
 nsresult
 nsHttpTransaction::OnDataWritable(nsIOutputStream *os)
@@ -215,8 +237,17 @@ nsHttpTransaction::OnDataWritable(nsIOutputStream *os)
     if (n != 0)
         return os->WriteFrom(mReqHeaderStream, NS_HTTP_BUFFER_SIZE, &n);
 
-    if (mReqUploadStream)
-        return os->WriteFrom(mReqUploadStream, NS_HTTP_BUFFER_SIZE, &n);
+    if (mReqUploadStream) {
+        nsresult rv = os->WriteFrom(mReqUploadStream, NS_HTTP_BUFFER_SIZE, &n);
+        if (NS_SUCCEEDED(rv)) {
+            mReqUploadStreamOffset += n;
+            if (mProgressSink)
+                mProgressSink->OnProgress(nsnull, nsnull,
+                                          mReqUploadStreamOffset,
+                                          mReqUploadStreamLength);
+        }
+        return rv;
+    }
 
     return NS_BASE_STREAM_CLOSED;
 }
@@ -266,12 +297,19 @@ nsHttpTransaction::OnStopTransaction(nsresult status)
 
     // if the connection was reset before we read any part of the response,
     // then we must try to restart the transaction.
-    if ((status == NS_ERROR_NET_RESET) && (mContentRead == 0)) {
+    if (status == NS_ERROR_NET_RESET) {
+        // if some data was read, then mask the reset error, so our listener
+        // will treat this as a normal failure.  XXX we might want to map
+        // this error to a special error code to indicate that the transfer
+        // was abnormally interrupted.
+        if (mContentRead > 0)
+            status = NS_ERROR_ABORT;
         // if restarting fails, then we must notify our listener.
-        if (NS_SUCCEEDED(Restart()))
+        else if (NS_SUCCEEDED(Restart()))
             return NS_OK;
     }
 
+    mTransactionDone = 1; // force this flag
     mStatus = status;
 
 	if (mListener) {
@@ -309,7 +347,7 @@ nsHttpTransaction::Restart()
     // limit the number of restart attempts - bug 92224
     if (++mRestartCount >= nsHttpHandler::get()->MaxRequestAttempts()) {
         LOG(("reached max request attempts, failing transaction @%x\n", this));
-        return NS_BINDING_FAILED;
+        return NS_ERROR_NET_RESET;
     }
 
     LOG(("restarting transaction @%x\n", this));
@@ -321,6 +359,7 @@ nsHttpTransaction::Restart()
     seekable = do_QueryInterface(mReqUploadStream);
     if (seekable)
         seekable->Seek(nsISeekableStream::NS_SEEK_SET, 0);
+    mReqUploadStreamOffset = 0;
 
     // just in case the connection is holding the last reference to us...
     NS_ADDREF_THIS();
@@ -361,28 +400,29 @@ nsHttpTransaction::ParseLine(char *line)
         mResponseHead->ParseHeaderLine(line);
 }
 
-void
+nsresult
 nsHttpTransaction::ParseLineSegment(char *segment, PRUint32 len)
 {
     NS_PRECONDITION(!mHaveAllHeaders, "already have all headers");
 
     if (!mLineBuf.IsEmpty() && mLineBuf.Last() == '\n') {
-        // if this segment is a continuation of the previous...
-        if (mHaveStatusLine && (*segment == ' ' || *segment == '\t')) {
-            // trim off the new line char
-            mLineBuf.Truncate(mLineBuf.Length() - 1);
-            mLineBuf.Append(segment, len);
-        }
-        else {
-            // trim off the new line char and parse the line
-            mLineBuf.Truncate(mLineBuf.Length() - 1);
+        // trim off the new line char, and if this segment is
+        // not a continuation of the previous or if we haven't
+        // parsed the status line yet, then parse the contents
+        // of mLineBuf.
+        mLineBuf.Truncate(mLineBuf.Length() - 1);
+        if (!mHaveStatusLine || (*segment != ' ' && *segment != '\t')) {
             ParseLine(NS_CONST_CAST(char*,mLineBuf.get()));
-            // stuff the segment into the line buf
-            mLineBuf.Assign(segment, len);
+            mLineBuf.Truncate();
         }
     }
-    else
-        mLineBuf.Append(segment, len);
+
+    // append segment to mLineBuf...
+    if (mLineBuf.Length() + len > MAX_LINEBUF_LENGTH) {
+        LOG(("excessively long header received, canceling transaction [trans=%x]", this));
+        return NS_ERROR_ABORT;
+    }
+    mLineBuf.Append(segment, len);
     
     // a line buf with only a new line char signifies the end of headers.
     if (mLineBuf.First() == '\n') {
@@ -392,10 +432,11 @@ nsHttpTransaction::ParseLineSegment(char *segment, PRUint32 len)
             LOG(("ignoring 100 response\n"));
             mHaveStatusLine = PR_FALSE;
             mResponseHead->Reset();
-            return;
+            return NS_OK;
         }
         mHaveAllHeaders = PR_TRUE;
     }
+    return NS_OK;
 }
 
 nsresult
@@ -403,6 +444,7 @@ nsHttpTransaction::ParseHead(char *buf,
                              PRUint32 count,
                              PRUint32 *countRead)
 {
+    nsresult rv;
     PRUint32 len;
     char *eol;
 
@@ -423,7 +465,7 @@ nsHttpTransaction::ParseHead(char *buf,
     // this must be the first time we've been called.
     if (!mHaveStatusLine && mLineBuf.IsEmpty()) {
         // tolerate some junk before the status line
-        char *p = LocateHttpStart(buf, PR_MIN(count, 32));
+        char *p = LocateHttpStart(buf, PR_MIN(count, 8));
         if (!p) {
             mResponseHead->ParseStatusLine("");
             mHaveStatusLine = PR_TRUE;
@@ -433,7 +475,6 @@ nsHttpTransaction::ParseHead(char *buf,
         if (p > buf) {
             // skip over the junk
             *countRead = p - buf;
-            count -= *countRead;
             buf = p;
         }
     }
@@ -450,7 +491,9 @@ nsHttpTransaction::ParseHead(char *buf,
             len--;
 
         buf[len-1] = '\n';
-        ParseLineSegment(buf, len);
+        rv = ParseLineSegment(buf, len);
+        if (NS_FAILED(rv))
+            return rv;
 
         if (mHaveAllHeaders)
             return NS_OK;
@@ -466,7 +509,9 @@ nsHttpTransaction::ParseHead(char *buf,
         // ParseLineSegment if buf only contains a carriage return.
         if ((buf[len-1] == '\r') && (--len == 0))
             return NS_OK;
-        ParseLineSegment(buf, len);
+        rv = ParseLineSegment(buf, len);
+        if (NS_FAILED(rv))
+            return rv;
     }
     return NS_OK;
 }
@@ -492,7 +537,7 @@ nsHttpTransaction::HandleContentStart()
 #endif
         // notify the connection, give it a chance to cause a reset.
         PRBool reset = PR_FALSE;
-        mConnection->OnHeadersAvailable(this, mResponseHead, &reset);
+        mConnection->OnHeadersAvailable(this, mRequestHead, mResponseHead, &reset);
 
         // looks like we should ignore this response, resetting...
         if (reset) {
@@ -553,19 +598,21 @@ nsHttpTransaction::HandleContentStart()
 nsresult
 nsHttpTransaction::HandleContent(char *buf,
                                  PRUint32 count,
-                                 PRUint32 *countRead)
+                                 PRUint32 *contentRead,
+                                 PRUint32 *contentRemaining)
 {
     nsresult rv;
 
     LOG(("nsHttpTransaction::HandleContent [this=%x count=%u]\n",
         this, count));
 
-    *countRead = 0;
+    *contentRead = 0;
+    *contentRemaining = 0;
 
     if (mTransactionDone)
         return NS_OK;
 
-    NS_PRECONDITION(mConnection, "no connection");
+    NS_ASSERTION(mConnection, "no connection");
 
     if (!mFiredOnStart) {
         rv = HandleContentStart();
@@ -575,7 +622,7 @@ nsHttpTransaction::HandleContent(char *buf,
     if (mChunkedDecoder) {
         // give the buf over to the chunked decoder so it can reformat the
         // data and tell us how much is really there.
-        rv = mChunkedDecoder->HandleChunkedContent(buf, count, countRead);
+        rv = mChunkedDecoder->HandleChunkedContent(buf, count, contentRead, contentRemaining);
         if (NS_FAILED(rv)) return rv;
     }
     else if (mContentLength >= 0) {
@@ -584,31 +631,33 @@ nsHttpTransaction::HandleContent(char *buf,
         // allowances for a possibly invalid Content-Length header. Thus, if
         // NOT persistent, we simply accept everything in |buf|.
         if (mConnection->IsPersistent()) {
-            *countRead = PRUint32(mContentLength) - mContentRead;
-            *countRead = PR_MIN(count, *countRead);
+            *contentRead = PRUint32(mContentLength) - mContentRead;
+            *contentRead = PR_MIN(count, *contentRead);
         }
         else {
-            *countRead = count;
+            *contentRead = count;
             // mContentLength might need to be increased...
-            if (*countRead + mContentRead > (PRUint32) mContentLength) {
-                mContentLength = *countRead + mContentRead;
+            if (*contentRead + mContentRead > (PRUint32) mContentLength) {
+                mContentLength = *contentRead + mContentRead;
                 //mResponseHead->SetContentLength(mContentLength);
             }
         }
+        *contentRemaining = (count - *contentRead);
     }
-    else
+    else {
         // when we are just waiting for the server to close the connection...
-        *countRead = count;
+        *contentRead = count;
+    }
 
-    if (*countRead) {
+    if (*contentRead) {
         // update count of content bytes read and report progress...
-        mContentRead += *countRead;
+        mContentRead += *contentRead;
         if (mProgressSink)
             mProgressSink->OnProgress(nsnull, nsnull, mContentRead, PR_MAX(0, mContentLength));
     }
 
     LOG(("nsHttpTransaction [this=%x count=%u read=%u mContentRead=%u mContentLength=%d]\n",
-        this, count, *countRead, mContentRead, mContentLength));
+        this, count, *contentRead, mContentRead, mContentLength));
 
     // check for end-of-file
     if ((mContentRead == PRUint32(mContentLength)) ||
@@ -617,6 +666,7 @@ nsHttpTransaction::HandleContent(char *buf,
         // OnTransactionComplete is fired only once!
         PRInt32 priorVal = PR_AtomicSet(&mTransactionDone, 1);
         if (priorVal == 0) {
+            mResponseIsComplete = PR_TRUE;
             // let the connection know that we are done with it; this should
             // result in OnStopTransaction being fired.
             return mConnection->OnTransactionComplete(this, NS_OK);
@@ -626,7 +676,7 @@ nsHttpTransaction::HandleContent(char *buf,
 
     // if we didn't "read" anything and this is not a no-content response,
     // then we must return NS_BASE_STREAM_WOULD_BLOCK so we'll be called again.
-    return (!mNoContent && !*countRead) ? NS_BASE_STREAM_WOULD_BLOCK : NS_OK;
+    return (!mNoContent && !*contentRead) ? NS_BASE_STREAM_WOULD_BLOCK : NS_OK;
 }
 
 void
@@ -712,7 +762,7 @@ NS_IMPL_THREADSAFE_QUERY_INTERFACE2(nsHttpTransaction,
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsHttpTransaction::GetName(PRUnichar **aName)
+nsHttpTransaction::GetName(nsACString &aName)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -743,10 +793,6 @@ nsHttpTransaction::Cancel(nsresult status)
         return NS_OK;
     }
 
-    // the status must be set immediately as the cancelation may only take
-    // action asynchronously.
-    mStatus = status;
-
     // if the transaction is already "done" then there is nothing more to do.
     // ie., our consumer _will_ eventually receive their OnStopRequest.
     PRInt32 priorVal = PR_AtomicSet(&mTransactionDone, 1);
@@ -755,6 +801,10 @@ nsHttpTransaction::Cancel(nsresult status)
         return NS_OK;
     }
 
+    // the status must be set immediately as the cancelation may only take
+    // action asynchronously.
+    mStatus = status;
+
     return nsHttpHandler::get()->CancelTransaction(this, status);
 }
 
@@ -832,10 +882,17 @@ nsHttpTransaction::Read(char *buf, PRUint32 count, PRUint32 *bytesWritten)
     // read some data from our source and put it in the given buf
     rv = mSource->Read(buf, count, bytesWritten);
     LOG(("mSource->Read [rv=%x count=%u countRead=%u]\n", rv, count, *bytesWritten));
-    if (NS_FAILED(rv)) {
+
+    // detect explicit socket RESET
+    if (rv == NS_ERROR_NET_RESET) {
+        LOG(("got NS_ERROR_NET_RESET\n"));
+        *bytesWritten = 0;
+    }
+    else if (NS_FAILED(rv)) {
         LOG(("nsHttpTransaction: mSource->Read() returned [rv=%x]\n", rv));
         return rv;
     }
+
     if (*bytesWritten == 0) {
         LOG(("nsHttpTransaction: reached EOF\n"));
         if (!mHaveStatusLine) {
@@ -849,7 +906,7 @@ nsHttpTransaction::Read(char *buf, PRUint32 count, PRUint32 *bytesWritten)
             // and there is still a header line unparsed.  let's make sure we parse
             // the remaining header line, and then hopefully, the response will be
             // usable (see bug 88792).
-            ParseLineSegment("\n", 1);
+            rv = ParseLineSegment("\n", 1);
         }
         return rv;
     }
@@ -876,8 +933,32 @@ nsHttpTransaction::Read(char *buf, PRUint32 count, PRUint32 *bytesWritten)
 
     // even though count may be 0, we still want to call HandleContent
     // so it can complete the transaction if this is a "no-content" response.
-    if (mHaveAllHeaders)
-        return HandleContent(buf, count, bytesWritten);
+    if (mHaveAllHeaders) {
+        PRUint32 bytesRemaining = 0;
+        //
+        // buf layout:
+        // 
+        // +-----------------------------------------+----------------+-----+
+        // |              bytesWritten               | bytesRemaining |     |
+        // +-----------------------------------------+----------------+-----+
+        //
+        // count          : bytes read from the socket
+        // bytesWritten   : bytes corresponding to this transaction
+        // bytesRemaining : bytes corresponding to next pipelined transaction
+        //
+        // NOTE:
+        // count > bytesWritten + bytesRemaining <==> chunked transfer encoding
+        //
+        rv = HandleContent(buf, count, bytesWritten, &bytesRemaining);
+        if (NS_FAILED(rv)) return rv;
+        // we may have read more than our share, in which case we must give
+        // the excess bytes back to the connection
+        if (mResponseIsComplete && bytesRemaining) {
+            NS_ASSERTION(mConnection, "no connection");
+            mConnection->PushBack(buf + *bytesWritten, bytesRemaining);
+        }
+        return rv;
+    }
 
     // wait for more data
     return NS_BASE_STREAM_WOULD_BLOCK;
@@ -891,18 +972,7 @@ nsHttpTransaction::ReadSegments(nsWriteSegmentFun writer, void *closure,
 }
 
 NS_IMETHODIMP
-nsHttpTransaction::GetNonBlocking(PRBool *result)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsHttpTransaction::GetObserver(nsIInputStreamObserver **obs)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-NS_IMETHODIMP
-nsHttpTransaction::SetObserver(nsIInputStreamObserver *obs)
+nsHttpTransaction::IsNonBlocking(PRBool *result)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
