@@ -47,6 +47,23 @@
 #include "nsIMultiPartChannel.h"
 #include "nsCRT.h"
 
+//
+// Helper function for determining the length of data bytes up to
+// the next multipart token.  A token is usually preceded by a LF
+// or CRLF delimiter.
+// 
+static PRUint32
+LengthToToken(const char *cursor, const char *token)
+{
+    PRUint32 len = token - cursor;
+    // Trim off any LF or CRLF preceding the token
+    if (len && *(token-1) == '\n') {
+        --len;
+        if (len && *(token-2) == '\r')
+            --len;
+    }
+    return len;
+}
 
 //
 // nsPartChannel is a "dummy" channel which represents an individual part of
@@ -451,11 +468,10 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             memcpy(buffer, mBuffer, mBufLen);
             free(mBuffer);
             mBuffer = 0;
+            mBufLen = 0;
         }
         
-        rv = inStr->Read(buffer + mBufLen, count, &read);
-
-        mBufLen = 0;
+        rv = inStr->Read(buffer + (bufLen - count), count, &read);
 
         if (NS_FAILED(rv) || read == 0) return rv;
         NS_ASSERTION(read == count, "poor data size assumption");
@@ -485,7 +501,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             if (!buffer)
                 return NS_ERROR_OUT_OF_MEMORY;
 
-            memmove(buffer+mTokenLen+1, buffer, bufLen);
+            memmove(buffer + mTokenLen + 1, buffer, bufLen);
             memcpy(buffer, token, mTokenLen);
             buffer[mTokenLen] = '\n';
 
@@ -503,8 +519,8 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         // for this "part" given the previous buffer given to 
         // us in the previous OnDataAvailable callback.
         PRBool done = PR_FALSE;
-		rv = ParseHeaders(channel, cursor, bufLen, &done);
-		if (NS_FAILED(rv)) ERR_OUT
+        rv = ParseHeaders(channel, cursor, bufLen, &done);
+        if (NS_FAILED(rv)) ERR_OUT
 
         if (done) {
             mProcessingHeaders = PR_FALSE;
@@ -517,17 +533,17 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
     while ( (token = FindToken(cursor, bufLen)) ) {
 
         if (*(token+mTokenLen+1) == '-') {
-			// This was the last delimiter so we can stop processing
-            rv = SendData(cursor, token - cursor);
+            // This was the last delimiter so we can stop processing
+            rv = SendData(cursor, LengthToToken(cursor, token));
             free(buffer);
             if (NS_FAILED(rv)) return rv;
             return SendStop(NS_OK);
         }
 
         if (!mNewPart && token > cursor) {
-			// headers are processed, we're pushing data now.
-			NS_ASSERTION(!mProcessingHeaders, "we should be pushing raw data");
-            rv = SendData(cursor, token - cursor);
+            // headers are processed, we're pushing data now.
+            NS_ASSERTION(!mProcessingHeaders, "we should be pushing raw data");
+            rv = SendData(cursor, LengthToToken(cursor, token));
             bufLen -= token - cursor;
             if (NS_FAILED(rv)) ERR_OUT
         }
@@ -556,6 +572,14 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         }
         else {
             mNewPart = PR_TRUE;
+            // Reset state so we don't carry it over from part to part
+            mContentType.Truncate();
+            mContentLength = -1;
+            mContentDisposition.Truncate();
+            mIsByteRangeRequest = PR_FALSE;
+            mByteRangeStart = 0;
+            mByteRangeEnd = 0;
+            
             rv = SendStop(NS_OK);
             if (NS_FAILED(rv)) ERR_OUT
             // reset the token to front. this allows us to treat
@@ -604,8 +628,8 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
 // nsIRequestObserver implementation
 NS_IMETHODIMP
 nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
-	// we're assuming the content-type is available at this stage
-	NS_ASSERTION(mToken.IsEmpty(), "a second on start???");
+    // we're assuming the content-type is available at this stage
+    NS_ASSERTION(mToken.IsEmpty(), "a second on start???");
     const char *bndry = nsnull;
     nsCAutoString delimiter;
     nsresult rv = NS_OK;
@@ -856,7 +880,7 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
                                PRUint32 &aLen, PRBool *_retval) {
     // NOTE: this data must be ascii.
     // NOTE: aPtr is NOT null terminated!
-	nsresult rv = NS_OK;
+    nsresult rv = NS_OK;
     char *cursor = aPtr, *newLine = nsnull;
     PRUint32 cursorLen = aLen;
     PRBool done = PR_FALSE;
@@ -955,7 +979,7 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
     aPtr = cursor;
     aLen = cursorLen;
 
-	*_retval = done;
+    *_retval = done;
     return rv;
 }
 
