@@ -177,7 +177,7 @@ public:
     nsReader()
         : mStartTime(0), mBytesRead(0)
     {
-        NS_INIT_REFCNT();
+        NS_INIT_ISUPPORTS();
         gNumReaders++;
     }
 
@@ -260,7 +260,7 @@ InitQueue() {
              do_GetService(kEventQueueServiceCID, &rv);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't get event queue service");
 
-    rv = eventQService->GetThreadEventQueue(PR_CurrentThread(), &eventQueue);
+    rv = eventQService->GetThreadEventQueue(PR_GetCurrentThread(), &eventQueue);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't get event queue for main thread");
 
     return NS_OK;
