@@ -62,6 +62,7 @@
 #include "nsIResumableEntityID.h"
 #include "nsIURL.h"
 #include "nsIHttpChannel.h"
+#include "nsIHttpHeaderVisitor.h"
 #include "nsIHttpEventSink.h" 
 #include "nsIInterfaceRequestor.h" 
 #include "nsIInterfaceRequestorUtils.h"
@@ -129,7 +130,7 @@ public:
 
 URLLoadInfo::URLLoadInfo(const char *aUrl) : mURLString(aUrl)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mBytesRead = 0;
   mConnectTime = mTotalTime = PR_Now();
@@ -158,7 +159,7 @@ public:
 
 TestHttpEventSink::TestHttpEventSink()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 TestHttpEventSink::~TestHttpEventSink()
@@ -201,7 +202,7 @@ public:
 
 InputTestConsumer::InputTestConsumer()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 InputTestConsumer::~InputTestConsumer()
@@ -402,7 +403,7 @@ public:
     NS_DECL_ISUPPORTS
 
     NotificationCallbacks() {
-        NS_INIT_REFCNT();
+        NS_INIT_ISUPPORTS();
     }
 
     NS_IMETHOD GetInterface(const nsIID& eventSinkIID, void* *result) {
