@@ -54,7 +54,9 @@
 #include "nsXPIDLString.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
+#include "nsCRT.h"
 #include "nsNetCID.h"
+#include "nsIComponentRegistrar.h"
 
 // Define CIDs...
 static NS_DEFINE_CID(kIOServiceCID,              NS_IOSERVICE_CID);
@@ -86,7 +88,7 @@ nsresult writeoutto(const char* i_pURL, char** o_Result, PRInt32 urlFactory = UR
                 return NS_ERROR_FAILURE;
             }
             pURL = url;
-            pURL->SetSpec((char*)i_pURL);
+            pURL->SetSpec(nsDependentCString(i_pURL));
             break;
         }
         case URL_FACTORY_DEFAULT: {
@@ -97,7 +99,7 @@ nsresult writeoutto(const char* i_pURL, char** o_Result, PRInt32 urlFactory = UR
                 cout << "Service failed!" << endl;
                 return NS_ERROR_FAILURE;
             }   
-            result = pService->NewURI(i_pURL, nsnull, getter_AddRefs(pURL));
+            result = pService->NewURI(nsDependentCString(i_pURL), nsnull, nsnull, getter_AddRefs(pURL));
         }
     }
 
@@ -105,44 +107,47 @@ nsresult writeoutto(const char* i_pURL, char** o_Result, PRInt32 urlFactory = UR
     if (NS_SUCCEEDED(result))
     {
         nsCOMPtr<nsIURL> tURL = do_QueryInterface(pURL);
-        nsXPIDLCString temp;
+        nsCAutoString temp;
         PRInt32 port;
+        nsresult rv;
 
-        tURL->GetScheme(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+#define RESULT() NS_SUCCEEDED(rv) ? temp.get() : ""
+
+        rv = tURL->GetScheme(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetUsername(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetUsername(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetPassword(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetPassword(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetHost(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetHost(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetPort(&port);
+        rv = tURL->GetPort(&port);
         output.AppendInt(port);
         output += ',';
-        tURL->GetDirectory(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetDirectory(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetFileBaseName(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetFileBaseName(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetFileExtension(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetFileExtension(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetParam(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetParam(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetQuery(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetQuery(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetRef(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetRef(temp);
+        output += RESULT();
         output += ',';
-        tURL->GetSpec(getter_Copies(temp));
-        output += temp ? (const char*)temp : "";
+        rv = tURL->GetSpec(temp);
+        output += RESULT();
         *o_Result = ToNewCString(output);
     } else {
         output = "Can not create URL";
@@ -239,7 +244,7 @@ nsresult makeAbsTest(const char* i_BaseURI, const char* relativePortion,
 {
     if (!i_BaseURI)
         return NS_ERROR_FAILURE;
-    
+
     // build up the base URL
     nsCOMPtr<nsIURI> baseURL;
     nsresult status = nsComponentManager::CreateInstance(kStdURLCID, nsnull, 
@@ -249,25 +254,25 @@ nsresult makeAbsTest(const char* i_BaseURI, const char* relativePortion,
         cout << "CreateInstance failed" << endl;
         return status;
     }
-    status = baseURL->SetSpec((char*)i_BaseURI);
+    status = baseURL->SetSpec(nsDependentCString(i_BaseURI));
     if (NS_FAILED(status)) return status;
 
 
     // get the new spec
-    nsXPIDLCString newURL;
-    status = baseURL->Resolve(relativePortion, getter_Copies(newURL));
+    nsCAutoString newURL;
+    status = baseURL->Resolve(nsDependentCString(relativePortion), newURL);
     if (NS_FAILED(status)) return status;
 
-    nsXPIDLCString temp;
-    baseURL->GetSpec(getter_Copies(temp));
+    nsCAutoString temp;
+    baseURL->GetSpec(temp);
 
-    cout << "Analyzing " << temp << endl;
+    cout << "Analyzing " << temp.get() << endl;
     cout << "With      " << relativePortion << endl;
-    
-    cout << "Got       " <<  newURL << endl;
+
+    cout << "Got       " << newURL.get() << endl;
     if (expectedResult) {
         cout << "Expect    " << expectedResult << endl;
-        int res = PL_strcmp(newURL, expectedResult);
+        int res = PL_strcmp(newURL.get(), expectedResult);
         if (res == 0) {
             cout << "\tPASSED" << endl << endl;
             return NS_OK;
@@ -393,12 +398,6 @@ int doMakeAbsTest(const char* i_URL = 0, const char* i_relativePortion=0)
     }
 }
 
-nsresult NS_AutoregisterComponents()
-{
-  nsresult rv = nsComponentManager::AutoRegister(nsIComponentManagerObsolete::NS_Startup, NULL /* default */);
-  return rv;
-}
-
 void printusage(void)
 {
     cout << "urltest [-std] [-file <filename>] <URL> " <<
@@ -421,66 +420,74 @@ int main(int argc, char **argv)
         printusage();
         return NS_OK;
     }
+    {
+        nsCOMPtr<nsIServiceManager> servMan;
+        NS_InitXPCOM2(getter_AddRefs(servMan), nsnull, nsnull);
+        nsCOMPtr<nsIComponentRegistrar> registrar = do_QueryInterface(servMan);
+        NS_ASSERTION(registrar, "Null nsIComponentRegistrar");
+        if (registrar)
+            registrar->AutoRegister(nsnull);
 
-    result = NS_AutoregisterComponents();
-    if (NS_FAILED(result)) return result;
+        // end of all messages from register components...
+        cout << "------------------" << endl << endl;
 
-    // end of all messages from register components...
-    cout << "------------------" << endl << endl; 
-
-    PRInt32 urlFactory = URL_FACTORY_DEFAULT;
-    PRBool bMakeAbs= PR_FALSE;
-    char* relativePath = 0;
-    char* url = 0;
-    for (int i=1; i<argc; i++) {
-        if (PL_strcasecmp(argv[i], "-std") == 0) 
-        {
-            urlFactory = URL_FACTORY_STDURL;
-            if (i+1 >= argc)
+        PRInt32 urlFactory = URL_FACTORY_DEFAULT;
+        PRBool bMakeAbs= PR_FALSE;
+        char* relativePath = 0;
+        char* url = 0;
+        for (int i=1; i<argc; i++) {
+            if (PL_strcasecmp(argv[i], "-std") == 0)
             {
-                printusage();
-                return NS_OK;
+                urlFactory = URL_FACTORY_STDURL;
+                if (i+1 >= argc)
+                {
+                    printusage();
+                    return NS_OK;
+                }
             }
-        }
-        else if (PL_strcasecmp(argv[i], "-abs") == 0)
-        {
-            if (!gFileIO) 
+            else if (PL_strcasecmp(argv[i], "-abs") == 0)
             {
-                relativePath = argv[i+1]; 
+                if (!gFileIO)
+                {
+                    relativePath = argv[i+1];
+                    i++;
+                }
+                bMakeAbs = PR_TRUE;
+            }
+            else if (PL_strcasecmp(argv[i], "-file") == 0)
+            {
+                if (i+1 >= argc)
+                {
+                    printusage();
+                    return NS_OK;
+                }
+                gFileIO = argv[i+1];
                 i++;
             }
-            bMakeAbs = PR_TRUE;
-        }
-        else if (PL_strcasecmp(argv[i], "-file") == 0)
-        {
-            if (i+1 >= argc)
+            else
             {
-                printusage();
-                return NS_OK;
+                url = argv[i];
             }
-            gFileIO = argv[i+1];
-            i++;
+        }
+        PRTime startTime = PR_Now();
+        if (bMakeAbs)
+        {
+            rv = (url && relativePath)
+               ? doMakeAbsTest(url, relativePath)
+               : doMakeAbsTest();
         }
         else
         {
-            url = argv[i];
+            rv = gFileIO ? testURL(0, urlFactory) : testURL(url, urlFactory);
         }
-    }
-    PRTime startTime = PR_Now();
-    if (bMakeAbs)
-    {
-        rv = (url && relativePath) ?  doMakeAbsTest(url, relativePath) : 
-            doMakeAbsTest();
-    }
-    else
-    {
-        rv = gFileIO ? testURL(0, urlFactory) : testURL(url, urlFactory);
-    }
-    if (gFileIO)
-    {
-        PRTime endTime = PR_Now();
-        printf("Elapsed time: %d micros.\n", (PRInt32) 
-            (endTime - startTime));
-    }
+        if (gFileIO)
+        {
+            PRTime endTime = PR_Now();
+            printf("Elapsed time: %d micros.\n", (PRInt32)
+                (endTime - startTime));
+        }
+    } // this scopes the nsCOMPtrs
+    // no nsCOMPtrs are allowed to be alive when you call NS_ShutdownXPCOM
+    rv = NS_ShutdownXPCOM(nsnull);
     return rv;
 }
