@@ -201,7 +201,7 @@ static jobject GetCurrentThread(JNIEnv* env)
 MRJSecurityContext::MRJSecurityContext(const char* location)
     :   mLocation(nsnull), mConnection(nsnull)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
     
     mLocation = ::strdup(location);
     if (mLocation) {
