@@ -61,7 +61,6 @@
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsIStringBundle.h"
-#include "nsISupportsPrimitives.h"
 #include "nsProxiedService.h"
 
 #include "nsNetUtil.h"
@@ -271,7 +270,7 @@ nsPrefMigration::GetInstance()
 
 nsPrefMigration::nsPrefMigration()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mErrorCode = NS_OK;
 }
 
@@ -336,7 +335,7 @@ nsPrefMigration::ProcessPrefs(PRBool showProgressAsModalWindow)
 {
   nsresult rv;
   
-  nsCOMPtr<nsIWindowWatcher> windowWatcher(do_GetService("@mozilla.org/embedcomp/window-watcher;1", &rv));
+  nsCOMPtr<nsIWindowWatcher> windowWatcher(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
   if (NS_FAILED(rv)) return rv;
 
   // WindowWatcher can work with or without parent window
@@ -473,10 +472,10 @@ NS_IMETHODIMP
 nsPrefMigration::ShowSpaceDialog(PRInt32 *choice)
 {
   nsresult rv;
-  nsCOMPtr<nsIWindowWatcher> windowWatcher(do_GetService("@mozilla.org/embedcomp/window-watcher;1", &rv));
+  nsCOMPtr<nsIWindowWatcher> windowWatcher(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIDialogParamBlock> ioParamBlock(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1", &rv));
+  nsCOMPtr<nsIDialogParamBlock> ioParamBlock(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID, &rv));
   if (NS_FAILED(rv)) return rv;
   
   ioParamBlock->SetInt(0,3); //set the Retry, CreateNew and Cancel buttons
@@ -2258,7 +2257,7 @@ nsPrefConverter::~nsPrefConverter()
 nsPrefConverter::nsPrefConverter()
 {
 
-   NS_INIT_REFCNT();
+   NS_INIT_ISUPPORTS();
 }
 
 NS_IMPL_ISUPPORTS1(nsPrefConverter, nsIPrefConverter)
