@@ -362,7 +362,7 @@ RDFContainerUtilsImpl::MakeAlt(nsIRDFDataSource *aDataSource, nsIRDFResource *aR
 
 RDFContainerUtilsImpl::RDFContainerUtilsImpl()
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 
     if (gRefCnt++ == 0) {
         nsresult rv;
