@@ -39,6 +39,9 @@
 #include "nsIX509CertDB.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
+#include "nsNSSCertificate.h"
+#include "nsNSSCertHelper.h"
+#include "nsINSSCertCache.h"
 
 #include "prlog.h"
 #ifdef PR_LOGGING
@@ -53,23 +56,93 @@ static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
 // field from a cert) has an element in the array.  The numChildren field
 // stores the number of certs corresponding to that thread.
 struct treeArrayElStr {
-  PRUnichar *orgName;     /* heading for thread                   */
+  nsString   orgName;     /* heading for thread                   */
   PRBool     open;        /* toggle open state for thread         */
   PRInt32    certIndex;   /* index into cert array for 1st cert   */
   PRInt32    numChildren; /* number of chidren (certs) for thread */
 };
 
+CompareCacheHashEntry::CompareCacheHashEntry()
+:key(nsnull)
+{
+  for (int i = 0; i < max_criterions; ++i) {
+    mCritInit[i] = PR_FALSE;
+  }
+}
+
+PR_STATIC_CALLBACK(const void *)
+CompareCacheGetKey(PLDHashTable *table, PLDHashEntryHdr *hdr)
+{
+  CompareCacheHashEntry *entry = NS_STATIC_CAST(CompareCacheHashEntry*, hdr);
+  return entry->key;
+}
+
+PR_STATIC_CALLBACK(PRBool)
+CompareCacheMatchEntry(PLDHashTable *table, const PLDHashEntryHdr *hdr,
+                         const void *key)
+{
+  const CompareCacheHashEntry *entry = NS_STATIC_CAST(const CompareCacheHashEntry*, hdr);
+  return entry->key == key;
+}
+
+PR_STATIC_CALLBACK(void)
+CompareCacheInitEntry(PLDHashTable *table, PLDHashEntryHdr *hdr,
+                     const void *key)
+{
+  new (hdr) CompareCacheHashEntry();
+  CompareCacheHashEntry *entry = NS_STATIC_CAST(CompareCacheHashEntry*, hdr);
+  entry->key = (void*)key;
+}
+
+PR_STATIC_CALLBACK(void)
+CompareCacheClearEntry(PLDHashTable *table, PLDHashEntryHdr *hdr)
+{
+  CompareCacheHashEntry *entry = NS_STATIC_CAST(CompareCacheHashEntry*, hdr);
+  entry->~CompareCacheHashEntry();
+}
+
+static PLDHashTableOps gMapOps = {
+  PL_DHashAllocTable,
+  PL_DHashFreeTable,
+  CompareCacheGetKey,
+  PL_DHashVoidPtrKeyStub,
+  CompareCacheMatchEntry,
+  PL_DHashMoveEntryStub,
+  CompareCacheClearEntry,
+  PL_DHashFinalizeStub,
+  CompareCacheInitEntry
+};
+
+
+
 NS_IMPL_ISUPPORTS2(nsCertTree, nsICertTree, nsITreeView)
 
 nsCertTree::nsCertTree() : mTreeArray(NULL)
 {
   NS_INIT_ISUPPORTS();
+  mCompareCache.ops = nsnull;
+  mNSSComponent = do_GetService(kNSSComponentCID);
+}
+
+void nsCertTree::ClearCompareHash()
+{
+  if (mCompareCache.ops) {
+    PL_DHashTableFinish(&mCompareCache);
+    mCompareCache.ops = nsnull;
+  }
+}
+
+void nsCertTree::InitCompareHash()
+{
+  ClearCompareHash();
+  PL_DHashTableInit(&mCompareCache, &gMapOps, nsnull,
+                    sizeof(CompareCacheHashEntry), 128);
 }
 
 nsCertTree::~nsCertTree()
 {
-  if (mTreeArray)
-    nsMemory::Free(mTreeArray);
+  ClearCompareHash();
+  delete [] mTreeArray;
 }
 
 void
@@ -91,83 +164,30 @@ nsCertTree::FreeCertArray()
   }
 }
 
-// CmpByToken
-//
-// Compare two certificate by their token name.  Returns -1, 0, 1 as
-// in strcmp.  No token name (null) is treated as <.
-PRInt32
-nsCertTree::CmpByToken(nsIX509Cert *a, nsIX509Cert *b)
+CompareCacheHashEntry *
+nsCertTree::getCacheEntry(void *cache, void *aCert)
 {
-  PRInt32 cmp1;
-  nsXPIDLString aTok, bTok;
-  a->GetTokenName(getter_Copies(aTok));
-  b->GetTokenName(getter_Copies(bTok));
-  if (aTok != nsnull && bTok != nsnull) {
-    cmp1 = Compare(aTok, bTok);
-  } else {
-    cmp1 = (aTok == nsnull) ? -1 : 1;
+  PLDHashTable &aCompareCache = *NS_REINTERPRET_CAST(PLDHashTable*, cache);
+  PLDHashEntryHdr *entry = PL_DHashTableOperate(&aCompareCache, aCert, PL_DHASH_LOOKUP);
+  if (PL_DHASH_ENTRY_IS_BUSY(entry))
+  {
+    return NS_STATIC_CAST(CompareCacheHashEntry*, entry);
+  }
+  else
+  {
+    return NS_STATIC_CAST(CompareCacheHashEntry*, PL_DHashTableOperate(&aCompareCache, aCert, PL_DHASH_ADD));
   }
-  return cmp1;
 }
 
-// CmpByIssuerOrg
-//
-// Compare two certificates by their O= field.  Returns -1, 0, 1 as
-// in strcmp.  No organization (null) is treated as <.
-PRInt32
-nsCertTree::CmpByIssuerOrg(nsIX509Cert *a, nsIX509Cert *b)
+void nsCertTree::RemoveCacheEntry(void *key)
 {
-  PRInt32 cmp1;
-  nsXPIDLString aOrg, bOrg;
-  a->GetIssuerOrganization(getter_Copies(aOrg));
-  b->GetIssuerOrganization(getter_Copies(bOrg));
-  if (aOrg != nsnull && bOrg != nsnull) {
-    cmp1 = Compare(aOrg, bOrg);
-  } else {
-    cmp1 = (aOrg == nsnull) ? -1 : 1;
-  }
-  return cmp1;
-}
-
-// CmpByName
-//
-// Compare two certificates by their CN= field.  Returns -1, 0, 1 as
-// in strcmp.  No common name (null) is treated as <.
-PRInt32
-nsCertTree::CmpByName(nsIX509Cert *a, nsIX509Cert *b)
-{
-  PRInt32 cmp1;
-  nsXPIDLString aName, bName;
-  a->GetOrganization(getter_Copies(aName));
-  b->GetOrganization(getter_Copies(bName));
-  if (aName != nsnull && bName != nsnull) {
-    cmp1 = Compare(aName, bName);
-  } else {
-    cmp1 = (aName == nsnull) ? -1 : 1;
-  }
-  return cmp1;
-}
-
-// CmpByTok_IssuerOrg_Name
-//
-// Compare two certificates by token name, issuer organization, 
-// and common name, in that order.  Used to sort cert list.
-PRInt32
-nsCertTree::CmpByTok_IssuerOrg_Name(nsIX509Cert *a, nsIX509Cert *b)
-{
-  PRInt32 cmp;
-  cmp = CmpByToken(a, b);
-  if (cmp != 0) return cmp;
-  cmp = CmpByIssuerOrg(a, b);
-  if (cmp != 0) return cmp;
-  return CmpByName(a, b);
+  PL_DHashTableOperate(&mCompareCache, key, PL_DHASH_REMOVE);
 }
 
 // CountOrganizations
 //
 // Count the number of different organizations encountered in the cert
-// list.  Note that the same organization of a different token is counted
-// seperately.
+// list.
 PRInt32
 nsCertTree::CountOrganizations()
 {
@@ -182,8 +202,8 @@ nsCertTree::CountOrganizations()
   for (i=1; i<certCount; i++) {
     isupport = dont_AddRef(mCertArray->ElementAt(i));
     nextCert = do_QueryInterface(isupport);
-    if (!(CmpByToken(orgCert, nextCert) == 0 &&
-          CmpByIssuerOrg(orgCert, nextCert) == 0)) {
+    // XXX we assume issuer org is always criterion 1
+    if (CmpBy(&mCompareCache, orgCert, nextCert, sort_IssuerOrg, sort_None, sort_None) != 0) {
       orgCert = nextCert;
       orgCount++;
     }
@@ -245,25 +265,127 @@ nsCertTree::GetCertAtIndex(PRInt32 index)
   return rawPtr;
 }
 
+nsCertTree::nsCertCompareFunc
+nsCertTree::GetCompareFuncFromCertType(PRUint32 aType)
+{
+  switch (aType) {
+    case nsIX509Cert::USER_CERT:
+      return CmpUserCert;
+    case nsIX509Cert::CA_CERT:
+      return CmpCACert;
+    case nsIX509Cert::EMAIL_CERT:
+      return CmpEmailCert;
+    case nsIX509Cert::SERVER_CERT:
+    default:
+      return CmpWebSiteCert;
+  }
+}
+
+PRBool
+nsCertTree::GetCertsByTypeFromCertList(CERTCertList *aCertList,
+                                       PRUint32 aType,
+                                       nsCertCompareFunc  aCertCmpFn,
+                                       void *aCertCmpFnArg,
+                                       nsISupportsArray **_certs)
+{
+  PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("GetCertsByTypeFromCertList"));
+  if (!aCertList)
+    return PR_FALSE;
+  nsCOMPtr<nsISupportsArray> certarray;
+  nsresult rv = NS_NewISupportsArray(getter_AddRefs(certarray));
+  if (NS_FAILED(rv)) return PR_FALSE;
+  CERTCertListNode *node;
+  int i, count = 0;
+  for (node = CERT_LIST_HEAD(aCertList);
+       !CERT_LIST_END(node, aCertList);
+       node = CERT_LIST_NEXT(node)) {
+    if (getCertType(node->cert) == aType) {
+      nsCOMPtr<nsIX509Cert> pipCert = new nsNSSCertificate(node->cert);
+      if (pipCert) {
+        for (i=0; i<count; i++) {
+          nsCOMPtr<nsISupports> isupport = 
+            dont_AddRef(certarray->ElementAt(i));
+          nsCOMPtr<nsIX509Cert> cert = do_QueryInterface(isupport);
+          if ((*aCertCmpFn)(aCertCmpFnArg, pipCert, cert) < 0) {
+            certarray->InsertElementAt(pipCert, i);
+            break;
+          }
+        }
+        if (i == count) certarray->AppendElement(pipCert);
+          count++;
+      }
+    }
+  }
+  *_certs = certarray;
+  NS_ADDREF(*_certs);
+  return PR_TRUE;
+}
+
+PRBool 
+nsCertTree::GetCertsByType(PRUint32           aType,
+                           nsCertCompareFunc  aCertCmpFn,
+                           void              *aCertCmpFnArg,
+                           nsISupportsArray **_certs)
+{
+  CERTCertList *certList = NULL;
+  nsCOMPtr<nsIInterfaceRequestor> cxt = new PipUIContext();
+  certList = PK11_ListCerts(PK11CertListUnique, cxt);
+  PRBool rv = GetCertsByTypeFromCertList(certList, aType, aCertCmpFn, aCertCmpFnArg, _certs);
+  if (certList)
+    CERT_DestroyCertList(certList);
+  return rv;
+}
+
+PRBool 
+nsCertTree::GetCertsByTypeFromCache(nsINSSCertCache   *aCache,
+                                    PRUint32           aType,
+                                    nsCertCompareFunc  aCertCmpFn,
+                                    void              *aCertCmpFnArg,
+                                    nsISupportsArray **_certs)
+{
+  NS_ENSURE_ARG_POINTER(aCache);
+  CERTCertList *certList = NS_REINTERPRET_CAST(CERTCertList*, aCache->GetCachedCerts());
+  if (!certList)
+    return NS_ERROR_FAILURE;
+  return GetCertsByTypeFromCertList(certList, aType, aCertCmpFn, aCertCmpFnArg, _certs);
+}
+
 // LoadCerts
 //
 // Load all of the certificates in the DB for this type.  Sort them
 // by token, organization, then common name.
+NS_IMETHODIMP 
+nsCertTree::LoadCertsFromCache(nsINSSCertCache *aCache, PRUint32 aType)
+{
+  nsresult rv;
+  if (mTreeArray) {
+    FreeCertArray();
+    delete [] mTreeArray;
+    mTreeArray = NULL;
+    mNumRows = 0;
+  }
+  InitCompareHash();
+  rv = GetCertsByTypeFromCache(aCache, aType, 
+                               GetCompareFuncFromCertType(aType), &mCompareCache,
+                               getter_AddRefs(mCertArray));
+  if (NS_FAILED(rv)) return rv;
+  return UpdateUIContents();
+}
+
 NS_IMETHODIMP 
 nsCertTree::LoadCerts(PRUint32 aType)
 {
   nsresult rv;
   if (mTreeArray) {
     FreeCertArray();
-    nsMemory::Free(mTreeArray);
+    delete [] mTreeArray;
     mTreeArray = NULL;
     mNumRows = 0;
   }
-  nsCOMPtr<nsIX509CertDB> certdb = do_GetService(NS_X509CERTDB_CONTRACTID);
-  if (certdb == nsnull) return NS_ERROR_FAILURE;
-  rv = certdb->GetCertsByType(aType, 
-                              CmpByTok_IssuerOrg_Name,
-                              getter_AddRefs(mCertArray));
+  InitCompareHash();
+  rv = GetCertsByType(aType, 
+                      GetCompareFuncFromCertType(aType), &mCompareCache,
+                      getter_AddRefs(mCertArray));
   if (NS_FAILED(rv)) return rv;
   return UpdateUIContents();
 }
@@ -275,20 +397,19 @@ nsCertTree::UpdateUIContents()
   nsresult rv = mCertArray->Count(&count);
   if (NS_FAILED(rv)) return rv;
   mNumOrgs = CountOrganizations();
-  mTreeArray = (treeArrayEl *)nsMemory::Alloc(
-                                           sizeof(treeArrayEl) * mNumOrgs);
+  mTreeArray = new treeArrayEl[mNumOrgs];
   PRUint32 j = 0;
   nsCOMPtr<nsISupports> isupport = dont_AddRef(mCertArray->ElementAt(j));
   nsCOMPtr<nsIX509Cert> orgCert = do_QueryInterface(isupport);
   for (PRInt32 i=0; i<mNumOrgs; i++) {
-    orgCert->GetIssuerOrganization(&mTreeArray[i].orgName);
+    orgCert->GetIssuerOrganization(mTreeArray[i].orgName);
     mTreeArray[i].open = PR_TRUE;
     mTreeArray[i].certIndex = j;
     mTreeArray[i].numChildren = 1;
     if (++j >= count) break;
     isupport = dont_AddRef(mCertArray->ElementAt(j));
     nsCOMPtr<nsIX509Cert> nextCert = do_QueryInterface(isupport);
-    while (CmpByIssuerOrg(orgCert, nextCert) == 0) {
+    while (0 == CmpBy(&mCompareCache, orgCert, nextCert, sort_IssuerOrg, sort_None, sort_None)) {
       mTreeArray[i].numChildren++;
       if (++j >= count) break;
       isupport = dont_AddRef(mCertArray->ElementAt(j));
@@ -297,8 +418,10 @@ nsCertTree::UpdateUIContents()
     orgCert = nextCert;
   }
   mNumRows = count + mNumOrgs;
-  if (mTree)
+  if (mTree) {
+    mTree->RowCountChanged(0, mNumRows);
     mTree->Invalidate();
+  }
   return NS_OK;
 }
 
@@ -309,8 +432,8 @@ nsCertTree::RemoveCert(PRUint32 index)
     return NS_ERROR_FAILURE;
   }
 
-  int i, idx = 0, cIndex = 0, nc;
-  nsIX509Cert *rawPtr = nsnull;
+  int i;
+  PRUint32 idx = 0, cIndex = 0, nc;
   // Loop over the threads
   for (i=0; i<mNumOrgs; i++) {
     if (index == idx)
@@ -319,8 +442,10 @@ nsCertTree::RemoveCert(PRUint32 index)
     nc = (mTreeArray[i].open) ? mTreeArray[i].numChildren : 0;
     if (index < idx + nc) { // cert is within range of this thread
       PRInt32 certIndex = cIndex + index - idx;
+      nsCOMPtr<nsISupports> isupport = dont_AddRef(mCertArray->ElementAt(certIndex));
+      RemoveCacheEntry(isupport);
       mCertArray->RemoveElementAt(certIndex);
-      nsMemory::Free(mTreeArray);
+      delete [] mTreeArray;
       mTreeArray = NULL;
       return UpdateUIContents();
     }
@@ -537,89 +662,95 @@ nsCertTree::GetCellText(PRInt32 row, const PRUnichar *colID,
   }
   nsCOMPtr<nsIX509Cert> cert = dont_AddRef(GetCertAtIndex(row));
   if (cert == nsnull) return NS_ERROR_FAILURE;
-  char *str = NULL;
-  PRUnichar *wstr = NULL;
   if (strcmp(col, "certcol") == 0) {
-    rv = cert->GetCommonName(&wstr);
-    if (NS_FAILED(rv) || !wstr) {
-      // can this be fixed to not do copying?
-      PRUnichar *tmp = nsnull;
-      rv = cert->GetNickname(&tmp);
-      nsAutoString nick(tmp);
-      char *tmps = ToNewCString(nick);
-      char *mark = strchr(tmps, ':');
-      if (mark) {
-        str = PL_strdup(mark + 1);
-      } else {
-        wstr = ToNewUnicode(nick);
+    rv = cert->GetCommonName(_retval);
+    if (NS_FAILED(rv) || _retval.IsEmpty()) {
+      // kaie: I didn't invent the idea to cut off anything before 
+      //       the first colon. :-)
+      nsAutoString nick;
+      rv = cert->GetNickname(nick);
+      
+      nsAString::const_iterator start, end, end2;
+      nick.BeginReading(start);
+      nick.EndReading(end);
+      end2 = end;
+
+      if (FindInReadable(NS_LITERAL_STRING(":"), start, end)) {
+        // found. end points to the first char after the colon,
+        // that's what we want.
+        _retval = Substring(end, end2);
+      }
+      else {
+        _retval = nick;
       }
-      nsMemory::Free(tmp);
-      nsMemory::Free(tmps);
     }
   } else if (strcmp(col, "tokencol") == 0) {
-    rv = cert->GetTokenName(&wstr);
+    rv = cert->GetTokenName(_retval);
   } else if (strcmp(col, "emailcol") == 0) {
-    rv = cert->GetEmailAddress(&wstr);
-  } else if (strcmp(col, "verifiedcol") == 0) {
+    rv = cert->GetEmailAddress(_retval);
+  } else if (strcmp(col, "purposecol") == 0 && mNSSComponent) {
     PRUint32 verified;
-    nsCOMPtr<nsINSSComponent> nssComponent(
-                                      do_GetService(kNSSComponentCID, &rv));
-    if (NS_FAILED(rv)) return rv;
     PRBool ocspEnabled;
     cert->GetUsesOCSP(&ocspEnabled);
     if (ocspEnabled) {
-      nssComponent->DisableOCSP();
+      mNSSComponent->DisableOCSP();
     }
-    rv = cert->GetPurposes(&verified, NULL);
-    if (verified == nsIX509Cert::VERIFIED_OK) {
-      nsAutoString vfy;
-      rv = nssComponent->GetPIPNSSBundleString(
-                                NS_LITERAL_STRING("VerifiedTrue").get(), vfy);
-      if (NS_SUCCEEDED(rv))
-        wstr = ToNewUnicode(vfy);
-    } else {
-      nsAutoString vfy;
-      rv = nssComponent->GetPIPNSSBundleString(
-                                NS_LITERAL_STRING("VerifiedFalse").get(), vfy);
-      if (NS_SUCCEEDED(rv))
-        wstr = ToNewUnicode(vfy);
+
+    nsAutoString dummyPurposes;
+    rv = cert->GetPurposes(&verified, dummyPurposes);
+    if (NS_FAILED(rv)) {
+      verified = nsIX509Cert::NOT_VERIFIED_UNKNOWN;
     }
+
+    switch (verified) {
+      case nsIX509Cert::VERIFIED_OK:
+        rv = cert->GetPurposes(&verified, _retval);
+        break;
+
+      case nsIX509Cert::CERT_REVOKED:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyRevoked").get(), _retval);
+        break;
+      case nsIX509Cert::CERT_EXPIRED:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyExpired").get(), _retval);
+        break;
+      case nsIX509Cert::CERT_NOT_TRUSTED:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyNotTrusted").get(), _retval);
+        break;
+      case nsIX509Cert::ISSUER_NOT_TRUSTED:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyIssuerNotTrusted").get(), _retval);
+        break;
+      case nsIX509Cert::ISSUER_UNKNOWN:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyIssuerUnknown").get(), _retval);
+        break;
+      case nsIX509Cert::INVALID_CA:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyInvalidCA").get(), _retval);
+        break;
+      case nsIX509Cert::NOT_VERIFIED_UNKNOWN:
+      case nsIX509Cert::USAGE_NOT_ALLOWED:
+      default:
+        rv = mNSSComponent->GetPIPNSSBundleString(
+                                  NS_LITERAL_STRING("VerifyUnknown").get(), _retval);
+        break;
+    }
+
     if (ocspEnabled) {
-      nssComponent->EnableOCSP();
-    }
-  } else if (strcmp(col, "purposecol") == 0) {
-    PRUint32 verified;
-    PRBool ocspEnabled;
-    nsCOMPtr<nsINSSComponent> nssComponent(
-                                      do_GetService(kNSSComponentCID, &rv));
-    if (NS_FAILED(rv)) return rv;
-  
-    cert->GetUsesOCSP(&ocspEnabled);
-    if (ocspEnabled) {
-      nssComponent->DisableOCSP();
-    }
-    rv = cert->GetPurposes(&verified, &wstr);
-    if (ocspEnabled) {
-      nssComponent->EnableOCSP();
+      mNSSComponent->EnableOCSP();
     }
   } else if (strcmp(col, "issuedcol") == 0) {
-    rv = cert->GetIssuedDate(&wstr);
+    rv = cert->GetIssuedDate(_retval);
   } else if (strcmp(col, "expiredcol") == 0) {
-    rv = cert->GetExpiresDate(&wstr);
+    rv = cert->GetExpiresDate(_retval);
   } else if (strcmp(col, "serialnumcol") == 0) {
-    rv = cert->GetSerialNumber(&wstr);
-/*
-  } else if (strcmp(col, "certdbkeycol") == 0) {
-    rv = cert->GetDbKey(&str);
-*/
+    rv = cert->GetSerialNumber(_retval);
   } else {
     return NS_ERROR_FAILURE;
   }
-  if (str) {
-    nsAutoString astr = NS_ConvertASCIItoUCS2(str);
-    wstr = ToNewUnicode(astr);
-  }
-  _retval = wstr;
   return rv;
 }
 
@@ -780,3 +911,120 @@ NS_IMETHODIMP nsCertTree::IsSorted(PRBool *_retval)
   *_retval = PR_FALSE;
   return NS_OK;
 }
+
+void 
+nsCertTree::CmpInitCriterion(nsIX509Cert *cert, CompareCacheHashEntry *entry,
+                             sortCriterion crit, PRInt32 level)
+{
+  entry->mCritInit[level] = PR_TRUE;
+  nsXPIDLString &str = entry->mCrit[level];
+  
+  switch (crit) {
+    case sort_IssuerOrg:
+      cert->GetIssuerOrganization(str);
+      break;
+    case sort_Org:
+      cert->GetOrganization(str);
+      break;
+    case sort_Token:
+      cert->GetTokenName(str);
+      break;
+    case sort_CommonName:
+      cert->GetCommonName(str);
+      break;
+    case sort_IssuedDateDescending:
+      cert->GetIssuedDateSortable(str);
+      break;
+    case sort_Email:
+      cert->GetEmailAddress(str);
+      break;
+    case sort_None:
+    default:
+      break;
+  }
+}
+
+PRInt32
+nsCertTree::CmpByCrit(nsIX509Cert *a, CompareCacheHashEntry *ace, 
+                      nsIX509Cert *b, CompareCacheHashEntry *bce, 
+                      sortCriterion crit, PRInt32 level)
+{
+  if (!ace->mCritInit[level]) {
+    CmpInitCriterion(a, ace, crit, level);
+  }
+
+  if (!bce->mCritInit[level]) {
+    CmpInitCriterion(b, bce, crit, level);
+  }
+
+  nsXPIDLString &str_a = ace->mCrit[level];
+  nsXPIDLString &str_b = bce->mCrit[level];
+
+  PRInt32 result;
+  if (str_a && str_b)
+    result = Compare(str_a, str_b);
+  else
+    result = !str_a ? (!str_b ? 0 : -1) : 1;
+
+  if (sort_IssuedDateDescending == crit)
+    result *= -1; // reverse compare order
+
+  return result;
+}
+
+PRInt32
+nsCertTree::CmpBy(void *cache, nsIX509Cert *a, nsIX509Cert *b, 
+                  sortCriterion c0, sortCriterion c1, sortCriterion c2)
+{
+  if (a == b)
+    return 0;
+
+  CompareCacheHashEntry *ace = getCacheEntry(cache, a);
+  CompareCacheHashEntry *bce = getCacheEntry(cache, b);
+
+  PRInt32 cmp;
+  cmp = CmpByCrit(a, ace, b, bce, c0, 0);
+  if (cmp != 0)
+    return cmp;
+
+  if (c1 != sort_None) {
+    cmp = CmpByCrit(a, ace, b, bce, c1, 1);
+    if (cmp != 0)
+      return cmp;
+    
+    if (c2 != sort_None) {
+      return CmpByCrit(a, ace, b, bce, c2, 2);
+    }
+  }
+
+  return cmp;
+}
+
+PRInt32
+nsCertTree::CmpCACert(void *cache, nsIX509Cert *a, nsIX509Cert *b)
+{
+  // XXX we assume issuer org is always criterion 1
+  return CmpBy(cache, a, b, sort_IssuerOrg, sort_Org, sort_Token);
+}
+
+PRInt32
+nsCertTree::CmpWebSiteCert(void *cache, nsIX509Cert *a, nsIX509Cert *b)
+{
+  // XXX we assume issuer org is always criterion 1
+  return CmpBy(cache, a, b, sort_IssuerOrg, sort_CommonName, sort_None);
+}
+
+PRInt32
+nsCertTree::CmpUserCert(void *cache, nsIX509Cert *a, nsIX509Cert *b)
+{
+  // XXX we assume issuer org is always criterion 1
+  return CmpBy(cache, a, b, sort_IssuerOrg, sort_Token, sort_IssuedDateDescending);
+}
+
+PRInt32
+nsCertTree::CmpEmailCert(void *cache, nsIX509Cert *a, nsIX509Cert *b)
+{
+  // XXX we assume issuer org is always criterion 1
+  return CmpBy(cache, a, b, sort_IssuerOrg, sort_Email, sort_CommonName);
+}
+
