@@ -31,7 +31,7 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  *
- * $Id: nsPKCS12Blob.cpp,v 1.26 2002/05/07 23:06:45 darin%netscape.com Exp $
+ * $Id: nsPKCS12Blob.cpp,v 1.31 2002/09/23 20:17:13 kaie%netscape.com Exp $
  */
 
 #include "prmem.h"
@@ -40,7 +40,6 @@
 #include "nsISupportsArray.h"
 #include "nsIFileSpec.h"
 #include "nsILocalFile.h"
-#include "nsINSSDialogs.h"
 #include "nsIDirectoryService.h"
 #include "nsIWindowWatcher.h"
 #include "nsIPrompt.h"
@@ -58,6 +57,7 @@
 #include "nsNSSCertificate.h"
 #include "nsKeygenHandler.h" //For GetSlotWithMechanism
 #include "nsPK11TokenDB.h"
+#include "nsICertificateDialogs.h"
 
 #include "pk11func.h"
 #include "secerr.h"
@@ -120,14 +120,6 @@ nsresult
 nsPKCS12Blob::ImportFromFile(nsILocalFile *file)
 {
   nsresult rv;
-  SECStatus srv = SECSuccess;
-  SEC_PKCS12DecoderContext *dcx = NULL;
-  SECItem unicodePw;
-
-  PK11SlotInfo *slot=nsnull;
-  nsXPIDLString tokenName;
-  nsXPIDLCString tokenNameCString;
-  const char *tokNameRef;
 
   if (!mToken && !mTokenSet) {
     SetToken(NULL); // Ask the user to pick a slot
@@ -139,7 +131,33 @@ nsPKCS12Blob::ImportFromFile(nsILocalFile *file)
   }
   // init slot
   rv = mToken->Login(PR_TRUE);
-  if (NS_FAILED(rv)) goto finish;
+  if (NS_FAILED(rv)) return rv;
+  
+  int wantRetry;
+  
+  do {
+    rv = ImportFromFileHelper(file, wantRetry);
+  }
+  while (NS_SUCCEEDED(rv) && wantRetry);
+  
+  return rv;
+}
+
+nsresult
+nsPKCS12Blob::ImportFromFileHelper(nsILocalFile *file, PRBool &aWantRetry)
+{
+  nsresult rv;
+  SECStatus srv = SECSuccess;
+  SEC_PKCS12DecoderContext *dcx = NULL;
+  SECItem unicodePw;
+
+  PK11SlotInfo *slot=nsnull;
+  nsXPIDLString tokenName;
+  nsXPIDLCString tokenNameCString;
+  const char *tokNameRef;
+  
+  aWantRetry = PR_FALSE;
+
   // get file password (unicode)
   unicodePw.data = NULL;
   rv = getPKCS12FilePassword(&unicodePw);
@@ -172,7 +190,13 @@ nsPKCS12Blob::ImportFromFile(nsILocalFile *file)
   }
   // read input file and feed it to the decoder
   rv = inputToDecoder(dcx, file);
-  if (NS_FAILED(rv)) goto finish;
+  if (NS_FAILED(rv)) {
+    if (NS_ERROR_ABORT == rv) {
+      // inputToDecoder indicated a NSS error
+      srv = SECFailure;
+    }
+    goto finish;
+  }
   // verify the blob
   srv = SEC_PKCS12DecoderVerify(dcx);
   if (srv) goto finish;
@@ -189,6 +213,9 @@ finish:
   // We should use that error code instead of inventing a new one
   // for every error possible.
   if (srv != SECSuccess) {
+    if (SEC_ERROR_BAD_PASSWORD == PORT_GetError()) {
+      aWantRetry = PR_TRUE;
+    }
     handleError(PIP_PKCS12_NSS_ERROR);
   } else if (NS_FAILED(rv)) { 
     handleError(PIP_PKCS12_RESTORE_FAILED);
@@ -398,7 +425,7 @@ finish:
 // a buffer of octets.  Must handle byte order correctly.
 // TODO: Is there a mozilla way to do this?  In the string lib?
 void
-nsPKCS12Blob::unicodeToItem(PRUnichar *uni, SECItem *item)
+nsPKCS12Blob::unicodeToItem(const PRUnichar *uni, SECItem *item)
 {
   int len = 0;
   int i = 0;
@@ -422,15 +449,16 @@ nsresult
 nsPKCS12Blob::newPKCS12FilePassword(SECItem *unicodePw)
 {
   nsresult rv = NS_OK;
-  PRUnichar *password;
+  nsAutoString password;
   PRBool canceled;
   nsCOMPtr<nsICertificateDialogs> certDialogs;
   rv = ::getNSSDialogs(getter_AddRefs(certDialogs), 
-                       NS_GET_IID(nsICertificateDialogs));
+                       NS_GET_IID(nsICertificateDialogs),
+                       NS_CERTIFICATEDIALOGS_CONTRACTID);
   if (NS_FAILED(rv)) return rv;
-  rv = certDialogs->SetPKCS12FilePassword(mUIContext, &password, &canceled);
+  rv = certDialogs->SetPKCS12FilePassword(mUIContext, password, &canceled);
   if (NS_FAILED(rv) || canceled) return rv;
-  unicodeToItem(password, unicodePw);
+  unicodeToItem(password.get(), unicodePw);
   return NS_OK;
 }
 
@@ -442,15 +470,16 @@ nsresult
 nsPKCS12Blob::getPKCS12FilePassword(SECItem *unicodePw)
 {
   nsresult rv = NS_OK;
-  PRUnichar *password;
+  nsAutoString password;
   PRBool canceled;
   nsCOMPtr<nsICertificateDialogs> certDialogs;
   rv = ::getNSSDialogs(getter_AddRefs(certDialogs), 
-                       NS_GET_IID(nsICertificateDialogs));
+                       NS_GET_IID(nsICertificateDialogs),
+                       NS_CERTIFICATEDIALOGS_CONTRACTID);
   if (NS_FAILED(rv)) return rv;
-  rv = certDialogs->GetPKCS12FilePassword(mUIContext, &password, &canceled);
+  rv = certDialogs->GetPKCS12FilePassword(mUIContext, password, &canceled);
   if (NS_FAILED(rv) || canceled) return rv;
-  unicodeToItem(password, unicodePw);
+  unicodeToItem(password.get(), unicodePw);
   return NS_OK;
 }
 
@@ -478,8 +507,11 @@ nsPKCS12Blob::inputToDecoder(SEC_PKCS12DecoderContext *dcx, nsILocalFile *file)
     // feed the file data into the decoder
     srv = SEC_PKCS12DecoderUpdate(dcx, buf, amount);
     if (srv) {
+      // don't allow the close call to overwrite our precious error code
+      int pr_err = PORT_GetError();
       fileStream.close();
-      return NS_ERROR_FAILURE;
+      PORT_SetError(pr_err);
+      return NS_ERROR_ABORT;
     }
     if (amount < PIP_PKCS12_BUFFER_SIZE)
       break;
@@ -597,7 +629,7 @@ nsPKCS12Blob::nickname_collision(SECItem *oldNick, PRBool *cancel, void *wincx)
   nsCOMPtr<nsINSSComponent> nssComponent(do_GetService(kNSSComponentCID, &rv));
   if (NS_FAILED(rv)) return nsnull;
   int count = 1;
-  nsXPIDLCString nickname;
+  nsCString nickname;
   nsString nickFromProp;
   nssComponent->GetPIPNSSBundleString(
                                  NS_LITERAL_STRING("P12DefaultNickname").get(),
@@ -646,7 +678,7 @@ nsPKCS12Blob::nickname_collision(SECItem *oldNick, PRBool *cancel, void *wincx)
     return nsnull;
 
   newNick->type = siAsciiString;
-  newNick->data = (unsigned char*) nsCRT::strdup(nickname);
+  newNick->data = (unsigned char*) nsCRT::strdup(nickname.get());
   newNick->len  = strlen((char*)newNick->data);
   return newNick;
 }
@@ -678,8 +710,6 @@ pip_ucs2_ascii_conversion_fn(PRBool toUnicode,
   return PR_TRUE;
 }
 
-#define kWindowWatcherCID "@mozilla.org/embedcomp/window-watcher;1"
-
 PRBool
 nsPKCS12Blob::handleError(int myerr)
 {
@@ -694,7 +724,7 @@ nsPKCS12Blob::handleError(int myerr)
                                       do_GetService(NS_XPCOMPROXY_CONTRACTID));
   if (!proxyman) return PR_FALSE;
   nsCOMPtr<nsIPrompt> errPrompt;
-  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(kWindowWatcherCID));
+  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   if (wwatch) {
     wwatch->GetNewPrompter(0, getter_AddRefs(errPrompt));
     if (errPrompt) {
