@@ -35,6 +35,7 @@
 #include <stdlib.h>
 
 #include "blapi.h"
+#include "secrng.h"
 #include "prmem.h"
 #include "prprf.h"
 #include "prtime.h"
@@ -399,8 +400,8 @@ typedef SECStatus (* bltestSymmCipherFn)(void *cx,
 					 unsigned int inputLen);
 
 typedef SECStatus (* bltestPubKeyCipherFn)(void *key,
-					   unsigned char *output,
-					   const unsigned char *input);
+					   SECItem *output,
+					   const SECItem *input);
 
 typedef SECStatus (* bltestHashCipherFn)(unsigned char *dest,
 					 const unsigned char *src,
@@ -575,7 +576,7 @@ SECStatus
 setupIO(PRArenaPool *arena, bltestIO *input, PRFileDesc *file,
 	char *str, int numBytes)
 {
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     SECItem fileData;
     SECItem *in;
     unsigned char *tok;
@@ -644,7 +645,7 @@ setupIO(PRArenaPool *arena, bltestIO *input, PRFileDesc *file,
 SECStatus
 finishIO(bltestIO *output, PRFileDesc *file)
 {
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     PRInt32 nb;
     unsigned char byteval;
     SECItem *it;
@@ -709,6 +710,7 @@ misalignBuffer(PRArenaPool *arena, bltestIO *io, int off)
     int length = io->buf.len;
     if (offset != off) {
 	SECITEM_ReallocItem(arena, &io->buf, length, length + 2*WORDSIZE);
+	io->buf.len = length + 2*WORDSIZE; /* why doesn't realloc do this? */
 	/* offset may have changed? */
 	offset = (ptrdiff_t)io->buf.data % WORDSIZE;
 	if (offset != off) {
@@ -798,15 +800,27 @@ aes_Decrypt(void *cx, unsigned char *output, unsigned int *outputLen,
 }
 
 SECStatus
-rsa_PublicKeyOp(void *key, unsigned char *output, const unsigned char *input)
+rsa_PublicKeyOp(void *key, SECItem *output, const SECItem *input)
 {
-    return RSA_PublicKeyOp((RSAPublicKey *)key, output, input);
+    return RSA_PublicKeyOp((RSAPublicKey *)key, output->data, input->data);
 }
 
 SECStatus
-rsa_PrivateKeyOp(void *key, unsigned char *output, const unsigned char *input)
+rsa_PrivateKeyOp(void *key, SECItem *output, const SECItem *input)
 {
-    return RSA_PrivateKeyOp((RSAPrivateKey *)key, output, input);
+    return RSA_PrivateKeyOp((RSAPrivateKey *)key, output->data, input->data);
+}
+
+SECStatus
+dsa_signDigest(void *key, SECItem *output, const SECItem *input)
+{
+    return DSA_SignDigest((DSAPrivateKey *)key, output, input);
+}
+
+SECStatus
+dsa_verifyDigest(void *key, SECItem *output, const SECItem *input)
+{
+    return DSA_VerifyDigest((DSAPublicKey *)key, output, input);
 }
 
 SECStatus
@@ -957,7 +971,7 @@ bltest_aes_init(bltestCipherInfo *cipherInfo, PRBool encrypt)
     int i;
     /* XXX */ int keylen, blocklen;
     keylen = aesp->key.buf.len;
-    blocklen = cipherInfo->input.buf.len;
+    blocklen = cipherInfo->input.pBuf.len;
     switch (cipherInfo->mode) {
     case bltestAES_ECB:	    minorMode = NSS_AES;	  break;
     case bltestAES_CBC:	    minorMode = NSS_AES_CBC;	  break;
@@ -1084,7 +1098,7 @@ bltest_dsa_init(bltestCipherInfo *cipherInfo, PRBool encrypt)
 	cipherInfo->cx = dsakey_from_filedata(&dsap->key.buf);
     }
     if (encrypt) {
-	cipherInfo->cipher.pubkeyCipher = DSA_SignDigest;
+	cipherInfo->cipher.pubkeyCipher = dsa_signDigest;
     } else {
 	/* Have to convert private key to public key.  Memory
 	 * is freed with private key's arena  */
@@ -1100,7 +1114,7 @@ bltest_dsa_init(bltestCipherInfo *cipherInfo, PRBool encrypt)
 	pubkey->params.base.data = key->params.base.data;
 	pubkey->publicValue.len = key->publicValue.len;
 	pubkey->publicValue.data = key->publicValue.data;
-	cipherInfo->cipher.pubkeyCipher = DSA_VerifyDigest;
+	cipherInfo->cipher.pubkeyCipher = dsa_verifyDigest;
     }
     return SECSuccess;
 }
@@ -1126,7 +1140,7 @@ md2_restart(unsigned char *dest, const unsigned char *src, uint32 src_length)
     unsigned char *cxbytes;
     unsigned int len;
     unsigned int i, quarter;
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     cx = MD2_NewContext();
     /* divide message by 4, restarting 3 times */
     quarter = src_length / 4;
@@ -1159,7 +1173,7 @@ finish:
 SECStatus
 md5_restart(unsigned char *dest, const unsigned char *src, uint32 src_length)
 {
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     MD5Context *cx, *cx_cpy;
     unsigned char *cxbytes;
     unsigned int len;
@@ -1175,6 +1189,7 @@ md5_restart(unsigned char *dest, const unsigned char *src, uint32 src_length)
 	cx_cpy = MD5_Resurrect(cxbytes, NULL);
 	if (!cx_cpy) {
 	    PR_fprintf(PR_STDERR, "%s: MD5_Resurrect failed!\n", progName);
+	    rv = SECFailure;
 	    goto finish;
 	}
 	rv = PORT_Memcmp(cx, cx_cpy, len);
@@ -1196,7 +1211,7 @@ finish:
 SECStatus
 sha1_restart(unsigned char *dest, const unsigned char *src, uint32 src_length)
 {
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     SHA1Context *cx, *cx_cpy;
     unsigned char *cxbytes;
     unsigned int len;
@@ -1212,6 +1227,7 @@ sha1_restart(unsigned char *dest, const unsigned char *src, uint32 src_length)
 	cx_cpy = SHA1_Resurrect(cxbytes, NULL);
 	if (!cx_cpy) {
 	    PR_fprintf(PR_STDERR, "%s: SHA1_Resurrect failed!\n", progName);
+	    rv = SECFailure;
 	    goto finish;
 	}
 	rv = PORT_Memcmp(cx, cx_cpy, len);
@@ -1235,7 +1251,7 @@ pubkeyInitKey(bltestCipherInfo *cipherInfo, PRFileDesc *file,
 	      int keysize, int exponent)
 {
     int i;
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     bltestRSAParams *rsap;
     bltestDSAParams *dsap;
     switch (cipherInfo->mode) {
@@ -1355,13 +1371,12 @@ SECStatus
 dsaOp(bltestCipherInfo *cipherInfo)
 {
     PRIntervalTime time1, time2;
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     int i;
     int maxLen = cipherInfo->output.pBuf.len;
     SECItem dummyOut = { 0, 0, 0 };
     SECITEM_AllocItem(NULL, &dummyOut, maxLen);
-    if (cipherInfo->cipher.pubkeyCipher == 
-             (bltestPubKeyCipherFn)DSA_SignDigest) {
+    if (cipherInfo->cipher.pubkeyCipher == dsa_signDigest) {
 	if (cipherInfo->params.dsa.sigseed.buf.len > 0) {
 	    rv = DSA_SignDigestWithSeed((DSAPrivateKey *)cipherInfo->cx,
 				       &cipherInfo->output.pBuf,
@@ -1412,7 +1427,7 @@ SECStatus
 cipherDoOp(bltestCipherInfo *cipherInfo)
 {
     PRIntervalTime time1, time2;
-    SECStatus rv;
+    SECStatus rv = SECSuccess;
     int i, len;
     int maxLen = cipherInfo->output.pBuf.len;
     unsigned char *dummyOut;
@@ -1436,12 +1451,15 @@ cipherDoOp(bltestCipherInfo *cipherInfo)
 	TIMEFINISH(cipherInfo->optime, 1.0);
     } else if (is_pubkeyCipher(cipherInfo->mode)) {
 	rv = (*cipherInfo->cipher.pubkeyCipher)(cipherInfo->cx,
-						cipherInfo->output.pBuf.data,
-						cipherInfo->input.pBuf.data);
+						&cipherInfo->output.pBuf,
+						&cipherInfo->input.pBuf);
 	TIMESTART();
 	for (i=0; i<cipherInfo->repetitions; i++) {
-	    (*cipherInfo->cipher.pubkeyCipher)(cipherInfo->cx, dummyOut,
-					       cipherInfo->input.pBuf.data);
+	    SECItem dummy;
+	    dummy.data = dummyOut;
+	    dummy.len = maxLen;
+	    (*cipherInfo->cipher.pubkeyCipher)(cipherInfo->cx, &dummy,
+					       &cipherInfo->input.pBuf);
 	}
 	TIMEFINISH(cipherInfo->optime, 1.0);
     } else if (is_hashCipher(cipherInfo->mode)) {
@@ -1470,6 +1488,10 @@ cipherFinish(bltestCipherInfo *cipherInfo)
     case bltestDES_EDE_CBC:
 	DES_DestroyContext((DESContext *)cipherInfo->cx, PR_TRUE);
 	break;
+    case bltestAES_ECB:
+    case bltestAES_CBC:
+	AES_DestroyContext((AESContext *)cipherInfo->cx, PR_TRUE);
+	break;
     case bltestRC2_ECB:
     case bltestRC2_CBC:
 	RC2_DestroyContext((RC2Context *)cipherInfo->cx, PR_TRUE);
@@ -1522,6 +1544,8 @@ print_td:
     case bltestDES_CBC:
     case bltestDES_EDE_ECB:
     case bltestDES_EDE_CBC:
+    case bltestAES_ECB:
+    case bltestAES_CBC:
     case bltestRC2_ECB:
     case bltestRC2_CBC:
     case bltestRC4:
@@ -1635,12 +1659,14 @@ get_params(PRArenaPool *arena, bltestParams *params,
     case bltestDES_CBC:
     case bltestDES_EDE_CBC:
     case bltestRC2_CBC:
+    case bltestAES_CBC:
 	sprintf(filename, "%s/tests/%s/%s%d", testdir, modestr, "iv", j);
 	load_file_data(arena, &params->sk.iv, filename, bltestBinary);
     case bltestDES_ECB:
     case bltestDES_EDE_ECB:
     case bltestRC2_ECB:
     case bltestRC4:
+    case bltestAES_ECB:
 	sprintf(filename, "%s/tests/%s/%s%d", testdir, modestr, "key", j);
 	load_file_data(arena, &params->sk.key, filename, bltestBinary);
 	break;
@@ -1712,7 +1738,7 @@ verify_self_test(bltestIO *result, bltestIO *cmp, bltestCipherMode mode,
 {
     int res;
     char *modestr = mode_strings[mode];
-    res = SECITEM_CompareItem(&result->buf, &cmp->buf);
+    res = SECITEM_CompareItem(&result->pBuf, &cmp->buf);
     if (is_sigCipher(mode)) {
 	if (forward) {
 	    if (res == 0) {
@@ -1767,7 +1793,7 @@ blapi_selftest(bltestCipherMode *modes, int numModes, int inoff, int outoff,
     PRArenaPool *arena;
     SECItem item;
     PRBool finished;
-    SECStatus rv, srv;
+    SECStatus rv = SECSuccess, srv;
 
     PORT_Memset(&cipherInfo, 0, sizeof(cipherInfo));
     arena = PORT_NewArena(BLTEST_DEFAULT_CHUNKSIZE);
@@ -1887,7 +1913,7 @@ blapi_selftest(bltestCipherMode *modes, int numModes, int inoff, int outoff,
 		bltestCopyIO(arena, &cipherInfo.input, &ct);
 	    else
 		bltestCopyIO(arena, &cipherInfo.input, &pt);
-	    misalignBuffer(arena, &cipherInfo.input, outoff);
+	    misalignBuffer(arena, &cipherInfo.input, inoff);
 	    memset(&cipherInfo.output.buf, 0, sizeof cipherInfo.output.buf);
 	    rv |= cipherInit(&cipherInfo, PR_FALSE);
 	    misalignBuffer(arena, &cipherInfo.output, outoff);
@@ -1991,6 +2017,7 @@ enum {
     opt_SeedFile,
     opt_InputOffset,
     opt_OutputOffset,
+    opt_MonteCarlo,
     opt_CmdLine
 };
 
@@ -2036,6 +2063,7 @@ static secuCommandFlag bltest_options[] =
     { /* opt_SeedFile	  */ 'z', PR_FALSE, 0, PR_FALSE },
     { /* opt_InputOffset  */ '1', PR_TRUE,  0, PR_FALSE },
     { /* opt_OutputOffset */ '2', PR_TRUE,  0, PR_FALSE },
+    { /* opt_MonteCarlo   */ '3', PR_FALSE, 0, PR_FALSE },
     { /* opt_CmdLine	  */ '-', PR_FALSE, 0, PR_FALSE }
 };
 
@@ -2063,11 +2091,12 @@ int main(int argc, char **argv)
     progName = strrchr(argv[0], '/');
     progName = progName ? progName+1 : argv[0];
 
-    rv = NSS_NoDB_Init(NULL);
+    rv = RNG_RNGInit();
     if (rv != SECSuccess) {
     	SECU_PrintPRandOSError(progName);
 	return -1;
     }
+    RNG_SystemInfoForRNG();
 
     rv = SECU_ParseCommandLine(argc, argv, progName, &bltest);
 
@@ -2331,7 +2360,17 @@ int main(int argc, char **argv)
     misalignBuffer(cipherInfo.arena, &cipherInfo.output, outoff);
 
     if (!bltest.commands[cmd_Nonce].activated) {
-	cipherDoOp(&cipherInfo);
+	if (bltest.options[opt_MonteCarlo].activated) {
+	    int mciter;
+	    for (mciter=0; mciter<10000; mciter++) {
+		cipherDoOp(&cipherInfo);
+		memcpy(cipherInfo.input.buf.data,
+		       cipherInfo.output.buf.data,
+		       cipherInfo.input.buf.len);
+	    }
+	} else {
+	    cipherDoOp(&cipherInfo);
+	}
 	cipherFinish(&cipherInfo);
 	finishIO(&cipherInfo.output, outfile);
     }
