@@ -531,8 +531,7 @@ printCertCB(CERTCertificate *cert, void *arg)
 }
 
 static SECStatus
-dumpChain(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
-          PRBool raw, PRBool ascii, PRFileDesc *outfile, void *pwarg)
+DumpChain(CERTCertDBHandle *handle, char *name)
 {
     SECStatus rv;
     CERTCertificate *the_cert;
@@ -625,46 +624,23 @@ ListCerts(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
     SECStatus rv;
 
     if (slot == NULL) {
-	PK11SlotList *list;
-	PK11SlotListElement *le;
+	CERTCertList *list;
+	CERTCertListNode *node;
 
-	list= PK11_GetAllTokens(CKM_INVALID_MECHANISM,
-						PR_FALSE,PR_FALSE,pwdata);
-	if (list) {
-	    for (le = list->head; le; le = le->next) {
-		rv = listCerts(handle,name,le->slot,raw,ascii,outfile,pwdata);
-	    }
-	    PK11_FreeSlotList(list);
+	list = PK11_ListCerts(PK11CertListUnique, pwdata);
+	for (node = CERT_LIST_HEAD(list); !CERT_LIST_END(node, list);
+	     node = CERT_LIST_NEXT(node)) 
+	{
+	    SECU_PrintCertNickname(node->cert, stdout);
 	}
+	CERT_DestroyCertList(list);
+	return SECSuccess;
     } else {
 	rv = listCerts(handle,name,slot,raw,ascii,outfile,pwdata);
     }
     return rv;
 }
 
-static SECStatus
-DumpChain(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
-          PRBool raw, PRBool ascii, PRFileDesc *outfile, secuPWData *pwdata)
-{
-    SECStatus rv;
-
-    if (slot == NULL) {
-	PK11SlotList *list;
-	PK11SlotListElement *le;
-
-	list= PK11_GetAllTokens(CKM_INVALID_MECHANISM,
-						PR_FALSE,PR_FALSE,pwdata);
-	if (list) for (le = list->head; le; le = le->next) {
-	    printf("\n\nListing certificates for %s:\n\n", PK11_GetSlotName(le->slot));
-	    rv = dumpChain(handle,name,le->slot,raw,ascii,outfile,pwdata);
-	}
-    } else {
-	rv = dumpChain(handle,name,slot,raw,ascii,outfile,pwdata);
-    }
-    return rv;
-}
-
-
 static SECStatus 
 DeleteCert(CERTCertDBHandle *handle, char *name)
 {
@@ -695,22 +671,22 @@ ValidateCert(CERTCertDBHandle *handle, char *name, char *date,
     SECStatus rv;
     CERTCertificate *cert = NULL;
     int64 timeBoundary;
-    SECCertUsage usage;
+    SECCertificateUsage usage;
     CERTVerifyLog reallog;
     CERTVerifyLog *log = NULL;
     
     switch (*certUsage) {
 	case 'C':
-	    usage = certUsageSSLClient;
+	    usage = certificateUsageSSLClient;
 	    break;
 	case 'V':
-	    usage = certUsageSSLServer;
+	    usage = certificateUsageSSLServer;
 	    break;
 	case 'S':
-	    usage = certUsageEmailSigner;
+	    usage = certificateUsageEmailSigner;
 	    break;
 	case 'R':
-	    usage = certUsageEmailRecipient;
+	    usage = certificateUsageEmailRecipient;
 	    break;
 	default:
 	    PORT_SetError (SEC_ERROR_INVALID_ARGS);
@@ -747,8 +723,8 @@ ValidateCert(CERTCertDBHandle *handle, char *name, char *date,
 	    }
 	}
  
-	rv = CERT_VerifyCert(handle, cert, checkSig, usage,
-			     timeBoundary, pwdata, log);
+	rv = CERT_VerifyCertificate(handle, cert, checkSig, usage,
+			     timeBoundary, pwdata, log, &usage);
 	if ( log ) {
 	    if ( log->head == NULL ) {
 		fprintf(stdout, "%s: certificate is valid\n", progName);
@@ -1494,7 +1470,7 @@ AddExtKeyUsage (void *extHandle)
     fprintf(stdout, "%-25s 2 - Code Signing\n", "");
     fprintf(stdout, "%-25s 3 - Email Protection\n", "");
     fprintf(stdout, "%-25s 4 - Timestamp\n", "");
-    fprintf(stdout, "%-25s 5 - OSCP Responder\n", "");
+    fprintf(stdout, "%-25s 5 - OCSP Responder\n", "");
 #ifdef DEBUG_NSSTEAM_ONLY
     fprintf(stdout, "%-25s 6 - Step-up\n", "");
 #endif /* DEBUG_NSSTEAM_ONLY */
@@ -2161,7 +2137,7 @@ main(int argc, char **argv)
     CERTCertDBHandle *certHandle;
     PK11SlotInfo *slot = NULL;
     CERTName *  subject         = 0;
-    PRFileDesc *inFile          = 0;
+    PRFileDesc *inFile          = PR_STDIN;
     PRFileDesc *outFile         = 0;
     char *      certfile        = "tempcert";
     char *      certreqfile     = "tempcertreq";
@@ -2531,10 +2507,7 @@ main(int argc, char **argv)
 	goto shutdown;
     }
     if (certutil.commands[cmd_DumpChain].activated) {
-	rv = DumpChain(certHandle, name, slot,
-	               certutil.options[opt_BinaryDER].activated,
-	               certutil.options[opt_ASCIIForIO].activated, 
-                       (outFile) ? outFile : PR_STDOUT, &pwdata);
+	rv = DumpChain(certHandle, name);
 	goto shutdown;
     }
     /*  XXX needs work  */
@@ -2693,6 +2666,7 @@ main(int argc, char **argv)
      */
  
     if (certutil.commands[cmd_CreateAndAddCert].activated) { 
+	PORT_Assert(inFile != PR_STDIN);
 	PR_Close(inFile);
 	PR_Close(outFile);
 	inFile = PR_Open(certfile, PR_RDONLY, 0);
@@ -2717,6 +2691,7 @@ main(int argc, char **argv)
     }
 
     if (certutil.commands[cmd_CreateAndAddCert].activated) {
+	PORT_Assert(inFile != PR_STDIN);
 	PR_Close(inFile);
 	PR_Delete(certfile);
 	PR_Delete(certreqfile);
