@@ -42,6 +42,15 @@
 #include "certdb.h"
 #include "cryptohi.h"
 
+#ifndef NSS_3_4_CODE
+#define NSS_3_4_CODE
+#endif /* NSS_3_4_CODE */
+#include "nsspki.h"
+#include "pkitm.h"
+#include "pkim.h"
+#include "pki3hack.h"
+#include "base.h"
+
 #define PENDING_SLOP (24L*60L*60L)
 
 /*
@@ -104,7 +113,7 @@ CERT_VerifySignedData(CERTSignedData *sd, CERTCertificate *cert,
     if ( !pubKey ) {
 	return(SECFailure);
     }
-    
+
     /* check the signature */
     sig = sd->signature;
     DER_ConvertBitString(&sig);
@@ -258,48 +267,7 @@ SECStatus
 SEC_CheckCRL(CERTCertDBHandle *handle,CERTCertificate *cert,
 	     CERTCertificate *caCert, int64 t, void * wincx)
 {
-    CERTSignedCrl *crl = NULL;
-    SECStatus rv = SECSuccess;
-    CERTCrlEntry **crlEntry;
-    SECCertTimeValidity validity;
-
-    /* first look up the CRL */
-    crl = SEC_FindCrlByName(handle,&caCert->derSubject, SEC_CRL_TYPE);
-    if (crl == NULL) {
-	/* XXX for now no CRL is ok */
-	goto done;
-    }
-
-    /* now verify the CRL signature */
-    rv = CERT_VerifySignedData(&crl->signatureWrap, caCert, t, wincx);
-    if (rv != SECSuccess) {
-	PORT_SetError(SEC_ERROR_CRL_BAD_SIGNATURE);
-        rv = SECWouldBlock; /* Soft error, ask the user */
-    	goto done;
-    }
-
-    /* Verify the date validity of the KRL */
-    validity = SEC_CheckCrlTimes(&crl->crl,t);
-    if (validity == secCertTimeExpired) {
-	PORT_SetError(SEC_ERROR_CRL_EXPIRED);
-        rv = SECWouldBlock; /* Soft error, ask the user */
-    } else if (validity == secCertTimeNotValidYet) {
-	PORT_SetError(SEC_ERROR_CRL_NOT_YET_VALID);
-	rv = SECWouldBlock; /* Soft error, ask the user */
-    }
-
-    /* now make sure the key is not on the revocation list */
-    for (crlEntry = crl->crl.entries; crlEntry && *crlEntry; crlEntry++) {
-	if (SECITEM_CompareItem(&(*crlEntry)->serialNumber,&cert->serialNumber) == SECEqual) {
-	    PORT_SetError(SEC_ERROR_REVOKED_CERTIFICATE);
-	    rv = SECFailure; /* cert is revoked */
-	    goto done;
-	}
-    }
-
-done:
-    if (crl) SEC_DestroyCrl(crl);
-    return rv;
+    return CERT_CheckCRL(cert, caCert, NULL, t, wincx);
 }
 
 /*
@@ -308,12 +276,11 @@ done:
 CERTCertificate *
 CERT_FindCertIssuer(CERTCertificate *cert, int64 validTime, SECCertUsage usage)
 {
+#ifdef NSS_CLASSIC
     CERTAuthKeyID *   authorityKeyID = NULL;  
     CERTCertificate * issuerCert     = NULL;
     SECItem *         caName;
     PRArenaPool       *tmpArena = NULL;
-    SECItem           issuerCertKey;
-    SECStatus         rv;
 
     tmpArena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     
@@ -351,14 +318,16 @@ CERT_FindCertIssuer(CERTCertificate *cert, int64 validTime, SECCertUsage usage)
 	     */
 
 	    if (caName != NULL) {
-		rv = CERT_KeyFromIssuerAndSN(tmpArena, caName,
-					     &authorityKeyID->authCertSerialNumber,
-					     &issuerCertKey);
-		if ( rv == SECSuccess ) {
-		    issuerCert = CERT_FindCertByKey(cert->dbhandle,
-						    &issuerCertKey);
-		}
-		
+		CERTIssuerAndSN issuerSN;
+
+		issuerSN.derIssuer.data = caName->data;
+		issuerSN.derIssuer.len = caName->len;
+		issuerSN.serialNumber.data = 
+			authorityKeyID->authCertSerialNumber.data;
+		issuerSN.serialNumber.len = 
+			authorityKeyID->authCertSerialNumber.len;
+		issuerCert = CERT_FindCertByIssuerAndSN(cert->dbhandle,
+								&issuerSN);
 		if ( issuerCert == NULL ) {
 		    PORT_SetError (SEC_ERROR_UNKNOWN_ISSUER);
 		    goto loser;
@@ -390,6 +359,39 @@ loser:
     }
 
     return(issuerCert);
+#else
+    NSSCertificate *me;
+    NSSTime *nssTime;
+    NSSUsage nssUsage;
+    NSSCertificate *chain[3];
+    PRStatus status;
+    me = STAN_GetNSSCertificate(cert);
+    nssTime = NSSTime_SetPRTime(NULL, validTime);
+    nssUsage.anyUsage = PR_FALSE;
+    nssUsage.nss3usage = usage;
+    nssUsage.nss3lookingForCA = PR_TRUE;
+    memset(chain, 0, 3*sizeof(NSSCertificate *));
+    if (!me) {
+	PORT_SetError (SEC_ERROR_BAD_DATABASE);
+	return NULL;
+    }
+    (void)NSSCertificate_BuildChain(me, nssTime, &nssUsage, NULL, 
+                                    chain, 2, NULL, &status);
+    nss_ZFreeIf(nssTime);
+    if (status == PR_SUCCESS) {
+	/* if it's a root, the chain will only have one cert */
+	if (!chain[1]) {
+	    /* already has a reference from the call to BuildChain */
+	    return cert;
+	} else {
+	    CERT_DestroyCertificate(cert); /* the first cert in the chain */
+	    return STAN_GetCERTCertificate(chain[1]); /* return the 2nd */
+	}
+    } else {
+	PORT_SetError (SEC_ERROR_UNKNOWN_ISSUER);
+    }
+    return NULL;
+#endif
 }
 
 /*
@@ -525,12 +527,89 @@ AddToVerifyLog(CERTVerifyLog *log, CERTCertificate *cert, unsigned long error,
     }
 
 
+typedef enum { cbd_None, cbd_User, cbd_CA } cbd_FortezzaType;
+
+static SECStatus
+cert_VerifyFortezzaV1Cert(CERTCertDBHandle *handle, CERTCertificate *cert,
+	cbd_FortezzaType *next_type, cbd_FortezzaType last_type,
+	int64 t, void *wincx)
+{
+    unsigned char priv = 0;
+    SECKEYPublicKey *key;
+    SECStatus rv;
+
+    *next_type = cbd_CA;
+
+    /* read the key */
+    key = CERT_ExtractPublicKey(cert);
+
+    /* Cant' get Key? fail. */
+    if (key == NULL) {
+    	PORT_SetError(SEC_ERROR_BAD_KEY);
+	return SECFailure;
+    }
 
 
-SECStatus
-CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
-		     PRBool checkSig, SECCertUsage certUsage, int64 t,
-		     void *wincx, CERTVerifyLog *log)
+    /* if the issuer is not an old fortezza cert, we bail */
+    if (key->keyType != fortezzaKey) {
+    	SECKEY_DestroyPublicKey(key);
+	/* CA Cert not fortezza */
+    	PORT_SetError(SEC_ERROR_NOT_FORTEZZA_ISSUER);
+	return SECFailure;
+    }
+
+    /* get the privilege mask */
+    if (key->u.fortezza.DSSpriviledge.len > 0) {
+	priv = key->u.fortezza.DSSpriviledge.data[0];
+    }
+
+    /*
+     * make sure the CA's keys are OK
+     */
+            
+    rv = SEC_CheckKRL(handle, key, NULL, t, wincx);
+    SECKEY_DestroyPublicKey(key);
+    if (rv != SECSuccess) {
+	return rv;
+    }
+
+    switch (last_type) {
+      case cbd_User:
+	/* first check for subordination */
+	/*rv = FortezzaSubordinateCheck(cert,issuerCert);*/
+	rv = SECSuccess;
+
+	/* now check for issuer privilege */
+	if ((rv != SECSuccess) || ((priv & 0x10) == 0)) {
+	    /* bail */
+	    PORT_SetError (SEC_ERROR_CA_CERT_INVALID);
+	    return SECFailure;
+	}
+	break;
+      case cbd_CA:
+	if ((priv & 0x20) == 0) {
+	    /* bail */
+	    PORT_SetError (SEC_ERROR_CA_CERT_INVALID);
+	    return SECFailure;
+	}
+	break;
+      case cbd_None:
+	*next_type = (priv & 0x30) ? cbd_CA : cbd_User;
+	break;
+      default:
+	/* bail */ /* shouldn't ever happen */
+    	PORT_SetError(SEC_ERROR_UNKNOWN_ISSUER);
+	return SECFailure;
+    }
+    return SECSuccess;
+}
+
+
+static SECStatus
+cert_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
+		     PRBool checkSig, PRBool* sigerror,
+                     SECCertUsage certUsage, int64 t, void *wincx,
+                     CERTVerifyLog *log, PRBool doCRL, PRBool* revoked)
 {
     SECTrustType trustType;
     CERTBasicConstraints basicConstraint;
@@ -540,7 +619,6 @@ CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
     PRBool isca;
     PRBool isFortezzaV1 = PR_FALSE;
     SECStatus rv;
-    SECComparison rvCompare;
     SECStatus rvFinal = SECSuccess;
     int count;
     int currentPathLen = -1;
@@ -555,8 +633,11 @@ CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
     int namesIndexLen = 10;
     int namesCount = 0;
 
-    enum { cbd_None, cbd_User, cbd_CA } last_type = cbd_None;
-    SECKEYPublicKey *key;
+    cbd_FortezzaType last_type = cbd_None;
+
+    if (revoked) {
+        *revoked = PR_FALSE;
+    }
 
     if (CERT_KeyUsageAndTypeForCertUsage(certUsage, PR_TRUE,
 					 &requiredCAKeyUsage,
@@ -601,36 +682,19 @@ CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
 	goto loser;
     }
 
-    /* determine if the cert is fortezza. Getting the key is an easy
-     * way to determine it, especially since we need to get the privillege
-     * from the key anyway.
+    /* determine if the cert is fortezza.
      */
-    key = CERT_ExtractPublicKey(cert);
+    isFortezzaV1 = (PRBool)
+	(CERT_GetCertKeyType(&subjectCert->subjectPublicKeyInfo) 
+							== fortezzaKey);
 
-    if (key != NULL) {
-	isFortezzaV1 = (PRBool)(key->keyType == fortezzaKey);
-
-	/* find out what type of cert we are starting with */
-	if (isFortezzaV1) {
-	    unsigned char priv = 0;;
-
-	    rv = SEC_CheckKRL(handle, key, NULL, t, wincx);
-	    if (rv == SECFailure) {
-		/**** PORT_SetError is already set by SEC_CheckKRL **/
-		SECKEY_DestroyPublicKey(key);
-		/**** Bob - should we log and continue when logging? **/
-		LOG_ERROR(log,subjectCert,0,0);
-		goto loser;
-	    }                
-
-	    if (key->u.fortezza.DSSpriviledge.len > 0) {
-		priv = key->u.fortezza.DSSpriviledge.data[0];
-	    }
-
-	    last_type = (priv & 0x30) ? cbd_CA : cbd_User;
+    if (isFortezzaV1) {
+	rv = cert_VerifyFortezzaV1Cert(handle, subjectCert, &last_type, 
+						cbd_None, t, wincx);
+	if (rv == SECFailure) {
+	    /**** PORT_SetError is already set by cert_VerifyFortezzaV1Cert **/
+	    LOG_ERROR_OR_EXIT(log,subjectCert,0,0);
 	}
-		
-	SECKEY_DestroyPublicKey(key);
     }
 
     arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
@@ -653,7 +717,7 @@ CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
 	subjectNameList = CERT_GetCertificateNames(subjectCert, arena);
 	subjectNameListLen = CERT_GetNamesLength(subjectNameList);
 	for (i = 0; i < subjectNameListLen; i++) {
-	    if (namesIndexLen < namesCount + i) {
+	    if (namesIndexLen <= namesCount + i) {
 		namesIndexLen = namesIndexLen * 2;
 		namesIndex = (SECItem *) PORT_Realloc(namesIndex, namesIndexLen * 
 						       sizeof(SECItem));
@@ -680,6 +744,9 @@ CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
 				       issuerCert, t, wincx);
     
 	    if ( rv != SECSuccess ) {
+                if (sigerror) {
+                    *sigerror = PR_TRUE;
+                }
 		if ( PORT_GetError() == SEC_ERROR_EXPIRED_CERTIFICATE ) {
 		    PORT_SetError(SEC_ERROR_EXPIRED_ISSUER_CERTIFICATE);
 		    LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
@@ -694,76 +761,13 @@ CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
 	 * XXX - fortezza may need error logging stuff added
 	 */
 	if (isFortezzaV1) {
-	    unsigned char priv = 0;
-
-	    /* read the key */
-	    key = CERT_ExtractPublicKey(issuerCert);
-
-	    /* Cant' get Key? fail. */
-	    if (key == NULL) {
-	    	PORT_SetError(SEC_ERROR_BAD_KEY);
-		LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
-		goto fortezzaDone;
+	    rv = cert_VerifyFortezzaV1Cert(handle, issuerCert, &last_type, 
+					last_type, t, wincx);
+	    if (rv == SECFailure) {
+		/**** PORT_SetError is already set by *
+		 * cert_VerifyFortezzaV1Cert **/
+		LOG_ERROR_OR_EXIT(log,subjectCert,0,0);
 	    }
-
-
-	    /* if the issuer is not an old fortezza cert, we bail */
-	    if (key->keyType != fortezzaKey) {
-	    	SECKEY_DestroyPublicKey(key);
-		/* CA Cert not fortezza */
-	    	PORT_SetError(SEC_ERROR_NOT_FORTEZZA_ISSUER);
-		LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
-		goto fortezzaDone;
-	    }
-
-	    /* get the privilege mask */
-	    if (key->u.fortezza.DSSpriviledge.len > 0) {
-		priv = key->u.fortezza.DSSpriviledge.data[0];
-	    }
-
-	    /*
-	     * make sure the CA's keys are OK
-	     */
-            
-	    rv = SEC_CheckKRL(handle, key, NULL, t, wincx);
-	    if (rv != SECSuccess) {
-	    	SECKEY_DestroyPublicKey(key);
-		LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
-		goto fortezzaDone;
-		/** fall through looking for more stuff **/
-	    } else {
-	        SECKEY_DestroyPublicKey(key);
-	    }
-
-	    switch (last_type) {
-	      case cbd_User:
-		/* first check for subordination */
-		/*rv = FortezzaSubordinateCheck(cert,issuerCert);*/
-		rv = SECSuccess;
-
-		/* now check for issuer privilege */
-		if ((rv != SECSuccess) || ((priv & 0x10) == 0)) {
-		    /* bail */
-		    PORT_SetError (SEC_ERROR_CA_CERT_INVALID);
-		    LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
-		}
-		break;
-	      case cbd_CA:
-	      case cbd_None:
-		if ((priv & 0x20) == 0) {
-		    /* bail */
-		    PORT_SetError (SEC_ERROR_CA_CERT_INVALID);
-		    LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
-		}
-		break;
-	      default:
-		/* bail */ /* shouldn't ever happen */
-	    	PORT_SetError(SEC_ERROR_UNKNOWN_ISSUER);
-		LOG_ERROR_OR_EXIT(log,issuerCert,count+1,0);
-	    }
-
-fortezzaDone:
-	    last_type =  cbd_CA;
 	}
 
 	/* If the basicConstraint extension is included in an immediate CA
@@ -823,17 +827,25 @@ fortezzaDone:
 	 * point
 	 */
 	/* check revoked list (issuer) */
-	rv = SEC_CheckCRL(handle, subjectCert, issuerCert, t, wincx);
-	if (rv == SECFailure) {
-	    LOG_ERROR_OR_EXIT(log,subjectCert,count,0);
-	} else if (rv == SECWouldBlock) {
-	    /* We found something fishy, so we intend to issue an
-	     * error to the user, but the user may wish to continue
-	     * processing, in which case we better make sure nothing
-	     * worse has happened... so keep cranking the loop */
-	    rvFinal = SECFailure;
-	    LOG_ERROR(log,subjectCert,count,0);
-	}
+        if (PR_TRUE == doCRL) {
+            rv = SEC_CheckCRL(handle, subjectCert, issuerCert, t, wincx);
+            if (rv == SECFailure) {
+                if (revoked) {
+                    *revoked = PR_TRUE;
+                }
+                LOG_ERROR_OR_EXIT(log,subjectCert,count,0);
+            } else if (rv == SECWouldBlock) {
+                /* We found something fishy, so we intend to issue an
+                 * error to the user, but the user may wish to continue
+                 * processing, in which case we better make sure nothing
+                 * worse has happened... so keep cranking the loop */
+                rvFinal = SECFailure;
+                if (revoked) {
+                    *revoked = PR_TRUE;
+                }
+                LOG_ERROR(log,subjectCert,count,0);
+            }
+        }
 
 
 	if ( issuerCert->trust ) {
@@ -911,9 +923,7 @@ fortezzaDone:
 	/* make sure that the issuer is not self signed.  If it is, then
 	 * stop here to prevent looping.
 	 */
-	rvCompare = SECITEM_CompareItem(&issuerCert->derSubject,
-				 &issuerCert->derIssuer);
-	if (rvCompare == SECEqual) {
+	if (issuerCert->isRoot) {
 	    PORT_SetError(SEC_ERROR_UNTRUSTED_ISSUER);
 	    LOG_ERROR(log, issuerCert, count+1, 0);
 	    goto loser;
@@ -945,13 +955,471 @@ done:
     }
     return rv;
 }
-			
+
+SECStatus
+CERT_VerifyCertChain(CERTCertDBHandle *handle, CERTCertificate *cert,
+		     PRBool checkSig, SECCertUsage certUsage, int64 t,
+		     void *wincx, CERTVerifyLog *log)
+{
+    return cert_VerifyCertChain(handle, cert, checkSig, NULL, certUsage, t,
+			 wincx, log, PR_TRUE, NULL);
+}
+
 /*
- * verify a certificate by checking if its valid and that we
- * trust the issuer.
- * Note that this routine does not verify the signature of the certificate.
+ * verify that a CA can sign a certificate with the requested usage.
  */
 SECStatus
+CERT_VerifyCACertForUsage(CERTCertDBHandle *handle, CERTCertificate *cert,
+		PRBool checkSig, SECCertUsage certUsage, int64 t,
+		void *wincx, CERTVerifyLog *log)
+{
+    SECTrustType trustType;
+    CERTBasicConstraints basicConstraint;
+    PRBool isca;
+    SECStatus rv;
+    SECComparison rvCompare;
+    SECStatus rvFinal = SECSuccess;
+    int flags;
+    unsigned int caCertType;
+    unsigned int requiredCAKeyUsage;
+    unsigned int requiredFlags;
+    CERTCertificate *issuerCert;
+
+
+    if (CERT_KeyUsageAndTypeForCertUsage(certUsage, PR_TRUE,
+					 &requiredCAKeyUsage,
+					 &caCertType) != SECSuccess ) {
+	PORT_Assert(0);
+	EXIT_IF_NOT_LOGGING(log);
+	requiredCAKeyUsage = 0;
+	caCertType = 0;
+    }
+
+    switch ( certUsage ) {
+      case certUsageSSLClient:
+      case certUsageSSLServer:
+      case certUsageSSLCA:
+      case certUsageSSLServerWithStepUp:
+      case certUsageEmailSigner:
+      case certUsageEmailRecipient:
+      case certUsageObjectSigner:
+      case certUsageVerifyCA:
+      case certUsageStatusResponder:
+	if ( CERT_TrustFlagsForCACertUsage(certUsage, &requiredFlags,
+					   &trustType) != SECSuccess ) {
+	    PORT_Assert(0);
+	    EXIT_IF_NOT_LOGGING(log);
+	    requiredFlags = 0;
+	    trustType = trustSSL;
+	}
+	break;
+      default:
+	PORT_Assert(0);
+	EXIT_IF_NOT_LOGGING(log);
+	requiredFlags = 0;
+	trustType = trustSSL;/* This used to be 0, but we need something
+			      * that matches the enumeration type.
+			      */
+	caCertType = 0;
+    }
+    
+    /* If the basicConstraint extension is included in an immediate CA
+     * certificate, make sure that the isCA flag is on.  If the
+     * pathLenConstraint component exists, it must be greater than the
+     * number of CA certificates we have seen so far.  If the extension
+     * is omitted, we will assume that this is a CA certificate with
+     * an unlimited pathLenConstraint (since it already passes the
+     * netscape-cert-type extension checking).
+     *
+     * In the fortezza (V1) case, we've already checked the CA bits
+     * in the key, so we're presumed to be a CA; however we really don't
+     * want to bypass Basic constraint or netscape extension parsing.
+     * 
+     * In Fortezza V2, basicConstraint will be set for every CA,PCA,PAA
+     */
+
+    rv = CERT_FindBasicConstraintExten(cert, &basicConstraint);
+    if ( rv != SECSuccess ) {
+	if (PORT_GetError() != SEC_ERROR_EXTENSION_NOT_FOUND) {
+	    LOG_ERROR_OR_EXIT(log,cert,0,0);
+	} 
+	/* no basic constraints found, if we're fortezza, CA bit is already
+	 * verified (isca = PR_TRUE). otherwise, we aren't (yet) a ca
+	 * isca = PR_FALSE */
+	isca = PR_FALSE;
+    } else  {
+	if ( basicConstraint.isCA == PR_FALSE ) {
+	    PORT_SetError (SEC_ERROR_CA_CERT_INVALID);
+	    LOG_ERROR_OR_EXIT(log,cert,0,0);
+	}
+
+	/* can't check path length if we don't know the previous path */
+	isca = PR_TRUE;
+    }
+	
+    if ( cert->trust ) {
+	/*
+	 * check the trust parms of the issuer
+	 */
+	flags = SEC_GET_TRUST_FLAGS(cert->trust, trustType);
+	    
+	if ( (flags & CERTDB_VALID_CA) ||
+		 (certUsage == certUsageStatusResponder)) {
+	    if ( ( flags & requiredFlags ) == requiredFlags ||
+		     certUsage == certUsageStatusResponder ) {
+		    /* we found a trusted one, so return */
+        /* Check  the special case of certUsageStatusResponder */
+        if(certUsage == certUsageStatusResponder) {
+           issuerCert = CERT_FindCertIssuer(cert, t, certUsage);
+	         if (issuerCert) {
+              if(SEC_CheckCRL(handle, cert, issuerCert, t, wincx) != SECSuccess) {
+                 PORT_SetError(SEC_ERROR_REVOKED_CERTIFICATE);
+                 CERT_DestroyCertificate(issuerCert);
+                 goto loser;
+              }
+              CERT_DestroyCertificate(issuerCert);
+           }
+        }
+		    rv = rvFinal; 
+		    goto done;
+	    }
+	}
+    }
+
+    /*
+     * Make sure that if this is an intermediate CA in the chain that
+     * it was given permission by its signer to be a CA.
+     */
+    if ( isca ) {
+	/*
+	 * if basicConstraints says it is a ca, then we check the
+	 * nsCertType.  If the nsCertType has any CA bits set, then
+	 * it must have the right one.
+	 */
+	if ( cert->nsCertType & NS_CERT_TYPE_CA ) {
+	    if ( cert->nsCertType & caCertType ) {
+		isca = PR_TRUE;
+	    } else {
+		isca = PR_FALSE;
+	    }
+	}
+    } else {
+	if ( cert->nsCertType & caCertType ) {
+	    isca = PR_TRUE;
+	} else {
+	    isca = PR_FALSE;
+	}
+    }
+	
+    if (  !isca  ) {
+	PORT_SetError(SEC_ERROR_CA_CERT_INVALID);
+	LOG_ERROR_OR_EXIT(log,cert,0,0);
+    }
+	    
+    /* make sure key usage allows cert signing */
+    if (CERT_CheckKeyUsage(cert, requiredCAKeyUsage) != SECSuccess) {
+	    PORT_SetError(SEC_ERROR_INADEQUATE_KEY_USAGE);
+	    LOG_ERROR_OR_EXIT(log,cert,0,requiredCAKeyUsage);
+    }
+    /* make sure that the issuer is not self signed.  If it is, then
+     * stop here to prevent looping.
+     */
+    rvCompare = SECITEM_CompareItem(&cert->derSubject,
+				 &cert->derIssuer);
+    if (rvCompare == SECEqual) {
+	    PORT_SetError(SEC_ERROR_UNTRUSTED_ISSUER);
+	    LOG_ERROR(log, cert, 0, 0);
+	    goto loser;
+    }
+
+    return CERT_VerifyCertChain(handle, cert, checkSig, certUsage, t, 
+		     					wincx, log);
+loser:
+    rv = SECFailure;
+done:
+    return rv;
+}
+
+#define NEXT_ITERATION() { \
+    i*=2; \
+    certUsage++; \
+    continue; \
+}
+
+#define VALID_USAGE() { \
+    NEXT_ITERATION(); \
+}
+
+#define INVALID_USAGE() { \
+    if (returnedUsages) { \
+        *returnedUsages &= (~i); \
+    } \
+    if (PR_TRUE == requiredUsage) { \
+        valid = SECFailure; \
+    } \
+    NEXT_ITERATION(); \
+}
+
+/*
+ * verify a certificate by checking if it's valid and that we
+ * trust the issuer.
+ *
+ * certificateUsage contains a bitfield of all cert usages that are
+ * required for verification to succeed
+ *
+ * a bitfield of cert usages is returned in *returnedUsages
+ * if requiredUsages is non-zero, the returned bitmap is only
+ * for those required usages, otherwise it is for all usages
+ *
+ */
+SECStatus
+CERT_VerifyCertificate(CERTCertDBHandle *handle, CERTCertificate *cert,
+		PRBool checkSig, SECCertificateUsage requiredUsages, int64 t,
+		void *wincx, CERTVerifyLog *log, SECCertificateUsage* returnedUsages)
+{
+    SECStatus rv;
+    SECStatus valid;
+    unsigned int requiredKeyUsage;
+    unsigned int requiredCertType;
+    unsigned int flags;
+    unsigned int certType;
+    PRBool       allowOverride;
+    SECCertTimeValidity validity;
+    CERTStatusConfig *statusConfig;
+    PRBool checkedChain = PR_FALSE;
+    PRInt32 i;
+    SECCertUsage certUsage = 0;
+    PRBool checkedOCSP = PR_FALSE;
+    PRBool checkAllUsages = PR_FALSE;
+    PRBool revoked = PR_FALSE;
+    PRBool sigerror = PR_FALSE;
+
+    if (!requiredUsages) {
+        /* there are no required usages, so the user probably wants to
+           get status for all usages */
+        checkAllUsages = PR_TRUE;
+    }
+
+    if (returnedUsages) {
+        *returnedUsages = 0;
+    } else {
+        /* we don't have a place to return status for all usages,
+           so we can skip checks for usages that aren't required */
+        checkAllUsages = PR_FALSE;
+    }
+    valid = SECSuccess ; /* start off assuming cert is valid */
+   
+#ifdef notdef 
+    /* check if this cert is in the Evil list */
+    rv = CERT_CheckForEvilCert(cert);
+    if ( rv != SECSuccess ) {
+	PORT_SetError(SEC_ERROR_REVOKED_CERTIFICATE);
+	LOG_ERROR(log,cert,0,0);
+	return SECFailure;
+    }
+#endif
+    
+    /* make sure that the cert is valid at time t */
+    allowOverride = (PRBool)((requiredUsages & certUsageSSLServer) ||
+                             (requiredUsages & certificateUsageSSLServerWithStepUp));
+    validity = CERT_CheckCertValidTimes(cert, t, allowOverride);
+    if ( validity != secCertTimeValid ) {
+        LOG_ERROR(log,cert,0,validity);
+	return SECFailure;
+    }
+
+    /* check key usage and netscape cert type */
+    CERT_GetCertType(cert);
+    certType = cert->nsCertType;
+
+    for (i=1;i<=certificateUsageHighest && !(SECFailure == valid && !returnedUsages) ;) {
+        PRBool requiredUsage = (i & requiredUsages) ? PR_TRUE : PR_FALSE;
+        if (PR_FALSE == requiredUsage && PR_FALSE == checkAllUsages) {
+            NEXT_ITERATION();
+        }
+        if (returnedUsages) {
+            *returnedUsages |= i; /* start off assuming this usage is valid */
+        }
+        switch ( certUsage ) {
+          case certUsageSSLClient:
+          case certUsageSSLServer:
+          case certUsageSSLServerWithStepUp:
+          case certUsageSSLCA:
+          case certUsageEmailSigner:
+          case certUsageEmailRecipient:
+          case certUsageObjectSigner:
+          case certUsageStatusResponder:
+            rv = CERT_KeyUsageAndTypeForCertUsage(certUsage, PR_FALSE,
+                                                  &requiredKeyUsage,
+                                                  &requiredCertType);
+            if ( rv != SECSuccess ) {
+                PORT_Assert(0);
+                /* EXIT_IF_NOT_LOGGING(log); XXX ??? */
+                requiredKeyUsage = 0;
+                requiredCertType = 0;
+                INVALID_USAGE();
+            }
+            break;
+
+          case certUsageAnyCA:
+          case certUsageProtectedObjectSigner:
+          case certUsageUserCertImport:
+          case certUsageVerifyCA:
+              /* these usages cannot be verified */
+              NEXT_ITERATION();
+
+          default:
+            PORT_Assert(0);
+            requiredKeyUsage = 0;
+            requiredCertType = 0;
+            INVALID_USAGE();
+        }
+        if ( CERT_CheckKeyUsage(cert, requiredKeyUsage) != SECSuccess ) {
+            if (PR_TRUE == requiredUsage) {
+                PORT_SetError(SEC_ERROR_INADEQUATE_KEY_USAGE);
+            }
+            LOG_ERROR(log,cert,0,requiredKeyUsage);
+            INVALID_USAGE();
+        }
+        if ( !( certType & requiredCertType ) ) {
+            if (PR_TRUE == requiredUsage) {
+                PORT_SetError(SEC_ERROR_INADEQUATE_CERT_TYPE);
+            }
+            LOG_ERROR(log,cert,0,requiredCertType);
+            INVALID_USAGE();
+        }
+
+        /* check trust flags to see if this cert is directly trusted */
+        if ( cert->trust ) { /* the cert is in the DB */
+            switch ( certUsage ) {
+              case certUsageSSLClient:
+              case certUsageSSLServer:
+                flags = cert->trust->sslFlags;
+
+                /* is the cert directly trusted or not trusted ? */
+                if ( flags & CERTDB_VALID_PEER ) {/*the trust record is valid*/
+                    if ( flags & CERTDB_TRUSTED ) {	/* trust this cert */
+                        VALID_USAGE();
+                    } else { /* don't trust this cert */
+                        if (PR_TRUE == requiredUsage) {
+                            PORT_SetError(SEC_ERROR_UNTRUSTED_CERT);
+                        }
+                        LOG_ERROR(log,cert,0,flags);
+                        INVALID_USAGE();
+                    }
+                }
+                break;
+              case certUsageSSLServerWithStepUp:
+                /* XXX - step up certs can't be directly trusted */
+                break;
+              case certUsageSSLCA:
+                break;
+              case certUsageEmailSigner:
+              case certUsageEmailRecipient:
+                flags = cert->trust->emailFlags;
+
+                /* is the cert directly trusted or not trusted ? */
+                if ( ( flags & ( CERTDB_VALID_PEER | CERTDB_TRUSTED ) ) ==
+                    ( CERTDB_VALID_PEER | CERTDB_TRUSTED ) ) {
+                    VALID_USAGE();
+                }
+                break;
+              case certUsageObjectSigner:
+                flags = cert->trust->objectSigningFlags;
+
+                /* is the cert directly trusted or not trusted ? */
+                if ( flags & CERTDB_VALID_PEER ) {/*the trust record is valid*/
+                    if ( flags & CERTDB_TRUSTED ) {	/* trust this cert */
+                        VALID_USAGE();
+                    } else { /* don't trust this cert */
+                        if (PR_TRUE == requiredUsage) {
+                            PORT_SetError(SEC_ERROR_UNTRUSTED_CERT);
+                        }
+                        LOG_ERROR(log,cert,0,flags);
+                        INVALID_USAGE();
+                    }
+                }
+                break;
+              case certUsageVerifyCA:
+              case certUsageStatusResponder:
+                flags = cert->trust->sslFlags;
+                /* is the cert directly trusted or not trusted ? */
+                if ( ( flags & ( CERTDB_VALID_CA | CERTDB_TRUSTED_CA ) ) ==
+                    ( CERTDB_VALID_CA | CERTDB_TRUSTED_CA ) ) {
+                    VALID_USAGE();
+                }
+                flags = cert->trust->emailFlags;
+                /* is the cert directly trusted or not trusted ? */
+                if ( ( flags & ( CERTDB_VALID_CA | CERTDB_TRUSTED_CA ) ) ==
+                    ( CERTDB_VALID_CA | CERTDB_TRUSTED_CA ) ) {
+                    VALID_USAGE();
+                }
+                flags = cert->trust->objectSigningFlags;
+                /* is the cert directly trusted or not trusted ? */
+                if ( ( flags & ( CERTDB_VALID_CA | CERTDB_TRUSTED_CA ) ) ==
+                    ( CERTDB_VALID_CA | CERTDB_TRUSTED_CA ) ) {
+                    VALID_USAGE();
+                }
+                break;
+              case certUsageAnyCA:
+              case certUsageProtectedObjectSigner:
+              case certUsageUserCertImport:
+                /* XXX to make the compiler happy.  Should these be
+                 * explicitly handled?
+                 */
+                break;
+            }
+        }
+
+        if (PR_TRUE == revoked || PR_TRUE == sigerror) {
+            INVALID_USAGE();
+        }
+
+        /* only check CRL and signature for the first usage check */
+        rv = cert_VerifyCertChain(handle, cert,
+            (PR_TRUE == checkedChain) ? PR_FALSE : checkSig, &sigerror,
+            certUsage, t, wincx, log,
+            (PR_TRUE ==  checkedChain) ? PR_FALSE : PR_TRUE, &revoked);
+        checkedChain = PR_TRUE;
+
+        if (rv != SECSuccess) {
+            /* EXIT_IF_NOT_LOGGING(log); XXX ???? */
+            INVALID_USAGE();
+        }
+
+        /*
+         * Check OCSP revocation status, but only if the cert we are checking
+         * is not a status reponder itself.  We only do this in the case
+         * where we checked the cert chain (above); explicit trust "wins"
+         * (avoids status checking, just as it avoids CRL checking) by
+         * bypassing this code.
+         */
+
+        if (PR_FALSE == checkedOCSP) {
+            checkedOCSP = PR_TRUE; /* only check OCSP once */
+            statusConfig = CERT_GetStatusConfig(handle);
+            if ( (! (requiredUsages & certificateUsageStatusResponder)) &&
+                statusConfig != NULL) {
+                if (statusConfig->statusChecker != NULL) {
+                    rv = (* statusConfig->statusChecker)(handle, cert,
+                                                                 t, wincx);
+                    if (rv != SECSuccess) {
+                        LOG_ERROR(log,cert,0,0);
+                        revoked = PR_TRUE;
+                        INVALID_USAGE();
+                    }
+                }
+            }
+        }
+
+        NEXT_ITERATION();
+    }
+    
+    return(valid);
+}
+			
+/* obsolete, do not use for new code */
+SECStatus
 CERT_VerifyCert(CERTCertDBHandle *handle, CERTCertificate *cert,
 		PRBool checkSig, SECCertUsage certUsage, int64 t,
 		void *wincx, CERTVerifyLog *log)
@@ -964,13 +1432,15 @@ CERT_VerifyCert(CERTCertDBHandle *handle, CERTCertificate *cert,
     PRBool       allowOverride;
     SECCertTimeValidity validity;
     CERTStatusConfig *statusConfig;
-    
+   
+#ifdef notdef 
     /* check if this cert is in the Evil list */
     rv = CERT_CheckForEvilCert(cert);
     if ( rv != SECSuccess ) {
 	PORT_SetError(SEC_ERROR_REVOKED_CERTIFICATE);
 	LOG_ERROR_OR_EXIT(log,cert,0,0);
     }
+#endif
     
     /* make sure that the cert is valid at time t */
     allowOverride = (PRBool)((certUsage == certUsageSSLServer) ||
@@ -1138,6 +1608,16 @@ loser:
  * trust the issuer.  Verify time against now.
  */
 SECStatus
+CERT_VerifyCertificateNow(CERTCertDBHandle *handle, CERTCertificate *cert,
+		   PRBool checkSig, SECCertificateUsage requiredUsages,
+                   void *wincx, SECCertificateUsage* returnedUsages)
+{
+    return(CERT_VerifyCertificate(handle, cert, checkSig, 
+		   requiredUsages, PR_Now(), wincx, NULL, returnedUsages));
+}
+
+/* obsolete, do not use for new code */
+SECStatus
 CERT_VerifyCertNow(CERTCertDBHandle *handle, CERTCertificate *cert,
 		   PRBool checkSig, SECCertUsage certUsage, void *wincx)
 {
@@ -1145,6 +1625,7 @@ CERT_VerifyCertNow(CERTCertDBHandle *handle, CERTCertificate *cert,
 		   certUsage, PR_Now(), wincx, NULL));
 }
 
+
 /* [ FROM pcertdb.c ] */
 /*
  * Supported usage values and types:
