@@ -77,10 +77,10 @@ const SEC_ASN1Template SECKEY_DSAPublicKeyTemplate[] = {
 };
 
 const SEC_ASN1Template SECKEY_PQGParamsTemplate[] = {
-    { SEC_ASN1_SEQUENCE, 0, NULL, sizeof(PQGParams) },
-    { SEC_ASN1_INTEGER, offsetof(PQGParams,prime) },
-    { SEC_ASN1_INTEGER, offsetof(PQGParams,subPrime) },
-    { SEC_ASN1_INTEGER, offsetof(PQGParams,base) },
+    { SEC_ASN1_SEQUENCE, 0, NULL, sizeof(SECKEYPQGParams) },
+    { SEC_ASN1_INTEGER, offsetof(SECKEYPQGParams,prime) },
+    { SEC_ASN1_INTEGER, offsetof(SECKEYPQGParams,subPrime) },
+    { SEC_ASN1_INTEGER, offsetof(SECKEYPQGParams,base) },
     { 0, }
 };
 
@@ -99,32 +99,32 @@ const SEC_ASN1Template SECKEY_DHParamKeyTemplate[] = {
 };
 
 const SEC_ASN1Template SECKEY_FortezzaParameterTemplate[] = {
-    { SEC_ASN1_SEQUENCE,  0, NULL, sizeof(PQGParams) },
-    { SEC_ASN1_OCTET_STRING, offsetof(PQGParams,prime), },
-    { SEC_ASN1_OCTET_STRING, offsetof(PQGParams,subPrime), },
-    { SEC_ASN1_OCTET_STRING, offsetof(PQGParams,base), },
+    { SEC_ASN1_SEQUENCE,  0, NULL, sizeof(SECKEYPQGParams) },
+    { SEC_ASN1_OCTET_STRING, offsetof(SECKEYPQGParams,prime), },
+    { SEC_ASN1_OCTET_STRING, offsetof(SECKEYPQGParams,subPrime), },
+    { SEC_ASN1_OCTET_STRING, offsetof(SECKEYPQGParams,base), },
     { 0 },
 };
  
 const SEC_ASN1Template SECKEY_FortezzaDiffParameterTemplate[] = {
-    { SEC_ASN1_SEQUENCE, 0, NULL, sizeof(DiffPQGParams) },
-    { SEC_ASN1_INLINE, offsetof(DiffPQGParams,DiffKEAParams), 
+    { SEC_ASN1_SEQUENCE, 0, NULL, sizeof(SECKEYDiffPQGParams) },
+    { SEC_ASN1_INLINE, offsetof(SECKEYDiffPQGParams,DiffKEAParams), 
                        SECKEY_FortezzaParameterTemplate},
-    { SEC_ASN1_INLINE, offsetof(DiffPQGParams,DiffDSAParams), 
+    { SEC_ASN1_INLINE, offsetof(SECKEYDiffPQGParams,DiffDSAParams), 
                        SECKEY_FortezzaParameterTemplate},
     { 0 },
 };
 
 const SEC_ASN1Template SECKEY_FortezzaPreParamTemplate[] = {
     { SEC_ASN1_EXPLICIT | SEC_ASN1_CONSTRUCTED |
-      SEC_ASN1_CONTEXT_SPECIFIC | 1, offsetof(PQGDualParams,CommParams),
+      SEC_ASN1_CONTEXT_SPECIFIC | 1, offsetof(SECKEYPQGDualParams,CommParams),
                 SECKEY_FortezzaParameterTemplate},
     { 0, }
 };
 
 const SEC_ASN1Template SECKEY_FortezzaAltPreParamTemplate[] = {
     { SEC_ASN1_EXPLICIT | SEC_ASN1_CONSTRUCTED |
-      SEC_ASN1_CONTEXT_SPECIFIC | 0, offsetof(PQGDualParams,DiffParams),
+      SEC_ASN1_CONTEXT_SPECIFIC | 0, offsetof(SECKEYPQGDualParams,DiffParams),
                 SECKEY_FortezzaDiffParameterTemplate},
     { 0, }
 };
@@ -139,11 +139,57 @@ const SEC_ASN1Template SECKEY_KEAParamsTemplate[] = {
     { 0, }
 };
 
+SEC_ASN1_CHOOSER_IMPLEMENT(SECKEY_DSAPublicKeyTemplate)
+SEC_ASN1_CHOOSER_IMPLEMENT(SECKEY_RSAPublicKeyTemplate)
+SEC_ASN1_CHOOSER_IMPLEMENT(CERT_SubjectPublicKeyInfoTemplate)
 
 /*
- * NOTE: This only generates RSA Private Key's. If you need more,
- * We need to pass in some more params...
+ * See bugzilla bug 125359
+ * Since NSS (via PKCS#11) wants to handle big integers as unsigned ints,
+ * all of the templates above that en/decode into integers must be converted
+ * from ASN.1's signed integer type.  This is done by marking either the
+ * source or destination (encoding or decoding, respectively) type as
+ * siUnsignedInteger.
  */
+static void
+prepare_rsa_pub_key_for_asn1(SECKEYPublicKey *pubk)
+{
+    pubk->u.rsa.modulus.type = siUnsignedInteger;
+    pubk->u.rsa.publicExponent.type = siUnsignedInteger;
+}
+
+static void
+prepare_dsa_pub_key_for_asn1(SECKEYPublicKey *pubk)
+{
+    pubk->u.dsa.publicValue.type = siUnsignedInteger;
+}
+
+static void
+prepare_pqg_params_for_asn1(SECKEYPQGParams *params)
+{
+    params->prime.type = siUnsignedInteger;
+    params->subPrime.type = siUnsignedInteger;
+    params->base.type = siUnsignedInteger;
+}
+
+static void
+prepare_dh_pub_key_for_asn1(SECKEYPublicKey *pubk)
+{
+    pubk->u.dh.prime.type = siUnsignedInteger;
+    pubk->u.dh.base.type = siUnsignedInteger;
+    pubk->u.dh.publicValue.type = siUnsignedInteger;
+}
+
+static void
+prepare_kea_pub_key_for_asn1(SECKEYPublicKey *pubk)
+{
+    pubk->u.kea.publicValue.type = siUnsignedInteger;
+}
+
+/* Create an RSA key pair is any slot able to do so.
+** The created keys are "session" (temporary), not "token" (permanent), 
+** and they are "sensitive", which makes them costly to move to another token.
+*/
 SECKEYPrivateKey *
 SECKEY_CreateRSAPrivateKey(int keySizeInBits,SECKEYPublicKey **pubk, void *cx)
 {
@@ -160,14 +206,25 @@ SECKEY_CreateRSAPrivateKey(int keySizeInBits,SECKEYPublicKey **pubk, void *cx)
     return(privk);
 }
 
+/* Create a DH key pair in any slot able to do so, 
+** This is a "session" (temporary), not "token" (permanent) key. 
+** Because of the high probability that this key will need to be moved to
+** another token, and the high cost of moving "sensitive" keys, we attempt
+** to create this key pair without the "sensitive" attribute, but revert to 
+** creating a "sensitive" key if necessary.
+*/
 SECKEYPrivateKey *
-SECKEY_CreateDHPrivateKey(DHParams *param, SECKEYPublicKey **pubk, void *cx)
+SECKEY_CreateDHPrivateKey(SECKEYDHParams *param, SECKEYPublicKey **pubk, void *cx)
 {
     SECKEYPrivateKey *privk;
     PK11SlotInfo *slot = PK11_GetBestSlot(CKM_DH_PKCS_KEY_PAIR_GEN,cx);
-    
-    privk = PK11_GenerateKeyPair(slot,CKM_DH_PKCS_KEY_PAIR_GEN,param,pubk,
-					PR_FALSE, PR_TRUE, cx);
+
+    privk = PK11_GenerateKeyPair(slot, CKM_DH_PKCS_KEY_PAIR_GEN, param, 
+                                 pubk, PR_FALSE, PR_FALSE, cx);
+    if (!privk) 
+	privk = PK11_GenerateKeyPair(slot, CKM_DH_PKCS_KEY_PAIR_GEN, param, 
+	                             pubk, PR_FALSE, PR_TRUE, cx);
+
     PK11_FreeSlot(slot);
     return(privk);
 }
@@ -194,7 +251,9 @@ SECKEY_DestroyPublicKey(SECKEYPublicKey *pubk)
 {
     if (pubk) {
 	if (pubk->pkcs11Slot) {
-	    PK11_DestroyObject(pubk->pkcs11Slot,pubk->pkcs11ID);
+	    if (!PK11_IsPermObject(pubk->pkcs11Slot,pubk->pkcs11ID)) {
+		PK11_DestroyObject(pubk->pkcs11Slot,pubk->pkcs11ID);
+	    }
 	    PK11_FreeSlot(pubk->pkcs11Slot);
 	}
     	if (pubk->arena) {
@@ -218,7 +277,7 @@ SECKEY_CopySubjectPublicKeyInfo(PRArenaPool *arena,
 }
 
 SECStatus
-SECKEY_KEASetParams(KEAParams * params, SECKEYPublicKey * pubKey) {
+SECKEY_KEASetParams(SECKEYKEAParams * params, SECKEYPublicKey * pubKey) {
 
     if (pubKey->keyType == fortezzaKey) {
         /* the key is a fortezza V1 public key  */
@@ -249,16 +308,12 @@ SECKEY_KEAParamCompare(CERTCertificate *cert1,CERTCertificate *cert2)
 {
 
     SECStatus rv;
-    SECOidData *oid=NULL;
-    CERTSubjectPublicKeyInfo * subjectSpki=NULL;
-    CERTSubjectPublicKeyInfo * issuerSpki=NULL;
-    CERTCertificate *issuerCert = NULL;
 
     SECKEYPublicKey *pubKey1 = 0;
     SECKEYPublicKey *pubKey2 = 0;
 
-    KEAParams params1;
-    KEAParams params2;
+    SECKEYKEAParams params1;
+    SECKEYKEAParams params2;
 
 
     rv = SECFailure;
@@ -377,7 +432,8 @@ seckey_UpdateCertPQGChain(CERTCertificate * subjectCert, int count)
              (tag != SEC_OID_MISSI_DSS) &&               
              (tag != SEC_OID_ANSIX9_DSA_SIGNATURE) &&
              (tag != SEC_OID_ANSIX9_DSA_SIGNATURE_WITH_SHA1_DIGEST) &&
-             (tag != SEC_OID_BOGUS_DSA_SIGNATURE_WITH_SHA1_DIGEST) ) {
+             (tag != SEC_OID_BOGUS_DSA_SIGNATURE_WITH_SHA1_DIGEST) &&
+             (tag != SEC_OID_SDN702_DSA_SIGNATURE) ) {
             
             return SECSuccess;
         }
@@ -423,7 +479,8 @@ seckey_UpdateCertPQGChain(CERTCertificate * subjectCert, int count)
              (tag != SEC_OID_MISSI_DSS) &&               
              (tag != SEC_OID_ANSIX9_DSA_SIGNATURE) &&
              (tag != SEC_OID_ANSIX9_DSA_SIGNATURE_WITH_SHA1_DIGEST) &&
-             (tag != SEC_OID_BOGUS_DSA_SIGNATURE_WITH_SHA1_DIGEST) ) {
+             (tag != SEC_OID_BOGUS_DSA_SIGNATURE_WITH_SHA1_DIGEST) &&
+             (tag != SEC_OID_SDN702_DSA_SIGNATURE) ) {
             
             return SECFailure;
         }
@@ -476,7 +533,7 @@ SECStatus
 SECKEY_FortezzaDecodePQGtoOld(PRArenaPool *arena, SECKEYPublicKey *pubk,
                               SECItem *params) {
         SECStatus rv;
-	PQGDualParams dual_params;
+	SECKEYPQGDualParams dual_params;
 
     if (params == NULL) return SECFailure; 
     
@@ -494,6 +551,7 @@ SECKEY_FortezzaDecodePQGtoOld(PRArenaPool *arena, SECKEYPublicKey *pubk,
             /* PQG params are in the standard format */
 
 	    /* Store DSA PQG parameters */
+	    prepare_pqg_params_for_asn1(&pubk->u.fortezza.params);
             rv = SEC_ASN1DecodeItem(arena, &pubk->u.fortezza.params,
                               SECKEY_PQGParamsTemplate,
                               params);
@@ -597,7 +655,7 @@ SECKEY_FortezzaDecodePQGtoOld(PRArenaPool *arena, SECKEYPublicKey *pubk,
 SECStatus
 SECKEY_DSADecodePQG(PRArenaPool *arena, SECKEYPublicKey *pubk, SECItem *params) {
         SECStatus rv;
-	PQGDualParams dual_params;
+	SECKEYPQGDualParams dual_params;
 
     if (params == NULL) return SECFailure; 
     
@@ -613,6 +671,7 @@ SECKEY_DSADecodePQG(PRArenaPool *arena, SECKEYPublicKey *pubk, SECItem *params)
         (params->data[0] != 0xa0)) {
     
          /* PQG params are in the standard format */
+         prepare_pqg_params_for_asn1(&pubk->u.dsa.params);
          rv = SEC_ASN1DecodeItem(arena, &pubk->u.dsa.params,
                              SECKEY_PQGParamsTemplate,
                              params);
@@ -821,6 +880,7 @@ CERT_GetCertKeyType (CERTSubjectPublicKeyInfo *spki) {
 	break;
       case SEC_OID_X942_DIFFIE_HELMAN_KEY:
 	keyType = dhKey;
+	break;
       default:
 	keyType = nullKey;
     }
@@ -831,7 +891,7 @@ static SECKEYPublicKey *
 seckey_ExtractPublicKey(CERTSubjectPublicKeyInfo *spki)
 {
     SECKEYPublicKey *pubk;
-    SECItem os;
+    SECItem os, newOs, newParms;
     SECStatus rv;
     PRArenaPool *arena;
     SECOidTag tag;
@@ -848,7 +908,7 @@ seckey_ExtractPublicKey(CERTSubjectPublicKeyInfo *spki)
 
     pubk->arena = arena;
     pubk->pkcs11Slot = 0;
-    pubk->pkcs11ID = CK_INVALID_KEY;
+    pubk->pkcs11ID = CK_INVALID_HANDLE;
 
 
     /* Convert bit string length from bits to bytes */
@@ -856,17 +916,25 @@ seckey_ExtractPublicKey(CERTSubjectPublicKeyInfo *spki)
     DER_ConvertBitString (&os);
 
     tag = SECOID_GetAlgorithmTag(&spki->algorithm);
+
+    /* copy the DER into the arena, since Quick DER returns data that points
+       into the DER input, which may get freed by the caller */
+    rv = SECITEM_CopyItem(arena, &newOs, &os);
+    if ( rv == SECSuccess )
     switch ( tag ) {
       case SEC_OID_X500_RSA_ENCRYPTION:
       case SEC_OID_PKCS1_RSA_ENCRYPTION:
 	pubk->keyType = rsaKey;
-	rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_RSAPublicKeyTemplate, &os);
+	prepare_rsa_pub_key_for_asn1(pubk);        
+        rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_RSAPublicKeyTemplate, &newOs);
 	if (rv == SECSuccess)
 	    return pubk;
 	break;
       case SEC_OID_ANSIX9_DSA_SIGNATURE:
+      case SEC_OID_SDN702_DSA_SIGNATURE:
 	pubk->keyType = dsaKey;
-	rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_DSAPublicKeyTemplate, &os);
+	prepare_dsa_pub_key_for_asn1(pubk);
+	rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_DSAPublicKeyTemplate, &newOs);
 	if (rv != SECSuccess) break;
 
         rv = SECKEY_DSADecodePQG(arena, pubk,
@@ -876,11 +944,18 @@ seckey_ExtractPublicKey(CERTSubjectPublicKeyInfo *spki)
 	break;
       case SEC_OID_X942_DIFFIE_HELMAN_KEY:
 	pubk->keyType = dhKey;
-	rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_DHPublicKeyTemplate, &os);
+	prepare_dh_pub_key_for_asn1(pubk);
+	rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_DHPublicKeyTemplate, &newOs);
 	if (rv != SECSuccess) break;
 
-        rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_DHParamKeyTemplate,
-                                 &spki->algorithm.parameters); 
+        /* copy the DER into the arena, since Quick DER returns data that points
+           into the DER input, which may get freed by the caller */
+        rv = SECITEM_CopyItem(arena, &newParms, &spki->algorithm.parameters);
+        if ( rv != SECSuccess )
+            break;
+
+        rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_DHParamKeyTemplate,
+                                 &newParms); 
 
 	if (rv == SECSuccess) return pubk;
 	break;
@@ -889,7 +964,7 @@ seckey_ExtractPublicKey(CERTSubjectPublicKeyInfo *spki)
       case SEC_OID_MISSI_DSS_OLD:
       case SEC_OID_MISSI_DSS:
 	pubk->keyType = fortezzaKey;
-	rv = SECKEY_FortezzaDecodeCertKey(arena, pubk, &os,
+	rv = SECKEY_FortezzaDecodeCertKey(arena, pubk, &newOs,
 				          &spki->algorithm.parameters);
 	if (rv == SECSuccess)
 	    return pubk;
@@ -898,32 +973,44 @@ seckey_ExtractPublicKey(CERTSubjectPublicKeyInfo *spki)
       case SEC_OID_MISSI_KEA:
 	pubk->keyType = keaKey;
 
-        rv = SEC_ASN1DecodeItem(arena, pubk,
-                                SECKEY_KEAPublicKeyTemplate, &os);
+	prepare_kea_pub_key_for_asn1(pubk);
+        rv = SEC_QuickDERDecodeItem(arena, pubk,
+                                SECKEY_KEAPublicKeyTemplate, &newOs);
         if (rv != SECSuccess) break;
 
+        /* copy the DER into the arena, since Quick DER returns data that points
+           into the DER input, which may get freed by the caller */
+        rv = SECITEM_CopyItem(arena, &newParms, &spki->algorithm.parameters);
+        if ( rv != SECSuccess )
+            break;
 
-        rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_KEAParamsTemplate,
-                        &spki->algorithm.parameters);
+        rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_KEAParamsTemplate,
+                        &newParms);
 
 	if (rv == SECSuccess)
 	    return pubk;
 
-      break;
+        break;
 
       case SEC_OID_MISSI_ALT_KEA:
 	pubk->keyType = keaKey;
 
-        rv = SECITEM_CopyItem(arena,&pubk->u.kea.publicValue,&os);
+        rv = SECITEM_CopyItem(arena,&pubk->u.kea.publicValue,&newOs);
         if (rv != SECSuccess) break;
  
-        rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_KEAParamsTemplate,
-                        &spki->algorithm.parameters);
+        /* copy the DER into the arena, since Quick DER returns data that points
+           into the DER input, which may get freed by the caller */
+        rv = SECITEM_CopyItem(arena, &newParms, &spki->algorithm.parameters);
+        if ( rv != SECSuccess )
+            break;
+
+        rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_KEAParamsTemplate,
+                        &newParms);
 
 	if (rv == SECSuccess)
 	    return pubk;
 
-      break;
+        break;
 
 
       default:
@@ -1024,7 +1111,7 @@ SECKEY_CopyPrivateKey(SECKEYPrivateKey *privk)
 	if (privk->pkcs11IsTemp) {
 	    copyk->pkcs11ID = 
 			PK11_CopyKey(privk->pkcs11Slot,privk->pkcs11ID);
-	    if (copyk->pkcs11ID == CK_INVALID_KEY) goto fail;
+	    if (copyk->pkcs11ID == CK_INVALID_HANDLE) goto fail;
 	} else {
 	    copyk->pkcs11ID = privk->pkcs11ID;
 	}
@@ -1058,8 +1145,14 @@ SECKEY_CopyPublicKey(SECKEYPublicKey *pubk)
 
 	copyk->arena = arena;
 	copyk->keyType = pubk->keyType;
-	copyk->pkcs11Slot = NULL;	/* go get own reference */
-	copyk->pkcs11ID = CK_INVALID_KEY;
+	if (pubk->pkcs11Slot && 
+			PK11_IsPermObject(pubk->pkcs11Slot,pubk->pkcs11ID)) {
+	    copyk->pkcs11Slot = PK11_ReferenceSlot(pubk->pkcs11Slot);
+	    copyk->pkcs11ID = pubk->pkcs11ID;
+	} else {
+	    copyk->pkcs11Slot = NULL;	/* go get own reference */
+	    copyk->pkcs11ID = CK_INVALID_HANDLE;
+	}
 	switch (pubk->keyType) {
 	  case rsaKey:
 	    rv = SECITEM_CopyItem(arena, &copyk->u.rsa.modulus,
@@ -1188,7 +1281,7 @@ SECKEY_ConvertToPublicKey(SECKEYPrivateKey *privk)
     }
     pubk->keyType = privk->keyType;
     pubk->pkcs11Slot = NULL;
-    pubk->pkcs11ID = CK_INVALID_KEY;
+    pubk->pkcs11ID = CK_INVALID_HANDLE;
     pubk->arena = arena;
 
     /*
@@ -1247,6 +1340,7 @@ SECKEY_CreateSubjectPublicKeyInfo(SECKEYPublicKey *pubk)
 		/*
 		 * DER encode the public key into the subjectPublicKeyInfo.
 		 */
+		prepare_rsa_pub_key_for_asn1(pubk);
 		rv_item = SEC_ASN1EncodeItem(arena, &spki->subjectPublicKey,
 					     pubk, SECKEY_RSAPublicKeyTemplate);
 		if (rv_item != NULL) {
@@ -1264,6 +1358,7 @@ SECKEY_CreateSubjectPublicKeyInfo(SECKEYPublicKey *pubk)
 	    break;
 	  case dsaKey:
 	    /* DER encode the params. */
+	    prepare_pqg_params_for_asn1(&pubk->u.dsa.params);
 	    rv_item = SEC_ASN1EncodeItem(arena, &params, &pubk->u.dsa.params,
 					 SECKEY_PQGParamsTemplate);
 	    if (rv_item != NULL) {
@@ -1274,6 +1369,7 @@ SECKEY_CreateSubjectPublicKeyInfo(SECKEYPublicKey *pubk)
 		    /*
 		     * DER encode the public key into the subjectPublicKeyInfo.
 		     */
+		    prepare_dsa_pub_key_for_asn1(pubk);
 		    rv_item = SEC_ASN1EncodeItem(arena, &spki->subjectPublicKey,
 						 pubk,
 						 SECKEY_DSAPublicKeyTemplate);
@@ -1356,6 +1452,7 @@ SECKEY_DecodeDERPublicKey(SECItem *pubkder)
     PRArenaPool *arena;
     SECKEYPublicKey *pubk;
     SECStatus rv;
+    SECItem newPubkder;
 
     arena = PORT_NewArena (DER_DEFAULT_CHUNKSIZE);
     if (arena == NULL) {
@@ -1368,8 +1465,14 @@ SECKEY_DecodeDERPublicKey(SECItem *pubkder)
 	pubk->arena = arena;
 	pubk->pkcs11Slot = NULL;
 	pubk->pkcs11ID = 0;
-	rv = SEC_ASN1DecodeItem(arena, pubk, SECKEY_RSAPublicKeyTemplate,
-				pubkder);
+	prepare_rsa_pub_key_for_asn1(pubk);
+        /* copy the DER into the arena, since Quick DER returns data that points
+           into the DER input, which may get freed by the caller */
+        rv = SECITEM_CopyItem(arena, &newPubkder, pubkder);
+        if ( rv == SECSuccess ) {
+	    rv = SEC_QuickDERDecodeItem(arena, pubk, SECKEY_RSAPublicKeyTemplate,
+				&newPubkder);
+        }
 	if (rv == SECSuccess)
 	    return pubk;
 	SECKEY_DestroyPublicKey (pubk);
@@ -1430,6 +1533,7 @@ SECKEY_DecodeDERSubjectPublicKeyInfo(SECItem *spkider)
     PRArenaPool *arena;
     CERTSubjectPublicKeyInfo *spki;
     SECStatus rv;
+    SECItem newSpkider;
 
     arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     if (arena == NULL) {
@@ -1441,8 +1545,14 @@ SECKEY_DecodeDERSubjectPublicKeyInfo(SECItem *spkider)
 		PORT_ArenaZAlloc(arena, sizeof (CERTSubjectPublicKeyInfo));
     if (spki != NULL) {
 	spki->arena = arena;
-	rv = SEC_ASN1DecodeItem(arena,spki,
-				CERT_SubjectPublicKeyInfoTemplate,spkider);
+
+        /* copy the DER into the arena, since Quick DER returns data that points
+           into the DER input, which may get freed by the caller */
+        rv = SECITEM_CopyItem(arena, &newSpkider, spkider);
+        if ( rv == SECSuccess ) {
+            rv = SEC_QuickDERDecodeItem(arena,spki,
+				    CERT_SubjectPublicKeyInfoTemplate, &newSpkider);
+        }
 	if (rv == SECSuccess)
 	    return spki;
 	SECKEY_DestroySubjectPublicKeyInfo(spki);
@@ -1490,7 +1600,7 @@ SECKEY_ConvertAndDecodePublicKeyAndChallenge(char *pkacstr, char *challenge,
     CERTSignedData sd;
     SECItem sig;
     SECKEYPublicKey *pubKey = NULL;
-    int len;
+    unsigned int len;
     
     signedItem.data = NULL;
     
@@ -1508,14 +1618,14 @@ SECKEY_ConvertAndDecodePublicKeyAndChallenge(char *pkacstr, char *challenge,
 
     /* decode the outer wrapping of signed data */
     PORT_Memset(&sd, 0, sizeof(CERTSignedData));
-    rv = SEC_ASN1DecodeItem(arena, &sd, CERT_SignedDataTemplate, &signedItem );
+    rv = SEC_QuickDERDecodeItem(arena, &sd, CERT_SignedDataTemplate, &signedItem );
     if ( rv ) {
 	goto loser;
     }
 
     /* decode the public key and challenge wrapper */
     PORT_Memset(&pkac, 0, sizeof(CERTPublicKeyAndChallenge));
-    rv = SEC_ASN1DecodeItem(arena, &pkac, CERT_PublicKeyAndChallengeTemplate, 
+    rv = SEC_QuickDERDecodeItem(arena, &pkac, CERT_PublicKeyAndChallengeTemplate, 
 			    &sd.data);
     if ( rv ) {
 	goto loser;
@@ -1707,16 +1817,26 @@ SECKEY_ImportDERPublicKey(SECItem *derKey, CK_KEY_TYPE type)
     pubk->arena = NULL;
     pubk->pkcs11Slot = NULL;
     pubk->pkcs11ID = CK_INVALID_HANDLE;
-    pubk->keyType = (type == CKK_RSA) ? rsaKey : dsaKey;
 
-    if( type == CKK_RSA) {
-        rv = SEC_ASN1DecodeItem(NULL, pubk, SECKEY_RSAPublicKeyTemplate,
-                                derKey);
-    } else if( type == CKK_DSA) {
-        rv = SEC_ASN1DecodeItem(NULL, pubk, SECKEY_DSAPublicKeyTemplate,
-                                derKey);
-    } else {
+    switch( type ) {
+      case CKK_RSA:
+	prepare_rsa_pub_key_for_asn1(pubk);
+        rv = SEC_ASN1DecodeItem(NULL, pubk, SECKEY_RSAPublicKeyTemplate,derKey);
+        pubk->keyType = rsaKey;
+        break;
+      case CKK_DSA:
+	prepare_dsa_pub_key_for_asn1(pubk);
+        rv = SEC_ASN1DecodeItem(NULL, pubk, SECKEY_DSAPublicKeyTemplate,derKey);
+        pubk->keyType = dsaKey;
+        break;
+      case CKK_DH:
+	prepare_dh_pub_key_for_asn1(pubk);
+        rv = SEC_ASN1DecodeItem(NULL, pubk, SECKEY_DHPublicKeyTemplate, derKey);
+        pubk->keyType = dhKey;
+        break;
+      default:
         rv = SECFailure;
+        break;
     }
 
 finish:
@@ -1802,3 +1922,80 @@ SECKEY_AddPrivateKeyToListTail( SECKEYPrivateKeyList *list,
 loser:
     return(SECFailure);
 }
+
+
+SECKEYPublicKeyList*
+SECKEY_NewPublicKeyList(void)
+{
+    PRArenaPool *arena = NULL;
+    SECKEYPublicKeyList *ret = NULL;
+
+    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+    if ( arena == NULL ) {
+        goto loser;
+    }
+
+    ret = (SECKEYPublicKeyList *)PORT_ArenaZAlloc(arena,
+                sizeof(SECKEYPublicKeyList));
+    if ( ret == NULL ) {
+        goto loser;
+    }
+
+    ret->arena = arena;
+
+    PR_INIT_CLIST(&ret->list);
+
+    return(ret);
+
+loser:
+    if ( arena != NULL ) {
+        PORT_FreeArena(arena, PR_FALSE);
+    }
+
+    return(NULL);
+}
+
+void
+SECKEY_DestroyPublicKeyList(SECKEYPublicKeyList *keys)
+{
+    while( !PR_CLIST_IS_EMPTY(&keys->list) ) {
+        SECKEY_RemovePublicKeyListNode(
+            (SECKEYPublicKeyListNode*)(PR_LIST_HEAD(&keys->list)) );
+    }
+
+    PORT_FreeArena(keys->arena, PR_FALSE);
+
+    return;
+}
+
+
+void
+SECKEY_RemovePublicKeyListNode(SECKEYPublicKeyListNode *node)
+{
+    PR_ASSERT(node->key);
+    SECKEY_DestroyPublicKey(node->key);
+    node->key = NULL;
+    PR_REMOVE_LINK(&node->links);
+    return;
+
+}
+
+SECStatus
+SECKEY_AddPublicKeyToListTail( SECKEYPublicKeyList *list,
+                                SECKEYPublicKey *key)
+{
+    SECKEYPublicKeyListNode *node;
+
+    node = (SECKEYPublicKeyListNode *)PORT_ArenaZAlloc(list->arena,
+                sizeof(SECKEYPublicKeyListNode));
+    if ( node == NULL ) {
+        goto loser;
+    }
+
+    PR_INSERT_BEFORE(&node->links, &list->list);
+    node->key = key;
+    return(SECSuccess);
+
+loser:
+    return(SECFailure);
+}
