@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: devtoken.c,v $ $Revision: 1.22.2.1 $ $Date: 2002/05/20 23:20:48 $ $Name: MOZILLA_1_1a_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: devtoken.c,v $ $Revision: 1.29 $ $Date: 2002/09/30 21:15:07 $ $Name: MOZILLA_1_2_1_RELEASE $";
 #endif /* DEBUG */
 
 #ifndef NSSCKEPV_H
@@ -50,6 +50,7 @@ static const char CVS_ID[] = "@(#) $RCSfile: devtoken.c,v $ $Revision: 1.22.2.1
 #ifdef NSS_3_4_CODE
 #include "pk11func.h"
 #include "dev3hack.h"
+#include "secerr.h"
 #endif
 
 /* The number of object handles to grab during each call to C_FindObjects */
@@ -393,24 +394,24 @@ find_objects
     CK_RV ckrv;
     CK_ULONG count;
     CK_OBJECT_HANDLE *objectHandles;
+    CK_OBJECT_HANDLE staticObjects[OBJECT_STACK_SIZE];
     PRUint32 arraySize, numHandles;
     void *epv = nssToken_GetCryptokiEPV(tok);
     nssCryptokiObject **objects;
-    NSSArena *arena;
     nssSession *session = (sessionOpt) ? sessionOpt : tok->defaultSession;
+
     /* the arena is only for the array of object handles */
-    arena = nssArena_Create();
-    if (!arena) {
-	if (statusOpt) *statusOpt = PR_FAILURE;
-	return (nssCryptokiObject **)NULL;
-    }
     if (maximumOpt > 0) {
 	arraySize = maximumOpt;
     } else {
 	arraySize = OBJECT_STACK_SIZE;
     }
     numHandles = 0;
-    objectHandles = nss_ZNEWARRAY(arena, CK_OBJECT_HANDLE, arraySize);
+    if (arraySize <= OBJECT_STACK_SIZE) {
+	objectHandles = staticObjects;
+    } else {
+	objectHandles = nss_ZNEWARRAY(NULL, CK_OBJECT_HANDLE, arraySize);
+    }
     if (!objectHandles) {
 	goto loser;
     }
@@ -443,9 +444,17 @@ find_objects
 	}
 	/* the array is filled, double it and continue */
 	arraySize *= 2;
-	objectHandles = nss_ZREALLOCARRAY(objectHandles, 
+	if (objectHandles == staticObjects) {
+	    objectHandles = nss_ZNEWARRAY(NULL,CK_OBJECT_HANDLE, arraySize);
+	    if (objectHandles) {
+		PORT_Memcpy(objectHandles, staticObjects, 
+			OBJECT_STACK_SIZE * sizeof(objectHandles[1]));
+	    }
+	} else {
+	    objectHandles = nss_ZREALLOCARRAY(objectHandles, 
 	                                  CK_OBJECT_HANDLE, 
 	                                  arraySize);
+	}
 	if (!objectHandles) {
 	    nssSession_ExitMonitor(session);
 	    goto loser;
@@ -462,11 +471,15 @@ find_objects
     } else {
 	objects = NULL;
     }
-    nssArena_Destroy(arena);
+    if (objectHandles && objectHandles != staticObjects) {
+	nss_ZFreeIf(objectHandles);
+    }
     if (statusOpt) *statusOpt = PR_SUCCESS;
     return objects;
 loser:
-    nssArena_Destroy(arena);
+    if (objectHandles && objectHandles != staticObjects) {
+	nss_ZFreeIf(objectHandles);
+    }
     if (statusOpt) *statusOpt = PR_FAILURE;
     return (nssCryptokiObject **)NULL;
 }
@@ -482,7 +495,7 @@ find_objects_by_template
   PRStatus *statusOpt
 )
 {
-    CK_OBJECT_CLASS objclass;
+    CK_OBJECT_CLASS objclass = (CK_OBJECT_CLASS)-1;
     nssCryptokiObject **objects = NULL;
     PRUint32 i;
     for (i=0; i<otsize; i++) {
@@ -492,6 +505,12 @@ find_objects_by_template
 	}
     }
     PR_ASSERT(i < otsize);
+    if (i == otsize) {
+#ifdef NSS_3_4_CODE
+	PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
+#endif
+	return NULL;
+    }
     /* If these objects are being cached, try looking there first */
     if (token->cache && 
         nssTokenObjectCache_HaveObjectClass(token->cache, objclass)) 
@@ -871,7 +890,8 @@ nssToken_FindCertificateByIssuerAndSerialNumber
     /* Set the search to token/session only if provided */
     if (searchType == nssTokenSearchType_SessionOnly) {
 	NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_TOKEN, &g_ck_false);
-    } else if (searchType == nssTokenSearchType_TokenOnly) {
+    } else if ((searchType == nssTokenSearchType_TokenOnly) ||
+               (searchType == nssTokenSearchType_TokenForced)) {
 	NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_TOKEN, &g_ck_true);
     }
     /* Set the unique id */
@@ -881,9 +901,15 @@ nssToken_FindCertificateByIssuerAndSerialNumber
     NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_SERIAL_NUMBER,  serial);
     NSS_CK_TEMPLATE_FINISH(cert_template, attr, ctsize);
     /* get the object handle */
-    objects = find_objects_by_template(token, sessionOpt,
+    if (searchType == nssTokenSearchType_TokenForced) {
+	objects = find_objects(token, sessionOpt,
+	                       cert_template, ctsize,
+	                       1, statusOpt);
+    } else {
+	objects = find_objects_by_template(token, sessionOpt,
                                        cert_template, ctsize,
                                        1, statusOpt);
+    }
     if (objects) {
 	rvObject = objects[0];
 	nss_ZFreeIf(objects);
@@ -902,9 +928,15 @@ nssToken_FindCertificateByIssuerAndSerialNumber
 	    return NULL;
 	}
     	NSS_CK_SET_ATTRIBUTE_ITEM(serialAttr,CKA_SERIAL_NUMBER,&serialDecode);
-	objects = find_objects_by_template(token, sessionOpt,
+	if (searchType == nssTokenSearchType_TokenForced) {
+	    objects = find_objects(token, sessionOpt,
+	                       cert_template, ctsize,
+	                       1, statusOpt);
+	} else {
+	    objects = find_objects_by_template(token, sessionOpt,
                                        cert_template, ctsize,
                                        1, statusOpt);
+	}
 	if (objects) {
 	    rvObject = objects[0];
 	    nss_ZFreeIf(objects);
@@ -1085,13 +1117,14 @@ get_ck_trust
 {
     CK_TRUST t;
     switch (nssTrust) {
-    case nssTrustLevel_Unknown: t = CKT_NETSCAPE_TRUST_UNKNOWN; break;
     case nssTrustLevel_NotTrusted: t = CKT_NETSCAPE_UNTRUSTED; break;
     case nssTrustLevel_TrustedDelegator: t = CKT_NETSCAPE_TRUSTED_DELEGATOR; 
 	break;
     case nssTrustLevel_ValidDelegator: t = CKT_NETSCAPE_VALID_DELEGATOR; break;
     case nssTrustLevel_Trusted: t = CKT_NETSCAPE_TRUSTED; break;
     case nssTrustLevel_Valid: t = CKT_NETSCAPE_VALID; break;
+    case nssTrustLevel_Unknown:
+    default: t = CKT_NETSCAPE_TRUST_UNKNOWN; break;
     }
     return t;
 }
@@ -1208,13 +1241,9 @@ nssToken_FindTrustForCertificate
     CK_ATTRIBUTE_PTR attr;
     CK_ATTRIBUTE tobj_template[5];
     CK_ULONG tobj_size;
-    PRUint8 sha1[20]; /* this is cheating... */
-    NSSItem sha1_result;
     nssSession *session = sessionOpt ? sessionOpt : token->defaultSession;
     nssCryptokiObject *object, **objects;
 
-    sha1_result.data = sha1; sha1_result.size = sizeof sha1;
-    sha1_hash(certEncoding, &sha1_result);
     NSS_CK_TEMPLATE_START(tobj_template, attr, tobj_size);
     if (searchType == nssTokenSearchType_SessionOnly) {
 	NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_TOKEN, &g_ck_false);
@@ -1222,19 +1251,8 @@ nssToken_FindTrustForCertificate
 	NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_TOKEN, &g_ck_true);
     }
     NSS_CK_SET_ATTRIBUTE_VAR( attr, CKA_CLASS,          tobjc);
-    NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_CERT_SHA1_HASH, &sha1_result);
-#ifdef NSS_3_4_CODE
-    if (!PK11_HasRootCerts(token->pk11slot)) {
-#endif
     NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_ISSUER,         certIssuer);
     NSS_CK_SET_ATTRIBUTE_ITEM(attr, CKA_SERIAL_NUMBER , certSerial);
-#ifdef NSS_3_4_CODE
-    }
-    /*
-     * we need to arrange for the built-in token to lose the bottom 2 
-     * attributes so that old built-in tokens will continue to work.
-     */
-#endif
     NSS_CK_TEMPLATE_FINISH(tobj_template, attr, tobj_size);
     object = NULL;
     objects = find_objects_by_template(token, session,
@@ -1662,3 +1680,26 @@ loser:
     return PR_FAILURE;
 }
 
+NSS_IMPLEMENT PRBool
+nssToken_IsPrivateKeyAvailable
+(
+  NSSToken *token,
+  NSSCertificate *c,
+  nssCryptokiObject *instance
+)
+{
+    CK_OBJECT_CLASS theClass;
+
+    if (token == NULL) return PR_FALSE;
+    if (c == NULL) return PR_FALSE;
+
+    theClass = CKO_PRIVATE_KEY;
+    if (!nssSlot_IsLoggedIn(token->slot)) {
+	theClass = CKO_PUBLIC_KEY;
+    }
+    if (PK11_MatchItem(token->pk11slot, instance->handle, theClass) 
+						!= CK_INVALID_HANDLE) {
+	return PR_TRUE;
+    }
+    return PR_FALSE;
+}
