@@ -32,7 +32,7 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  *
- # $Id: nssinit.c,v 1.47.2.4 2002/05/25 06:58:21 wtc%netscape.com Exp $
+ # $Id: nssinit.c,v 1.57 2002/10/05 02:24:25 jpierre%netscape.com Exp $
  */
 
 #include <ctype.h>
@@ -51,6 +51,7 @@
 #include "pk11func.h"
 
 #include "pki3hack.h"
+#include "certi.h"
 
 /*
  * On Windows nss3.dll needs to export the symbol 'mktemp' to be
@@ -71,7 +72,8 @@ nss_mktemp(char *path)
 #endif
 
 #define NSS_MAX_FLAG_SIZE  sizeof("readOnly")+sizeof("noCertDB")+ \
-	sizeof("noModDB")+sizeof("forceOpen")+sizeof("passwordRequired")
+	sizeof("noModDB")+sizeof("forceOpen")+sizeof("passwordRequired")+ \
+	sizeof ("optimizeSpace")
 #define NSS_DEFAULT_MOD_NAME "NSS Internal Module"
 #ifdef macintosh
 #define SECMOD_DB "Security Modules"
@@ -81,7 +83,8 @@ nss_mktemp(char *path)
 
 static char *
 nss_makeFlags(PRBool readOnly, PRBool noCertDB, 
-		PRBool noModDB, PRBool forceOpen, PRBool passwordRequired) 
+				PRBool noModDB, PRBool forceOpen, 
+				PRBool passwordRequired, PRBool optimizeSpace) 
 {
     char *flags = (char *)PORT_Alloc(NSS_MAX_FLAG_SIZE);
     PRBool first = PR_TRUE;
@@ -111,6 +114,11 @@ nss_makeFlags(PRBool readOnly, PRBool noCertDB,
         PORT_Strcat(flags,"passwordRequired");
         first = PR_FALSE;
     }
+    if (optimizeSpace) {
+        if (!first) PORT_Strcat(flags,",");
+        PORT_Strcat(flags,"optimizeSpace");
+        first = PR_FALSE;
+    }
     return flags;
 }
 
@@ -389,7 +397,8 @@ static PRBool nss_IsInitted = PR_FALSE;
 static SECStatus
 nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
 		 const char *secmodName, PRBool readOnly, PRBool noCertDB, 
-			PRBool noModDB, PRBool forceOpen, PRBool noRootInit)
+			PRBool noModDB, PRBool forceOpen, PRBool noRootInit,
+			PRBool optimizeSpace)
 {
     char *moduleSpec = NULL;
     char *flags = NULL;
@@ -403,8 +412,12 @@ nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
 	return SECSuccess;
     }
 
+    if (SECSuccess != InitCRLCache()) {
+        return SECFailure;
+    }
+
     flags = nss_makeFlags(readOnly,noCertDB,noModDB,forceOpen,
-						pk11_password_required);
+					pk11_password_required, optimizeSpace);
     if (flags == NULL) return rv;
 
     /*
@@ -462,6 +475,7 @@ loser:
 	    }
 	}
 #endif
+	pk11sdr_Init();
 	nss_IsInitted = PR_TRUE;
     }
     return rv;
@@ -472,14 +486,14 @@ SECStatus
 NSS_Init(const char *configdir)
 {
     return nss_Init(configdir, "", "", SECMOD_DB, PR_TRUE, 
-		PR_FALSE, PR_FALSE, PR_FALSE, PR_TRUE);
+		PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, PR_TRUE);
 }
 
 SECStatus
 NSS_InitReadWrite(const char *configdir)
 {
     return nss_Init(configdir, "", "", SECMOD_DB, PR_FALSE, 
-		PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE);
+		PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, PR_TRUE);
 }
 
 /*
@@ -509,7 +523,8 @@ NSS_Initialize(const char *configdir, const char *certPrefix,
 	((flags & NSS_INIT_NOCERTDB) == NSS_INIT_NOCERTDB),
 	((flags & NSS_INIT_NOMODDB) == NSS_INIT_NOMODDB),
 	((flags & NSS_INIT_FORCEOPEN) == NSS_INIT_FORCEOPEN),
-	((flags & NSS_INIT_NOROOTINIT) == NSS_INIT_NOROOTINIT));
+	((flags & NSS_INIT_NOROOTINIT) == NSS_INIT_NOROOTINIT),
+	((flags & NSS_INIT_OPTIMIZESPACE) == NSS_INIT_OPTIMIZESPACE));
 }
 
 /*
@@ -518,8 +533,8 @@ NSS_Initialize(const char *configdir, const char *certPrefix,
 SECStatus
 NSS_NoDB_Init(const char * configdir)
 {
-      return nss_Init(configdir?configdir:"","","",SECMOD_DB,
-				PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE);
+      return nss_Init("","","","",
+			PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE);
 }
 
 SECStatus
@@ -527,9 +542,11 @@ NSS_Shutdown(void)
 {
     SECStatus rv;
 
+    ShutdownCRLCache();
     SECOID_Shutdown();
     STAN_Shutdown();
     rv = SECMOD_Shutdown();
+    pk11sdr_Shutdown();
     nss_IsInitted = PR_FALSE;
     return rv;
 }
