@@ -40,6 +40,7 @@
 #include "pcert.h"
 #include "secasn1.h"
 #include "blapi.h"
+#include "secerr.h"
 
 /*
  * ******************** Attribute Utilities *******************************
@@ -284,6 +285,9 @@ static const PK11Attribute pk11_StaticTrueAttr =
 static const PK11Attribute pk11_StaticFalseAttr = 
   PK11_DEF_ATTRIBUTE(&pk11_staticFalseValue,sizeof(pk11_staticFalseValue));
 static const PK11Attribute pk11_StaticNullAttr = PK11_DEF_ATTRIBUTE(NULL,0);
+char pk11_StaticOneValue = 1;
+static const PK11Attribute pk11_StaticOneAttr = 
+  PK11_DEF_ATTRIBUTE(&pk11_StaticOneValue,sizeof(pk11_StaticOneValue));
 
 CK_CERTIFICATE_TYPE pk11_staticX509Value = CKC_X_509;
 static const PK11Attribute pk11_StaticX509Attr =
@@ -317,11 +321,6 @@ static const PK11Attribute pk11_StaticMustVerifyAttr =
   PK11_DEF_ATTRIBUTE(&pk11_staticMustVerifyValue,
 				sizeof(pk11_staticMustVerifyValue));
 
-static void pk11_FreeItem(SECItem *item)
-{
-    SECITEM_FreeItem(item, PR_TRUE);
-}
-
 static certDBEntrySMime *
 pk11_getSMime(PK11TokenObject *object)
 {
@@ -341,68 +340,64 @@ pk11_getSMime(PK11TokenObject *object)
     return entry;
 }
 
-static SECItem *
+static certDBEntryRevocation *
 pk11_getCrl(PK11TokenObject *object)
 {
-    SECItem *crl;
+    certDBEntryRevocation *crl;
     PRBool isKrl;
 
     if (object->obj.objclass != CKO_NETSCAPE_CRL) {
 	return NULL;
     }
     if (object->obj.objectInfo) {
-	return (SECItem *)object->obj.objectInfo;
+	return (certDBEntryRevocation *)object->obj.objectInfo;
     }
 
     isKrl = (PRBool) object->obj.handle == PK11_TOKEN_KRL_HANDLE;
-    crl = nsslowcert_FindCrlByKey(object->obj.slot->certDB,&object->dbKey,
-								NULL,isKrl);
+    crl = nsslowcert_FindCrlByKey(object->obj.slot->certDB,
+							&object->dbKey, isKrl);
     object->obj.objectInfo = (void *)crl;
-    object->obj.infoFree = (PK11Free) pk11_FreeItem;
+    object->obj.infoFree = (PK11Free) nsslowcert_DestroyDBEntry;
     return crl;
 }
 
-static char *
-pk11_getUrl(PK11TokenObject *object)
-{
-    SECItem *crl;
-    PRBool isKrl;
-    char *url = NULL;
-
-    if (object->obj.objclass != CKO_NETSCAPE_CRL) {
-	return NULL;
-    }
-
-    isKrl = (PRBool) object->obj.handle == PK11_TOKEN_KRL_HANDLE;
-    crl = nsslowcert_FindCrlByKey(object->obj.slot->certDB,&object->dbKey,
-								&url,isKrl);
-    if (object->obj.objectInfo == NULL) {
-	object->obj.objectInfo = (void *)crl;
-	object->obj.infoFree = (PK11Free) pk11_FreeItem;
-    } else {
-	if (crl) SECITEM_FreeItem(crl,PR_TRUE);
-    }
-    return url;
-}
-
 static NSSLOWCERTCertificate *
 pk11_getCert(PK11TokenObject *object)
 {
     NSSLOWCERTCertificate *cert;
+    CK_OBJECT_CLASS objClass = object->obj.objclass;
 
-    if ((object->obj.objclass != CKO_CERTIFICATE) &&
-	 		(object->obj.objclass != CKO_NETSCAPE_TRUST)) {
+    if ((objClass != CKO_CERTIFICATE) && (objClass != CKO_NETSCAPE_TRUST)) {
 	return NULL;
     }
-    if (object->obj.objectInfo) {
+    if (objClass == CKO_CERTIFICATE && object->obj.objectInfo) {
 	return (NSSLOWCERTCertificate *)object->obj.objectInfo;
     }
     cert = nsslowcert_FindCertByKey(object->obj.slot->certDB,&object->dbKey);
-    object->obj.objectInfo = (void *)cert;
-    object->obj.infoFree = (PK11Free) nsslowcert_DestroyCertificate ;
+    if (objClass == CKO_CERTIFICATE) {
+	object->obj.objectInfo = (void *)cert;
+	object->obj.infoFree = (PK11Free) nsslowcert_DestroyCertificate ;
+    }
     return cert;
 }
 
+static NSSLOWCERTTrust *
+pk11_getTrust(PK11TokenObject *object)
+{
+    NSSLOWCERTTrust *trust;
+
+    if (object->obj.objclass != CKO_NETSCAPE_TRUST) {
+	return NULL;
+    }
+    if (object->obj.objectInfo) {
+	return (NSSLOWCERTTrust *)object->obj.objectInfo;
+    }
+    trust = nsslowcert_FindTrustByKey(object->obj.slot->certDB,&object->dbKey);
+    object->obj.objectInfo = (void *)trust;
+    object->obj.infoFree = (PK11Free) nsslowcert_DestroyTrust ;
+    return trust;
+}
+
 static NSSLOWKEYPublicKey *
 pk11_GetPublicKey(PK11TokenObject *object)
 {
@@ -603,7 +598,7 @@ pk11_FindPublicKeyAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
         label = nsslowkey_FindKeyNicknameByPublicKey(object->obj.slot->keyDB,
 				&object->dbKey, object->obj.slot->password);
 	if (label == NULL) {
-	   return (PK11Attribute *)&pk11_StaticNullAttr;
+	   return (PK11Attribute *)&pk11_StaticOneAttr;
 	}
 	att = pk11_NewTokenAttribute(type,label,PORT_Strlen(label), PR_TRUE);
 	PORT_Free(label);
@@ -663,8 +658,11 @@ pk11_FindSecretKeyAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 	att = pk11_NewTokenAttribute(type,label,PORT_Strlen(label), PR_TRUE);
 	PORT_Free(label);
 	return att;
-    default:
+    case CKA_KEY_TYPE:
+    case CKA_VALUE:
 	break;
+    default:
+	return NULL;
     }
 
     key = pk11_GetPrivateKey(object);
@@ -853,10 +851,12 @@ pk11_FindSMIMEAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
     case CKA_NETSCAPE_EMAIL:
 	return pk11_NewTokenAttribute(type,object->dbKey.data,
 						object->dbKey.len-1, PR_FALSE);
-    case CKA_SENSITIVE:
-	return NULL;
-    default:
+    case CKA_NETSCAPE_SMIME_TIMESTAMP:
+    case CKA_SUBJECT:
+    case CKA_VALUE:
 	break;
+    default:
+	return NULL;
     }
     entry = pk11_getSMime(object);
     if (entry == NULL) {
@@ -881,10 +881,8 @@ pk11_FindSMIMEAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 static PK11Attribute *
 pk11_FindTrustAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 {
-    NSSLOWCERTCertificate *cert;
+    NSSLOWCERTTrust *trust;
     unsigned char hash[SHA1_LENGTH];
-    SECItem *item;
-    PK11Attribute *attr;
     unsigned int trustFlags;
 
     switch (type) {
@@ -892,43 +890,39 @@ pk11_FindTrustAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 	return (PK11Attribute *) &pk11_StaticFalseAttr;
     case CKA_MODIFIABLE:
 	return (PK11Attribute *) &pk11_StaticTrueAttr;
-    case CKA_SENSITIVE:
-	return NULL;
-    default:
+    case CKA_CERT_SHA1_HASH:
+    case CKA_CERT_MD5_HASH:
+    case CKA_TRUST_CLIENT_AUTH:
+    case CKA_TRUST_SERVER_AUTH:
+    case CKA_TRUST_EMAIL_PROTECTION:
+    case CKA_TRUST_CODE_SIGNING:
 	break;
+    default:
+	return NULL;
     }
-    cert = pk11_getCert(object);
-    if (cert == NULL) {
+    trust = pk11_getTrust(object);
+    if (trust == NULL) {
 	return NULL;
     }
     switch (type) {
     case CKA_CERT_SHA1_HASH:
-	SHA1_HashBuf(hash,cert->derCert.data,cert->derCert.len);
-	return pk11_NewTokenAttribute(type,hash,SHA1_LENGTH, PR_TRUE);
+	SHA1_HashBuf(hash,trust->derCert->data,trust->derCert->len);
+	return pk11_NewTokenAttribute(type, hash, SHA1_LENGTH, PR_TRUE);
     case CKA_CERT_MD5_HASH:
-	MD5_HashBuf(hash,cert->derCert.data,cert->derCert.len);
-	return pk11_NewTokenAttribute(type,hash,MD5_LENGTH, PR_TRUE);
-    case CKA_ISSUER:
-	return pk11_NewTokenAttribute(type,cert->derIssuer.data,
-						cert->derIssuer.len, PR_FALSE);
-    case CKA_SERIAL_NUMBER:
-	item = SEC_ASN1EncodeItem(NULL,NULL,cert,pk11_SerialTemplate);
-	if (item == NULL) break;
-	attr = pk11_NewTokenAttribute(type, item->data, item->len, PR_TRUE);
-	SECITEM_FreeItem(item,PR_TRUE);
-	return attr;
+	MD5_HashBuf(hash,trust->derCert->data,trust->derCert->len);
+	return pk11_NewTokenAttribute(type, hash, MD5_LENGTH, PR_TRUE);
     case CKA_TRUST_CLIENT_AUTH:
-	trustFlags = cert->trust->sslFlags & CERTDB_TRUSTED_CLIENT_CA ?
-		cert->trust->sslFlags | CERTDB_TRUSTED_CA : 0 ;
+	trustFlags = trust->trust->sslFlags & CERTDB_TRUSTED_CLIENT_CA ?
+		trust->trust->sslFlags | CERTDB_TRUSTED_CA : 0 ;
 	goto trust;
     case CKA_TRUST_SERVER_AUTH:
-	trustFlags = cert->trust->sslFlags;
+	trustFlags = trust->trust->sslFlags;
 	goto trust;
     case CKA_TRUST_EMAIL_PROTECTION:
-	trustFlags = cert->trust->emailFlags;
+	trustFlags = trust->trust->emailFlags;
 	goto trust;
     case CKA_TRUST_CODE_SIGNING:
-	trustFlags = cert->trust->objectSigningFlags;
+	trustFlags = trust->trust->objectSigningFlags;
 trust:
 	if (trustFlags & CERTDB_TRUSTED_CA ) {
 	    return (PK11Attribute *)&pk11_StaticTrustedDelegatorAttr;
@@ -952,14 +946,35 @@ trust:
     default:
 	break;
     }
+
+#ifdef notdef
+    switch (type) {
+    case CKA_ISSUER:
+	cert = pk11_getCertObject(object);
+	if (cert == NULL) break;
+	attr = pk11_NewTokenAttribute(type,cert->derIssuer.data,
+						cert->derIssuer.len, PR_FALSE);
+	
+    case CKA_SERIAL_NUMBER:
+	cert = pk11_getCertObject(object);
+	if (cert == NULL) break;
+	item = SEC_ASN1EncodeItem(NULL,NULL,cert,pk11_SerialTemplate);
+	if (item == NULL) break;
+	attr = pk11_NewTokenAttribute(type, item->data, item->len, PR_TRUE);
+	SECITEM_FreeItem(item,PR_TRUE);
+    }
+    if (cert) {
+	NSSLOWCERTDestroyCertificate(cert);	
+	return attr;
+    }
+#endif
     return NULL;
 }
 
 static PK11Attribute *
 pk11_FindCrlAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 {
-    SECItem *crl;
-    char *url;
+    certDBEntryRevocation *crl;
 
     switch (type) {
     case CKA_PRIVATE:
@@ -968,19 +983,30 @@ pk11_FindCrlAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
     case CKA_NETSCAPE_KRL:
 	return (PK11Attribute *) ((object->obj.handle == PK11_TOKEN_KRL_HANDLE) 
 			? &pk11_StaticTrueAttr : &pk11_StaticFalseAttr);
-    case CKA_NETSCAPE_URL:
-	url = pk11_getUrl(object);
-	if (url == NULL) {
-	    return (PK11Attribute *) &pk11_StaticNullAttr;
-	}
-	return pk11_NewTokenAttribute(type, url, PORT_Strlen(url)+1, PR_TRUE);
-    case CKA_VALUE:
-	crl = pk11_getCrl(object);
-	if (crl == NULL) break;
-	return pk11_NewTokenAttribute(type, crl->data, crl->len, PR_FALSE);
     case CKA_SUBJECT:
 	return pk11_NewTokenAttribute(type,object->dbKey.data,
-						object->dbKey.len, PR_FALSE);
+						object->dbKey.len, PR_FALSE);	
+    case CKA_NETSCAPE_URL:
+    case CKA_VALUE:
+	break;
+    default:
+	PORT_SetError(SEC_ERROR_INVALID_ARGS);
+    	return NULL;
+    }
+    crl =  pk11_getCrl(object);
+    if (!crl) {
+	return NULL;
+    }
+    switch (type) {
+    case CKA_NETSCAPE_URL:
+	if (crl->url == NULL) {
+	    return (PK11Attribute *) &pk11_StaticNullAttr;
+	}
+	return pk11_NewTokenAttribute(type, crl->url, 
+					PORT_Strlen(crl->url)+1, PR_TRUE);
+    case CKA_VALUE:
+	return pk11_NewTokenAttribute(type, crl->derCrl.data, 
+						crl->derCrl.len, PR_FALSE);
     default:
 	break;
     }
@@ -994,7 +1020,6 @@ pk11_FindCertAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
     NSSLOWKEYPublicKey  *pubKey;
     unsigned char hash[SHA1_LENGTH];
     SECItem *item;
-    PK11Attribute *attr;
 
     switch (type) {
     case CKA_PRIVATE:
@@ -1004,10 +1029,16 @@ pk11_FindCertAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
     case CKA_CERTIFICATE_TYPE:
         /* hardcoding X.509 into here */
         return (PK11Attribute *)&pk11_StaticX509Attr;
-    case CKA_SENSITIVE:
-	return NULL;
-    default:
+    case CKA_VALUE:
+    case CKA_ID:
+    case CKA_LABEL:
+    case CKA_SUBJECT:
+    case CKA_ISSUER:
+    case CKA_SERIAL_NUMBER:
+    case CKA_NETSCAPE_EMAIL:
 	break;
+    default:
+	return NULL;
     }
     cert = pk11_getCert(object);
     if (cert == NULL) {
@@ -1018,6 +1049,11 @@ pk11_FindCertAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 	return pk11_NewTokenAttribute(type,cert->derCert.data,
 						cert->derCert.len,PR_FALSE);
     case CKA_ID:
+	if (((cert->trust->sslFlags & CERTDB_USER) == 0) &&
+		((cert->trust->emailFlags & CERTDB_USER) == 0) &&
+		((cert->trust->objectSigningFlags & CERTDB_USER) == 0)) {
+	    return (PK11Attribute *) &pk11_StaticNullAttr;
+	}
 	pubKey = nsslowcert_ExtractPublicKey(cert);
 	if (pubKey == NULL) break;
 	item = pk11_GetPubItem(pubKey);
@@ -1040,11 +1076,8 @@ pk11_FindCertAttribute(PK11TokenObject *object, CK_ATTRIBUTE_TYPE type)
 	return pk11_NewTokenAttribute(type,cert->derIssuer.data,
 						cert->derIssuer.len, PR_FALSE);
     case CKA_SERIAL_NUMBER:
-	item = SEC_ASN1EncodeItem(NULL,NULL,cert,pk11_SerialTemplate);
-	if (item == NULL) break;
-	attr = pk11_NewTokenAttribute(type, item->data, item->len, PR_TRUE);
-	SECITEM_FreeItem(item,PR_TRUE);
-	return attr;
+	return pk11_NewTokenAttribute(type,cert->derSN.data,
+						cert->derSN.len, PR_FALSE);
     case CKA_NETSCAPE_EMAIL:
 	return cert->emailAddr ? pk11_NewTokenAttribute(type, cert->emailAddr,
 				PORT_Strlen(cert->emailAddr), PR_FALSE) :
@@ -1114,7 +1147,7 @@ pk11_FindAttribute(PK11Object *object,CK_ATTRIBUTE_TYPE type)
     }
 
     PK11_USE_THREADS(PZ_Lock(sessObject->attributeLock);)
-    pk11queue_find(attribute,type,sessObject->head,ATTRIBUTE_HASH_SIZE);
+    pk11queue_find(attribute,type,sessObject->head, sessObject->hashSize);
 #ifdef PKCS11_REF_COUNT_ATTRIBUTES
     if (attribute) {
 	/* atomic increment would be nice here */
@@ -1150,7 +1183,7 @@ pk11_hasAttribute(PK11Object *object,CK_ATTRIBUTE_TYPE type)
     }
 
     PK11_USE_THREADS(PZ_Lock(sessObject->attributeLock);)
-    pk11queue_find(attribute,type,sessObject->head,ATTRIBUTE_HASH_SIZE);
+    pk11queue_find(attribute,type,sessObject->head, sessObject->hashSize);
     PK11_USE_THREADS(PZ_Unlock(sessObject->attributeLock);)
 
     return (PRBool)(attribute != NULL);
@@ -1167,7 +1200,7 @@ pk11_AddAttribute(PK11Object *object,PK11Attribute *attribute)
     if (sessObject == NULL) return;
     PK11_USE_THREADS(PZ_Lock(sessObject->attributeLock);)
     pk11queue_add(attribute,attribute->handle,
-				sessObject->head,ATTRIBUTE_HASH_SIZE);
+				sessObject->head, sessObject->hashSize);
     PK11_USE_THREADS(PZ_Unlock(sessObject->attributeLock);)
 }
 
@@ -1180,7 +1213,6 @@ pk11_Attribute2SSecItem(PLArenaPool *arena,SECItem *item,PK11Object *object,
                                       CK_ATTRIBUTE_TYPE type)
 {
     PK11Attribute *attribute;
-    unsigned char *start;
 
     item->data = NULL;
 
@@ -1211,9 +1243,9 @@ pk11_DeleteAttribute(PK11Object *object, PK11Attribute *attribute)
     }
     PK11_USE_THREADS(PZ_Lock(sessObject->attributeLock);)
     if (pk11queue_is_queued(attribute,attribute->handle,
-				sessObject->head,ATTRIBUTE_HASH_SIZE)) {
+				sessObject->head, sessObject->hashSize)) {
 	pk11queue_delete(attribute,attribute->handle,
-				sessObject->head,ATTRIBUTE_HASH_SIZE);
+				sessObject->head, sessObject->hashSize);
     }
     PK11_USE_THREADS(PZ_Unlock(sessObject->attributeLock);)
     pk11_FreeAttribute(attribute);
@@ -1270,9 +1302,9 @@ pk11_SetCertAttribute(PK11TokenObject *to, CK_ATTRIBUTE_TYPE type,
     char *nickname = NULL;
     SECStatus rv;
 
-    /* we can't change the ID or the EMAIL values, but let the
+    /* we can't change  the EMAIL values, but let the
      * upper layers feel better about the fact we tried to set these */
-    if ((type == CKA_ID) || (type == CKA_NETSCAPE_EMAIL)) {
+    if (type == CKA_NETSCAPE_EMAIL) {
 	return CKR_OK;
     }
 
@@ -1280,7 +1312,7 @@ pk11_SetCertAttribute(PK11TokenObject *to, CK_ATTRIBUTE_TYPE type,
 	return CKR_TOKEN_WRITE_PROTECTED;
     }
 
-    if (type != CKA_LABEL)  {
+    if ((type != CKA_LABEL)  && (type != CKA_ID)) {
 	return CKR_ATTRIBUTE_READ_ONLY;
     }
 
@@ -1289,6 +1321,27 @@ pk11_SetCertAttribute(PK11TokenObject *to, CK_ATTRIBUTE_TYPE type,
 	return CKR_OBJECT_HANDLE_INVALID;
     }
 
+    /* if the app is trying to set CKA_ID, it's probably because it just
+     * imported the key. Look to see if we need to set the CERTDB_USER bits.
+     */
+    if (type == CKA_ID) {
+	if (((cert->trust->sslFlags & CERTDB_USER) == 0) &&
+		((cert->trust->emailFlags & CERTDB_USER) == 0) &&
+		((cert->trust->objectSigningFlags & CERTDB_USER) == 0)) {
+	    PK11Slot *slot = to->obj.slot;
+
+	    if (slot->keyDB && nsslowkey_KeyForCertExists(slot->keyDB,cert)) {
+		NSSLOWCERTCertTrust trust = *cert->trust;
+		trust.sslFlags |= CERTDB_USER;
+		trust.emailFlags |= CERTDB_USER;
+		trust.objectSigningFlags |= CERTDB_USER;
+		nsslowcert_ChangeCertTrust(slot->certDB,cert,&trust);
+	    }
+	}
+	return CKR_OK;
+    }
+
+    /* must be CKA_LABEL */
     if (value != NULL) {
 	nickname = PORT_ZAlloc(len+1);
 	if (nickname == NULL) {
@@ -1414,9 +1467,10 @@ pk11_forceTokenAttribute(PK11Object *object,CK_ATTRIBUTE_TYPE type,
     }
 
     /* if we are just setting it to the value we already have,
-     * allow it to happen. */
+     * allow it to happen. Let label setting go through so
+     * we have the opportunity to repair any database corruption. */
     attribute=pk11_FindAttribute(object,type);
-    if ((attribute->attrib.ulValueLen == len) &&
+    if ((type != CKA_LABEL) && (attribute->attrib.ulValueLen == len) &&
 	PORT_Memcmp(attribute->attrib.pValue,value,len) == 0) {
 	pk11_FreeAttribute(attribute);
 	return CKR_OK;
@@ -1710,10 +1764,10 @@ pk11_deleteTokenKeyByHandle(PK11Slot *slot, CK_OBJECT_HANDLE handle)
    PRBool rem;
 
    item = (SECItem *)PL_HashTableLookup(slot->tokenHashTable, (void *)handle);
-   if (item) {
+   rem = PL_HashTableRemove(slot->tokenHashTable,(void *)handle) ;
+   if (rem && item) {
 	SECITEM_FreeItem(item,PR_TRUE);
    }
-   rem = PL_HashTableRemove(slot->tokenHashTable,(void *)handle) ;
    return rem ? SECSuccess : SECFailure;
 }
 
@@ -1758,54 +1812,73 @@ pk11_tokenKeyUnlock(PK11Slot *slot) {
 
 
 /* allocation hooks that allow us to recycle old object structures */
-#ifdef MAX_OBJECT_LIST_SIZE
-static PK11Object * objectFreeList = NULL;
-static PZLock *objectLock = NULL;
-static int object_count = 0;
-#endif
+static PK11ObjectFreeList sessionObjectList = { NULL, NULL, 0 };
+static PK11ObjectFreeList tokenObjectList = { NULL, NULL, 0 };
+
 PK11Object *
-pk11_GetObjectFromList(PRBool *hasLocks) {
+pk11_GetObjectFromList(PRBool *hasLocks, PRBool optimizeSpace, 
+     PK11ObjectFreeList *list, unsigned int hashSize, PRBool isSessionObject)
+{
     PK11Object *object;
+    int size = 0;
 
-#if MAX_OBJECT_LIST_SIZE
-    if (objectLock == NULL) {
-	objectLock = PZ_NewLock(nssILockObject);
+    if (!optimizeSpace) {
+	if (list->lock == NULL) {
+	    list->lock = PZ_NewLock(nssILockObject);
+	}
+
+	PK11_USE_THREADS(PZ_Lock(list->lock));
+	object = list->head;
+	if (object) {
+	    list->head = object->next;
+	    list->count--;
+	}    	
+	PK11_USE_THREADS(PZ_Unlock(list->lock));
+	if (object) {
+	    object->next = object->prev = NULL;
+            *hasLocks = PR_TRUE;
+	    return object;
+	}
     }
+    size = isSessionObject ? sizeof(PK11SessionObject) 
+		+ hashSize *sizeof(PK11Attribute *) : sizeof(PK11TokenObject);
 
-    PK11_USE_THREADS(PZ_Lock(objectLock));
-    object = objectFreeList;
-    if (object) {
-	objectFreeList = object->next;
-	object_count--;
-    }    	
-    PK11_USE_THREADS(PZ_Unlock(objectLock));
-    if (object) {
-	object->next = object->prev = NULL;
-        *hasLocks = PR_TRUE;
-	return object;
+    object  = (PK11Object*)PORT_ZAlloc(size);
+    if (isSessionObject) {
+	((PK11SessionObject *)object)->hashSize = hashSize;
     }
-#endif
-
-    object  = (PK11Object*)PORT_ZAlloc(sizeof(PK11SessionObject));
     *hasLocks = PR_FALSE;
     return object;
 }
 
 static void
-pk11_PutObjectToList(PK11SessionObject *object) {
-#ifdef MAX_OBJECT_LIST_SIZE
-    if (object_count < MAX_OBJECT_LIST_SIZE) {
-	PK11_USE_THREADS(PZ_Lock(objectLock));
-	object->obj.next = objectFreeList;
-	objectFreeList = &object->obj;
-	object_count++;
-	PK11_USE_THREADS(PZ_Unlock(objectLock));
+pk11_PutObjectToList(PK11Object *object, PK11ObjectFreeList *list,
+						PRBool isSessionObject) {
+
+    /* the code below is equivalent to :
+     *     optimizeSpace = isSessionObject ? object->optimizeSpace : PR_FALSE;
+     * just faster.
+     */
+    PRBool optimizeSpace = isSessionObject && 
+				((PK11SessionObject *)object)->optimizeSpace; 
+    if (!optimizeSpace && (list->count < MAX_OBJECT_LIST_SIZE)) {
+	if (list->lock == NULL) {
+	    list->lock = PZ_NewLock(nssILockObject);
+	}
+	PK11_USE_THREADS(PZ_Lock(list->lock));
+	object->next = list->head;
+	list->head = object;
+	list->count++;
+	PK11_USE_THREADS(PZ_Unlock(list->lock));
 	return;
-     }
-#endif
-    PK11_USE_THREADS(PZ_DestroyLock(object->attributeLock);)
-    PK11_USE_THREADS(PZ_DestroyLock(object->obj.refLock);)
-    object->attributeLock = object->obj.refLock = NULL;
+    }
+    if (isSessionObject) {
+	PK11SessionObject *so = (PK11SessionObject *)object;
+	PK11_USE_THREADS(PZ_DestroyLock(so->attributeLock);)
+	so->attributeLock = NULL;
+    }
+    PK11_USE_THREADS(PZ_DestroyLock(object->refLock);)
+    object->refLock = NULL;
     PORT_Free(object);
 }
 
@@ -1817,29 +1890,36 @@ pk11_freeObjectData(PK11Object *object) {
    return next;
 }
    
-void
-pk11_CleanupFreeLists()
+static void
+pk11_CleanupFreeList(PK11ObjectFreeList *list, PRBool isSessionList)
 {
-#ifdef MAX_OBJECT_LIST_SIZE
     PK11Object *object;
 
-    if (!objectLock) {
+    if (!list->lock) {
 	return;
     }
-    PK11_USE_THREADS(PZ_Lock(objectLock));
-    for (object= objectFreeList; object != NULL; 
+    PK11_USE_THREADS(PZ_Lock(list->lock));
+    for (object= list->head; object != NULL; 
 					object = pk11_freeObjectData(object)) {
 #ifdef PKCS11_USE_THREADS
 	PZ_DestroyLock(object->refLock);
-	PZ_DestroyLock(((PK11SessionObject *)object)->attributeLock);
+	if (isSessionList) {
+	    PZ_DestroyLock(((PK11SessionObject *)object)->attributeLock);
+	}
 #endif
     }
-    object_count = 0;
-    objectFreeList = NULL;
-    PK11_USE_THREADS(PZ_Unlock(objectLock));
-    PZ_DestroyLock(objectLock);
-    objectLock = NULL;
-#endif
+    list->count = 0;
+    list->head = NULL;
+    PK11_USE_THREADS(PZ_Unlock(list->lock));
+    PK11_USE_THREADS(PZ_DestroyLock(list->lock));
+    list->lock = NULL;
+}
+
+void
+pk11_CleanupFreeLists(void)
+{
+    pk11_CleanupFreeList(&sessionObjectList, PR_TRUE);
+    pk11_CleanupFreeList(&tokenObjectList, PR_FALSE);
 }
 
 
@@ -1852,11 +1932,15 @@ pk11_NewObject(PK11Slot *slot)
     PK11Object *object;
     PK11SessionObject *sessObject;
     PRBool hasLocks = PR_FALSE;
-    int i;
+    unsigned int i;
+    unsigned int hashSize = 0;
 
+    hashSize = (slot->optimizeSpace) ? SPACE_ATTRIBUTE_HASH_SIZE :
+				TIME_ATTRIBUTE_HASH_SIZE;
 
 #ifdef PKCS11_STATIC_ATTRIBUTES
-    object = pk11_GetObjectFromList(&hasLocks);
+    object = pk11_GetObjectFromList(&hasLocks, slot->optimizeSpace,
+				&sessionObjectList,  hashSize, PR_TRUE);
     if (object == NULL) {
 	return NULL;
     }
@@ -1873,7 +1957,8 @@ pk11_NewObject(PK11Slot *slot)
     arena = PORT_NewArena(2048);
     if (arena == NULL) return NULL;
 
-    object = (PK11Object*)PORT_ArenaAlloc(arena,sizeof(PK11SessionObject));
+    object = (PK11Object*)PORT_ArenaAlloc(arena,sizeof(PK11SessionObject)
+		+hashSize * sizeof(PK11Attribute *));
     if (object == NULL) {
 	PORT_FreeArena(arena,PR_FALSE);
 	return NULL;
@@ -1881,7 +1966,9 @@ pk11_NewObject(PK11Slot *slot)
     object->arena = arena;
 
     sessObject = (PK11SessionObject *)object;
+    sessObject->hashSize = hashSize;
 #endif
+    sessObject->optimizeSpace = slot->optimizeSpace;
 
     object->handle = 0;
     object->next = object->prev = NULL;
@@ -1917,7 +2004,7 @@ pk11_NewObject(PK11Slot *slot)
     sessObject->attributeLock = NULL;
     object->refLock = NULL;
 #endif
-    for (i=0; i < ATTRIBUTE_HASH_SIZE; i++) {
+    for (i=0; i < sessObject->hashSize; i++) {
 	sessObject->head[i] = NULL;
     }
     object->objectInfo = NULL;
@@ -1948,7 +2035,7 @@ pk11_DestroySessionObjectData(PK11SessionObject *so)
 	/* clean out the attributes */
 	/* since no one is referencing us, it's safe to walk the chain
 	 * without a lock */
-	for (i=0; i < ATTRIBUTE_HASH_SIZE; i++) {
+	for (i=0; i < so->hashSize; i++) {
 	    PK11Attribute *ap,*next;
 	    for (ap = so->head[i]; ap != NULL; ap = next) {
 		next = ap->next;
@@ -1988,15 +2075,14 @@ pk11_DestroyObject(PK11Object *object)
     }
     if (object->objectInfo) {
 	(*object->infoFree)(object->objectInfo);
+	object->objectInfo = NULL;
+	object->infoFree = NULL;
     }
 #ifdef PKCS11_STATIC_ATTRIBUTES
     if (so) {
-	pk11_PutObjectToList(so);
+	pk11_PutObjectToList(object,&sessionObjectList,PR_TRUE);
     } else {
-	if (object->refLock) {
-	    PK11_USE_THREADS(PZ_DestroyLock(object->refLock);)
-	}
-	PORT_Free(to);
+	pk11_PutObjectToList(object,&tokenObjectList,PR_FALSE);
     }
 #else
     if (object->refLock) {
@@ -2031,7 +2117,7 @@ pk11_ObjectFromHandleOnSlot(CK_OBJECT_HANDLE handle, PK11Slot *slot)
     lock = slot->objectLock;
 
     PK11_USE_THREADS(PZ_Lock(lock);)
-    pk11queue_find(object,handle,head,TOKEN_OBJECT_HASH_SIZE);
+    pk11queue_find(object,handle,head,slot->tokObjHashSize);
     if (object) {
 	pk11_ReferenceObject(object);
     }
@@ -2085,8 +2171,7 @@ void
 pk11_AddSlotObject(PK11Slot *slot, PK11Object *object)
 {
     PK11_USE_THREADS(PZ_Lock(slot->objectLock);)
-    pk11queue_add(object,object->handle,slot->tokObjects,
-							TOKEN_OBJECT_HASH_SIZE);
+    pk11queue_add(object,object->handle,slot->tokObjects,slot->tokObjHashSize);
     PK11_USE_THREADS(PZ_Unlock(slot->objectLock);)
 }
 
@@ -2129,7 +2214,7 @@ pk11_DeleteObject(PK11Session *session, PK11Object *object)
 	PK11_USE_THREADS(PZ_Unlock(session->objectLock);)
 	PK11_USE_THREADS(PZ_Lock(slot->objectLock);)
 	pk11queue_delete(object,object->handle,slot->tokObjects,
-						TOKEN_OBJECT_HASH_SIZE);
+						slot->tokObjHashSize);
 	PK11_USE_THREADS(PZ_Unlock(slot->objectLock);)
 	pk11_FreeObject(object); /* reduce it's reference count */
     } else {
@@ -2198,14 +2283,14 @@ pk11_CopyObject(PK11Object *destObject,PK11Object *srcObject)
 {
     PK11Attribute *attribute;
     PK11SessionObject *src_so = pk11_narrowToSessionObject(srcObject);
-    int i;
+    unsigned int i;
 
     if (src_so == NULL) {
 	return CKR_DEVICE_ERROR; /* can't copy token objects yet */
     }
 
     PK11_USE_THREADS(PZ_Lock(src_so->attributeLock);)
-    for(i=0; i < ATTRIBUTE_HASH_SIZE; i++) {
+    for(i=0; i < src_so->hashSize; i++) {
 	attribute = src_so->head[i];
 	do {
 	    if (attribute) {
@@ -2278,14 +2363,15 @@ pk11_objectMatch(PK11Object *object,CK_ATTRIBUTE_PTR theTemplate,int count)
  * in the object list.
  */
 CK_RV
-pk11_searchObjectList(PK11SearchResults *search,PK11Object **head,
-        PZLock *lock, CK_ATTRIBUTE_PTR theTemplate, int count, PRBool isLoggedIn)
+pk11_searchObjectList(PK11SearchResults *search,PK11Object **head, 
+	unsigned int size, PZLock *lock, CK_ATTRIBUTE_PTR theTemplate, 
+						int count, PRBool isLoggedIn)
 {
-    int i;
+    unsigned int i;
     PK11Object *object;
     CK_RV crv = CKR_OK;
 
-    for(i=0; i < TOKEN_OBJECT_HASH_SIZE; i++) {
+    for(i=0; i < size; i++) {
         /* We need to hold the lock to copy a consistant version of
          * the linked list. */
         PK11_USE_THREADS(PZ_Lock(lock);)
@@ -2365,10 +2451,10 @@ pk11_update_state(PK11Slot *slot,PK11Session *session)
 void
 pk11_update_all_states(PK11Slot *slot)
 {
-    int i;
+    unsigned int i;
     PK11Session *session;
 
-    for (i=0; i < SESSION_HASH_SIZE; i++) {
+    for (i=0; i < slot->sessHashSize; i++) {
 	PK11_USE_THREADS(PZ_Lock(PK11_SESSION_LOCK(slot,i));)
 	for (session = slot->head[i]; session; session = session->next) {
 	    pk11_update_state(slot,session);
@@ -2485,7 +2571,7 @@ pk11_SessionFromHandle(CK_SESSION_HANDLE handle)
     PK11Session *session;
 
     PK11_USE_THREADS(PZ_Lock(PK11_SESSION_LOCK(slot,handle));)
-    pk11queue_find(session,handle,slot->head,SESSION_HASH_SIZE);
+    pk11queue_find(session,handle,slot->head,slot->sessHashSize);
     if (session) session->refCount++;
     PK11_USE_THREADS(PZ_Unlock(PK11_SESSION_LOCK(slot,handle));)
 
@@ -2593,10 +2679,12 @@ pk11_NewTokenObject(PK11Slot *slot, SECItem *dbKey, CK_OBJECT_HANDLE handle)
 {
     PK11Object *object = NULL;
     PK11TokenObject *tokObject = NULL;
+    PRBool hasLocks = PR_FALSE;
     SECStatus rv;
 
 #ifdef PKCS11_STATIC_ATTRIBUTES
-    object = (PK11Object *) PORT_ZAlloc(sizeof(PK11TokenObject));
+    object = pk11_GetObjectFromList(&hasLocks, PR_FALSE, &tokenObjectList,  0,
+							PR_FALSE);
     if (object == NULL) {
 	return NULL;
     }
@@ -2636,7 +2724,9 @@ pk11_NewTokenObject(PK11Slot *slot, SECItem *dbKey, CK_OBJECT_HANDLE handle)
 	goto loser;
     }
 #ifdef PKCS11_USE_THREADS
-    object->refLock = PZ_NewLock(nssILockRefLock);
+    if (!hasLocks) {
+	object->refLock = PZ_NewLock(nssILockRefLock);
+    }
     if (object->refLock == NULL) {
 	goto loser;
     }
