@@ -759,7 +759,7 @@ void nsScrollingView::HandleScrollEvent(nsGUIEvent *aEvent, PRUint32 aEventFlags
   }
 }
 
-NS_IMETHODIMP_(void) nsScrollingView::Notify(nsITimer * aTimer)
+NS_IMETHODIMP nsScrollingView::Notify(nsITimer * aTimer)
 {
   nscoord xoff, yoff;
 
@@ -801,7 +801,9 @@ NS_IMETHODIMP_(void) nsScrollingView::Notify(nsITimer * aTimer)
   nsresult rv;
   mScrollingTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
   if (NS_SUCCEEDED(rv))
-    mScrollingTimer->Init(this, 25);
+    mScrollingTimer->InitWithCallback(this, 25, nsITimer::TYPE_ONE_SHOT);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsScrollingView::CreateScrollControls(nsNativeWidget aNative)
@@ -950,17 +952,17 @@ NS_IMETHODIMP nsScrollingView::SetWidget(nsIWidget *aWidget)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsScrollingView::SetZIndex(PRBool aAuto, PRInt32 aZIndex)
+NS_IMETHODIMP nsScrollingView::SetZIndex(PRBool aAuto, PRInt32 aZIndex, PRBool aTopMost)
 {
-	nsView::SetZIndex(aAuto, aZIndex);
+	nsView::SetZIndex(aAuto, aZIndex, aTopMost);
 	
 	// inform all views that the z-index has changed.
   // XXX why are we doing this? they're all a child of this view, so they
   // shouldn't need to be re-z-indexed.
-	if (mClipView) mViewManager->SetViewZIndex(mClipView, aAuto, aZIndex);
-	if (mCornerView) mViewManager->SetViewZIndex(mCornerView, aAuto, aZIndex);
-	if (mVScrollBarView) mViewManager->SetViewZIndex(mVScrollBarView, aAuto, aZIndex);
-	if (mHScrollBarView) mViewManager->SetViewZIndex(mHScrollBarView, aAuto, aZIndex);
+	if (mClipView) mViewManager->SetViewZIndex(mClipView, aAuto, aZIndex, aTopMost);
+	if (mCornerView) mViewManager->SetViewZIndex(mCornerView, aAuto, aZIndex, aTopMost);
+	if (mVScrollBarView) mViewManager->SetViewZIndex(mVScrollBarView, aAuto, aZIndex, aTopMost);
+	if (mHScrollBarView) mViewManager->SetViewZIndex(mHScrollBarView, aAuto, aZIndex, aTopMost);
 
 	return NS_OK;
 }
@@ -1684,7 +1686,7 @@ PRBool nsScrollingView::CannotBitBlt(nsView* aScrolledView)
   return (mScrollProperties & NS_SCROLL_PROPERTY_NEVER_BLIT) ||
     (scrolledViewFlags & NS_VIEW_FLAG_DONT_BITBLT) ||
     (!(mScrollProperties & NS_SCROLL_PROPERTY_ALWAYS_BLIT)
-     && !mViewManager->CanScrollWithBitBlt(mClipView));
+     && !mViewManager->CanScrollWithBitBlt(aScrolledView));
 }
 
 void nsScrollingView::Scroll(nsView *aScrolledView, PRInt32 aDx, PRInt32 aDy,
