@@ -90,10 +90,13 @@ nsView::nsView()
   MOZ_COUNT_CTOR(nsView);
 
   mVis = nsViewVisibility_kShow;
-  mVFlags = 0;
+  // Views should be transparent by default. Not being transparent is
+  // a promise that the view will paint all its pixels opaquely. Views
+  // should make this promise explicitly by calling
+  // SetViewContentTransparency.
+  mVFlags = NS_VIEW_FLAG_TRANSPARENT;
   mOpacity = 1.0f;
   mViewManager = nsnull;
-  mCompositorFlags = 0;
   mChildRemoved = PR_FALSE;
 }
 
@@ -229,11 +232,6 @@ NS_IMETHODIMP nsView::Init(nsIViewManager* aManager,
   // we don't hold a reference to the view manager
   mViewManager = NS_STATIC_CAST(nsViewManager*, aManager);
 
-  mChildClip.mLeft = 0;
-  mChildClip.mRight = 0;
-  mChildClip.mTop = 0;
-  mChildClip.mBottom = 0;
-
   SetPosition(aBounds.x, aBounds.y);
   nsRect dim(0, 0, aBounds.width, aBounds.height);
 
@@ -550,28 +548,6 @@ NS_IMETHODIMP nsView::GetBounds(nsRect &aBounds) const
   return NS_OK;
 }
 
-NS_IMETHODIMP nsView::SetChildClip(nscoord aLeft, nscoord aTop, nscoord aRight, nscoord aBottom)
-{
-  NS_PRECONDITION(aLeft <= aRight && aTop <= aBottom, "bad clip values");
-  mChildClip.mLeft = aLeft;
-  mChildClip.mTop = aTop;
-  mChildClip.mRight = aRight;
-  mChildClip.mBottom = aBottom;
-
-  
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::GetChildClip(nscoord *aLeft, nscoord *aTop, nscoord *aRight, nscoord *aBottom) const
-{
-  *aLeft = mChildClip.mLeft;
-  *aTop = mChildClip.mTop;
-  *aRight = mChildClip.mRight;
-  *aBottom = mChildClip.mBottom; 
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsView::SetVisibility(nsViewVisibility aVisibility)
 {
 
@@ -958,41 +934,17 @@ NS_IMETHODIMP nsView::GetOffsetFromWidget(nscoord *aDx, nscoord *aDy, nsIWidget
   return NS_OK;
 }
 
-NS_IMETHODIMP nsView::GetDirtyRegion(nsIRegion *&aRegion) const
+nsresult nsView::GetDirtyRegion(nsIRegion*& aRegion)
 {
-	if (nsnull == mDirtyRegion) {
-		// The view doesn't have a dirty region so create one
-		nsresult rv = nsComponentManager::CreateInstance(kRegionCID, 
-		                               nsnull, 
-		                               NS_GET_IID(nsIRegion), 
-		                               (void**) &mDirtyRegion);
+  if (nsnull == mDirtyRegion) {
+    nsresult rv = GetViewManager()->CreateRegion(&mDirtyRegion);
+    if (NS_FAILED(rv))
+       return rv;
+  }
 
-		if (NS_FAILED(rv))
-			return rv;
-		
-		rv = mDirtyRegion->Init();
-		if (NS_FAILED(rv))
-			return rv;
-	}
-
-	aRegion = mDirtyRegion;
-	NS_ADDREF(aRegion);
-	
-	return NS_OK;
-}
-
-
-NS_IMETHODIMP nsView::SetCompositorFlags(PRUint32 aFlags)
-{
-	mCompositorFlags = aFlags;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsView::GetCompositorFlags(PRUint32 *aFlags)
-{
-	NS_ASSERTION((aFlags != nsnull), "no flags");
-	*aFlags = mCompositorFlags;
-	return NS_OK;
+  aRegion = mDirtyRegion;
+  NS_ADDREF(aRegion);
+  return NS_OK;
 }
 
 PRBool nsView::IsRoot()
@@ -1023,8 +975,9 @@ PRBool nsView::PointIsInside(nsView& aView, nscoord x, nscoord y) const
 
 NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, PRBool& aEmpty) const
 {
-  // Keep track of the view's offset
-  // from its ancestor.
+  // Keep track of this view's offset from its ancestor.
+  // This is the origin of this view's parent view in the
+  // coordinate space of 'parentView' below.
   nscoord ancestorX = 0;
   nscoord ancestorY = 0;
 
@@ -1032,13 +985,51 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
   aIsClipped = PR_FALSE;
   
   GetBounds(aClippedRect);
-  const nsView* parentView = GetParent();
   PRBool lastViewIsFloating = GetFloating();
 
   // Walk all of the way up the views to see if any
   // ancestor sets the NS_VIEW_PUBLIC_FLAG_CLIPCHILDREN.
   // don't consider non-floating ancestors of a floating view.
-  while (parentView) {
+  const nsView* view = this;
+  while (PR_TRUE) {
+    const nsView* zParent = view->GetZParent();
+    const nsView* parentView = view->GetParent();
+#if 0
+    // For now, we're disabling this code. This means that we only honor clipping
+    // set by parent elements which are containing block ancestors of this content.
+
+    if (zParent) {
+      // This view was reparented. We need to continue collecting clip
+      // rects from the zParent.
+
+      // First we need to get ancestorX and ancestorY into the right
+      // coordinate system.  They are the offset of this view within
+      // parentView
+      const nsView* zParentChain;
+      for (zParentChain = zParent; zParentChain != parentView && zParentChain != nsnull;
+           zParentChain = zParentChain->GetParent()) {
+        zParentChain->ConvertFromParentCoords(&ancestorX, &ancestorY);
+      }
+      if (zParentChain == nsnull) {
+        // normally we'll hit parentView somewhere, but sometimes we
+        // don't because of odd containing block hierarchies. In this
+        // case we walked from zParentChain all the way up to the
+        // root, subtracting from ancestorX/Y. So now we walk from
+        // parentView up to the root, adding to ancestorX/Y.
+        while (parentView != nsnull) {
+          parentView->ConvertToParentCoords(&ancestorX, &ancestorY);
+          parentView = parentView->GetParent();
+        }
+      }
+      parentView = zParent;
+      // Now start again at zParent to collect all its clip information
+    }
+#endif
+    
+    if (!parentView) {
+      break;
+    }
+
     PRBool parentIsFloating = parentView->GetFloating();
     if (lastViewIsFloating && !parentIsFloating) {
       break;
@@ -1047,17 +1038,11 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
     if (parentView->GetClipChildren()) {
       aIsClipped = PR_TRUE;
       // Adjust for clip specified by ancestor
-      nscoord clipLeft;
-      nscoord clipTop;
-      nscoord clipRight;
-      nscoord clipBottom;
-      parentView->GetChildClip(&clipLeft, &clipTop, &clipRight, &clipBottom);
       nsRect clipRect;
+      parentView->GetChildClip(clipRect);
       //Offset the cliprect by the amount the child offsets from the parent
-      clipRect.x = clipLeft + ancestorX;
-      clipRect.y = clipTop + ancestorY;
-      clipRect.width = clipRight - clipLeft;
-      clipRect.height = clipBottom - clipTop;
+      clipRect.x -= ancestorX;
+      clipRect.y -= ancestorY;
       PRBool overlap = aClippedRect.IntersectRect(clipRect, aClippedRect);
       if (!overlap) {
         aEmpty = PR_TRUE; // Does not intersect so the rect is empty.
@@ -1065,13 +1050,12 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
       }
     }
 
-    parentView->ConvertFromParentCoords(&ancestorX, &ancestorY);
+    parentView->ConvertToParentCoords(&ancestorX, &ancestorY);
 
     lastViewIsFloating = parentIsFloating;
-    parentView = parentView->GetParent();
+    view = parentView;
   }
  
   return NS_OK;
 }
 
-
