@@ -52,7 +52,7 @@ static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 nsWebBrowserChrome::nsWebBrowserChrome() : mBrowserWindow(nsnull), mTimerSet(PR_FALSE)
 
 {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 
    mActiveDocuments = 0;
    mChromeFlags = 0;
