@@ -172,7 +172,7 @@ nsWebCrawler::nsWebCrawler(nsViewerApp* aViewer)
   : mHaveURLList(PR_FALSE),
     mQueuedLoadURLs(0)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mBrowser = nsnull;
   mViewer = aViewer;
@@ -245,15 +245,15 @@ nsWebCrawler::DumpRegressionData()
           switch (mPrinterTestType) {
           case 1:
             // dump print data to a file for regression testing
-            viewerFile->Print(PR_TRUE, fp);
+            viewerFile->Print(PR_TRUE, fp, nsnull);
             break;
           case 2:
             // visual printing tests, all go to the printer, no printer dialog
-            viewerFile->Print(PR_TRUE, nsnull);
+            viewerFile->Print(PR_TRUE, nsnull, nsnull);
             break;
           case 3:
             // visual printing tests, all go to the printer, with a printer dialog
-            viewerFile->Print(PR_FALSE, nsnull);
+            viewerFile->Print(PR_FALSE, nsnull, nsnull);
             break;
           default:
             break;
@@ -298,10 +298,9 @@ nsWebCrawler::DumpRegressionData()
             }
           }
           else {
-            char* file;
-            (void)mLastURL->GetPath(&file);
-            printf("could not open output file for %s\n", file);
-            nsCRT::free(file);
+            nsCAutoString file;
+            (void)mLastURL->GetPath(file);
+            printf("could not open output file for %s\n", file.get());
           }
         }
         else {
@@ -338,7 +337,8 @@ nsWebCrawler::LoadNextURLCallback(nsITimer *aTimer, void *aClosure)
           viewerFile->GetPrintable(&printable);
           if (PR_TRUE !=printable){
             self->mTimer = do_CreateInstance("@mozilla.org/timer;1");
-            self->mTimer->Init(LoadNextURLCallback, self, self->mDelay);
+            self->mTimer->InitWithFuncCallback(LoadNextURLCallback, self, self->mDelay,
+                                               nsITimer::TYPE_ONE_SHOT);
             return;
           }
         }
@@ -362,7 +362,7 @@ nsWebCrawler::QueueExitCallback(nsITimer *aTimer, void *aClosure)
 NS_IMETHODIMP
 nsWebCrawler::OnStateChange(nsIWebProgress* aWebProgress, 
                             nsIRequest* aRequest, 
-                            PRInt32 progressStateFlags, 
+                            PRUint32 progressStateFlags, 
                             nsresult aStatus)
 {
   // Make sure that we're being notified for _our_ shell, and not some
@@ -411,8 +411,8 @@ nsWebCrawler::OnStateChange(nsIWebProgress* aWebProgress,
     // Ignore this notification unless its for the current url. That way
     // we skip over embedded webshell notifications (e.g. frame cells,
     // iframes, etc.)
-    nsXPIDLCString spec;
-    uri->GetSpec(getter_Copies(spec));
+    nsCAutoString spec;
+    uri->GetSpec(spec);
 
     PRTime delta, cvt, rounder;
     LL_I2L(cvt, 1000);
@@ -464,10 +464,12 @@ nsWebCrawler::OnStateChange(nsIWebProgress* aWebProgress,
     }    
     
     if ((0 < mQueuedLoadURLs) || (0 < mPendingURLs.Count())) {
-      mTimer->Init(LoadNextURLCallback, this, mDelay);
+      mTimer->InitWithFuncCallback(LoadNextURLCallback, this, mDelay,
+                                   nsITimer::TYPE_ONE_SHOT);
     }
     else if (mPostExit) {
-      mTimer->Init(QueueExitCallback, this, mDelay);
+      mTimer->InitWithFuncCallback(QueueExitCallback, this, mDelay,
+                                   nsITimer::TYPE_ONE_SHOT);
     }
   }
 
@@ -480,8 +482,10 @@ nsWebCrawler::OnProgressChange(nsIWebProgress *aWebProgress,
                                PRInt32 aCurSelfProgress,
                                PRInt32 aMaxSelfProgress,
                                PRInt32 aCurTotalProgress,
-                               PRInt32 aMaxTotalProgress) {
-    return NS_ERROR_NOT_IMPLEMENTED;
+                               PRInt32 aMaxTotalProgress)
+{
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -489,7 +493,8 @@ nsWebCrawler::OnLocationChange(nsIWebProgress* aWebProgress,
                                nsIRequest* aRequest,
                                nsIURI *location)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
@@ -499,16 +504,18 @@ nsWebCrawler::OnStatusChange(nsIWebProgress* aWebProgress,
                              nsresult aStatus,
                              const PRUnichar* aMessage)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
 NS_IMETHODIMP
 nsWebCrawler::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                nsIRequest *aRequest, 
-                               PRInt32 state)
+                               PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 FILE*
@@ -519,10 +526,9 @@ nsWebCrawler::GetOutputFile(nsIURI *aURL, nsString& aOutputName)
   if (nsnull!=aURL)
   {
     char *inputFileName;
-    char* file;
-    (void)aURL->GetPath(&file);
-    nsAutoString inputFileFullPath; inputFileFullPath.AssignWithConversion(file);
-    nsCRT::free(file);
+    nsCAutoString file;
+    (void)aURL->GetPath(file);
+    NS_ConvertUTF8toUCS2 inputFileFullPath(file);
     PRInt32 fileNameOffset = inputFileFullPath.RFindChar('/');
     if (-1==fileNameOffset)
     {
@@ -613,7 +619,8 @@ nsWebCrawler::Start()
   if (docShell) {
     nsCOMPtr<nsIWebProgress> progress(do_GetInterface(docShell));
     if (progress) {
-      progress->AddProgressListener(this);
+      progress->AddProgressListener(this,
+                                    nsIWebProgress::NOTIFY_STATE_DOCUMENT);
       LoadNextURL(PR_FALSE);
     }
   }
@@ -702,16 +709,16 @@ nsWebCrawler::OkToLoad(const nsString& aURLSpec)
   rv = NS_NewURI(&url, aURLSpec);
 
   if (NS_OK == rv) {
-    nsXPIDLCString host;
-    rv = url->GetHost(getter_Copies(host));
+    nsCAutoString host;
+    rv = url->GetHost(host);
     if (rv == NS_OK) {
-      PRInt32 hostlen = PL_strlen(host);
+      PRInt32 hostlen = host.Length();
 
       // Check domains to avoid
       PRInt32 i, n = mAvoidDomains.Count();
       for (i = 0; i < n; i++) {
         nsString* s = (nsString*) mAvoidDomains.ElementAt(i);
-        if (s && EndsWith(*s, host, hostlen)) {
+        if (s && EndsWith(*s, host.get(), hostlen)) {
           printf("Avoiding '");
           fputs(NS_LossyConvertUCS2toASCII(aURLSpec).get(), stdout);
           printf("'\n");
@@ -728,7 +735,7 @@ nsWebCrawler::OkToLoad(const nsString& aURLSpec)
       }
       for (i = 0; i < n; i++) {
         nsString* s = (nsString*) mSafeDomains.ElementAt(i);
-        if (s && EndsWith(*s, host, hostlen)) {
+        if (s && EndsWith(*s, host.get(), hostlen)) {
           return PR_TRUE;
         }
       }
@@ -758,10 +765,10 @@ nsWebCrawler::FindURLsIn(nsIDocument* aDocument, nsIContent* aNode)
     // Get absolute url that tag targets
     nsAutoString base, src, absURLSpec;
     if (atom == mLinkTag) {
-      aNode->GetAttr(kNameSpaceID_HTML, mHrefAttr, src);
+      aNode->GetAttr(kNameSpaceID_None, mHrefAttr, src);
     }
     else {
-      aNode->GetAttr(kNameSpaceID_HTML, mSrcAttr, src);
+      aNode->GetAttr(kNameSpaceID_None, mSrcAttr, src);
     }
     nsCOMPtr<nsIURI> docURL;
     aDocument->GetDocumentURL(getter_AddRefs(docURL));
