@@ -38,13 +38,7 @@
 #include "nsIPref.h"
 #include "prmem.h"
 
-#ifdef XP_MAC
 #include "nsXPBaseWindow.h"
-#define NS_IMPL_IDS
-#else
-#define NS_IMPL_IDS
-#include "nsXPBaseWindow.h"
-#endif
 
 #include "nsCOMPtr.h"
 #include "nsIPrompt.h"
@@ -80,13 +74,6 @@
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 
-
-#if defined(WIN32) || defined(XP_OS2_VACPP)
-#include <strstrea.h>
-#else
-#include <strstream.h>
-#endif
-
 #include <ctype.h> // tolower
 
 // XXX For font setting below
@@ -678,7 +665,7 @@ public:
 //----------------------------------------------------------------------
 nsXPBaseWindowFactory::nsXPBaseWindowFactory()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 //----------------------------------------------------------------------
