@@ -45,7 +45,7 @@
 
 #include "nsIPlatformCharset.h"
 #include "nsICharsetConverterManager.h"
-
+#include "nsIFileURL.h"
 
 //
 // copied over from nsMacControl since we it's not being built.
@@ -124,7 +124,7 @@ NS_IMPL_ISUPPORTS1(nsFilePicker, nsIFilePicker)
 nsFilePicker::nsFilePicker()
   : mAllFilesDisplayed(PR_TRUE)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   
   // Zero out the type lists
   for (int i = 0; i < kMaxTypeListCount; i++)
@@ -208,7 +208,7 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
   
   // XXX Ignore the filter list for now....
   
-  if (mMode == modeOpen)
+  if (mMode == modeOpen || mMode == modeOpenMultiple)
     userClicksOK = GetLocalFile(title, &theFile);
   else if (mMode == modeSave)
     userClicksOK = PutLocalFile(title, defaultName, &theFile);
@@ -699,6 +699,22 @@ NS_IMETHODIMP nsFilePicker::GetDefaultString(PRUnichar **aString)
   return NS_ERROR_FAILURE;
 }
 
+//-------------------------------------------------------------------------
+//
+// The default extension to use for files
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP nsFilePicker::GetDefaultExtension(PRUnichar **aExtension)
+{
+  *aExtension = nsnull;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsFilePicker::SetDefaultExtension(const PRUnichar *aExtension)
+{
+  return NS_OK;
+}
+
 //-------------------------------------------------------------------------
 //
 // Set the display directory
