@@ -325,6 +325,40 @@ PRUint32 nsConvertCharCodeToUnicode(GdkEventKey* aGEK)
     return 0;
   }
 
+#if defined(USE_XIM) && defined(_AIX)
+  // On AIX, GDK doesn't get correct keysyms from XIM. Follow GDK 
+  // reference recommending to use the 'string' member of GdkEventKey 
+  // instead. See:
+  //
+  // developer.gnome.org/doc/API/gdk/gdk-event-structures.html#GDKEVENTKEY
+
+  PRBool controlChar = (aGEK->state & GDK_CONTROL_MASK ||
+                        aGEK->state & GDK_MOD1_MASK ||
+                        aGEK->state & GDK_MOD4_MASK);
+
+  // Use 'string' as opposed to 'keyval' when control, alt, or meta is 
+  // not pressed. This allows keyboard shortcuts to continue to function
+  // properly, while fixing input problems and mode switching in certain
+  // locales where the 'keyval' does not correspond to the actual input.
+  // See Bug #157397 and Bug #161581 for more details.
+
+  if (!controlChar && gdk_im_ready() && aGEK->length > 0 
+        && aGEK->keyval != (guint)*aGEK->string) {
+    nsGtkIMEHelper* IMEHelper = nsGtkIMEHelper::GetSingleton();
+    if (IMEHelper != nsnull) {
+      PRUnichar* unichars = IMEHelper->GetUnichars();
+      PRInt32 unilen = IMEHelper->GetUnicharsSize();
+      PRInt32 unichar_size = IMEHelper->MultiByteToUnicode(
+                                          aGEK->string, aGEK->length,
+                                          &unichars, &unilen);
+      if (unichar_size > 0) {
+        IMEHelper->SetUnichars(unichars);
+        IMEHelper->SetUnicharsSize(unilen);
+        return (long)*unichars;
+      }
+    }
+  }
+#endif
   // we're supposedly printable, let's try to convert
   long ucs = keysym2ucs(aGEK->keyval);
   if ((ucs != -1) && (ucs < 0x10000))
@@ -510,7 +544,6 @@ void handle_scrollbar_value_changed(GtkAdjustment *adj, gpointer p)
 gint handle_key_press_event_for_text(GtkObject *w, GdkEventKey* event,
                                      gpointer p)
 {
-  nsKeyEvent kevent;
   nsTextWidget* win = (nsTextWidget*)p;
 
   // work around for annoying things.
@@ -530,16 +563,18 @@ gint handle_key_press_event_for_text(GtkObject *w, GdkEventKey* event,
     return PR_TRUE;
 
   NS_ADDREF(win);
-  InitKeyEvent(event, p, kevent, NS_KEY_DOWN);
-  win->OnKey(kevent);
+  nsKeyEvent keyDownEvent;
+  InitKeyEvent(event, p, keyDownEvent, NS_KEY_DOWN);
+  win->OnKey(keyDownEvent);
 
   //
   // Second, dispatch the Key event as a key press event w/ a Unicode
   //  character code.  Note we have to check for modifier keys, since
   // gtk returns a character value for them
   //
-  InitKeyPressEvent(event,p, kevent);
-  win->OnKey(kevent);
+  nsKeyEvent keyPressEvent;
+  InitKeyPressEvent(event,p, keyPressEvent);
+  win->OnKey(keyPressEvent);
 
   NS_RELEASE(win);
   if (w)
@@ -584,7 +619,6 @@ gint handle_key_release_event_for_text(GtkObject *w, GdkEventKey* event,
 //==============================================================
 gint handle_key_press_event(GtkObject *w, GdkEventKey* event, gpointer p)
 {
-  nsKeyEvent kevent;
   nsWidget *win = (nsWidget*)p;
 
   // if there's a focused window rewrite the event to use that window.
@@ -611,12 +645,13 @@ gint handle_key_press_event(GtkObject *w, GdkEventKey* event, gpointer p)
   //   but lie about where it came from and say it is from the
   //   window that currently has focus inside our app...
   //
-  InitKeyEvent(event, win, kevent, NS_KEY_DOWN);
+  nsKeyEvent keyDownEvent;
+  InitKeyEvent(event, win, keyDownEvent, NS_KEY_DOWN);
   // if we need to suppress this NS_KEY_DOWN event, reset the flag
   if (suppressNextKeyDown == PR_TRUE)
     suppressNextKeyDown = PR_FALSE;
   else
-    win->OnKey(kevent);
+    win->OnKey(keyDownEvent);
 
 
   //
@@ -626,18 +661,20 @@ gint handle_key_press_event(GtkObject *w, GdkEventKey* event, gpointer p)
   //
 
   // Call nsConvertCharCodeToUnicode() here to get kevent.charCode 
-  InitKeyPressEvent(event, win, kevent);
+  nsKeyEvent keyPressEvent;
+  InitKeyPressEvent(event, win, keyPressEvent);
 
   if (event->length) {
-    if (kevent.charCode || kevent.keyCode) {
-      // kevent.charCode or kevent.keyCode is valid, just pass to OnKey()
-      win->OnKey(kevent);
+    if (keyPressEvent.charCode || keyPressEvent.keyCode) {
+      // keyPressEvent.charCode or keyPressEvent.keyCode is valid, just 
+      // pass to OnKey()
+      win->OnKey(keyPressEvent);
     } else if (nsGtkIMEHelper::GetSingleton()) {
       // commit request from IME
       win->IMECommitEvent(event);
     }
   } else { // for Home/End/Up/Down/Left/Right/PageUp/PageDown key
-    win->OnKey(kevent);
+    win->OnKey(keyPressEvent);
   }
 
   NS_RELEASE(win);
