@@ -56,7 +56,6 @@
 #include "nsIInternetConfigService.h"
 
 #include "nsITimer.h"
-#include "nsITimerCallback.h"
 #include "nsCRT.h"
 
 #include <Gestalt.h>
@@ -66,6 +65,8 @@
 
 #include "nsSound.h"
 
+#include "nsString.h"
+
 //#define SOUND_DEBUG
 
 #pragma mark nsSoundRequest
@@ -81,7 +82,7 @@ public:
   NS_DECL_ISUPPORTS
 
   // nsITimerCallback
-  NS_IMETHOD_(void) Notify(nsITimer *timer) = 0;    // pure virtual
+  NS_IMETHOD Notify(nsITimer *timer) = 0;
 
   virtual nsresult  PlaySound() = 0;
 
@@ -109,7 +110,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsITimerCallback
-  NS_IMETHOD_(void) Notify(nsITimer *timer);
+  NS_DECL_NSITIMERCALLBACK
 
   nsresult          Init(nsISound* aSound, ConstStr255Param aSoundName);
   virtual nsresult  PlaySound();
@@ -143,7 +144,7 @@ public:
   NS_DECL_NSISTREAMLOADEROBSERVER
 
   // nsITimerCallback
-  NS_IMETHOD_(void) Notify(nsITimer *timer);
+  NS_DECL_NSITIMERCALLBACK
 
   nsresult          Init(nsISound* aSound, nsIURL *aURL);
   virtual nsresult  PlaySound();
@@ -433,9 +434,9 @@ nsSound::GetSoundResourceName(const char* inSoundName, StringPtr outResourceName
     nsCOMPtr <nsIInternetConfigService> icService = do_GetService(NS_INTERNETCONFIGSERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv))
       return rv;
-
-    nsXPIDLCString  newMailSound;
-    rv = icService->GetString(nsIInternetConfigService::eICString_NewMailSoundName, getter_Copies(newMailSound));
+    
+    nsCAutoString  newMailSound;
+    rv = icService->GetString(nsIInternetConfigService::eICString_NewMailSoundName, newMailSound);
     if (NS_FAILED(rv))
       return rv;
       
@@ -598,8 +599,8 @@ nsSystemSoundRequest::PlaySound()
   }
   
   const PRInt32   kSoundTimerInterval = 250;      // 250 milliseconds
-  rv = mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kSoundTimerInterval,
-          PR_TRUE, NS_TYPE_REPEATING_PRECISE);
+  rv = mTimer->InitWithCallback(NS_STATIC_CAST(nsITimerCallback*, this), kSoundTimerInterval,
+                                nsITimer::TYPE_REPEATING_PRECISE);
   if (NS_FAILED(rv)) {
     Cleanup();
     return rv;
@@ -608,13 +609,14 @@ nsSystemSoundRequest::PlaySound()
   return NS_OK;
 }
 
-NS_IMETHODIMP_(void)
+NS_IMETHODIMP
 nsSystemSoundRequest::Notify(nsITimer *timer)
 {
   if (mSoundDone)
   {
     Cleanup();
   }
+  return NS_OK;
 }
 
 
@@ -854,8 +856,8 @@ nsMovieSoundRequest::PlaySound()
     }
     
     const PRInt32   kMovieTimerInterval = 250;      // 250 milliseconds
-    rv = mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kMovieTimerInterval,
-            PR_TRUE, NS_TYPE_REPEATING_PRECISE);
+    rv = mTimer->InitWithCallback(NS_STATIC_CAST(nsITimerCallback*, this), kMovieTimerInterval,
+                                  nsITimer::TYPE_REPEATING_PRECISE);
     if (NS_FAILED(rv)) {
       Cleanup();
       return rv;
@@ -899,13 +901,13 @@ nsMovieSoundRequest::PlaySound()
   return NS_OK;
 }
 
-NS_IMETHODIMP_(void)
+NS_IMETHODIMP
 nsMovieSoundRequest::Notify(nsITimer *timer)
 {
   if (!mMovie)
   {
     NS_ASSERTION(0, "nsMovieSoundRequest has no movie in timer callback");
-    return;
+    return NS_OK;
   }
   
 #ifdef SOUND_DEBUG
@@ -920,6 +922,7 @@ nsMovieSoundRequest::Notify(nsITimer *timer)
   // so won't necessarily go away.
   if (moviesDone)
     Cleanup();
+  return NS_OK;
 }
 
 OSErr
