@@ -30,15 +30,16 @@
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
 #include "nsIServiceManager.h"
-#define NS_IMPL_IDS
 #include "nsIPlatformCharset.h"
-#undef NS_IMPL_IDS
 #include "nsWidgetDefs.h"
 #include "nsFilePicker.h"
 #include "nsILocalFile.h"
 #include "nsIURL.h"
-#include "nsIFileChannel.h"
+#include "nsIFileURL.h"
 #include "nsIStringBundle.h"
+#include "nsEnumeratorUtils.h"
+#include "nsModule.h"
+#include "nsCRT.h"
 
 /* Item structure */
 typedef struct _MyData
@@ -52,6 +53,8 @@ static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CI
 
 NS_IMPL_ISUPPORTS1(nsFilePicker, nsIFilePicker)
 
+char nsFilePicker::mLastUsedDirectory[MAX_PATH+1] = { 0 };
+
 MRESULT EXPENTRY DirDialogProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
 MRESULT EXPENTRY FileDialogProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
 
@@ -62,10 +65,11 @@ MRESULT EXPENTRY FileDialogProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2
 //-------------------------------------------------------------------------
 nsFilePicker::nsFilePicker()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mWnd = NULL;
   mUnicodeEncoder = nsnull;
   mUnicodeDecoder = nsnull;
+  mSelectedType   = 0;
   mDisplayDirectory = do_CreateInstance("@mozilla.org/file/local;1");
   pszFDFileExists[0] = '\0';
 }
@@ -100,15 +104,18 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     mDefault.ToCString(fileBuffer,MAX_PATH);
   }
   else {
-    PL_strcpy(fileBuffer, converted);
+    PL_strncpyz(fileBuffer, converted, MAX_PATH+1);
     nsMemory::Free( converted );
   }
 
   char *title = ConvertToFileSystemCharset(mTitle.get());
   if (nsnull == title)
     title = ToNewCString(mTitle);
-  char *initialDir;
-  mDisplayDirectory->GetPath(&initialDir);
+  nsCAutoString initialDir;
+  mDisplayDirectory->GetNativePath(initialDir);
+  // If no display directory, re-use the last one.
+  if(initialDir.IsEmpty())
+    initialDir = mLastUsedDirectory;
 
   mFile.SetLength(0);
 
@@ -118,9 +125,11 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
   filedlg.pszTitle = title;
 
   if (mMode == modeGetFolder) {
+    PL_strncat(filedlg.szFullFile, initialDir.get(), MAX_PATH);
+    PL_strncat(filedlg.szFullFile, "\\", 1);
+    PL_strncat(filedlg.szFullFile, "^", 1);
     filedlg.fl = FDS_OPEN_DIALOG | FDS_CENTER;
     filedlg.pfnDlgProc = DirDialogProc;
-    strcpy(filedlg.szFullFile, "^");
     DosError(FERR_DISABLEHARDERR);
     WinFileDlg(HWND_DESKTOP, mWnd, &filedlg);
     DosError(FERR_ENABLEHARDERR);
@@ -128,14 +137,19 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     *tempptr = '\0';
     if (filedlg.lReturn == DID_OK) {
       result = PR_TRUE;
-      mDisplayDirectory->InitWithPath(filedlg.szFullFile);
+      mDisplayDirectory->InitWithNativePath(nsDependentCString(filedlg.szFullFile));
       mFile.Append(filedlg.szFullFile);
     }
   }
   else {
+    PL_strncpy(filedlg.szFullFile, initialDir.get(), MAX_PATH);
+    PL_strncat(filedlg.szFullFile, "\\", 1);
+    PL_strncat(filedlg.szFullFile, fileBuffer, MAX_PATH);
     filedlg.fl = FDS_CENTER;
     if (mMode == modeSave) {
        filedlg.fl |= FDS_SAVEAS_DIALOG | FDS_ENABLEFILELB;
+    } else if (mMode == modeOpenMultiple) {
+       filedlg.fl |= FDS_MULTIPLESEL | FDS_OPEN_DIALOG;
     } else {
        filedlg.fl |= FDS_OPEN_DIALOG;
     }
@@ -144,15 +158,6 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     memset(pmydata, 0, sizeof(MYDATA));
     filedlg.ulUser = (ULONG)pmydata;
     filedlg.pfnDlgProc = FileDialogProc;
-    if (initialDir[0]) {
-      strcpy(filedlg.szFullFile, initialDir);
-      strcat(filedlg.szFullFile, "\\");
-    } else if (lastPath[0]) {
-      strcpy(filedlg.szFullFile, lastPath);
-      strcat(filedlg.szFullFile, "\\");
-    }
-
-    strcat(filedlg.szFullFile, fileBuffer);
 
     int i;
 
@@ -161,7 +166,16 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     for (i = 0; i < mTitles.Count(); i++)
     {
       const nsString& typeWide = *mTitles[i];
-      apszTypeList[i] = ToNewCString(typeWide);
+      PRInt32 l = (typeWide.Length()+2)*2;
+      char *filterBuffer = (char*) nsMemory::Alloc(l);
+      int len = WideCharToMultiByte(0,
+                                    typeWide.get(),
+                                    typeWide.Length(),
+                                    filterBuffer,
+                                    l);
+      filterBuffer[len] = NULL;
+      filterBuffer[len+1] = NULL;
+      apszTypeList[i] = filterBuffer;
     }
     apszTypeList[i] = 0;
     filedlg.papszITypeList = (PAPSZ)apszTypeList;
@@ -176,6 +190,8 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     apszFilterList[i] = 0;
     pmydata->papszIFilterList = (PAPSZ)apszFilterList;
 
+    pmydata->ulCurExt = mSelectedType;
+
     PRBool fileExists;
     do {
       DosError(FERR_DISABLEHARDERR);
@@ -218,6 +234,40 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
       }
     } while (mMode == modeSave && fileExists && filedlg.lReturn == DID_OK);
 
+    if (filedlg.lReturn == DID_OK) {
+      result = PR_TRUE;
+      if (mMode == modeOpenMultiple) {
+        nsresult rv = NS_NewISupportsArray(getter_AddRefs(mFiles));
+        NS_ENSURE_SUCCESS(rv,rv);
+
+        if (filedlg.papszFQFilename) {
+          for (int i=0;i<filedlg.ulFQFCount;i++) {
+            nsCOMPtr<nsILocalFile> file = do_CreateInstance("@mozilla.org/file/local;1", &rv);
+            NS_ENSURE_SUCCESS(rv,rv);
+
+            rv = file->InitWithNativePath(nsDependentCString(*(filedlg.papszFQFilename)[i]));
+            NS_ENSURE_SUCCESS(rv,rv);
+
+            rv = mFiles->AppendElement(file);
+            NS_ENSURE_SUCCESS(rv,rv);
+          }
+          WinFreeFileDlgList(filedlg.papszFQFilename);
+        } else {
+          nsCOMPtr<nsILocalFile> file = do_CreateInstance("@mozilla.org/file/local;1", &rv);
+          NS_ENSURE_SUCCESS(rv,rv);
+
+          rv = file->InitWithNativePath(nsDependentCString(filedlg.szFullFile));
+          NS_ENSURE_SUCCESS(rv,rv);
+
+          rv = mFiles->AppendElement(file);
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
+      } else {
+        mFile.Append(filedlg.szFullFile);
+      }
+      mSelectedType = (PRInt16)pmydata->ulCurExt;
+    }
+
     for (i = 0; i < mTitles.Count(); i++)
     {
       nsMemory::Free(*(filedlg.papszITypeList[i]));
@@ -230,20 +280,6 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     }
     free(pmydata->papszIFilterList);
     free(pmydata);
-
-
-    if (filedlg.lReturn == DID_OK) {
-      result = PR_TRUE;
-      mFile.Append(filedlg.szFullFile);
-
-      char* temp = (char*)gWidgetModuleData->DBCSstrrchr(filedlg.szFullFile, '\\');
-      *temp = '\0';
-      strcpy(lastPath, filedlg.szFullFile);
-
-      // Store the current directory in mDisplayDirectory
-      mDisplayDirectory->InitWithPath(filedlg.szFullFile);
-      mSelectedType = (PRInt16)pmydata->ulCurExt;
-    }
   }
 
   if (title)
@@ -252,15 +288,28 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
   if (result) {
     PRInt16 returnOKorReplace = returnOK;
 
+    // Remember last used directory.
+    nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
+    NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
+
+    file->InitWithNativePath(mFile);
+    nsCOMPtr<nsIFile> dir;
+    if (NS_SUCCEEDED(file->GetParent(getter_AddRefs(dir)))) {
+      nsCOMPtr<nsILocalFile> localDir(do_QueryInterface(dir));
+      if (localDir) {
+        nsCAutoString newDir;
+        localDir->GetNativePath(newDir);
+        if(!newDir.IsEmpty())
+          PL_strncpyz(mLastUsedDirectory, newDir.get(), MAX_PATH+1);
+        // Update mDisplayDirectory with this directory, also.
+        // Some callers rely on this.
+        mDisplayDirectory->InitWithNativePath( nsDependentCString(mLastUsedDirectory) );
+      }
+    }
+
     if (mMode == modeSave) {
       // Windows does not return resultReplace,
       //   we must check if file already exists
-      nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
-
-      NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
-
-      file->InitWithPath(mFile.get());
-
       PRBool exists = PR_FALSE;
       file->Exists(&exists);
       if (exists)
@@ -287,7 +336,7 @@ NS_IMETHODIMP nsFilePicker::GetFile(nsILocalFile **aFile)
     
   NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
 
-  file->InitWithPath(mFile.get());
+  file->InitWithNativePath(mFile);
 
   NS_ADDREF(*aFile = file);
 
@@ -299,7 +348,7 @@ NS_IMETHODIMP nsFilePicker::GetFileURL(nsIFileURL **aFileURL)
 {
   nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
   NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
-  file->InitWithPath(mFile.get());
+  file->InitWithNativePath(mFile);
 
   nsCOMPtr<nsIURI> uri;
   NS_NewFileURI(getter_AddRefs(uri), file);
@@ -311,6 +360,12 @@ NS_IMETHODIMP nsFilePicker::GetFileURL(nsIFileURL **aFileURL)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsFilePicker::GetFiles(nsISimpleEnumerator **aFiles)
+{
+  NS_ENSURE_ARG_POINTER(aFiles);
+  return NS_NewArrayEnumerator(aFiles, mFiles);
+}
+
 //-------------------------------------------------------------------------
 //
 // Get the file + path
@@ -319,6 +374,40 @@ NS_IMETHODIMP nsFilePicker::GetFileURL(nsIFileURL **aFileURL)
 NS_IMETHODIMP nsFilePicker::SetDefaultString(const PRUnichar *aString)
 {
   mDefault = aString;
+
+  //First, make sure the file name is not too long!
+  PRInt32 nameLength;
+  PRInt32 nameIndex = mDefault.RFind("\\");
+  if (nameIndex == kNotFound)
+    nameIndex = 0;
+  else
+    nameIndex ++;
+  nameLength = mDefault.Length() - nameIndex;
+  
+  if (nameLength > CCHMAXPATH) {
+    PRInt32 extIndex = mDefault.RFind(".");
+    if (extIndex == kNotFound)
+      extIndex = mDefault.Length();
+
+    //Let's try to shave the needed characters from the name part
+    PRInt32 charsToRemove = nameLength - CCHMAXPATH;
+    if (extIndex - nameIndex >= charsToRemove) {
+      mDefault.Cut(extIndex - charsToRemove, charsToRemove);
+    }
+  }
+
+  //Then, we need to replace illegal characters.
+  //Windows has the following statement:
+  //At this stage, we cannot replace the backslash as the string might represent a file path.
+  //But it is not correct - Windows assumes this is not a path as well,
+  //as one of the FILE_ILLEGAL_CHARACTERS is a colon (:)
+
+  mDefault.ReplaceChar("\"", '\'');
+  mDefault.ReplaceChar("\<", '(');
+  mDefault.ReplaceChar("\>", ')');
+
+  mDefault.ReplaceChar(FILE_ILLEGAL_CHARACTERS, '_');
+
   return NS_OK;
 }
 
@@ -327,6 +416,42 @@ NS_IMETHODIMP nsFilePicker::GetDefaultString(PRUnichar **aString)
   return NS_ERROR_FAILURE;
 }
 
+//-------------------------------------------------------------------------
+//
+// The default extension to use for files
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP nsFilePicker::GetDefaultExtension(PRUnichar **aExtension)
+{
+  *aExtension = ToNewUnicode(mDefaultExtension);
+  if (!*aExtension)
+    return NS_ERROR_OUT_OF_MEMORY;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsFilePicker::SetDefaultExtension(const PRUnichar *aExtension)
+{
+  mDefaultExtension = aExtension;
+  return NS_OK;
+}
+
+//-------------------------------------------------------------------------
+//
+// Set the filter index
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP nsFilePicker::GetFilterIndex(PRInt32 *aFilterIndex)
+{
+  *aFilterIndex = mSelectedType;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
+{
+  mSelectedType = aFilterIndex;
+  return NS_OK;
+}
+
 //-------------------------------------------------------------------------
 //
 // Set the display directory
@@ -441,7 +566,7 @@ PRUnichar * nsFilePicker::ConvertFromFileSystemCharset(const char *inString)
 
   // converts from the file system charset to unicode
   if (NS_SUCCEEDED(rv)) {
-    PRInt32 inLength = nsCRT::strlen(inString);
+    PRInt32 inLength = strlen(inString);
     PRInt32 outLength;
     rv = mUnicodeDecoder->GetMaxLength(inString, inLength, &outLength);
     if (NS_SUCCEEDED(rv)) {
@@ -594,12 +719,13 @@ MRESULT EXPENTRY FileDialogProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2
                                         NULL, NULL );
        WinSendMsg( hwndTypeCombo, LM_DELETEALL, (MPARAM)0, (MPARAM)0 );
        pfiledlg = (PFILEDLG)WinQueryWindowULong( hwndDlg, QWL_USER );
+       pmydata = (PMYDATA)pfiledlg->ulUser;
        i = 0;
        while (*(pfiledlg->papszITypeList[i]) != NULL) {
            WinSendMsg( hwndTypeCombo, LM_INSERTITEM, (MPARAM)LIT_END, (MPARAM)*(pfiledlg->papszITypeList[i]) );
            i++;
        }
-       WinSendMsg( hwndTypeCombo, LM_SELECTITEM, (MPARAM)0, (MPARAM)TRUE );
+       WinSendMsg( hwndTypeCombo, LM_SELECTITEM, (MPARAM)pmydata->ulCurExt, (MPARAM)TRUE );
 
        return mr;
     case WM_CONTROL:
