@@ -19,13 +19,9 @@
  * 
  * Contributor(s):
  *   Stuart Parmenter <pavlov@netscape.com>
+ *   Seth Spitzer <sspitzer@netscape.com>
  */
 
-// Define so header files for openfilename are included
-#ifdef WIN32_LEAN_AND_MEAN
-#undef WIN32_LEAN_AND_MEAN
-#endif
-
 #include "nsCOMPtr.h"
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
@@ -36,10 +32,15 @@
 #include "nsIURL.h"
 #include "nsIFileURL.h"
 #include "nsIStringBundle.h"
+#include "nsEnumeratorUtils.h"
 #include "nsCRT.h"
 #include <windows.h>
 #include <SHLOBJ.H>
 
+// commdlg.h and cderr.h are needed to build with WIN32_LEAN_AND_MEAN
+#include <commdlg.h>
+#include <cderr.h>
+
 #ifdef MOZ_UNICODE
 #include "nsString.h"
 #include "nsToolkit.h"
@@ -63,7 +64,7 @@ char nsFilePicker::mLastUsedDirectory[MAX_PATH+1] = { 0 };
 //-------------------------------------------------------------------------
 nsFilePicker::nsFilePicker()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mWnd = NULL;
   mUnicodeEncoder = nsnull;
   mUnicodeDecoder = nsnull;
@@ -124,9 +125,9 @@ NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
     browserInfo.iImage         = nsnull;
 
     // XXX UNICODE support is needed here --> DONE
-    LPITEMIDLIST list = ::SHBrowseForFolderW(&browserInfo);
+    LPITEMIDLIST list = nsToolkit::mSHBrowseForFolder(&browserInfo);
     if (list != NULL) {
-      result = ::SHGetPathFromIDListW(list, (LPWSTR)fileBuffer);
+      result = nsToolkit::mSHGetPathFromIDList(list, (LPWSTR)fileBuffer);
       if (result) {
           mUnicodeFile.Append(fileBuffer);
       }
@@ -191,11 +192,15 @@ NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
     if (mMode == modeOpen) {
       // FILE MUST EXIST!
       ofn.Flags |= OFN_FILEMUSTEXIST;
-      result = ::GetOpenFileNameW(&ofn);
+      result = nsToolkit::mGetOpenFileName(&ofn);
+    }
+    else if (mMode == modeOpenMultiple) {
+      ofn.Flags |= OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
+      result = nsToolkit::mGetOpenFileName(&ofn);
     }
     else if (mMode == modeSave) {
       ofn.Flags |= OFN_NOREADONLYRETURN;
-      result = ::GetSaveFileNameW(&ofn);
+      result = nsToolkit::mGetSaveFileName(&ofn);
       if (!result) {
         // Error, find out what kind.
         if (::GetLastError() == ERROR_INVALID_PARAMETER ||
@@ -203,12 +208,12 @@ NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
           // probably the default file name is too long or contains illegal characters!
           // Try again, without a starting file name.
           ofn.lpstrFile[0] = 0;
-          result = ::GetSaveFileNameW(&ofn);
+          result = nsToolkit::mGetSaveFileName(&ofn);
         }
       }
     }
     else {
-      NS_ASSERTION(0, "Only load, save and getFolder are supported modes"); 
+      NS_ASSERTION(0, "unsupported mode"); 
     }
   
     // Remember what filter type the user selected
@@ -216,9 +221,58 @@ NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
 
     // Set user-selected location of file or directory
     if (result == PR_TRUE) {
-      // I think it also needs a conversion here (to unicode since appending to nsString) 
-      // but doing that generates garbage file name, weird.
-      mUnicodeFile.Append(fileBuffer);
+      if (mMode == modeOpenMultiple) {
+        nsresult rv = NS_NewISupportsArray(getter_AddRefs(mFiles));
+        NS_ENSURE_SUCCESS(rv,rv);
+        
+        // from msdn.microsoft.com, "Open and Save As Dialog Boxes" section:
+        // If you specify OFN_EXPLORER,
+        // The directory and file name strings are NULL separated, 
+        // with an extra NULL character after the last file name. 
+        // This format enables the Explorer-style dialog boxes
+        // to return long file names that include spaces. 
+        PRUnichar *current = fileBuffer;
+        
+        nsAutoString dirName(current);
+        // sometimes dirName contains a trailing slash
+        // and sometimes it doesn't.
+        if (current[dirName.Length() - 1] != '\\')
+          dirName.Append((PRUnichar)'\\');
+        
+        while (current && *current && *(current + nsCRT::strlen(current) + 1)) {
+          current = current + nsCRT::strlen(current) + 1;
+          
+          nsCOMPtr<nsILocalFile> file = do_CreateInstance("@mozilla.org/file/local;1", &rv);
+          NS_ENSURE_SUCCESS(rv,rv);
+          
+          rv = file->InitWithPath(dirName + nsDependentString(current));
+          NS_ENSURE_SUCCESS(rv,rv);
+          
+          rv = mFiles->AppendElement(file);
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
+        
+        // handle the case where the user selected just one
+        // file.  according to msdn.microsoft.com:
+        // If you specify OFN_ALLOWMULTISELECT and the user selects 
+        // only one file, the lpstrFile string does not have 
+        // a separator between the path and file name.
+        if (current && *current && (current == fileBuffer)) {
+          nsCOMPtr<nsILocalFile> file = do_CreateInstance("@mozilla.org/file/local;1", &rv);
+          NS_ENSURE_SUCCESS(rv,rv);
+          
+          rv = file->InitWithPath(nsDependentString(current));
+          NS_ENSURE_SUCCESS(rv,rv);
+          
+          rv = mFiles->AppendElement(file);
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
+      }
+      else {
+        // I think it also needs a conversion here (to unicode since appending to nsString) 
+        // but doing that generates garbage file name, weird.
+        mUnicodeFile.Append(fileBuffer);
+      }
     }
 
   }
@@ -348,8 +402,8 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *aReturnVal)
                                   mFilterList.Length(),
                                   filterBuffer,
                                   l, NULL, NULL);
-    filterBuffer[len] = NULL;
-    filterBuffer[len+1] = NULL;
+    filterBuffer[len] = '\0';
+    filterBuffer[len+1] = '\0';
                                   
     if (!initialDir.IsEmpty()) {
       ofn.lpstrInitialDir = initialDir.get();
@@ -404,6 +458,10 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *aReturnVal)
       ofn.Flags |= OFN_FILEMUSTEXIST;
       result = ::GetOpenFileName(&ofn);
     }
+    else if (mMode == modeOpenMultiple) {
+      ofn.Flags |= OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
+      result = ::GetOpenFileName(&ofn);
+    }
     else if (mMode == modeSave) {
       ofn.Flags |= OFN_NOREADONLYRETURN;
       result = ::GetSaveFileName(&ofn);
@@ -433,9 +491,57 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *aReturnVal)
     if (result == PR_TRUE) {
       // I think it also needs a conversion here (to unicode since appending to nsString) 
       // but doing that generates garbage file name, weird.
-      mFile.Append(fileBuffer);
-    }
+      if (mMode == modeOpenMultiple) {
+        nsresult rv = NS_NewISupportsArray(getter_AddRefs(mFiles));
+        NS_ENSURE_SUCCESS(rv,rv);
 
+        // from msdn.microsoft.com, "Open and Save As Dialog Boxes" section:
+        // If you specify OFN_EXPLORER,
+        // The directory and file name strings are NULL separated, 
+        // with an extra NULL character after the last file name. 
+        // This format enables the Explorer-style dialog boxes
+        // to return long file names that include spaces. 
+        char *current = fileBuffer;
+
+        nsCAutoString dirName(current);
+        // sometimes dirName contains a trailing slash
+        // and sometimes it doesn't.
+        if (current[dirName.Length() - 1] != '\\')
+          dirName += '\\';
+
+        while (current && *current && *(current + strlen(current) + 1)) {
+          current = current + strlen(current) + 1;
+          
+          nsCOMPtr<nsILocalFile> file = do_CreateInstance("@mozilla.org/file/local;1", &rv);
+          NS_ENSURE_SUCCESS(rv,rv);
+
+          rv = file->InitWithNativePath(dirName + nsDependentCString(current));
+          NS_ENSURE_SUCCESS(rv,rv);
+           
+          rv = mFiles->AppendElement(file);
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
+         
+        // handle the case where the user selected just one
+        // file.  according to msdn.microsoft.com:
+        // If you specify OFN_ALLOWMULTISELECT and the user selects 
+        // only one file, the lpstrFile string does not have 
+        // a separator between the path and file name.
+        if (current && *current && (current == fileBuffer)) {
+          nsCOMPtr<nsILocalFile> file = do_CreateInstance("@mozilla.org/file/local;1", &rv);
+          NS_ENSURE_SUCCESS(rv,rv);
+
+          rv = file->InitWithNativePath(nsDependentCString(current));
+          NS_ENSURE_SUCCESS(rv,rv);
+           
+          rv = mFiles->AppendElement(file);
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
+      }
+      else {
+        mFile.Append(fileBuffer);
+      }
+    }
   }
 
   if (title)
@@ -528,6 +634,12 @@ NS_IMETHODIMP nsFilePicker::GetFileURL(nsIFileURL **aFileURL)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsFilePicker::GetFiles(nsISimpleEnumerator **aFiles)
+{
+  NS_ENSURE_ARG_POINTER(aFiles);
+  return NS_NewArrayEnumerator(aFiles, mFiles);
+}
+
 //-------------------------------------------------------------------------
 //
 // Get the file + path
@@ -749,9 +861,16 @@ nsFilePicker::AppendFilter(const PRUnichar *aTitle, const PRUnichar *aFilter)
   mFilterList.Append(PRUnichar('\0'));
 
   if (!nsCRT::strcmp(aFilter, NS_LITERAL_STRING("..apps").get()))
-    mFilterList.Append(NS_LITERAL_STRING("*.exe; *.com"));
+    mFilterList.Append(NS_LITERAL_STRING("*.exe;*.com"));
   else
-    mFilterList.Append(aFilter);
+  {
+    nsString filter;
+    filter.Assign(aFilter);
+    filter.StripWhitespace();
+    if (filter.Equals(NS_LITERAL_STRING("*")))
+      filter.Append(NS_LITERAL_STRING(".*"));
+    mFilterList.Append(filter);
+  }
 
   mFilterList.Append(PRUnichar('\0'));
 
