@@ -49,16 +49,6 @@
 
 #ifdef XPCOM_GLUE
 // nsTraceRefcnt needs a cleaning...
-#undef NS_LOG_ADDREF
-#undef NS_LOG_RELEASE
-#undef NS_LOG_NEW_XPCOM
-#undef NS_LOG_DELETE_XPCOM
-#undef NS_LOG_ADDREF_CALL
-#undef NS_LOG_RELEASE_CALL
-#undef MOZ_DECL_CTOR_COUNTER
-#undef MOZ_COUNT_CTOR
-#undef MOZ_COUNT_DTOR
-
 #define NS_LOG_ADDREF(_p, _rc, _type, _size)
 #define NS_LOG_RELEASE(_p, _rc, _type)
 #define NS_LOG_NEW_XPCOM(_p,_type,_size,_file,_line)
@@ -95,7 +85,27 @@ extern "C" NS_COM void NS_CheckThreadSafe(void* owningThread,
 
 #endif // !(defined(NS_DEBUG) && defined(NS_MT_SUPPORTED))
 
-////////////////////////////////////////////////////////////////////////////////
+class nsAutoRefCnt {
+
+ public:
+    nsAutoRefCnt() : mValue(0) {}
+    nsAutoRefCnt(nsrefcnt aValue) : mValue(aValue) {}
+
+    // only support prefix increment/decrement
+    nsrefcnt operator++() { return ++mValue; }
+    nsrefcnt operator--() { return --mValue; }
+    
+    nsrefcnt operator=(nsrefcnt aValue) { return (mValue = aValue); }
+    operator nsrefcnt() const { return mValue; }
+    nsrefcnt get() const { return mValue; }
+ private:
+    // do not define these to enforce the faster prefix notation
+    nsrefcnt operator++(int);
+    nsrefcnt operator--(int);
+    nsrefcnt mValue;
+};
+
+///////////////////////////////////////////////////////////////////////////////
 
 /**
  * Declare the reference count variable and the implementations of the
@@ -109,7 +119,7 @@ public:                                                                       \
   NS_IMETHOD_(nsrefcnt) AddRef(void);                                         \
   NS_IMETHOD_(nsrefcnt) Release(void);                                        \
 protected:                                                                    \
-  nsrefcnt mRefCnt;                                                           \
+  nsAutoRefCnt mRefCnt;                                                       \
   NS_DECL_OWNINGTHREAD                                                        \
 public:
 
@@ -120,7 +130,7 @@ public:
  * Initialize the reference count variable. Add this to each and every
  * constructor you implement.
  */
-#define NS_INIT_ISUPPORTS() (mRefCnt = 0, NS_IMPL_OWNINGTHREAD())
+#define NS_INIT_ISUPPORTS() (NS_IMPL_OWNINGTHREAD())
 
 /**
  * Use this macro to implement the AddRef method for a given <i>_class</i>
@@ -247,6 +257,11 @@ NS_IMETHODIMP _class::QueryInterface(REFNSIID aIID, void** aInstancePtr)      \
     foundInterface = NS_STATIC_CAST(_interface*, this);                       \
   else
 
+#define NS_IMPL_QUERY_BODY_CONDITIONAL(_interface, condition)                 \
+  if ( (condition) && aIID.Equals(NS_GET_IID(_interface)))                    \
+    foundInterface = NS_STATIC_CAST(_interface*, this);                       \
+  else
+
 #define NS_IMPL_QUERY_BODY_AMBIGUOUS(_interface, _implClass)                  \
   if ( aIID.Equals(NS_GET_IID(_interface)) )                                  \
     foundInterface = NS_STATIC_CAST(_interface*,                              \
@@ -314,6 +329,8 @@ NS_IMETHODIMP _class::QueryInterface(REFNSIID aIID, void** aInstancePtr)      \
   */
 #define NS_INTERFACE_MAP_BEGIN(_implClass)      NS_IMPL_QUERY_HEAD(_implClass)
 #define NS_INTERFACE_MAP_ENTRY(_interface)      NS_IMPL_QUERY_BODY(_interface)
+#define NS_INTERFACE_MAP_ENTRY_CONDITIONAL(_interface, condition)             \
+  NS_IMPL_QUERY_BODY_CONDITIONAL(_interface, condition)
 #define NS_INTERFACE_MAP_ENTRY_AGGREGATED(_interface,_aggregate)              \
   NS_IMPL_QUERY_BODY_AGGREGATED(_interface,_aggregate)
 
@@ -438,6 +455,24 @@ NS_IMETHODIMP _class::QueryInterface(REFNSIID aIID, void** aInstancePtr)      \
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, _i1)                        \
   NS_INTERFACE_MAP_END
 
+#define NS_IMPL_QUERY_INTERFACE11(_class, _i1, _i2, _i3, _i4, _i5, _i6,       \
+                                  _i7, _i8, _i9, _i10, _i11)                  \
+  NS_INTERFACE_MAP_BEGIN(_class)                                              \
+    NS_INTERFACE_MAP_ENTRY(_i1)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i2)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i3)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i4)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i5)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i6)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i7)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i8)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i9)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i10)                                              \
+    NS_INTERFACE_MAP_ENTRY(_i11)                                              \
+    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, _i1)                        \
+  NS_INTERFACE_MAP_END
+
+
 #define NS_IMPL_THREADSAFE_QUERY_INTERFACE0  NS_IMPL_QUERY_INTERFACE0
 #define NS_IMPL_THREADSAFE_QUERY_INTERFACE1  NS_IMPL_QUERY_INTERFACE1
 #define NS_IMPL_THREADSAFE_QUERY_INTERFACE2  NS_IMPL_QUERY_INTERFACE2
@@ -449,6 +484,7 @@ NS_IMETHODIMP _class::QueryInterface(REFNSIID aIID, void** aInstancePtr)      \
 #define NS_IMPL_THREADSAFE_QUERY_INTERFACE8  NS_IMPL_QUERY_INTERFACE8
 #define NS_IMPL_THREADSAFE_QUERY_INTERFACE9  NS_IMPL_QUERY_INTERFACE9
 #define NS_IMPL_THREADSAFE_QUERY_INTERFACE10  NS_IMPL_QUERY_INTERFACE10
+#define NS_IMPL_THREADSAFE_QUERY_INTERFACE11  NS_IMPL_QUERY_INTERFACE11
 
 /**
  * Declare that you're going to inherit from something that already
@@ -604,6 +640,13 @@ NS_IMETHODIMP_(nsrefcnt) Class::Release(void)                                 \
   NS_IMPL_QUERY_INTERFACE10(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7, _i8,   \
                             _i9, _i10)
 
+#define NS_IMPL_ISUPPORTS11(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7, _i8,   \
+                            _i9, _i10, _i11)                                  \
+  NS_IMPL_ADDREF(_class)                                                      \
+  NS_IMPL_RELEASE(_class)                                                     \
+  NS_IMPL_QUERY_INTERFACE11(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7, _i8,   \
+                            _i9, _i10, _i11)
+
 #define NS_IMPL_ISUPPORTS_INHERITED0(Class, Super)                            \
     NS_IMPL_QUERY_INTERFACE_INHERITED0(Class, Super)                          \
     NS_IMPL_ADDREF_INHERITED(Class, Super)                                    \
@@ -764,6 +807,13 @@ NS_IMETHODIMP_(nsrefcnt) _class::Release(void)                                \
   NS_IMPL_THREADSAFE_QUERY_INTERFACE10(_class, _i1, _i2, _i3, _i4, _i5, _i6,  \
                                        _i7, _i8, _i9, _i10)
 
+#define NS_IMPL_THREADSAFE_ISUPPORTS11(_class, _i1, _i2, _i3, _i4, _i5, _i6,  \
+                                       _i7, _i8, _i9, _i10, _i11)             \
+  NS_IMPL_THREADSAFE_ADDREF(_class)                                           \
+  NS_IMPL_THREADSAFE_RELEASE(_class)                                          \
+  NS_IMPL_THREADSAFE_QUERY_INTERFACE11(_class, _i1, _i2, _i3, _i4, _i5, _i6,  \
+                                       _i7, _i8, _i9, _i10, _i11)
+
 ///////////////////////////////////////////////////////////////////////////////
 // Macros for implementing nsIClassInfo-related stuff.
 ///////////////////////////////////////////////////////////////////////////////
@@ -1061,6 +1111,22 @@ NS_CI_INTERFACE_GETTER_NAME(_class)(PRUint32 *count, nsIID ***array)          \
      NS_CLASSINFO_HELPER_ENTRY(9, _i10)                                       \
    NS_CLASSINFO_HELPER_END
 
+#define NS_IMPL_CI_INTERFACE_GETTER11(_class, _i1, _i2, _i3, _i4, _i5, _i6,   \
+                                      _i7, _i8, _i9, _i10, _i11)              \
+   NS_CLASSINFO_HELPER_BEGIN(_class, 10)                                      \
+     NS_CLASSINFO_HELPER_ENTRY(0, _i1)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(1, _i2)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(2, _i3)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(3, _i4)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(4, _i5)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(5, _i6)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(6, _i7)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(7, _i8)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(8, _i9)                                        \
+     NS_CLASSINFO_HELPER_ENTRY(9, _i10)                                       \
+     NS_CLASSINFO_HELPER_ENTRY(10, _i11)                                      \
+   NS_CLASSINFO_HELPER_END
+
 #define NS_IMPL_QUERY_INTERFACE10_CI(_class, _i1, _i2, _i3, _i4, _i5, _i6,    \
                                      _i7, _i8, _i9, _i10)                     \
   NS_INTERFACE_MAP_BEGIN(_class)                                              \
@@ -1078,6 +1144,24 @@ NS_CI_INTERFACE_GETTER_NAME(_class)(PRUint32 *count, nsIID ***array)          \
     NS_IMPL_QUERY_CLASSINFO(_class)                                           \
   NS_INTERFACE_MAP_END
 
+#define NS_IMPL_QUERY_INTERFACE11_CI(_class, _i1, _i2, _i3, _i4, _i5, _i6,    \
+                                     _i7, _i8, _i9, _i10, _i11)               \
+  NS_INTERFACE_MAP_BEGIN(_class)                                              \
+    NS_INTERFACE_MAP_ENTRY(_i1)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i2)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i3)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i4)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i5)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i6)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i7)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i8)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i9)                                               \
+    NS_INTERFACE_MAP_ENTRY(_i10)                                              \
+    NS_INTERFACE_MAP_ENTRY(_i11)                                              \
+    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, _i1)                        \
+    NS_IMPL_QUERY_CLASSINFO(_class)                                           \
+  NS_INTERFACE_MAP_END
+
 #define NS_IMPL_ISUPPORTS10_CI(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7,     \
                                _i8, _i9, _i10)                                \
   NS_IMPL_ADDREF(_class)                                                      \
@@ -1087,6 +1171,15 @@ NS_CI_INTERFACE_GETTER_NAME(_class)(PRUint32 *count, nsIID ***array)          \
   NS_IMPL_CI_INTERFACE_GETTER10(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7,    \
                                 _i8, _i9, _i10)
 
+#define NS_IMPL_ISUPPORTS11_CI(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7,     \
+                               _i8, _i9, _i10, _i11)                          \
+  NS_IMPL_ADDREF(_class)                                                      \
+  NS_IMPL_RELEASE(_class)                                                     \
+  NS_IMPL_QUERY_INTERFACE11_CI(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7,     \
+                               _i8, _i9, _i10, _i11)                          \
+  NS_IMPL_CI_INTERFACE_GETTER11(_class, _i1, _i2, _i3, _i4, _i5, _i6, _i7,    \
+                                _i8, _i9, _i10, _i11)
+
 #define NS_INTERFACE_MAP_END_THREADSAFE NS_IMPL_QUERY_TAIL_GUTS
 
 #endif
