@@ -42,10 +42,6 @@
 #ifndef xptiprivate_h___
 #define xptiprivate_h___
 
-#ifndef XPCOM_STANDALONE
-#define XPTI_HAS_ZIP_SUPPORT 1
-#endif /* XPCOM_STANDALONE */
-
 #include "nscore.h"
 #include "nsISupports.h"
 
@@ -57,13 +53,14 @@
 #include "nsIInterfaceInfo.h"
 #include "nsIInterfaceInfoManager.h"
 #include "xptinfo.h"
+#include "nsIXPTLoader.h"
 
 #include "nsIServiceManager.h"
-#include "nsIFile.h"
 #include "nsILocalFile.h"
 #include "nsIDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
+#include "nsIWeakReference.h"
 
 #include "nsCRT.h"
 #include "nsMemory.h"
@@ -76,10 +73,6 @@
 
 #include "nsXPIDLString.h"
 
-#ifdef XPTI_HAS_ZIP_SUPPORT
-#include "nsIZipReader.h"
-#endif /* XPTI_HAS_ZIP_SUPPORT */
-
 #include "nsIInputStream.h"
 
 #include "nsAutoLock.h"
@@ -180,7 +173,7 @@ private:
     xptiTypelibGuts(); // not implemented
     xptiTypelibGuts(XPTHeader* aHeader);
     ~xptiTypelibGuts() {}
-    void* operator new(size_t, void* p) {return p;}
+    void* operator new(size_t, void* p) CPP_THROW_NEW {return p;}
 
 private:
     XPTHeader*           mHeader;        // hold pointer into arena
@@ -474,7 +467,7 @@ public:
     }
 
 private:
-    void* operator new(size_t, void* p) {return p;}
+    void* operator new(size_t, void* p) CPP_THROW_NEW {return p;}
     xptiInterfaceGuts(XPTInterfaceDescriptor* aDescriptor,
                       const xptiTypelib&      aTypelib,
                       xptiWorkingSet*         aWorkingSet)
@@ -538,6 +531,7 @@ class xptiInterfaceEntry
 {
 public:
     static xptiInterfaceEntry* NewEntry(const char* name,
+                                        int nameLength,
                                         const nsID& iid,
                                         const xptiTypelib& typelib,
                                         xptiWorkingSet* aWorkingSet);
@@ -646,7 +640,7 @@ private:
                        const xptiTypelib& typelib);
     ~xptiInterfaceEntry();
 
-    void* operator new(size_t, void* p) {return p;}
+    void* operator new(size_t, void* p) CPP_THROW_NEW {return p;}
 
     void SetResolvedState(int state) 
         {mFlags.SetState(uint8(state));}
@@ -696,7 +690,7 @@ public:
 
     // Use delegation to implement (most!) of nsIInterfaceInfo.
     NS_IMETHOD GetName(char * *aName) { return !mEntry ? NS_ERROR_UNEXPECTED : mEntry->GetName(aName); }
-    NS_IMETHOD GetIID(nsIID * *aIID) { return !mEntry ? NS_ERROR_UNEXPECTED : mEntry->GetIID(aIID); }
+    NS_IMETHOD GetInterfaceIID(nsIID * *aIID) { return !mEntry ? NS_ERROR_UNEXPECTED : mEntry->GetIID(aIID); }
     NS_IMETHOD IsScriptable(PRBool *_retval) { return !mEntry ? NS_ERROR_UNEXPECTED : mEntry->IsScriptable(_retval); }
     // Except this one.
     NS_IMETHOD GetParent(nsIInterfaceInfo * *aParent) 
@@ -787,51 +781,35 @@ private:
 
 /***************************************************************************/
 
-class xptiEntrySink
+class xptiZipLoaderSink : public nsIXPTLoaderSink
 {
 public:
+    xptiZipLoaderSink(xptiInterfaceInfoManager* aMgr,
+                      xptiWorkingSet* aWorkingSet) :
+        mManager(aMgr),
+        mWorkingSet(aWorkingSet) { NS_INIT_ISUPPORTS(); }
+    virtual ~xptiZipLoaderSink() {};
+    
+    NS_DECL_ISUPPORTS
+    NS_DECL_NSIXPTLOADERSINK
+    
+private:
+    xptiInterfaceInfoManager* mManager;
+    xptiWorkingSet* mWorkingSet;
 
-    virtual PRBool 
-    FoundEntry(const char* entryName,
-               int index,
-               XPTHeader* header,
-               xptiWorkingSet* aWorkingSet) = 0;
 };
 
-#ifdef XPTI_HAS_ZIP_SUPPORT
-
 class xptiZipLoader
 {
 public:
     xptiZipLoader();  // not implemented
 
-    static PRBool 
-    EnumerateZipEntries(nsILocalFile* file,
-                        xptiEntrySink* sink,
-                        xptiWorkingSet* aWorkingSet);
+    static XPTHeader*
+    ReadXPTFileFromInputStream(nsIInputStream *stream,
+                               xptiWorkingSet* aWorkingSet);
 
-    static XPTHeader* 
-    ReadXPTFileFromZip(nsILocalFile* file,
-                       const char* entryName,
-                       xptiWorkingSet* aWorkingSet);
-
-    static void
-    Shutdown();
-
-private:    
-    static XPTHeader* 
-    ReadXPTFileFromOpenZip(nsIZipReader* zip,
-                           nsIZipEntry* entry,
-                           const char* entryName,
-                           xptiWorkingSet* aWorkingSet);
-
-    static nsIZipReader*
-    GetZipReader(nsILocalFile* file);
-
-    static nsCOMPtr<nsIZipReaderCache> gCache;
 };
 
-#endif /* XPTI_HAS_ZIP_SUPPORT */
 
 /***************************************************************************/
 
@@ -879,18 +857,18 @@ private:
 /***************************************************************************/
 
 class xptiInterfaceInfoManager 
-    : public nsIInterfaceInfoManager,
-      public xptiEntrySink
+    : public nsIInterfaceInfoSuperManager
 {
     NS_DECL_ISUPPORTS
     NS_DECL_NSIINTERFACEINFOMANAGER
+    NS_DECL_NSIINTERFACEINFOSUPERMANAGER
 
-    // implement xptiEntrySink
+    // helper
     PRBool 
-    FoundEntry(const char* entryName,
-               int index,
-               XPTHeader* header,
-               xptiWorkingSet* aWorkingSet);
+    FoundZipEntry(const char* entryName,
+                  int index,
+                  XPTHeader* header,
+                  xptiWorkingSet* aWorkingSet);
 
 public:
     virtual ~xptiInterfaceInfoManager();
@@ -978,15 +956,17 @@ private:
     static PRBool BuildFileSearchPath(nsISupportsArray** aPath);
 
 private:
-    xptiWorkingSet              mWorkingSet;
-    nsCOMPtr<nsILocalFile>      mStatsLogFile;
-    nsCOMPtr<nsILocalFile>      mAutoRegLogFile;
-    PRFileDesc*                 mOpenLogFile;
-    PRLock*                     mResolveLock;
-    PRLock*                     mAutoRegLock;
-    PRMonitor*                  mInfoMonitor;
-    nsCOMPtr<nsILocalFile>      mManifestDir;
-    nsCOMPtr<nsISupportsArray>  mSearchPath;
+    xptiWorkingSet               mWorkingSet;
+    nsCOMPtr<nsILocalFile>       mStatsLogFile;
+    nsCOMPtr<nsILocalFile>       mAutoRegLogFile;
+    PRFileDesc*                  mOpenLogFile;
+    PRLock*                      mResolveLock;
+    PRLock*                      mAutoRegLock;
+    PRMonitor*                   mInfoMonitor;
+    PRLock*                      mAdditionalManagersLock;
+    nsSupportsArray              mAdditionalManagers;
+    nsCOMPtr<nsILocalFile>       mManifestDir;
+    nsCOMPtr<nsISupportsArray>   mSearchPath;
 };
 
 /***************************************************************************/
