@@ -54,6 +54,7 @@
 
 #include "nsIAppShell.h"
 #include "nsIWidget.h"
+#include "nsIWindowWatcher.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIWebShellWindow.h"
 #include "nsWebShellWindow.h"
@@ -95,7 +96,6 @@ static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
 static NS_DEFINE_CID(kXPConnectCID, NS_XPCONNECT_CID);
 
-#define sWindowWatcherContractID "@mozilla.org/embedcomp/window-watcher;1"
 #define gEQActivatedNotification "nsIEventQueueActivated"
 #define gEQDestroyedNotification "nsIEventQueueDestroyed"
 #define gSkinSelectedTopic       "skin-selected"
@@ -103,13 +103,13 @@ static NS_DEFINE_CID(kXPConnectCID, NS_XPCONNECT_CID);
 #define gInstallRestartTopic     "xpinstall-restart"
 
 nsAppShellService::nsAppShellService() : 
-  mDeleteCalled( PR_FALSE ),
-  mInProfileStartup( PR_FALSE ),
-  mModalWindowCount( 0 ),
-  mShuttingDown( PR_FALSE ),
-  mQuitOnLastWindowClosing( PR_TRUE )
+  mDeleteCalled(PR_FALSE),
+  mModalWindowCount(0),
+  mConsiderQuitStopper(0),
+  mShuttingDown(PR_FALSE),
+  mAttemptingQuit(PR_FALSE)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsAppShellService::~nsAppShellService()
@@ -178,7 +178,7 @@ nsAppShellService::Initialize( nsICmdLineService *aCmdLineService,
  
   // enable window mediation (and fail if we can't get the mediator)
   mWindowMediator = do_GetService(kWindowMediatorCID, &rv);
-  mWindowWatcher = do_GetService(sWindowWatcherContractID);
+  mWindowWatcher = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
 
 done:
   return rv;
@@ -229,6 +229,29 @@ nsresult nsAppShellService::ClearXPConnectSafeContext()
   return rv;
 }
 
+/* We know we're trying to quit the app, but may not be able to do so
+   immediately. Enter a state where we're more ready to quit.
+   (Does useful work only on the Mac.) */
+void
+nsAppShellService::AttemptingQuit(PRBool aAttempt)
+{
+#if defined(XP_MAC) || defined(XP_MACOSX)
+  if (aAttempt) {
+    // now even the Mac wants to quit when the last window is closed
+    if (!mAttemptingQuit)
+      ExitLastWindowClosingSurvivalArea();
+    mAttemptingQuit = PR_TRUE;
+  } else {
+    // changed our mind. back to normal.
+    if (mAttemptingQuit)
+      EnterLastWindowClosingSurvivalArea();
+    mAttemptingQuit = PR_FALSE;
+  }
+#else
+  mAttemptingQuit = aAttempt;
+#endif
+}
+
 NS_IMETHODIMP
 nsAppShellService::DoProfileStartup(nsICmdLineService *aCmdLineService, PRBool canInteract)
 {
@@ -237,11 +260,9 @@ nsAppShellService::DoProfileStartup(nsICmdLineService *aCmdLineService, PRBool c
     nsCOMPtr<nsIProfileInternal> profileMgr(do_GetService(NS_PROFILE_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv,rv);
 
-    mInProfileStartup = PR_TRUE;
-    PRBool saveQuitOnLastWindowClosing = mQuitOnLastWindowClosing;
-    mQuitOnLastWindowClosing = PR_FALSE;
-        
-    // If we being launched in turbo mode, profile mgr cannot show UI
+    EnterLastWindowClosingSurvivalArea();
+
+    // If we are being launched in turbo mode, profile mgr cannot show UI
     rv = profileMgr->StartupWithArgs(aCmdLineService, canInteract);
     if (!canInteract && rv == NS_ERROR_PROFILE_REQUIRES_INTERACTION) {
         NS_WARNING("nsIProfileInternal::StartupWithArgs returned NS_ERROR_PROFILE_REQUIRES_INTERACTION");       
@@ -254,8 +275,7 @@ nsAppShellService::DoProfileStartup(nsICmdLineService *aCmdLineService, PRBool c
         rv = NS_OK;
     }
 
-    mQuitOnLastWindowClosing = saveQuitOnLastWindowClosing;
-    mInProfileStartup = PR_FALSE;
+    ExitLastWindowClosingSurvivalArea();
 
     // if Quit() was called while we were starting up we have a failure situation...
     if (mShuttingDown)
@@ -453,95 +473,176 @@ nsAppShellService::Run(void)
 
 
 NS_IMETHODIMP
-nsAppShellService::Quit()
+nsAppShellService::Quit(PRUint32 aFerocity)
 {
   // Quit the application. We will asynchronously call the appshell's
   // Exit() method via the ExitCallback() to allow one last pass
   // through any events in the queue. This guarantees a tidy cleanup.
   nsresult rv = NS_OK;
+  PRBool postedExitEvent = PR_FALSE;
 
   if (mShuttingDown)
     return NS_OK;
 
+  /* eForceQuit doesn't actually work; it can cause a subtle crash if
+     there are windows open which have unload handlers which open
+     new windows. Use eAttemptQuit for now. */
+  if (aFerocity == eForceQuit)
+    return NS_ERROR_FAILURE;
+
   mShuttingDown = PR_TRUE;
 
-  // Shutdown native app support; doing this first will prevent new
-  // requests to open additional windows coming in.
-  if (mNativeAppSupport) {
-    mNativeAppSupport->Quit();
-    mNativeAppSupport = 0;
-  }
+  if (aFerocity == eConsiderQuit && mConsiderQuitStopper == 0) {
+    // attempt quit if the last window has been unregistered/closed
 
-  // Enumerate through each open window and close it
-  if (mWindowMediator) {
-    nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
-    mWindowMediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+    PRBool windowsRemain = PR_TRUE;
 
-    if (windowEnumerator) {
-      PRBool more;
+    if (mWindowMediator) {
+      nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
+      mWindowMediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+      if (windowEnumerator)
+        windowEnumerator->HasMoreElements(&windowsRemain);
+    }
+    if (!windowsRemain) {
+      aFerocity = eAttemptQuit;
 
-      while (1) {
-        rv = windowEnumerator->HasMoreElements(&more);
-        if (NS_FAILED(rv) || !more)
-          break;
-
-        nsCOMPtr<nsISupports> isupports;
-        rv = windowEnumerator->GetNext(getter_AddRefs(isupports));
-        if (NS_FAILED(rv))
-          break;
-
-        nsCOMPtr<nsIDOMWindowInternal> window = do_QueryInterface(isupports);
-        NS_ASSERTION(window != nsnull, "not an nsIDOMWindowInternal");
-        if (! window)
-          continue;
-
-        window->Close();
+      // Check to see if we should quit in this case.
+      if (mNativeAppSupport) {
+        PRBool serverMode = PR_FALSE;
+        mNativeAppSupport->GetIsServerMode(&serverMode);
+        if (serverMode) {
+          // stop! give control to server mode
+          mShuttingDown = PR_FALSE;
+          mNativeAppSupport->OnLastWindowClosing();
+          return NS_OK;
+        }
       }
     }
   }
 
-  {
+  /* Currently aFerocity can never have the value of eForceQuit here.
+     That's temporary (in an unscheduled kind of way) and logically
+     this code is part of the eForceQuit case, so I'm checking against
+     that value anyway. Reviewers made me add this comment. */
+  if (aFerocity == eAttemptQuit || aFerocity == eForceQuit) {
+
+    AttemptingQuit(PR_TRUE);
+
+    /* Enumerate through each open window and close it. It's important to do
+       this before we forcequit because this can control whether we really quit
+       at all. e.g. if one of these windows has an unload handler that
+       opens a new window. Ugh. I know. */
+    if (mWindowMediator) {
+      nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
+
+      mWindowMediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+
+      if (windowEnumerator) {
+
+        while (1) {
+          PRBool more;
+          if (NS_FAILED(rv = windowEnumerator->HasMoreElements(&more)) || !more)
+            break;
+
+          nsCOMPtr<nsISupports> isupports;
+          rv = windowEnumerator->GetNext(getter_AddRefs(isupports));
+          if (NS_FAILED(rv))
+            break;
+
+          nsCOMPtr<nsIDOMWindowInternal> window = do_QueryInterface(isupports);
+          NS_ASSERTION(window, "not an nsIDOMWindowInternal");
+          if (!window)
+            continue;
+
+          window->Close();
+        }
+      }
+
+      if (aFerocity == eAttemptQuit) {
+
+        aFerocity = eForceQuit; // assume success
+
+        /* Were we able to immediately close all windows? if not, eAttemptQuit
+           failed. This could happen for a variety of reasons; in fact it's
+           very likely. Perhaps we're being called from JS and the window->Close
+           method hasn't had a chance to wrap itself up yet. So give up.
+           We'll return (with eConsiderQuit) as the remaining windows are
+           closed. */
+        mWindowMediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
+        if (windowEnumerator) {
+          PRBool more;
+          if (NS_SUCCEEDED(windowEnumerator->HasMoreElements(&more)) && more) {
+            aFerocity = eAttemptQuit;
+            rv = NS_ERROR_FAILURE;
+          }
+        }
+      }
+    }
+  }
+
+  if (aFerocity == eForceQuit) {
+    // do it!
+
+    // No chance of the shutdown being cancelled from here on; tell people
+    // we're shutting down for sure while all services are still available.
+    nsCOMPtr<nsIObserverService> obsService = do_GetService("@mozilla.org/observer-service;1", &rv);
+    obsService->NotifyObservers(nsnull, "quit-application", nsnull);
+
+    // first shutdown native app support; doing this first will prevent new
+    // requests to open additional windows coming in.
+    if (mNativeAppSupport) {
+      mNativeAppSupport->Quit();
+      mNativeAppSupport = 0;
+    }
+
     nsCOMPtr<nsIWebShellWindow> hiddenWin(do_QueryInterface(mHiddenWindow));
     if (hiddenWin) {
       ClearXPConnectSafeContext();
       hiddenWin->Close();
     }
     mHiddenWindow = nsnull;
-  }
-  
-  // Note that we don't allow any premature returns from the above
-  // loop: no matter what, make sure we send the exit event.  If
-  // worst comes to worst, we'll do a leaky shutdown but we WILL
-  // shut down. Well, assuming that all *this* stuff works ;-).
-  nsCOMPtr<nsIEventQueueService> svc = do_GetService(kEventQueueServiceCID, &rv);
-  if (NS_FAILED(rv)) return rv;
+    
+    // no matter what, make sure we send the exit event.  If
+    // worst comes to worst, we'll do a leaky shutdown but we WILL
+    // shut down. Well, assuming that all *this* stuff works ;-).
+    nsCOMPtr<nsIEventQueueService> svc = do_GetService(kEventQueueServiceCID, &rv);
+    if (NS_SUCCEEDED(rv)) {
 
-  nsCOMPtr<nsIEventQueue> queue;
-  rv = svc->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(queue));
-  if (NS_FAILED(rv)) return rv;
+      nsCOMPtr<nsIEventQueue> queue;
+      rv = svc->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(queue));
+      if (NS_SUCCEEDED(rv)) {
 
-  ExitEvent* event = new ExitEvent;
-  if (!event)
-    return NS_ERROR_OUT_OF_MEMORY;
+        ExitEvent* event = new ExitEvent;
+        if (event) {
+          PL_InitEvent(NS_REINTERPRET_CAST(PLEvent*, event),
+                       nsnull,
+                       HandleExitEvent,
+                       DestroyExitEvent);
 
-  PL_InitEvent(NS_REINTERPRET_CAST(PLEvent*, event),
-                nsnull,
-                HandleExitEvent,
-                DestroyExitEvent);
+          event->mService = this;
+          NS_ADDREF(event->mService);
 
-  event->mService = this;
-  NS_ADDREF(event->mService);
+          rv = queue->EnterMonitor();
+          if (NS_SUCCEEDED(rv))
+            rv = queue->PostEvent(NS_REINTERPRET_CAST(PLEvent*, event));
+          if (NS_SUCCEEDED(rv))
+            postedExitEvent = PR_TRUE;
+          queue->ExitMonitor();
 
-  rv = queue->EnterMonitor();
-  if (NS_SUCCEEDED(rv))
-    rv = queue->PostEvent(NS_REINTERPRET_CAST(PLEvent*, event));
-  queue->ExitMonitor();
-
-  if (NS_FAILED(rv)) {
-    NS_RELEASE(event->mService);
-    delete event;
+          if (NS_FAILED(rv)) {
+            NS_RELEASE(event->mService);
+            delete event;
+          }
+        } else
+          rv = NS_ERROR_OUT_OF_MEMORY;
+      }
+    }
   }
 
+  // turn off the reentrancy check flag, but not if we have
+  // more asynchronous work to do still.
+  if (!postedExitEvent)
+    mShuttingDown = PR_FALSE;
   return rv;
 }
 
@@ -554,9 +655,7 @@ nsAppShellService::HandleExitEvent(PLEvent* aEvent)
   event->mService->mAppShell->Exit();
 
   // We're done "shutting down".
-  // unless we are currently "starting up" in which case this event wasn't properly handled.
-  if (!event->mService->mInProfileStartup)
-    event->mService->mShuttingDown = PR_FALSE;
+  event->mService->mShuttingDown = PR_FALSE;
 
   return nsnull;
 }
@@ -836,6 +935,9 @@ nsAppShellService::RegisterTopLevelWindow(nsIXULWindow* aWindow)
     }
   }
 
+  // an ongoing attempt to quit is stopped by a newly opened window
+  AttemptingQuit(PR_FALSE);
+
   return NS_OK;
 }
 
@@ -843,8 +945,6 @@ nsAppShellService::RegisterTopLevelWindow(nsIXULWindow* aWindow)
 NS_IMETHODIMP
 nsAppShellService::UnregisterTopLevelWindow(nsIXULWindow* aWindow)
 {
-  PRBool windowsRemain = PR_TRUE;
-
   if (mDeleteCalled) {
     /* return an error code in order to:
        - avoid doing anything with other member variables while we are in
@@ -860,14 +960,8 @@ nsAppShellService::UnregisterTopLevelWindow(nsIXULWindow* aWindow)
   NS_ENSURE_ARG_POINTER(aWindow);
 
   // tell the window mediator
-  if (mWindowMediator) {
-    nsCOMPtr<nsISimpleEnumerator> windowEnumerator;
+  if (mWindowMediator)
     mWindowMediator->UnregisterWindow(aWindow);
-
-    mWindowMediator->GetEnumerator(nsnull, getter_AddRefs(windowEnumerator));
-    if (windowEnumerator)
-      windowEnumerator->HasMoreElements(&windowsRemain);
-  }
 	
   // tell the window watcher
   if (mWindowWatcher) {
@@ -880,35 +974,6 @@ nsAppShellService::UnregisterTopLevelWindow(nsIXULWindow* aWindow)
     }
   }
 
-  // now quit if the last window has been unregistered (unless we shouldn't)
-
-  if (!mQuitOnLastWindowClosing)
-    return NS_OK;
-
-  if (!windowsRemain) {
-      
-#if defined(XP_MAC) || defined(XP_MACOSX)
-    // if no hidden window is available (perhaps due to initial
-    // Profile Manager window being cancelled), then just quit. We don't have
-    // to worry about focussing the hidden window, because it will get activated
-    // by the OS since it is visible (but waaaay offscreen).
-    nsCOMPtr<nsIBaseWindow> hiddenWin(do_QueryInterface(mHiddenWindow));
-    if (!hiddenWin)
-      Quit();
-#else
-    // Check to see if we should quit in this case.
-    if (mNativeAppSupport) {
-      PRBool serverMode = PR_FALSE;
-      mNativeAppSupport->GetIsServerMode(&serverMode);
-      if (serverMode) {
-        mNativeAppSupport->OnLastWindowClosing(aWindow);
-        return NS_OK;
-      }
-    }
-
-    Quit();
-#endif 
-  }
   return NS_OK;
 }
 
@@ -970,18 +1035,18 @@ nsAppShellService::TopLevelWindowIsModal(nsIXULWindow *aWindow, PRBool aModal)
 }
 
 NS_IMETHODIMP
-nsAppShellService::GetQuitOnLastWindowClosing(PRBool *aQuitOnLastWindowClosing)
+nsAppShellService::EnterLastWindowClosingSurvivalArea(void)
 {
-    NS_ENSURE_ARG_POINTER(aQuitOnLastWindowClosing);
-    *aQuitOnLastWindowClosing = mQuitOnLastWindowClosing;
-    return NS_OK;
+  PR_AtomicIncrement(&mConsiderQuitStopper);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAppShellService::SetQuitOnLastWindowClosing(PRBool aQuitOnLastWindowClosing)
+nsAppShellService::ExitLastWindowClosingSurvivalArea(void)
 {
-    mQuitOnLastWindowClosing = aQuitOnLastWindowClosing;
-    return NS_OK;
+  NS_ASSERTION(mConsiderQuitStopper > 0, "consider quit stopper out of bounds");
+  PR_AtomicDecrement(&mConsiderQuitStopper);
+  return NS_OK;
 }
 
 
