@@ -75,7 +75,7 @@ nsContentTreeOwner::nsContentTreeOwner(PRBool fPrimary) : mXULWindow(nsnull),
    mPrimary(fPrimary), mContentTitleSetting(PR_FALSE), 
    mChromeFlags(nsIWebBrowserChrome::CHROME_ALL)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   // note if this fails, QI on nsIEmbeddingSiteWindow(2) will simply fail
   mSiteWindow2 = new nsSiteWindow2(this);
@@ -345,8 +345,7 @@ NS_IMETHODIMP nsContentTreeOwner::SetStatus(PRUint32 aStatusType, const PRUnicha
       return NS_OK;
 
    nsCOMPtr<nsISupports> xpConnectObj;
-   nsAutoString xulBrowserWinId(NS_LITERAL_STRING("XULBrowserWindow"));
-   piDOMWindow->GetObjectProperty(xulBrowserWinId.get(), getter_AddRefs(xpConnectObj));
+   piDOMWindow->GetObjectProperty(NS_LITERAL_STRING("XULBrowserWindow").get(), getter_AddRefs(xpConnectObj));
    nsCOMPtr<nsIXULBrowserWindow> xulBrowserWindow(do_QueryInterface(xpConnectObj));
 
    if (xulBrowserWindow)
@@ -729,7 +728,7 @@ nsXULWindow* nsContentTreeOwner::XULWindow()
 
 nsSiteWindow2::nsSiteWindow2(nsContentTreeOwner *aAggregator)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mAggregator = aAggregator;
 }
 
