@@ -70,31 +70,38 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 #define DOWNLOAD_MANAGER_BUNDLE "chrome://communicator/locale/downloadmanager/downloadmanager.properties"
 #define INTERVAL 500
 
-nsIRDFResource* gNC_DownloadsRoot;
-nsIRDFResource* gNC_File;
-nsIRDFResource* gNC_URL;
-nsIRDFResource* gNC_Name;
-nsIRDFResource* gNC_ProgressMode;
-nsIRDFResource* gNC_ProgressPercent;
-nsIRDFResource* gNC_Transferred;
-nsIRDFResource* gNC_DownloadState;
-nsIRDFResource* gNC_StatusText;
+static nsIRDFResource* gNC_DownloadsRoot = nsnull;
+static nsIRDFResource* gNC_File = nsnull;
+static nsIRDFResource* gNC_URL = nsnull;
+static nsIRDFResource* gNC_Name = nsnull;
+static nsIRDFResource* gNC_ProgressMode = nsnull;
+static nsIRDFResource* gNC_ProgressPercent = nsnull;
+static nsIRDFResource* gNC_Transferred = nsnull;
+static nsIRDFResource* gNC_DownloadState = nsnull;
+static nsIRDFResource* gNC_StatusText = nsnull;
 
-nsIRDFService* gRDFService;
+static nsIRDFService* gRDFService = nsnull;
+
+static PRInt32 gRefCnt = 0;
 
 ///////////////////////////////////////////////////////////////////////////////
 // nsDownloadManager
 
 NS_IMPL_ISUPPORTS3(nsDownloadManager, nsIDownloadManager, nsIDOMEventListener, nsIObserver)
 
-nsDownloadManager::nsDownloadManager() : mBatches(0),
-                                         mCurrDownloads(nsnull)
+nsDownloadManager::nsDownloadManager() : mBatches(0)
 {
   NS_INIT_ISUPPORTS();
 }
 
 nsDownloadManager::~nsDownloadManager()
 {
+  if (--gRefCnt != 0 || !gRDFService)
+    // Either somebody tried to use |CreateInstance| instead of
+    // |GetService| or |Init| failed very early, so there's nothing to
+    // do here.
+    return;
+
   gRDFService->UnregisterDataSource(mDataSource);
 
   NS_IF_RELEASE(gNC_DownloadsRoot);                                             
@@ -107,16 +114,17 @@ nsDownloadManager::~nsDownloadManager()
   NS_IF_RELEASE(gNC_DownloadState);
   NS_IF_RELEASE(gNC_StatusText);
 
-  nsServiceManager::ReleaseService(kRDFServiceCID, gRDFService);                
-  gRDFService = nsnull;                                                         
-
-  delete mCurrDownloads;
-  mCurrDownloads = nsnull;
+  NS_RELEASE(gRDFService);
 }
 
 nsresult
 nsDownloadManager::Init()
 {
+  if (gRefCnt++ != 0) {
+    NS_NOTREACHED("download manager should be used as a service");
+    return NS_ERROR_UNEXPECTED; // This will make the |CreateInstance| fail.
+  }
+
   nsresult rv;
   mRDFContainerUtils = do_GetService("@mozilla.org/rdf/container-utils;1", &rv);
   if (NS_FAILED(rv)) return rv;
@@ -126,8 +134,7 @@ nsDownloadManager::Init()
   
   obsService->AddObserver(this, "quit-application", PR_FALSE);
 
-  rv = nsServiceManager::GetService(kRDFServiceCID, NS_GET_IID(nsIRDFService),
-                                    (nsISupports**) &gRDFService);
+  rv = CallGetService(kRDFServiceCID, &gRDFService);
   if (NS_FAILED(rv)) return rv;                                                 
 
   gRDFService->GetResource("NC:DownloadsRoot", &gNC_DownloadsRoot);
@@ -160,7 +167,7 @@ nsresult
 nsDownloadManager::DownloadStarted(const char* aPath)
 {
   nsCStringKey key(aPath);
-  if (mCurrDownloads->Exists(&key))
+  if (mCurrDownloads.Exists(&key))
     AssertProgressInfoFor(aPath);
 
   return NS_OK;
@@ -170,9 +177,9 @@ nsresult
 nsDownloadManager::DownloadEnded(const char* aPath, const PRUnichar* aMessage)
 {
   nsCStringKey key(aPath);
-  if (mCurrDownloads->Exists(&key)) {
+  if (mCurrDownloads.Exists(&key)) {
     AssertProgressInfoFor(aPath);
-    mCurrDownloads->Remove(&key);
+    mCurrDownloads.Remove(&key);
   }
 
   return NS_OK;
@@ -238,9 +245,6 @@ nsDownloadManager::GetDataSource(nsIRDFDataSource** aDataSource)
 nsresult
 nsDownloadManager::AssertProgressInfo()
 {
-  if (!mCurrDownloads)
-    mCurrDownloads = new nsHashtable();
-
   nsCOMPtr<nsISupports> supports;
   nsCOMPtr<nsIRDFResource> res;
   const char* uri;
@@ -268,10 +272,10 @@ nsresult
 nsDownloadManager::AssertProgressInfoFor(const char* aPath)
 {
   nsCStringKey key(aPath);
-  if (!mCurrDownloads->Exists(&key))
+  if (!mCurrDownloads.Exists(&key))
     return NS_ERROR_FAILURE;
  
-  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads->Get(&key));
+  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
   nsCOMPtr<nsIDownload> download;
   internalDownload->QueryInterface(NS_GET_IID(nsIDownload), (void**) getter_AddRefs(download));
   if (!download)
@@ -431,18 +435,8 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
   nsCOMPtr<nsIRDFResource> downloadRes;
   gRDFService->GetResource(path.get(), getter_AddRefs(downloadRes));
 
-  // if the resource is in the container already (the user has already
-  // downloaded this file), remove it
-  PRInt32 itemIndex;
   nsCOMPtr<nsIRDFNode> node;
-  downloads->IndexOf(downloadRes, &itemIndex);
-  if (itemIndex > 0) {
-    rv = downloads->RemoveElementAt(itemIndex, PR_TRUE, getter_AddRefs(node));
-    if (NS_FAILED(rv)) return rv;
-  }
-  rv = downloads->AppendElement(downloadRes);
-  if (NS_FAILED(rv)) return rv;
-  
+
   // Assert source url information
   nsCAutoString spec;
   aSource->GetSpec(spec);
@@ -454,11 +448,7 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
     rv = mDataSource->Change(downloadRes, gNC_URL, node, urlResource);
   else
     rv = mDataSource->Assert(downloadRes, gNC_URL, urlResource, PR_TRUE);
-  if (NS_FAILED(rv)) {
-    downloads->IndexOf(downloadRes, &itemIndex);
-    downloads->RemoveElementAt(itemIndex, PR_TRUE, getter_AddRefs(node));
-    return rv;
-  }
+  if (NS_FAILED(rv)) return rv;
 
   // Set and assert the "pretty" (display) name of the download
   nsAutoString displayName; displayName.Assign(aDisplayName);
@@ -474,11 +464,7 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
     rv = mDataSource->Change(downloadRes, gNC_Name, node, nameLiteral);
   else
     rv = mDataSource->Assert(downloadRes, gNC_Name, nameLiteral, PR_TRUE);
-  if (NS_FAILED(rv)) {
-    downloads->IndexOf(downloadRes, &itemIndex);
-    downloads->RemoveElementAt(itemIndex, PR_TRUE, getter_AddRefs(node));
-    return rv;
-  }
+  if (NS_FAILED(rv)) return rv;
   
   internalDownload->SetMIMEInfo(aMIMEInfo);
   internalDownload->SetStartTime(aStartTime);
@@ -487,11 +473,7 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
   nsCOMPtr<nsIRDFResource> fileResource;
   gRDFService->GetResource(path.get(), getter_AddRefs(fileResource));
   rv = mDataSource->Assert(downloadRes, gNC_File, fileResource, PR_TRUE);
-  if (NS_FAILED(rv)) {
-    downloads->IndexOf(downloadRes, &itemIndex);
-    downloads->RemoveElementAt(itemIndex, PR_TRUE, getter_AddRefs(node));
-    return rv;
-  }
+  if (NS_FAILED(rv)) return rv;
   
   // Assert download state information (NOTSTARTED, since it's just now being added)
   nsCOMPtr<nsIRDFInt> intLiteral;
@@ -501,20 +483,19 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
     rv = mDataSource->Change(downloadRes, gNC_DownloadState, node, intLiteral);
   else
     rv = mDataSource->Assert(downloadRes, gNC_DownloadState, intLiteral, PR_TRUE);
-  if (NS_FAILED(rv)) {
-    downloads->IndexOf(downloadRes, &itemIndex);
-    downloads->RemoveElementAt(itemIndex, PR_TRUE, getter_AddRefs(node));
-    return rv;
-  }
+  if (NS_FAILED(rv)) return rv;
   
+  PRInt32 itemIndex;
+  downloads->IndexOf(downloadRes, &itemIndex);
+  if (itemIndex == -1) {
+    rv = downloads->AppendElement(downloadRes);
+    if (NS_FAILED(rv)) return rv;
+  }
+
   // Now flush all this to disk
   nsCOMPtr<nsIRDFRemoteDataSource> remote(do_QueryInterface(mDataSource));
   rv = remote->Flush();
-  if (NS_FAILED(rv)) {
-    downloads->IndexOf(downloadRes, &itemIndex);
-    downloads->RemoveElementAt(itemIndex, PR_TRUE, getter_AddRefs(node));
-    return rv;
-  }
+  if (NS_FAILED(rv)) return rv;
 
   // if a persist object was specified, set the download item as the progress listener
   // this will create a cycle that will be broken in nsDownload::OnStateChange
@@ -524,15 +505,11 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
     aPersist->SetProgressListener(listener);
   }
 
-  // we use this table to manage ongoing downloads internally
-  if (!mCurrDownloads)
-    mCurrDownloads = new nsHashtable();
-  
   nsCStringKey key(path);
-  if (mCurrDownloads->Exists(&key))
-    mCurrDownloads->Remove(&key);
+  if (mCurrDownloads.Exists(&key))
+    mCurrDownloads.Remove(&key);
 
-  mCurrDownloads->Put(&key, *aDownload);
+  mCurrDownloads.Put(&key, *aDownload);
 
   return rv;
 }
@@ -546,8 +523,8 @@ nsDownloadManager::GetDownload(const char* aPath, nsIDownload** aDownloadItem)
   // XXX otherwise we should look for it in the datasource and
   //     create a new nsIDownload with the resource's properties
   nsCStringKey key(aPath);
-  if (mCurrDownloads->Exists(&key)) {
-    *aDownloadItem = NS_STATIC_CAST(nsIDownload*, mCurrDownloads->Get(&key));
+  if (mCurrDownloads.Exists(&key)) {
+    *aDownloadItem = NS_STATIC_CAST(nsIDownload*, mCurrDownloads.Get(&key));
     NS_ADDREF(*aDownloadItem);
     return NS_OK;
   }
@@ -561,11 +538,11 @@ nsDownloadManager::CancelDownload(const char* aPath)
 {
   nsresult rv = NS_OK;
   nsCStringKey key(aPath);
-  if (!mCurrDownloads->Exists(&key))
+  if (!mCurrDownloads.Exists(&key))
     return NS_ERROR_FAILURE;
   
   nsCOMPtr<nsIDownload> download;
-  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads->Get(&key));
+  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
   internalDownload->QueryInterface(NS_GET_IID(nsIDownload), (void**) getter_AddRefs(download));
   if (!download)
     return NS_ERROR_FAILURE;
@@ -612,7 +589,7 @@ nsDownloadManager::RemoveDownload(const char* aPath)
   
   // RemoveDownload is for downloads not currently in progress. Having it
   // cancel in-progress downloads would make things complicated, so just return.
-  PRBool inProgress = mCurrDownloads->Exists(&key);
+  PRBool inProgress = mCurrDownloads.Exists(&key);
   NS_ASSERTION(!inProgress, "Can't call RemoveDownload on a download in progress!");
   if (inProgress)
     return NS_ERROR_FAILURE;
@@ -723,7 +700,7 @@ nsDownloadManager::Open(nsIDOMWindow* aParent, nsIDownload* aDownload)
 
   // if we ever have the capability to display the UI of third party dl managers,
   // we'll open their UI here instead.
-  nsCOMPtr<nsIWindowWatcher> ww = do_GetService("@mozilla.org/embedcomp/window-watcher;1", &rv);
+  nsCOMPtr<nsIWindowWatcher> ww = do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
   // pass the datasource to the window
@@ -756,11 +733,11 @@ nsDownloadManager::OpenProgressDialogFor(const char* aPath, nsIDOMWindow* aParen
 {
   nsresult rv;
   nsCStringKey key(aPath);
-  if (!mCurrDownloads->Exists(&key))
+  if (!mCurrDownloads.Exists(&key))
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDownload> download;
-  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads->Get(&key));
+  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
   internalDownload->QueryInterface(NS_GET_IID(nsIDownload), (void**) getter_AddRefs(download));
   if (!download)
     return NS_ERROR_FAILURE;
@@ -863,9 +840,9 @@ nsDownloadManager::Observe(nsISupports* aSubject, const char* aTopic, const PRUn
     if (NS_FAILED(rv)) return rv;
     
     nsCStringKey key(path);
-    if (mCurrDownloads->Exists(&key)) {
+    if (mCurrDownloads.Exists(&key)) {
       // unset dialog since it's closing
-      nsDownload* download = NS_STATIC_CAST(nsDownload*, mCurrDownloads->Get(&key));
+      nsDownload* download = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
       download->SetDialog(nsnull);
       
       return CancelDownload(path.get());  
