@@ -86,7 +86,8 @@ if($inRedirIniURL eq "")
 
 $seiFileNameGeneric   = "nsinstall.exe";
 $seiFileNameSpecific  = "mozilla-win32-installer.exe";
-$seiFileNameSpecificStub  = "mozilla-win32-stub-installer.exe";
+$seiStubRootName = "mozilla-win32-stub-installer";
+$seiFileNameSpecificStub  = "$seiStubRootName.exe";
 $seuFileNameSpecific  = "MozillaUninstall.exe";
 $seuzFileNameSpecific = "mozillauninstall.zip";
 
@@ -103,6 +104,7 @@ print "The display version is: $versionMain\n";
 $versionLanguage               = "en";
 $ENV{WIZ_nameCompany}          = "mozilla.org";
 $ENV{WIZ_nameProduct}          = "Mozilla";
+$ENV{WIZ_nameProductNoVersion} = "Mozilla";
 $ENV{WIZ_fileMainExe}          = "Mozilla.exe";
 $ENV{WIZ_fileUninstall}        = $seuFileNameSpecific;
 $ENV{WIZ_fileUninstallZip}     = $seuzFileNameSpecific;
@@ -251,6 +253,38 @@ if(system("copy $inDistPath\\$seiFileNameSpecificStub $inDistPath\\stub"))
   die "\n Error: copy $inDistPath\\$seiFileNameSpecificStub $inDistPath\\stub\n";
 }
 
+# create the xpi for launching the stub installer
+print "\n**********************************\n";
+print "*                                  *\n";
+print "*  creating stub installer xpi...  *\n";
+print "*                                  *\n";
+print "************************************\n";
+if(-d "$inStagePath\\$seiStubRootName")
+{
+  unlink <$inStagePath\\$seiStubRootName\\*>;
+}
+else
+{
+  mkdir ("$inStagePath\\$seiStubRootName",0775);
+}
+if(system("copy $inDistPath\\stub\\$seiFileNameSpecificStub $gLocalTmpStage\\$seiStubRootName"))
+{
+  die "\n Error: copy $inDistPath\\stub\\$seiFileNameSpecificStub $gLocalTmpStage\\$seiStubRootName\n";
+}
+
+# Make .js files
+if(MakeJsFile($seiStubRootName))
+{
+  return(1);
+}
+
+# Make .xpi file
+if(system("perl makexpi.pl $seiStubRootName $gLocalTmpStage $inDistPath"))
+{
+  print "\n Error: perl makexpi.pl $seiStubRootName $gLocalTmpStage $inDistPath\n";
+  return(1);
+}
+
 # group files for CD
 print "\n************************************\n";
 print "*                                  *\n";
@@ -339,10 +373,10 @@ sub MakeExeZip
 
   $saveCwdir = cwd();
   chdir($aSrcDir);
-  if(system("$ENV{MOZ_TOOLS}\\bin\\zip $inDistPath\\xpi\\$aZipFile $aExeFile"))
+  if(system("zip $inDistPath\\xpi\\$aZipFile $aExeFile"))
   {
     chdir($saveCwdir);
-    die "\n Error: $ENV{MOZ_TOOLS}\\bin\\zip $inDistPath\\xpi\\$aZipFile $aExeFile";
+    die "\n Error: zip $inDistPath\\xpi\\$aZipFile $aExeFile";
   }
   chdir($saveCwdir);
 }
@@ -438,6 +472,16 @@ sub MakeUninstall
     print "\n Error: copy uninstall.ini $inDistPath\\uninstall\n";
     return(1);
   }
+  if(system("copy defaults_info.ini $inDistPath"))
+  {
+    print "\n Error: copy defaults_info.ini $inDistPath\n";
+    return(1);
+  }
+  if(system("copy defaults_info.ini $inDistPath\\uninstall"))
+  {
+    print "\n Error: copy defaults_info.ini $inDistPath\\uninstall\n";
+    return(1);
+  }
   if(system("copy $inDistPath\\uninstall.exe $inDistPath\\uninstall"))
   {
     print "\n Error: copy $inDistPath\\uninstall.exe $inDistPath\\uninstall\n";
